#
# 	sebastien.letort@toulouse.inra.fr
#	Created: April 09, 2008
#	$Id: Transcript.pm 3074 2009-11-26 15:37:20Z sletort $
#

package Bio::Narcisse::Transcript;

=pod

=head1 NAME

 Transcript

=head1  SYNOPSIS

 my $o_transcript = new Bio::Narcisse::Transcript( %hash );

=head1 DESCRIPTION

 This class is just a daugther of the Ensembl Transcript class.
 This is because we need an additionnal attribute, 'consistency'.

=cut

use strict;
use warnings;

use Data::Dumper;

use Bio::EnsEMBL::Transcript;

use base qw( Bio::EnsEMBL::Transcript );

sub new
{
	my ($class) = shift;

	$class = ref $class	if (ref $class);

	my $self = $class->SUPER::new(@_);

	# la partie specifique a Narcisse.
	my %h_args = @_;
	$self->{ nc_consistency } = $h_args{ nc_consistency };

	return $self;
}


=head2 function IsConsistent

 Usage        : $bool = $o_xx->IsConsistent();
 Prerequisite : none
 Function     : check if the transcript is consistent or not
 Returns      : $bool, &TRUE if the corresponding cds seems valid
 Args         : none

=cut

sub IsConsistent
{
	my $self = shift;

	return ( 'C' eq $self->{ nc_consistency } );
} # IsConsistent


=head2 function GetConsistencyErrorMessage

 Usage        : $msg = $o_xx->GetConsistencyErrorMessage();
 Prerequisite : please, use only after the test has passed
                	( better to use only for error _user point of view_ )
 Function     : get a message describing the consistency code
 Returns      : $msg, a string, the message corresponding to the error code
 Args         : none

=cut

sub GetConsistencyErrorMessage
{
	my $self = shift;

	# le hash de message
	my %h_msg = (
			'C' => "the protein is consistent",
			'F' => "the predicted protein does not start with a 'M' or the region to be translated doesn't end with a stop codon",
			'I' => "the predicted protein contains in-frame stop codon"
		);

	my $code = $self->{ nc_consistency };
	my $msg = $h_msg{ $code };

	return $msg	if( defined( $msg ) );

	return "No message found for code : '$code'";
} # GetConsistencyErrorMessage


1;
