=head1 NAME

Bio::Narcisse::Utils::ScriptUtils - Provides utilities routines for Narcisse API.

=head1 DESCRIPTION

Bio::Narcisse::Utils::ScriptUtils - Provides utilities routines for Narcisse API.

=head1 SYNOPSIS

=head1 METHODS

=cut


package Bio::Narcisse::Utils::ScriptUtils;

use strict;

use IO::File;	#Seb. L.
use LipmError::IOException;
use LipmError;
use Data::Dumper;
use Carp;

use Exporter;
use Cwd;
use Bio::EnsEMBL::Utils::Exception qw(throw warning);
use Bio::Narcisse::Utils::NarcisseDataReader;

our @ISA = qw(Exporter);

our @EXPORT_OK = qw(
  get_NarcisseConfiguration
  LoadData
  get_karyotype
  ExtractSeqInFasta
  ExtractByFmtseq
);

# lit des fichiers de parametres facon XX=val
# lit les fichiers de donnees tabules avec la 1e ligne comme nom de colonne
sub LoadData
{
    carp "API LoadData used.";
	my ( $file, $key ) = @_;

	my $o_ndr = New Bio::Narcisse::Utils::NarcisseDataReader();
	$o_ndr->Prepare( -select => [],
					 -from   => $file,
					 -where  => {} );
	my $rh_ = $o_ndr->FetchallHashref( $key );
	##### removed by MANU to make use of the NarcisseDataReader's cache $o_ndr->Finish();

	return $rh_;
}


# return a hash looks like
# $chr => { chr => $chr, length => $length, centro => $centro }
sub get_karyotype
{
    carp "API get_karyoype used";
	my ( $file ) = @_;

	my $o_ndr = New Bio::Narcisse::Utils::NarcisseDataReader();
	$o_ndr->Prepare( -select => [],
					 -from   => $file,
					 -where  => {} );
	my $rh_ = $o_ndr->FetchallHashref( 'chr' );
	##### removed by MANU to make use of the NarcisseDataReader's cache $o_ndr->Finish();

	return $rh_;
}


=head2 ExtractSeqInFasta

 Title        :
 Usage        :
 Prerequisite :
 Fonction     :
 Returns      :
 Args         :
 Globals      :

=cut

sub ExtractSeqInFasta
{
	my ( $ra_pos, $file, $o_fh_in ) = @_;

	my $bool_close = 0;
	my $UNIXGUNZIP = "/bin/gunzip";

	if ( !$o_fh_in )
	{
		`$UNIXGUNZIP $file` if ( $file =~ /\.gz$/ );
		$o_fh_in = new IO::File( $file, 'r' );
		$bool_close = 1;
	}

	$o_fh_in->seek( 0, SEEK_SET );
	my $header = <$o_fh_in>;
	my $decal_header = length($header) if ( $header =~ /^>/ );

	my $seq = "";
	foreach my $pos ( @{$ra_pos} )
	{
		next if ( $pos !~ /\d+\.\.\d+/);

		my ($s,$e) = split /\.\./, $pos;

		my $nb_ln_s = int($s/60);
		my $nb_ln_e = int($e/60);

		my $correctif_ln_start = ( ($s%60) == 0 ) ? 2 : 1;
		my $correctif_ln_end   = ( ($e%60) == 0 ) ? 2 : 1;
		my $seek_pos_s = $decal_header + $s + $nb_ln_s - $correctif_ln_start;
		my $seek_pos_e = $decal_header + $e + $nb_ln_e - $correctif_ln_end;

		$o_fh_in->seek( $seek_pos_s, SEEK_SET );
		my $err_msg = "You try to read outside the file.\n";
		$err_msg .= "the sequence into $file is probably too short to start at $e" ;
		throw LipmError( -text => $err_msg )	if( $o_fh_in->eof() ); # eof est une methode ? a verifier
		while( my $string = <$o_fh_in> )
		{
			chomp($string);
# 			$seq .= lc($string);
			$seq .= $string;

			last	if( $seek_pos_e < $o_fh_in->tell() );
		}

		my $correction = $o_fh_in->tell() - 2 - $seek_pos_e;
		for( my $i=0 ; $i<$correction; $i++ )
		{
			chop($seq);
		}

	}

	$o_fh_in->close()	if ($bool_close);

	return \$seq;
}


sub getLocationFromIndex
{
	my ( $index_file, $pos ) = @_;

	my $index = 0;
	open (INDEX, $index_file) || throw ("Can't open $index_file");
	<INDEX>;

	while (my $line = <INDEX>)
	{
		my @a_elements = split( ' ', $line );
		next	if ($#a_elements < 0);

		last if ( $a_elements[0] eq "last_elt" || $a_elements[0] > $pos );

		$index = $a_elements[1];
	}
	close(INDEX);

	return $index;
}


sub getColumnsPos{
	my $header = shift;

	chomp $header;

	my $rh_keys_pos = {};
  	my $rh_pos_keys = {};

	$header =~ s/^#//;
  	my @a_keys = split("\t", $header);
  	for (my $i = 0; $i <= $#a_keys; $i++){
  		$rh_keys_pos->{ $a_keys[$i] } = $i;
  	}

  	my %rev = reverse %$rh_keys_pos;
  	$rh_pos_keys = \%rev;

  	return ($rh_keys_pos, $rh_pos_keys);

}


sub openFileAndReadHeader{

my @a_ = caller();
die( "it called openFileAndReadHeader => @a_" );
	my $class = shift;
	my $file = shift;

	if (!defined $class->{'_current_file'} || $class->{'_current_file'} ne $file){

		open (FILE, $file) || throw "Unable to open file \"$file\"";

		# reading header line
		my $line = <FILE>;
		throw "$file does not seem to be in a correct format (no header)" if ($line !~ /^#/);
		my ($rh_keys_pos, $rh_pos_keys) = &getColumnsPos($line);

		$class->{'keys_pos'} = $rh_keys_pos;
		$class->{'pos_keys'} = $rh_pos_keys;

		$class->{'_current_file'} = $file;
		open my $fh_temp, "<&FILE";
		$class->file_handler($fh_temp);

		close FILE;
	}
}

1;
