#
#       emmanuel.courcelle@toulouse.inra.fr
#       Created: march 12, 2008
#       $Id$
#

package NarcisseDataReaderTest;

use strict;
use warnings;

use Test::More;		# to have access to all Test::More methods
use Data::Dumper;

use Bio::Narcisse::Utils::NarcisseDataReader;

# A mettre lorsque la classe a tester est finalisee
use constant NB_TEST => 13;

sub New
{
	my $class = shift;
	return bless({}, $class);
}


# - 2 tests-
sub RunStaticVariables
{
 	diag( "RunVariables\n" );
 	my $self = shift;
 	ok( defined( \$Bio::Narcisse::Utils::NarcisseDataReader::SEP ),"class variable \$SEP" );
    ok( defined( \%Bio::Narcisse::Utils::NarcisseDataReader::H_DATA_CACHE ),"class variable \%H_DATA_CACHE" );
	return;
}


# -0 test-
sub RunStaticMethods
{
# 	diag( "RunStaticsMethods\n" );
# 	my $self = shift;
# 
# 	# test la presence des methodes
# # 	can_ok( NarcisseDataReader, qw( methods ) );

	return;
}


# -2 tests-
sub RunConstructors
{
 	diag( "RunConstructors\n" );
 	my $self = shift;
 
    # testing New()
 	my $obj = Bio::Narcisse::Utils::NarcisseDataReader->New();
 	isa_ok( $obj, 'Bio::Narcisse::Utils::NarcisseDataReader', "constructor New");
    my @a_attributes = sort qw( __file __ra_data __rh_headers __ra_headers __ra_wanted_headers __ra_sorted_keys __rh_optimisation __rh_range_condition __ra_condition_values __ra_column_names );
    my @a_keys       = sort keys( %$obj );
    is_deeply( \@a_keys, \@a_attributes, "object attributes presence." );

	return;
}

# -7 tests-
sub RunMethods
{
 	diag( "RunMethods\n" );
 	my $self = shift;
 
 	# check presence of all methods
 	my $object = New Bio::Narcisse::Utils::NarcisseDataReader();
    can_ok( $object, qw (  Prepare Finish FetchallHashref FetchallArrayref GetSortedKeys ));

    # Read whole file
    $object->Prepare( -from => 'chromosome1.synt' );
    
    # FetchallHashref
    my ($ra_data, $rh_data);
    my $rh_should_be_1 =  {
          '6' => {
                   'q_chain' => '12:',
                   'chrom' => '1',
                   'end' => '1963186',
                   'element' => '6',
                   'matching' => '1787',
                   's_start' => '3283042',
                   's_end' => '3287257',
                   's_chrom' => '1',
                   's_order' => '18',
                   'start' => '1958971',
                   'sign' => '-'
                 },
          '4' => {
                   'q_chain' => '9:10:',
                   'chrom' => '1',
                   'end' => '1915797',
                   'element' => '4',
                   'matching' => '26683',
                   's_start' => '1404770',
                   's_end' => '1785646',
                   's_chrom' => '1',
                   's_order' => '6',
                   'start' => '1559087',
                   'sign' => '+'
                 },
          '1' => {
                   'q_chain' => '2:4:',
                   'chrom' => '1',
                   'end' => '1930615',
                   'element' => '1',
                   'matching' => '63204',
                   's_start' => '961719',
                   's_end' => '1077048',
                   's_chrom' => '1',
                   's_order' => '2',
                   'start' => '1068612',
                   'sign' => '+'
                 },
          '3' => {
                   'q_chain' => '7:8:',
                   'chrom' => '1',
                   'end' => '1559086',
                   'element' => '3',
                   'matching' => '18029',
                   's_start' => '1137713',
                   's_end' => '1402851',
                   's_chrom' => '1',
                   's_order' => '4',
                   'start' => '1294979',
                   'sign' => '+'
                 },
          '2' => {
                   'q_chain' => '5:6:',
                   'chrom' => '1',
                   'end' => '1291207',
                   'element' => '2',
                   'matching' => '36930',
                   's_start' => '1077196',
                   's_end' => '1137231',
                   's_chrom' => '1',
                   's_order' => '3',
                   'start' => '1217461',
                   'sign' => '-'
                 },
          '5' => {
                   'q_chain' => '11:',
                   'chrom' => '1',
                   'end' => '1957521',
                   'element' => '5',
                   'matching' => '28436',
                   's_start' => '1789892',
                   's_end' => '1833262',
                   's_chrom' => '1',
                   's_order' => '8',
                   'start' => '1915900',
                   'sign' => '+'
                 }
        };


    $rh_data = $object->FetchallHashref( 'element' );
    #warn Dumper $rh_data;
 
    is_deeply($rh_data,$rh_should_be_1,"FetchallHashref");
    
    # FetchallArrayref
    my $ra_should_be_2 =  [
          {
            'q_chain' => '2:4:',
            'chrom' => '1',
            'end' => '1930615',
            'element' => '1',
            'matching' => '63204',
            's_start' => '961719',
            's_end' => '1077048',
            's_chrom' => '1',
            's_order' => '2',
            'start' => '1068612',
            'sign' => '+'
          },
          {
            'q_chain' => '5:6:',
            'chrom' => '1',
            'end' => '1291207',
            'element' => '2',
            'matching' => '36930',
            's_start' => '1077196',
            's_end' => '1137231',
            's_chrom' => '1',
            's_order' => '3',
            'start' => '1217461',
            'sign' => '-'
          },
          {
            'q_chain' => '7:8:',
            'chrom' => '1',
            'end' => '1559086',
            'element' => '3',
            'matching' => '18029',
            's_start' => '1137713',
            's_end' => '1402851',
            's_chrom' => '1',
            's_order' => '4',
            'start' => '1294979',
            'sign' => '+'
          },
          {
            'q_chain' => '9:10:',
            'chrom' => '1',
            'end' => '1915797',
            'element' => '4',
            'matching' => '26683',
            's_start' => '1404770',
            's_end' => '1785646',
            's_chrom' => '1',
            's_order' => '6',
            'start' => '1559087',
            'sign' => '+'
          },
          {
            'q_chain' => '11:',
            'chrom' => '1',
            'end' => '1957521',
            'element' => '5',
            'matching' => '28436',
            's_start' => '1789892',
            's_end' => '1833262',
            's_chrom' => '1',
            's_order' => '8',
            'start' => '1915900',
            'sign' => '+'
          },
          {
            'q_chain' => '12:',
            'chrom' => '1',
            'end' => '1963186',
            'element' => '6',
            'matching' => '1787',
            's_start' => '3283042',
            's_end' => '3287257',
            's_chrom' => '1',
            's_order' => '18',
            'start' => '1958971',
            'sign' => '-'
          }
        ];
    
    $object->Prepare( -from => 'chromosome1.synt' );
    $ra_data = $object->FetchallArrayref();
    #warn Dumper $ra_data;
    is_deeply($ra_data,$ra_should_be_2,"FetchallArrayref");

    # GetSortedKeys
    my $ra_should_be_3 =  [
    '1',
    '2',
    '3',
    '4',
    '5',
    '6'
    ];
    
    $ra_data = $object->GetSortedKeys();
    #warn Dumper $ra_data;
    is_deeply($ra_data,$ra_should_be_3,"GetSortedKeys");
    
    # FetchallHashref under a range condition, default start_label and end_label  
    $object->Prepare( -from => 'chromosome1.synt',
        -where  => {
            RANGE => { start => 1300000, end => 1920000 }
        });

    my $rh_should_be_4 =  {
        '4' => {
            'q_chain' => '9:10:',
            'chrom' => '1',
            'end' => 1915797,
            'element' => '4',
            'matching' => '26683',
            's_start' => '1404770',
            's_end' => '1785646',
            's_chrom' => '1',
            's_order' => '6',
            'start' => 1559087,
            'sign' => '+'
        },
        '1' => {
            'q_chain' => '2:4:',
            'chrom' => '1',
            'end' => 1930615,
            'element' => '1',
            'matching' => '63204',
            's_start' => '961719',
            's_end' => '1077048',
            's_chrom' => '1',
            's_order' => '2',
            'start' => 1068612,
            'sign' => '+'
        },
        '3' => {
            'q_chain' => '7:8:',
            'chrom' => '1',
            'end' => 1559086,
            'element' => '3',
            'matching' => '18029',
            's_start' => '1137713',
            's_end' => '1402851',
            's_chrom' => '1',
            's_order' => '4',
            'start' => 1294979,
            'sign' => '+'
        },
        '5' => {
            'q_chain' => '11:',
            'chrom' => '1',
            'end' => 1957521,
            'element' => '5',
            'matching' => '28436',
            's_start' => '1789892',
            's_end' => '1833262',
            's_chrom' => '1',
            's_order' => '8',
            'start' => 1915900,
            'sign' => '+'
        }
    };
    
    $rh_data = $object->FetchallHashref();
    is_deeply($rh_data,$rh_should_be_4,"FetchallHashref range condition, default columns");

    # FetchallHashref under a range condition, specifying start_label and end_label   
    $object->Prepare( -from => 'chromosome1.synt',
        -where  => {
            RANGE => { start => 1300000, end => 1920000, start_label => 'start', end_label => 'end' }
        });
    
    $rh_data = $object->FetchallHashref();
    is_deeply($rh_data,$rh_should_be_4,"FetchallHashref range condition, start/end labels specified");
    
    # FetchallArrayref under a condition (element = 2 or s_order = 18)
    my $ra_should_be_5 =  [
          {
            'q_chain' => '5:6:',
            'chrom' => '1',
            'end' => '1291207',
            'element' => '2',
            'matching' => '36930',
            's_start' => '1077196',
            's_end' => '1137231',
            's_chrom' => '1',
            's_order' => '3',
            'start' => '1217461',
            'sign' => '-'
          },
          {
            'q_chain' => '12:',
            'chrom' => '1',
            'end' => '1963186',
            'element' => '6',
            'matching' => '1787',
            's_start' => '3283042',
            's_end' => '3287257',
            's_chrom' => '1',
            's_order' => '18',
            'start' => '1958971',
            'sign' => '-'
          }
        ];
    
    $object->Prepare( -from => 'chromosome1.synt' , -where  => { VALUE => { element => 2, s_order => 18 } });
    $ra_data = $object->FetchallArrayref();
    #warn Dumper $ra_data;
    is_deeply($ra_data,$ra_should_be_5,"FetchallArrayref under condition ");

    
	return;
}


# -2 tests-
sub RunExceptions
{
    my $self = shift;

 	diag( "RunExceptions\n" );
 	use Error qw( :try );
    use LipmError;

    my $object = New Bio::Narcisse::Utils::NarcisseDataReader();
    $object->Prepare( -from => 'chromosome1.synt', -where  => { RANGE => { start => 1300000, end => 1920000 } });
    my $test_name = "A simple error should be thrown by FetchallArrayref";
    try {
        my $rh_result = $object->FetchallArrayref('element');
        fail( "No error thrown ".$test_name);
    }
    catch Error::Simple with
    {
        my ($err) = @_;
        my $msg = $err->text();
        #isa_ok( $err, 'Error::Simple', $test_name.' '.$msg );
        isa_ok( $err, 'Error::Simple',$test_name );
    }
    otherwise 
    {
        my ($err) = @_;
        my $msg = $err->{-text};
        fail( "$test_name: Error thrown ".ref($err)." message = $msg" );
    };
    
    $object->Prepare( -from => 'chromosome1.synt', -where  => { VALUE =>  { element => 2, s_order => 18 } } );
    $test_name = "A simple error should be thrown by FetchallHashref";
    try {
        my $rh_result = $object->FetchallHashref('element');
        fail( "No error thrown ".$test_name);
    }
    catch Error::Simple with
    {
        my ($err) = @_;
        my $msg = $err->text();
        #isa_ok( $err, 'Error::Simple', $test_name.' '.$msg );
        isa_ok( $err, 'Error::Simple',$test_name );
    }
    otherwise 
    {
        my ($err) = @_;
        my $msg = $err->{-text};
        fail( "$test_name: Error thrown ".ref($err)." message = $msg" );
    };
    
	return;
}
  
sub Run
{
	my $self = shift;

	#$self->RunDependencies();
    #print "\n";
	$self->RunStaticVariables();
    print "\n";
	#$self->RunStaticMethods();
    #print "\n";
	$self->RunConstructors();
    print "\n";
	$self->RunMethods();
    print "\n";
	$self->RunExceptions(); 
    print "\n";
	#$self->RunBugs();

	return;
}


1;
