#!/usr/bin/perl

=pod

=head1 NAME

 fetch_ensEMBL.pl

=head1 SYNOPSIS

 perl fetch_ensEMBL.pl @args

 @args:
  --

=head1 DESCRIPTION

 # description

=head1 Author

 Sebastien.letort@toulouse.inra.fr

=head1 Version

 Created: January 21, 2009
 $Id: fetch_ensEMBL.pl 2767 2009-07-24 14:55:22Z sletort $

=cut


use strict;
use warnings;

use Error qw( :try );
use Data::Dumper;

BEGIN
{	# pathway to our modules
	use File::Basename;
	my $dirprg=dirname($0);
	unshift @INC, "$dirprg/..";
	unshift @INC, "$dirprg/../../lib";
}

use Bio::EnsEMBL::Registry;
use Bio::EnsEMBL::Upstream;
# use Bio::EnsEMBL::Utils::ConfigRegistry;
# use Bio::EnsEMBL::DBSQL::DBAdaptor;


# nos modules, nos classes
use LipmError;
use ParamParser;
use General;


# les constantes
use constant VERSION => do {my @r = (q$Rev: 2767 $ =~ /\d+/g); $r[0]};

MAIN:
{
	try
	{
		&Main();
	}
	catch LipmError with
	{
		my ( $o_err, $r ) = @_;

		warn( $o_err->Explain(), "\n" );
		print '*' . $o_err->GetUserMessage() . "*\n";
	}
	otherwise
	{
		my ( $o_err, $r ) = @_;

		my $str = $o_err->file;
		$str   .= ' line ' . $o_err->line;
		$str   .= "\n" . $o_err->text . "\n";

		print STDERR $str;
	};

	exit( 0 );
}


=head2 Procedure Main

 Title        : Main
 Usage        : &Main();
 Prerequisite : none
 Procedure    : the main function
 Args         : none

=cut

sub Main
{
	my $registry = 'Bio::EnsEMBL::Registry';

	$registry->load_registry_from_db(
		-host => 'ensembldb.ensembl.org',
		-user => 'anonymous'
	);

	my $ra_db_adaptors = $registry->get_all_DBAdaptors( -group => 'core' );

	foreach my $o_dba ( @$ra_db_adaptors)
	{
		if( ( $o_dba->species() =~ /rattus/i ) )
# 		 || ( $o_dba->species() =~ /sapiens/i )
# 		 || ( $o_dba->species() =~ /gallus/i )
# 		 || ( $o_dba->species() =~ /taurus/i ) )
		{
# 			&LookForTranscripts( $o_dba );
			&CheckSubSlice( $o_dba );
		}

# 		printf(
# 			"species/group\t%s/%s\n",
# 			$o_dba->species(),   $o_dba->group(),
# 		);
	}

	return;
}


sub LookForTranscripts
{
	my $o_dba = shift;

	my $nb_tr = 0;
	my $nb_tr_pareil = 0;

	print "looking for transcripts for " . $o_dba->species . "\n";
	warn "looking for transcripts for " . $o_dba->species . "\n";

	my $o_slice_adpt = $o_dba->get_SliceAdaptor();

	my @a_args  = ( 'chromosome', '1', 1_000, 1_000_000 );
	my $o_slice      = $o_slice_adpt->fetch_by_region( @a_args );
	return  if( !$o_slice );

	my $rao_genes = $o_slice->get_all_Genes();
	my $n = @$rao_genes;
	print "\tfound $n genes.\n";
	foreach my $o_gene ( @$rao_genes )
	{
		my $g_id = $o_gene->display_id(); #stable_id();
		next  if( !$g_id );
		print "\t gene: $g_id\n";

		my $rao_transc = $o_gene->get_all_Transcripts();
		$n = @$rao_transc;
		print "\t\tfound $n transcripts\n";

		foreach my $o_transc ( @$rao_transc )
		{

# 			&CheckStableId( $o_transc, $g_id, \$nb_tr, \$nb_tr_pareil );
			&LookForUpstream( $o_gene, $o_transc );
		}
		print "\n";
	}


	warn "$nb_tr ont ete trouves, $nb_tr_pareil ont le meme id que leur gene";

	return;
}


sub CheckStableId
{
	my( $o_transc, $g_id, $r_nb_tr, $r_nb_tr_pareil ) = @_;

	my $id = $o_transc->stable_id();
	return  if( !$id );
	print "\t\ttr: $id\n";

	++$$r_nb_tr;
	++$$r_nb_tr_pareil  if( $g_id eq $id );

}


sub LookForUpstream
{
	my( $o_gene, $o_tr ) = @_;

	my ( $start, $end, $strand );

	$start = $o_gene->start;
	$end   = $o_gene->end;
	$strand= $o_gene->strand;
	print "gene $start-$end ($strand)\n";

	$start = $o_tr->start;
	$end   = $o_tr->end;
	$strand= $o_tr->strand;
	print "\ttranscript $start-$end ($strand)\n";

	my $o_upstream = new Bio::EnsEMBL::Upstream( -transcript => $o_tr,
	                                             -length  => 20 );

# 	$start = $o_upstream->start;
# 	$end   = $o_upstream->end;
# 	$strand= $o_upstream->strand;
# 	print "upstream $start-$end ($strand)\n";
# 	print $o_upstream->seq() . "\n";

	$start = $o_upstream->upstart;
	$end   = $o_upstream->upend;
	$strand= $o_upstream->strand;
	my $length = $o_upstream->length;
	print "\tupstream $start-$end soit $length\n"; # ($strand)\n";
# 	print $o_upstream->seq() . "\n";

	my $flag = 0;
	if( $o_gene->strand == 1 )
	{
		print "\t**good\n"  if( $o_upstream->upend <= $o_gene->start );
	}
	else
	{
		print "\t**good\n"  if( $o_upstream->upstart >= $o_gene->end );
	}
}



sub CheckSubSlice
{
	my ( $o_dba ) = @_;

	print "looking for transcripts for " . $o_dba->species . "\n";
	warn "looking for transcripts for " . $o_dba->species . "\n";

	my $o_slice_adpt = $o_dba->get_SliceAdaptor();

	my @a_args  = ( 'chromosome', '1', 1_000, 1_000_000 );
	my $o_slice      = $o_slice_adpt->fetch_by_region( @a_args );
	return  if( !$o_slice );

	my $seq = $o_slice->{ seq };
	warn "ori possede une seq. " . $seq.length  if( $seq );


	my ( $start, $end ) = ( 10_050, 10_070 );
	my $o_slice1 = $o_slice->sub_Slice( $start, $end, -1 );

my @a_keys = qw( start end strand seq seq_region_length seq_region_name );
my %h_ = ();
@h_{ @a_keys } = @$o_slice1{ @a_keys };
warn Dumper \%h_;
	$seq = $o_slice1->{ seq };
	warn "possede une seq. " . $seq.length  if( $seq );

	my $seq1 = $o_slice1->seq;
	my $seq2 = $o_slice->subseq( $start, $end, -1 );

print $seq1 . "\n" . $seq2 . "\n";

	if( $seq1 eq $seq2 )
	{
		print "ok\n"; #never the case
	}
	else
	{
		print "ko\n";
	}


}

__END__
