#!/usr/bin/perl

=pod

=head1 NAME



=head1 SYNOPSIS

 perl fetch_ensEMBL.pl @args

 @args:
  --

=head1 DESCRIPTION

 # description

=head1 Author

 Sebastien.letort@toulouse.inra.fr

=head1 Version

 Created: January 21, 2009
 $Id: qd_check_subslice.pl 2771 2009-07-29 07:12:27Z sletort $

=cut


use strict;
use warnings;

use Error qw( :try );
use Data::Dumper;

BEGIN
{	# pathway to our modules
	use File::Basename;
	my $dirprg=dirname($0);
	unshift @INC, "$dirprg/..";
	unshift @INC, "$dirprg/../../lib";
}

use Bio::EnsEMBL::Registry;
use Bio::EnsEMBL::Upstream;

# nos modules, nos classes
use LipmError;
use ParamParser;
use General;

use Bio::Narcisse::DBAdaptor;

# les constantes
use constant VERSION => do {my @r = (q$Rev: 2771 $ =~ /\d+/g); $r[0]};

MAIN:
{
	try
	{
		&Main();
	}
	catch LipmError with
	{
		my ( $o_err, $r ) = @_;

		warn( $o_err->Explain(), "\n" );
		print '*' . $o_err->GetUserMessage() . "*\n";
	}
	otherwise
	{
		my ( $o_err, $r ) = @_;

		my $str = $o_err->file;
		$str   .= ' line ' . $o_err->line;
		$str   .= "\n" . $o_err->text . "\n";

		print STDERR $str;
	};

	exit( 0 );
}


=head2 Procedure Main

 Title        : Main
 Usage        : &Main();
 Prerequisite : none
 Procedure    : the main function
 Args         : none

=cut

sub Main
{

	#~ my %h_species = ( E => "gallus_gallus", N => 'chick' );
	my %h_species = ( E => "homo_sapiens", N => 'human' );
	#~ my @a_p = ( 10_000, 10_050, 1 );
# 	( $start, $end, $strand )

	# depuis Narcisse
	my $o_dba_N = &GetNarcisseDBA( $h_species{ N } );
	&GetSubSlice( $o_dba_N );

	# depuis ensembl
	my $o_dba_E = &GetEnsemblDBA( $h_species{ E } );
	&GetSubSlice( $o_dba_E );

	return;
}


sub GetSubSlice
{
	my $o_dba = shift;

	warn "pas d'adapteur\n" and next if( !$o_dba );
	
	my $o_slice_adpt = $o_dba->get_SliceAdaptor();
	if( $o_slice_adpt->isa( 'Bio::Narcisse::SliceAdaptor' ) )
	{
		print "adapteur de Narcisse\n";
	}
	else
	{
		print "adapteur de EnsEMBL\n";
	}

	#~ my @a_args  = ( 'chromosome', '1', 1_000, 1_050 );
	my @a_args  = ( 'chromosome', '22', 14597254, 14597254 );
	my $o_slice      = $o_slice_adpt->fetch_by_region( @a_args );
	return  if( !$o_slice );

	print "le slice n'a pas de seq\n"  if( !$o_slice->{ seq } );

	my $seq = $o_slice->seq;
	print $seq . '*****strand ' . $o_slice->strand . "\n";
		
#		my $subseq = $o_slice->subseq( 1_000, 1_010, 1 );
#		my $subseq = $o_slice->subseq( 1, 10, 1 );
#		my $substr = substr( $seq, 0, 10 );
#		
#		print $subseq . "\nvs\n" . $substr . "\n";
#
##		my $o_sub = $o_slice->sub_Slice( @a_p );
#		print "le pt slice n'a pas de seq\n"  if( !$o_sub->{ seq } );
#
#		my $seq1 = $o_slice->subseq( @a_p );
#		my $seq2 = $o_sub->seq();
#
#		if( $seq1 eq $seq2 )
#		{
#			print "ok :: $seq1\n";
#		}
#		else
#		{
#			print "ko\n$seq1\n$seq2\n";
#		}

	print "\nsuivant\n\n";
	
	return;
}


sub GetEnsemblDBA
{
	my ( $species ) = @_;

	my $registry = 'Bio::EnsEMBL::Registry';

	$registry->load_registry_from_db(
		-host => 'ensembldb.ensembl.org',
		-user => 'anonymous'
	);

	my $ra_db_adaptors = $registry->get_all_DBAdaptors();

	foreach my $o_dba ( @$ra_db_adaptors)
	{
		return $o_dba  if( $o_dba->species() =~ /$species/i );
	}

	return undef;
}


sub GetNarcisseDBA
{
	my ($species) = @_;

	my $o_dba = new Bio::Narcisse::DBAdaptor(
										-dbname => $species
									);
warn $o_dba->species();
warn ref( $o_dba );
	return $o_dba;
}

__END__

	my ( $start, $end ) = ( 10_050, 10_070 );
	my $o_slice1 = $o_slice->sub_Slice( $start, $end, -1 );

my @a_keys = qw( start end strand seq seq_region_length seq_region_name );
my %h_ = ();
@h_{ @a_keys } = @$o_slice1{ @a_keys };
warn Dumper \%h_;
	$seq = $o_slice1->{ seq };
	warn "possede une seq. " . $seq.length  if( $seq );

	my $seq1 = $o_slice1->seq;
	my $seq2 = $o_slice->subseq( $start, $end, -1 );

print $seq1 . "\n" . $seq2 . "\n";

	if( $seq1 eq $seq2 )
	{
		print "ok\n"; #never the case
	}
	else
	{
		print "ko\n";
	}


}

__END__
