#!/usr/bin/perl

use strict;
use warnings;

use Error qw( :try );
use Data::Dumper;

BEGIN
{	# pathway to our modules
	use File::Basename;
	my $dirprg=dirname($0);
	unshift @INC, "$dirprg/..";
	unshift @INC, "$dirprg/../../lib";
}

use Bio::EnsEMBL::Registry;

use LipmError;
use General;

use Bio::Narcisse::DBAdaptor;

MAIN:
{
	try
	{
		&Main();
	}
	catch LipmError with
	{
		my ( $o_err, $r ) = @_;

		warn( $o_err->Explain(), "\n" );
		print '*' . $o_err->GetUserMessage() . "*\n";
	}
	otherwise
	{
		my ( $o_err, $r ) = @_;

		my $str = $o_err->file;
		$str   .= ' line ' . $o_err->line;
		$str   .= "\n" . $o_err->text . "\n";

		print STDERR $str;
	};

	exit( 0 );
}

sub Main
{

	my %h_species = ( E => "homo_sapiens", N => 'human_ncbi37_1' );

	# sequence a extraire
	my $ra_seqs = [
		{ chrom => 'MT', start => 1, end => 10 },
		{ chrom => 'MT', start => 145, end => 155 },
		{ chrom => 12, start => 3_048_459, end => 3_048_615 }
	];

	# depuis Narcisse
	print "Narcisse ...\n";
	my $o_dba_N = &GetNarcisseDBA( $h_species{ N } );
	ExtractSeq( $o_dba_N, $ra_seqs );

	# depuis ensembl
	print "EnsEMBL ...\n";
	my $o_dba_E = &GetEnsemblDBA( $h_species{ E } );
	ExtractSeq( $o_dba_E, $ra_seqs );

	return;
}


sub ExtractSeq
{
	my( $o_dba, $ra_seqs ) = @_;

	my $o_slice_adpt = $o_dba->get_SliceAdaptor();

	for my $rh_seq ( @$ra_seqs )
	{
		my @a_args  = @$rh_seq{ qw( chrom start end ) };
		print "looking for : @a_args.\n";
		my $o_slice = $o_slice_adpt->fetch_by_region( 'chromosome', @a_args );
		if( !$o_slice )
		{
			warn "No slice found.\n";
		}
		else
		{
			my $seq = $o_slice->seq;
			if( !$seq )
			{
				print "le slice n'a pas de seq\n";
			}
			else
			{
				print $seq . '*****strand ' . $o_slice->strand . "\n";
			}
		}
	}

	print "\n";

	return;
}


sub GetEnsemblDBA
{
	my ( $species ) = @_;

	my $registry = 'Bio::EnsEMBL::Registry';

	$registry->load_registry_from_db(
		-host => 'ensembldb.ensembl.org',
		-user => 'anonymous'
	);

	my $ra_db_adaptors = $registry->get_all_DBAdaptors();

	foreach my $o_dba ( @$ra_db_adaptors)
	{
		return $o_dba  if( $o_dba->species() =~ /$species/i );
	}

	return undef;
}


sub GetNarcisseDBA
{
	my ($species) = @_;

	my $o_species = New SpeciesObserver( species => $species );
	my $o_dba     = $o_species->GetDBAdaptor();
	
	return $o_dba;
}

