#!/usr/bin/perl

#
#    sebastien.letort@toulouse.inra.fr
#    Created: august 25, 2009
#    $Id: ApiFunctionnalTest.t 2823 2009-09-04 14:31:42Z sletort $
#
#    test file for ApiFunctionnalTest.pm

use strict;
use warnings;

use Data::Dumper;
use FindBin;
use Error qw( :try );

# Lorsque la classe est finalisee, supprimez le no_plan
# use Test::More qw(no_plan);
use Test::More;


use lib "$FindBin::Bin/../../"; # API
use lib "$FindBin::Bin/../../../lib"; # Core/lib

use SpeciesObserver;
use Bio::EnsEMBL::Slice;
use Bio::EnsEMBL::Feature;
use Bio::Narcisse::Utils::NarcisseDataReader::ChromosomeFeatures;

#------------------------------------------

MAIN:
{
# Lorsque la classe est finalisee, decommentez cette ligne
	plan( tests	=>	14 );

	try
	{
		my $o_species = New SpeciesObserver( species => 'human' );

		&CheckSliceAdaptor( $o_species );
		&CheckExonAdaptor ( $o_species );
		&CheckGeneAdaptor ( $o_species );
		&CheckTranscriptAdaptor ( $o_species );
# 		&CheckTranslationAdaptor ( $o_species );
# # 		&CheckExonAdaptor ( $o_species );
	}
	catch LipmError with
	{
		my( $o_err, $r ) = @_;

		warn $o_err->Explain();
	}
	otherwise
	{
		my( $o_err, $r ) = @_;

		warn "error thrown by $o_err->{ -file } on line $o_err->{ -line }\n";
		warn "text : " . $o_err->text;
	};


	exit(0);
}


# 3 tests
=head2 Procedure CheckSliceAdaptor

 Usage        : CheckSliceAdaptor( $o_species )
 Function     : .
 Args         : $o_species
 Error        : none.

=cut

sub CheckSliceAdaptor
{
	my( $o_species ) = @_;

	diag( "Checking SliceAdaptor" );

	&__CheckKaryotype( $o_species );
	&__CheckFetchAll ( $o_species );
	&__CheckFetchByRegion ( $o_species );
	&__CheckFetchByFeature( $o_species );

	return;

	sub __CheckFetchAll
	{
		my( $o_species ) = @_;
		diag( "fetch_all is not tested because not used yet (and quite long to run)." );

# 		my $o_dba = $o_species->GetDBAdaptor( $o_species );
# 		my $o_sla = $o_dba->get_SliceAdaptor();
# 
# 		return;
	}

	sub __CheckFetchByFeature
	{
		my( $o_species ) = @_;
# 		diag( "... next can be long, please wait." );

		my $o_dba = $o_species->GetDBAdaptor( $o_species );
		my $o_sla = $o_dba->get_SliceAdaptor();

		# besoin d'un objet Feature, donc aussi d'un objet Slice
		my %h_args  = ( -start => 1, -end => 1000, -strand => 1,
						-seq_region_name => 'MT',
						-coord_system => $o_dba->default_CoordSystem );
		my $o_slice_ref = new Bio::EnsEMBL::Slice( %h_args );

		%h_args = ( -start => 100, -end => 200, -strand => 1,
					-slice => $o_slice_ref );
		my $o_feature    = new Bio::EnsEMBL::Feature( %h_args );
		my $o_slice_api  = $o_sla->fetch_by_Feature( $o_feature );
		my $ra_slice_api = &GetImportantFeatureAttributes( $o_slice_api );

		my $o_subslice   = $o_slice_ref->sub_Slice( 100, 200, 1 );
		my $ra_slice_ref = &GetImportantFeatureAttributes( $o_subslice );

		TODO:
		{
			local $TODO = "We have a problem to retrieve to sequence of a sub_Slice.";
			is_deeply( $ra_slice_api, $ra_slice_ref, "fetch_by_Feature method" );
		}
	}

	sub __CheckFetchByRegion
	{
		my( $o_species ) = @_;
		diag( "... next can be long, please wait." );

		my $o_dba = $o_species->GetDBAdaptor( $o_species );
		my $o_sla = $o_dba->get_SliceAdaptor();

		my( $chrom, $start, $end, $strand ) = ( '22', 40_951_000, 40_951_400, 1 );

		my $file       = $o_species->GetChromFastaFile( chrom => $chrom );
		my $o_fasta    = New FastaReader( $file );
		my $ra_headers = $o_fasta->GetHeaders();
		my $chrom_seq  = $o_fasta->GetRawSequence( $$ra_headers[0] );
		my $seq_ref    = substr( $chrom_seq, $start-1, ($end-$start+1) );

		my $o_slice    = $o_sla->fetch_by_region( 'chromosome', $chrom, $start, $end, $strand );

		my $ra_slice_ref = [ $start, $end, $strand, $chrom, $seq_ref ];
		my $ra_slice_api = &GetImportantFeatureAttributes( $o_slice );

		is_deeply( $ra_slice_api, $ra_slice_ref, "fetch_by_region method" );

		return;
	}

	sub __CheckKaryotype
	{
		my( $o_species ) = @_;

		my $o_dba = $o_species->GetDBAdaptor( $o_species );
		my $o_sla = $o_dba->get_SliceAdaptor();

		my $o_ndr = New Bio::Narcisse::Utils::NarcisseDataReader();
		my $file = $o_species->GetKaryotypeFile();
		$o_ndr->Prepare( -from => $file );

		my $rh_karyo_api = $o_sla->karyotype();
		my $rh_karyo_ref = $o_ndr->FetchallHashref();
		is_deeply( $rh_karyo_api, $rh_karyo_ref, "karyotype method" );

		return;
	}

} #CheckSliceAdaptor

# 4 tests
=head2 Procedure CheckExonAdaptor

 Usage        : CheckExonAdaptor( $o_species )
 Function     : .
 Args         : $o_species
 Error        : none.

=cut

sub CheckExonAdaptor
{
	my( $o_species ) = @_;

	diag( "Checking ExonAdaptor" );

	&__CheckFetchByDbID( $o_species );
	&__CheckFetchByStableID( $o_species );

	&__CheckFetchAllByTranscript( $o_species );
	&__CheckFetchAllBySlice( $o_species );

	sub __CheckFetchAllByTranscript
	{
		my( $o_species ) = @_;
		diag( "... AllByTranscript, please wait." );

		my $o_dba = $o_species->GetDBAdaptor( $o_species );
		my $o_exa = $o_dba->get_ExonAdaptor();
		my $tr_id = 'human_s22g349t2';

		# pas sur que ce soit très propre d'utiliser une méthode que l'on testera plus tard.
		my $o_tra        = $o_dba->get_TranscriptAdaptor();
		my $o_transcript = $o_tra->fetch_by_dbID( $tr_id );
		my $rao_exons    = $o_exa->fetch_all_by_Transcript( $o_transcript );
		my $ra_exons_api = &GetImportantFeatureAttributesFromList( $rao_exons );

		my $o_ndr = New Bio::Narcisse::Utils::NarcisseDataReader();
		my $chrom = '22'; # je triche, j'ai pas le temps de faire une recherche dans tous les fichiers
		my $file  = $o_species->GetExonFiles( -chrom => $chrom );
		$o_ndr->Prepare( -from => $file,
						 -where => { VALUE => { parent => $tr_id } } );
		my $ra_data      = $o_ndr->FetchallArrayref();
		my $ra_exons_ref = &ExtractImportantFeatureAttributesFromList( $ra_data, 'Exon', $o_species );

		is_deeply( $ra_exons_api, $ra_exons_ref, "fetch_by_Transcript with( $tr_id )" );

		return;
	}

	sub __CheckFetchAllBySlice
	{
		my( $o_species ) = @_;
		diag( "... AllBySlice, please wait." );

		my $o_dba = $o_species->GetDBAdaptor( $o_species );
		my $o_exa = $o_dba->get_ExonAdaptor();

		my( $chrom, $start, $strand ) = ( '22', 30_530_000, 1 );
		my $end   = $start + 1_000;

		# pas sur que ce soit très propre d'utiliser une méthode que l'on testera plus tard.
		my $o_sla   = $o_dba->get_SliceAdaptor();
		my $o_slice = $o_sla->fetch_by_region( 'chromosome', $chrom, $start, $end, $strand );

		my $rao_exons = $o_exa->fetch_all_by_Slice( $o_slice );
		my $ra_exons_api = &GetImportantFeatureAttributesFromList( $rao_exons );

		my $o_ndr = New Bio::Narcisse::Utils::NarcisseDataReader();
		my $file  = $o_species->GetExonFiles( -chrom => $chrom );
		$o_ndr->Prepare( -from => $file,
						 -where => { RANGE => { start => $start, end => $end } } );
		my $ra_data      = $o_ndr->FetchallArrayref();
		my $ra_exons_ref = &ExtractImportantFeatureAttributesFromList( $ra_data, 'Exon', $o_species );

		my $msg = "fetch_by_Slice with( $chrom : [ $start-$end ] )";
		is_deeply( $ra_exons_api, $ra_exons_ref, $msg );

		return;
	}

	sub __CheckFetchByDbID
	{
		my( $o_species ) = @_;
		diag( "... ByDbID, please wait." );

		my $o_dba = $o_species->GetDBAdaptor( $o_species );
		my $o_exa = $o_dba->get_ExonAdaptor();

		my $elt_id = "human_s22g349t2e19";
		my $o_exon = $o_exa->fetch_by_dbID( $elt_id );
		my $ra_exon_api = &GetImportantFeatureAttributes( $o_exon );

		my $o_ndr = New Bio::Narcisse::Utils::NarcisseDataReader();
		my $chrom = '22'; # je triche, j'ai pas le temps de faire une recherche dans tous les fichiers
		my $file = $o_species->GetExonFiles( -chrom => $chrom );
		$o_ndr->Prepare( -from => $file,
						 -where => { VALUE => { element => $elt_id } } );
		my $ra_data = $o_ndr->FetchallArrayref();
		my $ra_exon_ref = &ExtractImportantFeatureAttributes( $$ra_data[0], 'Exon', $o_species );

		is_deeply( $ra_exon_api, $ra_exon_ref, "fetch_by_dbID with( $elt_id )" );

		return;
	}

	sub __CheckFetchByStableID
	{
		my( $o_species ) = @_;
		diag( "... ByStableID, please wait." );

		my $o_dba = $o_species->GetDBAdaptor( $o_species );
		my $o_exa = $o_dba->get_ExonAdaptor();

		my $symbol = "DEPDC5#2_19";
		my $o_exon = $o_exa->fetch_by_stable_id( $symbol );
		my $ra_exon_api = &GetImportantFeatureAttributes( $o_exon );

		my $o_ndr = New Bio::Narcisse::Utils::NarcisseDataReader();
		my $file = $o_species->GetExonFiles( -chrom => '22' );
		$o_ndr->Prepare( -from => $file,
						 -where => { VALUE => { symbol => $symbol } } );
		my $ra_data = $o_ndr->FetchallArrayref();
		my $ra_exon_ref = &ExtractImportantFeatureAttributes( $$ra_data[0], 'Exon', $o_species );

		is_deeply( $ra_exon_api, $ra_exon_ref, "fetch_by_stable_id with( $symbol )" );

		return;
	}

	sub __GetImportantExonAttributesFromList
	{
		my( $rao_exons ) = @_;

		my @a_exons = ();

		foreach my $o_exon ( sort { $a->dbID cmp $b->dbID } @$rao_exons )
		{
			push( @a_exons, &GetImportantFeatureAttributes( $o_exon ) );
		}

		return \@a_exons;
	}

	sub ExtractImportantFeatureAttributesFromList
	{
		my( $rah_exons, $o_species ) = @_;

		my @a_exons = ();

		# pour accelerer le traitement
		my $rh_exon = $$rah_exons[0];
		my $file    = $o_species->GetChromFastaFile( chrom => $$rh_exon{ chrom } );
		my $o_fasta = New FastaReader( $file );
		my $ra_headers = $o_fasta->GetHeaders( );
		my $chrom_seq  = $o_fasta->GetRawSequence( $$ra_headers[0] );

		foreach my $rh_exon ( sort { $$a{ element } cmp $$b{ element } } @$rah_exons )
		{
			my $ra_exon = &ExtractImportantFeatureAttributes( $rh_exon, 'Exon', $o_species, \$chrom_seq );
			push( @a_exons, $ra_exon );
		}

		return \@a_exons;
	}

	return;
} #CheckExonAdaptor


# 7 tests
=head2 Procedure CheckGeneAdaptor

 Usage        : CheckGeneAdaptor( $o_species )
 Function     : .
 Args         : $o_species
 Error        : none.

=cut

sub CheckGeneAdaptor
{
	my( $o_species ) = @_;

	diag( "Checking GeneAdaptor" );

	&__CheckFetchAllBySlice ( $o_species );

	&__CheckFetchByStableId ( $o_species );
	&__CheckFetchByDbId ( $o_species );
	&__CheckFetchBySymbol ( $o_species );
	&__CheckFetchByTranscriptStableId( $o_species );
	&__CheckFetchByTranscriptId( $o_species );
	&__CheckFetchByTranslationStableId( $o_species );

	# note : on pourrait factoriser toutes ces fonctions !

	sub __CheckFetchAllBySlice
	{
		my( $o_species ) = @_;
		diag( "... AllBySlice, please wait." );

		my $o_dba = $o_species->GetDBAdaptor( $o_species );
		my $o_gen_a = $o_dba->get_GeneAdaptor();

		my( $chrom, $start, $strand ) = ( '22', 30_530_000, 1 );
		my $end   = $start + 1_000;

		# pas sur que ce soit très propre d'utiliser une méthode que l'on testera plus tard.
		my $o_sla   = $o_dba->get_SliceAdaptor();
		my $o_slice = $o_sla->fetch_by_region( 'chromosome', $chrom, $start, $end, $strand );

		my $rao_genes = $o_gen_a->fetch_all_by_Slice( $o_slice );
		my $ra_genes_api = &GetImportantFeatureAttributesFromList( $rao_genes );

		my $o_ndr = New Bio::Narcisse::Utils::NarcisseDataReader();
		my $file  = $o_species->GetGeneFiles( -chrom => $chrom );
		$o_ndr->Prepare( -from => $file,
						 -where => { RANGE => { start => $start, end => $end } } );
		my $ra_data      = $o_ndr->FetchallArrayref();
		my $ra_genes_ref = &ExtractImportantFeatureAttributesFromList( $ra_data, 'Gene', $o_species );

		my $msg = "fetch_by_Slice with( $chrom : [ $start-$end ] )";
		is_deeply( $ra_genes_api, $ra_genes_ref, $msg );

		return;
	}

	sub __CheckFetchByTranslationStableId
	{
		my( $o_species ) = @_;

		my $o_dba   = $o_species->GetDBAdaptor( $o_species );
		my $o_gen_a = $o_dba->get_GeneAdaptor();
		my $tr_id  = 'NP_001017437.1';

		my $o_gene		= $o_gen_a->fetch_by_translation_stable_id( $tr_id );
		my $ra_gene_api = &GetImportantFeatureAttributes( $o_gene );

		my $o_ndr = New Bio::Narcisse::Utils::NarcisseDataReader();
		my $chrom = '22'; # je triche, j'ai pas le temps de faire une recherche dans tous les fichiers
		my $file  = $o_species->GetTranscriptFiles( -chrom => $chrom );
		$o_ndr->Prepare( -from => $file,
						 -where => { VALUE => { protein_id => $tr_id } } );
		my $rh_trans    = $o_ndr->FetchallHashref();

		my $gene_id = $$rh_trans{ $chrom }{ parent };
		$file       = $o_species->GetGeneFiles( -chrom => $chrom );
		$o_ndr->Prepare( -from => $file,
						 -where => { VALUE => { element => $gene_id } } );
		my $ra_data     = $o_ndr->FetchallArrayref();
		my $ra_gene_ref = &ExtractImportantFeatureAttributes( $$ra_data[0], 'Gene', $o_species );

		is_deeply( $ra_gene_api, $ra_gene_ref, "fetch_by_transcript_id with( $tr_id )" );

		return;
	}

	sub __CheckFetchByTranscriptId
	{
		my( $o_species ) = @_;

		my $o_dba   = $o_species->GetDBAdaptor( $o_species );
		my $o_gen_a = $o_dba->get_GeneAdaptor();
		my $tr_id  = 'human_s22g310t1';

		my $o_gene		= $o_gen_a->fetch_by_transcript_id( $tr_id );
		my $ra_gene_api = &GetImportantFeatureAttributes( $o_gene );

		my $o_ndr = New Bio::Narcisse::Utils::NarcisseDataReader();
		my $chrom = '22'; # je triche, j'ai pas le temps de faire une recherche dans tous les fichiers
		my $file  = $o_species->GetTranscriptFiles( -chrom => $chrom );
		$o_ndr->Prepare( -from => $file,
						 -where => { VALUE => { element => $tr_id } } );
		my $rh_trans    = $o_ndr->FetchallHashref();

		my $gene_id = $$rh_trans{ $chrom }{ parent };
		$file       = $o_species->GetGeneFiles( -chrom => $chrom );
		$o_ndr->Prepare( -from => $file,
						 -where => { VALUE => { element => $gene_id } } );
		my $ra_data     = $o_ndr->FetchallArrayref();
		my $ra_gene_ref = &ExtractImportantFeatureAttributes( $$ra_data[0], 'Gene', $o_species );

		is_deeply( $ra_gene_api, $ra_gene_ref, "fetch_by_transcript_id with( $tr_id )" );

		return;
	}

	sub __CheckFetchByTranscriptStableId
	{
		my( $o_species ) = @_;

		my $o_dba   = $o_species->GetDBAdaptor( $o_species );
		my $o_gen_a = $o_dba->get_GeneAdaptor();
		my $tr_id  = 'LOC550631';

		my $o_gene		= $o_gen_a->fetch_by_transcript_stable_id( $tr_id );
		my $ra_gene_api = &GetImportantFeatureAttributes( $o_gene );

		my $o_ndr = New Bio::Narcisse::Utils::NarcisseDataReader();
		my $chrom = '22'; # je triche, j'ai pas le temps de faire une recherche dans tous les fichiers
		my $file  = $o_species->GetGeneFiles( -chrom => $chrom );
		$o_ndr->Prepare( -from => $file,
						 -where => { VALUE => { locus_tag => $tr_id } } );
		my $ra_data     = $o_ndr->FetchallArrayref();
		my $ra_gene_ref = &ExtractImportantFeatureAttributes( $$ra_data[0], 'Gene', $o_species );

		is_deeply( $ra_gene_api, $ra_gene_ref, "fetch_by_dbID with( $tr_id )" );

		return;
	}

	sub __CheckFetchByDbId
	{
		my( $o_species ) = @_;

		my $o_dba   = $o_species->GetDBAdaptor( $o_species );
		my $o_gen_a = $o_dba->get_GeneAdaptor();
		my $elt_id  = 'human_s22g310';

		my $o_gene		= $o_gen_a->fetch_by_dbID( $elt_id );
		my $ra_gene_api = &GetImportantFeatureAttributes( $o_gene );

		my $o_ndr = New Bio::Narcisse::Utils::NarcisseDataReader();
		my $chrom = '22'; # je triche, j'ai pas le temps de faire une recherche dans tous les fichiers
		my $file  = $o_species->GetGeneFiles( -chrom => $chrom );
		$o_ndr->Prepare( -from => $file,
						 -where => { VALUE => { element => $elt_id } } );
		my $ra_data     = $o_ndr->FetchallArrayref();
		my $ra_gene_ref = &ExtractImportantFeatureAttributes( $$ra_data[0], 'Gene', $o_species );

		is_deeply( $ra_gene_api, $ra_gene_ref, "fetch_by_dbID with( $elt_id )" );

		return;
	}

	sub __CheckFetchBySymbol
	{
		my( $o_species ) = @_;

		my $o_dba   = $o_species->GetDBAdaptor( $o_species );
		my $o_gen_a = $o_dba->get_GeneAdaptor();
		my $symbol  = 'LOC550631';

		my $o_gene		= $o_gen_a->fetch_by_symbol( $symbol );
		my $ra_gene_api = &GetImportantFeatureAttributes( $o_gene );

		my $o_ndr = New Bio::Narcisse::Utils::NarcisseDataReader();
		my $chrom = '22'; # je triche, j'ai pas le temps de faire une recherche dans tous les fichiers
		my $file  = $o_species->GetGeneFiles( -chrom => $chrom );
		$o_ndr->Prepare( -from => $file,
						 -where => { VALUE => { locus_tag => $symbol } } );
		my $ra_data     = $o_ndr->FetchallArrayref();
		my $ra_gene_ref = &ExtractImportantFeatureAttributes( $$ra_data[0], 'Gene', $o_species );

		is_deeply( $ra_gene_api, $ra_gene_ref, "fetch_by_symbol with( $symbol )" );

		return;
	}

	sub __CheckFetchByStableId
	{
		my( $o_species ) = @_;

		my $o_dba   = $o_species->GetDBAdaptor( $o_species );
		my $o_gen_a = $o_dba->get_GeneAdaptor();
		my $gene_id = 'LOC550631';

		my $o_gene		= $o_gen_a->fetch_by_stable_id( $gene_id );
		my $ra_gene_api = &GetImportantFeatureAttributes( $o_gene );

		my $o_ndr = New Bio::Narcisse::Utils::NarcisseDataReader();
		my $chrom = '22'; # je triche, j'ai pas le temps de faire une recherche dans tous les fichiers
		my $file  = $o_species->GetGeneFiles( -chrom => $chrom );
		$o_ndr->Prepare( -from => $file,
						 -where => { VALUE => { locus_tag => $gene_id } } );
		my $ra_data     = $o_ndr->FetchallArrayref();
		my $ra_gene_ref = &ExtractImportantFeatureAttributes( $$ra_data[0], 'Gene', $o_species );

		is_deeply( $ra_gene_api, $ra_gene_ref, "fetch_by_stable_id with( $gene_id )" );

		return;
	}

	sub GetImportantFeatureAttributesFromList
	{
		my( $rao_genes ) = @_;

		my @a_genes = ();

		foreach my $o_gene ( sort { $a->dbID cmp $b->dbID } @$rao_genes )
		{
			push( @a_genes, &GetImportantFeatureAttributes( $o_gene ) );
		}

		return \@a_genes;
	}

	sub ExtractImportantFeatureAttributesFromList
	{
		my( $rah_genes, $o_species ) = @_;

		my @a_genes = ();

		# pour accelerer le traitement
		my $rh_gene = $$rah_genes[0];
		my $file    = $o_species->GetChromFastaFile( chrom => $$rh_gene{ chrom } );
		my $o_fasta = New FastaReader( $file );
		my $ra_headers = $o_fasta->GetHeaders( );
		my $chrom_seq  = $o_fasta->GetRawSequence( $$ra_headers[0] );

		foreach my $rh_gene ( sort { $$a{ element } cmp $$b{ element } } @$rah_genes )
		{
			my $ra_gene = &ExtractImportantFeatureAttributes( $rh_gene, $o_species, \$chrom_seq );
			push( @a_genes, $ra_gene );
		}

		return \@a_genes;
	}

	return;
} #CheckGeneAdaptor


=head2 Procedure CheckTranscriptAdaptor

 Usage        : CheckTranscriptAdaptor( $o_species )
 Function     : .
 Args         : $o_species
 Error        : none.

=cut

sub CheckTranscriptAdaptor
{
	my( $o_species ) = @_;

	diag( "Checking TranscriptAdaptor" );

# 	&__CheckFetchByDbId( $o_species );
# 	&__CheckFetchByStableId( $o_species );
# 	&__CheckFetchByTranslationId( $o_species );
# 	&__CheckFetchByTranslationStableId( $o_species );
# 
# 	&__CheckFetchAllBySlice ( $o_species );
# 	&__CheckFetchAllByGene ( $o_species );

# 	sub __CheckFetchByDbId
# 	{
# 		my( $o_species ) = @_;
# 
# 		my $o_dba  = $o_species->GetDBAdaptor( $o_species );
# 		my $o_tr_a = $o_dba->get_TranscriptAdaptor();
# 		my $elt_id = 'human_s22g361t1';
# 
# 		my $o_transcript = $o_tr_a->fetch_by_dbID( $elt_id );
# 		my $ra_trans_api = &__GetImportantTranscriptAttributes( $o_gene );
# 
# 		my $o_ndr = New Bio::Narcisse::Utils::NarcisseDataReader();
# 		my $chrom = '22'; # je triche, j'ai pas le temps de faire une recherche dans tous les fichiers
# 		my $file  = $o_species->GetGeneFiles( -chrom => $chrom );
# 		$o_ndr->Prepare( -from => $file,
# 						 -where => { VALUE => { element => $elt_id } } );
# 		my $ra_data     = $o_ndr->FetchallArrayref();
# 		my $ra_gene_ref = &ExtractImportantFeatureAttributes( $$ra_data[0], $o_species );
# 
# 		is_deeply( $ra_gene_api, $ra_gene_ref, "fetch_by_dbID with( $elt_id )" );
# 
# 		return;
# 	}


	return;
} #CheckTranscriptAdaptor


# =head2 Procedure CheckTranslationAdaptor
# 
#  Usage        : CheckTranslationAdaptor( $o_species )
#  Function     : .
#  Args         : $o_species
#  Error        : none.
# 
# =cut
# 
# sub CheckTranslationAdaptor
# {
# 	my( $o_species ) = @_;
# 
# 	diag( "Checking TranslationAdaptor" );
# 
# 	return;
# } #CheckTranslationAdaptor
# 

# == tools functions == #

=head2 GetImportantFeatureAttributes

 Usage    : my $ra_feature = $o_x->GetImportantFeatureAttributes( $o_feature )
 Function : .
 Returns  : $ra_feature
 Args     : $o_feature, API feature object
 Error    : none

=cut

sub GetImportantFeatureAttributes
{
	my( $o_feature ) = @_;

	my @a_feature = ( $o_feature->start, $o_feature->end, $o_feature->strand );

	my $ref = ref( $o_feature );
	if( $ref =~ /Slice/ )
	{
		push( @a_feature, $o_feature->seq_region_name, $o_feature->seq );
	}
	elsif( $ref =~ /Exon/ )
	{
		push( @a_feature, $o_feature->slice->seq_region_name,
						  $o_feature->dbID,
						  $o_feature->stable_id,
						  $o_feature->seq->seq );
	}
	elsif( $ref =~ /Gene/ )
	{
		push( @a_feature, $o_feature->slice->seq_region_name,
						  $o_feature->dbID,
						  $o_feature->stable_id,
						  $o_feature->external_name,
						  $o_feature->seq );
	}

	return \@a_feature;
} #GetImportantFeatureAttributes


=head2 Procedure ExtractImportantFeatureAttributes

 Usage        : $ra_feature = ExtractImportantFeatureAttributes( $rh_feature, $feature, $o_species, $r_fasta )
 Function     : .
 Args         :
     $rh_feature, key are column names.
     $feature, 'Gene', 'Exon', ...
     $o_species,
     $r_fasta, ref to the sequence associated with the feature, optionnal
 Error        : none.

=cut

sub ExtractImportantFeatureAttributes
{
	my( $rh_feature, $feature, $o_species, $r_fasta ) = @_;

	my @a_fields  = qw( start end strand chrom element );
	push( @a_fields, 'locus_tag' )  unless( $feature eq 'Exon' );
	push( @a_fields, 'symbol' );
	my @a_feature = ( @$rh_feature{ @a_fields } );

	#extraction de la sequence
	# 1- obtenir la sequence du chromosome
	my $chrom_seq;
	if( !defined $r_fasta )
	{
		my $file   = $o_species->GetChromFastaFile( chrom => $$rh_feature{ chrom } );
		my $o_fasta    = New FastaReader( $file );
		my $ra_headers = $o_fasta->GetHeaders( );
		$chrom_seq     = $o_fasta->GetRawSequence( $$ra_headers[0] );
	}
	else
	{
		$chrom_seq = $$r_fasta;
	}

	# 3- extraire la sous-sequence correspondant à l'exon
	my $start  = $$rh_feature{ start } - 1;
	my $length = $$rh_feature{ end } - $start;
	my $seq    = substr( $chrom_seq, $start, $length );
	push( @a_feature, $seq );

	return \@a_feature;
} #ExtractImportantFeatureAttributes


__END__
