#! /usr/bin/perl

BEGIN
{			
		use File::Basename;
		my $dirprg=dirname($0);
		unshift @INC, "$dirprg/../../lib";
		unshift @INC, "$dirprg/../../API";
}

use strict;
use warnings;
use Data::Dumper;
use NARCISSE_utils;
use Bio::Narcisse::DBAdaptor;
use Bio::SeqIO;
use Bio::Narcisse::Utils::ScriptUtils;

use ConfigurationFactory;

my $O_PARAM;

use constant AvailableOptions => ( "dbID=s", "stable_id=s", "translation_stable_id=s", "symbol=s", "transcript_id=s", 
									  "transcript_stable_id=s", "exon_stable_id=s", "translation_id=s", "region=s" );

MAIN:{
	
	$O_PARAM =  New ParamParser;
    $O_PARAM->SetUsage(my $usage=sub { &_Usage(); } );
    
    # Update from ENV, for $NARCISSE_CFG
    $O_PARAM->Update('ENV','O'); 
	
	my @a_ProgramArguments=("cfg=s","verbose","adaptor=s","seq","verbose", (AvailableOptions));
		
	$O_PARAM->Update('GETOPTLONG',"O", @a_ProgramArguments );

	$O_PARAM->AssertDefined("cfg");
	$O_PARAM->AssertDefined("adaptor");
	
	# Init Configuration
	# my $o_io = ConfigurationFactory::GetConfiguration();
    my $o_io = New InstanceObserver();
    #my $rh_cfg = $o_io->GetConfigHash();
    #%CFG = %$rh_cfg;
    
	&NARCISSE_utils::AssertCfgParam($o_io,$O_PARAM,'cfg');
	my $file_arg = $O_PARAM->Get('cfg');
	$O_PARAM->Update($file_arg,'A');
		
	my $species = $O_PARAM->Get('ORG_SHORT');

    my $o_so = $o_io->GetSpeciesObserver( $species );
	my $o_dba= $o_so->GetDBAdaptor();
	die "Impossible de creer un Bio::Narcisse::DBAdaptor..." if (! defined $o_dba);

    #TODO s'assurer que l'espèce est bien valide pour l'instance étudiée
		
	my $adaptor = $O_PARAM->Get('adaptor');
	my $adaptor_name = "get_\u$adaptor"."Adaptor";
	my $feature_adaptor = $o_dba->$adaptor_name;
		
	print STDERR "Testing  \u$adaptor"."Adaptor\n".('-' x 15)."\n";
	#Getting available methods
	my @a_AvailableFetchbyMethods=&GetAdaptorFetchByMethods( $feature_adaptor );	
	
	unless ( &ExistsOptParam($O_PARAM,\@a_AvailableFetchbyMethods) )
	{
			&_Usage();
			print STDERR "Error: At least of the following additional optional arguments must be set :\n\t".join(" ",@a_AvailableFetchbyMethods)."\n";
			exit(1);
	}
	
	&AdaptorTest($feature_adaptor,\@a_AvailableFetchbyMethods);
	
}


#########################################################################################
#		Routines
#########################################################################################

=head2 function GetAdaptorFetchByMethods

 Title        : GetAdaptorFetchByMethods
 Usage        : GetAdaptorFetchByMethods($fature_adaptor)
 Prerequisite : none
 Fonction     : retrieves the fetch by methods of the given adaptor
 Returns      : an array with the methods
 Args         : an Bio::Narcisse::XXXXAdaptor
 Globals      : none

=cut
sub GetAdaptorFetchByMethods
{
	my $feature_adaptor = shift;
	
	my @fetchby_methods = ();
	
	#Using the Universal package defined below try to fetch all the available methods, keeping only "fetch_by" methods
	for my $method_name ( $feature_adaptor->methods() )
	{
		if ( $method_name=~/fetch_by_([a-z]{1}\w+$)/g )
		{
			push @fetchby_methods, $1;
		}
	}
	
	return @fetchby_methods;
}


=head2 function AdaptorTest

 Title        : AdaptorTest
 Usage        : AdaptorTest($fature_adaptor)
 Prerequisite : none
 Fonction     : test the Adaptor with the fetch method specified from the command line argument
 Returns      : none
 Args         : an Bio::Narcisse::XXXXAdaptor
 Globals      : $O_PARAM

=cut

sub AdaptorTest
{
	my $feature_adaptor = shift;
	my $ra_attributes= shift;
		
	# For each defined method retrieve the corresponding information
	foreach my $attribute ( @$ra_attributes )
	{
		if ( $O_PARAM->IsDefined( $attribute ) ) 
		{
			my $accessor = "fetch_by_$attribute";
			my $attribute_value = $O_PARAM->Get( $attribute );
			print STDERR "  ==> $accessor('$attribute_value')\n";
			my @attribute_values=();
			if ($attribute eq "region")
			{
				my $region = $attribute_value;
				my ( $chrom, $start, $end, $strand);
				unless ( ( $chrom, $start, $end, $strand)=( $region=~/(\w+)\:(\d+)\-(\d+)\:(\d)/) )
				{
						print STDERR "Error: region attribute must be of the form : \"chr:start-end:strand\" \n";
						&_Usage();
						exit(1);
				}
				@attribute_values = ( 'chromosome', $chrom, $start, $end, $strand );
			}
			else
			{
				@attribute_values = ( $attribute_value );
			}

			my $feature = $feature_adaptor->$accessor( @attribute_values );
			
			# If the seq flag is on, the corresponding seq will be printed in fasta format			
			if ( $O_PARAM->IsDefined( 'seq' ) ) 
			{
			 	my $seq_out = Bio::SeqIO->new(	-fh     => \*STDOUT,
                               				-format => 'fasta');
			 	my $seq=$feature->seq;
	
			 	if  ( $seq->isa('Bio::Seq') )   #The seq method of the gene or slice object returns a string and not a seq object
			 	{				 		
					$seq_out->write_seq($seq);
				 } 
				 else
				 {
				 	my $seq_obj = Bio::Seq->new(-seq => $seq, -display_id => $feature->display_id());
					$seq_out->write_seq($seq_obj); 
				 }
			}
			else
			{
				print "\t".&feature2string( $feature )."\n";
			}
		}
	}	
}

#--
# The following methods are designed for printing the corresponding 
# feature information.
#--

sub ExistsOptParam
{	
	my $O_PARAM = shift ;
	my $ra_attributes = shift;
	
	my $bool_opt_param=0;
	
	foreach my $attribute (@$ra_attributes)
	{
		$bool_opt_param |= ( $O_PARAM->IsDefined($attribute) );
	}
		
	return $bool_opt_param;
}

#--
# The following methods are designed for printing the corresponding 
# feature information.
#--

sub feature2string
{
	my $f = shift;

	my $str;
	if( !defined $f )
	{
		$str = 'undef!';
	}
	elsif ( ref($f) =~ /RepeatFeature/ )
	{
		$str = __RepeatFeature2string($f);
	}
	elsif ( ref($f) =~ /Slice/ )
	{
		$str = __Slice2string($f);
	}
	elsif ( ref($f) =~ /Translation/ )
	{
		$str = __Translation2string($f);
	}
	elsif ( ref($f) =~ /Transcript/ )
	{
		$str = __Transcript2string($f);
	}
	else
	{
		$str = __feature2string($f);
	}

	return $str;
}

sub __feature2string
{

	my $f = shift;
		
	my $display_id = $f->dbID();
	my $stable_id = $f->stable_id();
	my $seq_region = $f->slice->seq_region_name();
	my $slice_start= $f->slice->start();
	my $slice_end= $f->slice->end();
	my $start = $f->start ;
	my $end = $f->end;
	my $strand = $f->strand;
	
	return "$stable_id ($display_id): $seq_region(slice : $slice_start-$slice_end): $start-$end ($strand)";
	
}

sub __Transcript2string
{

	my $f = shift;
		
	my $display_id = $f->dbID();
	my $stable_id = $f->stable_id();
	my $seq_region = $f->slice->seq_region_name();
	my $slice_start= $f->slice->start();
	my $slice_end= $f->slice->end();
	my $consistency = $f->GetConsistencyErrorMessage();
	my $start = $f->start ;
	my $end = $f->end;
	my $strand = $f->strand;
	
	return "$stable_id ($display_id): $seq_region(slice : $slice_start-$slice_end): $start-$end ($strand)\n $consistency";
	
}


sub __RepeatFeature2string
{

	my $f = shift;
		
	my $display_id = $f->dbID();
	my $seq_region = $f->slice->seq_region_name();
	my $start = $f->hstart ;
	my $end = $f->hend;
	my $strand = $f->hstrand;
	
	return "$display_id : $seq_region:$start-$end ($strand)";
	
}

sub __Slice2string
{

	my $f = shift;
		
	my $display_id = $f->display_id();
	my $seq_region = $f->seq_region_name();
	my $start = $f->start ;
	my $end = $f->end;
	my $strand = $f->strand;
	
	return "$display_id : $seq_region:$start-$end ($strand)";
	
}

sub __Translation2string
{

	my $f = shift;
		
	my $display_id = $f->dbID();
	my $start = $f->genomic_start;
	my $end = $f->genomic_end;
	my $cdna_start =  $f->cdna_start;
	my $cdna_end =  $f->cdna_end;
	
	return "$display_id : :$start-$end (cdna : $cdna_start-$cdna_end)";
	
}


=head2 procedure USAGE

 Title        : USAGE
 Usage        : USAGE($code)
 Prerequisite : none
 Fonction     : fonction usage
 Returns      : none
 Args         : none
 Globals      : none

=cut
sub _Usage()
{
    $O_PARAM->SelectNameSpace('');
    my $narcisse_cfg = $O_PARAM->Get('NARCISSE_CFG');
	
    print <<END

NARCISSE_CFG = $narcisse_cfg

Usage : $0

mandatory:
	--cfg		: species configuration file or genome short name 
	--adaptor	: adaptor to test ( Exon, Gene,  Repeat, Slice, Transcript, Translation ) 
optional
	for gene, exon, transcript, repeat 
		--dbID	
	only for gene, exon, transcript	 
		--stable_id	
	only for gene, transcript	
		--translation_stable_id
	only for gene	
		--symbol	
		--transcript_id 
		--transcript_stable_id 
		--exon_stable_id
	only for transcript
		--translation_id
	only for slice
		--region    in the following format \"chr:start-end:strand\"
	--seq	(flag) when this option is invoqued the corresponding fasta formated 
	              sequences provided as the only output.

Note : gene_adaptor needs a slice_adaptor.

END
;	
}

package UNIVERSAL;

    use strict;

    sub methods {
        my ($class, $types) = @_;
        $class = ref $class || $class;
        $types ||= '';
        my %classes_seen;
        my %methods;
        my @class = ($class);

        no strict 'refs';
        while ($class = shift @class) {
            next if $classes_seen{$class}++;
            unshift @class, @{"${class}::ISA"} if $types eq 'all';
            # Based on methods_via() in perl5db.pl
            for my $method (grep {not /^[(_]/ and 
                                  defined &{${"${class}::"}{$_}}} 
                            keys %{"${class}::"}) {
                $methods{$method} = wantarray ? undef : $class->can($method); 
            }
        }
      
        wantarray ? keys %methods : \%methods;
    }

    1;
