#!/usr/bin/perl

# $Id: test.pl 294 2007-02-19 14:42:45Z faraut $

BEGIN
{			
    use File::Basename;
    my $dirprg=dirname($0);
	unshift @INC, "$dirprg/../../lib";
	unshift @INC, "$dirprg/../../API";
	unshift @INC, "$dirprg/../../API/Bio/Narcisse/Utils";
}

use strict;
use warnings;

use Cwd;
use Error qw(:try);
use File::Basename;
use File::Path qw( mkpath );
use Data::Dumper;

use NARCISSE_utils;
use Sufine;
use Logger;
use ParamParser;
use FastaReader;

use Bio::Narcisse::DBAdaptor;

my $O_PARAM;
my $LOGGER;
my $VERBOSE = 0;
my $WARNING_VERBOSE = 0;
my $ONLY_CONSISTENCY = 0;
my $NUM_SLICE;
my $TEST_ALL_CHROMOSOMES=0;
my @SELECTED_CHROMS=();

my $FH_LOG;

use constant FAMOUS_GENE => "IGF1";
use constant NUM_SLICE => "5";
use constant SLICE_SIZE => "30000";
use constant VALID_PROPORTION => "0.4";

my %CFG;
MAIN:
{

    &ParamParser::SetDefaultBehaviour('use_substitution_table');
    $O_PARAM =  New ParamParser;
    $O_PARAM->SetUsage(my $usage=sub { &_Usage(); } );
    
    # Update from ENV, for $NARCISSE_CFG
    $O_PARAM->Update('ENV','O'); 
	
	$O_PARAM->Update('GETOPTLONG',"O",("cfg=s","v","wv","num_slice=s","all","log","chr=s","region=s","only_consistency"));
			
    $O_PARAM->AssertDefined('NARCISSE_CFG');
    $O_PARAM->AssertFileExists('NARCISSE_CFG');
    $O_PARAM->SetUnlessDefined('v',$VERBOSE);
    $O_PARAM->SetUnlessDefined('wv',$WARNING_VERBOSE);
	
	$O_PARAM->SetUnlessDefined('num_slice',&NUM_SLICE);
   		
	# Init global variables
    my $o_io = New InstanceObserver();
    my $rh_cfg = $o_io->GetConfigHash();
    %CFG = %$rh_cfg;
    
	&NARCISSE_utils::AssertCfgParam($o_io,$O_PARAM,'cfg');
	my $file_arg = $O_PARAM->Get('cfg');
	$O_PARAM->Update($file_arg,'A');
	
	$VERBOSE=$O_PARAM->Get('v');
	$WARNING_VERBOSE=$O_PARAM->Get('wv');
	
	$ONLY_CONSISTENCY=$O_PARAM->Get('only_consistency');
		
	$NUM_SLICE=$O_PARAM->Get('num_slice');
		
	my $log_dir="$CFG{ROOT_LOGS}/api";
	mkpath ($log_dir);
	
	my $species = $O_PARAM->Get('ORG_SHORT');				
		
	if ($O_PARAM->IsDefined("log"))
	{		
		$FH_LOG=New Sufine(-dir =>  $log_dir, -ext => 'log', -name => ">$species");			
	}
	else
	{
		$FH_LOG= new IO::File ">&=STDOUT";
	}
	select $FH_LOG; $|=1;
		 
    my $o_so = $o_io->GetSpeciesObserver( $species );
	my $o_dba= $o_so->GetDBAdaptor();
	die "Impossible to create un Bio::Narcisse::DBAdaptor... for $species" if (! defined $o_dba); 
	
	print  $FH_LOG "Species $species \n";
	   	
	my $ra_slices=&SelectChromosomes($O_PARAM,$o_dba);  		   	

	# $rdata_status est une ref de hash
	my $rdata_status=&test_Species($o_dba,$ra_slices);

	if( $$rdata_status{ proteins } != 0 )
	{
		my $rounded_proportion = sprintf("%.1f", 100*$rdata_status->{valid_proteins}/$rdata_status->{proteins});
		if ($rdata_status->{valid_proteins}/$rdata_status->{proteins}>VALID_PROPORTION && $rdata_status->{nb_errors}==0 ) {
			print $FH_LOG "PASSED :  the API test was successful, $rdata_status->{valid_proteins} valid proteins (i.e without in-frame stop codons) out of $rdata_status->{proteins} ($rounded_proportion%)\n";
		} 	
		elsif ( $rdata_status->{nb_errors}==0 ) {	
			print $FH_LOG  "\nWARNING: Invalid proportion of valid proteins : only $rdata_status->{valid_proteins} valid out of $rdata_status->{proteins} ($rounded_proportion%)\n";
		}	
		else
		{
			print $FH_LOG  "\nERROR: $rdata_status->{nb_errors} unvalid proteins annotated as consistent \n";
			print $FH_LOG  "$rdata_status->{valid_proteins} valid proteins (i.e without in-frame stop codons) out of $rdata_status->{consistent_proteins} predicted valid proteins and $rdata_status->{transcripts} transcripts \n";
		}
	}

	$FH_LOG->close();

	exit( 0 );
}

=head2 procedure test_Species

 Title        : test_Species
 Usage        : test_Species($species_name)
 Prerequisite : none
 Fonction     : test the SliceAdptor on all chromosomes
 Returns      : the referenced has returned by test_Sliceadaptator
 Args         : a species short name (terminology of Narcisse)

 Globals      : none

=cut
sub test_Species
{
	my $o_dba = shift;
	my $ra_slices = shift;
			
	my $rdata_status=&test_Sliceadaptator($o_dba,$ra_slices);
	
	return 	$rdata_status;
}


=head2 function test_Sliceadaptator

 Title        : test_Sliceadaptator
 Usage        : test_Sliceadaptator($dba,$gene)
 Prerequisite : none
 Fonction     : test the SliceAdptor on a random subset of 5 SLICE_SIZE bases regions in the genome
 Returns      : a reference on a hash providing information concerning the valid proteins observed (np stop codon in phase)
 Args         : a DBAdaptor
 Globals      : none

=cut
sub test_Sliceadaptator
{
	my $o_dba=shift;  
	my $ra_slices= shift;
	my %status=();
	
	my $slice_adaptor = $o_dba->get_SliceAdaptor;
	
	foreach my $hslice (@$ra_slices) {
		my $slice=$slice_adaptor->fetch_by_region('chromosome', $hslice->{chr}, $hslice->{start}, $hslice->{end});
		$status{$hslice}=&test_Slice($o_dba,$slice);
	}
	
	my $nb_valid_proteins=0;
	my $nb_consistent_proteins=0;
	my $nb_proteins=0;
	my $nb_transcripts=0;
	my $nb_errors=0;
	foreach my $hslice (@$ra_slices) {
		$nb_valid_proteins+=$status{$hslice}{nb_valid_proteins};
		$nb_consistent_proteins+=$status{$hslice}{nb_consistent_proteins};
		$nb_proteins+=$status{$hslice}{nb_proteins};
		$nb_transcripts+=$status{$hslice}{nb_transcripts};
		$nb_errors+=$status{$hslice}{nb_errors};
	}
	
	print $FH_LOG  "STATUS $nb_valid_proteins valid proteins (i.e without in-frame stop codons) out of $nb_proteins proteins and $nb_transcripts transcripts \n\n" if ($VERBOSE);	
	
	my %data_status=(valid_proteins => $nb_valid_proteins, consistent_proteins => $nb_consistent_proteins, proteins => $nb_proteins, transcripts => $nb_transcripts, nb_errors => $nb_errors);
	
	return \%data_status;
	
}


=head2 procedure test_Slice

 Title        : test_Slice
 Usage        : test_Slice($dba,$slice)
 Prerequisite : none
 Fonction     : test the selected Slice
 Returns      : none
 Args         : the DBAdaptor, and a slice
 Globals      : none

=cut
sub test_Slice
{
 	my $o_dba = shift;
 	my $slice = shift;
 	
 	my %status=();

	print $FH_LOG "Testing slice ".$slice->name."\n";
	&verbose(('-' x 35)."\n\n");
	
	my $ra_genes = $slice->get_all_Genes;
	my $nb       = @$ra_genes;
	&verbose( "\tNumber of genes: $nb\n" );
	
	my $ra_transcripts = $slice->get_all_Transcripts;
	$nb          = @$ra_transcripts;
	&verbose("\tNumber of transcripts: $nb\n");
	my $nb_transcripts=scalar @$ra_transcripts;
	
	my $ra_exons = $slice->get_all_Exons;
	$nb          = @$ra_exons;
	&verbose("\tNumber of exons: $nb\n");

	my $ra_repeats = $slice->get_all_RepeatFeatures;
	$nb          = @$ra_repeats;
	&verbose("\tNumber of repeats: $nb\n");
	
	my $seq = $slice->seq;
	&verbose("\tLength of the sequence: ".length($seq)."\n");
	
	my $gene_adaptor = $o_dba->get_GeneAdaptor;
	my $translation_adaptor = $o_dba->get_TranslationAdaptor;
		
	my $ra_gene = $gene_adaptor->fetch_all_by_Slice($slice);
	my $nb_proteins= 0;
	my $nb_consistent_proteins = 0;
	my $nb_valid_proteins=0;	
	my $nb_errors = 0;
	foreach my $gene (@$ra_gene){
		&verbose("\tGene: ".&feature2string($gene)."\n");
		my $ra_transcripts=$gene->get_all_Transcripts;
		foreach my $transcript (@$ra_transcripts) {
			
			&verbose("\t\ttranscript: ".&feature2string($transcript)."\n");		
			
			my $ra_exons=$transcript->get_all_Exons;
			foreach my $exon (@$ra_exons) {
				&verbose("\t\t\texon:".&feature2string($exon)."\n");			
			}
			my $translation=$translation_adaptor->fetch_by_Transcript($transcript);
			&verbose("\t\ttranslation:".&translationfeature2string($translation)."\n");
						
			if ($translation->start > 0 && $translation->end > 0) {											
				
				my $protein=$translation->seq;
				$nb_consistent_proteins++ if ( $transcript->IsConsistent );
				$nb_proteins++;
				my $isvalid=&validprotein($protein);
				
				if ( $isvalid ) {
 				
 					$nb_valid_proteins++;
 					&verbose("Valid protein : ".$translation->dbID."\n");
					
					if ( $transcript->IsConsistent )
					{						
						&GetAllAboutTranscript($transcript) unless ( $ONLY_CONSISTENCY );
					}						
					
				} elsif ( $transcript->IsConsistent ) {
					
					$nb_errors++;		
					my $msg = "WARNING, UnValid Consistent protein : \n";
					print $FH_LOG "\t!! $msg";
					print "\t".$transcript->dbID." ".$transcript->GetConsistencyErrorMessage()."\n";
					print $FH_LOG "\t!! Translation: ".&translationfeature2string($translation)."\n\t!! From transcript: ".&feature2string($transcript)."\n";
					&warning_verbose(&sequence2string($protein)."\n");
							
				}

			}
		}
	}
	my $repeat_adaptor = $o_dba->get_RepeatFeatureAdaptor;

    $ra_repeats = $repeat_adaptor->fetch_all_by_Slice($slice);
	foreach my $repeat (@$ra_repeats){
		&verbose("\t".&repeatfeature2string($repeat,"repeat")."\n");
	}
	
	
	$status{nb_errors}=$nb_errors;
	$status{nb_valid_proteins}=$nb_valid_proteins;
	$status{nb_consistent_proteins}=$nb_consistent_proteins;
	$status{nb_proteins}=$nb_proteins;
	$status{nb_transcripts}=$nb_transcripts;

	print $FH_LOG "\t$nb_valid_proteins valid proteins (i.e without in-frame stop codons) out of $nb_proteins predicted valid proteins and $nb_transcripts transcripts\n";					
						
	return \%status;
}


sub SelectChromosomes
{
	my $o_param = shift;
	my $o_dba = shift;
	my @selected_chromosomes=();
	my $region = undef;
	
	my $slice_adaptor = $o_dba->get_SliceAdaptor;
	
	my $karyotype = $slice_adaptor->karyotype;
	my @chromosomes= keys %$karyotype;
	
	if ($o_param->IsDefined('all'))
	{		
		@selected_chromosomes=@chromosomes;
		print $FH_LOG "Performing a test on all chromosomes \n";
		
	} elsif ( $O_PARAM->IsDefined('chr') )
	{
	 	@selected_chromosomes = split(",",$O_PARAM->Get('chr'));
	 	&verbose("Performing a test on the following chromosome(s): ".join(" ",@selected_chromosomes)." \n");
	} elsif ( $O_PARAM->IsDefined('region') )
	{
		$region = $O_PARAM->Get('region');
		my ($chr) = ( $region =~ /^(\w+)/ );
		@selected_chromosomes=	( $chr );	
	}
	else
	{
		&verbose("Testing $NUM_SLICE slices \n");	
		foreach my $i (1..$NUM_SLICE) {
			push @selected_chromosomes, $chromosomes[int (rand $#chromosomes)];
		}	
	}
	my @slices=();	
	foreach my $chrom (@selected_chromosomes) {
		if ($o_param->IsDefined('all') || $O_PARAM->IsDefined('chr') )
		{
			push @slices, {chr => $chrom, start => 1, end => $karyotype->{$chrom}{ length }};
		} elsif ( defined $region )
		{
			my ($chr,$start,$end) = ( $region =~ /^(\w+)\:(\d+)\-(\d+)$/ );
		 	push @slices, {chr => $chrom, start => $start, end => $end };
			print "$region \n";
			&verbose("Chrom $chrom slice : $start-$end  \n");
		} else
		{
			my $chrom_slice_size=$karyotype->{$chrom}{ length }/20;
			my $rand_pos = int( rand( $karyotype->{$chrom}{ length } - $chrom_slice_size ) );
			push @slices, {chr => $chrom, start => $rand_pos, end => $rand_pos+$chrom_slice_size};
			&verbose("Chrom $chrom slice :$rand_pos-".($rand_pos+$chrom_slice_size) ." \n");
		}		
	}	
	
	return \@slices;
}


=head2 function validprotein

 Title        : validprotein
 Usage        : &validprotein($protein_sequence)
 Prerequisite : none
 Fonction     : return a string with a description of the feature
 Returns      : none
 Args         : a feature
 Globals      : none

=cut
sub validprotein {
	my $protein_sequence=shift;
	
	return ($protein_sequence=~/\*/) ? 0 : 1;
}

=head2 function feature2string

 Title        : feature2string
 Usage        : &feature2string($feature)
 Prerequisite : none
 Fonction     : return a string with a description of the feature
 Returns      : none
 Args         : a feature
 Globals      : none

=cut
sub feature2string{

	my $f = shift;
	
	my $db_id = $f->dbID();
	my $stable_id = $f->stable_id();
	my $seq_region = $f->slice->seq_region_name();
	my $start = $f->start;
	my $end = $f->end;
	my $strand = $f->strand;
	
	return "$stable_id ($db_id): $seq_region:$start-$end ($strand)";
	
}

=head2 function sequence2string

 Title        : sequence2string
 Usage        : &sequence2string($feature)
 Prerequisite : none
 Fonction     : returns a formatted sequence 60 carachter per line 
 Returns      : the formatted sequence as a string
 Args         : a sequence as string
 Globals      : none

=cut
sub sequence2string{

	my $s = shift;
	my $str="";

	
	while ( $s =~ s/^([\w\*]{60})// )
	{
		$str.=$1."\n";
	}
	$str.=$s."\n";
	
	return $str;
	
}

=head2 function repeatfeature2string

 Title        : repeatfeature2string
 Usage        : &repeatfeature2string($feature)
 Prerequisite : none
 Fonction     : return a string with a description of the feature
 Returns      : none
 Args         : a feature
 Globals      : none

=cut
sub repeatfeature2string{

	my $f = shift;
	
	my $display_id = $f->display_id();
	my $seq_region = $f->slice->seq_region_name();
	my $start = $f->hstart;
	my $end = $f->hend;
	my $strand = $f->hstrand;
	
	return "$display_id: $seq_region:$start-$end ($strand)";
	
}

=head2 function translationfeature2string

 Title        : translationfeature2string
 Usage        : &translationfeature2string($feature)
 Prerequisite : none
 Fonction     : return a string with a description of the feature
 Returns      : none
 Args         : a feature
 Globals      : none

=cut
sub translationfeature2string{

	my $f = shift;
	return ""	if( !$f );
	
	my $db_id = $f->dbID();
	my $stable_id = $f->stable_id();
	my $start = $f->start;
	my $end = $f->end;
	
	return "$stable_id ($db_id): $start-$end";
	
}

=head2 function chromosomefeature2string

 Title        : feature2string
 Usage        : &chromosomefeature2string($feature)
 Prerequisite : none
 Fonction     : return a string with a description of the feature
 Returns      : none
 Args         : a feature
 Globals      : none

=cut
sub chromosomefeature2string{

	my $f = shift;
	
	return $f->name();	
}

=head2 function GetAllAboutTranscript

 Title        : GetAllAboutTranscript
 Usage        : $msg = &GetAllAboutTranscript( $o_transcript );
 Prerequisite : none
 Function     : none
 Returns      : $msg
 Args         : $o_transcript
 Globals      : none

=cut

sub GetAllAboutTranscript
{
	my ( $o_transcript ) = @_;
	
	my ($r_our, $api);
	my $id = $o_transcript->display_xref;

	my $bool_error = 0;
	my $bool_absent = 0;

	my $msg="Sequences are not identical:";
	
	#transcript
	$r_our = GetOurSequence( $o_transcript->display_xref, $CFG{ TRANSCRIPT } );
	$bool_absent=(! defined $r_our);
	&warning_verbose($o_transcript->dbID. " is absent from $CFG{TRANSCRIPT} \n") if ( $bool_absent );
	$api = $o_transcript->seq->seq;
	$bool_error |= ($bool_absent  || 0 == &AreSequencesIdenticals( $r_our, \$api, 'transcript', $o_transcript->display_xref ) );

	#cds
	$r_our = GetOurSequence( $o_transcript->external_name, $CFG{ CDS } );
	$bool_absent=(! defined $r_our);
	&warning_verbose($o_transcript->dbID. " is absent from $CFG{CDS} \n") if ( $bool_absent );
	$api = $o_transcript->translateable_seq();
	$bool_error |= 	($bool_absent  || 0 == &AreSequencesIdenticals( $r_our, \$api, 'cds', $o_transcript->external_name ) );

	#protein
	$r_our = GetOurSequence( $o_transcript->external_name, $CFG{ PROT } );
	$bool_absent=(! defined $r_our);
	&warning_verbose($o_transcript->dbID. " is absent from $CFG{PROT} \n") if ( $bool_absent );
	$api = $o_transcript->translate()->seq();
	$bool_error |=	($bool_absent  || 0 == &AreSequencesIdenticals( $r_our, \$api, 'protein', $o_transcript->external_name ) );

	&warning_verbose($o_transcript->dbID. " is incorrect \n") if ( $bool_error );

	return ""; #$msg;
}


=head2 function GetOurSequence

 Title        : GetOurSequence
 Usage        : $seq = &GetOurSequence( $id, $type );
 Prerequisite : none
 Function     : none
 Returns      : $seq
 Args         : $type
                $id
 Globals      : none

=cut

sub GetOurSequence
{
	my ( $id, $type ) = @_;
	my $short = $O_PARAM->Get('ORG_SHORT');
	my $file_ext;

	if( $type eq $CFG{ PROT } )
	{	$file_ext = $CFG{ FASTA_PROT };	}
	elsif( $type eq $CFG{ CDS } )
	{	$file_ext = $CFG{ FASTA_CDS };	}
	elsif( $type eq $CFG{ TRANSCRIPT } )
	{	$file_ext = $CFG{ FASTA_TRANSCRIPT };	}
	else
	{	warn "$type is not known"; return;	}

	my $db = $CFG{ ROOT_DATA } . "/$short/" . $CFG{ DB } . "/$type/$short";
	$db .= '_' . $file_ext;

	my $o_reader = New FastaReader( $db );
	my $header_id = "\Q$id"; # \Q permet de ne pas considérer $id comme une regex
	my $ra_headers = $o_reader->GetHeaders( $header_id );
	warn "$id is absent or not unique in fasta db $db " and return	undef if( 1 != @$ra_headers );

	my $seq = $o_reader->GetRawSequence( $$ra_headers[0] );

	return \$seq;
}


=head2 function CompareSequence

 Title        : CompareSequence
 Usage        : $bool = &CompareSequence( $type, $id );
 Prerequisite : none
 Function     : none
 Returns      : $bool
 Args         : $type, $id
 Globals      : none

=cut

sub AreSequencesIdenticals
{
	my ( $r_our_seq, $r_api_seq, $type, $id ) = @_;

	if( ( !$r_api_seq ) or ( !$r_our_seq )
		or ( uc($$r_our_seq) ne uc($$r_api_seq) ) )
	{
		print $FH_LOG  "\t!! WARNING : $id $type is different than the one returned by the API\n";
			&warning_verbose("api_seq: ".uc($$r_api_seq)." \n");
			&warning_verbose("db_seq : ".uc($$r_our_seq)." \n");
		return 0;
	}

	return 1;
}

sub verbose
{
	my $msg=shift;
	
	print $FH_LOG $msg if ($VERBOSE);
}

sub warning_verbose
{
	my $msg=shift;
	
	print $FH_LOG $msg if ($WARNING_VERBOSE);
}



=head2 procedure USAGE

 Title        : USAGE
 Usage        : USAGE($code)
 Prerequisite : none
 Fonction     : fonction usage
 Returns      : none
 Args         : none
 Globals      : none

=cut
sub _Usage()
{
    $O_PARAM->SelectNameSpace('');
    my $narcisse_cfg = $O_PARAM->Get('NARCISSE_CFG');
	my $num_slice = &NUM_SLICE;
	
    print <<END

NARCISSE_CFG = $narcisse_cfg

Usage : $0

mandatory:
    --cfg  : species configuration file or genome short name 
optional
	--num_slice        : the number of slices (default = $num_slice)
	--all              : if set the program will check the consistency of the proteins of all chromosomes
	--chr              : the set of chromosomes (comma separated) to test i.e. 1,2,3 
	--region           : a specific region in the following format chr:start-end
	--only_consistency : does not compare computed sequences to sequences in the db directory
	--v                : being verbose on all the test and warnings
	--wv               : being verbose on the warnings only
	--log              : log the outputs (stderr and stdout) to a log file in the standard log directory (Site/logs/api)

After a complete installation (ie after ./nc_install.pl), and the construction of one or more species data repositories
you may run this script to check that the API is working fine on one of these species. 
Example:
$0 --cfg chick
will check a randomly selected subset of $num_slice slices of the chricken genome
END
;	
}
