#!/usr/bin/perl

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod

=head1 NAME

 ncs_GetCDSsFromLocusTags.pl

=head1 SYNOPSIS

 perl ncs_GetCDSsFromLocusTags.pl @args

 @args:
  --

=head1 DESCRIPTION

 # description

=head1 Author

 Sebastien.letort@toulouse.inra.fr

=head1 Version

 Created: March 2, 2009
 $Id$

=cut


BEGIN
{	# pathway to our modules
	use File::Basename;
	my $dirprg=dirname($0);

	# pour Appli
	unshift @INC, "$dirprg/../../multi/web_services/playmoby/lib/";

	# bien mettre ../lib apres playmoby sinon pb avec ParamParser.
	unshift @INC, "$dirprg/../lib";
	unshift @INC, "$dirprg/../API";
}

# ----------------------------------------------------------------
# -- -- Specific Multiservice Class for this NarcisseService -- --

package MultiService::ThroughSpecies::GetCDSsFromLocusTags;

use strict;
use warnings;

use Data::Dumper;

use base qw( MultiService::ThroughSpecies );

use NarcisseService::GrepBased::GetCDSsFromLocusTags;

use constant OUT_NOT_FOUND => 'not_found_file';
use constant TAGS          => 'locus_tags';
use constant TAGS_FILE     => &TAGS . '_file';
use constant DB            => 'db';
# use constant SPECIES       => 'species';


=head2 function _GetHelpParams

 Usage        : $string = $o_xx->__GetHelpParams();
 Prerequisite : none
 Function     : none
 Returns      : $string, description of parameters
 Args         : none

=cut

sub _GetHelpParams
{
	my $self   = shift;

	my $string = $self->SUPER::_GetHelpParams();

	my $format = '%-20s';

	my $p_tags   = sprintf( $format, &TAGS . '|' . &TAGS_FILE );
	$string .= '
    --' . $p_tags . ' : string, locus tags or name of the file containing one locus tags per line.';

	return $string;
}


=head2 function _GetHelpOptions

 Usage        : $string = $o_xx->__GetHelpOptions();
 Prerequisite : none
 Function     : none
 Returns      : $string, description of options
 Args         : none

=cut

sub _GetHelpOptions
{
	my $self   = shift;

	my $string = $self->SUPER::_GetHelpOptions();

	my $format = '%-20s';

	my $p_not_found = sprintf( $format, &OUT_NOT_FOUND );
# 	my $p_species   = sprintf( $format, &SPECIES );

	$string .= '
    --' . $p_not_found . ' : string, name of the file where not found entries will be written ( this will be empty as it is here just to have a standard output with all accessors ).';
#     --' . $p_species . ' : string, species name, short/long';

	return $string;
}


# =head2 Procedure
# 
#  Title        : _InitAttributes
#  Usage        : $o_xx->_InitAttributes( $o_param );
#  Prerequisite : none
#  Procedure    : none
#  Args         : $o_param
# 
# =cut
# 
# sub _InitAttributes
# {
# 	my $self = shift;
# 	my ( $o_param ) = @_;
# 
# 	$self->SUPER::_InitAttributes( $o_param );
# 
# # 	my $str_species = $o_param->Get( &SPECIES );
# # 
# # 	$self->_Set( __species => $str_species );
# 
# 	return;
# } # _InitAttributes
# 

=head2 _FetchAndSetMandatories

 Usage        : $o_x->_FetchAndSetMandatories( $o_param )
 Function     : Read mandatories attributes and throw Error if they are missing.
                Then set attributes with values.
 Args         :
     $o_param
 Error        : none.

=cut

sub _FetchAndSetMandatories
{
	my $self   = shift;
	my( $o_param ) = @_;

	$self->SUPER::_FetchAndSetMandatories( $o_param );

	my $ra_tags  = $self->_FetchArgs( $o_param->Get( &TAGS, &TAGS_FILE ) );

	if( 0 == @$ra_tags )
	{
		my $msg = "You have to specify a list of locus tags --" . &TAGS;
		$msg   .= " or --" . &TAGS_FILE . "\n";
		my $dbg = "No tags were found.\n$msg";
		throw LipmError( -text => $dbg, -user_msg => $msg );
	}
	$self->_Set( __ra_tags => $ra_tags );


	my $not_found = $o_param->Get( &OUT_NOT_FOUND );

	# attention pour le site web, on ne peut pas mettre de fichier de sortie (a verifier)
	my $fs_out;
	if( $not_found eq '' )
	{
# 		my $msg = "You need to specify an output file for not_found entries.";
# 		my %h_msg = ( -user_msg => $msg );
# 		throw LipmError::ParameterException( &OUT_NOT_FOUND,'missing', %h_msg );
		use IO::Handle;
		my $o_handle = new IO::Handle();
		$fs_out = $o_handle->fdopen( fileno( STDERR ), 'w' ); # il y a pe + simple
	}
	else
	{
		$fs_out = new IO::File( $not_found, 'w' )
						or throw LipmError::IOException( $not_found, 'f' );
	}
	my $rh_out = { filehandle => { not_found => $fs_out } };
	$self->_Set( __rh_out => $rh_out ); # pas le temps de faire + propre

	return;
}


=head2 _BuildNarcisseService

 Usage        : my $o_ns = $o_x->_BuildNarcisseService( %h_args ).
 Prerequiste  : none
 Function     : This function must build and return a NarcisseService object.
 Returns      : $o_ns, NarcisseService object
 Args         : %h_params,
     o_instance_obs => $o_inst_obs,
 Error        : none.

=cut

sub _BuildNarcisseService
{
	my $self   = shift;

	my $rh_current = $self->_GetCurrentData();

	# je ne verifie pas que les params sont definis, je fais confiance.
# 	my( $ra_tags, $species ) = $self->_Get( qw( __ra_tags __species ) );
	my $ra_tags = $self->_Get( qw( __ra_tags ) );

	my %h_params = ( ra_tags   => $ra_tags,
	                 -o_logger => $self->_GetLogger(),
	                 -o_instance_obs => $$rh_current{ o_inst_obs },
	                 -species  => $$rh_current{ species } );
# 	$h_params{ -species } = $species  if( $species ne '' );

	my $o_ns = New GetCDSsFromLocusTags( %h_params );

	return $o_ns;
}


=head2 Procedure

 Title        : _LaunchServiceOnInstance
 Usage        : $o_xx->_LaunchServiceOnInstance( $ra_species );
 Prerequisite : none
 Procedure    : none
 Args         : $ra_species

 Note         : copy-paste of the mother class method + store not_found

=cut

sub _LaunchServiceOnSpecies
{
	my $self = shift;
	my ( $o_inst_obs ) = @_;

# 	my %h_params = ( o_instance_obs => $o_inst_obs );

	my $o_ns = $self->_BuildNarcisseService();

	my ( $ra_results, $rh_results ) = $o_ns->FetchResults();
	$self->_StoreResults( { hash => $rh_results, list => $ra_results } );

	my $nb = @$ra_results;
	my $msg   = "found $nb results\n"; # ! faux si data => []
	$self->_Trace( $msg, '.', 'ISV' );

	my $ra_not_found = $o_ns->FetchNotFound();
	$self->__StoreNotFound( $ra_not_found );
	$nb = @$ra_not_found;
	$msg   = "not found $nb results\n"; # ! faux si data => []
	$self->_Trace( $msg, '.', 'ISV' );

	return;
}


# =head2 _PrintHash
# 
#  Usage        : $o_x->_PrintHash( $rh_results, $ra_results )
#  Function     : .
#  Args         :
#      $rh_results
#      $ra_results
#  Error        : none.
# 
# =cut
# 
# sub _PrintHash
# {
# 	my $self   = shift;
# 	my( $rh_results, $ra_results ) = @_;
# 
# 	my $fh_out = $self->_GetFileAttribute();
# 
# 	foreach my $locus_tags ( keys %$rh_results )
# 	{
# 		my $ra_primers = $$rh_results{ $locus_tags };
# 
# 		print $fh_out $locus_tags, "\t#", join( ';', @$ra_primers ), "\n";
# 	}
# 
# 	return;
# }


=head2 Procedure

 Title        : __PrintOutNotFound
 Usage        : $o_xx->__PrintOutNotFound();
 Prerequisite : none
 Procedure    : none
 Args         : none

=cut

sub __PrintOutNotFound
{
	my $self     = shift;

	my $rh_out = $self->_Get( '__rh_out' );
	my $o_fs = $$rh_out{ filehandle }{ not_found };

	my $ra_not_found = $$rh_out{ ra_not_found };
	return if( 0 == @$ra_not_found );

	foreach my $rh_data ( @$ra_not_found )
	{
		my( $primer, $ra_regions ) = each( %$rh_data );
		print $o_fs $primer, "\t", join( ", ", @$ra_regions ), "\n";
	}

	return;
}


# ======= copy paste of 2 methods of MultiService::Accessor ========
# ======= To deal with not_found results =======
# ==> don't have time nor envy to think of a better way <==

=head2 Procedure _PrintOutResults

 Usage        : $o_xx->_PrintOutResults( none );
 Prerequisite : none
 Procedure    : call _PrintArray or _PrintHash on every results
 Args         : none

=cut

sub _PrintOutResults
{
	my $self    = shift;

	#found
	$self->SUPER::_PrintOutResults();

	#not_found
	$self->__PrintOutNotFound();

	return;
}


=head2 Procedure __StoreNotFound

 Usage        : $o_xx->__StoreNotFound( $ra_data );
 Prerequisite : none
 Procedure    : push $ra_data into __rh_out attribute (key ra_not_found).
 Args         : $ra_data, a list of hash ref

=cut

sub __StoreNotFound
{
	my $self = shift;
	my ( $ra_data ) = @_;

	my $rh_out = $self->_Get( '__rh_out' );

	my $ra_not_found = $$rh_out{ ra_not_found } || [];

	push( @$ra_not_found, @$ra_data );

	$$rh_out{ ra_not_found } = $ra_not_found;

	return;
}


1;

# ----------------------------------------------------------------

use strict;
use warnings;

use Error qw( :try );
use Data::Dumper;
use Cwd;

# nos modules, nos classes
use LipmError;
use General;

use NarcisseService::GrepBased::GetAnnotationFromLocusTags;

# les constantes
use constant VERSION => do {my @r = (q$Rev$ =~ /\d+/g); $r[0]};

use constant DEBUG => &TRUE;
# use constant DEBUG => &FALSE;
# $LipmError::Error::Debug = &DEBUG;

MAIN:
{
	try
	{
		&Main();
	}
	catch LipmError with
	{
		my ( $o_err, $r ) = @_;

		if( &TRUE == &DEBUG )
		{
			warn( $o_err->Explain(), "\n", $o_err->stacktrace() );
# 			warn( $o_err->Explain(), "\n" );
		}
		else
		{	print '*' . $o_err->GetUserMessage() . "*\n";	}
	}
	otherwise
	{
		my ( $o_err, $r ) = @_;

		my $str = $o_err->file;
		$str   .= ' line ' . $o_err->line;
		$str   .= "\n" . $o_err->text . "\n";

		print STDERR $str;
	};

	exit( 0 );
}


=head2 Procedure Main

 Title        : Main
 Usage        : &Main();
 Prerequisite : none
 Procedure    : the main function
 Args         : none

=cut

sub Main
{
	my $service_name = 'NarcisseGetCDSsFromLocusTags';
	my $cmd = &Cwd::abs_path( $0 );
	my %h_general = (
			name    => $service_name,
			descr   => "It extracts the CDS' sequences corresponding to the given locus tags. many CDS if alternative transcription.",
			authors => 'sebastien.letort@toulouse.inra.fr',
			doc     => 'no doc yet',
			version => '1.0',
			cmd     => $cmd
		);

	# ouputs
	my %h_outputs = (
			'MultiFastaFile' => {
				descr => "the multi fasta file containing the genes' sequences.",
				namespace    => 'Narcisse',
				type_biomoby => 'FASTA_NA',
				datatype     => 'text',
				cmd          => '--' . &MultiService::OUT_FILE . '=$value'
			},
			'NotFoundFile' => {
				descr => "the file containing the locus tags not found.",
				namespace    => 'Narcisse',
				type_biomoby => 'List_Text',
# 				datatype     => 'text-plain',
				cmd          => '--' . &OUT_NOT_FOUND . '=$value'
			}
		);
	#input
	my %h_inputs = (
			'LocusTagsFile' => {
	                descr        => "The locus tags to search.",
	                namespace    => 'Narcisse',
	                type_biomoby => 'List_Text',
# 	                datatype     => 'string',
	                cmd          => '--' . &TAGS_FILE . '=$value'
			}
		);

	my %h_params = ( rh_descr  => { -general => \%h_general,
									-outputs => \%h_outputs,
									-inputs  => \%h_inputs },
# 	                rh_help    => { params   => $params_str,
# 	                           		options  => $options_str,
# 	                           		base     => MultiService::GetBasicHelp($0)},
					prog_name  => $0,
	                ra_options => [ #&SPECIES . '=s',
	                                &TAGS . '=s',
		                            &TAGS_FILE . '=s',
		                            &OUT_NOT_FOUND . '=s' ],
	                log_file   => $service_name . '.log' );

	my $o_service = New MultiService::ThroughSpecies::GetCDSsFromLocusTags( %h_params );
	$o_service->Launch()  if( defined $o_service );

	return;
}


__END__
