#!/usr/bin/perl

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod

=head1 NAME

 ncs_GetTranscriptsFromLocusTags.pl

=head1 SYNOPSIS

 perl ncs_GetTranscriptsFromLocusTags.pl @args

 @args:
  --

=head1 DESCRIPTION

 # description

=head1 Author

 Sebastien.letort@toulouse.inra.fr

=head1 Version

 Created: March 2, 2009
 $Id: ncs_GetTranscriptsFromLocusTags.pl 3142 2009-12-22 12:38:52Z manu $

=cut


use strict;
use warnings;

use Error qw( :try );
use Data::Dumper;
use Cwd;

BEGIN
{	# pathway to our modules
	use File::Basename;
	my $dirprg=dirname($0);

	# pour Appli
	unshift @INC, "$dirprg/../../multi/web_services/playmoby/lib/";

	# bien mettre ../lib apres playmoby sinon pb avec ParamParser.
	unshift @INC, "$dirprg/../lib";
	unshift @INC, "$dirprg/../API";
}

# nos modules, nos classes
use LipmError;
use General;

use MultiService::Accessor;
use NarcisseServiceKeywords qw( &NS_TRANSCRIPTS );

# les constantes
use constant VERSION => do {my @r = (q$Rev: 3142 $ =~ /\d+/g); $r[0]};

use constant DEBUG => &TRUE;
# use constant DEBUG => &FALSE;

MAIN:
{
	try
	{
		&Main();
	}
	catch LipmError with
	{
		my ( $o_err, $r ) = @_;

		if( &TRUE == &DEBUG )
		{
			warn( $o_err->Explain(), "\n", $o_err->stacktrace() );
		}
		else
		{	print '*' . $o_err->GetUserMessage() . "*\n";	}
	}
	otherwise
	{
		my ( $o_err, $r ) = @_;

		my $str = $o_err->file;
		$str   .= ' line ' . $o_err->line;
		$str   .= "\n" . $o_err->text . "\n";

		print STDERR $str;
	};

	exit( 0 );
}


=head2 Procedure Main

 Title        : Main
 Usage        : &Main();
 Prerequisite : none
 Procedure    : the main function
 Args         : none

=cut

sub Main
{
	my $service_name = 'NarcisseGetTranscriptsFromLocusTags';
	my $cmd = &Cwd::abs_path( $0 ) . ' --';
	$cmd   .= &MultiService::Accessor::OUT_TYPE . '=fasta';
	$cmd   .= &MultiService::Accessor::SAFE;
	my %h_general = (
			name    => $service_name,
			descr   => "It extracts the Transcripts' sequences corresponding to the given locus tags.",
			authors => 'sebastien.letort@toulouse.inra.fr',
			doc     => 'no doc yet',
			version => '1.0',
			cmd     => $cmd
		);

	# ouputs
	my %h_outputs = (
			'MultiFastaFile' => {
				descr => "the multi fasta file containing the Transcripts' sequences.",
				namespace    => 'Narcisse',
				type_biomoby => 'FASTA_NA_multi',
				datatype     => 'text',
				cmd          => '--' . &MultiService::OUT_FILE . '=$value'
			},
			'NotFoundFile' => {
				descr => "the file containing the locus tags not found.",
				namespace    => 'Narcisse',
				type_biomoby => 'List_Text',
# 				datatype     => 'text-plain',
				cmd          => '--' . &MultiService::Accessor::OUT_NOT_FOUND . '=$value'
			}
		);

	# Lequel ?
	my $o_service = New MultiService::Accessor(
	                                  what    => &NS_TRANSCRIPTS,
	                                  from    => 'LocusTags',
	                                  rh_descr => { -general => \%h_general,
	                                                -outputs => \%h_outputs },
	                                  log_file => $service_name . '.log'
	                                );

	$o_service->Launch();

	return;
}

__END__
