#!/usr/bin/perl -w

# Florence Corpet INRA 7avril98 - 31mars00
#DisplayFam is a browser for the analysis of multiple alignments of large numbers
#of protein sequences.
#
#DisplayFam users must cite:
#F. Corpet, J. Gouzy and D. Kahn (1999), 'Browsing protein families via the "Rich
#Family Description" data structure.', Bioinformatics, 15:12, 1020-1027.
#
# Permission granted to use and modify this script so long as the
# copyright above is maintained, modifications are documented, and
# credit is given for any use of the script.
#
#For more information e-mail to jerome.gouzy@toulouse.inra.fr

#
###################################################
# this file includes three parts
#	1: a personalisable part
#	2: a common part (flo_proc.pl)
#	3: a common part
#	4: a default main program as an example

# installation
# edit part 1 with your own values
# copy cgi-lib2.pl in this file directory.
# copy all the following files in a same directory ($PROG)
# 	standalone programs called : protdist, bionj, TriangleTree, reroot, consens, subtree, drawtree
# 	data files : species, speciesCG
# copy the following images in a same directory (which URL is $HTTPIMAGE)
# 	images (.gif): plus, redball, triangle, (yellow, blue, red, green, black)square, alignment, domains, tree
###################################################

###################################################
#### 1: a personalisable part
#### Look at all comment lines and edit as you wish
###################################################

#### Choose whether you include stylesheet ($CSS=1) or not ($CSS=0) in the Html pages



#### you must personalise these strings for a cgi-bin utilisation
#### $PROG: program and working directory
#### $DOMDIR: data directory
#### $HTTPDATA: data directory url
#### $HTTPIMAGE: standard image directory url
#### $SUBTREEPL: URL of the main script that calls DisplayFam
####
#### you must personalise this program call
#### $FETCHFAM: program that prints the family ID_FAM to stdout
####(at run-time ID_FAM is substituted by $id_fam)
#### if $FETCHFAM uses data files, be careful to use absolute file names or
#### names relative to the working directory ($PROG)
#### you can use the msf2rfd function to translate msf data to
#### "Rich Family Description" data
#### $rfd_multi-line_string = &msf2rfd ($msf_multi-line_string);
####
#### you can personalise these strings
#### $Sorry: warning message when id_fam is not valid

########################################################################
#
# NARCISSE INTEGRATION
#
# DisplayFam.pl lives inside its own directory (Core/cluster_displayfam)
# but there is a symbolic link from the standard narcisse cgi-bin to Core/cluster_displayfam
# so that the url is http://narcisse.somewhere.org/cgi-bin/DisplayFam.pl
#
# There is also a symbolic ling from  http://narcisse.somewhere.org/Web/logo/cluster_displayfam_images to the images subdirectory
# The library and image pathes are computed with those facts in mind
#
# Besides, the standard narcisse initialization ($O_IO) was integrated, ParamParser and Webbuilder are now used
# The DisplayFam.xml file was also added for the WebBuilder support
#
# Emmanuel Courcelle
#
#########################################################################

BEGIN
{
	use File::Basename;
	my $dirprg=dirname($0);
	unshift @INC, "$dirprg/../Core/lib";
    unshift @INC, "$dirprg/../Core/API";
	unshift @INC, "$dirprg/../Core/cluster_displayfam";
}

use strict;
use CGI;
require 'cgi-lib2.pl';
use ParamParser;
use WebBuilder;
use DisplayFam_Cons;
use Configuration::InstanceObserver;

my $O_IO   = New InstanceObserver();
#my $rh_cfg = $O_IO->GetConfigHash();
#my %CFG    = %$rh_cfg;

# -- constante --
my $REGEX = chr( 0243 ) . '(\d+)' . chr( 0243 );
# 0243 est la valeur octale du caractere special, separateur du fichier arbre.


#----------------------------------------
#------------- Parametres ---------------
#----------------------------------------

my $RH_PARAM = New ParamParser('CGIPM');

my $SESSDIR   = $RH_PARAM->Get('__wb_session');
my $ID_FAMILY = ($RH_PARAM->IsDefined('id_fam'))    ? $RH_PARAM->Get('id_fam')    : "";

my $TREE_OLDNAME  = $RH_PARAM->Get('tree_oldname');
my $ALIGN_OLDNAME = $RH_PARAM->Get('align_oldname');

my $WANTED = ($RH_PARAM->IsDefined('tree_wanted'))    ? $RH_PARAM->Get('tree_wanted')    : "align";

my $ALIGN_ROOT = ($RH_PARAM->IsDefined('align_root')) ? $RH_PARAM->Get('align_root')	  : 0;
my $TREE_ROOT  = ($RH_PARAM->IsDefined('tree_root'))  ? $RH_PARAM->Get('tree_root') 	  : 0;

my $ALIGN_MAXLEAVES = ($RH_PARAM->IsDefined('align_maxleaves')) ? $RH_PARAM->Get('align_maxleaves') : 12;
my $TREE_MAXLEAVES  = ($RH_PARAM->IsDefined('tree_maxleaves'))  ? $RH_PARAM->Get('tree_maxleaves')  : 12;
my $ALIGN_MINDIST   = ($RH_PARAM->IsDefined('align_mindist'))   ? $RH_PARAM->Get('align_mindist')   : 20;
my $TREE_MINDIST    = ($RH_PARAM->IsDefined('tree_mindist'))    ? $RH_PARAM->Get('tree_mindist')    : 20;

my $REFX = ($RH_PARAM->IsDefined('ref.x')) ? $RH_PARAM->Get('ref.x') : -1;
my $REFY = $RH_PARAM->Get('ref.y');

my $MSF;

my $ALIGNMENT_FORM = ($RH_PARAM->IsDefined('alignment')) ? $RH_PARAM->Get('alignment') : "";
$ALIGNMENT_FORM =~ s/\r//g if ($ALIGNMENT_FORM =~ /\n/);

if ( $RH_PARAM->IsDefined('msf') )
{
	$MSF = $RH_PARAM->Get('msf');
	my $cutoff = $RH_PARAM->Get('Cutoff');
	$MSF = $O_IO->GetCfg('ROOT_DISPLAYFAM_CACHE').'/'.$cutoff.'/'.$MSF;
	$ALIGNMENT_FORM = &UpLoadFile($MSF);
}

#----------------------------------------
#--------Variables Globales -------------
#----------------------------------------
my $CGI = new CGI;
my $TITLE = "DisplayFam from F.Corpet (c) I.N.R.A. 1999";
my $UNIXCAT   = "/bin/cat";
my $UNIXRM_F = "/bin/rm -f";

my ($ROOT_DF, $SUBTREEPL) = $O_IO->GetCfg('ROOT_DISPLAYFAM') =~ /(.+)\/(.+)/;
$ROOT_DF .= "/";

#my $HTTPIMAGE = "images";
my $HTTPIMAGE = $O_IO->GetCfg('HTTP_LOGO') . "/cluster_displayfam_images";

my $DOMDIR    = $O_IO->GetCfg('ROOT_TMP') . "/$SESSDIR/" . $O_IO->GetCfg('PROT_FAMILY') . "/";
my $HTTPDATA  = $O_IO->GetCfg('HTTP_TMP') . "/$SESSDIR/" . $O_IO->GetCfg('PROT_FAMILY') . "/";

my $PROG      = $ROOT_DF;
my $FETCHFAM  = "$UNIXCAT $DOMDIR"."ID_FAM.in";


my ($ARBRE,$LENGTH,$CONSENSUS);
my ($ID_FAM,$AC_FAM);
my ($ALIGN_SUBTITLE,$TREE_SUBTITLE,$WCONS,$NBCONS,$CONS,$NBSEQ,$RELEASE);
my (@A_TITLE,@A_NAMES,@A_SEQUENCES,@A_BOTTOM);
my (%H_DATA_RELATED);
my ($TREETYPE,$TREEMETHOD,$SUBTREE,$DRAWTREE);
my ($ALIGNNAME,$TREENAME);

my ($PROG_ALIGN,$GAP_OPEN,$GAP_EXT,$MATRICE);
my $PROG_ALIGN_MASK = "^Multalin";
my $GAP_OPEN_MASK   = "^Gap weight";
my $GAP_EXT_MASK    = "^Gap length";
my $MATRICE_MASK    = "^Symbol";

&DisplayFam_Cons::init_const($HTTPIMAGE);

my $GET_ALCP="^(AL|CP\\s+\\d+)\\s+(\\S+)(\\s+\\d+\\s+\\d+\\s+|\\s+)(\\d*\\.*\\d+)\\s+(\\S.*)";
my $GET_DR = "^DR\\s+([^ ;]+);\\s+(.*)";

MAIN:
{
    #	&AddKingdom();
    #	&FInitDocument();
    # object webBuilder	/!\__wb_xmlstructure must be defined /!\
	$RH_PARAM->SetUnlessDefined('__wb_xmlstructure', "$PROG/DisplayFam.xml");
	$RH_PARAM->AssertFileExists('__wb_xmlstructure');

	my $o_web_builder = New WebBuilder($RH_PARAM,'',
    '%v' => $O_IO->GetNarcisseVersion(),
    '%n' => $O_IO->GetName(),
    '%i' => $O_IO->GetVersion(),
    '%e' => $O_IO->GetCfg('NARCISSE_CENTRAL'),
    '%T' => $O_IO->GetCfg('ROOT_TMP'),
    '%t' => $O_IO->GetCfg('HTTP_TMP'),
    '%c' => $O_IO->GetCfg('HTTP_CSS'),
    '%l' => $O_IO->GetCfg('HTTP_LOGO'),
    '%j' => $O_IO->GetCfg('HTTP_JAVASCRIPT'),
    '%g' => $O_IO->GetCfg('HTTP_CGI'))
    ;

    # Don't repeat the title
	$o_web_builder->UnsetBehaviour('display_title');

    # Call FDomDisplay
	if ($ID_FAMILY ne "")
	{
        $o_web_builder->SetFunction(\&FDomDisplay);
    }
    else
    {
        $o_web_builder->SetFunction(sub {print STDOUT "<P><H3>Sorry, your request is not valid.</H3></P>\n";});
    }

    # display the page
    $o_web_builder->Realize();

}

=head2 function UpLoadFile

 Usage        : UpLoadFile($file)
 Prerequisite : Utilisation de l'argument CGI msf
 Fonction     : Lit le fichier msf passe en argument
 Returns      : Le contenu du fichier dans un scalaire
 Args         : $file : fichier msf.

=cut
sub UpLoadFile
{
	my $file = shift;
	my $msf_file;

	open (UPFILE, "$file") or Carp::croak "Cannot open $file";
	while ( my $buf = <UPFILE> )
	{
		$msf_file .= $buf;
	}

	return $msf_file;
}

=head2 procedure CreateRFD

 Usage        : CreateRFD
 Prerequisite : $ALIGNMENT_FORM = alignement au format msf
 Fonction     : Cree le fichier .in au format rfd
 Globals      : $ALIGNMENT_FORM, $ID_FAMILY, $AC_FAM
 Args         : none

=cut
sub CreateRFD
{
	$AC_FAM = $ID_FAMILY;
	if ($ALIGNMENT_FORM ne "")
	{
		my $file_in = $DOMDIR.$AC_FAM.".in";
		open (INSEQ, ">$file_in")	or Carp::croak( "$file_in can't be opened for reading" );
		print INSEQ &msf2rfd( $ALIGNMENT_FORM );
 		close (INSEQ);

 		$ALIGNMENT_FORM="";
 	}
}

=head2 function getRef

 Usage        : getRef($ac)
 Fonction     : Create the url of the link that uses $ac
 Return       : URL
 Args         : $ac : accession number

=cut
sub getRef
{
	my ($ac) = shift;
	return "http://www.ncbi.nlm.nih.gov/entrez/viewer.fcgi?db=protein&val=".&RefAc($ac);

}

######### setSubtitles sets 2 titles for the alignment frame and the tree window
## setSubtitles($name) : if ($name) it is a sub-family
## $Subtitle is a true title
## you must set $Subtitle
## you can use the Ahref and Image functions
###
# sub setSubtitles
# {
# 	my ($align_name, $tree_name) = (shift,shift);
# 	$ALIGN_SUBTITLE = '';
# 	$TREE_SUBTITLE = '';
#
# 	$ALIGN_SUBTITLE = &CenterText("Subfamily: $align_name and ".($NBCONS-1)." other similar sequences") if ( $align_name );
# 	$TREE_SUBTITLE  = &CenterText("Subfamily: $tree_name and ".($NBCONS-1)." other similar sequences")  if ( $tree_name );
# }

=head2 function setRelease

 Usage        : setRelease($release)
 Fonction     : Set the $RELEASE variable to your value
 Return       : $RELEASE
 Globals      : $RELEASE
 Args         : $relase : your value

=cut
sub setRelease
{
	my $release = shift;
	$RELEASE = ( defined($release) ) ? $release : "";
}


=head2 function ValidDom

 Usage        : ValidDom($id)
 Fonction     : check if $id is valid ( is not null )
 Return       : booleen
 Args         : $id

=cut
sub ValidDom
{
	my $id = shift;
	return ( $id ne "" ) ? 1 : 0;
}

##############################
#### 2: a common part (flo_proc.pl)
##############################
###########################################################
#
# general tools
#
###########################################################

=head2 function ExistFile

 Usage        : ExistFile($name)
 Fonction     : check if file ($name) exist and is not null
 Return       : booleen : 1 if file exist and is not null
 Args         : $name : file path

=cut
sub ExistFile
{
 	my $name = shift;

  	my($ret_val)=(0);
	if ( (-e $name) && !(-z $name) )
    {
        $ret_val = 1;
    }
    return ($ret_val);
}

###########################################################
#
# functions related to our rfd format
#
###########################################################
=head2 function msf2rfd

 Usage        : msf2rfd($align)
 Fonction     : convert msf --> rdf
 Return       : le contenu du fichier rdf
 Args         : $align : msf alignment
 Globals      : @$A_NAMES

=cut
sub msf2rfd
{
	my $align = shift;

	my ($t, @lines);

	if ( $align =~ /\.\.\s*$/mi )
	{
		$t     = $`;	# $`: la chaine avant le match. Ici MSF:    134    Check:   0
		$align = $';	# $': la chaine apres le match. Ici le reste du fichier MSF
		$t     =~ s/^/DE /mg;
	}
	push @lines, &getMSFname($1,$2,$3) while ( $align =~ /Name:\s+(\S+)\s+Len:\s+(\d+).*Weight:\s+([\d\.]+)/mig );

	foreach my $line (@lines)
	{
  		$line =~ tr/a-z/A-Z/;
		next if ( $line =~ s/^AL (\S+)\D(\d+)\-(\d+) 1 \d+/AL $1 $2 $3/ );
		$line=~ s/^AL (\S+)\((\d+)\-(\d+)\) 1 \d+/AL $1 $2 $3/;
	}

	return unless ( $align =~ /^\/\//mi );
	$align = $';

	my $i=0;
	while ( $align =~ /(\S+)\s+(.*)/mig )
	{
		my $n  = $1;
		my $ss = $2;
		my $s  = $A_NAMES[$i];
		$s =~ s/(\W)/\\$1/g;
		next unless ($n =~ /$s/i);
		if ($n gt $A_NAMES[$i])
		{
			$lines[$i]=~ s/^AL \S+ \d+ (\d+)/AL $n 1 $1/;
			$lines[$i]=~ tr/a-z/A-Z/;
			$lines[$i]=~ s/^AL (\S+)\((\d+)\-(\d+)\) 1 \d+/AL $1 $2 $3/ unless $lines[$i]=~ s/^AL (\S+)\D(\d+)\-(\d+) 1 \d+/AL $1 $2 $3/;
			$A_NAMES[$i]=$n;
		}
		$ss =~ tr/ //d;
		$lines[$i] .= $ss;
		$i++;
		$i=0 if ($i > $#A_NAMES);
	}

	return join("\n",$t,@lines)."\n//\n";
}

=head2 Get functions

 Usage  : getID($ID_line)
 	    : getAC($AC_line)
	    : getTR($TR_line)
	    : getAlignmentLength($LA_line)
	    : getSequenceNumber($ND_line)
 Return : The string after ID\s+, AC\s+, TR\s+, LA\s+ or ND\s+

=cut
sub getID
{
	my $string = shift;
 	if ( $string =~ /^ID\s+(\S+)/mi )
	{
 		return $1
	}
}

sub getAC
{
	my $string = shift;
	if ( $string =~ /^AC\s+(PD0*\d+)/mi )
	{
		return $1;
	}
}

sub getTR
{
	my $string = shift;
	join ("\n",( $string =~ /^TR\s+(.*)/mig) );
}

sub getDE
{
	my $string = shift;
 	if ( $string =~ /^DE\s+(.*)/mi )
	{
 		return $1
	}
}

sub getCO
{
	my $consensus = shift;
 	if ( $consensus =~ /^CO\s+(\S.*)/mi )
 	{
		return $1;
	}
}

sub getAlignmentLength
{
 	my $la = shift;

	if ( $la =~ /^LA\s+(\d+)/mi )
	{
		return $1;
	}
}

sub getSequenceNumber
{
	my $nd = shift;
 	if ( $nd =~ /^ND\s+(\d+)/mi )
	{
		return $1;
	}
}

sub getALCP
{
	my ($tnb,$name,$begend,$w,$ali)=@_;

	my ($nb, $beg, $end);
	if ($tnb =~ /^AL/i)
	{
		$nb =1;
		($beg,$end) = $begend =~ /\s+(\d+)\s+(\d+)/;
	}
	else
	{
		($nb) = $tnb=~ /(\d+)/;
		$beg = $end = '.';
	}
	$ali =~ tr/ //d;
   	$ali =~ tr/a-z/A-Z/;
	return ($nb,$name,$beg,$end,$w,$ali);
}

# lines that are not AL, CP, TR nor DR  are comment
# they can be split into two parts by the position of the firs AL line
sub getCommentOnly
{
	my $string = shift;
	$string =~ s/^AL.*\n//mgi;
	$string =~ s/^CP.*\n//mgi;
	$string =~ s/^TR.*\n//mgi;
	$string =~ s/^DR.*\n//mgi;
	$string =~ s!$DOMAIN_END!!mgi;
 	my @st = split(/\n/,$string);
 	return @st;
}

sub getComment
{
	my ($string, $t1, $t2)=@_;
	$string =~ /^AL.*\n/mi;

	@$t1 = getCommentOnly($`);
	@$t2 = getCommentOnly($');
}

sub getMSFname
{
	my ($name, $len, $weight)=@_;
	return "" if ($name =~ /consens/i);

	push @A_NAMES, $name;
	return "AL $name 1 $len $weight ";
}

sub putEnd
{
	return $DOMAIN_END;
}

sub putID
{
	my $id = shift;
	return "ID $id\n";
}

sub putParsableComment
{
	my ( $st1, $st2 ) = (shift,shift);
	return "CC   -!- $st1: $st2\n";
}

sub putDR
{
	my ($database, @links) = @_;
	return "DR   $database;         ". join(' ',@links)."\n";
}

sub putTR
{
	return "TR ".join ("\nTR ",@_)."\n";
}

sub putAL
{
	my $rh_seq = shift;
	return "AL $rh_seq->{name} $rh_seq->{begin} $rh_seq->{end} $rh_seq->{weight} $rh_seq->{alignment}\n";
}
sub putCP
{
	my $rh_seq = shift;
	return "CP $rh_seq->{nb} $rh_seq->{name} $rh_seq->{weight} $rh_seq->{alignment}\n"
}

sub putCO
{
	my $co = shift;
	return "CO $co\n";
}

sub putLA
{
	my $la = shift;
	return "LA $la\n";
}

sub putND
{
	my $nd = shift;
	return "ND $nd\n";
}

sub RefAc
{
	my $name = shift;
	$name =~ s/^\S+://;
	return $name;
}

sub isSequenceNumberLine
{
	/^ND/i;
}


###################################
# domain data internal organisation
###################################
# comment lines are stored in @A_TITLE and @A_BOTTOM
# one is written before the alignment, the other one after.

# domain format
# $Id_fam family identificator
# $LENGTH is the alignment length
# $CONSENSUS is the original consensus sequence (not used ?)
# $NBSEQ is the number of aligned domains
# @A_SEQUENCES is an array of
# {
#	nb => $nb,
#	name => $seqName,
#	begin => $beg,
#	end => $end,
#	weight => $w,
#	alignment => $ali
# }

# tree format : tree is not used directly in perl => keeps as a string
# $ARBRE = "(((1:0.64264,(((2:0.61888,3:0.67096):0.06406,..."

# @H_DATA_RELATED is an array of links to databases

sub getSubfamily
{
	#local ($_) = @_;
        my $line = shift;
	#(/�(\d+)�/g);
# 	my (@a_names) = ($line =~ /�(\d+)�/g);
	my (@a_names) = ($line =~ /$REGEX/g);

	return @a_names;
}

sub AddSequence
{
	my ($nb,$seqName,$beg, $end,$w,$ali)=@_;
	$NBSEQ++ if ($nb < 2);
	push @A_SEQUENCES, {"nb"=>$nb, "name"=>$seqName, "begin"=>$beg, "end"=>$end, "weight"=>$w, "alignment"=>$ali};
}

sub AddDR
{
	my ($name,$data) = (shift,shift);
	$H_DATA_RELATED{$name}=[] unless $H_DATA_RELATED{$name};
	$name = $H_DATA_RELATED{$name};
	push @{$name}, [split (' ',$data)];
}

sub Read1Domain
{
	my $dom = shift;

	@A_TITLE     = ();
	@A_BOTTOM    = ();
	@A_SEQUENCES = ();

	%H_DATA_RELATED = ();
	$ARBRE = "";
	$NBSEQ = 0;
	$ID_FAM = &getID($dom);
	$AC_FAM = &getAC($dom);
	$LENGTH = &getAlignmentLength($dom);
	$ARBRE  = &getTR($dom);

	$CONSENSUS = &getCO($dom);

	foreach my $line ( split /\n+/, $dom )
	{
		if ( $line =~ /$GET_ALCP/mig )
		{
			&AddSequence(&getALCP($1,$2,$3,$4,$5));
		}

		if ( $line =~ /$GET_DR/mig )
		{
			&AddDR($1,$2);
		}
	}
	&getComment($dom,\@A_TITLE,\@A_BOTTOM);
}

sub ReadDomain
{
# init all global domain variables
	my $file  = shift;
	local $/ = $DOMAIN_END;

	### MANU open (DOMAIN,$file) or return 0;
    open (DOMAIN,$file) or die "cannot open $file";
	my $dom = <DOMAIN>;
	return 0 unless $dom;

	&Read1Domain($dom);

	close(DOMAIN);
	return !(($file =~ /\|/) && $?);
}

sub VerifyDomain
{
	my ($nd_line, @a_dummy) = grep &isSequenceNumberLine, @A_TITLE;

	return 0 if (@a_dummy);

	if ($nd_line)
	{
		my $nb = &getSequenceNumber($nd_line);

		if ( $NBSEQ != $nb )
		{
			print STDOUT "ND=$nb \t nb_seq_in_alignment=$NBSEQ\n";
			return (0);
		}
	}
	else
	{
		my $nb_seq = &putND($NBSEQ);
		chomp($nb_seq);
		push @A_TITLE, $nb_seq;
	}

	my $seq = $A_SEQUENCES[0]{alignment};
	$seq =~ tr/()[]//d;
	$seq = length($seq);
	if ($LENGTH)
	{
		return 0 if ($LENGTH != $seq);
	}
	else
	{
		$LENGTH = $seq;
		my $len = &putLA($LENGTH);
		chomp($len);
		push @A_TITLE, $len;
	}
	$AC_FAM = $ID_FAM unless $AC_FAM;
	return 1;
}

sub Print1Domain
{
	my $string  = join ("\n",@A_TITLE)."\n" if (@A_TITLE);

	$string .= &putTR(split("\n",$ARBRE)) if ($ARBRE);

	foreach my $rh_seq (@A_SEQUENCES)
	{

		if ($rh_seq->{nb} > 1)
		{
			$string .= &putCP($rh_seq);
		}
		else
		{
			$string .= &putAL($rh_seq);
		}
	}
	$string .= join ("\n",@A_BOTTOM)."\n" if (@A_BOTTOM);
	while ( my ($db, $links) = each (%H_DATA_RELATED) )
	{
		foreach my $ra_dr (@{$links})
		{
			$string .= &putDR($db, @{$ra_dr});
		}
	}
	$string .= &putEnd;
	return $string;
}

sub PrintDomain
{
	my $file = shift;
	open (DOMAIN,$file);
	print DOMAIN &Print1Domain;
	close DOMAIN;
}

sub CleanConsensus
{
	for ( my $i=0 ; $i<@A_SEQUENCES ; $i++)
	{
		next if ($A_SEQUENCES[$i]{nb} < 2);
		splice (@A_SEQUENCES, $i, 1);
		$i--;
	}
	$NBSEQ = @A_SEQUENCES;
}

sub ReadOrder_Weight
{
	my $i=0;
	my @a_oldseq = @A_SEQUENCES;
	my $weight = join("",@_);
	$weight =~ s/\n//g;

	my @a_weight = split /\s+/, $weight;

	for ( my $j=0 ; $j<=$#a_weight ; $j=$j+2 )
	{
		$A_SEQUENCES[$i] = $a_oldseq[$a_weight[$j]];
		$A_SEQUENCES[$i]{weight} = $a_weight[$j+1];
		$i++;
	}
}


sub CompareStr
{
	my ($seq1, $seq2) = @_;
	my($res,$length, $ident, $i);
	$res = $seq1 ^ $seq2;# pos with identical chars become \0
	$ident = $res =~ tr/\0//;#count identical chars
	$seq1 =~ tr/A-Z/A/;
	$seq2 =~ tr/A-Z/A/;
	$res = $seq1 ^ $seq2;# pos with 2 letters become \0
	$length = $res =~ tr/\0//;#count pos with 2 letters
	($ident,$length) = (-1,1) if ($length < 10);
	return ($ident,$length);
}



sub PAMdist
{
	my $ident = shift;

	return -1 if ($ident < 0);
	return 0 if $ident >= 1;

	my $tt = 1.0 - $ident;
	if ($tt < 0.75)
	{
		$tt = -log(1-$tt-0.2*$tt*$tt);
	}
	else
	{
		if ($tt > 0.93)
		{
			$tt = 10.0;
		}
		else
		{
			$tt = 0.01*$DAYHOFF_PAMS[int (1000*$tt - 750)];
		}
	}
	return $tt;
}

sub Protdist
{
	my ($ret, $ident, $length, $seq, @a_dist);
	foreach my $i (0..$#A_SEQUENCES)
	{
		$a_dist[$i][$i] = '0.00';
		$seq = $A_SEQUENCES[$i]{alignment};
		$seq =~ tr/A-Z/g/c; #code gaps as 'g';
		foreach my $j ($i+1 .. $#A_SEQUENCES)
		{
			($ident, $length) = &CompareStr($seq,$A_SEQUENCES[$j]{alignment});
			$ident = 0.01 * int(100*&PAMdist($ident/$length)+0.5);
			$ret = 1 if (($a_dist[$j][$i]=$a_dist[$i][$j]= $ident)<0);
		}
	}
	return ($ret, @a_dist);
}

sub Make2Tree
{
	my ($d, @a_dist) = &Protdist;
	$d = $a_dist[0][1];
	if ($d == 0)
	{
		$d = "0.00000";
	}
	else
	{
		$d = ($d + 0.000018)/2;
		chop($d);
	}
	$ARBRE = "(1:$d,2:$d);";

	$ARBRE ="(1:10.0000,2:10.0000);" if ($?!=0);  # infinite distance
}

sub MakeTree
{
	my $tmp = "$DOMDIR/maketree$RELEASE.$ID_FAM";
	my $rh_seq;
	local $/ = "";
	&CleanConsensus if ($NBSEQ  < @A_SEQUENCES);
	$ARBRE = "";
	$TREEMETHOD = -1;
	if (@A_SEQUENCES < 3)
	{
		foreach $rh_seq (@A_SEQUENCES)
		{
			$rh_seq->{weight}=1.00;
		}
		&Make2Tree ($A_SEQUENCES[0]{alignment},$A_SEQUENCES[1]{alignment}) if (@A_SEQUENCES == 2);
	}
	else
	{
		if (&ExistFile($tmp))
		{
			$TREEMETHOD = `grep -c "\-" $tmp`;
			$TREEMETHOD = ($TREEMETHOD > 0)?2:1;
		}
		else
		{
			open (SEQDATA,"| $PROG/protdist >$tmp") or return;

			print SEQDATA "$NBSEQ\n";
			for my $i (0..$#A_SEQUENCES)
			{
				print SEQDATA "$i ",$A_SEQUENCES[$i]{alignment},"\n";
			}
			close(SEQDATA);
			$TREEMETHOD = $?>>8;
		}
		if ($TREEMETHOD > 1)
		{
			print STDERR "TriangleTree used for $ID_FAM\n";
			unless (open (TREE, "$PROG/TriangleTree $tmp | $PROG/reroot |"))
			{
				$TREEMETHOD=3;
				print STDERR "can't compute a complete matrix distance\n";
				return(-1);
			}

		}
		else
		{
			open(TREE,"$PROG/bionj $tmp| $PROG/reroot |")
				or return;
		}
		$ARBRE = <TREE>;

		&ReadOrder_Weight(<TREE>);
		close(TREE);
		print STDERR "pb in maketree $ID_FAM\n" if ($ARBRE eq "");
# if some distances were estimated or if NbSeq is very large, keep the distance array
		system ("$UNIXRM_F $tmp") unless (($TREEMETHOD==1)or($NBSEQ > 500));
	}
}

sub EditGap
{
	my $seq = shift;
	my ($j, $k);
	return $seq unless ($seq=~ /[\.\-]/);

	($j,$seq,$k) = $seq=~ /(^[\.\-]*)([A-Z\(\[].*[A-Z\)\]])([\.\-]*$)/;

	$j   =~ tr/-/./;
	$seq =~ tr/./-/;
	$k   =~ tr/-/./;
	join("",$j,$seq,$k);
}

sub Consens
{
	my $tmp = "$DOMDIR/consens$RELEASE.$ID_FAM";
	my $SW = ($A_SEQUENCES[0]{name} =~ /\|/);    # to avoid XXXXX species
	my @a_oldseq;
	my ($j, $k, $name, $ac, $id);

	&CleanConsensus if ($NBSEQ  < @A_SEQUENCES);
	return if ($NBSEQ < 2);

	open (TREE,"| $PROG/consens >$tmp") or return;

	print TREE "$NBSEQ\n";
	$ARBRE = "(1:10.0000,2:10.0000);\n" if ($ARBRE eq "");
	print TREE "$ARBRE\n";
	for my $i (0..$#A_SEQUENCES)
	{
		$A_SEQUENCES[$i]{alignment} = &EditGap($A_SEQUENCES[$i]{alignment});
		print TREE "$i ",$A_SEQUENCES[$i]{weight}," ",$A_SEQUENCES[$i]{alignment},"\n";

	}
	close(TREE);

	return if ($?);   # Code d'erreur de la derniere commande externe executee
	open (CONS,"$tmp") or return;
	@a_oldseq = splice (@A_SEQUENCES, 0);
	while ( my $line = <CONS> )
	{
		chop($line);
		my ($n, $i, $w, $seq)=split(' ',$line,4);
		if ($n>1)
		{
			$name=$a_oldseq[$i]{name};# cluster name = first seq name
			if ($SW)# search a better name for the cluster
			{
				for ( $j=0,$k=-1 ; $j<$n ; $j++ )
				{
					($ac,$id) = split ('\|',$a_oldseq[$i+$j]{name},2);
					if ($id !~ /$ac/) # excellent candidate
					{
						$k = $j;
						last;
					}
					$k=$j if ( ($k<0) && ($id !~ /XXXXX/) );# good candidate
				}
				$name = $a_oldseq[$i+$k]{name} if ($k > 0);
			}
			&AddSequence($n,"+$name",'.','.',$w,$seq);
		}
		else
		{
			push @A_SEQUENCES, $a_oldseq[$i];
		}
	}
	close(CONS);
	system("$UNIXRM_F $tmp");
	unless ($CONSENSUS)
	{
		$CONSENSUS = $A_SEQUENCES[0]{alignment};
		$CONSENSUS =~ tr/()[]//d;
		my $co = &putCO($CONSENSUS);
		chomp($co);
		push @A_BOTTOM, $co;
	}
}

# sub AddKingdom
# {
# 	my @a_kingdom = ();
#
# 	# read the species table
# 	open (SPECIES,$SPECIES) or return ;
# 	while ( my $line = <SPECIES> )
# 	{
# 		my ($key,$val,$subval) = split /\s+/, $line;
# 		$H_SPECIES{$key}{val}    = $val;
# 		$H_SPECIES{$key}{subval} = $subval;
# 	}
# 	close (SPECIES);
# }

sub isRedundant
{
	my ($new, $ra_list)=@_;
	return 0 if (!$ra_list or (@{$ra_list}==0));

	my ($new_begin,$new_end) = $new =~ /\((\d+)\-(\d+)\)/;
	foreach my $old ( @{$ra_list} )
	{
		my ($old_begin,$old_end) = $old =~ /\((\d+)\-(\d+)\)/;
		return 1 if (($new_begin >= $old_begin) && ($new_end <= $old_end));
	}
}

sub PDBnr
{
	my %Idx;
	my ($cles, $cle, $dr);
	my $PDB = $H_DATA_RELATED{PDB};
	for (my $i=0; $i < @$PDB; $i++)
	{
		$dr = $$PDB[$i];
		$cles = "$$dr[0]";
		$cle = "$$dr[4]";

		$Idx{$cles} = [] unless $Idx{$cles};
		$Idx{$cle} = [] unless $Idx{$cle};
		if (!&isRedundant($$dr[3],$Idx{$cles}) && !&isRedundant($$dr[5],$Idx{$cle}))
		{
				push @{$Idx{$cles}}, $$dr[3];
				push @{$Idx{$cle}}, $$dr[5];
		}
		else
		{
			splice @$PDB, $i, 1;
			$i--;
		}
	}
}

sub Common2Complete
{
	my $tree_method_key = "TREE BUILDING";
	my ($ret_val) = &MakeTree();
	if ($TREEMETHOD >= 0)
	{
		my @a_MethodText = ("Bionj","MissDist+Bionj","TriangleTree","Error");
		my $co = &putParsableComment($tree_method_key,$a_MethodText[$TREEMETHOD]);
		chomp($co);
		push @A_TITLE, $co;
	}

	if ($ret_val < 0)
	{
		return ($ret_val);
	}



	return 0 unless ($ARBRE or ($NBSEQ<2));
	&Consens;
	return 0 unless (2*$NBSEQ -1 == @A_SEQUENCES);
	&PDBnr if $H_DATA_RELATED{PDB};
	return 1;
}

sub CodeAsCons
{
	my ($seq, $cons) = @_;
	my ($ouvre, $motcons, $motseq, $ferme, $l, $ident, $motident, $motdiff);
	my $s = "";
	while ($cons ne "")
	{
		($motcons, $cons) = $cons =~ /^([^\(\[]*)(.*)/;
		if ($l = length($motcons))
		{
			($motseq, $seq)= $seq =~ /(.{$l})(.*)/;
			$s .= $motseq;
		}

		last if ($cons eq "");
		($ouvre, $motcons, $ferme, $cons) = $cons=~ /([\(\[])([^\)\]]+)([\)\]])(.*)/;
		if ($l = length($motcons))
		{
			($motseq) = $seq =~ /(.{$l})/;
			$ident = ($motcons ^ $motseq);
			while ($ident ne "")
			{
#$Ident can includes \n because "\n"="L"^"F" for example so / /s is necessary
				($motident, $motdiff, $ident) = $ident =~ /^(\0*)([^\0]*)(.*)$/s;
				if ($l = length($motident))
				{
					($motseq, $seq) = $seq =~ /(.{$l})(.*)/;
					$s .= $ouvre.$motseq.$ferme;
				}
				if ($l = length($motdiff))
				{
					($motseq, $seq) = $seq =~ /(.{$l})(.*)/;
					$s .= $motseq;
				}
			}
		}
	}
	$s;
}

sub getFather
{
	my $root = shift;

	return 0 if ($root < 2) ;

	my $n=$A_SEQUENCES[$root]{nb};
	my $x=$root-1;
	if ($x+2*$n < @A_SEQUENCES)
	{
		my $m = $A_SEQUENCES[$x]{nb};
		return $x if ($m == $n + $A_SEQUENCES[$x+2*$n]{nb});
	}

	for ($x=0; $x<$root; )
	{
		my $m=$A_SEQUENCES[$x]{nb};
		if ($x+2*$m-1 < $root)
		{
			$x += 2*$m-1;
		}
		else
		{
			return $x if ($m == $A_SEQUENCES[$x+1]{nb} + $n);
			$x++;
		}
	}
	return $root;
}

################
#
# global parameters to define subtrees
#
#	$root			0              root node identificator
#	$MAXLEAVES	12					maximal number of leaves
#	$MINDIST		20					minimal distance between leaves
#	$SEARCH		""					pattern looked for in the clustered names
#	$WANTED		align				align|tree choice of the representation
#	$TREETYPE	'p'             'h'|'r'|'p' horiz star, radial star, phenogram
#
# C programs
#  $SUBTREE $id $MAXLEAVES $MINDIST <$ARBRE : compute a subtree for domain $id
#	$DRAWTREE $gif_file $TREETYPE <$tree_file |sort >cf_file:
#		draw the tree described in tree_file (type: $TREETYPE), put the image into
#		gif_file and the node map into cf_file
#
################

sub ValidateParameters
{
	$ALIGN_ROOT = int($ALIGN_ROOT);
	$TREE_ROOT =  int($TREE_ROOT);

	$TREE_MAXLEAVES= $NBSEQ+0 if (($TREE_MAXLEAVES eq "all") or ($TREE_MAXLEAVES > $NBSEQ));
	$TREE_MAXLEAVES = 1 if ($TREE_MAXLEAVES <= 0);
	$TREE_MAXLEAVES = int($TREE_MAXLEAVES);
	$TREE_MINDIST = int($TREE_MINDIST);

	$ALIGN_MAXLEAVES= $NBSEQ+0 if (($ALIGN_MAXLEAVES eq "all") or ($ALIGN_MAXLEAVES > $NBSEQ));
	$ALIGN_MAXLEAVES = 1 if ($ALIGN_MAXLEAVES <= 0);
	$ALIGN_MAXLEAVES = int($ALIGN_MAXLEAVES);
	$ALIGN_MINDIST = int($ALIGN_MINDIST);
	if ($WANTED eq 'rtree')
	{
		$TREETYPE = 'r';
	}
	elsif ($WANTED eq 'ptree')
	{
		$TREETYPE = 'p';
	}
	elsif ($WANTED eq 'htree')
	{
		$TREETYPE = 'h';
	}
	else
	{
		$TREETYPE ='p';
		$WANTED = 'align' unless ($WANTED =~ /tree/);
	}
}

sub Subtree
{
	my ($r, $l, $d, $file)=@_;
	open (TREE,"| $SUBTREE $r $l $d >$file") or return 0;
	print TREE $ARBRE;
	close (TREE);
	return (ExistFile($file));
}

sub Rootmap
{
	my ($cf_file, $x, $y) = @_;
	my @a_coord;
	my ($d,$dx,$dy);
	my $root = $TREE_ROOT;
	open (MAP, $cf_file) or return 0;

	my $line;
	while ( $line = <MAP> )
	{
		@a_coord = split (/\s+/, $line);
		last if (($dx=abs($a_coord[0]-$x))<4);
	}
	$d=4;
	while ($dx < $d)
	{
		if (($dy=abs($a_coord[1] - $y))<$d)
		{
			$d = ($dx>$dy) ? $dx : $dy;
			$root = $a_coord[2];
			last if ($d == 0);
		}
		last unless ( defined($line = <MAP>) );
		@a_coord = split (/\n+/, $line);
		$dx = abs($a_coord[0]-$x);
	}
	close(MAP);
	return $root;
}


sub GetName
{
	my $i = shift;
	my ($name, @G);
#	my $last = $i + 2*$A_SEQUENCES[$i]{nb}-1;

	$name = $A_SEQUENCES[$i]{name};
	$name =~ s/^\+//;

	@G = split(':',$name);
	return ($name,$G[0]) if $G[0];
	return $name;
}

sub getIDforTREE
{
	my ( $id, $scrap ) = @_;

	$id =~ s/:/_/g;
# 	$id =~ /(\S+):/;
# 	my $sp =  $1;
# 	$id =~ s/:/-/g;
#	$id .= "_".$H_SPECIES{$sp}{val};

	return $id;
}

sub Drawtree
{
	my ($gif_file,$cf_file,$tree_file) = @_;
	my ($i, $name);

	chdir($DOMDIR);

	open (INTREE, $tree_file) or return 0;
	open (OUTTREE,"|$DRAWTREE $gif_file $TREETYPE|sort >$cf_file") or return 0;

    # $regex = �(\d+)� (but the chr notation is more robust to the editor's encoding
	while ( my $line = <INTREE> )
	{
		#$line =~ s/�(\d+)�/&getIDforTREE(&GetName($1))/ge;
        $line =~ s/$REGEX/&getIDforTREE(&GetName($1))/ge;

		print OUTTREE $line;
	}

	close(INTREE);

	return close(OUTTREE);
}

sub FetchFile
{
	my $id_dom = shift;
	my $fetch_fam = $FETCHFAM;
	$fetch_fam =~ s/ID_FAM/$id_dom/;

	return "$fetch_fam |";
}

sub FetchDomain
{
	my $id_dom = shift;
	return 0 unless (&ReadDomain (&FetchFile($id_dom)) and &VerifyDomain);

	unless ($ID_FAM)
	{
		$ID_FAM = $id_dom ;
		my $id = &putID($ID_FAM);
		chomp($id);
		unshift @A_TITLE, $id;
	}
}

sub CreateProdomCompleteDomain
{
	my $id_dom1 = shift;
	my $ret_val = 0;

	if(&ExistFile(${DOMDIR}.$id_dom1))
	{
		system ("touch ${DOMDIR}$id_dom1");
		return 1;
	}
	return -1 unless (&ValidDom($id_dom1));
	return -2 unless (&FetchDomain($id_dom1));

	$ret_val = &Common2Complete;
	if ($ret_val>0)
	{
		&PrintDomain(">${DOMDIR}$id_dom1");
	}
	elsif ( $ret_val < 0 )
	{
		return ($ret_val);
	}

	$ret_val = &ExistFile("${DOMDIR}$id_dom1");
	return ($ret_val);
}

sub ReadProdomDomain
{
	my $id_dom = shift;
	my $ret_val = 0;
	&setRelease;

	($ret_val) = &CreateProdomCompleteDomain($id_dom);

	if ( $ret_val <= 0 )
	{
		return($ret_val);
	}

	if ( (!defined($ID_FAM)) or ($ID_FAM ne $id_dom) )
	{
		&ReadDomain("${DOMDIR}$id_dom");
	}

	return (($ID_FAM eq $id_dom) and &VerifyDomain);
}



##############################
#### 3: a common part
##############################
#### pseudo code
####		Initialisation
####		unless ReadFamily
####			print Sorry
####			die
####
####		printBigTitle
####		if ($NBSEQ > 2)
####			ValidateParameters
####			SelectRoot
####			UpdateNames
####			if ($ROOT>0)
####				$Subtitle= "Subfamily: ..."
####
####			if ($WANTED eq "tree")
####				printTree
####			else
####				printAlignment
####		else
####			printSmallAlignment
#######################


sub getColor
{
  my $col = shift;

#   if ($col)
#   {
#   		$col = $H_SPECIES{$col}{val};
# 		$col = $COLOR{$col} if (!$CSS);
# 		return "$SPFORM=$col";
#   }
#   else
  {
		return "$SPFORM=black";
  }
}

sub MultiLines
{
	return join('<br>',@_);
}

sub SimpleText
{
	return "<P>".join('<br>',@_)."</P>";
}

sub HtmlCenter
{
	return "<center>".join("\n",@_)."</center>";
}

sub CenterText
{
	return HtmlCenter("<h1>".join('<br>',@_)."</h1>");
}

sub TableHeader
{
	return join ('<th>','<tr>',@_)."\n";
}

sub TableRow
{
	my ($h, @d) = @_;
	return join('<td>',"<tr><th>$h",@d);
}

sub Ahref
{
	my ($ref, $comp, $text) = @_;
	return "<A HREF = \"$ref\" $comp>$text</A>\n";
}

sub Image
{
	my ($ref, $comp, $alt) = @_;
	return "<img src=\"$ref\" $comp alt=\"$alt\">";
}

sub FormBegin
{
 	return join("", "<form method=get action=\"", @_,"\">\n");
}

sub SubmitB
{
	my $name = shift;
	return "<input type=submit".(($name)? " value='$name'": "") .">";
}
sub TextParam
{
	my ($name, $value, $size)=@_;
	return "<input id='$name' type=text name='$name' value='$value' size=$size maxlength=$size>";
}

sub Select
{
	my ($name, $size, $selected, @options) = @_;
	my $tmp = "<select name=$name size=$size>";
	for ( my $i=0;$i<@options;$i++)
	{
		if ($options[$i] ne "")
		{
			$tmp.="<option value=$options[$i]";
			$tmp.=" selected" if ($i == 2*$selected);
			$tmp.=">$options[++$i]</option>";
		}
		else
		{
			$i++;
		}
	}
	$tmp.="</select>";
}

sub HiddenParam
{
	my ($name, $value) = @_;
	return "<input name='$name' type='hidden' value='$value'>\n";
}

sub ImageParam
{
	my ($name, $src, $alt, $border)=@_;
	return "<input name='$name' type=image src='$src' alt='$alt' border=$border>\n";
}

sub printTableHeader
{
	my ($rule)= @_;
	my $mask = $SEQHEADERMASK;
	$mask =~ s/RULE/$rule/;
	print STDOUT $mask;
}

sub printTableLine
{
	my ($nb,$plus,$name,$start,$end,$weight,$seq)= @_;
	my $mask = $SEQLINEMASK;
	$mask =~ s/NB/$nb/;
	$mask =~ s/PLUS/$plus/;
	$mask =~ s/NAME/$name/;
	$mask =~ s/START/$start/;
	$mask =~ s/END/$end/;
	$mask =~ s/WEIGHT/$weight/;
	$mask =~ s/SEQ/$seq/;
	print STDOUT $mask;
}

sub printHeaders
{
	my ($rule, $i) = ($RULEMASK0, 10);

	while ( $i <= $LENGTH )
	{
		$rule .= $RULEMASK1.$i.$RULEMASK2;
		$i += 10;
	}

	$rule .= $RULEMASK3;
	my $tiret = "-"x$LENGTH;
	$tiret =~ s/\-{10}/---------|$RULEMASK2$RULEMASK11/g;
	$rule .= $RULEMASK11.$tiret.$RULEMASK2;
	$rule .= $RULEMASK4;
	printTableHeader ($rule);
}

sub colorSequence
{
	my ($seq, $cons) = @_;
	$seq =~ tr/()[]//d;
	my $s = &CodeAsCons($seq,$cons);
	$s =~ s/\)\(//g;
	$s =~ s/\]\[//g;
	$s =~ s/\(/$TT{'low'}/g;
	$s =~ s/\[/$TT{'high'}/g;
	$s =~ s/\)/$TT{'no'}/g;
	$s =~ s/\]/$TT{'no'}/g;
	return $s;
}
###################### end of HTML functions

sub InitProg
{
#######MANU	chdir("$PROG");

	$SUBTREE  = $PROG."subtree";
	$DRAWTREE = $PROG."drawtree";
}

sub Script_align
{
	my ($i, $all) = @_;
	my ($ret);
	$ret = "$SUBTREEPL?"."id_fam=$AC_FAM&amp;align_mindist=";
	if ($all)
	{
		$ret .= "0&amp;align_maxleaves=all";
	}
	else
	{
		$ret .= "$ALIGN_MINDIST&amp;align_maxleaves=$ALIGN_MAXLEAVES";
	}
	$ret .= "&amp;align_root=$i&amp;align_oldname=$ALIGN_OLDNAME&amp;__wb_session=$SESSDIR";
	$ret .= "&amp;tree_root=$TREE_ROOT&amp;tree_mindist=$TREE_MINDIST&amp;tree_maxleaves=$TREE_MAXLEAVES&amp;tree_oldname=$TREE_OLDNAME";

	return $ret;
}


sub Script_Tree_as_Align
{
	my $as = shift;
	my ($html,$script);

	my $root      = ( $as eq "as_tree" ) ? $TREE_ROOT      : $ALIGN_ROOT;
	my $mindist   = ( $as eq "as_tree" ) ? $TREE_MINDIST   : $ALIGN_MINDIST;
	my $maxleaves = ( $as eq "as_tree" ) ? $TREE_MAXLEAVES : $ALIGN_MAXLEAVES;
	my $oldname   = ( $as eq "as_tree" ) ? $TREE_OLDNAME   : $ALIGN_OLDNAME;
	my $wanted    = ( $as eq "as_tree" ) ? "tree"           : "alignment";
	my $img       = ( $as eq "as_tree" ) ? "alignment"     : "tree";

	$script = "$SUBTREEPL?"."id_fam=$AC_FAM&amp;__wb_session=$SESSDIR;";

	$script .= "align_root=$root&amp;align_oldname=$oldname&amp;";
	$script .= "align_mindist=$mindist&amp;align_maxleaves=$maxleaves&amp;";

	$script .= "tree_root=$root&amp;tree_oldname=$oldname&amp;";
	$script .= "tree_mindist=$mindist&amp;tree_maxleaves=$maxleaves&amp;";

	my $ahref = &Ahref($script,"",&Image("$HTTPIMAGE/$img.gif",$MIDPOS,$img));

	$html = $SUBTITLE_2_MASK;
	$html =~ s/~TARGET~/$img/;
	$html =~ s/~VIEW~/$wanted/;
	$html =~ s/WANTEDREF/$ahref/;

	return $html;
}

sub printAlignParam
{
	my $ra_title = shift;

	foreach my $line ( @{$ra_title} )
	{
		$line = &getDE($line);
		$PROG_ALIGN = $line if ( $line =~ /$PROG_ALIGN_MASK/i );
		$GAP_OPEN   = $line if ( $line =~ /$GAP_OPEN_MASK/i );
		$GAP_EXT    = $line if ( $line =~ /$GAP_EXT_MASK/i );
		$MATRICE    = $line if ( $line =~ /$MATRICE_MASK/i );
	}
	print STDOUT "<table><tr STYLE={font-size:12px}><td valign=top>$PROG_ALIGN</td><td width=10></td><td>$GAP_OPEN<br>$GAP_EXT<br> $MATRICE</td></tr></table>"
}

sub printSequence
{
	my $i = shift;
	my $rh_seq = $A_SEQUENCES[$i];
	my ($tname, $plus, $col, $ac);
	($tname,$col) = &GetName($i);
	$ac = $tname;

	my $ref = $NCBI_REF;
	$ref =~ s/COL/&getColor($col)/e;
	$ref =~ s/NAME/$tname/;
	$ref =~ s/GETREF/&getRef($ac)/e;

	$tname = $ref;
	$plus = "";
	if ($rh_seq->{nb} > 1)
	{
		$plus =  &Ahref(&Script_align($i),"", &Image("$HTTPIMAGE/plus.gif","width=9 height=11 border=noborder","subfamily display"));
		$plus .= "\n";
	}
	printTableLine ($rh_seq->{nb},$plus,$tname,$rh_seq->{begin},$rh_seq->{end},$rh_seq->{weight},colorSequence($rh_seq->{alignment},$CONS));
}


sub printForm
{
	my ($istree) = shift;

	my $label = ( $istree ) ? "tree_" : "align_";
	my $maxleaves = ( $istree ) ? "$TREE_MAXLEAVES" : "$ALIGN_MAXLEAVES";
	my $mindist = ( $istree ) ? "$TREE_MINDIST" : "$ALIGN_MINDIST";
	my $oldname = ( $istree ) ? "$TREE_OLDNAME" : "$ALIGN_OLDNAME";

	print STDOUT $TABLE_BEGIN_1;
	print STDOUT &HiddenParam($label."oldname",$oldname);
	print STDOUT &TableHeader("Minimal distance between sequences (in PAM)",&TextParam($label."mindist",$mindist,4),
							  "Maximal number of clusters",                 &TextParam($label."maxleaves",$maxleaves,4)
							 );
	print STDOUT "$TABLE_END\n";

}

sub printTree
{
	my ($src)    = $DOMDIR.$TREE_OLDNAME.".gif";
	my ($hsrc)   = $HTTPDATA.$TREE_OLDNAME.".gif";
	my %selection =(t=>2,ht=>0,rt=>1,pt=>2);
	my ($father) = &getFather($TREE_ROOT);

	if (!&Drawtree ("${TREE_OLDNAME}.gif","${TREE_OLDNAME}.cf",$TREENAME))
	{
		print STDOUT &SimpleText("Sorry, can't draw the tree");
		return;
	}

	print STDOUT &Script_Tree_as_Align("as_tree");
 	print STDOUT "<br><table><tr><td>";
	print STDOUT &ImageParam('ref',$hsrc,'Click on the root of a subfamily to display subtree',2);
	print STDOUT "</td></tr></table>";

 	print STDOUT &SimpleText("The number of sequences in each subfamily is indicated after the ID");

 	&printForm(1);
	print STDOUT $TABLE_BEGIN_1;
	print STDOUT &TableHeader("Type of tree or labels",&Select("tree_wanted",1,$selection{$TREETYPE.'t'},
 		                                                       'htree','Radial tree (horiz. labels)',
 															   'rtree','Radial tree and labels',
 															   'ptree','Phenogram'));

 	print STDOUT "$TABLE_END\n";
 	print STDOUT &SimpleText( "To display a new tree with these parameters, select a root \n".
 							  &Select ('tree_root',1,0,$TREE_ROOT,'current',
 							  		   ($father)?($father,'up one node'):("",""),
 									   ($TREE_ROOT)?('0','global'):("","")).
 							  " and ".
 							  &SubmitB(), "or click on the root of a subfamily directly in the tree.\n");
 	print STDOUT $FORM_END;
	print STDOUT $SEPARATOR;
}

sub CreateCfFile
{

	my $oldname = shift;

	if (ExistFile("${DOMDIR}${oldname}.cf"))
	{
		system("touch ${DOMDIR}${oldname}.*");
	}
	else
	{
		my @a_param = split('_',$oldname);
		$TREENAME = join ('_',@a_param[0..3]);
		$TREETYPE = $a_param[4];

		if (ExistFile($DOMDIR.$TREENAME))
		{
			system("touch ${DOMDIR}$TREENAME");
		}
		else
		{
			return 0 unless Subtree(@a_param[1..3],$DOMDIR.$TREENAME);
		}

		return 0 unless &Drawtree("${oldname}.gif","${oldname}.cf",$TREENAME);
	}
	return 1;
}

sub UpdateOldname_tree
{
# update oldname & TREENAME with new parameter values
	my $OK = 1;
	$TREENAME = join ('_',$ID_FAMILY,$TREE_ROOT,$TREE_MAXLEAVES,$TREE_MINDIST);

	if ( &ExistFile($DOMDIR.$TREENAME) )
	{
		system ("touch ${DOMDIR}$TREENAME");
	}
	else
	{
		$OK = &Subtree($TREE_ROOT,$TREE_MAXLEAVES,$TREE_MINDIST,$DOMDIR.$TREENAME);
	}
	$TREE_OLDNAME = join ('_',$TREENAME);
	$TREE_OLDNAME .= "_$TREETYPE" if ($TREETYPE ne '');
	return $OK;
}

sub UpdateOldname_align
{
# update oldname & TREENAME with new parameter values
	my $OK = 1;
	$ALIGNNAME = join ('_',$ID_FAMILY,$ALIGN_ROOT,$ALIGN_MAXLEAVES,$ALIGN_MINDIST);
	$ALIGNNAME = "align.".$ALIGNNAME;
	if ( &ExistFile($DOMDIR.$ALIGNNAME) )
	{
		system ("touch ${DOMDIR}$ALIGNNAME");
	}
	else
	{
		$OK = &Subtree($ALIGN_ROOT,$ALIGN_MAXLEAVES,$ALIGN_MINDIST,$DOMDIR.$ALIGNNAME);
	}
	$ALIGN_OLDNAME = join ('_',$ALIGNNAME);
	$ALIGN_OLDNAME .= "_$TREETYPE" if ($TREETYPE ne '');

	return $OK;
}

sub SelectRoot
{
	$TREE_ROOT = &Rootmap ("${DOMDIR}${TREE_OLDNAME}.cf",$REFX,$REFY) if &CreateCfFile($TREE_OLDNAME);
}

sub printAlignment
{
	my $sub = shift;
	$sub = "" if ( !defined($sub) );
	my $name;

	print STDOUT "<br>";
#	print STDOUT $ALIGN_SUBTITLE;
	print STDOUT &Script_Tree_as_Align("as_align");
	printHeaders();

	unless (open (TREE, $DOMDIR.$ALIGNNAME))
	{
		&CreateCfFile($ALIGN_OLDNAME);
		unless (open (TREE, $DOMDIR.$ALIGNNAME))
		{
			print STDOUT &SimpleText("Sorry, can't read the alignment");
			return;
		}
	}

	while ( my $line = <TREE> )
	{
		foreach $name (&getSubfamily($line))
		{
			&printSequence($name);
		}
	}
	close(TREE);



	print STDOUT &TableRow("");
	printTableLine ($NBCONS,"","Consensus","","",$WCONS,&colorSequence($CONS,$CONS));

	print STDOUT "$TABLE_END";

	print STDOUT &HiddenParam("wanted","alignment");
	print STDOUT "<br><br>";
	&printForm(0);
	print STDOUT &MultiLines("To display a new alignment with these parameters, ");
	if ($ALIGN_ROOT > 0)
	{
		print STDOUT &MultiLines("select a root \n".&Select ('align_root',1,0,$ALIGN_ROOT,'current','0','global')." and ");
	}
	else
	{
		print STDOUT &HiddenParam("align_root",$ALIGN_ROOT);
	}
	print STDOUT &MultiLines(&SubmitB(),"or click ".&Ahref(&Script_align($ALIGN_ROOT,1),"class=control_FC","here")." to  display all sequences in the ${sub}family<br>");
	print STDOUT "<br>$SEPARATOR<br>";
}

sub printSmallAlignment
{
	$ALIGN_ROOT   = 0;
	$CONS   = $A_SEQUENCES[0]{alignment};
	$NBCONS = $A_SEQUENCES[0]{nb};
	$WCONS  = $A_SEQUENCES[0]{weight};
	printHeaders();
	if ($NBSEQ > 1)
	{
		foreach my $i (1..2)
		{
			printSequence ($i);
		}
		print STDOUT &TableRow("");
		printTableLine ($NBCONS,"","Consensus","","",$WCONS,&colorSequence($CONS,$CONS));
	}
	else
	{
		printSequence (0);
	}

	print STDOUT $TABLE_END;

}


###################################################################
##
## main subroutine
##
###################################################################
sub FDomDisplay
{
	my ($name);

    print STDOUT "<div id=\"ID_STANDARD_PAGE\">";
	my $Sorry= "Sorry, your request is not valid";

	my $sub = '';
	&CreateRFD ();
	&InitProg();

	unless (&ReadProdomDomain($ID_FAMILY)>0)
	{
		print STDOUT &SimpleText("$SEPARATOR <H3>$Sorry </H3>"), "</BODY></HTML>";
		die;
	}

	if ($NBSEQ > 2)
	{
		&ValidateParameters;
		&SelectRoot if ($REFX>=0);
		unless (&UpdateOldname_tree && &UpdateOldname_align)
		{
				print STDOUT &SimpleText("$SEPARATOR <H3>$Sorry </H3>"), "</BODY></HTML>";
				die;
		}

		$CONS      = $A_SEQUENCES[$ALIGN_ROOT]{alignment};
		$NBCONS    = $A_SEQUENCES[$ALIGN_ROOT]{nb};
		$WCONS     = $A_SEQUENCES[$ALIGN_ROOT]{weight};

# 		my ($align_name,$titi) = ($ALIGN_ROOT>0) ? GetName($ALIGN_ROOT) : "";
# 		my ($tree_name,$toto)  = ($TREE_ROOT>0)  ? GetName($TREE_ROOT)  : "";
		$sub = 'sub' if ($name);

#		&setSubtitles($align_name,$tree_name);


		print STDOUT &FormBegin($SUBTREEPL);
		print STDOUT &HiddenParam("__wb_session",$SESSDIR);
		print STDOUT &HiddenParam("id_fam",$AC_FAM);
		&printAlignParam(\@A_TITLE);
		&printAlignment($sub);
		&printTree;
	}
	else
	{
		&printSmallAlignment;
	}
    print STDOUT "</div>\n";
}

###### 4: example of main

#####################################################
#### DisplayFam.pl can work in a cgi-bin directory or locally
#### syntaxe : cgi-bin : DisplayFam.pl?param1=value1&param2=value2.....
####	       locally : DisplayFam.pl param1=value1 param2=value2....."
####
#### These parameters must be initialised in the main program
#### 	parameter	default_value	meaning
####    id_fam		no default	family identificator used by FETCHFAM
####	root		0		root node identificator
####	maxleaves	12		maximal number of leaves
####	mindist		20		minimal distance between leaves
####	search		""		pattern looked for in the clustered names
####	wanted		align		choice of the representation
####					align : two-framed window with description and alignment
####					descript : family description
####					alignment : multiple alignment
####					?tree : tree as horiz star(?=h), radial star(?=r), phenogram(?=p or nothing)
####	oldname		no default	map file name
####	ref.x		-1		x coordinate of the map click (or -1 for no click)
####	ref.y		no default	y coordinate of the map click
####
#####################################################



