/*
   $Id: LowComplexityMasker.h 650 2011-10-31 14:25:33Z manu $
*/
/*
 * Copyright INRA/CNRS - Thomas.Faraut@toulouse.inra.fr, Emmanuel.Courcelle@toulouse.inra.fr
 * This software is a computer program whose purpose is to align nucleic sequences, or perform other utility tasks
 * This software is governed by the CeCILL-C license under French law and abiding by the rules of distribution of free software.
 * 
 * Please read the file Licence_CeCILL-C_V1-en.txt which should have been distributed with this file
 * The fact that you are presently reading this, and the aforementioned file, means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
*/

#ifndef _LOW_COMPLEXITY_MASKER_H
#define _LOW_COMPLEXITY_MASKER_H

#include "dust.h"

/**
 * \brief LowComplexityMasker is an interface towards to low complexity detection algorithms that may be implemented in glint.\n
 *        It is not really useful for now, because we ONLY manage the dust algorithm. However, thanks to this class, it should 
 *        easy to add new algorithms later.\n
 *        A low complexity algorithm should be able to:
 *          - Detect low complexity area and replace uppercase letters by lowercase letters in those areas
 *          - be thread-safe
 */
class LowComplexityMasker {	
	public:	
		void Run(unsigned char* base, size_t length, uint16_t CompexityType);
	
	private:
		dust __dust;
};

/* SEPT 2011 - DUST IS NOW THE ONLY ALGORITHM USED FOR LOW COMPLEXITY DETECTION AND SUPPRESSION */

/**
 * \brief Call a lowcomplexity algorithm, following the cxtype parameter. Arrange for the LAST character of the buffer to be a '$'
 * \param base The base address of the buffer 
 * \param length The size of the buffer
 * \param cxtype The algorithm identification, currently 0 means nothing to do, != 0 means call dust
 */

inline void LowComplexityMasker::Run(unsigned char* base, size_t length, uint16_t cxtype)
{
	if ( length == 0 || cxtype == COMPLEXITY_NONE )
        return;

	char sav = *(base+length-1); // keep the LAST character of the buffer

	__dust(length,base);         // call the dust algorithm

    if (sav=='$')                // The last $ may be masked too, retrieve it !
        *(base+length-1) = sav;
}

#endif
