/*
   $Id: Parameters.cpp 764 2014-04-02 13:51:23Z tfaraut $
*/
/*
 * Copyright INRA/CNRS - Thomas.Faraut@toulouse.inra.fr, Emmanuel.Courcelle@toulouse.inra.fr
 * This software is a computer program whose purpose is to align nucleic sequences, or perform other utility tasks
 * This software is governed by the CeCILL-C license under French law and abiding by the rules of distribution of free software.
 * 
 * Please read the file Licence_CeCILL-C_V1-en.txt which should have been distributed with this file
 * The fact that you are presently reading this, and the aforementioned file, means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
*/
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string>
#include <iostream>
#include <vector>
using namespace std;

#include "ptypes.h"
#include "utils.h"
#include "SimpleOpt.h"
#include "Parameters.h"
#include "Command.h"

/*
 * The constructor of Parameters
 */

Parameters::Parameters():
	
	__pipe(false),
	__to_upper(false),
	__index_mode(true),
	__save_index_mode(false),
	__verbose(false),
	__silent(false),
	__is_hsp_on_the_fly(false),	
	__genome(false),
	__self(false),
	__dyn(true),
	__high_throughput_mode(false),
	__threads_nb(1),		
	__pair_end(false),
	__maxdistmate(DEFAULT_MAX_DISTMATE),	
	__epcr(false),			
	__strict(false),
	__step(DEFAULT_STEP),	
	__complexity(DEFAULT_COMPLEXITY),
	__lower_level(DEFAULT_LOWER_LEVEL),
	__deepest_level(DEFAULT_DEEPEST_LEVEL), 
	__cutoff_slide(DEFAULT_CUTOFF_SLIDE),
	__cutoff_dyn(DEFAULT_CUTOFF_DYN),
	__cutoff_hsp(DEFAULT_CUTOFF_HSP),	
	__maxdisttandem(DEFAULT_TANDEM_SIZE),
	__match(ALIGN_CONSTANTS_MATCH),                 
    __mismatch( ALIGN_CONSTANTS_MISMATCH),               
    __Nmatch(ALIGN_CONSTANTS_NMATCH),                 
    __gapop(ALIGN_CONSTANTS_GAPOPEN),                  
    __gapext(ALIGN_CONSTANTS_GAPEXTEN),                 		
	__ungapdropoff(ALIGN_CONSTANTS_UNGAP_DROPOFF),
	__gapdropoff(ALIGN_CONSTANTS_GAP_DROPOFF),	
	__best_score(false),	
	__bwalike(false),
	__max_mismatch(-1),
	__max_gap(-1),__size_min(0),__ident_min(0),
	__min_ratio_of_max_score(-1),
	__min_ratio_of_max_length(-1),		
	__output_format(OUTPUT_FORMAT_UNKNOWN),
	__output_chunk_size(0),
	__input_format(INPUT_FORMAT_UNKNOWN),	
	__results_filename(""),
	__results_bwd_filename(""),
    __query_filename(""),
    __bank_filename(""),
    __bwd_filename(""),
	__bank_nb_ind_pass(0),
	__black_list_max_count(DEFAULT_MAX_COUNT),
	__msk_string(DEFAULT_MASK),
	__count_cutoff(6), //ComparisonFrequency
	__condemned_min_size(20),
	__conseq_freq(50),
	__conseq_un_freq(20),
	__suitable_min_size(1000),
	__suitable_max_size(50000),
	__suitable_chunk_size(20000),
	__stat_frequency(false),
	__stat_descending(false),
	__stat_positions(false),
	__stat_all(false),
	__stat_cutoff(0),
	__stat_level(0),
	__glint_output_filename(""),
	__blastp_output_filename(""),
	__anchors_output_filename(""),
	__target_dir(""),
	__chain_cutoffs(DEFAULT_CHAINING_CUTOFFS),
	__max_overlap(DEFAULT_MAX_CHAIN_OVERLAP),
	__rbh(true),
	__argc(0),
	__Help(NULL)
	{}

void Parameters::SetChunkNb(size_t chunk_nb) 
{ 
	__results_filename.SetChunkNb( chunk_nb );
	__results_bwd_filename.SetChunkNb( chunk_nb );
}

/**
 * \brief Parse the seq file name and search for a comma, indicating two filenames
 * \param [in,out] seq_filename  The filename for the sequence
 * \param [in,out] bwd_filename The filename for the bwd, if any
**/ 
void Parameters::__ParseFileName(string& seq_filename,string& bwd_filename)
{
	// Parse the seq file name and search for a comma, indicating two filenames
	string::size_type p = seq_filename.find(',');
	if ( p != string::npos && p != seq_filename.length()-1 )
	{
		bwd_filename = seq_filename.substr(p+1);
		seq_filename = seq_filename.substr(0,p);
	}
} 
 
/**
 * \brief If __bank_name is not initialized (the switch -N was not specified) we init it from the bank file name
 */

void Parameters::__InitBankName()
{
	if (__bank_name=="")
	{
		string bank_filename = __bank_filename.GetName();
		string::size_type pt = bank_filename.find_last_of('.');
		if (pt != string::npos && pt != 0)
			__bank_name = bank_filename.substr(0,pt);
		else
			__bank_name = bank_filename;
	}
}

/**
 * \brief Patch to remove compression extension to the __bwd_filename in fast(a|q) pair-end mode
 * \todo MANU - Je ne comprends pas trop ce que fait ce patch mais je ne coirs pas qu'il soit encore utile
 */
//void Parameters::__PATCH_BwdCompressionExtension( CompressionAlgorithm& compress_fwd, string& bwd_filename)
//{
//	CompressionAlgorithm compress_bwd = __GuessCompression( bwd_filename );
//	
//	if ( compress_bwd != compress_fwd )
//	{
//		fprintf(stderr,"ERROR - When reading compressed files in pair-end mode, both files must have been compressed\n");
//		fprintf(stderr,"with the same compression algorithm (i.e must have the same extension)\n\n");		
//		exit(1);
//	}
//	
//}

/** 
 *  \brief Guess the compression algorithm from the file name
 *         If compression found, change the file name, removing the last extension
 *  \todo A JETER - REMPLACER PAR CompressedFileName !!! (utilise ONLY dans la PATCH ci-dessus i on vire la patch on vire ca aussi)
 *  \param [inout] filename The file name to guess
 *  \return The guessed compression algorithm
*/
//CompressionAlgorithm Parameters::__GuessCompression ( string& filename )
//{
	//CompressionAlgorithm compress_algo = NOCOMPRESSION;
	//string::size_type last_dot = filename.find_last_of('.');
	//if (last_dot != string::npos )
	//{
		//string extension = filename.substr(last_dot+1);
		//const CompressionInfo &compression_infos = CompressionInfo::Instance();
		//compress_algo = compression_infos.GetInfo(extension).algorithm;
		//if ( compress_algo != NOCOMPRESSION )
		//{
			//filename = filename.substr(0,last_dot);
		//}
	//}
	//return compress_algo;
//}

/** 
 *  \brief constructor for CompressedFileName
 *         We guess the compression algorithm from the file extension
 *         If some compression is detected, the extension is removed from the file name
 *         It the substring %chuink is dected in the file name, the file name is considered as a template
 *         and the SetChunkNb function can be used to replace %chunk by a number and compute a real file name
 * 
 *  \param [in] filename The file name
*/
CompressedFileName::CompressedFileName ( const string& name ): 
__compression_extension(""),
__compression(NOCOMPRESSION)
{
	string::size_type last_dot = name.find_last_of('.');
	if (last_dot != string::npos )
	{
		string extension = name.substr(last_dot+1);
		SetCompression(extension);
		if ( __compression != NOCOMPRESSION )
		{
			SetFileName ( name.substr(0,last_dot) );
		}
		else
		{
			SetFileName ( name );
		}
	}
	else
	{
		SetFileName ( name );
	}
}

/** 
 *  \brief Set file name for this CompressedFileName, resetting the template if any
 *         The Compression field is NOT CHANGED
 *
 *  \example CompressedFileName cfn = "file.%chunk.out.gz";
 *           cfn.SetFileName("file.new.out") sets the template but cfn.GetCompression() still returns gz
 *
 *  \param [in] The chunk nb
*/
void CompressedFileName::SetFileName(const string& name="") 
{
	__chunk_pos = name.find( "%chunk" );
	if ( __chunk_pos == string::npos )
	{
		__template  = "";
		__file_name = name;
	}
	else
	{
		__template = name;
		__file_name= "";
	}
}

/** 
 *  \brief Get the file name, or may be the template name (if not yet initialized)
 *
*/
const string& CompressedFileName::GetName() const
{
	return (__file_name.size()!=0?__file_name:__template);
}		
		
/** 
 *  \brief Set compression extension and algorithm, from a CompressionAlogrithm OR from an extension
 *         The file name or file template fields are NOT changed
 *
 *  \example CompressedFileName cfn = "file.out";
 *           cfn.SetCompression("bz2") sets the compression but cfn.GetName() still "file.out"
 *
 *  \param [in] The compression
*/
void CompressedFileName::SetCompression(const string& extension) 
{
	const CompressionInfo &compression_info = CompressionInfo::Instance();
	CompressionAlgorithm compression        = compression_info.GetInfo(extension).algorithm;
	if ( compression == ANYCOMPRESSION )
	{
		compression = compression_info.SearchBestCompression();
	}
	__compression           = compression;
	__compression_extension = compression_info.GetExtension(compression);
}
void CompressedFileName::SetCompression(CompressionAlgorithm compression) 
{
	const CompressionInfo &compression_infos = CompressionInfo::Instance();
	__compression           = compression;
	__compression_extension = compression_infos.GetExtension(compression);
}
	
/** 
 *  \brief Set the chunk nb for this CompressedFileName, computing a new file name
 *  \pre __template should not be empty
 *  \example CompressedFileName cfn = "file.%chunk.out.gz";
 *           cfn.SetChunkNb(5);
 *           cfn.GetName() returns "file.5.out", cfn.GetCompression() returns gz
 *  \param [in] The chunk nb
*/
void CompressedFileName::SetChunkNb(size_t chunk_nb)
{
	if ( (bool) *this )
	{
		if ( __chunk_pos == string::npos )
		{
			string msg = "ERROR - The file name should contain the string: %chunk (";
			msg += __file_name;
			msg += ")";
			throw (logic_error ( msg ) );
		}
		else
		{
			ostringstream otmp;
			otmp << __template.substr(0,__chunk_pos) << chunk_nb << __template.substr(__chunk_pos+6);
			__file_name = otmp.str();
		}
	}
}

CompressionInfo::CompressionInfo()
{
	CmpInf tmp1 = {NOCOMPRESSION, "",     ""};
	CmpInf tmp2 = {ANYCOMPRESSION, "any",     ""};
	CmpInf tmp3 = {XZ,            "xz",   "xz"};
	CmpInf tmp4 = {LZMA,          "lzma", "lzma"};
	CmpInf tmp5 = {BZ2,           "bz2",  "bzip2"};
	CmpInf tmp6 = {GZ,            "gz",   "gzip"};
	CmpInf tmp7 = {Z,             "Z",    "compress"};
	
	__cmp_infos.push_back( tmp1 );
	__cmp_infos.push_back( tmp2 );
	__cmp_infos.push_back( tmp3 );
	__cmp_infos.push_back( tmp4 );
	__cmp_infos.push_back( tmp5 );
	__cmp_infos.push_back( tmp6 );
	__cmp_infos.push_back( tmp7 );
}

/**
 * \brief Try to execute the registered compression programs, return when a running compression program is found
 * \return The fisrt running compression algorithm
 * \note The programs should be stored most efficient first
 * \exception If no program found, throw a runtime_error
 */

CompressionAlgorithm CompressionInfo::SearchBestCompression() const
{
	int i = (int) ANYCOMPRESSION;
	int syserr = 0;
	i++;
	do
	{
		CompressionAlgorithm comp = (CompressionAlgorithm) i;
		string command = GetProgram(comp);
		command += "< /dev/null >/dev/null 2>&1";
		syserr = system(command.c_str());
	} 
	while ( syserr!=0 && ++i < NB_COMPRESSIONS );
	if (syserr==0)
	{
		return (CompressionAlgorithm) i;
	}
	else
	{
		string msg = "ERROR - No compression program found ";
		msg += " - Please install one of those compression programs: compress, gzip, bzip2, lzma, xz";
		throw runtime_error(msg);
	}
}	

/** 
 *  \brief Find the indexation files - Called by InitializeFromCommandLine
 *  \return true if indexation files found, false if not
 *  \private

*/

bool Parameters::__FindIndex()
{
    struct stat buf_stat;
    string dot = ".";
    string file_bi = __bank_filename.GetName() + dot + BI_EXTENSION;
    string file_fa = __bank_filename.GetName() + dot + FA_EXTENSION;
    string file_he = __bank_filename.GetName() + dot + HE_EXTENSION;
    
    CompressionAlgorithm compression = __bank_filename.GetCompression();
    string extension  = __bank_filename.GetCompressionExtension();
    if ( compression != NOCOMPRESSION )
    {
		file_bi += dot + extension;
		file_fa += dot + extension;
		file_he += dot + extension;
	}

    bool found = true;
    if (stat(file_bi.c_str(),&buf_stat)==-1)
        found = false;
    if (stat(file_fa.c_str(),&buf_stat)==-1)
        found = false;
    if (stat(file_he.c_str(),&buf_stat)==-1)
        found = false;

    return found;
}

/* Only if Debug */

void Parameters::__LogParameters()
{
	cerr <<"Ouput format : "<<(int)__output_format<<endl;
    cerr <<"Cutoff slide : "<<__cutoff_slide<<endl;
    cerr <<"Cutoff dyn   : "<<__cutoff_dyn<<endl;
    cerr <<"Cutoff hsp   : "<<__cutoff_hsp<<endl;
    cerr <<"Lower level  : "<<(int)__lower_level<<endl;
    cerr <<"Deepest level : "<<(int)__deepest_level<<endl;
    string outputstream=( GetOutputFilename() ) ? GetOutputFilename().GetName() : "stdout";
    cerr <<"Ouput file   : "<<outputstream<<endl;
    cerr <<"Follow  : "<< __is_hsp_on_the_fly <<endl;
    cerr <<"Complexity  : "<< __complexity <<endl;
    cerr <<"__self  : " << __self <<endl;
    cerr <<"__genome  : "<< __genome <<endl;
    cerr <<"__pipe  : "<<__pipe<<endl;
    cerr <<"Strict  : "<<__strict<<endl;
    cerr <<"Bank filename  : "<<__bank_filename<<endl;
    cerr <<"Query filename  : "<< __query_filename<<endl;
    cerr <<"Log level  : "<<__log_level<<endl;
    cerr <<"Count cutoff  : "<<(int)__count_cutoff<<endl;
    cerr <<"Condemned min size  : "<<(int)__condemned_min_size<<endl;
    cerr <<"Suitable min size  : "<<(int)__suitable_min_size<<endl;
    cerr <<"Conseq freq  : "<<(int)__conseq_freq<<endl;
    cerr <<"Conseq unfreq  : "<<(int)__conseq_un_freq<<endl;
}

/** 
 *  \brief Manage the warnings differently in high-throughput and in normal mode
 *         In high-throughput mode just count the warnings if the parameters is != "", or print the summary if == ""
 *         Else print the parameter to stderr
 *  \public
*/
void Parameters::Warn(const string & msg, const string & data ) const
{
	if (IsSilent())
	{
		return;
	}
	if ( IsHighThroughputMode()||IsPairEnd() )
	{
		if ( msg != "" )
		{
			#pragma omp critical
			{
				__warnings[msg]++;
				if ( __warnings[msg] < MAX_WARNINGS )
				{
					__warnings_data[msg].push_back(data);
				}
			}
		}
		else
		{
			if ( __warnings.size() > 0 )
			{
				cerr << "WARNINGS WERE ENCOUNTERED !!!\n=============================\n";
				for (map<string,int>::iterator i=__warnings.begin(); i!=__warnings.end(); ++i) 
				{
					cerr << i -> second << " times => " << i -> first << "\n";
					cerr << "=== SEQUENCES AND OTHER DATA ";
					if ( i->second > MAX_WARNINGS )
					{
						cerr << "    " << "(Only first " << MAX_WARNINGS << " data printed) ===\n";
					}
					else
					{
						cerr << "\n";
					}
					vector<string>& wdata = __warnings_data[i -> first];
					for (unsigned int j=0; j<wdata.size(); ++j)
					{
						cerr << "    " << wdata[j] + "\n";
					}
					cerr << "===\n";
				}
				cerr << "=============================\n";
			}
		}
	}
	else
	{
		if ( msg != "" )
		{
			cerr << "WARNING - " << data << " - " << msg << '\n';
		}
	}
}

/** 
 *  \brief Init the Parameters object, calling the virtual functions _Init and _Init2
*/
void ParametersInitializer::Init(int argc,char* argv[],const Command& cmd) 
{ 
	CSimpleOpt arg = _Init(argc,argv,cmd);
	_Init2(arg,cmd);
	bool ok = _CommonValidation() && _Validation();
	if ( !ok)
	{
		cmd.Usage();
		exit(1);
	};
}


/** 
 *  \brief Validate the parameters common to all binaries.
 *  \return true if all good
 *  \private
*/

bool ParametersInitializer::_CommonValidation()
{
    bool ok = true;
    if ( p.__complexity < 0 || p.__complexity > 2 )
    {
        cerr << "ERROR - Complexity should be 0,1,2\n\n";
        ok = false;
    };
    if (p.__bank_nb_ind_pass<0 || p.__bank_nb_ind_pass >10)
    {
        cerr << "ERROR - nb_pass should be in the interval 0..9\n\n";
        ok = false;
    };
	if (p.__black_list_max_count<2 || p.__black_list_max_count>255)
	{
        cerr << "ERROR - max count (switch -n) should be in the interval 2..255\n\n";
        ok = false;
    };
	return ok;
}


/* LE CODE QUI SUIT EST PRESQUE OBSOLETE - ON LE VIRERA DANS QUELQUE TEMPS !!! */
/* (conserve pour l'instant pour glintchain ) */


/*

 Both programs, dnaindex and glint share the same set of options in order to enable the auto-indexing mode.
 We put therefore the set of options in this single options.h file

 */
/**
 * \brief All programs (glint, glintindex, glinstat, glintchain) share the same set of switches
 */


/*
CSimpleOpt::SOption g_GlintOptions[] = {
    // ID       TEXT          TYPE
	//	Specific options of the glint executable
    { OPT_CUTOFF_SLIDE,  (char*)"-c",    SO_REQ_SEP }, // "-c"
    { OPT_CUTOFF_DYN,  (char*)"-X",      SO_REQ_SEP }, // "-X"
    { OPT_CUTOFF_HSP,  (char*)"-s",      SO_REQ_SEP }, // "-s"
    { OPT_LOWER_LEVEL,  (char*)"-w",     SO_REQ_SEP }, // "-w"
    { OPT_DEEPEST_LEVEL, (char*) "-d",   SO_REQ_SEP }, // "-d"
    { OPT_COMPLEXITY,    (char*) "-C",   SO_REQ_SEP }, // "-C"
    { OPT_LC_FILTERING,  (char*) "--no-lc-filtering", SO_NONE }, // "--no-lc-filtering"
    { OPT_OUTPUTFORMAT,  (char*)"-m",    SO_REQ_SEP }, // "-m"
    { OPT_OUTPUTFORMAT,  (char*)"--output-format",SO_REQ_SEP }, // "--output-format"
    { OPT_CHUNKSIZE,     (char*)"--chunk-size",SO_REQ_SEP    }, // "--output-format"
    { OPT_INPUTFORMAT,   (char*)"--input-format", SO_REQ_SEP }, // "--input-format"
    { OPT_RESULTSFILENAME,  (char*)"-o", SO_REQ_SEP }, // "-o"
    { OPT_RESULTSFILENAME,  (char*)"--output-file", SO_REQ_SEP }, // "--output-file"
    { OPT_GENOME, (char*) "-G",       SO_NONE    }, // "-G"
    { OPT_OUTPUTSTREAM, (char*) "-f", SO_NONE    }, // "-f"
    { OPT_PIPE, (char*) "-",          SO_NONE    }, // "-"
    { OPT_SELF,  (char*)"-S",         SO_NONE    }, // "-S"
    { OPT_SELF,  (char*)"--self",     SO_NONE    }, // "--self"
    { OPT_TOUPPER, (char*) "--to-upper", SO_NONE }, // "--to-upper"
    { OPT_TANDEM,  (char*)"--tandem",   SO_REQ_SEP  }, // "--tandem"
    { OPT_STRICT,  (char*)"--strict",   SO_NONE     }, //
    { OPT_STEP,  (char*)"--step",       SO_REQ_SEP  }, //
    { OPT_SILENT,  (char*)"--silent",   SO_NONE }, //
    { OPT_VERBOSE,  (char*)"--verbose", SO_NONE }, //
    { OPT_VERBOSE,  (char*)"-v",        SO_NONE }, //
    { OPT_BEST, (char*)"--best-score", SO_NONE }, // --best-score
    { OPT_SLICESIZE,  (char*)"--slice-size",  SO_REQ_SEP   }, //

    // high throughput mode
    { OPT_HIGHTHROUGHPUT, (char*)"--high-throughput", SO_NONE  },
    
   	#ifdef _OPENMP
	// asking for a number of threads
	{ OPT_A,          (char*) "-a", SO_REQ_SEP }, // -a
	#endif

    //pair-end mapping
    { OPT_PAIREND, (char*)"--pair-end", SO_NONE  }, // "--pair-end"
    { OPT_EPCR, (char*)"--epcr", SO_NONE  }, // "--pair-end"
    { OPT_MAX_DISTMATE, (char*)"--mate-maxdist", SO_REQ_SEP  }, // "--mate-maxdist"
    
     // Options for the alignments socring scheme 
    { OPT_MATCH, (char*)"--match", SO_REQ_CMB   }, // 
	{ OPT_MISMATCH, (char*)"--mismatch", SO_REQ_CMB   }, //
	{ OPT_GAP_OPENING, (char*)"--gapop", SO_REQ_CMB   }, // 
	{ OPT_GAP_EXTENSION, (char*)"--gapext", SO_REQ_CMB   }, //
    { OPT_NOGAP,  (char*)"--no_gap",    SO_NONE }, //
	{ OPT_NOGAP,  (char*)"--no-gap",    SO_NONE }, //
	{ OPT_MAX_MIS, (char*)"--mmis",     SO_REQ_SEP }, //
    { OPT_MAX_GAP, (char*)"--mgap",     SO_REQ_SEP }, //
    { OPT_SIZE_MIN, (char*)"--lmin",    SO_REQ_SEP }, //
    { OPT_SCORE_RATIO_MIN, (char*)"--srmin",  SO_REQ_SEP }, //
    { OPT_LENGTH_RATIO_MIN, (char*)"--lrmin", SO_REQ_SEP }, //
#ifdef DEBUGLOG
    { OPT_LOG_LEVEL,  (char*)"-D",     SO_REQ_SEP }, // "-D"
#endif

	// writing compressed data
	{ OPT_COMPRESS,   (char*) "--compress",     SO_REQ_SEP },
	
	// saving the index when using glint, index mode
	{ OPT_SAVE_INDEX, (char*) "--save-index", SO_NONE }, // --save-index
	
    //	Options specific to the indexation process
    { OPT_MAX_COUNT, (char*) "-n",     SO_REQ_SEP    }, // "-n"
    { OPT_MASK,  (char*) "-M",     SO_REQ_SEP    },     // "-M"
    { OPT_MASK,  (char*) "--mask", SO_REQ_SEP    },     // "---mask"
    { OPT_BANK_NAME,  (char*)"-N",     SO_REQ_SEP   },  // "-w"
    { OPT_NB_PASS,  (char*)"-p",     SO_REQ_SEP    },   // "-p"
    
    //For the chaining algorithm
    { OPT_CUTOFFS,  (char*)"--cutoffs",     SO_REQ_SEP    },
    { OPT_MAXOVERLAP,  (char*)"--maxoverlap",     SO_REQ_SEP    },
    { OPT_GLINTOUT,  (char*)"--glintout",     SO_REQ_SEP    },
    { OPT_BLASTPOUT,  (char*)"--blastpout",     SO_REQ_SEP    },
    { OPT_ANCHORSOUT,  (char*)"--anchorsout",     SO_REQ_SEP    },
    { OPT_TARGETDIR,  (char*)"--targetdir",     SO_REQ_SEP    },
    { OPT_NORBH,  (char*)"--noRBH",     SO_NONE    }, //
    
    // To get some help
    { OPT_HELP, (char*)"-?",     SO_NONE    }, // "-?"
    { OPT_HELP, (char*)"-h",     SO_NONE    }, // "-?"
    { OPT_HELP, (char*)"--help", SO_NONE    }, // "--help"
    
    // For glintstat
    { OPT_STAT_FREQUENCY,  (char*)"--frequency",   SO_NONE    }, // --frequency, print the frequency of the words
    { OPT_STAT_DESCENDING, (char*)"--descending",  SO_NONE   }, // --descending, with --frequency: print descending order   
    { OPT_STAT_CUTOFF,     (char*)"--stat-cutoff", SO_REQ_SEP   }, // --stat-cutoff, with --frequency: do not display words with nb of positions lower than this
	{ OPT_STAT_LEVEL,      (char*)"--stat-level",  SO_REQ_SEP   }, // --stat-level, with --frequency: do not display words at level specified
    { OPT_STAT_POSITIONS,  (char*)"--positions",   SO_NONE    }, // --positions, print the positions of the words, when there are some
    { OPT_STAT_ALL,        (char*)"--all-positions", SO_NONE    }, // --all-positions, print all positions, even if there are more than 10

    // For Comparison Frequency
    { OPT_COUNT_CUTOFF, (char*)"--count_cutoff", SO_REQ_SEP    }, 
    { OPT_CONDEMNED_MIN_SIZE, (char*)"--condemned_min_size", SO_REQ_SEP    }, 
    { OPT_CONSEQFREQ, (char*)"--conseqFreq", SO_REQ_SEP    }, 
    { OPT_CONSEQUNFREQ, (char*)"--conseqUnFreq", SO_REQ_SEP    }, 
    { OPT_SUITABLE_MIN_SIZE, (char*)"--suitable_min_size", SO_REQ_SEP    }, 
    
    SO_END_OF_OPTIONS                       // END
};
*/

