/*
   $Id: ParametersSlice.cpp 666 2011-11-15 10:16:53Z manu $
*/
/*
 * Copyright INRA/CNRS - Thomas.Faraut@toulouse.inra.fr, Emmanuel.Courcelle@toulouse.inra.fr
 * This software is a computer program whose purpose is to align nucleic sequences, or perform other utility tasks
 * This software is governed by the CeCILL-C license under French law and abiding by the rules of distribution of free software.
 * 
 * Please read the file Licence_CeCILL-C_V1-en.txt which should have been distributed with this file
 * The fact that you are presently reading this, and the aforementioned file, means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
*/
#include "ParametersSlice.h"
#include "CommandChain.h"

CSimpleOpt ParametersInitializerSlice::_Init(int argc, char* argv[], const Command& cmd)
{
	CSimpleOpt::SOption options[] = {
	    { OPT_CHUNKSIZE,     (char*)"--chunk-size",SO_REQ_SEP    }, // "--output-format"
	    { OPT_SILENT,  (char*)"--silent",   SO_NONE }, //
	    { OPT_VERBOSE,  (char*)"--verbose", SO_NONE }, //
	    { OPT_VERBOSE,  (char*)"-v",        SO_NONE }, //
	    { OPT_RESULTSFILENAME,  (char*)"-o", SO_REQ_SEP }, // "-o"
	    { OPT_RESULTSFILENAME,  (char*)"--output-file", SO_REQ_SEP }, // "--output-file"
	
	    //pair-end mapping
	    { OPT_PAIREND, (char*)"--pair-end", SO_NONE  }, // "--pair-end"
	
	    // To get some help
	    { OPT_HELP, (char*)"-?",     SO_NONE    }, // "-?"
	    { OPT_HELP, (char*)"-h",     SO_NONE    }, // "-?"
	    { OPT_HELP, (char*)"--help", SO_NONE    }, // "--help"
	    
	    SO_END_OF_OPTIONS                       // END
	};

	//p.__Help = help;
	p.__argc = argc;

	// declare our options parser, pass in the arguments from main
    // as well as our array of valid options.
    CSimpleOpt args(argc, argv, options);
	while (args.Next()) {
		if (args.LastError() == SO_SUCCESS) 
		{
			if (args.OptionId() == OPT_HELP) 
			{
				cmd.Usage();
				exit(0);
			}
			else if (args.OptionId() ==  OPT_VERBOSE )
			{
				p.SetVerbose();
			}
			else if (args.OptionId() ==  OPT_SILENT )
			{
				p.SetSilent();
			}
            else if (args.OptionId() == OPT_CHUNKSIZE)
            {
				p.__output_chunk_size = atoi(args.OptionArg());
			}
            else if (args.OptionId() ==  OPT_PAIREND )
            {
            	p.__pair_end = true;
            }
            else if (args.OptionId() == OPT_RESULTSFILENAME )
            {
            	p.__results_filename = (CompressedFileName) args.OptionArg();
			}
		}			
	 }
     return args;
}

/**
 * \brief called by Init to finish the job
 */
void ParametersInitializerSlice::_Init2( const CSimpleOpt& args, const Command& cmd )
{
	if (args.FileCount()==0)
	{
		p.__query_filename = (CompressedFileName) "-";
	}
	else if ( args.FileCount()==1)
	{
		p.__query_filename = (CompressedFileName) args.File(0);
	}
	else
	{	
		 cerr << "ERROR - Zero or one filename is required \n\n";
		 cmd.Usage();
		 exit(1);
	}
}

/** 
 *  \brief Validate the parameters useful for glintindex
 *  \return true if all good
*/
bool ParametersInitializerSlice::_Validation()
{
	bool ok = true;
	if ( p.__argc == 1 )
	{
		ok = false;
	}
	string query_filename = p.__results_filename.GetName();
	string bwd_filename;
	p.__ParseFileName(query_filename,bwd_filename);
	if ( bwd_filename != "" )
	{
		cerr << "ERROR - You should NOT specify two output files, as the bam file may store both wd and bwd reads\n";
		ok = false;
	}
	return ok;
}
