/*
   $Id: ParametersStat.cpp 649 2011-10-31 13:57:42Z manu $
*/
/*
 * Copyright INRA/CNRS - Thomas.Faraut@toulouse.inra.fr, Emmanuel.Courcelle@toulouse.inra.fr
 * This software is a computer program whose purpose is to align nucleic sequences, or perform other utility tasks
 * This software is governed by the CeCILL-C license under French law and abiding by the rules of distribution of free software.
 * 
 * Please read the file Licence_CeCILL-C_V1-en.txt which should have been distributed with this file
 * The fact that you are presently reading this, and the aforementioned file, means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
*/
#include "ParametersStat.h"
#include "Command.h"

CSimpleOpt ParametersInitializerStat::_Init(int argc, char* argv[], const Command& cmd)
{
	CSimpleOpt::SOption options[] = {
	    // For glintstat
	    { OPT_STAT_FREQUENCY,  (char*)"--frequency",     SO_NONE    }, // --frequency, print the frequency of the words
	    { OPT_STAT_DESCENDING, (char*)"--descending",    SO_NONE   }, // --descending, with --frequency: print descending order   
	    { OPT_STAT_CUTOFF,     (char*)"--cutoff",        SO_REQ_SEP   }, // --stat-cutoff, with --frequency: do not display words with nb of positions lower than this
		{ OPT_STAT_LEVEL,      (char*)"--level",         SO_REQ_SEP   }, // --stat-level, with --frequency: do not display words at level specified
	    { OPT_STAT_POSITIONS,  (char*)"--positions",     SO_NONE    }, // --positions, print the positions of the words, when there are some
	    { OPT_STAT_ALL,        (char*)"--all-positions", SO_NONE    }, // --all-positions, print all positions, even if there are more than 10
	    { OPT_HELP, (char*)"-?",     SO_NONE    }, // "-?"
	    { OPT_HELP, (char*)"-h",     SO_NONE    }, // "-?"
	    { OPT_HELP, (char*)"--help", SO_NONE    }, // "--help"
	
	    SO_END_OF_OPTIONS                       // END
	};

	//p.__Help = help;
	p.__argc = argc;

	// declare our options parser, pass in the arguments from main
    // as well as our array of valid options.
    CSimpleOpt args(argc, argv, options);
	while (args.Next()) {
		if (args.LastError() == SO_SUCCESS) 
		{
			if (args.OptionId() == OPT_HELP) 
			{
				cmd.Usage();
				exit(0);
			}
			else if (args.OptionId() ==  OPT_STAT_FREQUENCY )
            {
				p.__stat_frequency = true;
			}
			else if (args.OptionId() ==  OPT_STAT_CUTOFF )
            {
				p.__stat_cutoff = (uint8_t) atoi(args.OptionArg());
			}
			else if (args.OptionId() ==  OPT_STAT_LEVEL )
            {
				p.__stat_level = (uint8_t) atoi(args.OptionArg());
			}
				
			else if (args.OptionId() ==  OPT_STAT_DESCENDING )
			{
				p.__stat_frequency = true;
				p.__stat_descending = true;
			}
			else if (args.OptionId() ==  OPT_STAT_POSITIONS )
            {
				p.__stat_frequency = false;
				p.__stat_positions = true;
			}
			else if (args.OptionId() ==  OPT_STAT_ALL )
            {
				p.__stat_frequency = false;
				p.__stat_positions = true;
				p.__stat_all       = true;
			}		
		}
        else 
        {
            fprintf(stderr,"Invalid argument: %s\n", args.OptionText());
            fprintf(stderr,"Please use option --help to see the usage\n");
            exit(1);	
        }
    }
    return args;
}

/**
 * \brief called by _Init to finish the job
 */
void ParametersInitializerStat::_Init2 ( const CSimpleOpt& args, const Command& cmd )
{
	if ( args.FileCount()>=1 )
	{
		p.__bank_filename = (CompressedFileName) args.File(0);
	}
	p.__index_mode = !(p.__FindIndex());
}

/** 
 *  \brief Validate the parameters useful for glintindex
 *  \return true if all good
*/
bool ParametersInitializerStat::_Validation()
{
    bool ok = _CommonValidation();
    if ( p.__complexity < 0 || p.__complexity > 2 )
    {
        cerr << "ERROR - Complexity should be 0,1,2\n\n";
        ok = false;
    };
    if (p.__bank_nb_ind_pass<1 || p.__bank_nb_ind_pass >10)
    {
        cerr << "ERROR - nb_pass should be in the interval 1..9\n\n";
        ok = false;
    };
	if (p.__black_list_max_count<2 || p.__black_list_max_count>255)
	{
        cerr << "ERROR - max count (switch -n) should be in the interval 2..255\n\n";
        ok = false;
    };
	return ok;
}
