/*
   $Id: main_glint.cpp 673 2011-12-09 15:10:47Z manu $
*/
/*
 * Copyright INRA/CNRS - Thomas.Faraut@toulouse.inra.fr, Emmanuel.Courcelle@toulouse.inra.fr
 * This software is a computer program whose purpose is to align nucleic sequences, or perform other utility tasks
 * This software is governed by the CeCILL-C license under French law and abiding by the rules of distribution of free software.
 * 
 * Please read the file Licence_CeCILL-C_V1-en.txt which should have been distributed with this file
 * The fact that you are presently reading this, and the aforementioned file, means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
*/

#include <iostream>
#include <string>
#include <set>

#include "ptypes.h"
#include "../samtools/bam.h"
#include "utils.h"
#include "CommandIndex.h"
#include "CommandAlign.h"
#include "CommandFreq.h"
#include "CommandConvert.h"
#include "CommandStat.h"
#include "CommandChain.h"
#include "CommandSlice.h"

#ifndef __LP64__
    #error "GLINT MAY BE COMPILED ONLY ON A 64 BITS PLATFORM"
#endif

void Usage();
RealLogger* logger;
void RemoveCommand(int& argc,char* argv[]);

/*////////////// Main ///////////////*/
int main(int argc,char* argv[])
{
	Parameters::Init();
	CompressionInfo::Init();

	set<string> prgms;
	prgms.insert("--help");
	prgms.insert("index");
	prgms.insert("align");
	prgms.insert("self");
	prgms.insert("map");
	prgms.insert("mappe");
	prgms.insert("chain");
	prgms.insert("freq");
	prgms.insert("stat");
	prgms.insert("convert");
	prgms.insert("slice");

	if (argc<2)
	{
		Usage();
		cerr << "ERROR ! No command specified\n";
		exit(1);
	}	
	string command = argv[1];
	
	int value = 0;
	if ( command == "--help" )
	{
		Usage();
	}
	else
	{
		if ( prgms.find(command) == prgms.end() )
		{
			string executable = argv[0];
			executable += "old";
			execv(executable.c_str(),argv);
			
			cerr << "\nERROR ! The command " << command << " is NOT found\n\n";
			Usage();
			exit(1);
		}
		RemoveCommand(argc,argv);
		
		Command* cmd = NULL;
		if ( command == "align" )
		{
			cmd = new CommandAlign();
		}
		else if ( command == "self" )
		{
			cmd = new CommandSelf();
		}
		else if ( command == "index" )
		{
			cmd = new CommandIndex();
		}
		else if ( command == "mappe" )
		{
			cmd = new CommandMappe();
		}
		else if ( command == "map" )
		{
			cmd = new CommandMap();
		}
		else if ( command == "chain" )
		{
			cmd = new CommandChain();
		}
		else if ( command == "freq" )
		{
			cmd = new CommandFreq();
		}
		else if ( command == "stat" )
		{
			cmd = new CommandStat();
		}
		else if ( command == "convert" )
		{
			cmd = new CommandConvert();
		}
		else if ( command == "slice" )
		{
			cmd = new CommandSlice();
		}
		
		value = cmd -> Main(argc,argv);
		delete cmd;
	}
	
	return value;
}

/** 
 * \brief Remove the command, ie argv[1], from the argv array
 * \param argc[in,out] The parameters count
 * \param argv[in,out] The array of char*
 * \return void
*/
void RemoveCommand(int& argc,char* argv[])
{
	argc--;
	for (int i = 1; i < argc; ++i)
	{
		argv[i] = argv[i+1];
	}
}
		
/*Utilisation du programme */
void Usage()
{
	PrintHeader();
	cerr << "Usage: glint <command> [options]\n\n";
	cerr << "Command:    index          index a bank of sequences (a genome) for later comparison\n";
	cerr << "            align          align a genomic sequence against a whole genome\n";
	cerr << "            self           align a genomic sequence against itself\n";
	cerr << "            map            map reads against a genome\n";
	cerr << "            mappe          map paired-end reads against a genome\n";
	cerr << "            chain          chain glint (genomic) or blast (proteic) hsps\n";
	//cerr << "            stat           Collect statistics about an indexed bank\n";
	
	cerr << "            convert        convert an alignment file between some common formats\n";
	cerr << "            slice          cut a BAM file into several slices\n";
	//cerr << "            freq           Extract regions from the indexed genome, with frequency criteria\n\n";
	cerr << "More help may be obtained for a command with:\n"; 
	cerr << "glint <command> --help\n\n";
}
