/*
   $Id: masque.h 649 2011-10-31 13:57:42Z manu $
*/
/*
 * Copyright INRA/CNRS - Thomas.Faraut@toulouse.inra.fr, Emmanuel.Courcelle@toulouse.inra.fr
 * This software is a computer program whose purpose is to align nucleic sequences, or perform other utility tasks
 * This software is governed by the CeCILL-C license under French law and abiding by the rules of distribution of free software.
 * 
 * Please read the file Licence_CeCILL-C_V1-en.txt which should have been distributed with this file
 * The fact that you are presently reading this, and the aforementioned file, means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
*/

#ifndef _MASQUE_H
#define _MASQUE_H

using namespace std;
#include <string>
#include <iostream>
#include <stdlib.h>
#include "ptypes.h"
#include "rtstack.h"

class Word;
class Window;

/* The Mask object - operator= and copy constructor are private, to disable them */

class Mask: public Singleton<Mask> {
private:
	Mask():initialized(false){};
	~Mask();

public:
	friend class Singleton<Mask>;
	void Initialize(const string& ) throw (rtstack);
	
public:
	short operator [] ( size_t i ) const { return pos[i]; };
	string Reconstruct(const string &, uint32_t, int) const;

	/* Some accessors... */
    uint8_t GetLength() const { return length;};                          // Get the total length of the mask
    uint8_t GetLength(uint8_t level) const { return level_length[level];};     // Get the length of the mask at a given level
    string GetString() const {return string_mask;};
    int GetLeafLevel() const { return leaf_level;};
	bool GetBlastzBehavior() const { return blastzlike;};
	uint8_t GetBitCount(uint8_t level)const { return bm[level];};

	/* Some mutators  */
	/* void SetBlastzbBehavior(const bool& b) { blastzlike=b; __Init(string_mask);};
	 void Change(const string& s, const bool &b) { blastzlike=b; __Init(s);};*/
	friend class Window;
	friend ostream & operator<<(ostream&, const Mask&);

	uint64_t* bm;     /* bitscore mask for bitcount scoring  */

private:
    void __Init(const string&);
    void __Free() {
        if (length==0)  // Nothing was allocated, thus nothing to free
            return;
        for(int i=ROOT_LEVEL;i<=leaf_level;i++)
            free(level_pos[i]);
        free(level_pos);
        free(pos);
    };
	bool initialized;
    uint8_t length;               /* The length of the mask */
    string string_mask;         /* The string representation */

    int level_length[MAX_LEAF_LEVEL]; /* taille du masque pour chacun des niveaux */
    int min_level;
    int max_level;
	short **level_pos;          /*                                         */
	short* pos;				    /*positions dans le masque = an array of 'length' short (0..length-1) */
    uint16_t leaf_level;
	bool blastzlike;
};


#endif

