#
# $Id: Fasta.pm 292 2007-02-19 14:30:09Z faraut $
#

package AnnotatedSeqFile::Fasta;
 
# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=head1 NAME

AnnotatedSeqFile::Fasta - parse a Fasta formatted file

=head1 SYNOPSIS

   my %h_param=( 'seq'          => "sequence_file",       # Mandatory: either a single sequence file or a multifasta 
                                                          # file (which will be assembled using the seqctg) 
                'chr'          => "chromosome_id",        # Mandatory
                'species'      => "short_species_name",   # Mandatory
                'kingdom'      => "species kingdom",      # Mandatory: either bacteria or eukaryota
                'seqctg'       => "golden_path_description_file"
                'agp_assembly' => "group_label_in_seqctg_will_be_used_for_the_assembly"	# default=reference
                'outformat'    => "output format",        # default=NARCISSE
                'tmpdir'       => "output directory"      # default=.

   my $o_gbk = New AnnotatedSeqFile::Fasta(%h_param);


=head1 DESCRIPTION

=cut

use strict;
use warnings;
use Carp;

use Bio::SeqIO;
use Bio::Seq;
use AnnotatedSeqFile;
use Data::Dumper;
@AnnotatedSeqFile::Fasta::ISA = qw(AnnotatedSeqFile);


BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 292 $ =~ /\d+/g); $r[0]};
}

=head2 procedure _Init

 Title        : _Init
 Usage        : my $o_fasta = New AnnotatedSeqFile::Fasta(...);
 Prerequisite : none
 Function     : Constructor, initialize the object, do not use directly, use New method instead
 Returns      : none
 Args         : see SYNOPSIS
 Globals      : none

=cut


sub _Init
{
	my ($self,%h_param) = @_;

	$self->SUPER::_Init(%h_param);

	my $seq_file = $self -> GetSequenceFile();
	
	if ( $seq_file eq '' || ! -e $seq_file || -z $seq_file )
	{
		throw LipmError::ParameterException('SEQUENCE FILE', 'missing');
	}
}
sub GetFormat
{
	return 'FASTA';
}
=head2 procedure Load

 Title        : Load
 Usage        : $o_fasta->Load();
 Prerequisite : none
 Function     : Do nothing as a fasta file does not contain annotation
 Returns      : none
 Args         : none
 Globals      : none

=cut

sub Load
{
	my($self) = @_;

	# easy to debug, nothing to load :)

}

=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license - www.cecill.info

=cut

1;
