#
# $Id$
#

package AnnotatedSeqFile::TIGRXML;
 
# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


=head1 NAME

AnnotatedSeqFile::TIGRXML - parse a TIGRXML formatted file

=head1 SYNOPSIS

   my %h_param=('infile'       => "filename",             # Mandatory: GFF3 formatted file
                'seq'          => "sequence_file",        # Mandatory: either a single sequence file or a multifasta 
                                                          # file (which will be assembled using the seqctg) 
                'chr'          => "chromosome_id",        # Mandatory
                'species'      => "short_species_name",   # Mandatory
                'kingdom'      => "species kingdom",      # Mandatory: either bacteria or eukaryota
                'outformat'    => "output format",        # default=NARCISSE
                'tmpdir'       => "output directory"      # default=.

				
   my $o_tigxml = New AnnotatedSeqFile::TIGRXML(%h_param);

   see the description of other public methods in the SYNOPSIS of AnnotatedSeqFile.pm


=head1 DESCRIPTION


#
# ATTENTION IL FAUT VERIFIER LES COORDONNEES DES EXTREMITES
#

=cut

BEGIN
{
	use File::Basename;
	my $dirprg=dirname($0);
	              # narcisse                               # eugene toolkit
	unshift @INC, "$dirprg/../lib/AnnotatedSeqFile","$dirprg/../../lib/narcisse";
}

use strict;
use warnings;
use Carp;

#use TIGRXMLLIB::Gene_obj;

use AnnotatedSeqFile;
use Data::Dumper;
use Locus::Gene;
use Locus::Rna;
use Locus::Repeat;
use TIGRXMLLIB::TIGR_XML_parser;
@AnnotatedSeqFile::TIGRXML::ISA = qw(AnnotatedSeqFile);


BEGIN
{
    our $VERSION = do {my @r = (q$Rev$ =~ /\d+/g); $r[0]};
}


=head2 procedure _Init

 Title        : _Init
 Usage        : my $o_gff3 = New AnnotatedSeqFile::GFF3(...);
 Prerequisite : none
 Function     : Constructor, initialize the object, do not use directly, use New method instead
 Returns      : none
 Args         : see SYNOPSIS
 Globals      : none

=cut

#sub _Init
#{
#	my ($self,%h_param) = @_;
#	$self->SUPER::_Init(%h_param);
#}
sub GetFormat
{
	return 'TIGRXML';
}
=head2 procedure Load

 Title        : Load
 Usage        : $o_tigrxml->Load();
 Prerequisite : the infile must be in TIGRXML format and should contain unique IDs
 Function     : Load the protein-coding genes, the ncRNA contained in a TIGRXML formatted file
                Warning: only the features are loaded
 Returns      : none
 Args         : none
 Globals      : none

=cut

sub Load
{
my ($self) = @_;

	my $filename = &AnnotatedSeqFile::GetStream($self->{'__infile'});

	my $tigr = new TIGR_XML_parser();
	$tigr->capture_genes_from_assembly_xml($filename);

	#print  $tigr->get_cds_coords();
	my $organism = $tigr->{organism};
	$organism=~s/\s/_/g;
	#$self->{'__species'}=$organism;

	my @a_genes = $tigr->get_genes();
	my $x;

	foreach my $gene (@a_genes)
	{
		$gene->refine_gene_object();
		my $code = $gene->{gene_type};
		my %h_param=();
		my ($start,$end) = $gene->get_coords();
		($h_param{start},$h_param{end}) = ($start < $end ) ? ($start,$end) : ($end,$start);
		$h_param{locus_tag} = $gene->{pub_locus};
		$h_param{locus_tag}=$gene->{TU_feat_name} if ( $h_param{locus_tag} eq '' );
		$h_param{strand} = ( $gene->{strand} eq '+' ) ? 1 : -1;
		$h_param{gene_name} = $gene->{locus};
		$h_param{symbol} = $gene->{locus};
		$h_param{gene_name} = $h_param{locus_tag} if ( !defined($h_param{gene_name}) && defined($h_param{locus_tag}));
		$h_param{locus_tag} = $h_param{gene_name} if ( !defined($h_param{locus_tag}) && defined($h_param{gene_name}));
		$h_param{product} = $gene->{com_name};
		$h_param{accession} = $h_param{locus_tag};
		$h_param{id} = $h_param{feat_name};
		$h_param{regions} = "$h_param{start}..$h_param{end}";
		$h_param{type} = "gene";

		if ( $code eq 'protein-coding' )
		{
			my $o_item = New Locus::Gene(%h_param);
			$self->AddGene($o_item);
			my $parent = $h_param{locus_tag};

			my $cpt=1;
			$self->__AddmRNA($gene,$parent,$cpt,\%h_param);
			foreach my $isoform ($gene->get_additional_isoforms())
			{
				$cpt++;
				$self->__AddmRNA($isoform,$parent,$cpt,\%h_param);
			}
		}

		if ( $code =~ /(\w+RNA)$/i )
		{
			my $id = $1;
			$h_param{type} = $id;
			my $o_item = New Locus::Rna(%h_param);
			$self->AddRna($o_item);
		}
		if ( $code =~ /repeat_region/ )
		{
			$h_param{type} = 'repeat';
			my $o_item = New Locus::Repeat(%h_param);
			$self->AddRepeat($o_item);
		}
		
		foreach my $key (keys(%h_param))
		{
			delete($h_param{$key});
		}
	}
	if ( ! defined($$self{'__seq_file'}) )
	{
		my $seqfile = "$$self{'__tmpdir'}/seqfastmp.$$self{'__chr'}";
		my $fseq = new IO::File(">$seqfile") or Carp::croak("Error: failed to create filename >$seqfile<");
		print $fseq ">",$tigr->{clone_name},"\n";
		print $fseq $tigr->get_assembly_sequence;
		$fseq->close;
		$$self{'__listofseqfiles'} = "$seqfile";
	}
	else
	{
		$$self{'__listofseqfiles'} = $$self{'__seq_file'};
	}
	$self->_Synchronize();
}

=head2 procedure __AddmRNA

 Title        : __AddmRNA
 Usage        : $o_tigrxml->__AddmRNA($gene,$parent,\%h_param);
 Prerequisite : none
 Function     : 
 Returns      : none
 Args         : none
 Globals      : none

=cut

sub __AddmRNA
{
	my($self,$gene,$parent,$iso,$rh_param) = @_;

	$$rh_param{locus_tag} = $gene->{model_pub_locus};
	if ( $$rh_param{locus_tag} eq '' )
	{
		$$rh_param{locus_tag} = $parent;
	}
	if ( ($$rh_param{locus_tag} eq $parent) && $iso>1 )
	{
		$$rh_param{locus_tag} = $parent;
		$$rh_param{locus_tag} .= "#$iso";
	}
	$$rh_param{gene_name} = $gene->{locus};
	$$rh_param{symbol} = $gene->{locus};
	$$rh_param{id} = $gene->{feat_name};
	$$rh_param{product} = $gene->{com_name};
	$$rh_param{accession} = $$rh_param{locus_tag};
	$$rh_param{type} = "mRNA";

	my @a_exons = $gene->get_exons();
	
	my @a_region_mrna = ();
	my @a_region_cds = ();
	my ($min_mrna,$max_mrna) = (99999999999999,0);
	foreach my $exon (@a_exons)
	{
		my ($mrna5,$mrna3) = $exon->get_coords();
		($mrna5,$mrna3) = ($mrna5<$mrna3) ? ($mrna5,$mrna3) : ($mrna3,$mrna5);
		my $cds = $exon->get_CDS_obj();
		my ($cds5,$cds3) =('','');
		push(@a_region_mrna,"$mrna5..$mrna3");
		if ( $cds )
		{
			($cds5,$cds3) = $cds->get_coords();
			($cds5,$cds3) = ($cds5<$cds3) ? ($cds5,$cds3) : ($cds3,$cds5);
			push(@a_region_cds,"$cds5..$cds3");
		}
	}
	# MRNA
	$$rh_param{type} = "mrna";
	my ($start,$end) = $gene->get_gene_span();	# attention
	($$rh_param{start},$$rh_param{end}) = ($start < $end ) ? ($start,$end) : ($end,$start);
	if ( $$rh_param{strand} > 0 )
	{
		$$rh_param{regions} = join(',',@a_region_mrna);
	}
	else
	{
		$$rh_param{regions} = join(',',reverse @a_region_mrna);
	}
	my $o_mrna = New Locus::Mrna(%{$rh_param});
	$self->AddGeneMRNA($o_mrna,$parent);

	# CDS
	$$rh_param{type} = "cds";
	($start,$end) = $gene->get_gene_span();	# attention
	($$rh_param{start},$$rh_param{end}) = ($start < $end ) ? ($start,$end) : ($end,$start);
	if ( $#a_region_cds > -1 )
	{
		if ( $$rh_param{strand} > 0 )
		{
			$$rh_param{regions} = join(',',@a_region_cds);
		}
		else
		{
			$$rh_param{regions} = join(',',reverse @a_region_cds);
		}
		my $o_cds = New Locus::Cds(%{$rh_param});
		$o_mrna -> AddCDS( $o_cds );		
	}
}

=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license - www.cecill.info

=cut

1;
