#
# $Id: Locus.pm 3308 2010-04-21 15:56:05Z manu $
#

package AnnotatedSeqFileWriter;

use strict;
use warnings;

use base qw( LipmObject );

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=head1 NAME

Locus - describes any locus on a sequence

=head1 SYNOPSIS

 my $o_annot_wrt = New AnnotatedSeqfileWriter::Narcisse( $o_annot );
 $o_annot_wrt -> Dump();

 

=head1 DESCRIPTION

 this ABSTRACT class defined two main abstract procedures:
	- Dump
	- DumpFasta
 DO NOT use this class, use a derived class instead

=cut

use Carp;
use LipmError::ObjectException;
use LipmObject;

use base qw( LipmObject );

BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 3308 $ =~ /\d+/g); $r[0]};
}

=head2 procedure _Init

 Title        : _Init
 Function     : Constructor, initialize the object, do not use directly, use New method instead
 Args         : $o_annot An AnnotatedSeqfile object

=cut

sub _Init
{
	my $self = shift;
	my ($o_annot) = @_;

	$self->SUPER::_Init();
	$self -> _Set ('_o_annot', $o_annot);
	$self -> _Set ('_rh_fasta_sequences', {});
}

=head2 procedure Dump

 Title        : Dump
 Usage        : $o_annot_writer -> DumpFastaDb ( %h_param )
 Procedure    : Dump the gene objects to some formatted file
 Args         : %h_param = The (keys, values) depend on the class (on the format)

=cut
sub Dump {
	my $self = shift;
	throw LipmError::ObjectException( 'AnnotatedSeqFileWriter', 'abstract method' );
}

=head2 procedure DumpFastaDb

 Title        : DumpFastaDb
 Usage        : $o_annot_writer -> DumpFastaDb ( %h_param )
 Procedure    : Dump the gene objects to fasta-formatted files
 Args         : %h_param = The (keys, values) depend on the class (on the format)

=cut
sub DumpFastaDb 
{
	my $self = shift;
	throw LipmError::ObjectException( 'AnnotatedSeqFileWriter', 'abstract method' );
}

=head2 function GetFastaNbSequences

 Title        : GetFastaNbSequences
 Prerequisite : The function DumpFastaDb must be called first
 Usage        : my $nb_proteins = $o_annot_writer -> GetFastaNbSequences ( 'prot' );
 Args         : $mode (gene,prot,cds,mrna)
                If an "illegal" mode is wanted, we return 0
 
=cut
sub GetFastaNbSequences
{
	my $self = shift;
	my ($mode) = @_;
	
	my $rh_fasta_sequences = $self -> _Get( '_rh_fasta_sequences' );
	
	return (defined $rh_fasta_sequences -> { $mode } ) ? $rh_fasta_sequences -> { $mode } : 0;
}

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license - www.cecill.info

=cut


1;
