#
# $Id: Gene.pm 3308 2010-04-21 15:56:05Z manu $
#

package AnnotatedSeqFileWriter::Narcisse;

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


=head1 NAME

AnnotatedSeqFileWriter::Narcisse - Dumping AnnotatedSeqFile objects for Narcisse

=head1 SYNOPSIS

 my $o_dumper = New AnnotatedSeqFileWriter::Narcisse( o_annot => $o_annot );

=head1 DESCRIPTION

 this class extends the AnnotatedSeqFileWriter, it dumps data from AnnotatedSeqFile to Narcisse files

=cut

use strict;
use warnings;
use Carp;

use Error qw(:try);
use Bio::Seq;
use Data::Dumper;
use General;

use Locus;
use base qw( AnnotatedSeqFileWriter );

BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 3308 $ =~ /\d+/g); $r[0]};
}


=head2 procedure DumpFastaDb

 Title        : DumpFastaDb
 Usage        : $o_annot_writer -> DumpFastaDb ( %h_param )
 Procedure    : Dump the genes objects to 4 fasta-formatted files
 Args         : Mandatory: 'gene' => "gene_fastadb_filename"
                Mandatory: 'prot' => "protein_fastadb_filename"
                Mandatory: 'cds'  => "cds_fastadb_filename"
                Mandatory: 'mrna' => "cds_transcript_filename"
 Globals      : none

=cut

__END__

=head2 procedure __DumpFastaDbEgn

 Title        : __DumpFastaDbEgn
 Usage        : &__DumpFastaDbEgn(%h_param)
 Prerequisite : none
 Function     : Specialized dumping procedure: manages fasta files for eugene
 Returns      : none
 Args         : 'intron' =>   "intron_fastadb_filename"
                'cds'=> "cds_fastadb_filename"
 Globals      : none

=cut

sub __DumpFastaDbEgn
{
my ($self,$r_seq,%h_param) = @_;

	my %h_fh  = ();
	foreach my $type ('intron','cds','annotastruct','accAG','donorGT','start_codon','utr5','utr3')
	{
		if (  defined($h_param{$type}) )
		{
			$h_fh{$type} = new IO::File($h_param{$type},"a") or Carp::croak("Error: unable to create >$h_param{$type}<\n");
		}
	}
	my $chr      = $$self{'__chr'};
	my $sp       = $$self{'__species'};
	my $kingdom  = $$self{'__kingdom'};
	my $source   = $h_param{source};
	my $ra_genes = $self -> {'_a_genes'};
	foreach my $o_gene (@$ra_genes)
    {
		foreach my $type ('intron','cds','annotastruct','accAG','donorGT','start_codon','utr5','utr3')
		{
			if ( defined($h_fh{$type}) )
			{
				select($h_fh{$type});
				$o_gene -> DumpFastaEgn($chr,$sp,$r_seq,$type,$kingdom,$source);
			}
		}
	}
	foreach my $type ('intron','cds','annotastruct','accAG','donorGT','start_codon','utr5','utr3')
	{
		if ( defined($h_fh{$type}) )
		{
			$h_fh{$type}->close;
		}
	}
	select(STDOUT);
}

=head2 procedure DumpFastaEgn

 Title        : DumpFastaEgn
 Function     : Print on the current filehandle the fasta sequences corresponding to the CDS, the protein, the mrna and the gene
                Warning: only protein coding genes are considered
 Usage        : print $o_gene->DumpFastaEgn($chr,$sp,$r_seq,$mode,$kingdom)
 Prerequisite : none
 Returns      : none
 Args         : $chr: chromosome id
                $sp: species short code
                $r_seq: reference on the fasta sequence
                $mode: prot|mrna_old|cds|gene
                $kingdom: bacteria|eukaryota
 Globals      : none

=cut

sub DumpFastaDb
{
my($self,$chr,$sp,$r_seq,$mode,$kingdom,$source)=@_;

	my $code = $mode;
	if ( $mode =~ /cds/i )
	{
		if ( ! $self->IsProteinCoding() )
		{
			#print STDERR "dump mode=$mode: Warning: ".$self->{__locus_tag}." is not a protein coding gene (no CDS), skipped...\n";
			return;
		}
		for(my $i=0; $i<$$self{'__nb_cds'}; $i++)
		{
			my $r_cds       = $$self{'__a_cds'}[$i];
			my $r_mrna      = $$self{'__a_mrnas'}[$i];
			my $exoncds     = $r_cds->GetRegions();
			if ( $exoncds eq '' )
			{
				print STDERR "No region for cds $i  ".$r_cds->GetAccession()."\n";
				next;
			}
			my %h_param     = $r_cds->GetAll('locus_tag','strand','product','gene_name','accession');
			my ($len,$sseq) = $self->GetSequence($r_seq,$h_param{strand},$exoncds);
			next if ( $len == 0 );
			my $acc = $h_param{accession}; # $r_cds->GetAccession();
			$acc = $h_param{locus_tag} if ( $acc eq '');

			my $gene_obj = Bio::Seq->new( -display_id => "$code:$acc", -seq => $sseq);
			my $prot_obj = ( $kingdom !~ /eukar/i ) ? $gene_obj->translate(undef,undef,undef,11,1) : $gene_obj->translate(undef,undef,undef,undef,1);
			my $seq_prot = $prot_obj->seq;
			chop($seq_prot) if ($seq_prot =~ /\*$/ );
			if ( $seq_prot =~ /\*/ )
			{
				print STDERR "DumpFastaEgn/Warning: in-frame stop codon for $code:$acc, cds removed\n>$seq_prot\n\n";
				return;
			}

			my $gn = ( defined($h_param{gene_name}) && $h_param{gene_name} ne '')  ? " gn=$h_param{gene_name}" : "";
			print "$h_param{locus_tag} $sseq\n";
		}
	}
	elsif ( $mode =~ /^utr/i )
	{
		for(my $i=0; $i<$$self{'__nb_mrna'}; $i++)
		{
			my $r_mrna = $$self{'__a_mrnas'}[$i];
			my $r_cds  = $$self{'__a_cds'}[$i];
			my $strand = &GetStrandForGff3($r_mrna->Get('strand'));
			my ($ra_utrmin,$ra_utrmax) = &GetUTR($strand,$r_cds->GetRegions(),$r_mrna->GetRegions());
			my $seq_utrmin = '';
			my $seq_utrmax = '';
			for(my $j=0;$j<=$#$ra_utrmin;$j++)
			{
				my ($start,$end) = split(/\.\./,$$ra_utrmin[$j]);
				my ($len,$sseq) = $self->GetSequence($r_seq,$strand,$start."..".$end);
				$seq_utrmin = ( $strand eq '+' ) ? "$seq_utrmin$sseq" : "$sseq$seq_utrmin";
			}
			for(my $j=0;$j<=$#$ra_utrmax;$j++)
			{
				my ($start,$end) = split(/\.\./,$$ra_utrmax[$j]);
				my ($len,$sseq) = $self->GetSequence($r_seq,$strand,$start."..".$end);
				$seq_utrmax = ( $strand eq '+' ) ? "$seq_utrmax$sseq" : "$sseq$seq_utrmax";
			}
			if ( $mode =~ /^utr5/ )
			{
				my $sseq = ( $strand eq '+' ) ? $seq_utrmin : $seq_utrmax;
				printf $r_mrna->Get('locus_tag').".utr5.%d $sseq\n",$i+1 if ( length($sseq) );
			}
			if ( $mode =~ /^utr3/ )
			{
				my $sseq = ( $strand eq '+' ) ? $seq_utrmax : $seq_utrmin;
				printf $r_mrna->Get('locus_tag').".utr3.%d $sseq\n",$i+1 if ( length($sseq) );
			}
		}
	}
	elsif ( $mode =~ /intron/i )
	{
		my $seqlen = length($$r_seq);
		for(my $i=0; $i<$$self{'__nb_mrna'}; $i++)
		{
			my @a_regions=split(',',$$self{'__a_mrnas'}[$i]->GetRegions());
			my %h_param   = $$self{'__a_mrnas'}[$i]->GetAll('locus_tag','strand','product','gene_name','accession');
			for(my $i=0;$i<$#a_regions;$i++)
			{
				my($start,$end)=split(/\.\./,$a_regions[$i]);
				my($startnext,$endnext)=split(/\.\./,$a_regions[$i+1]);
				my ($len,$sseq) = $self->GetSequence($r_seq,$h_param{strand},($end+1)."..".($startnext-1));
				printf "$h_param{locus_tag}.I%d $sseq\n",$i+1 if ( $len );
			}
		}
	}
	elsif ( $mode =~ /accAG|donorGT|start_codon/i )
	{
		use constant CONTEXT_SITES => 120;
		my $ctx = &CONTEXT_SITES;
		my $seqlen = length($$r_seq);
		for(my $i=0; $i<$$self{'__nb_cds'}; $i++)
		{
			my @a_regions=split(',',$$self{'__a_cds'}[$i]->GetRegions());
			my %h_param   = $$self{'__a_cds'}[$i]->GetAll('locus_tag','strand','product','gene_name','accession');
			my $site = ($mode =~ /accAG/ ) ? "AG" : (($mode =~ /donorGT/) ? "GT" : 'ATG');
			my $len_site = length($site);
			for(my $i=0;$i<=$#a_regions;$i++)
			{
				my($start,$end)=split(/\.\./,$a_regions[$i]);
				my($start_site,$end_site) = (0,0);
				if ( $site eq 'ATG' )
				{
					if ( $h_param{strand} !~ /\-/ ) # +
					{
						next if ( $i != 0 );
						$start_site = $start-&CONTEXT_SITES;
						$end_site   = $start+$len_site+&CONTEXT_SITES-1;
					}
					else
					{
						next if ( $i != $#a_regions );
						$start_site   = $end-$len_site-&CONTEXT_SITES+1;
						$end_site     = $end+&CONTEXT_SITES;
					}
				}
				else
				{
					if ( $h_param{strand} !~ /\-/ && $mode =~ /accAG/ ||
						 $h_param{strand} =~ /\-/ && $mode =~ /donorGT/  )
					{
						next if ( $i == 0 );
						$start_site = $start-$len_site-&CONTEXT_SITES;
						$end_site   = $start+&CONTEXT_SITES-1;
					}
					else
					{
						next if ( $i == $#a_regions );
						$start_site = $end-&CONTEXT_SITES+1;
						$end_site   = $end+$len_site+&CONTEXT_SITES;
					}
				}
				next if ( $end_site > $seqlen || $start_site < 1 );
				my ($len,$sseq) = $self->GetSequence($r_seq,$h_param{strand},$start_site."..".$end_site);
				printf ">$h_param{locus_tag}.$site%d\n$sseq\n",$i+1 if ( $len && $sseq =~ /^.{$ctx}$site.{$ctx}$/ );
				if (  $sseq !~ /^.{$ctx}$site.{$ctx}$/ )
				{
					my ($curr) = $sseq =~ /^.{$ctx}(\w{$len_site}).{$ctx}$/;
					print STDERR "Warning: unexpected site: ($start,$end) site:$start_site-$end_site $i/$#a_regions $h_param{strand} expected:$site current:$curr $sseq $len\n"; 
				}
				print STDERR "Warning: wrong length: $sseq $len\n" if ( $len != (2*$ctx+length($site)) );
			}
		}
	}
	elsif ( $mode =~ /annotastruct/i )
	{
		# pour commencer, seulement les exons codants
		for(my $i=0; $i<$$self{'__nb_cds'}; $i++)
		{
			my $r_cds       = $$self{'__a_cds'}[$i];
			my @a_regions=split(',',$r_cds->GetRegions());
			my %h_param     = $r_cds->GetAll('locus_tag','strand','product','gene_name','accession');
			for(my $i=0;$i<=$#a_regions;$i++)
			{
				my($start,$end)=split(/\.\./,$a_regions[$i]);
				my $strand = ( $h_param{strand} > 0 ) ? '+' : '-';
				my $code = undef;
				if ( $#a_regions == 0 )
				{
					$code = "E.Sngl";
				}
				else
				{
					if ( $i == 0 )
					{
						$code = ( $strand eq '+' ) ? "E.Init" : "E.Term";
					}
					elsif ( $i == $#a_regions )
					{
						$code = ( $strand eq '-' ) ? "E.Init" : "E.Term";
					}
					else
					{
						$code = "E.Intr";
					}
				}
				printf "$chr.$h_param{locus_tag}.%d\t$source\t$code\t$start\t$end\t.\t$strand\t.\t# Parent=$h_param{locus_tag}\n",$i+1;
			}
		}
	}
}

=head2 function GetUTR

 Title        : GetUTR
 Function     : 
 Usage        : 
 Prerequisite : none
 Returns      : none
 Args         : 
 Globals      : none

=cut

sub GetUTR
{
my($strand,$cds_region,$mrna_region) = @_;
my @a_utrmin = ();
my @a_utrmax = ();

	my @a_exon = split(',',$mrna_region);
	my ($mincds) = $cds_region =~ /^(\d+)/;
	my ($maxcds) = $cds_region =~ /(\d+)$/;
	my $mintrans=0;
	my $cptmin=0;
	my $cptmax=0;
	for(my $i=0;$i<=$#a_exon;$i++)
	{
		my($start,$end) = split(/\.\./,$a_exon[$i]);
		last if (  $mincds == $start );
		if ( $mincds > $end )
		{
			$a_utrmin[$cptmin++] = "$start..$end";
		}
		else
		{
			$a_utrmin[$cptmin++] = sprintf("$start..%d",$mincds-1);
			last;
		}
	}
	for(my $i=$#a_exon;$i>=0;$i--)
	{
		my($start,$end) = split(/\.\./,$a_exon[$i]);
		last if (  $maxcds == $end );
		if ( $maxcds < $start )
		{
			$a_utrmax[$cptmax++] = "$start..$end";
		}
		else
		{
			$a_utrmax[$cptmax++] = sprintf("%d..$end",$maxcds+1,$end);
			last;
		}
	}
	my @a_s_utrmin = sort @a_utrmin;
	my @a_s_utrmax = sort @a_utrmax;
	return (\@a_s_utrmin,\@a_s_utrmax);
}

=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license - www.cecill.info

=cut

1;


