#
# $Id: Gene.pm 3308 2010-04-21 15:56:05Z manu $
#

package AnnotatedSeqFileWriter::Narcisse;

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


=head1 NAME

AnnotatedSeqFileWriter::Narcisse - Dumping AnnotatedSeqFile objects for Narcisse

=head1 SYNOPSIS

 my $o_dumper = New AnnotatedSeqFileWriter::Narcisse( o_annot => $o_annot );

=head1 DESCRIPTION

 this class extends the AnnotatedSeqFileWriter, it dumps data from AnnotatedSeqFile to Narcisse files

=cut

use strict;
use warnings;
use Carp;

use Error qw(:try);
use Bio::Seq;
use Data::Dumper;
use General;

use Locus;
use base qw( AnnotatedSeqFileWriter );

BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 3308 $ =~ /\d+/g); $r[0]};
}

=head2 procedure Dump

 Title        : Dump
 Usage        : $o_annot_writer -> Dump( %h_params );
 Procedure    : Dump data to Narcisse-formatted files
 
 Args         : %h_params : a hash wth named parameters
                Mandatory: 'repeat'    => "repeat_filename"
                Mandatory: 'rna'       => "rna_filename"
                Mandatory: 'exon'      => "exon_filename"
                Mandatory: 'gene'      => "gene_filename"
                Mandatory: 'transcript'=> "transcript_filename"

=cut

__END__

TODO ==> A REPRENDRE AVEC LES NOUVEAUX OBJETS / NOUVELLES STRUCTURES DE DONNEES !!!


=head2 Procedure __DumpGFF3

 Title        : __DumpGFF3
 Usage        : &__DumpGFF3(%h_param)
 Prerequisite : none
 Procedure    : Specialized dumping procedure: manages GFF3 files
                The __length private field is updated
 Args         : see Dump
 Globals      : none

=cut

sub __DumpGFF3
{
	my ($self,%h_param) = @_;

	my $chr = $self->{'__chr'};
	foreach my $file ('outfile')
	{
		if ( !defined($h_param{$file}) )
		{
			Carp::croak("Fatal error: Unable to dump Narcisse file, filename for $file is not defined\n");
			return;
		}
	}
	my $seq = '';
	my $len = &AnnotatedSeqFile::LoadOneSeq(\$seq,$h_param{sequence});
	# NON !!! METHODE GetGenomicSequence de AnnotatedSeqFile
	my $f_gff3 = new IO::File("$h_param{outfile}","w") or Carp::croak("Error: unable to create >$h_param{outfile}<\n");
	select($f_gff3);
	print "##gff-version 3\n";
	print "##sequence-region $chr 1 $len\n";

	my $ra_genes = $self -> { '__a_genes' };
	foreach my $o_gene (@$ra_genes)
    {
		#my $uid = $self->__GetNewUID('g');
		my $uid = $o_gene -> GetNarcisseId();
		$o_gene -> DumpGFF3($chr,$uid,'gene');
    }

    my $ra_repeats = $self -> { 'a_repeats' };
    foreach my $o_repeat ( @$ra_repeats)
    {
		my $uid = $o_repeat -> GetNarcisseId();
		$o_repeat -> DumpGFF3($chr,$uid);
    }
    
    my $ra_ncrnas = $self -> { '__a_ncrnas' };
	foreach my $o_ncrna (@$ra_ncrnas)
    {
		my $uid = $o_ncrna -> GetNarcisseId();
		$o_ncrna -> DumpGFF3($chr,$uid);
    }
	$f_gff3->close;
	select(STDOUT);

	$self->{__length} = $len;
}







=head2 procedure Dump

 Title        : Dump
 Function     : 
 Usage        : 
 Prerequisite : none
 Returns      : none
 Args         : $chr: chromosome id
                $uid: gene unique id
 Globals      : none

=cut

sub DumpGFF3
{
my($self,$chr,$uid)=@_;

	my $strand = &GetStrandForGff3($self->{__strand});
	my $source = 'egn_io';
	my $len = $self->{__end}-$self->{__start}+1;
	print "$chr\t$source\tgene\t$self->{__start}\t$self->{__end}\t.\t$strand\t.\tID=$self->{__locus_tag};Name=$self->{__gene_name};length=$len\n";
	for(my $i=0; $i<$$self{'__nb_mrna'}; $i++)
	{
		my $locus_id = $self->GetLocusID($$self{'__nb_mrna'},$i+1);
		$locus_id =~ s/^\w+:/mRNA:/;
		my $mrnalocus_id = $locus_id;
		$mrnalocus_id = "mRNA:$mrnalocus_id" if ( $mrnalocus_id !~ /^mRNA/ );
		my $r_mrna = $$self{'__a_mrnas'}[$i];
		my $r_cds  = $$self{'__a_cds'}[$i];
		if ( ! defined($r_cds) )
		{
			print STDERR "dump: Warning: $locus_id, Inconsistency between mRNA and CDS detected, removed...\n";
			next;
		}
		my @a_exon_cds = split(',',$r_cds->GetRegions());
		my %h_phases=&GetPhases(\@a_exon_cds,$strand);
		$strand = &GetStrandForGff3($r_mrna->Get('strand'));
		my @a_exon = split(',',$r_mrna->GetRegions());
		my $len     = 0;
		my $nb_exon = 0;
		for(my $j=0;$j<=$#a_exon;$j++)
		{
			my $locusmrna_id  = sprintf("${locus_id}_e%d",($strand eq '+' ) ? $j+1 : $#a_exon-$j+1);
			$locusmrna_id =~ s/^\w+:/exon:/;
			$locusmrna_id = "exon:$locusmrna_id" if ( $locusmrna_id !~ /^exon/ );
			my ($start,$end) = split(/\.\./,$a_exon[$j]);
			$len += ($end-$start+1);
			$nb_exon++;
			my $oterm=&GetOntologyTerm('exon',$strand,$j,$#a_exon);
			my $phase = ( defined($h_phases{"$start-$end"}) ) ? $h_phases{"$start-$end"} : '.';
			print "$chr\t$source\texon\t$start\t$end\t.\t$strand\t$phase\tID=$locusmrna_id;Parent=$mrnalocus_id;Ontology_term=$oterm\n";
		}
		print "$chr\t$source\tmRNA\t".$r_mrna->Get('start')."\t".$r_mrna->Get('end')."\t.\t$strand\t.\tID=$mrnalocus_id;Name=".$self->Get('gene_name').";Parent=".$self->Get('locus_tag').";nb_exon=$nb_exon;length=$len\n";

		my ($ra_utrmin,$ra_utrmax) = &GetUTR($strand,$r_cds->GetRegions(),$r_mrna->GetRegions());
		my $utrc=0;
		for(my $j=0;$j<=$#$ra_utrmin;$j++)
		{
			my $locusutr_id  = sprintf("${locus_id}_u%d",$utrc++);
			my ($code) = ( $strand eq '+' ) ? 'UTR5' : 'UTR3';
			$locusutr_id =~ s/^\w+:/$code:/;
			my ($start,$end) = split(/\.\./,$$ra_utrmin[$j]);
			my $oterm=&GetOntologyTerm($code,$strand,$j,$#$ra_utrmin);
			my $label=( $code =~ /5$/ ) ? 'five_prime_UTR' : 'three_prime_UTR';
			$locus_id = "$label:$locus_id" if ( $locus_id =~ /^label/ || $locus_id !~ /^$code/ );
			print "$chr\t$source\t$label\t$start\t$end\t.\t$strand\t.\tID=$locusutr_id;Parent=$mrnalocus_id;Ontology_term=$oterm\n";
		}
		for(my $j=0;$j<=$#a_exon_cds;$j++)
		{
			my $locuscds_id  = sprintf("${locus_id}_c%d",($strand eq '+' ) ? $j+1 : $#a_exon_cds-$j+1);
			$locuscds_id =~ s/^\w+:/CDS:/;
			$locuscds_id = "CDS:$locuscds_id" if ( $locuscds_id !~ /^CDS/ );
			my ($start,$end) = split(/\.\./,$a_exon_cds[$j]);
			my $oterm=&GetOntologyTerm('CDS',$strand,$j,$#a_exon_cds);
			my $phase = ( defined($h_phases{"$start-$end"}) ) ? $h_phases{"$start-$end"} : '.';
			print "$chr\t$source\tCDS\t$start\t$end\t.\t$strand\t".$h_phases{"$start-$end"}."\tID=$locuscds_id;Parent=$mrnalocus_id;Ontology_term=$oterm\n";
		}
		for(my $j=0;$j<=$#$ra_utrmax;$j++)
		{
			my $locusutr_id  = sprintf("${locus_id}_u%d",$utrc++);
			my ($code) = ( $strand eq '+' ) ? 'UTR3' : 'UTR5';
			$locusutr_id =~ s/^\w+:/$code:/;
			my ($start,$end) = split(/\.\./,$$ra_utrmax[$j]);
			my $oterm=&GetOntologyTerm($code,$strand,$j,$#$ra_utrmax);
			my $label=( $code =~ /5$/ ) ? 'five_prime_UTR' : 'three_prime_UTR';
			print "$chr\t$source\t$label\t$start\t$end\t.\t$strand\t.\tID=$locusutr_id;Parent=$mrnalocus_id;Ontology_term=$oterm\n";
		}
	}
	print "###\n";
}



=head2 function GetPhases

 Title        : GetPhases
 Function     : 
 Usage        : 
 Prerequisite : none
 Returns      : none
 Args         : 
 Globals      : none

=cut

sub GetPhases
{
my($ra_exon,$strand) = @_;
my $len = 0;
my %h_phases=();

	if ( $strand eq '+' )
	{
		for(my $i=0;$i<=$#$ra_exon;$i++)
		{
			my ($start,$end) = split(/\.\./,$$ra_exon[$i]);
			$h_phases{"$start-$end"} = (3 - ($len % 3))%3;
			$len += ($end-$start+1);
		}
	}
	else
	{
		for(my $i=$#$ra_exon;$i>=0;$i--)
		{
			my ($start,$end) = split(/\.\./,$$ra_exon[$i]);
			$h_phases{"$start-$end"} = (3 - ($len % 3))%3;
			$len += ($end-$start+1);
		}
	}
	if ( $strand eq '-' && $len % 3 != 0 )
	{
		foreach my $key (keys(%h_phases))
		{
			$h_phases{$key} = '.';
		}
	}
	return %h_phases;
}

=head2 function GetOntologyTerm

 Title        : GetOntologyTerm
 Function     : 
 Usage        : 
 Prerequisite : none
 Returns      : none
 Args         : $type: 'CDS' or 'exon'
                $it: rank of the current element
				$itmax: rank of the latest element
 Globals      : none

 	ATTENTION: il faudrait complexifier pour identifier les exon contenant une partie codante des autres

=cut


sub GetOntologyTerm
{
my($type,$strand,$it,$itmax)=@_;
my $oterm = '';

	# UTR
	if ( $type=~/UTR/ )
	{
		$oterm = ( $type eq 'UTR5' ) ? 'SO:0000204' : 'SO:0000205';
		return $oterm;
	}

	# exon / CDS
	if ( $itmax == 0 )
	{
		$oterm = ($type eq 'CDS' ) ? 'SO:0005845' : 'SO:0000147';
	}
	elsif ( $strand eq '+' )
	{
		if ( $it == 0 )		        # E.Init
		{
			$oterm = ($type eq 'CDS' ) ? 'SO:0000196' : 'SO:0000200';
		}
		elsif ( $it == $itmax )	# E.Term
		{
			$oterm = ($type eq 'CDS' ) ? 'SO:0000197' : 'SO:0000202';
		}
		else
		{
			$oterm = ($type eq 'CDS' ) ? 'SO:0000004' : 'SO:0000004';	# E.Intr 
			# attention doit etre codant et lorsqu'on a un intron d'UTR ce n'est pas forcement le cas
		}
	}
	else
	{
		if ( $it == 0 )		        # E.Term
		{
			$oterm = ($type eq 'CDS' ) ? 'SO:0000197' : 'SO:0000202' ;
		}
		elsif ( $it == $itmax )	# E.Init
		{
			$oterm = ($type eq 'CDS' ) ? 'SO:0000196' : 'SO:0000200';
		}
		else
		{
			$oterm = ($type eq 'CDS' ) ? 'SO:0000004' :  'SO:0000004';	# E.Intr
			# attention doit etre codant et lorsqu'on a un intron d'UTR ce n'est pas forcement le cas
		}
	}
	return $oterm;
}


=head2 function GetStrandForGff3

 Title        : GetStrandForGff3
 Function     : 
 Usage        : 
 Prerequisite : none
 Returns      : none
 Args         : 
 Globals      : none

=cut

sub GetStrandForGff3
{
my($strand) = @_;

	return '+' if ( $strand eq '+'  || $strand eq '1' || $strand eq '+1' );
	return '-' if ( $strand eq '-'  || $strand eq '-1' );
}



=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license - www.cecill.info

=cut

1;


