#
# $Id: Gene.pm 3308 2010-04-21 15:56:05Z manu $
#

package AnnotatedSeqFileWriter::Narcisse;

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


=head1 NAME

AnnotatedSeqFileWriter::Narcisse - Dumping AnnotatedSeqFile objects for Narcisse

=head1 SYNOPSIS

 my $o_dumper = New AnnotatedSeqFileWriter::Narcisse( o_annot => $o_annot );

=head1 DESCRIPTION

 this class extends the AnnotatedSeqFileWriter, it dumps data from AnnotatedSeqFile to Narcisse files

=cut

use strict;
use warnings;
use Carp;

use Error qw(:try);
use Bio::Seq;
use Data::Dumper;
use General;

use Locus;
use base qw( AnnotatedSeqFileWriter );

BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 3308 $ =~ /\d+/g); $r[0]};
}

=head2 procedure Dump

 Title        : Dump
 Usage        : $o_annot_writer -> Dump( %h_params );
 Procedure    : Dump data to Narcisse-formatted files
 
 Args         : %h_params : a hash wth named parameters
                Mandatory: 'repeat'    => "repeat_filename"
                Mandatory: 'rna'       => "rna_filename"
                Mandatory: 'exon'      => "exon_filename"
                Mandatory: 'gene'      => "gene_filename"
                Mandatory: 'transcript'=> "transcript_filename"

=cut

__END__

TODO ==> A REPRENDRE AVEC LES NOUVEAUX OBJETS / NOUVELLES STRUCTURES DE DONNEES !!!


=head2 procedure __DumpTIGRXML

 Title        : __DumpTIGRXML
 Usage        : &__DumpTIGRXML(%h_param)
 Prerequisite : none
 Procedure    : Specialized dumping procedure: manages TIGRXML files
                BUG: SHOULD UPDATE __length !!!
 Returns      : none
 Args         : Mandatory: 'outfile' =>   "output filename"
                Mandatory: 'sequence'=> "fasta_sequence_filename"
 Globals      : none

=cut

sub __DumpTIGRXML
{
my ($self,%h_param) = @_;

my $cpt=0;
my @a_pred=();

	my $chr = $self->{'__chr'};
	foreach my $file ('outfile','sequence','cfg')
	{
		if ( !defined($h_param{$file}) )
		{
			Carp::croak("Fatal error: Unable to dump Narcisse file, undefined value for $file\n");
			return;
		}
	}
	# NON !!! METHODE GetGenomicSequence de AnnotatedSeqFile
	my $seq = '';
	&AnnotatedSeqFile::LoadOneSeq(\$seq,$h_param{sequence});

	my $o_param = New ParamParser($h_param{cfg});

	my($filedate) = &GetFileDate($h_param{sequence});
	&__DumpTIGRXMLHeader($h_param{outfile},\$seq, $h_param{seq_id},$h_param{seq_acc},$h_param{seq_com},$filedate,$o_param->Get('organism'),$o_param->Get('taxonomy'),$h_param{chromosome},$o_param->Get('comment'),$o_param->Get('division'),$o_param->Get('contact'),$o_param->Get('database'));

	my $ra_genes = $self -> { '__a_genes' };
	foreach my $o_gene (@$ra_genes)
    {
		#print $o_gene->Print;
		my $tigr_gene = new Gene_obj();
		$tigr_gene->{TU_feat_name}=$o_gene->{'__locus_tag'};
		$tigr_gene->{pub_locus}=$o_gene->{'__locus_tag'};
		for(my $j=0; $j<$o_gene->{'__nb_cds'}; $j++)
		{
			my $r_cds       = $o_gene->{'__a_cds'}[$j];
			my $r_mrna      = $o_gene->{'__a_mrna'}[$j];

			# format 10..100,200..300
			my %h_mrna = ();
			my %h_cds = ();
			#print $r_mrna->GetRegions(),"\n";
			#foreach my $exon (split(',',$r_mrna->GetRegions()))
			foreach my $exon ($r_mrna->GetRegions())
			{
				my ($min,$max) = split(/\.\./,$exon);
				my ($end5,$end3) = ( $r_mrna->{'__strand'} > 0 ) ? ($min,$max) : ($max,$min);
				$h_mrna{$end5} = $end3;
			}
			#print "\nMRNA\n",Dumper(%h_mrna);
			#print $r_cds->GetRegions(),"\n";
			foreach my $exon ($r_cds->GetRegions())
			{
				my ($min,$max) = split(/\.\./,$exon);
				my ($end5,$end3) = ( $r_cds->{'__strand'} > 0 ) ? ($min,$max) : ($max,$min);
				#print "($end5,$end3) $exon\n";
				$h_cds{$end5} = $end3;
			}
			#print "\nCDS\n",Dumper(%h_cds);
			if ( $j > 0 )
			{
				my $tigr_gene_alt = new Gene_obj();
				$tigr_gene_alt->{TU_feat_name}=$o_gene->{'__locus_tag'};
				$tigr_gene_alt->{pub_locus}=$o_gene->{'__locus_tag'};
				$tigr_gene_alt->{com_name} = $r_mrna->{'__product'};
				$tigr_gene_alt->{Model_feat_name}=$r_mrna->{'__locus_tag'};
				$tigr_gene_alt->{model_pub_locus}=$r_mrna->{'__locus_tag'};
				$tigr_gene_alt->{locus}=$r_mrna->{'__symbol'};
				$tigr_gene_alt->populate_gene_obj(\%h_cds, \%h_mrna, \$seq);
				$tigr_gene_alt->create_gene_sequence(\$seq);
				$tigr_gene->add_isoform($tigr_gene_alt);
			}
			else
			{
				$tigr_gene->{com_name} = $r_mrna->{'__product'};
				$tigr_gene->{Model_feat_name}=$r_mrna->{'__locus_tag'};
				$tigr_gene->{model_pub_locus}=$r_mrna->{'__locus_tag'};
				$tigr_gene->{locus}=$r_mrna->{'__symbol'};
				$tigr_gene->populate_gene_obj(\%h_cds, \%h_mrna, \$seq);
			}
		}
		$tigr_gene->refine_gene_object();
		$tigr_gene->create_gene_sequence(\$seq);
		my $error ='';
		$error = $tigr_gene->validate_splice_sites(\$seq);
		if ( !defined($error) || $error eq '' )
		{
			$a_pred[$cpt++] = &__TIGRXMLGene($tigr_gene);
		}
		else
		{
			print STDERR "Removed due to error on sites: $error\n";
		}
	}
	
	my $ra_ncrnas = $self -> { __a_ncrnas };
	foreach my $o_ncrna (@$ra_ncrnas)
    {
		next if ( $o_ncrna -> {'__type'} !~ /^trna$/i );
		my ($min,$max) = ($o_ncrna->{'__start'},$o_ncrna->{'__end'});
		my ($end5,$end3) = ( $o_ncrna->{'__strand'} > 0 ) ? ($min,$max) : ($max,$min);
		my $mRNA_exon = new mRNA_exon_obj ($end5,$end3);
		my $gene_obj = new Gene_obj();
		$gene_obj->{gene_type}='trna';
		$gene_obj->{TU_feat_name}=$o_ncrna->{'__locus_tag'};
		$gene_obj->{pub_locus}=$o_ncrna->{'__locus_tag'};
		$gene_obj->add_mRNA_exon_obj($mRNA_exon);
		$gene_obj->refine_gene_object();
    }
	&__DumpTIGRXMLContent($h_param{outfile},\@a_pred);

}

sub __TIGRXMLGene
{
my ($gene) = @_;

	my($date) = &GetRunDate();

	my ($gn_end5,$gn_end3) = $gene->get_gene_span();
	my $text = "
	<TU>
		<FEAT_NAME>$gene->{pub_locus}</FEAT_NAME>
		<DATE>$date</DATE>
		<GENE_INFO>
			<LOCUS>$gene->{pub_locus}</LOCUS>
			<COM_NAME> </COM_NAME>
			<IS_PSEUDOGENE/>
			<DATE>$date</DATE>
		</GENE_INFO>
		<COORDSET>
			<END5>$gn_end5</END5>
			<END3>$gn_end3</END3>
		</COORDSET>
";
	my $cpt=1;
	foreach my $model ($gene,$gene->get_additional_isoforms())
	{
		$text .= &__TIGRXMLModel($model,$cpt);
	}
	my $seq_gene = $gene->get_gene_sequence();
	if ( length($seq_gene) != (abs($gn_end5-$gn_end3)+1) )
	{
		carp ("Error: the length of the gene sequence $gene->{pub_locus} (".length($seq_gene).")is not consistent with gene boundaries $gn_end5-$gn_end3\n");
	}
	$text .= "
	<TRANSCRIPT_SEQUENCE>$seq_gene</TRANSCRIPT_SEQUENCE>;
	</TU>
"
;

	return $text;
}


sub __TIGRXMLModel
{
my($model,$cpt) = @_;

	my($date) = &GetRunDate();
	my ($model_end5, $model_end3) = @{$model->{model_span}};
	my $text = "
	<MODEL COMMENT=\"\">
		<FEAT_NAME>$model->{Model_feat_name}</FEAT_NAME>
		<CDNA_SUPPORT/>
		<PUB_LOCUS>$model->{pub_locus}</PUB_LOCUS>
		<DATE>$date</DATE>
		<COORDSET>
			<END5>$model_end5</END5>
			<END3>$model_end3</END3>
		</COORDSET>
		<MODEL_EVIDENCE/>
";

	# To be able to map evidences, I need to keep the numbering mapped on exon with a cds
	my @a_exons = $model->get_exons();
	my $strand = $model->{strand};
	my $nb_cdsexons = 0;
    for(my $i=0; $i <= $#a_exons; $i++)
	{
		my $exon = $a_exons[$i];
		my $cds = $exon->{CDS_exon_obj};
		$nb_cdsexons++ if ( $cds );
	}
	my $cdsexoncpt=0;
	my $utrexoncpt=0;
    for(my $i=0; $i <= $#a_exons; $i++)
	{
		my $exon = $a_exons[$i];
		my $cds = $exon->{CDS_exon_obj};
		my $cpte = '';
		if ( $cds )
		{
			($cpte) = ($strand eq '+' ) ? ($cdsexoncpt+1) : ($nb_cdsexons-$cdsexoncpt);
			$cdsexoncpt++;
		}
		else
		{
			$utrexoncpt++;
			$cpte = "utr$utrexoncpt";
		}
		$text .= "
		<EXON>
			<FEAT_NAME>$model->{Model_feat_name}.e$cpte</FEAT_NAME>
			<DATE>$date</DATE>
			<COORDSET>
				<END5>$exon->{end5}</END5>
				<END3>$exon->{end3}</END3>
			</COORDSET>
		";

		if ($cds)
		{
			$text .= "
			<CDS>
				<FEAT_NAME>$model->{Model_feat_name}.c$cpte</FEAT_NAME>
				<DATE>$date</DATE>
				<COORDSET>
					<END5>$cds->{end5}</END5>
					<END3>$cds->{end3}</END3>
				</COORDSET>
			</CDS>";
			my $utr = '';
			if ( $cds->{end5} != $exon->{end5} )
			{
				my ($e5,$e3) = ( $exon->{strand} ) ? ($exon->{end5},$cds->{end5}-1) : ($exon->{end5},$cds->{end5}+1);
				$utr .= "<LEFT_UTR>
							<COORDSET>
								<END5>$e5</END5>
								<END3>$e3</END3>
							</COORDSET>
						</LEFT_UTR>";
			}
			elsif ( $cds->{end3} != $exon->{end3} )
			{
				my ($e5,$e3) = ( $exon->{strand} ) ? ($cds->{end5}+1,$exon->{end3}) : ($cds->{end5}-1,$exon->{end3});
				$utr .= "<RIGHT_UTR>
							<COORDSET>
								<END5>$e5</END5>
								<END3>$e3</END3>
							</COORDSET>
						</RIGHT_UTR>";
			}
			if ( $utr ne '' )
			{
				$text .= "<UTRS>$utr</UTRS>";
			}
		}
		else
		{
			$text .= "<UTRS>
				<EXTENDED_UTR>
					<COORDSET>
						<END5>$exon->{end5}</END5>
						<END3>$exon->{end3}</END3>
					</COORDSET>
				</EXTENDED_UTR>
			</UTRS>";
		}
		$text .= "
		</EXON> ";
    }
	my $seq_cds = $model->get_CDS_sequence();
	my $seq_prot = $model->get_protein_sequence();
	my $stop=chop($seq_prot);
	if ( $stop ne '*' || $seq_cds !~ /ATG/ || $seq_prot =~ /\*/ )
	{
		warn("Warning : $model->{Model_feat_name} contains in-frame stop or invalide start/stop codons");
		$seq_cds = '';
		$seq_prot = '';
	}

	$text .= "
	<CDS_SEQUENCE>$seq_cds</CDS_SEQUENCE>
	<PROTEIN_SEQUENCE>$seq_prot</PROTEIN_SEQUENCE>
	</MODEL>";
	return $text;
}


sub __DumpTIGRXMLHeader
{
my ($outfile,$r_seq,$seq_id,$seq_acc,$seq_com,$filedate,$organism,$taxonomy,$chromosome,$comment,$division,$contact,$database) = @_;

	my $len = length($$r_seq);
	my($date) = &GetRunDate();
	my($date2) = &GetRunDate2();

	my $f = new IO::File(">$outfile") or croak ("ERROR, unable to create >$outfile<\n");

	print $f <<END
<?xml version="1.0"?>
<!DOCTYPE TIGR PUBLIC "-\/\/TIGR\/\/TIGR TIGRXML/EN" "http://medicago.toulouse.inra.fr/Mt/EuGene/Exchange/XML/DTD/tigrxml.dtd">

<TIGR>
        <ASSEMBLY CLONE_ID="$seq_id" DATABASE="$database" CHROMOSOME="$chromosome" CURRENT_DATE="$date">
                <ASMBL_ID CLONE_NAME="$seq_id">$seq_id</ASMBL_ID>
                <COORDSET>
                        <END5>1</END5>
                        <END3>$len</END3>
                </COORDSET>
                <HEADER>
                        <CLONE_NAME>$seq_id</CLONE_NAME>
                        <SEQ_LAST_TOUCHED>
                                <DATE>$filedate</DATE>
                        </SEQ_LAST_TOUCHED>
                        <GB_ACCESSION>$seq_acc</GB_ACCESSION>
                        <ORGANISM>$organism</ORGANISM>
                        <LINEAGE>$taxonomy</LINEAGE>
                        <SEQ_GROUP>$division</SEQ_GROUP>
                        <KEYWORDS/>
                        <GB_DESCRIPTION>$seq_com</GB_DESCRIPTION>
                        <GB_COMMENT>$comment $date2</GB_COMMENT>
                        <AUTHOR_LIST CONTACT="$contact"/>
                </HEADER>
                <GENE_LIST>
                        <PROTEIN_CODING/>
                        <RNA_GENES/>
                </GENE_LIST>
                <ASSEMBLY_SEQUENCE>$$r_seq</ASSEMBLY_SEQUENCE>
        </ASSEMBLY>
</TIGR>
END
;
	$f->close;
}

sub __DumpTIGRXMLContent
{
my($xml_filename,$ra_pred) = @_;

	my($xml_twig)=XML::Twig->new();
	$xml_twig->parsefile($xml_filename);
	my($root)=$xml_twig->root;
	my $cpt=0;
	my(@a_pred2);
	my($anchor) = $xml_twig->first_elt('PROTEIN_CODING');
	foreach my $pred (@$ra_pred)
	{
		my $p = parse XML::Twig::Elt($pred);
		$a_pred2[$cpt++] = $p;
	}
	$anchor->set_content(@a_pred2);
	$xml_twig->set_pretty_print('record');
	my $xml_file = new FileHandle ">$xml_filename" or die "ERROR $0 >$xml_filename<\n";
	$xml_twig->print($xml_file);
	$xml_twig->purge;
	return;
}



=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license - www.cecill.info

=cut

1;


