# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

package Args_utils;

=pod

=head1 NAME

 Args_utils - a module that provide function to deal with commande line arguments.

=head1 DESCRIPTION

 This module provides some functions to deal with command line arguments.
 TODO - IS IT REALLY USEFUL ? cf ParamParser

=cut

use strict;
use warnings;

use Data::Dumper;
use IO::File;
use LipmError::IOException;

use General;

my $VERSION = 0.1;


=head2 function GetFromStringOrFile

 Usage        : $ra_list = GetFromStringOrFile( file => $filename,
                                                str => $str, sep => $sep );
 Prerequisite : none
 Function     : when args can be separated list or when values can be
                into a file, this function read all those sources and return
                one ref list. If $filename is '-', will read STDIN.
 Returns      : $ra_list, a ref list of all values read
 Args         : $str, string, a value or a coma separated list of values
                $filename, string, name of the file where to read data
                           if '-', will read STDIN.
 Error        : LipmError::IOException

=cut

sub GetFromStringOrFile
{
	my %h_args = ( file => undef,
	               str  => undef,
	               sep  => '[,;\t]\s*',
	               @_ );

	my( $filename, $str, $sep ) = @h_args{ qw( file str sep ) };

	my @a_list = ();

	if( General::IsNotNull( $str ) )
	{
		push( @a_list, split( /$sep/, $str ) );
	}

	if( General::IsNotNull( $filename ) )
	{
		# if the string is stdin
		if( $filename eq '-' )
		{
			&__ReadStream( *STDIN, \@a_list, $sep );
		}
		# if the string is a file
		elsif( -e $filename )
		{
			my $fe = new IO::File( $filename, 'r' )
				or throw LipmError::IOException( $filename, 'f' );
			&__ReadStream( $fe, \@a_list, $sep );
		}
		else
		{
			my $msg = "$filename cannot been reached, maybe it does not exist";
			$msg   .= " or you don't have the right to read it";
			warn $msg;
		}
	}

	# mise en majuscule pour le service qui ne passe pas par l'API
	# => GetAnnotationFromLocus_tags
# 	return map( uc, @a_words );

	return \@a_list;

=head2 Procedure __ReadStream

 Title        : __ReadStream
 Usage        : &__ReadStream( $flux, $ra_words );
 Prerequisite : none
 Procedure    : none
 Args         : $flux, $ra_words

=cut

	sub __ReadStream
	{
		my ( $flux, $ra_words, $sep ) = @_;

		while( my $line = <$flux> )
		{
			next	if( $line =~ /^#/ );	# comments
			next	if( $line =~ /^\s*$/ );	# empty line
			chomp( $line );
			my( $tag, @a_scrap ) = split( '#', $line );
			if( $tag )
			{
				$tag =~ s/^\s+|\s+$//g ;	#trim function
				push( @$ra_words, split( /$sep/, $tag )  );
			}
		}

		return;
	}

} # __FetchArgs

=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license - www.cecill.info

=cut

1;
