# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=head1 NAME

ChromosomeSliceBox - 

=head1 SYNOPSIS



=head1 DESCRIPTION



B<CONSTRUCTOR PARAMETERS>



=head1 SUBROUTINES



=head1 AUTHOR

Stahl Olivier : Olivier.Stahl@toulouse.inra.fr

=cut

package Box::ChromosomeSliceBox;

use strict;
use warnings;

use Data::Dumper;
use FrameTransformer::2D;

use base qw( Box );
use LipmError::ObjectException;

my $VERSION = do {my @r = (q$Rev: 1 $ =~ /\d+/g); $r[0]};


##################################################
###########			Constructor		 #############
##################################################
=head2  _Init

	Title        : 	_Init
	Usage        : 	_Init(%h_param)
	Prerequisite : 	none
	Procedure    : 	Initialization
	Returns      : 	reference of the blessed object
				   	else
					undef if the object can't be initialized
	Args         :	X0,Y0	=> coordinates of the box origin (top-left)
					X1,Y1	=> coordinates of the box end (bottom-right)
	Globals	  	 : 	object attributes
	Errors		 : 	LipmError::ObjectException

=cut
sub _Init 
{
	my $self = shift;
	
	my ($package, $filename, $line) = caller;
		
	# 0) Set default parameters
	my %h_param = (
					# Optional parameters
					justification	=> 'right',
					ref_neighbor	=> 1,										
					@_
				);
	
	# 1) Store new attributes
	my @a_attributes = ('justification','ref_neighbor');
	@$self{@a_attributes} = @h_param{@a_attributes};
	
	# 2) Store inherited attributes
	$self -> SUPER::_Init( %h_param );	
			
	return $self;
}


##################################################
###########			Functions		 #############
##################################################
=head2  GetAttributes
	Title        :	GetAttributes
	Usage        :	$self->GetAttributes()
	Prerequisite :	This object should be initialized
	Function     :	Get the attributes of this object to make a copy
	Returns      : 	%h_attributes
	Args         : 	none
	Globals      : 	none

=cut 
sub GetAttributes 
{
	my $self = shift;
	
	my %h_attributes = $self->SUPER::GetAttributes();
	my @a_attributes = ('justification','ref_neighbor');
	
	@h_attributes{@a_attributes} = @$self{@a_attributes};
	
	return %h_attributes;
}

=head2  GetJustification
	Title        :	GetJustification
	Usage        :	$self->GetJustification()
	Prerequisite :	none
	Function     :	Return the attribute 'justification' value.
	Returns      :	'right', 'left' or 'center
	Args         :	none
	Globals      :	none
	
=cut 
sub GetJustification 
{
	my $self = shift; 
	
	return $self->{justification};
}

=head2  IsReferenceNeighbor
	Title        :	IsReferenceNeighbor
	Usage        :	$self->IsReferenceNeighbor()
	Prerequisite :	none
	Function     :	Return the attribute 'ref_neighbor' value.
	Returns      :	boolean
	Args         :	none
	Globals      :	none
	
=cut 
sub IsReferenceNeighbor 
{
	my $self = shift; 
	
	return $self->{ref_neighbor};
}


=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license - www.cecill.info

=cut


1;
