#
# 	sebastien.letort@toulouse.inra.fr
#	Created: January 30, 2009
#	$Id$
#
# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

package BoxesAdministrator;

=pod

=head1 NAME

 BoxAdministrator - create and manage Box objects.

=head1  SYNOPSIS

 # code

=head1 DESCRIPTION

 #descr

=cut

# J'ai fait ca pour pas tout reecrire ( tout ce qui utilise Box )
# perso, je n'aurai pas gere les Box comme ca _ pour moi une boite n'a pas a connaitre sa position dans l'image.

use strict;
use warnings;

use Data::Dumper;

use base qw( LipmObject );

use Box;

=head2 method _Init

 Title        : _Init
 Usage        : my $o_x = New BoxAdministrator( x, -o_logger => $o_logger );
 Prerequisite : none
 Fonction     : initialise an instance of BoxAdministrator class.
 Returns      : a valid BoxAdministrator object
 Args         : x
 Attributes   :

=cut

sub _Init
{
	my $self = shift;
	my %h_args = ( -o_logger    => undef,
	               -rh_padding  => {},
	               @_ );
	$self->SUPER::_Init( @_ );

	my $rh_padding = $h_args{ -rh_padding };
	$self->_SetPadding( %$rh_padding );


	$self->_Set( '__rh_boxes',   {} );

	# bottom-right corner ( top left is padding-left, padding-top )
	$self->_Set( '__x1', undef );
	$self->_Set( '__y1', undef );

	return $self;
}# _Init


=head2 Procedure _SetPadding

 Usage        : $o_xx->_SetPadding( %h_args );
 Prerequisite : Should be called by _Init only _ pour ca qu'elle est incluse !
                j'ai fait une fonction parce que je pense qu'elle pourrait
 Procedure    : none
 Args         : %h_args

=cut

sub _SetPadding
{
	my $self   = shift;
	my %h_args = ( top => 0,
				   bottom => 0,
				   right => 0,
				   left => 0,
				   @_ );

	$self->_Set( '_rh_padding', \%h_args );

	return;
}


=head2 Procedure _BuildBox

 Usage        : $o_xx->_BuildBox( $name, $x,$y, $width, $height );
 Prerequisite : none
 Procedure    : Build a Box named $name
 Args         : $name,
                $x,$y, integers, coords of the top left corner
                $width, int,
                $height, int

=cut

sub _BuildBox
{
	my $self = shift;
	my ( $name, $x,$y, $width, $height ) = @_;
# warn Dumper { x => $x, y => $y, chr => $name };

	my $rh_boxes = $self->_Get( '__rh_boxes' );

	my %h_param = (	X0 => $x,
					Y0 => $y,
					X1 => $x + $width,
					Y1 => $y + $height,
					-o_logger => $self->_GetLogger() );
	my $o_box = New Box( %h_param );
	$$rh_boxes{ $name } = $o_box;

	# MaJ de __x1, __y1
	my %h_coords = $o_box->GetBoxArea();
	( $x,$y )    = @h_coords{ qw( X1 Y1 ) };

	# check if bottom-right corner can be updated
	my $this_x1 = $self->_Get( '__x1' );
	$self->_Set( '__x1', $x )  if( !defined( $this_x1 ) || ( $this_x1 < $x ) );

	my $this_y1 = $self->_Get( '__y1' );
	$self->_Set( '__y1', $y )  if( !defined( $this_y1 ) || ( $this_y1 < $y ) );


	return;
}


# =head2 function
#
#
#  Title        : GetCoords
#  Usage        : ( $x, $y ) = $o_xx->GetCoords( $box_name, $x,$y );
#  Prerequisite : none
#  Function     : Given a point in Box coordinates,
#                 return a point in BoxAdministration coordinates
#  Returns      : ( $x, $y )
#  Args         : $box_name, $x,$y
#
#  Note : Ne sert jamais apparemment
#
# =cut
#
# sub GetCoords
# {
# 	my $self = shift;
# 	my( $box_name, $x,$y ) = @_;
#
# 	# de la facon dont Box est faite, c'est elle qui en a la responsabilite
# 	my $rh_boxes = $self->_Get( '__rh_boxes' );
# 	my ( $new_x,$new_y ) = $$rh_boxes{ $box_name }->GetCoords( $x,$y );
# 	return ( $new_x,$new_y );
# }


=head2 function

 Title        : GetImageParameters
 Usage        : New Image( $o_x->GetImageParameters() );
 Prerequisite : none
 Function     : return Image constructor args
 Returns      : hash like follows
                width => $width,
                height => $height,
                layers => \@a_layers
 Args         : none

=cut

sub GetImageParameters
{
	my $self      = shift;

	my $x1 = $self->_Get( '__x1' );
	my $y1 = $self->_Get( '__y1' );
	my $rh_padding = $self->_Get( '_rh_padding' );

	my $width  = $x1 + $$rh_padding{ right };
	my $height = $y1 + $$rh_padding{ bottom };

	# je ne sais pas ce que layers fait ici !
	return ( width => $width, height => $height ); #, layers =>[] );
}


# =head2 GetBoxesNumber
#
#  Title        : GetBoxesNumber
#  Usage        : my $nb = $o_x->GetBoxesNumber()
#  Prerequisite : none
#  Function     : Get the number of boxes managed by the object
#  Returns      : $nb, integer
#  Args         : none
#  Globals	  : none
#
# =cut
#
# sub GetBoxesNumber
# {
# 	my $self = shift;
#
# 	my $rh_boxes = $self->_Get( '__rh_boxes' );
# 	my $nb       = scalar( keys %$rh_boxes );
#
# 	# je n'ai trouve aucune justification pour ca
# # 	$return = 1 if ($return == 0);
#
# 	return $nb;
# }



=head2 GetBox
 Usage        : my $o_box = $o_x->GetBox( $chr )
 Function     : .
 Returns      : $o_box
 Args         : $chr

=cut

sub GetBox
{
	my $self = shift;
	my( $chr ) = @_;

	my $rh_boxes = $self->_Get( '__rh_boxes' );

	return $$rh_boxes{ $chr };
}

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license - www.cecill.info

=cut

1;

