#
# 	emmanuel.courcelle@toulouse.inra.fr
#	Created: May 2009
#	$Id$
#
# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

package ConfigurationFactory;

=pod

=head1 NAME

 ConfigurationFactory - Creates an InstanceObserver or a NarcisseObserver

=head1  SYNOPSIS

 my $o_conf = ConfigurationFactory::GetConfiguration();

=head1 DESCRIPTION

 This factory generates an InstanceObserver or a NarcisseObserver object, from some environment variables:
      - Using $ENV{REQUEST_URI}, we compute the instance name
        - If the instance name is 'multi' ==> Create a NarcisseObserver
        - Else ==> Create an InstanceObserver
        - If $ENV{REQUEST_URI} does not exist, throw an exception, because this factory cannot be used if not from a cgi

=cut

use strict;
use warnings;

use Data::Dumper;
use File::Basename;
use Error qw( :try );
use LipmError::ParameterException;

use General;
use Configuration::InstanceObserver;
use Configuration::NarcisseObserver;
use WebBuilder;


# ----	----	Static methods	----	----


=head2 function GetConfiguration,GetConfigurationNosingleton

 Title    : GetConfiguration,GetConfigurationNosingleton
 Usage    : my $o_conf = ConfigurationFactory::GetConfiguration();
            my $o_conf = ConfigurationFactory::GetConfigurationNoSingleton();
 Function : Create an object deriving from Configuration, using:
                -The env REQUEST_URI if it exists 
                     ==> From the web server we look for /xxx/instance/cgi-bin
                     ==> From the web_services we look for /xxx/instance/web_services
                -The env NARCISSE_CFG if it exists
            Using one of those two variables, we check the instance name:
                -if 'multi' we create a NarcisseObserver
                -else we create an InstanceObserver
            The GetConfigurationNoSingleton version does NOT use the singleton pattern
 Return   : a Configuration object
 Args     : $file_cfg = The bootstrap file
            $light    = Light initialization if &TRUE

=cut

sub GetConfiguration
{
	if ( !exists $ENV{REQUEST_URI} and !exists $ENV{NARCISSE_CFG})
	{
		throw LipmError::ParameterException('REQUEST_URI and NARCISSE_CFG', 'missing');
	};

	if (exists $ENV{REQUEST_URI})
	{
		my $request_uri = $ENV{REQUEST_URI};
		if ( $request_uri =~ /\/multi\/+cgi-bin\// or  $request_uri =~ /\/multi\/+web_services\//)
		{
			return New NarcisseObserver();
		}
		else
		{
			return InstanceObserver::Singleton();
		}
	}
	else
	{
		my $narcisse_cfg = $ENV{NARCISSE_CFG};
		if ( $narcisse_cfg =~ /\/multi\/+Site\// )
		{
			return New NarcisseObserver(@_);
		}
		else
		{
			return InstanceObserver::Singleton(@_);
		}
	}
}

sub GetConfigurationNoSingleton
{
	if ( !exists $ENV{REQUEST_URI} and !exists $ENV{NARCISSE_CFG})
	{
		throw LipmError::ParameterException('REQUEST_URI and NARCISSE_CFG', 'missing');
	};

	if (exists $ENV{REQUEST_URI})
	{
		my $request_uri = $ENV{REQUEST_URI};
		if ( $request_uri =~ /\/multi\/+cgi-bin\// or  $request_uri =~ /\/multi\/+web_services\//)
		{
			return New NarcisseObserver();
		}
		else
		{
			return New InstanceObserver();
		}
	}
	else
	{
		my $narcisse_cfg = $ENV{NARCISSE_CFG};
		if ( $narcisse_cfg =~ /\/multi\/+Site\// )
		{
			return New NarcisseObserver(@_);
		}
		else
		{
			return New InstanceObserver(@_);
		}
	}
}
=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license - www.cecill.info

=cut

1;
