# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=head1 NAME

	Histograms - a class to draw Histograms with Narcisse supplementary data.

=head1 DESCRIPTION

	Narcisse supplementary data are located in supp_X directories
	and sub-directories. Those data are in NarcisseData format.
	Mandatories column are start, end, id.

	This class draw an histogramm over the chromosome length to
	represent those data.

=head1 SUBROUTINES

=cut

package ElementsSet::SuppChart::Histograms;

use strict;
use warnings;

use Data::Dumper;

use base qw( ElementsSet::SuppChart );
my $VERSION = do {my @r = (q$Rev: 2614 $ =~ /\d+/g); $r[0]};


##################################################
###########			Procedures		 #############
##################################################

=head2 Display

 Title        : Display
 Usage        : $x->Display( %h_params );
 Procedure    : Display data of the Element inherited object on the Image object.
 Args         : %h_params, as follow
                	image, Image object
                	side, set the position of the representation
                		'left', 'center' or 'right' (default = 'right')
                	x,y, int,int, position of the (0,0) point.
 Globals      : none

=cut

sub Display
{
	my $self = shift;
	my %h_param = (
			image => undef,
			side  => undef,
			x     => 0,
			y     => 0,
			@_ );

	return if ( $self->IsEmpty() );

	my ( $x0,$y0 ) = @h_param{ qw( x y ) };
	my $o_image    = $h_param{ image };

	my @a_attr = qw( _file_ext _feature _pos_x );
	my( $layer, $feature, $pos_x ) = $self->_Get( @a_attr );

	my $ra    = $self->_GetColor( $o_image );
	my $color = $o_image->colorAllocate( @$ra, $layer );
	my $black = $o_image->colorAllocate( 'black', $layer );

	# from now on objects will be drawn on a single layer : $layer	
	$o_image -> SetCurrentLayer( $layer );

	# Draw and store each elements. => et le $y0 ne sert a rien ?
	my @a_fields= qw( start end value);
    my $elt_start = $self->_GetStart();
    my $elt_end   = $self->_GetEnd();
	foreach my $ref_elt ( $self->GetSortElements() )
	{
		my ( $start, $end, $value ) =  $self->GetFields( $ref_elt, @a_fields);
		next if( $value eq "" );

		next if ( $end <$elt_start || $start > $elt_end );
		
		my $x = $pos_x + $self->_NormalizeValue( $value );

		$o_image->filledRectangle( $x0, $start, $x,$end, $color );
		$o_image->rectangle( $x0, $start, $x,$end, $black );
		
	}

	return;
}

=pod

=head1 AUTHOR

Stahl Olivier : Olivier.Stahl@toulouse.inra.fr
Letort Sebastien : sebastien.letort@toulouse.inra.fr

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license - www.cecill.info

=cut

1;
