# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=head1 NAME

FrameTransformer - a class to define the frame in which an Image object will draw its elements

=head1 SYNOPSIS

FrameTransformer -> New(X0,Y0,X1,Y1);
FrameTransformer -> New(X0,Y0,X1,Y1,x0,y0,x1,y1);

=head1 DESCRIPTION

FrameTransformer is a class that permit to calculate absolute positions in a final image from relative coordinates, like nucleic positions in Narcisse.
Moreover this is a virtual representation of the frame in which we will draw particular elements. (like a ChromosomeSlice representation)


B<CONSTRUCTOR PARAMETERS>

X0,Y0,X1,Y1			=> 	Absolute coordinates of the frame. In pixel

x0,y0,x1,y1			=>	Relative coordinates of the frame. In base pair in Narcisse data.
						If you want to draw normally, just don't set those parameters (absolute values will be automatically used)


=head1 SUBROUTINES

	ToAbsoluteCoordinates
	RelativeCoordinate
	ValideCoordinate
	IsSetAsDefault
	CalculateScaleFactor
	ChangeReferences
	GetAttributes
	GetRelativeBounds
	GetBottomCorner
	GetFactor
	GetTopCorner
	MaskHeight
	MaskWidth

=head1 AUTHOR

Stahl Olivier : Olivier.Stahl@toulouse.inra.fr


## Cette classe ne sert à rien !
## Si on voulait faire de FrameTransformer une classe abstraite
## il fallait moins la fournir en méthodes !
## et comme c'est ce qui a été essayé, on est obligé de la laisser.

=cut

package FrameTransformer::1D;

use strict;
use warnings;

use base qw( FrameTransformer );

use LipmError::ObjectException;

my $VERSION = do {my @r = (q$Rev: 1 $ =~ /\d+/g); $r[0]};


##################################################
###########			Constructor		 #############
##################################################
=head2  _Init

	Title        : 	_Init
	Usage        : 	_Init(%h_param)
	Prerequisite : 	none
	Procedure    : 	Initialization
	Returns      : 	reference of the blessed object
				   	else
					undef if the object can't be initialized
	Args         :	X0,Y0	=> absolute coordinates of the frame origin (top-left)
					X1,Y1	=> absolute coordinates of the frame end (bottom-right)
					x0,y0	=> relative coordinates of the frame origin (top-left)
					x1,y1	=> relative coordinates of the frame end (bottom-right)
	Globals	  	 : 	object attributes
	Errors		 : 	LipmError::ObjectException

=cut
sub _Init
{
	my $self = shift;

	$self->SUPER::_Init( @_ );

	return $self;
}


# je l'ai commente car la fonction existe deja dans FrameTransformer <--Seb.
# si on veut garder la classe abstraite, il faut lui enlever cette méthode et la remettre ici.

# =head2  PointCoodinateChange
# 	Title		 :	PointCoodinateChange
# 	Usage		 :	$self -> PointCoodinateChange(x,y)
# 	Prerequisite :	none
#  	Function	 :  calculate the absolute coordinate (X,Y) of the point (x,y) in the final image
#  	Returns	  	 :	X,Y	(in pixel)
#  	Args		 :	x,y	(in base-pair)
# =cut
# sub PointCoordinateChange
# {
# 	my $self = shift;
# 	my ($x,$y) = (shift, shift);
# 	
# 	my $X = $x ;
# 	my $Y = int( $self->{Y0} + ($y-$self->{y0})*$self->{fy} );
# 
# 	return ($X,$Y);	
# }

=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license - www.cecill.info

=cut

1;

