#
# $Id: Gene.pm 2955 2009-10-23 08:48:47Z manu $
#

package Locus::Marker;

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


=head1 NAME

Locus::Rna - describes a rna

=head1 SYNOPSIS

=head1 DESCRIPTION

=cut

use strict;
use warnings;
use Carp;
use Bio::Seq;
use Data::Dumper;

use Locus;

use base qw( Locus );


BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 2955 $ =~ /\d+/g); $r[0]};
}

=head2 procedure _Init

 Title        : _Init
 Function     : Constructor, initialize the object, do not use directly, use New method instead
 Usage        : my $o_marker = New Locus::Marker(...); (see Locus for the details)
 Args         : See Locus
                The region is set to start::end
 Globals      : none

=cut

sub _Init
{
	my ($self,%h_param) = @_;
	$self->SUPER::_Init(%h_param);
}

=head2 procedure SetParent

 Title        : SetParent
 Usage        : $o_marker->SetParent( $o_annot );
 Procedure    : Set the parent
 Prerequisite : The parent should not already set
 
 Args         : $o_annot : The parent, an AnnotatedSeqfile.pm object (also a chromosome !)
 Globals      : none

=cut
sub SetParent
{
	my $self = shift;
	my ($o_annot) = @_;
	
	$self->SUPER::SetParent( $o_annot );
	
	my $nc_id = $o_annot -> GetNarcisseId();
	$nc_id .= 'g' . $o_annot->GetNbMarkers();
	$self -> { __narcisse_id } = $nc_id;
}

=head2 function GetNarcisseTranscriptId,GetNarcisseTranscriptParent

 Title        : GetNarcisseTranscriptId
 Function     : Return the narcisse Id, "as if" I were a transcript
                this is to fake Narcisse, pretending there is an mRNA inside this marker
 Usage        : my $id = $o_locus -> GetTranscriptNarcisseId()
 Returns      : the Narcisse Id
 Args         : none
 Globals      : none

=cut
sub GetNarcisseTranscriptId
{
	my $self = shift;
	return $self -> GetNarcisseId() . 't1';
}
sub GetNarcisseTranscriptParent
{
	my $self = shift;
	return $self -> GetNarcisseId();
}

=head2 functions GetLength, GetForwardPrimer,GetReversePrimer, GetcM

 Title        : Several accessors

=cut
sub GetLength 
{
	my $self = shift;
	return $self -> { __length };
}
sub GetcM
{
	my $self = shift;
	return $self -> { __cM };
}
sub GetForwardPrimer
{
	my $self = shift;
	return $self -> { __forward_primer };
}
sub GetReversePrimer
{
	my $self = shift;
	return $self -> { __reverse_primer };
}

=head2 function GetNarcisseExonId,GetNarcisseExonParent

 Title        : GetNarcisseExonId
 Function     : Return the narcisse Id, "as if" I were an exon
                this is to fake Narcisse, pretending there is a CDS inside the mRNA which should be inside this marker
 Usage        : my $id = $o_locus -> GetNarcisseExonId()
 Returns      : the Narcisse Id
 Args         : none
 Globals      : none

=cut
sub GetNarcisseExonId
{
	my $self = shift;
	return $self -> GetNarcisseTranscriptId() . 'e1';
}
sub GetNarcisseExonParent
{
	my $self = shift;
	return $self -> GetNarcisseTranscriptId();
}

=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license - www.cecill.info

=cut

1;


