#-----------------------------------------------------------------
# MOBY::RDF::Utils
# Author: Edward Kawas <edward.kawas@gmail.com>,
# For copyright and disclaimer see below.
#
# $Id: Utils.pm 2106 2008-10-10 13:10:42Z sletort $
#-----------------------------------------------------------------

package MOBY::RDF::Utils;

use XML::LibXML;
#use XML::LibXSLT;

use strict;

#-----------------------------------------------------------------
# load all modules needed for my attributes
#-----------------------------------------------------------------

=head1 NAME

MOBY::RDF::Utils - Create RDF/OWL for Moby datatypes

=head1 SYNOPSIS

	use MOBY::RDF::Utils;

 
=head1 DESCRIPTION

This module contains utility methods

=head1 AUTHORS

 Edward Kawas (edward.kawas [at] gmail [dot] com)

=cut

#-----------------------------------------------------------------

=head1 SUBROUTINES

=cut

#-----------------------------------------------------------------
# new
#-----------------------------------------------------------------
sub new {
	my ($class) = @_;

	# create an object
	my $self = bless {}, ref($class) || $class;

	$self->{xslt} = <<END;
<xsl:stylesheet version="1.0"
      xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
   <xsl:output method="xml" version="1.0" encoding="UTF-8" omit-xml-declaration="yes" indend="yes"/>
   <xsl:param name="indent-increment" select="'  '" />
   <xsl:template match="*">
      <xsl:param name="indent" select="'&#xA;'"/>
      <xsl:value-of select="\$indent"/>
      <xsl:copy>
        <xsl:copy-of select="@*" />
        <xsl:apply-templates>
          <xsl:with-param name="indent"
               select="concat(\$indent, \$indent-increment)"/>
        </xsl:apply-templates>
        <xsl:if test="*">
          <xsl:value-of select="\$indent"/>
        </xsl:if>
      </xsl:copy>
   </xsl:template>
   <xsl:template match="comment()|processing-instruction()">
      <xsl:copy ></xsl:copy>
   </xsl:template>
   <!-- WARNING: this is dangerous. Handle with care -->
   <xsl:template match="text()[normalize-space(.)='']"/>
</xsl:stylesheet>
END

	# done
	return $self;
}

=head2 prettyPrintXML

Return a string of XML that is formatted nicely 

=cut

sub prettyPrintXML {
	my ( $self, $hash ) = @_;
	my $xml        = $hash->{xml};
	unless ( $xml ) {
		$xml = <<END;
	<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"></rdf:RDF>
END
	}
	my $parser     = XML::LibXML->new();
#	my $xslt       = XML::LibXSLT->new();
	my $source     = $parser->parse_string($xml);
#	my $style_doc  = $parser->parse_string( $self->{xslt} );
#	my $stylesheet = $xslt->parse_stylesheet($style_doc);
#	my $results    = $stylesheet->transform($source);
#	$xml = $stylesheet->output_string($results);
	$xml = $source->toString(0);
	return $xml;
}

1;
__END__
