#
# 	sebastien.letort@toulouse.inra.fr
#	Created: 05 March 2009
#	$Id: ThroughSpecies.pm 3142 2009-12-22 12:38:52Z manu $
#

package MultiService::ThroughSpecies;

=pod

=head1 NAME

 MultiService::ThroughSpecies

=head1  SYNOPSIS

 # code

=head1 DESCRIPTION

 This class is just an interface

=cut

use strict;
use warnings;

use Data::Dumper;

use base qw( MultiService );

# use Error qw( :try );
# 
# use General;
# use Appli;
# use ParamParser;
# use Logger;
# use Configuration::NarcisseObserver;
use ConfigurationFactory;
# use LipmError;
# use LipmError::ObjectException;
# use Sequence_utils;

# le nom des options
use constant SPECIES    => 'species';


# -------Help functions---------------------------------

=head2 function _GetHelpOptions

 Usage        : $string = $o_xx->_GetHelpOptions();
 Prerequisite : none
 Function     : none
 Returns      : $string, description of options &INSTANCES and &SPECIES
 Args         : none

=cut

sub _GetHelpOptions
{
	my $self      = shift;

	my $string = $self->SUPER::_GetHelpOptions();

	my $format = '%-20s';
	my $p_species  = sprintf( $format, &SPECIES );

	$string .= '
    --' . $p_species . ' : string, short species name or coma separated list of short species name.';

	return $string;
}

# ------------------------------------------------------

=head2 function _GetOptLongList

 Usage        : $ra_options = $o_xx->_GetOptLongList( $ra_options );
 Prerequisite : none
 Function     : builf the option string list for ParamParser
 Returns      : $ra_options
 Args         : none

=cut

sub _GetOptLongList
{
	my $self = shift;
	my( $ra_options ) = @_;

	my $ra_mother = $self->SUPER::_GetOptLongList( $ra_options );
	push( @$ra_mother, &SPECIES  . '=s' );

	return $ra_mother;
}

# -------Mobyle related methods-------------------------

=head2 function

 Title      : _GetCommonParams
 Usage      : @a_list = $o_x->_GetCommonParams();
 Prerequiste: none
 Function   : return a list of hash ref of 2 parameters &INSTANCES and &SPECIES
              to all NarcisseService _instance and species.
 Args       : none.
 Return     : a hash containing the description of the instance parameter
              and the species param.
 Error      : none.

=cut

sub _GetCommonParams
{
	my $self         = shift;

	my @a_params = $self->SUPER::_GetCommonParams();

	my @a_names = ();

	# pour obtenir la liste des instances dispo
	# j'ai besoin d'un objet NarcisseObserver
	my $o_nar_obs = New NarcisseObserver( -o_logger => $self->_GetLogger() );
	my $ra_io     = $o_nar_obs->FindInstanceObservers() ;
	foreach my $o_io ( @$ra_io )
	{
		my @a_long     = $o_io->GetSpeciesLongName();
		my @a_no_space = ();
		foreach my $long ( @a_long )
		{
			$long =~ s/\s/_/g;
			push( @a_no_space, $long );
		}
		push( @a_names, @a_no_space );
	}

	my %h_names = map { $_ => '' } @a_names;
	$h_names{ all } = 'All the species';

	my %h_species = (
				name        =>	&SPECIES,
				datatype    =>	'Choice',
				type_biomoby=>	'String',
				enum        =>	\%h_names,
				default 	=>	'all',
				descr       =>	"the name of a species of narcisse DB. You can also use a coma separated list."
			);

	push( @a_params, \%h_species );

	return @a_params;
}

# ------------------------------------------------------

=head2 Procedure

 Title        : _InitAttributes
 Usage        : $o_xx->_InitAttributes( $o_param );
 Prerequisite : none
 Procedure    : almost copy-paste of mother method.
     retrieve all InstanceObserver object regarding &MultiService::INSTANCE
     argument of the program, and the &SPECIES argument.
     also set __ra_species attributes and call fetchandsetmandatories.
 Args         : $o_param

=cut

sub _InitAttributes
{
	my $self = shift;
	my ( $o_param ) = @_;

	my $str_instance = $o_param->Get( &MultiService::INSTANCES );

	my $str_species = $o_param->Get( &SPECIES );
	my $ra_species  = $self->_FetchArgs( $str_species );
	if( 1 == @$ra_species && $$ra_species[0] eq 'all' )
	{
		$ra_species = [];
	}

	$self->_Set( __ra_species => $ra_species );

	my $o_conf = &ConfigurationFactory::GetConfiguration();

	my $rao_inst_obs = [];
	if( $o_conf->isa( 'NarcisseObserver' ) )
	{
		# definir la liste des objets InstanceObserver
		my $ra_names     = $self->_FetchArgs( $str_instance );

		$rao_inst_obs = $o_conf->FindInstanceObservers( ra_instances => $ra_names,
														ra_species => $ra_species );
	}
	else
	{
		push( @$rao_inst_obs, $o_conf );
	}

	if( 0 == @$rao_inst_obs )
	{
		my $msg = "No instance found, check your parameters, \n";
		$msg   .= "instance '$str_instance' may not exist.\n";
		if( $str_species ne '' )
		{
			$msg   .= "maybe species '$str_species' do(es) not belong to the instance '$str_instance'.\n";
		}
		throw LipmError::ObjectException( $self, '!build', -text => $msg );
	}

	$self->_SetInstanceObserversListAttribute( $rao_inst_obs );

	# set attributes for mandatories args -//appele aussi apres par init de mother ?
# 	$self->_FetchAndSetMandatories( $o_param );

	return;
} # __InitAttributes


=head2 function _LaunchServiceOnInstance

 Usage        : $bool = $o_xx->_LaunchServiceOnInstance( $o_inst_obs );
 Prerequisite : method overridden to go through species list.
 function     :
 Args         : $o_inst_obs
 return       : $bool

=cut

sub _LaunchServiceOnInstance
{
	my $self = shift;
	my ( $o_inst_obs ) = @_;

	my %h_params = ( o_instance_obs => $o_inst_obs );

	my $rh_current = $self->_GetCurrentData();
	my $ra_species_args = $self->_Get( '__ra_species' );
	my $ra_species = $o_inst_obs->ExtractMySpecies( $ra_species_args );
	foreach my $species ( @$ra_species )
	{
		my $msg = "working with $species\n";
		$self->_Trace( $msg, '.', 'IS' );

		$$rh_current{ species } = $species;
		my $stop_flag = $self->_LaunchServiceOnSpecies();
		if( defined( $stop_flag ) )
		{
			return 1  if( 1 == $stop_flag );
		}
		# apres conversion de toute les procedures _LaunchServiceOnSpecies en fonction
		# virer le defined
	}

	return 0;
}



1;
