#
# 	Thomas.Faraut@toulouse.inra.fr
#	Created: March 22, 2009
#	$Id: NarcisseData.pm 2115 2009-03-22 07:24:38Z manu $
#

=pod

=head1 NAME

	NarcisseData - Interface class between NarcisseDataReader and Objects using the data

=head1  SYNOPSIS

	# code

=head1 DESCRIPTION

	This class is used to enable an interface between data reading and actual use of data
	This class is created in order to enable an easy reimplementation of NarcisseDataReader class with Inline::C++

=cut

package NarcisseData::ArrayReturn;

use strict;
use warnings;

use base qw( NarcisseData );

use Data::Dumper;
use Carp;

use General;

=head2 method _Init

 Usage        : my $o_x = New NarcisseData::ArrayReturn ( ra_data => \%a_data, rh_header => \%h_header )
 Prerequisite : none
 Fonction     : Constructor
 Args         : ra_data         The data, should be an array of arrays
                rh_header       The header, a hash: key = labels, values = column numbers
=cut
sub _Init
{
  my $self = shift;
  my (%h_args) = @_;

  throw LipmError::ParameterException("ra_data", 'missing')   unless defined $h_args{ ra_data };
  throw LipmError::ParameterException("rh_headers", 'missing') unless defined $h_args{ rh_headers };

  $self -> _Set ( _ra_data        => $h_args { ra_data } );
  $self -> _Set ( _rh_headers     => $h_args { rh_headers } );
  $self -> _Set ( _ra_sorted_elts => $self->_Get( '_ra_data' ) ); 
  
  return $self;
}

#----------------------------------
#    The methods
#---------------------------------


=head2 GetFields

 Title        : GetFields
 Usage        : $object -> GetFields($ref_a, @a_fields )
 Prerequisite : none
 Function     : return the values of each field of @fields in the hash ra_data corresponding to the
                columns of the keys $id
 Returns      : @fields values
 Args         : ref_elt : a reference on the element
 				fields : list of the fields to return

=cut
sub GetFields
{
	my $self = shift;
	my ( $ref_a, @a_fields ) = @_;
		
	# The different labels
	#my %h_headers = %{ $self->_Get( '_rh_headers' ) };
	
	# We make the assumption that the fields exist in the header of the file!!
	#return @{$ref_a}[ @h_headers{@a_fields} ];
	return @{$ref_a}[ @{$self->{ _rh_headers }}{@a_fields} ];
}

=head2 GetLabelIndices

 Title GetLabelIndices  
 Usage        : $object -> GetLabelIndices( )
 Prerequisite : none
 Function     : the rh_headers hash in order to write an inline version of GetFields
 Returns      : a hash with the field names as keys and the col indices of each field as values
 Args         : none

=cut
sub GetLabelIndices
{
	my $self = shift;
	
	my %h_headers = %{ $self->_Get( '_rh_headers' ) };
	
	return %h_headers;
}


1;
