#
#	Created: Feb 26, 2010
#	$Id: FastaReader.pm 3201 2010-02-05 13:32:36Z manu $
#

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

#
# ATTENTION - INCOMPATIBILITE ENTRE Runner et le module CGI::Fast a cause de la maniere dont la redirection de stdout est effectuee
#             POUR QUE CA MARCHE AVEC CGI::Fast il ne FAUT PAS utiliser -stdout 
#             NarcisseDataWriter::OnDisk a ete modifie pour prendre cela en compte
#

package NarcisseDataWriter::OnDisk;

=pod

=head1 NAME

 NarcisseDataWriter::OnDisk - a class to write data, Narcisse-formatted, with rather low memory print

=head1  SYNOPSIS

 See NarcisseDataWriter
    
=head1 DESCRIPTION

 At each call to Update, the data are simply appended to the file
 The file is sorted, using the Unix sort utility, when the object is destroyed.
 
 Log is obtained with 'W'.
 
=head1 BUGS


=cut

use base qw( NarcisseDataWriter );

use strict;
use warnings;

use Data::Dumper;
use IO::File;

use General;
use LipmError::IOException;
use LipmError::ParameterException;
use Runner;
use ConfigurationFactory;

use File::Basename;
use File::Path;

=head2 _Openfile

 Usage    : $self->_OpenFile($wrt_head_flg,$comment);
 Procedure: If the $wrt_head_flg is set (ie the file does not exist or is Zero length),
            open the file, write the comment and the headers
            Else, just open in append mode
            Set the _fh_file attribute
 Args     : $wrt_head_flg (true/false)
            $comment [undef]: A comment to insert at top of file
 Error    : IOException

=cut

sub _OpenFile
{
	my $self = shift;
	my ($wrt_head_flg,$comment) = @_;
	
	$self->__Reopen();
	
	# write the headers
	if ($wrt_head_flg == &TRUE)
	{
		$self->_PrintHeaders();
	}
	
	# Write the comment
	if (defined $comment)
	{
		$self->_PrintComments ( $comment );
	}
}
		
=head2 Procedure __Close

 Title        : __Close
 Usage        : $self->__Close()
 Procedure    : Close the file and set __fh_file to undef
 Args         : $bfr_flg  (default &TRUE) If &FALSE, we DO NOT FLUSH the buffer
                This should be used ONLY by __Flush, to avoid entering an infinite loop
           
=cut
sub __Close { 
	my $self = shift;
	my ($bfr_flg) = @_;
	$bfr_flg = &TRUE unless defined ( $bfr_flg );
	$self -> __Flush() if ($bfr_flg); 
	my $fh_file = $self->_Get('_fh_file');
	if (defined($fh_file))
	{
		$fh_file->close();
		$self->_Set('_fh_file',undef);
	}
}

=head2 Procedure __Flush

 Title        : __Flush
 Usage        : $self->__Flush()
 Procedure    : If there is a buffer, write it to the file and clear the buffer, then close the file
 Args         : None
           
=cut
sub __Flush
{
	my $self = shift;
	
	if ( defined $self -> _Get ('__ra_buffer') )
	{
		my $ra_buffer = $self -> _Get ('__ra_buffer');
		$self -> Update ( $ra_buffer );
		$self -> __Close( &FALSE );
		$ra_buffer = [];
		$self -> _Set ( '__ra_buffer', $ra_buffer );
	};	
}

=head2 Procedure __Reopen

 Title        : __Reopen
 Usage        : $self->__Reopen()
 Procedure    : If already opened, does nothing
                If closed, opens the file
 Args         : none
 
 Error        : IOException
           
=cut
sub __Reopen {
	my $self = shift;
	my $fh_file = $self->_Get('_fh_file');
	unless (defined $fh_file)
	{
		my $filename = $self->_Get('_filename');
		$fh_file = new IO::File($filename, 'a') or throw LipmError::IOException($filename,'f');
		$self->_Set('_fh_file',$fh_file);
	}
}

=head2 Procedure Update

 Title        : Update
 Usage        : $o_obj->Update( $rx_data, $comment );
 Procedure    : Write a chunk of data to the underlying file
                See also __SortFile

 Args         : $rx_data: A ref to a hash OR to an array
                          Each element may be:
                               - a hash, the key names are the names of the header array
                                 NOTE - We do not check if those names are the same as declared in $ra_header
                               - an array, the ORDER of data in the array should be the same as the order of columns in the header
                                 NOTE - We do not check the order !
                $comment: A comment to insert before the data (optional)
                	
=cut

sub Update
{
	my $self = shift;
	my ( $rx_data, $comment ) = @_;
	
	my $filename = $self->_Get('_filename');

	# Do nothing if no data
	my $is_array_ref = ( 'ARRAY' eq ref( $rx_data ) );
	my @a_rows = $is_array_ref ? @$rx_data : values %$rx_data;
	if( 0 == scalar( @a_rows ) )
	{
		my $msg = "no data, $filename was NOT written.\n";
		$self->_Trace( $msg, '*', 'W' );
		return;
	}
	# warn Dumper \@a_rows;

	$self->__Reopen();
	my $fh_file = $self->_Get('_fh_file');

	# print comments
	if( defined $comment )
	{
		$self->_PrintComments( $comment );
	}

	# print data
	my $ra_headers = $self->GetHeaders();
	foreach my $rx_values ( @a_rows )
	{
		if ( 'ARRAY' eq ref( $rx_values ) )
		{
			next if ( 0 == @$rx_values );
			print $fh_file join( &NarcisseDataWriter::SEPARATOR, @$rx_values ) . "\n";
		}
		else
		{
			next if( 0 == keys %$rx_values );
			my @a_values = @$rx_values{ @$ra_headers };
			print $fh_file join( &NarcisseDataWriter::SEPARATOR, @a_values ) . "\n";
		}
	}

	my $msg = "chunk successfully written in $filename - ";
	$msg .= $comment if (defined $comment);
	$msg .= "\n";
	$self->_Trace( $msg, '.', 'W' );

	return;
}

=head2 Procedure BfrdUpdate

 Title        : BfrdUpdate
 Usage        : $o_obj->BfrdUpdate( $rx_data );
 Procedure    : Put data inside a buffer
                When the buffer is full, flush it to the file and close the file
                So, we do not have a lot of opened files, even if we manage a lot of objects

 Args         : $rx_data: A ref to a hash OR to an array (see Update for the details)
                	
=cut

sub BfrdUpdate
{
	my $self = shift;
	my ( $rx_data ) = @_;
	
	# Do nothing if no data
	my $is_array_ref = ( 'ARRAY' eq ref( $rx_data ) );
	my @a_rows = $is_array_ref ? @$rx_data : values %$rx_data;
	if( 0 == scalar( @a_rows ) )
	{
		return;
	}
	# warn Dumper \@a_rows;
	
	# Create the buffer if it does not already exist
	my $ra_buffer;
	if ( defined $self -> _Get ('__ra_buffer') )
	{
		$ra_buffer = $self -> _Get ('__ra_buffer');
	}
	else
	{
		$ra_buffer = [];
		$self -> _Set ( '__ra_buffer', $ra_buffer );
	}
	
	# Put the new data inside the buffer
	push @$ra_buffer, @a_rows;
	
	# If the buffer is full, flush the buffer
	# TODO - Remplacer 10000 par un parametre !
	if ( @$ra_buffer > 10000 )
	{
		$self -> __Flush();
	}

	return;
}

=head2 Procedure __SortFile

 Usage     : $self->__SortFile ();
 Procedure : Call the unix program sort to sort the Narcisse file
             The comments are concateneted and put BEFORE the header
             This procedure is called by the destructor
 Args      : none
=cut 

sub __SortFile
{
	my $self = shift;
	
	
	my $filename = $self->_Get('_filename');

	# close the file
	$self->__Close();
	
	return unless (-e $filename);
	
	my $ra_keys  = $self->_Get('_ra_sort_keys');
	my $ascending= $self->_Get('_sort_ascending');
	my $numerical= $self->_Get('_sort_numerical');
	my $rh_numerical_byfield = $self->_Get('_sort_numerical_byfield');
	my $uniq     = $self->_Get('_sort_uniq');
	
	# Convert the keys to positions in the header
	# The convention is = If numerical sort required for this field, store the opposite (-3 = means sort field nb 3, numerical)
	my @a_positional_keys;
	my $ra_headers = $self->GetHeaders();
	foreach my $k (@$ra_keys)
	{
		my $fnd_flg=0;
		for (my $i=0; $i<=$#$ra_headers; $i++)
		{
			if ($k eq $ra_headers -> [$i])
			{
				push @a_positional_keys,(defined($rh_numerical_byfield->{$k})&&$rh_numerical_byfield->{$k}!=&FALSE)?-($i+1):$i+1;
				$fnd_flg=1;
				last;
			}
		}
		if ($fnd_flg==0)
		{
			throw LipmError::ParameterException('ra_keys','value', -text => "ERROR - Cannot find any column named $k in the file $filename");
		}
	}
	
	# build the unix cmd
	my $tmp_file = $filename.'.tmp';
		
	my $o_conf   = &ConfigurationFactory::GetConfiguration();
	my $unix_sort= $o_conf -> GetUnixPath('UNIX_SORT');
	my $unix_grep= $o_conf -> GetUnixPath('UNIX_GREP');
	my $unix_tail= $o_conf -> GetUnixPath('UNIX_TAIL');
	
	# Remove header and comments from the file
	# NOTE - Avoiding simple quotes (') in runExt
	my $cmd = "$unix_grep -v -E \"^;\" $filename | $unix_tail -n +2 >$tmp_file";
	#warn $cmd;
	&RunExt ( -snippet => 1, -cmd => $cmd );
	
	# Keep the comments, do not change their order
	# Using `` to keep the standard output to a variable
	$cmd = "$unix_grep -E '^;' $filename";
	#warn $cmd;
	my @a_comments = `$cmd`;
	
	# DEBUG ONLY: Copy the file to file.orig
	#link $filename,"${filename}.orig";
	unlink $filename;

	# Write the comments and header to the destination file
	$self->__Reopen();
	$self->_PrintComments( @a_comments );
	$self->_PrintHeaders();
	$self->__Close();
	
	# build and execute the sort command
	# using RunExt here to transfer the \t TAB separator to the Unix sort command
	$cmd = $unix_sort;
	# the following is correct from the shell, but does NOT work from RunExt - prefer to generate the character from perl
	# $cmd .= ' -t "`/bin/echo -e \'\\t\'`" ';
	$cmd .= ' -t "' . chr(9) . '"';
	$cmd .= ' -n' if ($numerical == &TRUE);
	$cmd .= ' -r' if ($ascending == &FALSE);
	$cmd .= ' -u' if ($uniq      == &TRUE);
	#$cmd .= ' -o ' . $filename;
	$cmd .= ' ';
	foreach my $pos (@a_positional_keys)
	{
		# Prefer using -k2,2 -k4,4 than -k2 -k4
		# Add 'n' if $pos < 0
		if ($pos<0)
		{
			$pos = -$pos;
			$cmd .= " -k$pos,$pos" . 'n';
		}
		else
		{
			$cmd .= " -k$pos,$pos";
		}
	}
	$cmd .= " $tmp_file >>$filename";
	#warn $cmd;
	#&RunExt (-cmd => $cmd, -stdout => ">>$filename");
	&RunExt ( -snippet => 1,-cmd => $cmd);

#    warn "KOUKOU $cmd";	
	unlink $tmp_file;
	
	return;
}

=head2 Procedure DESTROY

 Usage     : THE DESTRUCTOR
 Procedure : Close the file, then call __SortFile if the sort was asked by the constructor
 Args      : none
=cut 

sub DESTROY
{
	my $self = shift;
	if ($self -> _Get('_sort') == &TRUE)
	{
		$self -> __SortFile();
	}
	else
	{
		$self -> __Close();
	}
}

=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license - www.cecill.info

=cut

1;
