
# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


package NarcisseError;

=pod

=head1 NAME

 NarcisseError - a class to interface Error classes related to some Narcisse behaviour

=head1 SYNOPSIS

 # how to throw NarcisseError
 use NarcisseError;
 
 if (some_error_condition)
 {
    my $hd = "some title";
    my $bd = "some body, xhtml formatted";
    my $ft = "some footer"
    throw NarcisseError( -hd => $hd, -bd => $bd, -ft => $ft)
 }
 
 # how to catch NarcisseError
 # NEVER CATCH NarcisseErrors, just have a look to narcisse.cgi, the error will be nicely catched and displayed

=head1 DESCRIPTION
 Used to throw a Narcisse specific runtime error, which could be nicely displayed to the end user

=cut

use warnings;
use strict;

# use Data::Dumper;

## this class inherits from Error.pm
use base qw(Error);

BEGIN
{
	our $VERSION = do {	my @r = (q$Rev$ =~ /\d+/g); $r[0]	};
}


=head2 Function new

 Title        : new
 Usage        : throw NarcisseError( -hd => $hd, -bd => $bd, -ft => $ft);
 Function     : Constructor
 Returns      : an Error object that should be catched.
 Args         : $hd: should be displayed in the header part of the popup
                $bd: should be displayed in the body part of the popup
                $ft: should be displayed in the footer part of the popup
                $cv: Configuration file version
                $pv: Program Version
 Attributes   :
	#inherited
	-line       int
	-file       string
	-text       string
	-package    string

	_debug      string, build-in message. (no always present. )
	-value      int, error code, odd value should kill the program.
	-user_msg   string, message for a terminal user.

=cut

sub new
{
	my $class = shift;
	my %h_param = ( 
        -hd => '',
        -bd => '',
        -ft => '',
        -pv => '',
        -cv => '',
        @_
        );

	my $self = $class->SUPER::new();
    $self->{__hd} = $h_param{-hd};
    $self->{__bd} = $h_param{-bd};
    $self->{__ft} = $h_param{-ft};
    $self->{__cv} = $h_param{-cv};
    $self->{__pv} = $h_param{-pv};
	return $self;
}

sub hd {my $self=shift; return $self->{__hd};};
sub bd {my $self=shift; return $self->{__bd};};
sub ft {my $self=shift; return $self->{__ft};};
sub pv {my $self=shift; return $self->{__pv};};
sub cv {my $self=shift; return $self->{__cv};};

sub text {
    my $self = shift;
    return "<div class='hd'>$self->{__hd}</div><div class='bd'>$self->{__bd}</div><div class='ft'>$self->{__ft}</div>";
}

1;

=pod

=head1 AUTHOR

 Sebastien Letort : sebastien.letort@toulouse.inra.fr

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license - www.cecill.info

=cut
