#
# 	sebastien.letort@toulouse.inra.fr
#	Created: December 12, 2008
#	$Id: FromDecorator.pm 3142 2009-12-22 12:38:52Z manu $
#
# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


package FromDecorator;

=pod

=head1 NAME

 FromDecorator - one type of AccessorDecorator

=head1  SYNOPSIS

 # code

=head1 DESCRIPTION

 This class should be inherited.
 It redefines _GetAPILinks, the method that give a way to enter the API.

=cut

use strict;
use warnings;

use Data::Dumper;

use base qw( AccessorDecorator );


=head2 procedure

 Title        : _Fetch
 Usage        : $o_xx->_Fetch();
 Prerequisite : Design pattern Decorator
 Procedure    : just call _Fetch of the inner object.
 Args         : none

=cut

sub _Fetch
{
	my $self = shift;

	$self->{ _o_accessor }->_Fetch( @_ );

	return;
}


=head2 procedure

 Title        : _StoreNotFound
 Usage        : $o_xx->_StoreNotFound();
 Prerequisite : Design pattern Decorator
 Procedure    : just call _StoreNotFound of the inner object.
 Args         : none

=cut

sub _StoreNotFound
{
	my $self = shift;

	$self->{ _o_accessor }->_StoreNotFound( @_ );

	return;
}


=head2 function

 Title        : _GetAPILinks
 Usage        : $rao_links = $o_xx->_GetAPILinks();
 Prerequisite : Have to be override
 Function     : By defining this methods I assure that it is not the mother one
                that is called.
 Returns      : $rao_links, a ref list of API objects.
 Args         : none

=cut

sub _GetAPILinks
{
	my $self = shift;

	my $dbg = "_GetAPILinks should be overidden by a specific FromDecorator.\n";
	my $msg = "Somebody doesn't do his job well,";
	$msg   .= " tell your administrator to check error file.\n";

	my %h_args = ( -text => $dbg, -user_msg => $msg );
	throw LipmError::ObjectException( $self, 'abstract method', %h_args );

	my $rao_links = undef;

	return $rao_links;
}


1;

