#
# 	sebastien.letort@toulouse.inra.fr
#	Created: February 25, 2009
#	$Id: CDS_TransDecorator.pm 3142 2009-12-22 12:38:52Z manu $
#
# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


package WhatDecorator::CDS_TransDecorator;

=pod

=head1 NAME

 WhatDecorator::TranscriptsDecorator - a WhatDecorator to fetch Transcript through API

=head1  SYNOPSIS

 # code

=head1 DESCRIPTION

 NarcisseService::Accessor needs a FromDecorator and a WhatDecorator to be
 usable. This is a WhatDecorator, specific to access transcripts through API.
 What is found by _Fetch is stored with _StoreResults.


=cut

use strict;
use warnings;

use Data::Dumper;

# chemin d'access a WhatDecorator depuis lib/
# 'use' obligatoire pour avoir plusieurs paquets dans le meme fichier.
# si on ne le met pas ici,il faut le mettre dans l'executable
# je trouve plus propre de n'inclure dans l'executable que les paquets
# que l'on instancie.
use NarcisseService::Accessor::WhatDecorator;
use base qw( WhatDecorator );

=head2 Procedure _FetchFeaturesFromGene

 Usage        : $o_xx->_FetchFeaturesFromGene( $o_gene );
 Prerequisite : none
 Procedure    : Get all transcript for the gene
                and store them if they are consistency
 Args         : $o_gene, EnsEMBL Gene

=cut

sub _FetchFeaturesFromGene
{
	my $self = shift;
	my ( $o_gene ) = @_;

	my $locus_tag = $o_gene->stable_id();

	# obtenir tout les transcripts du gene
	my $o_transcript_adpt = $self->_GetDBAdaptor()->get_TranscriptAdaptor();
	my $rao_transcripts = $o_transcript_adpt->fetch_all_by_Gene( $o_gene );
	if( 0 == scalar( @$rao_transcripts ) )
	{
		$self->_StoreNotFound( $locus_tag );
	}

	foreach my $o_transcript ( @$rao_transcripts )
	{
		if( !$o_transcript->IsConsistent() )
		{
			my $msg = $o_transcript->GetConsistencyErrorMessage();
			$self->_StoreNotFound( { $locus_tag => $msg } );
			#/!\ on peut avoir un transcript incoherent et un autre bon
			# pour un meme gene.
		}
		else
		{
			$self->_StoreFeature( $o_transcript );
		}
	}

	return;
}


=head2 Procedure _FetchFeaturesFromSlice

 Usage        : $o_xx->_FetchFeaturesFromSlice();
 Prerequisite : none
 Procedure    : Get all Transcripts that belong to the slice.
 Args         : none

=cut

sub _FetchFeaturesFromSlice
{
	my $self = shift;
	my ( $o_slice ) = @_;

	my $rao_transcripts = $o_slice->get_all_Transcripts();

	if( 0 == scalar( @$rao_transcripts ) )
	{
		my $start = $o_slice->start();
		my $end   = $o_slice->end();
		$self->_StoreNotFound( "$start..$end" );
	}

	foreach my $o_transcript ( @$rao_transcripts )
	{
		$self->_StoreFeature( $o_transcript );
	}

	return;
}


1;

# ---------------------------------------------------
# ---------------------------------------------------

package WhatDecorator::TranscriptsDecorator;

=pod

=head1 NAME

 WhatDecorator::TranscriptsDecorator - a WhatDecorator to fetch Transcript through API

=head1  SYNOPSIS

 # code

=head1 DESCRIPTION

 NarcisseService::Accessor needs a FromDecorator and a WhatDecorator to be
 usable. This is a WhatDecorator, specific to access transcripts through API.
 What is found by _Fetch is stored with _StoreResults.

=cut

use strict;
use warnings;

use Data::Dumper;

use base qw( WhatDecorator::CDS_TransDecorator );


=head2 Function _GetFeatureSeq

 Usage        : my $seq = $o_x->_GetFeatureSeq( $o_transcript )
 Function     : just : return $o_transcript->spliced_seq();
 Returns      : $seq, the transcript sequence.
 Args         :
     $o_transcript, EnsEMBL Transcript
 Error        : none

=cut

sub _GetFeatureSeq
{
	my $self = shift;
	my( $o_transcript ) = @_;

	return $o_transcript->spliced_seq();
}


1;


# ---------------------------------------------------
# ---------------------------------------------------

#
# 	sebastien.letort@toulouse.inra.fr
#	Created: February 25, 2009
#	$Id: CDS_TransDecorator.pm 3142 2009-12-22 12:38:52Z manu $
#

package WhatDecorator::CDSsDecorator;

=pod

=head1 NAME

 WhatDecorator::TranscriptsDecorator::CDSsDecorator - a WhatDecorator to fetch CDS through API

=head1  SYNOPSIS

 # code

=head1 DESCRIPTION

 NarcisseService::Accessor needs a FromDecorator and a WhatDecorator to be
 usable. This is a WhatDecorator, specific to access CDSs through API.
 What is found by _Fetch is stored with _StoreResults.

 CDS is the nucleic sequence that translated give the protein seq.

 TODO - j'ai laisse cette classe dans transcript pour garder la relation entre eux
      - le code a ete duplique, mais quand fetch sera dans la classe mere, FetchTranscript
      deviendra FetchFeature et il y aura réduction de code.

=cut

use strict;
use warnings;

use Data::Dumper;

# chemin d'access a FromDecorator depuis lib/
# 'use' obligatoire pour avoir plusieurs paquets dans le meme fichier.
# si on ne le met pas ici,il faut le mettre dans l'executable
# je trouve plus propre de n'inclure dans l'executable que les paquets
# que l'on instancie.
use NarcisseService::Accessor::WhatDecorator;
use base qw( WhatDecorator::CDS_TransDecorator );

# 
# =head2 Procedure
# 
#  Title        : _FetchFeaturesFromGene
#  Usage        : $o_xx->_FetchFeaturesFromGene( $o_gene );
#  Prerequisite : none
#  Procedure    : Get all transcript for the gene
#                 and store them if they are consistency
#  Args         : $o_gene, EnsEMBL Gene
# 
# =cut
# 
# sub _FetchFeaturesFromGene
# {
# 	my $self = shift;
# 	my ( $o_gene ) = @_;
# 
# 	my $locus_tag = $o_gene->stable_id();
# 
# 	# obtenir tout les transcripts du gene
# 	my $o_transcript_adpt = $self->_GetDBAdaptor()->get_TranscriptAdaptor();
# 	my $rao_transcripts = $o_transcript_adpt->fetch_all_by_Gene( $o_gene );
# 	if( 0 == scalar( @$rao_transcripts ) )
# 	{
# 		$self->_StoreNotFound( $locus_tag );
# 	}
# 
# 	foreach my $o_transcript ( @$rao_transcripts )
# 	{
# 		if( !$o_transcript->IsConsistent() )
# 		{
# 			my $msg = $o_transcript->GetConsistencyErrorMessage();
# 			$self->_StoreNotFound( { $locus_tag => $msg } );
# 			#/!\ on peut avoir un transcript incoherent et un autre bon
# 			# pour un meme gene.
# 		}
# 		else
# 		{
# 			$self->_StoreFeature( $o_transcript );
# 		}
# 	}
# 
# 	return;
# }
# 
# 
# =head2 Procedure _FetchFeaturesFromSlice
# 
#  Usage        : $o_xx->_FetchFeaturesFromSlice();
#  Prerequisite : none
#  Procedure    : Get all CDSs that belong to the slice.
#  Args         : none
# 
# =cut
# 
# sub _FetchFeaturesFromSlice
# {
# 	my $self = shift;
# 	my ( $o_slice ) = @_;
# 
# 	my $rao_transcripts = $o_slice->get_all_Transcripts();
# 
# 	if( 0 == scalar( @$rao_transcripts ) )
# 	{
# 		my $start = $o_slice->start();
# 		my $end   = $o_slice->end();
# 		$self->_StoreNotFound( "$start..$end" );
# 	}
# 
# 	foreach my $o_transcript ( @$rao_transcripts )
# 	{
# 		$self->_StoreFeature( $o_transcript );
# 	}
# 
# 	return;
# }
# 

=head2 Function _GetFeatureSeq

 Usage        : my $seq = $o_x->_GetFeatureSeq( $o_transcript )
 Function     : just : return $o_transcript->translateable_seq();
 Returns      : $seq, the transcript sequence.
 Args         :
     $o_transcript, EnsEMBL Transcript
 Error        : none

=cut

sub _GetFeatureSeq
{
	my $self = shift;
	my( $o_transcript ) = @_;

	return $o_transcript->translateable_seq();
}



=head2 Function _GetHeader

 Usage        : my $header = $o_x->_GetHeader( $o_transcript )
 Function     : call mother and replace transcript type by cds type.
 Returns      : $header, string
 Args         :
     $o_transcript, EnsEMBL Transcript
 Error        : none

=cut

sub _GetHeader
{
	my $self = shift;
	my( $o_transcript ) = @_;

	my $header = $self->SUPER::_GetHeader( $o_transcript );

	# besoin de changer le mot clef transcript en cds
	$header =~ s/^>nc\|.+?\|/>nc|cds|/;

	return $header;
}


1;