#
# 	sebastien.letort@toulouse.inra.fr
#	Created: September 18, 2008
#	$Id: ExonsDecorator.pm 3185 2010-01-20 12:55:30Z sletort $
#

package WhatDecorator::ExonsDecorator;

=pod

=head1 NAME

 WhatDecorator::ExonsDecorator - a WhatDecorator to fetch Exon through API

=head1  SYNOPSIS

 # code

=head1 DESCRIPTION

 NarcisseService::Accessor needs a FromDecorator and a WhatDecorator to be
 usable. This is a WhatDecorator, specific to access exons through API.
 What is found by _Fetch is stored with _StoreResults.

=cut

use strict;
use warnings;

use Data::Dumper;

# chemin d'access a FromDecorator depuis lib/
# 'use' obligatoire pour avoir plusieurs paquets dans le meme fichier.
# si on ne le met pas ici,il faut le mettre dans l'executable
# je trouve plus propre de n'inclure dans l'executable que les paquets
# que l'on instancie.
use NarcisseService::Accessor::WhatDecorator;
use base qw( WhatDecorator );

# use NarcisseServiceKeywords qw( &NS_INFOS &NS_FASTA &NS_LOCUS );


# =head2 Procedure
# 
#  Usage        : $o_xx->__FetchGenesFromSlice();
#  Prerequisite : none
#  Procedure    : Get all Genes that belong to the slice.
#                 For each gene, get what we want.
#  Args         : none
# 
# =cut
# 
# sub _FetchFeaturesFromSlice
# {
# 	my $self = shift;
# 	my ( $o_slice ) = @_;
# 
# 	my $ra_genes = $o_slice->get_all_Genes();
# 
# 	if( 0 == scalar( @$ra_genes ) )
# 	{
# 		my $start = $o_slice->start();
# 		my $end   = $o_slice->end();
# 		$self->_StoreNotFound( "$start..$end" );
# 	}
# 
# 	foreach my $o_gene ( @$ra_genes )
# 	{
# 		$self->_StoreFeature( $o_gene );
# 	}
# 
# 	return;
# }


=head2 Procedure _FetchFeaturesFromGene

 Usage        : $o_xx->_FetchFeaturesFromGene( $o_gene );
 Prerequisite : none
 Procedure    : retrieve all Exons of the Gene object with get_all_Exons
                Then store each of them with _StoreFeature
 Args         : $o_gene, EnsEMBL Gene

=cut

sub _FetchFeaturesFromGene
{
	my $self = shift;
	my( $o_gene ) = @_;

	my $rao_exons = $o_gene->get_all_Exons();

	foreach my $o_exon ( @$rao_exons )
	{
		$self->_StoreFeature( $o_exon );
	}

	return;
}


1;
