#
# 	sebastien.letort@toulouse.inra.fr
#	Created: October 31, 2010
#	$Id: SNPsDecorator.pm 3695 2011-07-22 14:30:15Z sletort $
#
# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


package WhatDecorator::SNPsDecorator;

=pod

=head1 NAME

 WhatDecorator::SNPsDecorator - a WhatDecorator to fetch SNP through API

=head1  SYNOPSIS

 # code

=head1 DESCRIPTION

 NarcisseService::Accessor needs a FromDecorator and a WhatDecorator to be
 usable. This is a WhatDecorator, specific to access SNP through API.
 What is found by _Fetch is stored with _StoreResults.

=cut

use strict;
use warnings;

use Data::Dumper;

# chemin d'access a FromDecorator depuis lib/
# 'use' obligatoire pour avoir plusieurs paquets dans le meme fichier.
# si on ne le met pas ici,il faut le mettre dans l'executable
# je trouve plus propre de n'inclure dans l'executable que les paquets
# que l'on instancie.
use NarcisseService::Accessor::WhatDecorator;
use base qw( WhatDecorator );

use General;

# use NarcisseServiceKeywords qw( &NS_INFOS &NS_FASTA &NS_LOCUS );


=head2 Procedure

 Title        : _FetchFeaturesFromSlice
 Usage        : $o_xx->_FetchFeaturesFromSlice();
 Prerequisite : none
 Procedure    : Get all Genes that belong to the slice.
                For each gene, get what we want.
 Args         : none

=cut

sub _FetchFeaturesFromSlice
{
	my $self = shift;
	my ( $o_slice ) = @_;

	my $start = $o_slice->start();
	my $end   = $o_slice->end();

	# remplace un $o_slice->get_all_snp
	my $ra_snps;
	{
		# BaseFeature
		my $start = $o_slice->start();
		my $end   = $o_slice->end();
		my $chrom = $o_slice->seq_region_name;

		my $feature_key = 'SNP';

		my $o_species   = $o_slice->adaptor()->db()->conf();
		my %h_args      = ( -chrom => $chrom, key => $feature_key );
		my $chrom_file  = $o_species->GetFeatureFiles( %h_args );

		# BaseAdaptor
		my $o_ndr = New Bio::Narcisse::Utils::NarcisseDataReader();
		my %h_where = ( RANGE => {
							start => $start, start_label => 'position',
							end   => $end,   end_label => 'position' } );

		$o_ndr->Prepare( -from  => $chrom_file,
						 -where => \%h_where,
						 -case_sensitive => &FALSE );
		$ra_snps = $o_ndr->FetchallArrayref();
	}

	if( 0 == scalar( @$ra_snps ) )
	{
		##
		$self->_StoreNotFound( "$start..$end" );
	}

	$self->_StoreResults( $ra_snps );

	return;
}

1;
