#
# 	sebastien.letort@toulouse.inra.fr
#	Created: January 20, 2009
#	$Id: SlicesDecorator.pm 3408 2010-07-06 09:10:12Z sletort $
#
# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


package WhatDecorator::SlicesDecorator;

=pod

=head1 NAME

 WhatDecorator::SlicesDecorator - a WhatDecorator to fetch sequences through API

=head1  SYNOPSIS

 # code

=head1 DESCRIPTION

 NarcisseService::Accessor needs a FromDecorator and a WhatDecorator to be
 usable. This is a WhatDecorator, specific to access sequences (Slice) through API.
 What is found by _Fetch is stored with _StoreResults.

=cut

use strict;
use warnings;

use Data::Dumper;

# chemin d'access a WhatDecorator depuis lib/
# 'use' obligatoire pour avoir plusieurs paquets dans le meme fichier.
# si on ne le met pas ici,il faut le mettre dans l'executable
# je trouve plus propre de n'inclure dans l'executable que les paquets
# que l'on instancie.
use NarcisseService::Accessor::WhatDecorator;
use base qw( WhatDecorator );

# valeur par defaut pour les up/downstream
use constant DEF_LENGTH => 1_000;



=head2 method _Init

 Title        : _Init
 Usage        : my $o_x = New WhatDecorator::SlideDecorator::UpstreamDecorator(
                                       o_accessor => $o_accessor,
                                       -length    => $length
                                      );
 Prerequisite : none
 Fonction     : call grand-ma _Init (AccessorDecorator).
                set the specific attribute _length.
 Returns      : a valid NarcisseService::Accessor object
 Args         : $o_accessor, a NarcisseService::Accessor object
                $length, the length of the sequence to return
 Attributes   :
     __o_logger       : Logger object

     _o_instance_obs : InstanceObserver object

     _species        : species short name
     _o_db_adaptor   : DBAdaptor object (API)
     _ra_results     : array ref that contains any results.

     _length         : the length of the sequence to return

=cut

sub _Init
{
	my $self = shift;
	my %h_args = ( -length => &DEF_LENGTH,
	               @_ );

	$self->SUPER::_Init( %h_args );

	$self->_Set( _length => $h_args{ -length } );

	return $self;
}



=head2 function _GetFeatureId

 Usage        : $id = $o_xx->_GetFeatureId( $o_feature );
 Prerequisite : none
 Function     : return an Id that describes the feature.
                I use a function to allow overloading by SliceDecorator.
 Returns      : $id, string, id of $o_feature
 Args         : $o_feature, a Bio::EnsEMBL::x object

=cut

sub _GetFeatureId
{
	my $self = shift;
	my( $o_feature ) = @_;

	my $id = '';

	if( $o_feature->isa( 'Bio::EnsEMBL::Slice' ) )
	{
		$id  = $o_feature->seq_region_name();
		$id .= '_subseq';
	}
	else
	{
		# exception ?
		warn "o_feature is a " . ref( $o_feature );
	}

	return $id;
}


=head2 Procedure

 Title        : __FetchFeaturesFromSlice
 Usage        : $o_xx->__FetchFeaturesFromSlice( $o_slice );
 Prerequisite : none
 Procedure    : Call _StoreFeature.
 Args         : $o_slice, EnsEMBL Slice

=cut

sub _FetchFeaturesFromSlice
{
	my $self = shift;
	my ( $o_slice ) = @_;

	$self->_StoreFeature( $o_slice );

	return;
}

1;
