#
# 	sebastien.letort@toulouse.inra.fr
#	Created: January 20, 2009
#	$Id: DownstreamsDecorator.pm 3689 2011-05-26 07:01:25Z manu $
#
# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


package WhatDecorator::SlicesDecorator::DownstreamsDecorator;

=pod

=head1 NAME

 WhatDecorator::SlicesDecorator::DownstreamsDecorator - a WhatDecorator to fetch sequences through API

=head1  SYNOPSIS

 # code

=head1 DESCRIPTION

 NarcisseService::Accessor needs a FromDecorator and a WhatDecorator to be
 usable. This is a WhatDecorator, specific to access sequences (Slice) through API.
 What is found by _Fetch is stored with _StoreResults.

=cut

use strict;
use warnings;

use Data::Dumper;

# chemin d'access a WhatDecorator depuis lib/
# 'use' obligatoire pour avoir plusieurs paquets dans le meme fichier.
# si on ne le met pas ici,il faut le mettre dans l'executable
# je trouve plus propre de n'inclure dans l'executable que les paquets
# que l'on instancie.
use NarcisseService::Accessor::WhatDecorator::SlicesDecorator;
use base qw( WhatDecorator::SlicesDecorator );

use NarcisseServiceKeywords qw( &NS_INFOS &NS_FASTA &NS_LOCUS );

use Bio::EnsEMBL::Utils::Sequence  qw(reverse_comp);


=head2 function _GetFeatureId

 Usage        : $id = $o_xx->_GetFeatureId( $o_feature );
 Prerequisite : none
 Function     : return an Id that describes the feature.
                I use a function to allow overloading by SliceDecorator.
                MANU 24 MAI 2011 - RETURNS $locus_tag_upstream
 Returns      : $id, string, id of $o_feature
 Args         : $o_feature, a Bio::EnsEMBL::x object
 			    $locus_tag The locus tag of the gene


=cut

sub _GetFeatureId
{
	my $self = shift;
	my( $o_feature, $locus_tag ) = @_;

	my $id = '';

	if( $o_feature->isa( 'Bio::EnsEMBL::Slice' ) )
	{
		if ( defined $locus_tag )
		{
			$id = $locus_tag . '_downstream';
		}
		else
		{
			$id  = $o_feature->seq_region_name();
			$id .= '_downstream';
		}
	}
	else
	{
		# exception ?
		warn "o_feature is a " . ref( $o_feature );
	}
	
	return $id;
}


=head2 procedure

 Title        : _Fetch
 Usage        : $o_xx->_Fetch( links => $rao_links );
 Prerequisite : should populate _ra_results attribut.
 Procedure    : Foreach link, fetch Genes through a method that is dependant of
                the object link.
 Args         : $rao_links, list ref of link objects.

=cut

sub _Fetch
{
	my $self = shift;
	my %h_args = ( links => undef,
	               @_ );

	my $rao_links = $h_args{ links };
	foreach my $o_link ( @$rao_links )
	{
		if( $o_link->isa( 'Bio::EnsEMBL::Gene' ) )
		{
			$self->__FetchSlicesFromGene( $o_link );
		}
		else
		{
			my $msg = "Your query can't be achieved due to a lack of programmation\n";
			my $dbg = "Tried to fetch genes with -" . ref( $o_link );
			$dbg   .= "- no function defined for that\n";
			throw LipmError( -text => $dbg, -user_msg => $msg );
		}
	}

	return;
}


=head2 Procedure

 Title        : __FetchSlicesFromGene
 Usage        : $o_xx->__FetchSlicesFromGene( $o_gene );
 Prerequisite : none
 Procedure    : none
 Args         : $o_gene

=cut

sub __FetchSlicesFromGene
{
	my $self = shift;
	my ( $o_gene ) = @_;

	# $o_gene->slice() renvoie une tranche sur tout le chrom
	my $o_slice = $o_gene->slice();

	my $length = $self->_Get( qw( _length ) );
	my $seq;
	my $seq2;
	my $o_sub_slice;

	# je fais un appel a seq() uniquement pour mettre la sequence en cache

	if( -1 == $o_gene->strand )
	{
		$o_sub_slice = $o_slice->sub_Slice( $o_gene->start - $length,
											$o_gene->start - 1,
											$o_gene->strand );
		$seq2 = $o_sub_slice->seq();
		reverse_comp( \$seq2 );
		$o_sub_slice->{ seq } = $seq2;
	}
	elsif( 1 == $o_gene->strand )
	{
		$o_sub_slice = $o_slice->sub_Slice( $o_gene->end + 1,
											$o_gene->end + $length,
											$o_gene->strand );
		$o_sub_slice->{ seq } = $o_sub_slice->seq();
	}

	$self->_StoreFeature( $o_sub_slice );

	return;
}


1;
