#
# 	sebastien.letort@toulouse.inra.fr
#	Created: 25 03, 2009
#	$Id: GetAnnotationFromLocusTags.pm 3142 2009-12-22 12:38:52Z manu $
#
# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


package GetAnnotationFromLocusTags;

=pod

=head1 NAME

 GetLocusTagsFromKeywords - 

=head1  SYNOPSIS

 # code

=head1 DESCRIPTION

 #descr

=cut

use strict;
use warnings;

use Data::Dumper;
use File::Temp;

use base qw( NarcisseService::GrepBased );

=head2 Procedure

 Title        : _TreatOptions
 Usage        : $o_xx->_TreatOptions( -species  => $species
                                      -db => $db,
                                      ra_tags => $ra_tags);
                OR
                $o_xx->_TreatOptions( -species  => $species
                                      -db => $db,
                                      tags_file => $file_name);
 Prerequisite : called by NarcisseService::_Init
 Procedure    : Copy the list of tags to a temporary file, or just keep the tags file name
 Args         : $species:  string, short species name
                tags_file: string, a file containing a list of tags (1 tag / line)
                ra_tags:   array, a list of tags. Used ONLY if tags_file is not defined

=cut

sub _TreatOptions
{
	my $self   = shift;
	my %h_args = ( ra_tags   => [],
                   tags_file => undef,
	               @_ );

	$self->SUPER::_TreatOptions( %h_args );

	my $ra_tags   = $h_args{ ra_tags };
    my $tags_file = $h_args{ tags_file };
    if ( !defined $tags_file )
    {
        if( 0 == @$ra_tags )
        {
            my $msg = "You must specify at least one locus tags.";
            throw LipmError::ObjectException( $self, '!build', -text => $msg );
        }
        
        # copy the array to a temporary file, it will be used by the grep command
        # (see the _Run function)
        my $o_fh_tmp = new File::Temp;
        print $o_fh_tmp join ("\n",@$ra_tags);
        $o_fh_tmp->close();
        
        # Keep a ref to the object. The file will be automatically removed when this object will be destroyed
        $self->_Set( __o_fh_tmp => $o_fh_tmp );

        #Initialize $tags_file
        $tags_file = $o_fh_tmp->filename();
    }
    
    $self->_Set( '__tags_file' => $tags_file );
    
	return;
}


=head2 Procedure _Run

 Usage        : $o_xx->_Run();
 Prerequisite : Overide to make something
 Procedure    : This method should do the job.
 Args         : none

=cut

sub _Run
{
	my $self = shift;

    my $o_io = $self->_Get( '_o_instance_obs' );

	my $fasta_file = $self->_GetDbFile();
    my $tags_file  = $self->_Get('__tags_file');
    
	my $msg = "Will grep through $fasta_file\n";
	$self->_Trace( $msg, '.', 'IS' );

    my $grep = $o_io->GetUnixPath('UNIX_GREP') . " -F";
    my $cmd = "$grep '>' $fasta_file |$grep -f $tags_file";
    
    $self->_Trace( "Now executing $cmd", '.', 'ISV' );
    my @a_output = `$cmd`;
    foreach my $line (@a_output)
    {
        chomp( $line );
        my ($locus_tag) = $line =~ /locus_tag=(\S+)/i;
        my $tmp = $line;
        $tmp =~ /\schr=(\S+)/;	
        my $chr = $1;
        
        $tmp =~ /\ssp=(\S+)/; 	
        my $species = $o_io->GetSpeciesLongName($1);
        
        $tmp =~ s/>\S+|\S+=\S+//g;
        $tmp =~ s/^\s+//;
        
        $self->_StoreResults( "$locus_tag\t$species\t$chr\t$tmp" );
    }    
	return;
}




1;

__END__

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
