#
# 	sebastien.letort@toulouse.inra.fr
#	Created: 25 03, 2009
#	$Id: GetCDSsFromLocusTags.pm 3142 2009-12-22 12:38:52Z manu $
#
# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


package GetCDSsFromLocusTags;

=pod

=head1 NAME


=head1  SYNOPSIS

 # code

=head1 DESCRIPTION

 #descr

=cut

use strict;
use warnings;

use Data::Dumper;

use base qw( NarcisseService::GrepBased );

=head2 Procedure

 Title        : _TreatOptions
 Usage        : $o_xx->_TreatOptions( -species  => $species
                                      -db => $db,
                                      ra_tags => $ra_tags );
 Prerequisite : called by NarcisseService::_Init
 Procedure    : set $species as attribute _species
 Args         : $species, string, short species name

=cut

sub _TreatOptions
{
	my $self   = shift;
	my %h_args = ( ra_tags => [],
	               -db => 'cds', # ou 'nuc'
	               # -species => undef,
	               @_ );

	$self->SUPER::_TreatOptions( %h_args );

	my $ra_tags = $h_args{ ra_tags };
	if( 0 == @$ra_tags )
	{
		my $msg = "You must specified at least one locus tags.";
		throw LipmError::ObjectException( $self, '!build', -text => $msg );
	}

	$self->_Set( __ra_tags => $ra_tags );

	return;
}


=head2 Procedure _Run

 Usage        : $o_xx->_Run();
 Prerequisite : Overide to make something
 Procedure    : This method should do the job.
 Args         : none

=cut

sub _Run
{
	my $self = shift;

	my( $ra_tags ) = $self->_Get( qw( __ra_tags ) );
	my $fasta_file = $self->_GetDbFile();

	my $regexp_tags = join( '|', @$ra_tags );

	# je ne comprends pas mon commentaire lie au majuscule
	# il faut des majuscule pour comparer des majuscules
	$regexp_tags    = uc( $regexp_tags );

	$self->_Trace( "Parsing $fasta_file with $regexp_tags\n", '.', 'ISV' );

	my $fe = new IO::File( $fasta_file, 'r' )
					or throw LipmError::IOException( $fasta_file, 'f' );
	while( my $line = <$fe> )
	{
		if( $line =~ /^>/ )
		{
			if( $line =~ /locus_tag=(\S+)/i )
			{
				my $locus_lu = uc( $1 );	#mise en majuscule car on ne passe pas par l'API
				next	unless( $locus_lu =~ $regexp_tags );

				chomp( $line );
				my $head = $line;

				my $seq_fasta = '';
				my $pos = $fe->getpos();
				while( ( defined( $fe ) )
						&& ( $line = <$fe> )
						&& ( $line !~ /^>/ ) )
				{
					$pos=$fe->getpos();
					chomp( $line );
					$seq_fasta .= $line;
				}

				$self->_StoreResults( { $head => $seq_fasta } );
				$fe->setpos( $pos ); # on revient sur la ligne d'avant pour la relecture
			} #ids
		} # fasta header
	} # end <file>

	return;
}

1;

__END__

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
