#
# 	sebastien.letort@toulouse.inra.fr
#	Created: 05 March 2009
#	$Id: UsingExternalProgram.pm 3704 2011-07-27 13:39:27Z manu $
#
# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

package NarcisseService::UsingExternalProgram;

=pod

=head1 NAME

 NarcisseService::UsingExternalProgram - 

=head1  SYNOPSIS

 # code

=head1 DESCRIPTION

 #descr

=cut

use strict;
use warnings;

use Data::Dumper;

use base qw( NarcisseService );

#use General;

=head2 method _Init

 Usage        : my $o_x = New NarcisseService::UsingExternalProgram(
                                               -o_logger => $o_logger,
                                               -o_instance_obs => $o_inst_obs );
 Prerequisite : none
 Fonction     : initialise an instance of NarcisseService::UsingExternalProgram class.
 Returns      : a valid NarcisseService::UsingExternalProgram object
 Args         :
   both optionals
      $o_logger => a logger object, if none, no log.
      $o_inst_obs => an InstanceObserver object, if none, an new one
                              is build with default args ($ENV{ NARCISSE_CFG })
 Attributes   :
 	# inherited from NarcisseService
     _o_instance_obs : InstanceObserver object [undef]

 private Attributes :
	__more_params  => string additionnal parameters of the prog.

 NOTE - The external program is ran when FetchResults() is called.
        The result file name is stored inside $rh_results->{data} (see NarcisseService::FetchReult for the details)

=cut

=head2 Procedure

 Title        : _TreatOptions
 Usage        : $o_xx->_TreatOptions( -more_params => $more_params );
 Prerequisite : none.
 Procedure    : store the $more_params string in __more_params attributes
 Args         : cf _Init, _Init will call _TreatOptions with all args.

 Error        : LipmError::ObjectException

=cut

sub _TreatOptions
{
	my $self = shift;
	my %h_args = ( -more_params => '',
				   @_ );
	$self->SUPER::_TreatOptions( %h_args );

	$self->_Set( __more_params => $h_args{ -more_params } );
}


=head2 Function _GetMoreParams

 Usage        : my $params = $o_x->_GetMoreParams()
 Function     : accessor on __more_params attribute.
 Returns      : $params, a string
 Args         : none
 Error        : none

=cut

sub _GetMoreParams
{
	my $self = shift;

	my $params = $self->_Get( '__more_params' );

	return $params;
}


=head2 Procedure RunCommand

 Usage        : $o_x->RunCommand( $cmd )
 Function     : Trace and execute the command.
 Args         : $cmd, the command to execute
 Error        : LipmError.

=cut

sub RunCommand
{
	my $self   = shift;
	my( $cmd ) = @_;

	my $msg = "will execute the following command\n$cmd\n";
	$self->_Trace( $msg, '.', 'IS' );

	# execution de la commande
	my $error = `($cmd) 2>&1 1>/dev/null`;

	# pas sure que cette façon de lever une exception en cas de pb soit la bonne
	#if ($error =~ /error/i or $error =~ /WARNING:/ )
    if ($error =~ /error/i)
	{
		my $usr_msg = "a problem occurred while executing an external program.\n";
		throw LipmError( -text => $error, -user_msg => $usr_msg.$error );
	}
	# ca vient de blast qui ecrit des alertes sur stderr.
	# des que qqc est ecrit sur STDERR, biomoby leve une exception et arrete tout.

	return;
}



## fonctions communes a PSIBLAST et BLAST

sub _CheckSpeciesAndGetDB
{
	my $self = shift;
	my( $species, $db_type ) = @_;

	my $o_inst_obs = $self->_Get( qw( _o_instance_obs ) );
	if( defined( $species ) && ( $species ne 'none' ) && !$o_inst_obs->HaveSpecies( $species ) )
	{
		my $name = $o_inst_obs->GetName();
		my $msg = "$species does not belong to instance $name.";
		$msg   .= " We don't go further.";

		# 002 pour permettre de passer a une autre instance
		throw LipmError( -text => $msg, -usr_msg => $msg, -value => 002 );
	}

	my $db = $o_inst_obs->GetDB( $db_type, $species );

	# DB est un fichier .pal
# 	$db =~ s/\.fasta$/\.pal/  if( $db_name eq 'MULTI' );

	return $db;
}


=head2 Function GetOutputFilename

 Usage        : my $filename = $o_x->GetOutputFilename()
 Function     : return _outfile attribute or
                build an empty tempfile and return its name.
 Returns      : $filename
 Args         : none
 Error        : none

=cut

sub GetOutputFilename
{
	my $self = shift;

	my $filename = $self->_Get( qw( _outfile ) );

	if( !defined $filename )
	{
		use File::Temp;
		my $fh_tmp = new File::Temp( TEMPLATE => 'ncsBlast_XXXX',
									SUFFIX   => '.tmp',
									DIR      => '/tmp',
									UNLINK   => 0 );
		$fh_tmp->close();

		$filename  = $fh_tmp->filename();
	}

	return $filename;
} #GetOutputFilename


1;

package NarcisseService::Blast;

use base qw( NarcisseService::UsingExternalProgram );

use General;
use LipmError::ParameterException;
use Enum;

use constant O_PROGS_PROT => New Enum( qw( blastp blastx ) );
use constant O_PROGS_NUC  => New Enum( qw( blastn tblastn tblastx ) );



=head2 _Init

 Usage    : $o_x->_Init( seq            => $seq_filename,
                        -prog           => $prog,
                        -expect         => $expected_value,
                        -o_logger       => $o_logger,
                        -more_params    => $params,
                        -outfile        => $out_file,
                        -o_instance_obs => $o_inst_obs )
 Function : Build a Blast object.
 Args     :
     $seq_filename, string, the name of the fasta file to blast against NarcisseDB
     $prog, string, the program to use [blastn]
     $expected_value, the expected value given to the blast program
     $o_logger => a logger object, if none, no log.
     $o_inst_obs => an InstanceObserver object, if none will be created
     $params => string, additionnal params
     $out_file => string, the output file name. If none, a temporary file will be created
 
 NOTE - Blast is ran when FetchResults() is called.
        The result file name is stored inside $rh_results->{data}
     
 private Attributes :
	__seq_filename => $seq,
	__prog    => $prog,
	__expect  => $expect,
	__db      => $db)

=cut


=head2 Procedure _TreatOptions

 Title        : _TreatOptions
 Usage        : $o_xx->_TreatOptions( %h_args );
 Prerequisite : Overide to make something
 Procedure    : This method will be call to treat specific constructor's
                arguments. That's why it have to be redefined if needed.
 Args         : cf _Init, _Init will call _TreatOptions with all args.

 Error        : LipmError::ObjectException

=cut

sub _TreatOptions
{
	my $self = shift;
	my %h_args = ( seq => undef,
# 	               -o_instance_obs => undef,
	               -prog     => 'blastn',
	               -expect   => 10,
	               -species  => undef,
	               -outfile  => undef,
					@_ );

	# pour -more_params
	$self->SUPER::_TreatOptions( %h_args );

	# mandatory args
	my @a_keys = qw( seq );
	my( $seq ) = @h_args{ @a_keys };
	LipmError::ParameterException::Assert( seq => $seq );

	# optional args
	@a_keys = qw( -prog -expect -species -outfile );
	my( $prog, $expect, $species, $outfile ) = @h_args{ @a_keys };

	# db_type prot or cds ?
	my $db_type;
	if( &O_PROGS_PROT->Includes( $prog ) )
	{
		$db_type = 'prot';
	}
	elsif( &O_PROGS_NUC->Includes( $prog ) )
	{
		$db_type = 'cds';
	}
	else
	{
		my @a_progs = &O_PROGS_PROT->ListItems();
		push( @a_progs, &O_PROGS_NUC->ListItems() );

		my $msg = "$prog does not belong to the list '@a_progs',";
		throw LipmError::ObjectException( $self, '!build', -text => $msg );
	}

	# check if species belong to the instance and set DB
	my $db = $self->_CheckSpeciesAndGetDB( $species, $db_type );

	$self->_Set( __seq_filename => $seq,
	             _outfile  => $outfile,
	             __prog    => $prog,
	             __expect  => $expect,
	             __db      => $db,
	             # besoin de species ?
	             __db_type => $db_type ); # c'est utile db_type ?

	return;
}


=head2 Procedure _Run

 Usage        : $o_xx->_Run();
 Prerequisite : none
 Procedure    : This method do the job.
 Args         : none

=cut

sub _Run
{
	my $self = shift;

	my( $o_inst_obs, $blast_db ) = $self->_Get( qw( _o_instance_obs __db ) );

	# get blast param from conf file
	my ( $blast_exe, $blast_mat )
					= $o_inst_obs->GetCfg( qw( ROOT_BLASTALL ROOT_BLASTMAT ) );

	my @a_attr = qw( __expect __prog __seq_filename );
	my ( $expect, $prog, $seq ) = $self->_Get( @a_attr );

	#controls
	my $msg = undef;
	$blast_exe =~ s/\s+$//;	# parce qu'il y a un espace apres le nom du prog
	if( !-e $blast_exe )
	{	$msg = "'$blast_exe' is not executable."	}
	elsif( !-d $blast_mat )
	{	$msg = "'$blast_mat' doesn't exist.";	}
	elsif( !-f $blast_db )
	{	$msg = "'$blast_db' doesn't exist.";	}

	if( $msg )	# IOException ?
	{	throw LipmError( -text => $msg );	}

	my $filename = $self->GetOutputFilename();

	$blast_db =~ s/\.pal$//; # si .pal, on l'enleve

	# additionnal params
	my $params = $self->_GetMoreParams();

	# on est bon, on ecrit la commande
	my $cde = "export BLASTMAT=$blast_mat;";
	$cde .= "$blast_exe $params -p $prog -d $blast_db -i $seq";
	$cde .= " -e $expect"	if( IsNotNull( $expect ) );
	$cde .= " -o $filename";
	$self->RunCommand( $cde );

	$self->_StoreResults( $filename );

	return;
}

1;

# -------------------------------------
package NarcisseService::PsiBlast;

use base qw( NarcisseService::UsingExternalProgram );

use General;
use LipmError::ParameterException;

=head2 _Init

 Usage    : $o_x->_Init( seq            => $seq_filename,
                        -expect         => $expected_value,
                        -o_logger       => $o_logger,
                        -more_params    => $params,
                        -outfile        => $out_file,
                        -o_instance_obs => $o_inst_obs )
 Function : Build a PsiBlast object.
 Args     :
     $seq_filename, string, the name of the fasta file to blast against NarcisseDB
     $expected_value, the expected value given to the blast program
     $o_logger => a logger object, if none, no log.
     $o_inst_obs => an InstanceObserver object, if none will be created
     $params => string, additionnal params
     $out_file => string, the output file name. If none, a temporary file will be created
 
 NOTE - Blast is ran when FetchResults() is called.
        The result file name is stored inside $rh_results->{data}
     
 private Attributes :
	__seq_filename => $seq,
	__expect  => $expect,
	__db      => $db)

=cut


=head2 Procedure _TreatOptions

 Title        : _TreatOptions
 Usage        : $o_xx->_TreatOptions( %h_args );
 Prerequisite : Overide to make something
 Procedure    : This method will be call to treat specific constructor's
                arguments. That's why it have to be redefined if needed.
 Args         : cf _Init, _Init will call _TreatOptions with all args.

 Error        : LipmError::ObjectException

=cut

sub _TreatOptions
{
	my $self = shift;
	my %h_args = ( input => undef,
# 	               -o_instance_obs => undef,
	               -outfile  => undef,
	               -expect => 1e-6,
	               -species  => undef,
	               @_ );

	#more params
	$self->SUPER::_TreatOptions( %h_args );

	# mandatory args
	my @a_keys = qw( input );
	my( $input ) = @h_args{ @a_keys };
	LipmError::ParameterException::Assert( input => $input );

	# optional args
	@a_keys = qw( -expect -species -outfile );
	my( $expect, $species, $outfile ) = @h_args{ @a_keys };

	# check if species belong to the instance and set DB
	my $db = $self->_CheckSpeciesAndGetDB( $species, 'prot' );

	my $o_inst_obs = $self->_Get( qw( _o_instance_obs ) );
	my( $prog ) = $o_inst_obs->GetCfg( 'ROOT_BLASTPGP' );

	$self->_Set( __input_filename => $input,
	             _outfile  => $outfile,
	             __expect  => $expect,
	             __db      => $db,
	             __prog    => $prog
	             # besoin de species ? <- non, tout est dans __db
	           );
	return;
}


=head2 Procedure _Run

 Usage        : $o_xx->_Run();
 Prerequisite : none
 Procedure    : This method do the job.
 Args         : none

=cut

sub _Run
{
	my $self = shift;

	my( $o_inst_obs, $blast_db ) = $self->_Get( qw( _o_instance_obs __db ) );

	# get blast param from conf file
	my ( $blast_mat )
					= $o_inst_obs->GetCfg( qw( ROOT_BLASTMAT ) );

	my @a_attr = qw( __expect __input_filename __prog );
	my ( $expect, $input, $blast_exe ) = $self->_Get( @a_attr );

	my $params = $self->_GetMoreParams();

	#controls
	my $msg = undef;
	if( !-e $blast_exe )
	{	$msg = "'$blast_exe' is not executable."	}
	elsif( !-d $blast_mat )
	{	$msg = "'$blast_mat' doesn't exist.";	}
	elsif( !-f $blast_db )
	{	$msg = "'$blast_db' doesn't exist.";	}

	if( $msg )	# IOException ?
	{	throw LipmError( -text => $msg );	}

	my $filename = $self->GetOutputFilename();

	$blast_db =~ s/\.pal$//; # si .pal, on l'enleve

	# on est bon, on ecrit la commande
	my $cde = "export BLASTMAT=$blast_mat;";
	$cde .= "$blast_exe $params -d $blast_db -i $input";
	$cde .= " -e $expect"	if( IsNotNull( $expect ) );
	$cde .= ' -m 0';
	$cde .= " -o $filename";

	$self->RunCommand( $cde );

	$self->_StoreResults( $filename );

	$msg = "End of PsiBlast treatment.\n";
	$self->_Trace( $msg, '.', 'IS' );

	return;
}

1;


__END__

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
