#
# 	sebastien.letort@toulouse.inra.fr
#	Created: December 16, 2008
#	$Id: NarcisseServiceFactory.pm 3479 2010-08-26 15:00:58Z manu $
#
# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
package NarcisseServiceFactory;

=pod

=head1 NAME

 NarcisseServiceFactory - a factory to simplifie the use of NarcisseService.

=head1  SYNOPSIS

 my $o_ns = &GetXFromY();
 my ($ra_headers,$rh_data) = $o_ns->FetchResults();

=head1 DESCRIPTION

 This class implements the Factory design pattern to hide the complexity of
 creation of NarcisseService. Users can call one service in one line.
 
 The out_type input parameter in one of &NS_LOCUS etc., see NarcisseServiceKeywords for the details
 The return value of all Getxxx functions is a NarcisseService object
 
 It contains statics methods to hide the use of decoration for
 NarcisseService::Accessor

=cut

use strict;
use warnings;

use Data::Dumper;

use base qw( LipmObject );
use General;

use NarcisseService::Accessor;

use NarcisseService::Accessor::WhatDecorator::GenesDecorator;
use NarcisseService::Accessor::WhatDecorator::ProteinsDecorator;
use NarcisseService::Accessor::WhatDecorator::CDS_TransDecorator;
use NarcisseService::Accessor::WhatDecorator::SlicesDecorator;
use NarcisseService::Accessor::WhatDecorator::SlicesDecorator::DownstreamsDecorator;
use NarcisseService::Accessor::WhatDecorator::SlicesDecorator::UpstreamsDecorator;

use NarcisseService::Accessor::FromDecorator::RegionsDecorator;
use NarcisseService::Accessor::FromDecorator::IdsDecorator;

# PROVISOIRE (cf PATCH API)
###use NarcisseService::GrepBased::GetCDSsFromLocusTags;
###use NarcisseService::GrepBased::GetProteinsFromLocusTags;

=head2 function GetGenesFromRegions

 Title        : GetGenesFromRegions
 Usage        : $o_ns = &GetGenesFromRegions( species    => $species,
                                              chrom      => $chrom,
                                              ra_regions => $ra_regions,
                                              -out_type  => $out_type,
                                              -o_instance_obs => $o_inst_obs,
                                              -o_logger  => $o_logger
                                            );
 Function     : Build a NarcisseService::Accessor, then decor it with
                a FromDecorator::RegionsDecorator and
                a WhatDecorator::GenesDecorator
 Returns      : $o_ns, a NarcisseService object
 Args         : $species, string, short species name
                $chrom, string, chromosome label
                $ra_regions, array ref of hashes ref:
                [{start=>x,end=>y}, ...]
                $o_inst_obs, the InstanceObserver onto which you want to work
                $o_logger, Logger object
                $out_type, string &NS_FASTA, &NS_LOCUS, &NS_INFOS

=cut

sub GetGenesFromRegions
{
	# pour infos
	my %h_args = ( species    => undef,
	               chrom      => undef,
	               ra_regions => undef,
# 	               -out_type  => 
	               -o_instance_obs => undef,
	               -o_logger => undef,
	               @_ );

	my $o_ns     = New NarcisseService::Accessor( @_ );

	my %h_params = ( o_accessor => $o_ns, @_ );
	$o_ns        = New FromDecorator::RegionsDecorator( %h_params );

	%h_params    = ( o_accessor => $o_ns, @_ );
	$o_ns        = New WhatDecorator::GenesDecorator( %h_params );

# 	my %h_params = ( species => $h_args{ species } );
# 	my $o_ns     = New NarcisseService::Accessor( %h_params );
#
# 	%h_params    = ( o_accessor => $o_ns,
# 	                 chrom      => $h_args{ chrom },
# 	                 ra_regions => $h_args{ ra_regions } );
# 	$o_ns        = New FromDecorator::RegionsDecorator( %h_params );
#
# 	%h_params    = ( o_accessor => $o_ns );
# 	$o_ns        = New WhatDecorator::GenesDecorator( %h_params );

	return $o_ns;
}


=head2 function GetCDSsFromRegions

 Title        : GetCDSsFromRegions
 Usage        : $o_ns = &GetCDSsFromRegions( species    => $species,
                                             chrom      => $chrom,
                                             ra_regions => $ra_regions,
                                             -o_instance_obs => $o_inst_obs,
                                              -out_type  => $out_type,
                                             -o_logger  => $o_logger
                                            );
 Function     : Build a NarcisseService::Accessor, then decor it with
                a FromDecorator::RegionsDecorator and
                a WhatDecorator::CDSsDecorator
 Returns      : $o_ns, a NarcisseService object
 Args         : $species, string, short species name
                $chrom, string, chromosome label
                $ra_regions, array ref of hashes ref like
                    [{start=>x,end=>y}, {start=>a,end=>b}, ...]
                $o_inst_obs, the InstanceObserver onto which you want to work
                $o_logger, Logger object
                $out_type, string &NS_FASTA, &NS_LOCUS, &NS_INFOS

=cut

sub GetCDSsFromRegions
{
	# pour infos
	my %h_args = ( species    => undef,
	               chrom      => undef,
	               ra_regions => undef,
	               -o_instance_obs => undef,
	               -o_logger => undef,
# 	               -out_type  => undef,
	               @_ );

	my $o_ns     = New NarcisseService::Accessor( @_ );

	my %h_params = ( o_accessor => $o_ns, @_ );
	$o_ns        = New FromDecorator::RegionsDecorator( %h_params );

	%h_params    = ( o_accessor => $o_ns, @_ );
	$o_ns        = New WhatDecorator::CDSsDecorator( %h_params );

# 	my %h_params = ( species => $h_args{ species } );
# 	my $o_ns     = New NarcisseService::Accessor( %h_params );
#
# 	%h_params    = ( o_accessor => $o_ns,
# 	                 chrom      => $h_args{ chrom },
# 	                 ra_regions => $h_args{ ra_regions } );
# 	$o_ns        = New FromDecorator::RegionsDecorator( %h_params );
#
# 	%h_params    = ( o_accessor => $o_ns );
# 	$o_ns        = New WhatDecorator::CDSsDecorator( %h_params );

	return $o_ns;
}


=head2 function GetTranscriptsFromRegions

 Title        : GetTranscriptsFromRegions
 Usage        : $o_ns = &GetTranscriptsFromRegions( species    => $species,
                                             chrom      => $chrom,
                                             ra_regions => $ra_regions,
                                             -o_instance_obs => $o_inst_obs,
                                              -out_type  => $out_type,
                                             -o_logger  => $o_logger
                                            );
 Function     : Build a NarcisseService::Accessor, then decor it with
                a FromDecorator::RegionsDecorator and
                a WhatDecorator::TranscriptsDecorator
 Returns      : $o_ns, a NarcisseService object
 Args         : $species, string, short species name
                $chrom, string, chromosome label
                $ra_regions, array ref of hashes ref like
                    [{start=>x,end=>y}, {start=>a,end=>b}, ...]
                $o_inst_obs, the InstanceObserver onto which you want to work
                $o_logger, Logger object
                $out_type, string &NS_FASTA, &NS_LOCUS, &NS_INFOS

=cut

sub GetTranscriptsFromRegions
{
	# pour infos
	my %h_args = ( species    => undef,
	               chrom      => undef,
	               ra_regions => undef,
	               -o_instance_obs => undef,
	               -o_logger => undef,
# 	               -out_type  => undef,
	               @_ );

	my $o_ns     = New NarcisseService::Accessor( @_ );

	my %h_params = ( o_accessor => $o_ns, @_ );
	$o_ns        = New FromDecorator::RegionsDecorator( %h_params );

	%h_params    = ( o_accessor => $o_ns, @_ );
	$o_ns        = New WhatDecorator::TranscriptsDecorator( %h_params );

# 	my %h_params = ( species => $h_args{ species } );
# 	my $o_ns     = New NarcisseService::Accessor( %h_params );
#
# 	%h_params    = ( o_accessor => $o_ns,
# 	                 chrom      => $h_args{ chrom },
# 	                 ra_regions => $h_args{ ra_regions } );
# 	$o_ns        = New FromDecorator::RegionsDecorator( %h_params );
#
# 	%h_params    = ( o_accessor => $o_ns );
# 	$o_ns        = New WhatDecorator::TranscriptsDecorator( %h_params );

	return $o_ns;
}


=head2 function GetProteinsFromRegions

 Title        : GetProteinsFromRegions
 Usage        : $o_ns = &GetProteinsFromRegions( species    => $species,
                                              chrom      => $chrom,
                                              -out_type  => $out_type,
                                              ra_regions => $ra_regions,
                                              -o_instance_obs => $o_inst_obs,
                                              -o_logger  => $o_logger
                                            );
 Function     : Build a NarcisseService::Accessor, then decor it with
                a FromDecorator::RegionsDecorator and
                a WhatDecorator::ProteinsDecorator
 Returns      : $o_ns, a NarcisseService object
 Args         : $species, string, short species name
                $chrom, string, chromosome label
                $ra_regions, array ref of hashes ref like
                    [{start=>x,end=>y}, {start=>a,end=>b}, ...]
                $o_inst_obs, the InstanceObserver onto which you want to work
                $o_logger, Logger object
                $out_type, string &NS_FASTA, &NS_LOCUS, &NS_INFOS


 Note : Beaucoup trop lente ! utiliser la version sans API

=cut

sub GetProteinsFromRegions
{
	# pour infos
	my %h_args = ( species    => undef,
	               chrom      => undef,
	               ra_regions => undef,
	               -o_instance_obs => undef,
	               -o_logger => undef,
# 	               -out_type  => undef,
	               @_ );

	my $o_ns     = New NarcisseService::Accessor( @_ );

	my %h_params = ( o_accessor => $o_ns, @_ );
	$o_ns        = New FromDecorator::RegionsDecorator( %h_params );

	%h_params    = ( o_accessor => $o_ns, @_ );
	$o_ns        = New WhatDecorator::ProteinsDecorator( %h_params );

# 	my %h_params = ( species => $h_args{ species } );
# 	my $o_ns     = New NarcisseService::Accessor( %h_params );
#
# 	%h_params    = ( o_accessor => $o_ns,
# 	                 chrom      => $h_args{ chrom },
# 	                 ra_regions => $h_args{ ra_regions } );
# 	$o_ns        = New FromDecorator::RegionsDecorator( %h_params );
#
# 	%h_params    = ( o_accessor => $o_ns );
# 	$o_ns        = New WhatDecorator::ProteinsDecorator( %h_params );

	return $o_ns;
}


=head2 function GetSlicesFromRegions

 Title        : GetSlicesFromRegions
 Usage        : $o_ns = &GetSlicesFromRegions( species    => $species,
                                               chrom      => $chrom,
                                               ra_regions => $ra_regions,
                                              -out_type  => $out_type,
                                               -o_instance_obs => $o_inst_obs,
                                               -o_logger  => $o_logger
                                             );
 Function     : Build a NarcisseService::Accessor, then decor it with
                a FromDecorator::RegionsDecorator and
                a WhatDecorator::SlicesDecorator
 Returns      : $o_ns, a NarcisseService object
 Args         : $species, string, short species name
                $chrom, string, chromosome label
                $ra_regions, array ref of hashes ref like
                    [{start=>x,end=>y}, {start=>a,end=>b}, ...]
                $o_inst_obs, the InstanceObserver onto which you want to work
                $o_logger, Logger object
                $out_type, string &NS_FASTA, &NS_LOCUS, &NS_INFOS

=cut

sub GetSlicesFromRegions
{
	# pour infos
	my %h_args = ( species    => undef,
	               chrom      => undef,
	               ra_regions => undef,
	               -o_instance_obs => undef,
	               -o_logger => undef,
# 	               -out_type  => undef,
	               @_ );

	my $o_ns     = New NarcisseService::Accessor( @_ );

	my %h_params = ( o_accessor => $o_ns, @_ );
	$o_ns        = New FromDecorator::RegionsDecorator( %h_params );

	%h_params    = ( o_accessor => $o_ns, @_ );
	$o_ns        = New WhatDecorator::SlicesDecorator( %h_params );

# 	my %h_params = ( species => $h_args{ species } );
# 	my $o_ns     = New NarcisseService::Accessor( %h_params );
#
# 	%h_params    = ( o_accessor => $o_ns,
# 	                 chrom      => $h_args{ chrom },
# 	                 ra_regions => $h_args{ ra_regions } );
# 	$o_ns        = New FromDecorator::RegionsDecorator( %h_params );
#
# 	%h_params    = ( o_accessor => $o_ns );
# 	$o_ns        = New WhatDecorator::SlicesDecorator( %h_params );

	return $o_ns;
}

#------------------------------------------------------

=head2 function GetGenesFromLocusTags

 Title        : GetGenesFromLocusTags
 Usage        : $o_ns = &GetGenesFromLocusTags( species    => $species,
                                                ra_ids     => $ra_ids,
                                                -o_instance_obs => $o_inst_obs,
                                                -out_type  => $out_type,
                                                -o_logger  => $o_logger
                                            );
 Function     : Build a NarcisseService::Accessor, then decor it with
                a FromDecorator::RegionsDecorator and
                a WhatDecorator::GenesDecorator
 Returns      : $o_ns, a NarcisseService object
 Args         : $species, string, short species name
                $ra_ids, array ref of locus tags
                $o_inst_obs, the InstanceObserver onto which you want to work
                $o_logger, Logger object
                $out_type, string &NS_FASTA, &NS_LOCUS, &NS_INFOS

=cut

sub GetGenesFromLocusTags
{
	# pour infos
	my %h_args = ( species   => undef,
	               ra_ids    => undef,
	               -o_instance_obs => undef,
	               -o_logger => undef,
# 	               -out_type  => undef,
	               @_ );

	my $o_ns     = New NarcisseService::Accessor( @_ );

	my %h_params = ( o_accessor => $o_ns, feature => 'Gene', @_ );
	$o_ns        = New FromDecorator::LocusTagsDecorator( %h_params );

	%h_params    = ( o_accessor => $o_ns, @_ );
	$o_ns        = New WhatDecorator::GenesDecorator( %h_params );

	return $o_ns;
}

=head2 function GetTranscriptsFromLocusTags

 Title        : GetTranscriptsFromLocusTags
 Usage        : $o_ns = &GetTranscriptsFromLocusTags( species    => $species,
                                                ra_ids    => $ra_ids,
                                              -out_type  => $out_type,
                                                -o_instance_obs => $o_inst_obs,
                                                -o_logger  => $o_logger
                                            );
 Function     : Build a NarcisseService::Accessor, then decor it with
                a FromDecorator::RegionsDecorator and
                a WhatDecorator::TranscriptsDecorator
 Returns      : $o_ns, a NarcisseService object
 Args         : $species, string, short species name
                $chrom, string, chromosome label
                $ra_ids, array ref of locus tags
                $o_inst_obs, the InstanceObserver onto which you want to work
                $o_logger, Logger object
                $out_type, string &NS_FASTA, &NS_LOCUS, &NS_INFOS

=cut

sub GetTranscriptsFromLocusTags
{
	# pour infos
	my %h_args = ( species   => undef,
	               ra_ids    => undef,
	               -o_instance_obs => undef,
	               -o_logger => undef,
# 	               -out_type  => undef,
	               @_ );

	my $o_ns     = New NarcisseService::Accessor( @_ );

	my %h_params = ( o_accessor => $o_ns, feature => 'Gene', @_ );
	$o_ns        = New FromDecorator::LocusTagsDecorator( %h_params );

	%h_params    = ( o_accessor => $o_ns, @_ );
	$o_ns        = New WhatDecorator::TranscriptsDecorator( %h_params );

	return $o_ns;
}


=head2 function GetCDSsFromLocusTags

 Title        : GetCDSsFromLocusTags
 Usage        : $o_ns = &GetCDSsFromLocusTags( species    => $species,
                                               ra_ids    => $ra_ids,
                                               -o_instance_obs => $o_inst_obs,
                                              -out_type  => $out_type,
                                               -o_logger  => $o_logger
                                            );
 Function     : Build a NarcisseService::Accessor, then decor it with
                a FromDecorator::RegionsDecorator and
                a WhatDecorator::CDSsDecorator
 Returns      : $o_ns, a NarcisseService object
 Args         : $species, string, short species name
                $ra_ids, array ref of locus tags
                $o_inst_obs, the InstanceObserver onto which you want to work
                $o_logger, Logger object
                $out_type, string &NS_FASTA, &NS_LOCUS, &NS_INFOS

=cut

sub GetCDSsFromLocusTags
{
	# pour infos
	my %h_args = ( species   => undef,
	               ra_ids   => undef,
	               -o_instance_obs => undef,
	               -o_logger => undef,
# 	               -out_type  => undef,
	               @_ );

	# PATCH EN ATTENDANT QUE L'API MARCHE AVEC LES BACTERIES
	# PATCH PROVISOIREMENT REMOVED
	#my $GREP_BASED = &TRUE;
	my $GREP_BASED = &FALSE;
	my $o_ns;
	if ( $GREP_BASED == &TRUE )
	{
		$o_ns = New GetCDSsFromLocusTags( -species => $h_args{ species }, @_ );
	}
	else
	{
		$o_ns     = New NarcisseService::Accessor( @_ );

		my %h_params = ( o_accessor => $o_ns, feature => 'Gene', @_ );
		$o_ns        = New FromDecorator::LocusTagsDecorator( %h_params );

		%h_params    = ( o_accessor => $o_ns, @_ );
		$o_ns        = New WhatDecorator::CDSsDecorator( %h_params );
	}

	return $o_ns;
}


=head2 function GetDownstreamsFromLocusTags

 Title        : GetDownstreamsFromLocusTags
 Usage        : $o_ns = &GetDownstreamsFromLocusTags( species    => $species,
                                               ra_ids    => $ra_ids,
                                               -o_instance_obs => $o_inst_obs,
                                              -out_type  => $out_type,
                                               -o_logger  => $o_logger
                                            );
 Function     : Build a NarcisseService::Accessor, then decor it with
                a FromDecorator::RegionsDecorator and
                a WhatDecorator::DownstreamsDecorator
 Returns      : $o_ns, a NarcisseService object
 Args         : $species, string, short species name
                $ra_ids, array ref of locus tags
                $o_inst_obs, the InstanceObserver onto which you want to work
                $o_logger, Logger object
                $out_type, string &NS_FASTA, &NS_LOCUS, &NS_INFOS

=cut

sub GetDownstreamsFromLocusTags
{
	# pour infos
	my %h_args = ( species   => undef,
	               ra_ids   => undef,
	               -o_instance_obs => undef,
	               -o_logger => undef,
# 	               -out_type  => undef,
	               @_ );

	my $o_ns     = New NarcisseService::Accessor( @_ );

	my %h_params = ( o_accessor => $o_ns, feature => 'Gene', @_ );
	$o_ns        = New FromDecorator::LocusTagsDecorator( %h_params );

	%h_params    = ( o_accessor => $o_ns, @_ );
	$o_ns        = New WhatDecorator::SlicesDecorator::DownstreamsDecorator( %h_params );

	return $o_ns;
}


=head2 function GetUpstreamsFromLocusTags

 Title        : GetUpstreamsFromLocusTags
 Usage        : $o_ns = &GetUpstreamsFromLocusTags( species    => $species,
                                               ra_ids    => $ra_ids,
                                               -o_instance_obs => $o_inst_obs,
                                              -out_type  => $out_type,
                                               -o_logger  => $o_logger
                                            );
 Function     : Build a NarcisseService::Accessor, then decor it with
                a FromDecorator::RegionsDecorator and
                a WhatDecorator::UpstreamsDecorator
 Returns      : $o_ns, a NarcisseService object
 Args         : $species, string, short species name
                $ra_ids, array ref of locus tags
                $o_inst_obs, the InstanceObserver onto which you want to work
                $o_logger, Logger object
                $out_type, string &NS_FASTA, &NS_LOCUS, &NS_INFOS

=cut

sub GetUpstreamsFromLocusTags
{
	# pour infos
	my %h_args = ( species   => undef,
	               ra_ids    => undef,
	               -o_instance_obs => undef,
	               -o_logger => undef,
# 	               -out_type  => undef,
	               @_ );

	my $o_ns     = New NarcisseService::Accessor( @_ );

	my %h_params = ( o_accessor => $o_ns, feature => 'Gene', @_ );
	$o_ns        = New FromDecorator::LocusTagsDecorator( %h_params );

	%h_params    = ( o_accessor => $o_ns, @_ );
	$o_ns        = New WhatDecorator::SlicesDecorator::UpstreamsDecorator( %h_params );

	return $o_ns;
}


=head2 function GetProteinsFromLocusTags

 Title        : GetProteinsFromLocusTags
 Usage        : $o_ns = &GetProteinsFromLocusTags( species    => $species,
                                                ra_ids    => $ra_ids,
                                                -o_instance_obs => $o_inst_obs,
                                              -out_type  => $out_type,
                                                -o_logger  => $o_logger
                                            );
 Function     : Build a NarcisseService::Accessor, then decor it with
                a FromDecorator::RegionsDecorator and
                a WhatDecorator::ProteinsDecorator
 Returns      : $o_ns, a NarcisseService object
 Args         : $species, string, short species name
                $ra_ids, array ref of locus tags
                $o_inst_obs, the InstanceObserver onto which you want to work
                $o_logger, Logger object
                $out_type, string &NS_FASTA, &NS_LOCUS, &NS_INFOS

 Note : Beaucoup trop lente ! utiliser la version sans API

=cut

sub GetProteinsFromLocusTags
{
	# pour infos
	my %h_args = ( species   => undef,
	               ra_ids   => undef,
	               -o_instance_obs => undef,
	               -o_logger => undef,
# 	               -out_type  => undef,
	               @_ );

	# PATCH EN ATTENDANT QUE L'API MARCHE AVEC LES BACTERIES
	# PATCH PROVISOIREMENT REMOVED
	# PATCH REMIS - SINON CA NE MARCHE PAS DU TOUT ! (MANU)
	my $GREP_BASED = &TRUE;
	#my $GREP_BASED = &FALSE;
	my $o_ns;
	if ( $GREP_BASED == &TRUE )
	{
		eval( 'use NarcisseService::GrepBased::GetProteinsFromLocusTags' ); 
		$o_ns = New GetProteinsFromLocusTags( -species => $h_args{ species }, @_ );
	}
	else
	{
		$o_ns     = New NarcisseService::Accessor( @_ );

		my %h_params = ( o_accessor => $o_ns, feature => 'Gene', @_ );
		$o_ns        = New FromDecorator::LocusTagsDecorator( %h_params );

		%h_params    = ( o_accessor => $o_ns, @_ );
		$o_ns        = New WhatDecorator::ProteinsDecorator( %h_params );
	}

	return $o_ns;
}


=head2 function GetGenesFromTranscriptLocusTags

 Title        : GetGenesFromTranscriptLocusTags
 Usage        : $o_ns = &GetGenesFromTranscriptLocusTags( species    => $species,
                                                          ra_ids    => $ra_ids,
                                                          -o_instance_obs => $o_inst_obs,
                                                          -out_type  => $out_type,
                                                          -o_logger  => $o_logger
                                                          );
 Function     : Build a NarcisseService::Accessor, then decor it with
                a FromDecorator::TranscriptLocusTagsDecorator and
                a WhatDecorator::GenesDecorator
 Returns      : $o_ns, a NarcisseService object
 Args         : $species, string, short species name
                $ra_ids, array ref, array of locus tags
                $o_inst_obs, the InstanceObserver onto which you want to work
                $o_logger, Logger object
                $out_type, string &NS_FASTA, &NS_LOCUS, &NS_INFOS

=cut

sub GetGenesFromTranscriptLocusTags
{
	# pour infos
	my %h_args = ( species    => undef,
	               ra_ids    => undef,
	               -o_instance_obs => undef,
	               -o_logger => undef,
# 	               -out_type  => undef,
	               @_ );

	my $o_ns     = New NarcisseService::Accessor( @_ );

	my %h_params = ( o_accessor => $o_ns, feature => 'Gene', @_ );
	$o_ns        = New FromDecorator::TranscriptLocusTagsDecorator( %h_params );

	%h_params    = ( o_accessor => $o_ns, @_ );
	$o_ns        = New WhatDecorator::GenesDecorator( %h_params );

	return $o_ns;
}


=head2 function GetGenesFromProteinIds

 Title        : GetGenesFromProteinIds
 Usage        : $o_ns = &GetGenesFromProteinIds( species    => $species,
                                                 ra_prot    => $ra_prot,
                                                -o_instance_obs => $o_inst_obs,
                                                -out_type  => $out_type,
                                                -o_logger  => $o_logger
                                            );
 Function     : Build a NarcisseService::Accessor, then decor it with
                a FromDecorator::ProteinIdsDecorator and
                a WhatDecorator::GenesDecorator
 Returns      : $o_ns, a NarcisseService object
 Args         : $species, string, short species name
                $ra_prot, array ref of protein ids
                $o_inst_obs, the InstanceObserver onto which you want to work
                $o_logger, Logger object
                $out_type, string &NS_FASTA, &NS_LOCUS, &NS_INFOS

=cut

sub GetGenesFromProteinIds
{
	# pour infos
	my %h_args = ( species   => undef,
	               ra_prot   => undef,
	               -o_instance_obs => undef,
	               -o_logger => undef,
# 	               -out_type  => undef,
	               @_ );

	my $o_ns     = New NarcisseService::Accessor( @_ );

	my %h_params = ( o_accessor => $o_ns, feature => 'Gene', @_ );
	$o_ns        = New FromDecorator::ProteinIdsDecorator( %h_params );

	%h_params    = ( o_accessor => $o_ns, @_ );
	$o_ns        = New WhatDecorator::GenesDecorator( %h_params );

	return $o_ns;
}


=head2 function GetGenesFromSymbols

 Title        : GetGenesFromSymbols
 Usage        : $o_ns = &GetGenesFromSymbols( species    => $species,
                                                 ra_prot    => $ra_prot,
                                                -o_instance_obs => $o_inst_obs,
                                                -out_type  => $out_type,
                                                -o_logger  => $o_logger
                                            );
 Function     : Build a NarcisseService::Accessor, then decor it with
                a FromDecorator::ProteinIdsDecorator and
                a WhatDecorator::GenesDecorator
 Returns      : $o_ns, a NarcisseService object
 Args         : $species, string, short species name
                $ra_prot, array ref of protein ids
                $o_inst_obs, the InstanceObserver onto which you want to work
                $o_logger, Logger object
                $out_type, string &NS_FASTA, &NS_LOCUS, &NS_INFOS

=cut

sub GetGenesFromSymbols
{
	# pour infos
	my %h_args = ( species   => undef,
	               ra_prot   => undef,
	               -o_instance_obs => undef,
	               -o_logger => undef,
# 	               -out_type  => undef,
	               @_ );

	my $o_ns     = New NarcisseService::Accessor( @_ );

	my %h_params = ( o_accessor => $o_ns, feature => 'Gene', @_ );
	$o_ns        = New FromDecorator::SymbolsDecorator( %h_params );

	%h_params    = ( o_accessor => $o_ns, @_ );
	$o_ns        = New WhatDecorator::GenesDecorator( %h_params );

	return $o_ns;
}


=head2 function GetGenesFromAccessionNumbers

 Title        : GetGenesFromAccessionNumbers
 Usage        : $o_ns = &GetGenesFromAccessionNumbers( species    => $species,
                                                 ra_prot    => $ra_prot,
                                                -o_instance_obs => $o_inst_obs,
                                                -out_type  => $out_type,
                                                -o_logger  => $o_logger
                                            );
 Function     : Build a NarcisseService::Accessor, then decor it with
                a FromDecorator::ProteinIdsDecorator and
                a WhatDecorator::GenesDecorator
 Returns      : $o_ns, a NarcisseService object
 Args         : $species, string, short species name
                $ra_prot, array ref of protein ids
                $o_inst_obs, the InstanceObserver onto which you want to work
                $o_logger, Logger object
                $out_type, string &NS_FASTA, &NS_LOCUS, &NS_INFOS

=cut

sub GetGenesFromAccessionNumbers
{
	# pour infos
	my %h_args = ( species   => undef,
	               ra_prot   => undef,
	               -o_instance_obs => undef,
	               -o_logger => undef,
# 	               -out_type  => undef,
	               @_ );

	my $o_ns     = New NarcisseService::Accessor( @_ );

	my %h_params = ( o_accessor => $o_ns, feature => 'Gene', @_ );
	$o_ns        = New FromDecorator::AccessionNumbersDecorator( %h_params );

	%h_params    = ( o_accessor => $o_ns, @_ );
	$o_ns        = New WhatDecorator::GenesDecorator( %h_params );

	return $o_ns;
}




=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license - www.cecill.info

=cut

1;
