# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=head1 NAME

Planner

=head1 DESCRIPTION

Planner is the mother class for ChromosomeMap (used by NcDisplayMap) and GenomeMap (used by NcDotplot)
Common parameters are initialized here, tha's all


=head1 AUTHOR

Stahl Olivier : Olivier.Stahl@toulouse.inra.fr

=cut

package Planner;

use strict;
use warnings;

use base qw( LipmObject );

use ParamParser;
use LipmError::ObjectException;
use LipmError::ParameterException;
use MapArea;
use General; #SafeIsa

my $VERSION = do {my @r = (q$Rev: 1 $ =~ /\d+/g); $r[0]};

# Default parameters.
our $DEFAULT_PROG 	= 'genomic';
our $DEFAULT_LEVEL	= undef;


=head2 _Init
 
 Title        : _Init
 Usage        : _Init( o_io => $o_io, o_param  => $o_param, o_logger => $o_logger )
 Args         :
   $o_io, an InstanceObserver object,
   $o_param, a ParamParser object,
   $o_logger, a Logger object
   $rh_organisms, a hash whose structured is described in NcScript.pm object
 Errors		  : LipmError::ParameterException

=cut
sub _Init
{
	my $self = shift;
	my %h_args = (
				o_io        => undef,
				o_param     => undef,
# 				-o_logger    => undef,
                rh_organisms=> undef,
				@_ );

	$self->SUPER::_Init( %h_args );
    
    my $o_logger = $h_args{ -o_logger };
    $self->SetLogger($o_logger);

	# o_io is mandatory, NOT o_param not rh_organisms
	if( 0 == &SafeIsa( $h_args{ o_io }, 'InstanceObserver' ) )
	{
		throw LipmError::ParameterException("o_io value", 'missing');
	}

	$self->_Set( _o_param => $h_args{ o_param },
				 _o_io    => $h_args{ o_io },
                 _rh_organisms => $h_args{ rh_organisms });

    return $self;
}


=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license under French law - www.cecill.info

=cut

1;
