
# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

#
# 	sebastien.letort@toulouse.inra.fr
#	Created: 21-07, 2009
#	$Id: Snp.pm 3728 2011-09-28 13:24:44Z manu $
#

package SequencesLocator::Snp;

=pod

=head1 NAME

 SequencesLocator::Snp -

=head1  SYNOPSIS

 # code

=head1 DESCRIPTION

 #descr

=cut

use strict;
use warnings;

use Data::Dumper;
use IO::File;
use File::Temp;

use Runner;
use FastaReader::SnpReader;
use General;
use SnpCurator;
use DataReader::whole;

use base qw( SequencesLocator );

#use General;

=head2 method _Init

 Usage        : my $o_x = New SnpIntegrator( file => $file,
                                          o_species => $o_species );
 Prerequisite : none
 Fonction     : initialise an instance of SnpIntegrator class.
 Returns      : a valid SnpIntegrator object
 Args         : $file
    $o_species, SpeciesObserver object.

 Attributes   :
    __?

=cut

sub _Init
{
	my $self = shift;
# 	my %h_args = ( -o_logger => undef,
# 	               file      => undef,
# 	               o_species => undef,
# 	               -raw_output => undef,
# 	               -huge_file => &FALSE,
# 	               @_ );
	$self->SUPER::_Init( @_ );

	# clean input file
	#my $ofh_tmp = new File::Temp( TEMPLATE => 'snp_input_XXXX',
	#							  UNLINK => 0, DIR => '/tmp/' );
	#my $cleaned_file = $ofh_tmp->filename();

	#$self->_CleanInput( $cleaned_file );

# 	my $ra_headers = $o_reader->GetHeaders();
# 	my $msg = "nb of SNPs in the input file : " . scalar( @$ra_headers );
# 	$self->_Trace( "$msg\n", '.', 'Is' );

	my $cleaned_file = $self->_Get('file');
	$self->_Set( __cleaned_file => $cleaned_file,
				 __nb_snps_ok   => 0,
				 __nb_snps_filtered => 0 );


	return;
} # _Init


=head2 DESTROY

 Usage    : $o_x->DESTROY( none )
 Procedure: remove the cleaned file created in the constructor.
 Args     : none
 Error    : none.

=cut

sub DESTROY
{
	my $self = shift;

	my $file = $self->_Get( '__cleaned_file' );
# 	warn "call unlink $file.";
	unlink( $file ) if defined $file;

	$self->SUPER::DESTROY();

	return;
} #DESTROY


=head2 GenerateData

 Usage    : $o_x->GenerateData()
 Procedure: This is the method that final user should use.
            Developpers should implement this method in daughter class.
 Args     : none.
 Error    : none.

=cut

sub GenerateData
{
	my $self = shift;

	my $o_io = $self->_Get( '_o_species' )->GetInstanceObserver();
	my $glint_opt = $o_io->GetCfg( 'GLINT_SNP' );

# pour MT 13/2 sur MT
# 	$self->LaunchGlint( -glint_opt => '--mmis=13 --mgap=2' );

	$self->LaunchGlint( -glint_opt => $glint_opt );
	$self->CureData();

	return;
} #GenerateData


=head2 CureData

 Usage        : $o_x->CureData( $o_species )
 Function     : parse _raw_output file and store valuable data in __rh_snp_data.
                valuable means "one mismatch, once in the whole genome".
 Args         : $o_species, SpeciesObserver.
 Error        : none.

=cut

sub CureData
{
	my $self   = shift;
	my ($o_reader) = @_;

	my @a_attrs = qw( _raw_output _file ); # _file not used yet
	my( $glint_output, $snp_file ) = $self->_Get( @a_attrs );

	# read glint output
	$self->_Trace("now reading glint results\n",'*','IsV');
	my $o_ndr   = New Datareader::Whole();
	$o_ndr->Prepare( -from => $glint_output );
	my $ra_data = $o_ndr->FetchallArrayref(-line_type => 'HASH');
	$self->_Trace("finished reading glint results\n",'*','IsV');
	my $nb_results = scalar( @$ra_data );
	my $msg = "glint : nb results = " . $nb_results . "\n";
	$self->_Trace( $msg, '*', 'Is' );

	# filtration
	my %h_to_delete = ();
	# ici grouper d'abord par SNP les résulats de glint.
	print STDERR "Now searching SNP from glint results\n";
	my $cnt=0;
	while (my $rh_glint_data = shift @$ra_data)
	#foreach my $rh_glint_data ( @$ra_data )
	{
		$cnt++;
		print STDERR "\r" . $cnt . '/' . $nb_results . "..." if ($cnt % 100 == 0);
		my $o_snp_cur = New SnpCurator( rh_glint_data  => $rh_glint_data,
										o_snp_reader   => $o_reader,
										o_species      => $self->GetSpecies(),
										-o_logger      => $self->GetLogger() );
		next if( !$o_snp_cur->IsValid() ); # si pas trouve dans $o_reader

		$o_snp_cur->Filter();
		my $o_snp  = $o_snp_cur->GetSnp();
		next unless( defined $o_snp );

		$self->__StoreSnp( $o_snp, \%h_to_delete );
	} # foreach glint data
	print STDERR "\r" . $cnt . '/' . $nb_results . " done.\n";

	# deletion of the one marked
	foreach my $o_snp ( values %h_to_delete )
	{
		$self->__RejectSnpCauseNotSpecificEnough( $o_snp );
	}
# 	%h_to_delete = (); # vidage memoire

	my $nb_snps_ok = $self->_Get( '__nb_snps_ok' );
	$msg = "nb snps that will be printed $nb_snps_ok\n";
	$self->_Trace( $msg, '*', 'Is' );

	return;
} #CureData


=head2 __StoreSnp

 Usage    : $o_x->__StoreSnp( $o_snp, $rh_to_delete )
 Procedure: Store $o_snp unless it has already be stored.
            In such a case, the SNP is not stored and
            the previous one is marked to be deleted.
 Args     :
     $o_snp, a SNP object
     $rh_to_delete, hash ref. first key is chrom, second the id of the snp to remove.
 Error    : none.

=cut

sub __StoreSnp
{
	my $self = shift;
	my( $o_snp, $rh_to_delete ) = @_;

	$self->_Increments( '__nb_snps_filtered' );

	my $extern_id  = $o_snp->GetExternId();
	my $chrom   = $o_snp->GetChrom();
	my $score   = $o_snp->GetScore();

	my $rh_snps = $self->GetData();
	if( defined( $$rh_snps{ $extern_id } ) )
	{
		my $prev_score = $$rh_snps{ $extern_id }->GetScore();
		if( $score < $prev_score )
		{
			$self->__RejectSnpCauseNotBestScore( $o_snp, $$rh_snps{ $extern_id } );
		}
		elsif( $prev_score < $score )
		{
			$self->__RejectSnpCauseNotBestScore( $$rh_snps{ $extern_id }, $o_snp );
			$$rh_snps{ $extern_id } = $o_snp;
		}
		else # $score == $prev_score
		{
			# mark the previous one to be deleted and delete current
			$$rh_to_delete{ $extern_id } = $$rh_snps{ $extern_id };
			$self->__RejectSnpCauseNotSpecificEnough( $o_snp );
		}
	}
	else
	{
		$$rh_snps{ $extern_id } = $o_snp;
		$self->_Increments( '__nb_snps_ok' );
	}

	return;
} #__StoreSnp


=head2 __RejectSnpCauseNotBestScore

 Usage    : $o_x->__RejectSnpCauseNotBestScore( $o_snp_reject, $o_snp_best )
 Procedure: Both SNP object have the same extern_id.
 Args     :
     $o_snp_reject, $o_snp_best
 Error    : none.

=cut

sub __RejectSnpCauseNotBestScore
{
	my $self = shift;
	my( $o_snp_bad, $o_snp_good ) = @_;

	my $id     = $o_snp_bad->GetExternId();
	my( $b_pos, $b_chrom, $b_strand ) = $o_snp_bad->GetPosition();
	my( $g_pos, $g_chrom, $g_strand ) = $o_snp_good->GetPosition();
	my $b_score  = $o_snp_bad->GetScore();
	my $g_score = $o_snp_good->GetScore();

	my $err_msg = "$id has a better score on chrom $g_chrom at position $g_pos, strand $g_strand with $g_score than $b_score on chrom $b_score at position $b_pos, strand $b_strand.\n";
	$self->_Trace( $err_msg, '*', 'Is' );
	$self->_Decrements( '__nb_snps_ok' );

	return;
} #__RejectSnpCauseNotBestScore


=head2 __RejectSnpCauseNotSpecificEnough

 Usage    : $o_x->__RejectSnpCauseNotSpecificEnough( $o_snp )
 Procedure: .
 Args     :
     $o_snp
 Error    : none.

=cut

sub __RejectSnpCauseNotSpecificEnough
{
	my $self = shift;
	my( $o_snp ) = @_;

	my $err_msg = $o_snp->GetExternId() . " is not enough specific.\n";
	my( $pos, $chrom, $strand ) = $o_snp->GetPosition();
	$err_msg   .= "-> found at position $pos of chrom $chrom, strand $strand.\n";
	$self->_Trace( $err_msg, '*', 'Is' );
	$self->_Decrements( '__nb_snps_ok' );

	return;
} #__RejectSnpCauseNotSpecificEnough


=head2 Procedure _Increments

 Usage    : $val = $self->_Increments( $attribute_name );
 Access   : Protected
 Procedure: Increments the attribute $attribute_name.
            This function is needed because since _Get is used, attributes cannot be accessed directly.
 Args     : $attribute_name, a private attribute start with '__'
 Globals  : none

=cut

sub _Increments
{
	my $self    = shift;
	my( $key ) = @_;

# A mettre dans un LipmObjectEnhanced ?
# 	if( $key =~ /^__/ )
# 	{
# 		my ( $mother, @a_scrap) = caller();
# 		$key = "__$mother$key";
# 	}
#
# 	++$self->{ $key };

	my $val = $self->_Get( $key );
	++$val;
	$self->_Set( $key => $val );

	return;
}


=head2 Procedure _Decrements

 Usage    : $val = $self->_Decrements( $attribute_name );
 Access   : Protected
 Procedure: Increments the attribute $attribute_name.
            This function is needed because since _Get is used, attributes cannot be accessed directly.
 Args     : $attribute_name, a private attribute start with '__'
 Globals  : none

=cut

sub _Decrements
{
	my $self    = shift;
	my( $key ) = @_;

# 	if( $key =~ /^__/ )
# 	{
# 		my ( $mother, @a_scrap ) = caller();
# 		$key = "__$mother$key";
# 	}
# 	--$self->{ $key };

	my $val = $self->_Get( $key );
	--$val;
	$self->_Set( $key => $val );

	return;
}



# =head2 UpdateSnps
#
#  Usage    : $o_x->UpdateSnps( $chrom => $rao_snps )
#  Procedure: Because SNP effect have now to be present in the SNP file.
#             replace the SNPs of this class by new snps, without any check.
#             Just to access data the way they have to through
#             GetNarcissePrintableData.
#  Args     : hash, keys are chrom, values are ref list of SNP objects
#  Error    : none.
#
# =cut
#
# sub UpdateSnps
# {
# 	my $self = shift;
# 	my %h_snps = @_;
#
# 	my $rh_data = $self->GetData();
#
# 	foreach my $chrom ( keys %h_snps )
# 	{
# 		$$rh_data{ $chrom } = {}; # effacement du precedent
# 		my $rh_snps = $$rh_data{ $chrom };
#
# 		my $rao_snps = $h_snps{ $chrom };
# 		foreach my $o_snp ( @$rao_snps )
# 		{
# 			my $snp_id = $o_snp->GetIdentifier();
# 			$$rh_snps{ $snp_id } = $o_snp;
# 		}
# 	}
#
# 	return;
# } #UpdateSnps
#

1;

__END__
