#
# 	emmanuel.courcelle@toulouse.inra.fr
#	Created: december 05, 2008
#	$Id: WebBuilderFactory.pm 2317 2009-01-22 14:23:43Z sletort $
#

package NcAccess;

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod
=head1 NAME

 NcAccess.pm

=head1 SYNOPSIS


=head1 DESCRIPTION

 NcAccess derives from NCScript 
 NcAccess is used by the "Keyword" entry point in the home page
 
=cut

use strict;
use warnings;

use Data::Dumper;
use File::Basename;

use General;
use WebBuilder;
use cgilib::NcScript;
use NARCISSE_utils;
use LipmError::IOException;
use Digest::MD5;

use base qw( NcScript );


=head2 Function GetHtml

 Usage    : $html = $o_ncs->GetHtml()
 Function : Generate and return the html
 Returns  : The html generated
 Args     : none

=cut

sub GetHtml()
{
	my $self = shift;

	my $o_wb  = $self->_Get('_o_wb');
    my $o_io  = $self->_Get('_o_io');

	my $keyword = $o_wb->GetParams('keyword');

	$self->_Trace("GetHtml called - keyword = $keyword\n","*","NV");

    #
    # PATCH PAR MANU - Utilisation d'une requete directe via grep plutot que de deux services
    #   Ex. Recherche polymerase I sur animals avec les services renvoie 7156 resultats et met un temps fou
    #                                          avec le grep met qq sec et renvoie 100 resultats
    #   Raison = 1/ ncs_get_locus_tags  n'est pas suffisamment specifique (revoir le grep)
    #            2/ ncs_get_annotations renvoie 10 résultats par locus_tag, mais les 10 resultats sont-ils tous pertinents ?
    #
    
    my @a_result;
    use constant PATCH_GREP => &TRUE;
    if (PATCH_GREP == &FALSE)
    {    
    	my $ncs_get_locus_tags  = $o_io->GetCfg( 'ROOT_NCS_GETLOCUSTAGS' );
    	my $ncs_get_annotations = $o_io->GetCfg( 'ROOT_NCS_GETANNOTATIONSFROMLOCUSTAGS' );
    	my $cde                 = "$ncs_get_locus_tags --keywords \"$keyword\" | $ncs_get_annotations --locus_tags_file -";
    	$self->_Trace( $cde."\n", '*', 'NV' );
        @a_result = `$cde`;
    }
    else
    {    
        @a_result = $self->__DoTheGrep($keyword);
    }
	my @a_result_all_sorted = sort { $a cmp $b } @a_result;
	
	# Uniq
	my @a_result_sorted;
	my $prev_line = '';
	foreach my $line (@a_result_all_sorted)
	{
		next if ($line eq $prev_line);
		$prev_line = $line;
		push @a_result_sorted, $line;
	}

    my $html;
    if (@a_result_sorted>0)
    {
        my $nb_results = @a_result_sorted;
        # Save the file as a csv file
        my $headers = "#".join( "\t", qw( locus_tag species chrom start end annotation ) );
        
        unshift( @a_result_sorted, "$headers\n");
        my $fh_csv = new IO::File ('>annotations.csv') or throw LipmError::IOException('annotations.csv','f');
        #print $fh_csv "$headers\n";
        foreach my $line (@a_result_sorted)
		{
            print $fh_csv "$line\n";
        }
        $fh_csv->close();
        
        # Store the loci in a file, 1line/locus
        # The file name is the request name, md5-coded
        my $md5 = new Digest::MD5;
        $md5->add($keyword);
        my $f_loci  = $md5->hexdigest() . '.txt';
        
        # Generate html code
        $html = "<div id=\"ID_HOME\"><h1>Requested keyword: $keyword" .": " . $nb_results . " results</h1>";
        if (@a_result_sorted < $o_io->GetCfg('MAX_ACCESS_TABLE'))
        {
            my $base_url = $o_io->GetCfg( 'HTTP_NARCISSE_CGI' ).'?__wb_session='.$o_wb->GetParams('__wb_session').'&amp;script=DISPLAY&amp;search=1';
            
            my $fh_loci = new IO::File( $f_loci, 'w') || throw LipmError::IOException ($f_loci,'f');
            foreach my $line (@a_result_sorted)
            {
                next if ($line =~ /^#/);
                # locus\tspecies\tchrom_nb\tstart\tend (any annotation follows)
                my ($locus,$species,$chrom,$start,$end,$annotation) = split(/\t/, $line,6);
                $annotation =~ s|AutoIPR:(.+)$|<em>$1</em>|;
                my $url = $base_url.'&amp;'."gene=$locus".'&amp;'."reference=$species" . '&amp;'."chroms_reference=$chrom".'&amp;target1=none';
                $line  = "<a class=\"ExtLinks\" href=\"$url\">$locus<\/a>\t" . $species. "\t$chrom\t$start\t$end\t$annotation\n";
                print $fh_loci "$species\t$locus\n";
            }
            $html .= &NARCISSE_utils::TabArray2Html(\@a_result_sorted);
        }
        else
        {
            $html .= "<p class=\"Warning\">WARNING ! There are too many results to be displayed</p>";
        }
        $html .= "<h2>Download the results</h2>";
        $html .= "<p><a href=\"" . $o_io->GetCfg('HTTP_SESSDIR'). '/annotations.csv' . "\" >download</a> the results (csv file)</p>";
        
        my $base_url = $o_io->GetCfg( 'HTTP_NARCISSE_CGI' ).'?__wb_session='.$o_io->GetSessionDirectory().'&amp;script=SENDSEQUENCES&amp;__wba_popup=1&amp;';
        $html .= "<h2>Download the sequences</h2>";
        $html .= "<ul>";
        foreach my $object (qw( Gene Cds Protein))
        {
            my $url = $base_url . $object . 'Seq=' . $f_loci;
            $html .= "<li><a class=\"WindowSequence\" href=\"" . $url . "\" >$object </a> sequence</li>";
        }
        $html .= "</ul>";
        $html .= "</div>";
	}
	else
	{
        $html = "<div id=\"ID_HOME\"><h1>Requested keyword: $keyword" .": no result</h1>";
		$html .= "<p class=\"Warning\">WARNING ! No result found.</p>";
        $html .= "</div>";
	};
	return $html;
}

=head2 Function __DoTheGrep

 Usage    : @a_result = $self-->__DoTheGrep($keyword)
 Function : Calling grep and returning the reformatted results
            Merci Jerome pour la commande grep precise
 Returns  : The results found as an array of \t separated lines
 Args     : $keyword, the keyword

=cut
sub __DoTheGrep
{
    my $self = shift;
    my ($keyword) = @_;
    
    $keyword =~ s/\s+/ /g;
    
    my $o_io = $self->_Get('_o_io');
    my $db_dir      = File::Spec->catfile($o_io->GetCfg('ROOT_DATA'),$o_io->GetCfg('DB'));
    my $db_prot     = File::Spec->catfile($db_dir,$o_io->GetCfg('DB_PROT_MERGE'));
    
    my $unix_grep  = $o_io->GetUnixPath('UNIX_GREP');
    
    my $cmd = $unix_grep . " '>' " . $db_prot . ' | ' . $unix_grep . ' -E -i "\b' . $keyword . '\b" ';
    $self->_Trace( $cmd."\n", '*', 'NV' );
    my @a_return = `$cmd`;
    
    # Extract pertinent information 
    my @a_rvl;
    foreach my $line (@a_return)
    {
        chomp $line;
        my @a_fields = split (' ',$line);
        my ($locus_tag,$species,$chrom,$annotation);
        while (@a_fields != 0)
        {
            #warn Dumper \@a_fields;
            my $f = shift @a_fields;
            next if $f =~ /^>/;
            $locus_tag = $1,next if ($f =~ /locus_tag=(.+)$/);
            $species   = $1,next if ($f =~ /sp=(.+)$/);
            $chrom     = $1,next if ($f =~ /chr=(.+)$/);
            next if ($f =~ /(gn|len|strand)=/);
            
            unshift @a_fields, $f;
            $annotation = join (' ',@a_fields);
            last;
        }
        my ($start, $end) = $self->__ExtractStartEnd($o_io,$species,$chrom,$locus_tag);
        push @a_rvl, join("\t",$locus_tag,$o_io->GetSpeciesLongName($species),$chrom,$start,$end,$annotation);
    }
    return @a_rvl;
}

=head2 Function __ExtractStartEnd

 Usage    : my ($start,$end) = $self->__ExtractStartEnd($o_io,$species,$chrom,$locus_tag)
 Function : From a known locus tag, extract the start/end positions from the gene files
 Returns  : $start,$end
 Args     : species, chromosome, locus_tag

=cut
sub __ExtractStartEnd
{
    my $self = shift;
    
    my ($o_io,$species,$chrom,$locus_tag) = @_;
    my $o_sp       = $o_io->GetSpeciesObserver($species);
    my $chrom_file = $o_sp->GetGeneFiles(
        -chrom => $chrom,
        -check => &TRUE
        );
    my $rh_data = &NARCISSE_utils::ReadNarcisseTabFile(
        file => $chrom_file,
        primary_key_name => 'locus_tag'
        );
    
    my ($start,$end) = (0,0);

    if ( defined $rh_data->{$locus_tag} )
    {
        ($start,$end) = @{$rh_data->{$locus_tag}}{ 'start', 'end' };
    }
    return ($start,$end);
}

1;
