#
# 	emmanuel.courcelle@toulouse.inra.fr
#	Created: december 05, 2008
#	$Id: WebBuilderFactory.pm 2317 2009-01-22 14:23:43Z sletort $
#

package NcAddBookmark;

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod
=head1 NAME

 NcAddBookmark.pm

=head1 SYNOPSIS


=head1 DESCRIPTION

 NcAddBookmark derives from NCScript
 
=cut

use strict;
use warnings;

use LipmError;
use LipmError::IOException;

use Data::Dumper;
use File::Basename;
use Error qw( :try );

use General;
use WebBuilder;
use ParamParser;
use cgilib::NcScript;

use base qw( NcScript );


=head2 function _Init

 Title        : _Init
 Fonction     : the constructor
 Args         : $o_io         : the configuration object
                $o_web_builder: A valid WebBuilder object

=cut
sub _Init
{
    my $self = shift;
    
    my ($o_io,$o_wb,$o_logger) = (@_);
    $self->SUPER::_Init($o_io,$o_wb,$o_logger,&FALSE);
    
    $o_wb->UnsetBehaviour('send_parameters');
    $o_wb->SetBehaviour('do_not_send_header');
    
    my $o_param = $self->_Get('_o_param');
    return $self unless ($o_param->IsDefined('add_bookmark'));

    # Save the bookmark
    $self->__UpdateBookmarks();

    # Delete add_bookmark and script
    my $bookmark = $o_param->Get('add_bookmark');
    $o_param->Delete('add_bookmark');
    $o_param->Delete('script');

    # Retrieve the last local.cfg, because the script parameter is now lost !!!
    # We also dump local.cfg, because some parameters may have been changed since last request
    my $o_param = $self->_Get('_o_param');
    my $o_param_alt = New ParamParser('local.cfg');
    $o_param->Update('PARAMPARSER','A',$o_param_alt);
    $o_param->Dump('local.cfg');

    # Dump $o_param to the bookmark file
    my $bookmark_cfg = $o_io->GetCfg('ROOT_SESSDIR').'/'.$bookmark.'.cfg';
    $o_param->Dump($bookmark_cfg);
    
    return $self;
}

=head2 procedure __UpdateBookmarks

 Title        : __UpdateBookmarks
 Usage        : $self->__UpdateBookmarks()
 procedure    : create or append the bookmarks file and add the add_bookmark parameter to this file
 Args         : none

=cut

sub __UpdateBookmarks {
    my $self = shift;
    
    my $o_io    = $self->_Get('_o_io');
    my $o_param = $self->_Get('_o_param');
    
    my $name = $o_param->Get('add_bookmark');

    my $bmk_file = $o_io->GetCfg('ROOT_NARCISSE_BOOKMARKS');
    my $fh_bmk_file;
    if (-e $bmk_file)
    {
        $fh_bmk_file = new IO::File(">>$bmk_file") or throw Lipm::IOEXception($bmk_file,'f');
    }
    else
    {
        $fh_bmk_file = new IO::File(">>$bmk_file") or throw Lipm::IOEXception($bmk_file,'f');
        print $fh_bmk_file "#name\n";
    };
    print $fh_bmk_file "$name\n";
}

sub _PrepareData
{
    my $self = shift;
    
    # Nothing to do, we just have to set the status to "204": nothing to send to the browser
    $self->_Set('_http_status',"204");
    $self->_Set('_http_message',"BOOKMARK RECORDED");
}

=head2 Function GetHtml

 Title    : GetHtml
 Usage    : $html = $o_ncs->GetHtml()
 Function : Generate and return the html
 Returns  : The html generated
 Args     : none

=cut
sub GetHtml
{
    return "=== 204 ===";
}

1;
