#
# 	emmanuel.courcelle@toulouse.inra.fr
#	Created: december 05, 2008
#	$Id: WebBuilderFactory.pm 2317 2009-01-22 14:23:43Z sletort $
#

package NcBlast;

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod
=head1 NAME

 blast.pm

=head1 SYNOPSIS


=head1 DESCRIPTION

 NcBlast derives from NCScriptresult, it is used by the "Blast" entry point
 
=cut

use strict;
use warnings;

use Error qw( :try );
use LipmError;
use LipmError::ObjectException;
use NarcisseError;

use Data::Dumper;
use File::Basename;

use Carp;

use General;
use WebBuilder;
use Runner;
use cgilib::NcScript;
use NARCISSE_utils;
use NarcisseService::UsingExternalProgram;
use base qw( NcScript );
use Configuration::InstanceObserver;

=head2 Function GetHtml

 Title    : Gethtml
 Usage    : $html = $o_ncs->GetHtml()
 Function : Generate and return the html
 Returns  : The html generated
 Args     : none

=cut

sub GetHtml
{
    my $self = shift;

    my $o_wb    = $self->_Get('_o_wb');
    my $o_io    = $self->_Get('_o_io');
	my $o_param = $o_wb->GetObjectParam();

    # Default values for parameters
    $o_param->SetUnlessDefined('e_value','1e-3');
    $o_param->SetUnlessDefined('blast_prog','blastp');
    $o_param->SetUnlessDefined('target','prot');
    $o_param->SetUnlessDefined('max_per_species',2);

    # Get the necessary parameters
    my $evalue        = $o_param->Get('e_value');
    my $maxperspecies = $o_param->Get('max_per_species');
    my $file_query    = $o_param->Get('fasta_file');
    my $target        = $o_param->Get('target');
    my $blast_prog    = $o_param->Get('blast_prog');

    my $more_params   = '-m 7';

    my $time_stamp = Logger::CalcStamp();
    $time_stamp =~ s/#//g;
    my $blast_out  = "blast.".$time_stamp.".out";    
    $self->_Trace("Now running blast - prog=$blast_prog expect=$evalue more_params=$more_params\n","*","BV");
    my $o_blast = New NarcisseService::Blast(
        seq                  => $self->__PrepareBlast($o_param),
        -prog                => $blast_prog,
        -expect              => $evalue,
        -o_logger            => $self->_GetLogger(),
        -more_params         => $more_params,
        -o_instance_observer => $o_io,
        -outfile             => $blast_out);
    
    $self->_GetLogger()->StartTimer("B1");
    #my ($dummy,$rh_data) = $o_blast->FetchResults();

    $o_blast->FetchResults();
    #my $blast_out = ${$rh_data->{data}}[0];
    $self->_Trace("Returning from blast - output in $blast_out\n","B1","B");

    # Check the output
    if (-z $blast_out ||!-e $blast_out) 
    {
        my $msg = "No output produced by BLAST - Please verify your input sequence";
        $msg .= "<br />or the BLAST program (blastp/blastn)";
        throw NarcisseError(-hd => "BLAST ERROR",-bd => $msg); 
    }
    
    if ($self->_Get('_rest') != &FALSE)
    {
        #throw NarcisseError(-bd => 'coucou');
        my %h_rvl;
        # TODO - NOOOOOON Ne pas exposer le fichier de sortie brut car il y a des chemins absolus
        #        Il faudra plutot formatter la sortie en JSON mais c'est pas immediat
        $h_rvl{blast_xml_output} = $o_io->GetCfg('HTTP_SESSDIR') . '/' . $blast_out;
        return $self->_ToJson(\%h_rvl);
    }
    else
    {
        
        # Get the results
        my $xs = new XML::Simple(KeyAttr => 'Hit_num', ForceArray => [ 'Hit' ] );
        my $r_xml_blast = $xs->XMLin($blast_out);
        my $o_parameters 	= $r_xml_blast->{BlastOutput_param}->{Parameters};
        my $query_def 		= $r_xml_blast->{'BlastOutput_query-def'};
        my $prg_version  	= $r_xml_blast->{BlastOutput_version};
        
        # Get the header
        my $html = $self->__GetHeader($o_parameters,$query_def,$prg_version);
        
        # Reorder the hits
        if ( ref ($r_xml_blast->{BlastOutput_iterations}->{Iteration}) ne 'HASH')
        {
            throw NarcisseError (-hd => "BLAST ERROR",-bd => "A problem happened with blast. <br />May be you did not select the correct BLAST program ? <br />(blastp/blastx)");
        }
        my $rh_hits = $r_xml_blast->{BlastOutput_iterations}->{Iteration}->{Iteration_hits}->{Hit};
        
        my %h_hit_by_species = ();
        my $sp_list = $self->__Reorder(\%h_hit_by_species,$rh_hits);
        
        # Get the hits
        $html .= $self->__GetAllHits($sp_list,\%h_hit_by_species,$rh_hits,$maxperspecies,$blast_prog);
        
        
        #print the body part of the page
        #&LaunchPerlCmd($cde_script_pl);
        
        return $html;
    }
}



################################################
# P R I V A T E    F U N C T I O N S
################################################

=head2 Function __PrepareBlast

 Title    : __PrepareBlast
 Usage    : $self->PrepareBlast($o_param, $session_path)
 Function : get the sequence from the uploaded file or from the text area
 Returns  : the path to the local copy of the data
 Args     : $o_param, the paramParser object (modified)
 Access   : private
 globals  : none

=cut

sub __PrepareBlast
{
    my $self = shift;
    
	my ($o_param) = @_;

	my $fasta_file = $o_param->Get('UploadSeq');
	my $sequence = $o_param->Get('sequence');

    if ($fasta_file ne "")
    {
        return $fasta_file;
    }

	if ( $sequence ne "" )
	{
        $o_param->Delete('sequence');
        $fasta_file = "query_blast".time;

		my $file = new IO::File(">$fasta_file") or throw LipmError::IOException($fasta_file,'f');
        print $file $sequence;
        $file->close;
		return $fasta_file;
	}

    # no filename and no sequence = blank request
    my $error_msg = "Please fill the sequence field OR the filename field !";
    throw NarcisseError( -hd => "NARCISSE ERROR", -bd => $error_msg );
}

=head2 Function __GetHeader

 Title        : __GetHeader
 Usage        : $self->__GetHeader($o_parameters,$query_def,$prg_version)
 Prerequisite : none
 Function     : Get the blast parameters in HTML format
 Returns      : The html header
 Args         : Some data extracted from the Blast output
 Globals      : none

=cut

sub __GetHeader
{
    my $self = shift;
    
    my $html = "";
    
    my($o_parameters,$query_def,$prg_version)= (@_);
    $html .= "<h1>The Narcisse blast results</h1>";
    $html .= "<h2>The query</h2>";
    $html .= "<p>$query_def</p>";
    $html .= "<h2>The blast program</h2>";
	$html .= "<table><tr><th colspan=\"2\">$prg_version</th></tr>";

	foreach my $k (keys(%$o_parameters))
	{
		my $key = $k;
		$key =~ s/Parameters_//;
		$html .= "<tr><th>$key:</th><td>$o_parameters->{$k}</td></tr>\n";
	}
	$html .= "</table>\n";
    return $html;
}

=head2 Function __Reorder

 Title        : __Reorder
 Usage        : $self->__Reorder($rh_hit_by_species,$rh_hits)
 Function     : Sort the HSPs by species
 Returns      : a sorted list of species (string, items separated by blank)
 Args         : $rh_hit_by_species: hash of hits by species (key=species, value=hit IDs of this species)
                $rh_hits: original hits (key=blast id, value=hit description)
 Globals      : none

=cut

sub __Reorder
{
    my $self = shift;
    
    my ($rh_hit_by_species,$rh_hits) = @_;
    my $sp_list = "";

	foreach my $hit (sort {$a <=> $b } keys(%$rh_hits))
	{
		my ($desc) = $rh_hits->{$hit}->{Hit_def};
		my ($sp) = $desc =~ / sp=(\S+) /;
		$sp_list .= " $sp " if ( $sp_list !~ / $sp / );
		$rh_hit_by_species->{$sp} .= " $hit ";
	}
	return $sp_list;
}

=head2 Function __GetAllHits

 Title        : __GetAllHits
 Usage        : $self->__GetAllHits($sp_list,$rh_hit_by_species,$rh_hits,$max, $blast_prog)
 Function     : Main printing loop.
 Returns      : the html code
 Args         : $sp_list:           Sorted list of all matched species (items separated by blank)
                $rh_hit_by_species: key=species, value=blast hits id
                $rh_hits:           original blast hit data structure
                $max:               maximum number of displayed hits per species
                $blast_prog:        The blast program used
 Globals      : none

=cut

sub __GetAllHits
{
    my $self = shift;
    
    my ($sp_list,$rh_hit_by_species,$rh_hits,$max,$blast_prog,$o_logger) = @_;
    my $o_io = $self->_Get('_o_io');
    
    my $html = '';
    my @a_species = split(' ',$sp_list);
    
    # Loop on the species
    for(my $i=0;$i<=$#a_species;$i++)
    {
        $html .= "<h2>".$o_io->GetSpeciesLongName($a_species[$i])."</h2>";

        my @a_hits = split(' ',$rh_hit_by_species->{$a_species[$i]});
        my $i_max = ( $max ne "" && ($max<@a_hits) ) ? $max : @a_hits;
        $self->_Trace("species $a_species[$i] - ".@a_hits." hits found $i_max displayed (max $max)\n","*","BV");
        
        # Loop on the hits
        $html .= "<ol>\n";
        for(my $j=0;$j<$i_max;$j++)
        {
            $html .= $self->__GetHit($rh_hits->{$a_hits[$j]},$a_species[$i],$blast_prog);
        }
        $html .= "</ol>\n";
    }
    
    if ( $#a_species == -1 )
    {
        $html .= "<h3>no hit</h3> check your input sequence or consider less stringent thresholds\n";
    }
    
    return $html;
    
}

=head2 Function __GetHit

 Title        : __GetHit
 Usage        : $self->__GetHit($rh_hit,$sp)
 Function     : Print one hit in html format with appropriate links
 Returns      : html
 Args         : $rh_hit: a blast hit
                $sp: the current species
                $blast_prog The blast program used
 Globals      : none

=cut

sub __GetHit
{
    my $self = shift;
    
    my ($rh_hit,$sp,$blast_prog) = @_;

    my $o_io = $self->_Get('_o_io');
    my $html = '';
    
    # The following may be useful when debugging the code
    #local $Data::Dumper::Purity = 1;
    #my $msg = Data::Dumper->Dump([$rh_hit], [qw(rh_hit)]);
    #my $serial_file = $sp.db';
    #open (SERIAL, ">$serial_file") or warn "merde $serial_file";
    #print (SERIAL "$msg\n");
    #close(SERIAL);

	my $desc = $rh_hit->{Hit_def};
    $self->_Trace("hit: $desc\n","*","BV");
	my ($target) = $desc =~ /$sp:(\S+)/;
    my ($chr)    = $desc =~ /chr=(\S+)/;
    
    my $narcisse_cgi = $o_io->GetCfg('HTTP_NARCISSE_CGI');
    my $sessdir      = $o_io->GetSessionDirectory();
    
    # TODO program_target1 A SUPPRIMER LORSQUE LE BUG AURA ETE CORRIGE
	my $url = "<a href=\"${narcisse_cgi}?";
	my @a_url_args = (	'script=DISPLAY',
						'target1=none',
						'search=1',
						"__wb_session=${sessdir}",
						'reference=' . $o_io->GetSpeciesLongName($sp),
						"chroms_reference=$chr",
						"gene=$target",
						"program_target1=prbh" );
	$url   .= join( '&amp;', @a_url_args );
    $url   .= "\">$target</a>";

	if ( $blast_prog =~ /blast[px]/)
	{
		#$desc .= " <a href=\"$rh_cfg->{HTTP_NARCISSE_CGI}?__wb_session=$rh_cfg->{SESSDIR}&amp;script=BUILD_FAM&amp;obj_id=prot:$target;orga=$sp;cut_of=80\">[Display protein family]</a>";
		$desc .= " <a href=\"${narcisse_cgi}?";
		@a_url_args = (	"__wb_session=${sessdir}",
						'script=DISPLAY',
						"species=$sp",
						"ProteinFam=$target;Cutoff=80" );
		$desc .= join( '&amp;', @a_url_args );
		$desc .= "\">[Display protein family]</a>";
	}
	$desc =~ s|$target|$url </b><span>|;
    $desc =~ s|^$sp:||;
	$html .= "<li><b>$desc</span><br />";

    # $rah_hsp is a reference to something which can be an Array OR a Hash
    my $rah_hsp=$rh_hit->{Hit_hsps}->{Hsp};
    if (ref($rah_hsp) eq 'HASH')
    {
        $html .= __GetHitSimple($rah_hsp,$rh_hit->{Hit_len});
    }
    elsif (ref($rah_hsp) eq 'ARRAY')
    {
        foreach my $rh_hsp (@$rah_hsp)
        {
            $html .= &__GetHitSimple($rh_hsp,$rh_hit->{Hit_len});
        }
    }
    
    return $html;
    
=head2 function __GetHitSimple
 Title        : __GetHitSimple - Simply called by __GetHit
=cut
    sub __GetHitSimple
    {
        my ($rh_hsp,$hit_len)=@_;
        my $html = '';
        
        my $positive = int(($rh_hsp->{Hsp_positive}*100)/$hit_len);
        my $identity = int(($rh_hsp->{Hsp_identity}*100)/$hit_len);
        $html .= "<b>E_value=$rh_hsp->{Hsp_evalue}</b><span>Score=$rh_hsp->{Hsp_score}, Bit-Score=$rh_hsp->{'Hsp_bit-score'}, P=$positive\%, I=$identity\%</span><br />";
        $html .= "<pre>";
        $html .= sprintf "%5s $rh_hsp->{Hsp_qseq}  %5s\n",$rh_hsp->{'Hsp_query-from'},$rh_hsp->{'Hsp_query-to'};
        $html .= sprintf "%5s $rh_hsp->{Hsp_midline}%5s\n"," "," ";
        $html .= sprintf "%5s $rh_hsp->{Hsp_hseq}  %5s\n",$rh_hsp->{'Hsp_hit-from'},$rh_hsp->{'Hsp_hit-to'};
        $html .= sprintf "</pre></li>\n";
        return $html;
    }

}



1;
