#
# 	emmanuel.courcelle@toulouse.inra.fr
#	Created: december 05, 2008
#	$Id: WebBuilderFactory.pm 2317 2009-01-22 14:23:43Z sletort $
#

package NcCircos;

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod
=head1 NAME

 NcCircos.pm

=head1 SYNOPSIS


=head1 DESCRIPTION

 NcCircos NcView, this object implements the 'Circos' representation
 
=cut

use strict;
use warnings;

use LipmError;
use LipmError::ObjectException;

use Data::Dumper;
use File::Basename;
use Error qw( :try );
use Carp;

use General;
use WebBuilder;
use Runner;
use cgilib::NcScript;
use cgilib::NcScript::NcView;
use Legend;

use NARCISSE_utils;

use Image;
use Planner;
use Dotplot;
use Planner::GenomeMap;

use constant IMG_EXT   => ".png";
use constant COORD_EXT => ".coord";
use constant JS_EXT    => ".js";
use constant SPECIES_NAME_BAND_SIZE => 80;
use constant DEBUG     => 1;

use base qw( NcView );


=head2 function _Init

 Title        : _Init
 Fonction     : the constructor
                Build the object, setting some default values (only if necessary) to a minimal set of parameters
 Args         : $o_io         : the instance observer
                $o_web_builder: A valid WebBuilder object
                $o_logger     : a Logger

=cut

sub _Init
{
	my $self = shift;
	my( $o_io, $o_web_builder, $o_logger ) = @_;

	#my $o_param = $o_web_builder->GetObjectParam();

	# Minimal default values for the parameters, they may be changed in guidef.cfg
	#my ($level,$program) = $o_io->GetGuidef('level','program');
	#$o_param->SetUnlessDefined('level',$level) if (defined $level);
	#$o_param->SetUnlessDefined('program',$program) if (defined $program);

	$self->SUPER::_Init($o_io,$o_web_builder,$o_logger,&TRUE);
    
    # There should be at least 1 target parameter
    my $ra_species = $self->_Get('_rh_organisms')->{SPECIES};
    if ($self->GetTargetsNumber()==0)
    {
        throw NarcisseError( -hd => "NARCISSE ERROR", -bd => "Error in parameters: problem in the targets parameters");
    };

    # Circos accepts only ONE COMMON program parameter
    # We check that ALL program_targetN parameters are equal to program_target1,
    # and if not we thow a NarcisseError
    
    my $ra_programs = $self->_Get('_rh_organisms')->{PROGRAMS};
    my $program     = $ra_programs->[1];
    for (my $i=2; $i<=$#$ra_programs; $i++)
    {
        if ( $ra_programs->[$i] ne $program )
        {
            throw NarcisseError( -hd => "NARCISSE ERROR", -bd => "for a Circos representation, there should be only ONE comparison method (ex. genomic or genetic, not both)");
        }
    };
    
	return $self;
}

=head2 Procedure _ComputeImages

 Title        : _ComputeImages
 Usage        : $self->_ComputeImage() - Called by the _Init from NcView
 Fonction     : Compute the images
 Args         : none
 Access       : Protected
 Globals      : none

=cut

sub _ComputeImages
{
	my $self = shift;
	my $o_param = $self->_Get('_o_param');
	my $o_io    = $self->_Get('_o_io');

	#my ( $http_sessdir, $rh_param ) = @_;

	#my $args = 'script=DISPLAY&amp;reference=&amp;target1=none&amp;chroms_reference=';
	#$args .= '&amp;__wb_session=' . $o_param->Get('__wb_session');
	#my $args = 'javascript:O_CONTROL_BLOCK.SubmitAjax("script=DISPLAY&amp;reference=&amp;target1=none&amp;target2=none&amp;chroms_reference=","DISPLAY ")';
	# NOT USED ANY MORE - See the file Core/circos/etc/Narcisse/chrs.conf

	my $rh_orgs = $self->__CollectCircosOrgsAndChrs();
	my $url     = $o_io->GetCfg('HTTP_NARCISSE_CGI');

	# Build the parameters for building Circos::Narcisse
    my $program = $self->_Get('_rh_organisms')->{PROGRAMS}[1];
    #my $program = 'genomic';
	my %h_param = (
		circos_dir	=> $o_io->GetCfg('ROOT_CIRCOS'),
		data_dir	=> $o_io->GetCfg('ROOT_DATA'),
		output_dir	=> '.',
		level		=> $o_param->Get('level'),
        method      => $program,
		rh_orgs		=> $rh_orgs,
		auto_hide	=> $o_param->Get('ck_hide'),
		image_size  => $o_io->GetCfg('CIRCOS_DIM'),
		nb_links_max=> $o_io->GetCfg('CIRCOS_MAX_LINKS')
		#chroms_url	=> $args
		);


	# l'objet circos pour narcisse
	my $o_circos='';
	$o_circos = New Circos::Narcisse(%h_param, imagemap => &TRUE );
	$self->_Set( '__o_circos', $o_circos );

	if( $o_circos->{exist} == &FALSE )
	{
		$o_circos->BuildImage();
		$self->__BuildCircosLegend();
		$self->__ModifyHtmlMap( $o_circos->GetImageBasename() );
	}
	return;
}


=head2 __ModifyHtmlMap

 Usage    : $o_x->__ModifyHtmlMap( $basename )
 Procedure: .
 Args     :
     $basename
 Error    : none.

=cut

sub __ModifyHtmlMap
{
	my $self = shift;
	my( $basename ) = @_;

	my $map_file = $basename . '.html';

	my $o_html_map = new IO::File( $map_file, 'r' )
			or throw LipmError::IOException( $map_file, 'f' );
	my $o_new_map  = new IO::File( 'map', 'w' )
			or throw LipmError::IOException( $map_file, 'f' );

	my $SEP  = &Circos::Narcisse::NAME_SEP;
	my $o_io = $self->_Get( '_o_io' );

	while( my $line = <$o_html_map> )
	{
		# Matching the xhtml norm
		$line =~ s|<map name=('.+')>|<map id=$1 name=$1>|;
		my( $short, $chrom );

		#url
		while( $line =~ /chrom=~(.+?)~/g )
		{
			my $circos_chrom = $1;
			( $short, $chrom ) = split( $SEP, $circos_chrom );
			$line =~ s/chrom=~$circos_chrom~/reference=$short&amp;chroms_reference=$chrom/;

			# Matching the xhtml norm			
			$line =~ s|<area([^/]+)>|<area$1/>|;
			my( $chr_len ) = $line =~ /range=0%3A(\d+)'/;
			my $title = &Circos::Narcisse::CircosChromToNarcisseLabel( $circos_chrom, $chr_len );
			$line =~ s/title='.+?'/title='$title'/;
		}

# 		#title - on change le nom court en long
# 		if( defined $short ) # else => boulette
# 		{
# 			my $o_species = $o_io->GetSpeciesObserver( $short );
# 			my $species   = $o_species->GetLongName();
# 			$line =~ s/title='.+?'/title='$species ($chrom)'/;
# 		}

		print $o_new_map $line;
	}
	$o_html_map->close();
	$o_new_map->close();

	rename( $map_file, 'circos_map.old' );
	rename( 'map', $map_file );

	return;
} #__ModifyHtmlMap


=head2 Function __CollectCircosOrgsAndChrs

	Title      :	__CollectCircosOrgsAndChrs
	Usage      :	my $rh_orgs = this->__CollectCircosOrgsAndChrs();
	Function   :	Collect data from $o_param and reformat the parameters relative to the reference and the target organisms
                    A hash of hash is created, structures as follows:
                    key = species short name
                    val = hash:
                          short = species short name
                          long  = species long name
                          chr   = hash describing the chromosome information (see the file karyotype)
	Returns    :	the hash described above.
	Args       :	none
	globals    :	none

=cut

sub __CollectCircosOrgsAndChrs
{
    my $self = shift;
    
    my $o_param = $self->_Get('_o_param');
    my $o_io    = $self->_Get('_o_io');

    my $rh_organisms = $self->_Get('_rh_organisms');
    my $ra_species   = $rh_organisms->{ SPECIES };
    my $ra_chroms    = $rh_organisms->{ CHROMS };
    my $ra_programs  = $rh_organisms->{ PROGRAMS };

    my $ref = $ra_species->[0];

	# we must mark the reference _circos requirement
	#$ref = &Circos::Narcisse::GetPrefixRef() . $ref;

	# la liste des clefs des orgs
	#my @a_keys = ( 'reference' );
	#for( my $i=1; $i<=$o_io->GetCfg('CIRCOS_MAX_ORG'); ++$i )
	#{
	#	push( @a_keys, "target$i" );
	#}

	# la liste des org _short name
	#my %h_orgs;
	#foreach my $key ( @a_keys )
	#{
    #    last unless defined $self->_Get('_'.$key);
    #    my ( $short, $str_chrs ) = ( $self->_Get('_'.$key), $o_param->Get ( 'chroms_'.$key ) );
    #
    #		my $o_species = New SpeciesObserver( species => $short,
    #											-o_inst_obs => $o_io );
    #		my $rh_karyotype = $o_species->GetChromsInfo();
    #		$str_chrs = 'all'	if( $str_chrs eq '' );	# non renseigne
    #		my %h_chrs;
    #
    #		if( $str_chrs eq 'all' )
    #		{
    #            %h_chrs = %$rh_karyotype;	
    #        }
    #		else
    #		{
    #			foreach my $chr ( split( &ParamParser::SEPARATOR, $str_chrs ) )
    #			{
    #				if( !defined $$rh_karyotype{ $chr } )
    #				{
    #					warn "chromosome '$chr' of $short doesn't not exist, it will be ignored\n";
    #					next;
    #				}
    #				$h_chrs{ $chr } = $$rh_karyotype{ $chr };
    #			}
    #		}
    #
    #		my $long_name = $o_species->GetLongName();
    #
    #		my $circos_key = ( $key eq 'reference' ) ? $ref : $short;
    #
    #		$h_orgs{ $circos_key } = { 
    #            short => $short,
    #            long => $long_name,
    #            chrs => \%h_chrs
    #        };
    #	}

    my %h_orgs;
    my $tgt_nb = $self->GetTargetsNumber();
    for (my $i=0; $i <= $tgt_nb; $i++)
    {
        my $short_name   = $ra_species->[$i];
        my $o_species    = New SpeciesObserver( species => $short_name,-o_inst_obs => $o_io );
        my $long_name    = $o_species->GetLongName();
        my $str_chrs     = $ra_chroms->[$i];
        $str_chrs        = 'all' unless( &General::IsNotNull( $str_chrs ) );

        my $rh_karyotype;
        if ($o_species -> IsPseudo())
        {
			$rh_karyotype = $o_species->GetChromsInfo();
		}
		else
		{
			my $max_displayed_scaffolds = $o_io -> GetCfg( 'MAX_DISPLAYED_SCAFFOLDS' );
			$rh_karyotype = $o_species->GetChromsInfo($max_displayed_scaffolds);
		}
		
        my %h_chrs;
        if( $str_chrs eq 'all' )
        {
            %h_chrs = %$rh_karyotype;	
        }
        else
        {
            foreach my $chr ( split( &ParamParser::SEPARATOR, $str_chrs ) )
            {
                if( !defined $$rh_karyotype{ $chr } )
                {
                    warn "chromosome '$chr' of $short_name doesn't not exist, it will be ignored\n";
                    next;
                }
                $h_chrs{ $chr } = $$rh_karyotype{ $chr };
            }
        }
        
        # we must mark the reference (circos requirement)
        my $key = ($i==0) ? &Circos::Narcisse::GetPrefixRef() . $short_name : $short_name;
        $h_orgs{ $key } = {
            short => $short_name,
            long => $long_name,
            chrs => \%h_chrs
        };
    }
	return \%h_orgs;
} # fin CollectCircosOrgsAndChrs


=head2 procedure __BuildCircosLegend

	Title       :	__BuildCircosLegend
	Usage       :	$o_circos->__BuildCircosLegend( $o_circos, $url );
	Prerequiste :	the legend name must be the circos image filename ended with _lgd
	Procedure   :	this is a one use-function, to clarify code.
	            	It creates a Legend object for every orgs.
	            	It concatenates the image of legend as they are created.
	            	If an org has to much chrs more than $max_per_col, it is divided by 2.
	Returns     :	nothing
	Args        :	$o_circos, a valid Narcisse::circos object
	Globals     :	none

=cut


sub __BuildCircosLegend
{
	my $self = shift;
    my $o_circos = $self->_Get('__o_circos');

	#the legend
	my %h_legend_data = $o_circos->GetLegendInfos();

	my %h_margins = ( top_bottom => 17, left_right => 7,
				inter_elt_width => 7, inter_elt_height => 13 );
	my $max_per_col = 20;
	my $filename = $o_circos->GetImageFilename() . '_lgd.png';

	my $img = undef;
	foreach my $org (sort keys %h_legend_data)
	{
		# il faudrait un accesseur pour etre propre
		next	if( $org eq $o_circos->{ __org_ref } );

		my $ra_data = $h_legend_data{$org};
		my $half_size = int(@$ra_data / 2);
		$half_size += 1	if( ($half_size % 2) == 0 );
		if( $half_size > $max_per_col )
		{# we do it in 2 times
			my @a_partial_data = @$ra_data[0..$half_size];
			my $o_legend_1 = New Legend(\@a_partial_data, \%h_margins, 20,10);
			@a_partial_data = @$ra_data[($half_size+1)..(@$ra_data-1)];
			my $o_legend_2 = New Legend(\@a_partial_data, \%h_margins, 20,10 );
			$img = &Legend::ConcatenateLegend($img, 'width', $o_legend_1, $o_legend_2);
		}
		else
		{
			my $o_legend = New Legend($ra_data, \%h_margins, 20,10 );
# 									'left', GD::Font->Giant);
			$img = &Legend::ConcatenateLegend($img, 'width', $o_legend);
		}
	}
	# to make it smart, I add a border line
	my $black = $img->colorExact(0,0,0);
	$img->setThickness(3);
	my ($width, $height) = $img->getBounds();

	$img->rectangle(0,0, $width-1, $height-1, $black);

	my $file = new IO::File($filename, "w");
	$file->binmode;
	print $file $img->png;

	return ;
}

=head2 function _GetMainImages

 Title        : _GetMainImages
 Usage        : $rvl .= $self->_GetMainImages()
 Fonction     : generate the html code useful to read the main images
 Returns      : html code
 Args         : none
 Globals      : none

=cut

sub _GetMainImages
{
	my $self = shift;

	my $o_io     = $self->_Get('_o_io');
	my $o_circos = $self->_Get('__o_circos');
	my $html;
	if ($o_circos eq '')
	{
		$html .= "<p>Sorry, the circos representation could not be drawn</p>";
	}
	else
	{
		$html .= $o_circos->Show($o_io->GetCfg('HTTP_SESSDIR'));
		# displaying the legend
		my $img_name = $o_circos->GetImageFilename() . '_lgd.png';
		my $src = $o_io->GetCfg('HTTP_SESSDIR') . '/' . $img_name;
		$html .= "<img src='$src' alt='legend circos' />\n";
	}

	# ajout en champs caches du nombre total de nucleotides
	$html .= '<input type="hidden" name="genomes_size" value="';
	$html .= $o_circos->GetNbBasesInWholeImage() . "\" />\n";

	return $html;
}

=head2 function _GetMainImagesJson

 Title        : _GetMainImagesJson
 Usage        : $html .= $self->_GetMainImagesFson()
 Fonction     : generate the json code with urls to the main images
                Implement an abstract function from NcView
 Returns      : Json code
 Args         : none
 Globals      : none

=cut

sub _GetMainImagesJson
{
    my $self = shift;
	my $o_io     = $self->_Get('_o_io');
	my $o_circos = $self->_Get('__o_circos');

    if ($o_circos eq '')
	{
		throw NarcisseError(-bd => 'Sorry, the circos representation could not be drawn');
	}
	else
	{
        my $circos_name = $o_circos->GetImageFilename();
        my $circos_url  = $o_io->GetCfg('HTTP_SESSDIR') . '/' . $circos_name . '.png';
        my $legend_url  = $o_io->GetCfg('HTTP_SESSDIR') . '/' . $circos_name . '_lgd.png';
        
        my %h_urls;
        $h_urls{circos} = $circos_url;
        $h_urls{legend} = $legend_url;
        return $self->_ToJson(\%h_urls);
    }
}

=head2 function _GetPrintableImage

 Title        : _GetPrintableImage
 Usage        : $rvl .= $self->_GetPrintableImage()
 Fonction     : generate the html code useful to read the single image
 Returns      : html code
 Args         : none
 Globals      : none

=cut

sub _GetPrintableImage
{
    my $self = shift;
    return $self->_GetMainImages();
}


=head2 function _GetMainTitle

 Title        : _GetMainTitle
 Usage        : $rvl .= $self->_GetMainTitle()
 Fonction     : generate a little table, to be displayed at top of images, containing the title
 Returns      : html code
 Args         : none
 Globals      : none

=cut

sub _GetMainTitle
{
    my $self = shift;
    return '<div id="ID_TITLE">Circos</div>';
}


=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license - www.cecill.info

=cut

1;
