#
# 	emmanuel.courcelle@toulouse.inra.fr
#	Created: december 05, 2008
#	$Id: WebBuilderFactory.pm 2317 2009-01-22 14:23:43Z sletort $
#

package NcDisplayFam;

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod
=head1 NAME

 NcDisplayFam.pm

=head1 SYNOPSIS

 NcDisplayFam derives from NCScript
 
=head1 DESCRIPTION

=head2 ComputeFam

 NcDisplayFam computes the protein family from the protein whose locus tag is read from the parameter ProteinFam
 Then, the browser is redirected to DisplayFam

=cut

use strict;
use warnings;

use LipmError;
use LipmError::ObjectException;

use Data::Dumper;
use File::Basename;
use Error qw( :try );

use General;
use WebBuilder;
use ParamParser;
use cgilib::NcScript;

use base qw( NcScript );

=head2 function _Init

 Title        : _Init
 Fonction     : the constructor
 Args         : $o_io         : the Instance Observer
                $o_web_builder: A valid WebBuilder object

=cut
sub _Init
{
    my $self = shift;
    
    my ($o_io,$o_wb,$o_logger) = (@_);
    $self->SUPER::_Init($o_io,$o_wb,$o_logger,&FALSE);
    
    $o_wb->UnsetBehaviour('send_parameters');
}

=head2 Function GetHtml

 Title    : Gethtml
 Usage    : $html = $o_ncs->GetHtml()
 Function : Generate and return the html
 Returns  : The html generated
 Args     : none

=cut

sub GetHtml
{
    my $self = shift;

	my $o_param = $self->_Get('_o_param');
    my $o_io    = $self->_Get('_o_io');

     # TODO - FINALISER CE CODE
     my $sessdir     = $o_param->Get('__wb_session');
     my $species     = $o_param->Get('species');
     my $species_short = $o_io->GetSpeciesShortName( $species );
     my $protein_fam = $o_param->Get('ProteinFam');
     my $cutoff      = $o_param->Get('Cutoff');

     # je ne vois pas pourquoi
     $protein_fam    =~ s/[^:]+:/$species:/ if ( $species ne "");
     my $dir_db_family  = $o_io->GetCfg('ROOT_CACHE') .'/' . $o_io->GetCfg('PROT_FAMILY') . '/' . $cutoff;
     my $msf_file    = $dir_db_family.'/'.$protein_fam.'.msf';

     # CREATE AND CHANGE TO A NEW DIRECTORY, BECAUSE THERE IS A NAME CONFLICT
     # ==> The file <gene_name> is used by DisplayFam
     # ==> The SAME file is used by the request ncs_GetProteinsFromGenesLocus_tags.pl, as it tries to read the gene names from this file if it exists

     mkdir $o_io->GetCfg('PROT_FAMILY');
     chdir $o_io->GetCfg('PROT_FAMILY');

     if (! -e $msf_file)
     {
         # Extract the protein to the current directory
         my $prot_file = $protein_fam.'.fasta';

         # bidouille pour palier le pb du fichier portant le nom du locus
         unlink( $protein_fam );

         $self->_GetSequenceLocusTag('Protein',$species,$protein_fam,$prot_file);
         
         if ( !-e $prot_file || -z $prot_file )
         {
             my $err_file = $prot_file; 
             $err_file =~ s/\.fasta$/\.err/;
             my $err_msg = "No protein $protein_fam found !";
             $err_msg .= '<br />' . $self->_IncludeFromTmpl( $err_file ) if ( ! -z $err_file );
             &NARCISSE_utils::ShowErrorPage("Data error", $err_msg );
             return;
         }
         
         my $prot_file_tmp    = $prot_file . '.tmp';
         
         my $fh_prot_file     = new IO::File( "$prot_file" )      or die "Cannot open $prot_file     for reading";
         my $fh_prot_file_tmp = new IO::File( "$prot_file_tmp", 'w' ) or die "Cannot open $prot_file_tmp for writing";
         while  ( my $line = <$fh_prot_file> )
         {
             if ( $line =~ /^>/)
             {
                 my ($len) = ($line=~/len=(\d+)/);
                 $line = ">$species_short:$protein_fam  len=$len  sp=$species \n";
             }
             print $fh_prot_file_tmp $line;
         }
         
         $fh_prot_file_tmp = "";
         $fh_prot_file     = "";
         unlink($prot_file);
         link($prot_file_tmp, $prot_file);
         unlink($prot_file_tmp);
         
         # Call blastp
         print "<p>Now calling blast to build the family</p>\n";
         my $path_blast    = $o_io->GetCfg('ROOT_BLASTALL');
         my $path_blastmat = $o_io->GetCfg('ROOT_BLASTMAT');
         my $blast2cluster = $o_io->GetCfg('ROOT_BLAST2CLUSTER');
         
         my $db_merge       = $o_io->GetCfg('ROOT_DATA_DB') . '/' .$o_io->GetCfg('DB_PROT_MERGE');
         my $blastp_out     = "blastp.out";
         
         my $o_gap  = "9";
         my $e_gap  = "2";
         my $l_db   = "30000000";
         my $evalue = "1e-6";
         my $blastp_cmd = "export BLASTMAT=$path_blastmat ; $path_blast -p blastp -d $db_merge -i $prot_file -m 9 -G $o_gap -E $e_gap -e $evalue -z $l_db -o $blastp_out";
         $self->_Trace("cmd $blastp_cmd\n","","MV");
         $self->_Trace("Running blastp\n","*","M");
         
         system ($blastp_cmd);
         &NARCISSE_utils::ShowErrorPage("Blast Error!") if ( ! -e $blastp_out || -z $blastp_out );
         
         # Call blast2cluster
         my $blast2cluster_cmd = "$blast2cluster  --multalin --collect $blastp_out --format m9 --fam $protein_fam --dir $dir_db_family --pcq $cutoff --pcs $cutoff --seq $db_merge --evalue $evalue";
         $self->_Trace("cmd $blast2cluster_cmd\n","","MV");
         $self->_Trace("Running blast2cluster\n","*","M");
         `$blast2cluster_cmd`;
     } # $msf_file doit exister ou le prog doit s'arreter
     
     # Si le fichier msf n'existe toujours pas,
     #	il ne doit pas y avoir de donnees
     #	on affiche un message d'erreur
     if( !-e $msf_file )
     {
         my $msg = "No data found with those parameters.";
         print "<p class=\"Error\">ERROR: $msg</p>\n";
     }
     else
     {
         # Redirecting to displayfam
         my $displayfam    = $o_io->GetCfg('HTTP_DISPLAYFAM');
         
         my @a_params = ( "__wb_session=$sessdir", "id_fam=$protein_fam" );
         push( @a_params, "Cutoff=$cutoff", "msf=${protein_fam}.msf" );
         push( @a_params, "wanted=alignment" );
         my $redirect_url = "$displayfam?" . join( '&', @a_params );
         
         $self->_Trace( "$redirect_url\n", '.', 'MV');
         print "<script>window.location.href = '$redirect_url';</script>";
     }
     
     # Back to session directory
     chdir '..';
     
     return;
};

1;
