#
# 	emmanuel.courcelle@toulouse.inra.fr
#	Created: december 05, 2008
#	$Id: WebBuilderFactory.pm 2317 2009-01-22 14:23:43Z sletort $
#

package NcGlintFish;

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod
=head1 NAME

 NcGlintFish.pm

=head1 SYNOPSIS

=head1 DESCRIPTION

 NcGlintFish derives from NCScript and is used to launch GlintFish
 (parameters GeneSeq, ProteinSeq, CdsSeq, GenomicSeq)
 If the SaveData parameter is != 0, we just propose to download data
 Else, we send a real html page

=cut

use strict;
use warnings;
#~use Email::Valid;
use Mail::Sendmail;

use LipmError;
use LipmError::ObjectException;
use LipmError::IOException;


use Data::Dumper;
use File::Basename;
use Error qw( :try );

#use Region;
use GlintFish;
use Probe;

use General;
use WebBuilder;
use ParamParser;
use cgilib::NcScript;

use base qw( NcScript );

=head2 function _Init

 Title        : _Init
 Fonction     : the constructor
 Args         : $o_io         : the instance observer
                $o_web_builder: A valid WebBuilder object

=cut
sub _Init
{
	my $self = shift;

	my ($o_io,$o_wb,$o_logger) = (@_);
	$self->SUPER::_Init($o_io,$o_wb,$o_logger,&FALSE);

	my $o_param = $self->_Get('_o_param');	
	$o_wb->UnsetBehaviour('send_parameters');

	$self->_Trace( "Hello\n" , '.', 'GS' );

	return;
}

=head2 Function GetHtml

 Title    : Gethtml
 Usage    : $html = $o_ncs->GetHtml()
 Function : Generate and return the html, calling a function 
            The function called depends on the parameter
 Returns  : The html generated
 Args     : none

=cut

sub GetHtml
{
    my $self = shift;
		
	my $o_param   = $self->_Get('_o_param');
	#~warn "print" .$o_param -> Print('html') ;
	my $display_info = $o_param -> Get ('reset_display');
	if ( $display_info eq "NO")
	{
		return $self->__LaunchGlintFish();
	}
	else
	{
		return $self -> __ResetProbeDisplay();
	}
	
}

=head2 __ResetProbeDisplay

 Title     : __ResetProbeDisplay
 Usage     : $o_xxx->__ResetProbeDisplay
 Function  : Rest the display of glintFish probes
 Args      : none
 Return    : an html string
=cut

sub __ResetProbeDisplay
{	
	my $self = shift;
	my $o_param   = $self->_Get('_o_param');
	my $o_wb    = $self->_Get('_o_wb');
	my $o_instanceObserver = $self -> _Get( '_o_io' );
	my $o_logger= $self->_Get('_o_logger');
	my ($species, $chrom, $start, $end)= $o_param -> Get(qw (species chrom start end)); 
	my ($primer_size, $tm_value,$probe_max_gc)= $o_param -> Get(qw (primer_size tm_value probe_max_gc));
	my $o_speciesObserver = $o_instanceObserver ->	GetSpeciesObserver( $species );	
	my @a_headers = qw( chrom element start end color label );
	
	# Modify the range of the display: To all chromosome 
	my ($dft_start, $dft_end) = $o_param -> Get(qw (start end));
	$o_param -> Set('range',"$dft_start:$dft_end");
	$o_param -> Set ( 'ck_personal','1');
	$self -> _SaveParamParser();
	
	my $file_by_chrom = $o_speciesObserver->GetPersoFiles( -check => &FALSE, -chrom => $chrom);
	my $o_ndw         = New NarcisseDataWriter::OnMemory( $file_by_chrom, \@a_headers, { append => &FALSE }, undef, $o_logger );

	#Afficher à nouveau le formulaire
	my $file   = File::Spec->catfile( $o_instanceObserver->GetCfg( 'ROOT_INC' ), 'launchglintfish.inc' );
	#Here Get default values for form
   	my $rh_cfg = $o_instanceObserver -> GetConfigHash();
   	
	my $html   = $self->_IncludeFromTmpl( $file, 
										species => $species, 
										chrom => $chrom, 
										start => $start,
										end => $end,  
										probe_size => $rh_cfg -> { GLINTFISH_PRIMER3_PRIMER_PRODUCT_SIZE_RANGE },
										primer_size => $primer_size,
										max_primer_size => $rh_cfg -> { GLINTFISH_PRIMER3_PRIMER_MAX_SIZE}, 
										min_primer_size => $rh_cfg -> { GLINTFISH_PRIMER3_PRIMER_MIN_SIZE},
										tm_value => $tm_value,
										probe_max_gc => $probe_max_gc, 
										max_tm_value => $rh_cfg -> { GLINTFISH_PRIMER3_PRIMER_MAX_TM },
										min_tm_value => $rh_cfg -> { GLINTFISH_PRIMER3_PRIMER_MIN_TM },
										'__wb_session' => $o_wb-> GetSession() );
	
	return $html;
	
}

=head2 __LaunchGlintFish

 Title     : __LaunchGlintFish
 Usage     : $o_xxx->__LaunchGlintFish()
 Function  : Launch the whole GlintFish process (selection of probes)
 Args      : none
 Return    : an html string
=cut

sub __LaunchGlintFish
{	
	my $self = shift;
	
	#The objects defined by Narcisse API	
	my $o_param = $self->_Get('_o_param');
	my $o_wb    = $self->_Get('_o_wb');
	
	my ( $species, $chrom, $start, $end ) = $o_param -> Get(('species', 'chrom', 'start', 'end'));
	my ( $probe_size, $probe_max_gc, $primer_size, $tm_value ) = $o_param -> Get(('probe_size', 'probe_max_gc','primer_size', 'tm_value'));
	my $user_email = $o_param -> Get ('email'); 
	my $check_mail = $self -> __CheckEmail($user_email);
	my $validation_message = "";
	
	if ($check_mail == 0)
	{	
		$validation_message = "<br><b><font color=\"#FF0000\">Invalid email address: </font></b>No email has been sent</b></br>";	
	}
	else
	{
		$validation_message = "<br><b><font color=\"#00FF00\">Valid email address. </font></b></br>An email has been sent to $user_email<br>";
	}
	
	my $o_instanceObserver = $self -> _Get( '_o_io' );		
	my $o_speciesObserver  = $o_instanceObserver -> GetSpeciesObserver( $species );	
	my $shortname = $o_speciesObserver->GetShortName();
	
	#New GlintFish
	my $o_probeSelection = New GlintFish(	chr          => $chrom, 
										    start        => $start, 
										    end          => $end,
										    species      => $shortname,
										    -web         => &TRUE,
										    -no_clean    => &TRUE,
										    instanceObs  => $o_instanceObserver,
										    -param       => { 	verbose => 1, 
																GLINTFISH_PRIMER3_PRIMER_PRODUCT_SIZE_RANGE => $probe_size,
																GLINTFISH_PRIMER3_PRIMER_OPT_SIZE => $primer_size, 
																GLINTFISH_PRIMER3_PRIMER_OPT_TM => $tm_value,
																GLINTFISH_PROBE_MAX_GC_PERCENT => $probe_max_gc}
								          );	
	
	#Checking parameter errors and display potential errors
	my $update_message = $o_probeSelection -> DisplayErrors();
	
	if ($update_message ne "000")
	{
		#There is an error in parameters
		my $file   = File::Spec->catfile( $o_instanceObserver->GetCfg( 'ROOT_INC' ), 'glintfish.inc' );
		my $html = $self->_IncludeFromTmpl($file, 
										species => $species, 
										chrom => $chrom, 
										start => $start, 
										end => $end,
										probe_size => $probe_size,
										primer_size => $primer_size,
										tm_value => $tm_value,
										probe_max_gc => $probe_max_gc,
										'__wb_session' => $o_wb->GetSession() );
	
		#Write probe results in the html format
		$html .= $validation_message;
		$html .= $update_message;
		$html .= "<h2> Selected probes = NONE</h2>";
		$self -> __SendEmail(undef, undef, $update_message) if ($check_mail == 1);
		
		return $html;
	}
	
	#Run GlintFish
	$o_probeSelection->Run();
	
	#Computing the region probes
	my $ra_probes = $o_probeSelection->GetSelectedProbesHash();
	
	#Reformat the probes into an array
	my $html_array = __FormatArrayHtmlProbes( $ra_probes );
	
	#Write probe results in the text file
	my $probe_file = $self->CalcTsvNames( 1, "probes" );
	$self -> __WriteProbeFile( $probe_file, $ra_probes ); 
	
	#Write log in a text file
	my %h_parameters = (species => $species, 
						chrom => $chrom, 
						start => $start, 
						end => $end,
						probe_size => $probe_size,
						primer_size => $primer_size,
						tm_value => $tm_value,
						probe_max_gc => $probe_max_gc
						);
						
	$self -> __WriteLogInProbeFile($probe_file, \%h_parameters);
	
	my $file   = File::Spec->catfile( $o_instanceObserver->GetCfg( 'ROOT_INC' ), 'glintfish.inc' );
	my $html   = $self->_IncludeFromTmpl($file, 
										species => $species, 
										chrom => $chrom, 
										start => $start, 
										end => $end,
										probe_size => $probe_size,
										primer_size => $primer_size,
										tm_value => $tm_value,
										probe_max_gc => $probe_max_gc,
										'__wb_session' => $o_wb->GetSession() );
	
	#Write probe results in the html format
	$html .= $validation_message;
	$html .= "<h2> Selected probes </h2>";
	$html .= $html_array;
		
	my $http_probe_file = $self->CalcTsvNames( 0, "probes" ); 	
	$html .= "<h2>Download files</h2>";
	$html .= "<ul> ";
	$html .= "<li> <a href=\"$http_probe_file\" >Probes</a></li></ul>\n";
	
	# Modify the range of the display: set range to be the start and end provided by the GlintFish form 
	$o_param -> Set('range',"$start:$end");
	
	# Finally write probe results in the session dir as personal data
	# Force the parameter ck_personal, to display personal data
	$o_param -> Set ( 'ck_personal','1');
	$self -> _SaveParamParser();
	$self->__CreateNarcissePrivateInput( $species, $chrom, $ra_probes );
	
	#Find the url of the probe display
	
	my $display_url = $self -> __GetDisplayUrl();
	
	#Send Email if provided

	$self -> __SendEmail($http_probe_file, $display_url, $update_message) if ($check_mail == 1);
	
	return $html;
}

=head2 ___ViewGlintFish

 Title     : ___ViewGlintFish
 Usage     : $self->___ViewGlintFish()
 Function  : THIS IS A DEMO  FUNCTION
             You may use this code as a template to write your own functions
 Args      : none
 Return    : The html code
 Errors	   : IOException
 Globals   : none
=cut

sub __ViewGlintFish
{
	my $self = shift;
	print "Application started ";
	print time();
	
	#The objects defined by Narcisse API
	my $o_instanceObserver = $self->_Get( '_o_io' );
	my $o_param = $self->_Get('_o_param');
	my $o_logger= $self->_Get('_o_logger');
	
	# The parameter GlintFish describes the region as follows: species:chrom:start:end
	my $region  = $o_param -> Get ( 'GlintFish' );
	my ( $species, $chrom, $start, $end) = split( ':',$region );
	
	#warn "$species, $chrom, $start, $end \n";

	#Retrieving the region probes
	my $ra_data = $self->__GlintFishProbes( $species, $chrom, $start, $end );
								
	#Reformat the probes into an array
	my $ra_selprobes = __FormatArrayProbes( $ra_data );
	
	#Write probe results in the text file
	my $probe_file = $self->CalcCsvNames( 1, "probes" );
	__WriteProbeFile( $probe_file, $ra_data ); 
	
	my $file   = File::Spec->catfile( $o_instanceObserver->GetCfg( 'ROOT_INC' ), 'glintfish.inc' );
	my $html   = $self->_IncludeFromTmpl($file, species => $species, start => $start, end => $end );
	
	#Write probe results in the html format
	$html .= "<h2> Selected probes </h2>";
	$html .= &NARCISSE_utils::TabArray2Html( $ra_selprobes );
		
	my $http_probe_file = $self->CalcCsvNames( 0, "probes" ); 	
	$html .= "<h2>Download files</h2>";
	$html .= "<ul> ";
	$html .= "<li> <a href=\"$http_probe_file\" >Probes</a></li>\n";
	
	# Finally write probe results in the session dir as personal data
	# Force the parameter ck_personal, to display personal data
	$o_param -> Set ( 'ck_personal','1');
	$self -> _SaveParamParser();
	$self->__CreateNarcissePrivateInput( $species, $chrom, $ra_data );
		
	return $html;
}


=head2 __IsFishProbeFishAvailable

 Title     : __IsFishProbeFishAvailable
 Usage     : $o_xxx->__IsFishProbeFishAvailable
 Function  : Check the probe repository data_path/probe
				return true unless inexistent of empty directory
 Args      : none
 Return    : TRUE or FALSE
=cut
sub __IsFishProbeFishAvailable
{	
	my $self = shift;
	
	my $o_param = $self->_Get('_o_param');
	my $region  = $o_param -> Get ( 'GlintFish' );
	my ( $species, $chrom, $start, $end) = split( ':',$region );
		
	#The objects defined by Narcisse API
	my $o_instanceObserver = $self->_Get( '_o_io' );		
	my $o_speciesObserver  = $o_instanceObserver->GetSpeciesObserver( $species );	
	my $data_path = $o_speciesObserver->GetDataPath( 'INSTANCE' );		
	my $probe_path = $data_path."/"."probe" ;
	
	my $bool_IsAvailable;	
	if ( -s $probe_path )
	{
		$bool_IsAvailable = &TRUE;
	}
	else
	{
		$bool_IsAvailable = &FALSE;	
	}
	
	return $bool_IsAvailable;
}


=head2 __RemoveFile

 Title     : __RemoveFile
 Usage     : __RemoveFile( $file)
 Function  : Remove file
 Args      : file to remove
 Return    : none

=cut

sub __RemoveFile
{	
	my $self = shift;
	my ( $file2remove)  = @_;
	unlink $file2remove or die "Can't delete $file2remove: $!";
	return;
}

=head2 __WriteProbeFile

 Title     : __WriteProbeFile
 Usage     : __WriteProbeFile( $file, $ra_data )
 Function  : Dump the data array into a file
 Args      : output file and a reference on an array
 Return    : none

=cut
sub __WriteProbeFile
{	
	my $self = shift;
	my ( $output, $ra_probes )  = @_;
		
	my $fh = new IO::File( 	$output, "w" )
		or die "Impossible to open $output for writing";	
		
	my @a_fields = qw( element chrom start end size leftPrimer rightPrimer leftTm rightTm leftEndStability rightEndStability leftSelfEnd rightSelfEnd leftSelfAny rightSelfAny PairPenality PairComplAny);
	print $fh "num"."\t".join("\t",@a_fields)."\n";
	my $num = 1;
	foreach my $r_probe ( @$ra_probes )
	{	
		print $fh $num."\t".join("\t", @{$r_probe}{ @a_fields })."\n";
		$num++;	
	}
	$fh->close();
}

=head2 __WriteLogInProbeFile

 Title     : __WriteLogInProbeFile
 Usage     : __WriteLogInProbeFile( $file, $ra_data )
 Function  : Dump the parameters &NARCISSE_utils::GetTime();setting into a file
 Args      : output file
 Return    : none

=cut
sub __WriteLogInProbeFile
{	
	my $self = shift;
	my ( $output, $rh_parameters)  = @_;
	my $time_info = &NARCISSE_utils::GetTime();
	
	my $fh = new IO::File( 	$output, "a" )
		or die "Impossible to open $output for appending";
		
	print $fh "\n\n\n\n#GlintFish-log\n";
	print $fh "Time:\t $time_info\n";
	
	my @a_parameters = qw( species chrom start end probe_size probe_max_gc primer_size tm_value);
	
	foreach my $parameter (@a_parameters)
	{
		my $parameterName = "\U$parameter";
		$parameterName =~ s/\_/ /g;	
		print $fh "$parameterName:\t$rh_parameters->{$parameter}\n";
	}
	print $fh "#End-of-log\n";
	$fh->close();	
}

=head2 __GlintFishProbes

 Title     : __GlintFishProbes
 Usage     : $o_xxx->__GlintFishProbes( $species, $chrom, $start, $end  )
 Function  : Compute the file path and call _GetSelectedProbes
 Args      : reference on an array with the probes
 Return    : none

=cut
sub __GlintFishProbes
{
	my $self = shift;
	my ( $species, $chrom, $start, $end) = @_;
	
	#The objects defined by Narcisse API
	my $o_instanceObserver = $self->_Get( '_o_io' );		
	my $o_speciesObserver  = $o_instanceObserver->GetSpeciesObserver( $species );	
	my $data_path = $o_speciesObserver->GetDataPath( 'INSTANCE' );
		
	my $probe_file = $data_path."/"."probe"."/"."chromosome$chrom.probe";
				
	my $ra_data = _GetSelectedProbes(  $probe_file, $start, $end );	
		
	return $ra_data;
}

=head2 _GetSelectedProbes

 Title     : _GetSelectedProbes
 Usage     : _GetSelectedProbes(  file => $probe_file, start => $start, end => $end );
 Function  : Retrieve from a file in Narcisse format the probes corresponding to the specified region 
 Args      : file, start and end
 Return    : reference on an array of hash
=cut

sub _GetSelectedProbes
{
	my ( $file, $start, $end ) = @_;
	
	my @a_data = ();	
	my $o_data = &NARCISSE_utils::ReadChromFeatureData( file => $file, start => $start, end => $end );	
	my @a_fields = qw( chrom start end element leftPrimer rightPrimer size leftTm rightTm leftEndStability rightEndStability leftSelfEnd rightSelfEnd leftSelfAny rightSelfAny PairPenality	PairComplAny);
	foreach my $element_id ( $o_data->GetSortedElements() )
	{
		my %h_probe;
		@h_probe{ @a_fields } = $o_data->GetFields( $element_id, @a_fields );
		push @a_data, \%h_probe;			
	}
	
	return \@a_data;
}

=head2 __FormatArrayProbes

 Title     : __FormatArrayProbes
 Usage     : __FormatArrayProbes( $ra_probes )
 Function  : Reformat the array of probes for printing (html and txt)
 Args      : reference on an array of probes
 Return    : reference on an array of string
=cut

sub __FormatArrayProbes
{
	my $ra_probes = shift;	
	
	my @a_data = ();
	my @a_fields = qw(element leftPrimer rightPrimer chrom start end size);
	
	push @a_data, "#"."num"."\t".join("\t",@a_fields)."\n";
	my $num = 1;
	foreach my $r_probe ( @$ra_probes )
	{	
		push @a_data, $num."\t".join("\t", @{$r_probe}{ @a_fields })."\n";
		$num++;	
	}
	return \@a_data;
}

=head2 __FormatArrayHtmlProbes

 Title     : __FormatArrayHtmlProbes
 Usage     : __FormatArrayHtmlProbes( $ra_probes )
 Function  : Reformat the array of probes for printing (html and txt)
 Args      : reference on an array of probes
 Return    : reference on an array of string
=cut

sub __FormatArrayHtmlProbes
{
	my $ra_probes = shift;	
	
	my @a_data = ();
	my @a_headers = qw(probe element leftPrimer rightPrimer chrom start end size);
	
	# The headers
    my $html = "<table class=\"TableData\">\n<tr><th>";
    $html .= join("</th><th>",@a_headers);
    $html .= "</th></tr>\n";
	my $num = 1;
	
	# The data
    my $p=0;
    my @a_parity=('odd','even');
    
	foreach my $r_probe ( @$ra_probes )
	{	
		$html .= "<tr class=\"$a_parity[$p]\"><td>";
		$html .= "<font size = \"3\">$num</font>";
        $html .= join("</td><td><font face=\"monospace\"><font size = \"4\">",@{$r_probe}{ @a_headers });
        $html .= "</font></td></tr>\n";
        $p = 1 - $p;
		$num++;	
	}
    # The trailer
    $html .= "</table>\n";

    return $html;
}
=head2 __CreateNarcissePrivateInput

 Title        : __CreateNarcissePrivateInput
 Usage        : $o_xxx->__CreateNarcissePrivateInput(  )
 Function     : Create a file, in the private hierarchy, corresponding to the csv data file format used as an input for Narcisse 
 Returns      : void
 Args         : species name, chrom and reference on an array of probes
=cut

sub __CreateNarcissePrivateInput
{
	my $self = shift;
	my ( $species, $chr, $ra_probes ) = @_;
	
	my $o_instanceObserver = $self->_Get( '_o_io' );		
	my $o_speciesObserver  = $o_instanceObserver->GetSpeciesObserver( $species );			
	my $o_logger= $self->_Get('_o_logger');
	
	my $rh_probes = __FormatProbeOutputForNarcisse( $ra_probes );
     
	my @a_headers = qw( chrom element start end color label );
	my $file_by_chrom = $o_speciesObserver->GetPersoFiles( -check => &FALSE, -chrom => $chr);
	my $o_ndw         = New NarcisseDataWriter::OnMemory( $file_by_chrom, \@a_headers, { append => &TRUE }, undef, $o_logger );
	$o_ndw -> Update($rh_probes);
}

=head2 __FormatProbeOutputForNarcisse

 Title        : __FormatProbeOutputForNarcisse
 Usage        : __FormatProbeOutputForNarcisse(  )
 Function     : Reformat the array of probes in a format suitable for NarcisseDataWriter
 Returns      : a reference on a hash with the probes
 Args         : reference on an array of probes
=cut

sub __FormatProbeOutputForNarcisse
{	
   my $ra_probes  = shift;		
   
   my %h_probes = ();
   my $id = 1;
   my @a_fields = qw( chrom element start end );
   foreach my $r_probe ( @$ra_probes )
   {
	   my ( $chr, $element, $start, $end ) = @{$r_probe}{@a_fields};
	   $h_probes{ $element } ={ chrom => $chr, element => $element, start => $start, end => $end, color => "green", label => "probe$id"};
	   $id++;
   } 
	return \%h_probes;	
}


=head2 __CheckEmail
 Title        : __CheckEmail
 Usage        : __CheckEmail( $user_email )
 Function     : Check e-mail validity
 Returns      : error message if unvalid
 Args         : $user_email
=cut

sub __CheckEmail
{	
	my $self = shift;
	my ($user_email) = (@_);
	
	#It is a very simpl check : something@somewhere.com
	#Should add something for white spaces
	
	if ($user_email =~ /^.+\@.+\.[a-z]{2,3}$/)
	{
		return 1;
	}
	else 
	{
		return 0;
	}
}

=head2 __SendEmail

 Title        : __SendEmail
 Usage        : __SendEmail(  )
 Function     : Send e-mail when GlintFish is over
 Returns      : Ok the e-mail was sent
 Args         : $o_param for the e-mail adress
=cut

sub __SendEmail
{	
	my $self = shift;
	
	my ($probe_link, $display_link, $success_tag) = (@_);
	my $o_param = $self -> _Get('_o_param');
	my $user_email = $o_param -> Get('email');
	my $o_instanceObserver = $self->_Get( '_o_io' );
	my $unix_mail = $o_instanceObserver -> GetUnixPath('UNIX_MAIL');
	my $subject = "Narcisse - GlintFish";
	my $narcisse_msg_file = "/tmp/narcisse_GlintFish_email.txt";
	my $f = new IO::File($narcisse_msg_file,"w") or Carp::croak("Cannot open $narcisse_msg_file");
	my $narcisse_msg ="";
	
	if ($success_tag eq "000")
	{
		$narcisse_msg = "Hello dear Narcisse user.\n\nYour job is now finished and your results are available at $probe_link.\n\n";
		$narcisse_msg .= "They are displayed on selected chromosome at $display_link.\n\n";
		$narcisse_msg .= "We thank you for using Narcisse - GlintFish.\n\n";
		$narcisse_msg .= "***WARNING: Results are available for 24 hours. Please get them back before automatic cleaning.***\n";
		$narcisse_msg .= "\n\n ---The Narcisse team - http://narcisse.toulouse.inra.fr/ ---\n";	
	}
	else
	{
		#Error in parameters
		$success_tag =~ s/<b>//g;
		$success_tag =~ s/<\/b>//g;
		$success_tag =~ s/<br>//g;
		$success_tag =~ s/<\/br>/\n/g;
		$narcisse_msg = "Hello dear Narcisse user.\n\nYour job failed due to parameter error: \n\n";
		$narcisse_msg .= "$success_tag\n\n***Please try again later.***\n\n";
		$narcisse_msg .= "We thank you for using Narcisse - GlintFish.\n\n";
		$narcisse_msg .= "\n\n ---The Narcisse team - http://narcisse.toulouse.inra.fr/ ---\n";
	}
	
	print $f "$narcisse_msg\n";
	$f->close;
	my $cde_mail = "$unix_mail -s \"$subject\" $user_email < $narcisse_msg_file";
	system ($cde_mail);
	unlink("$narcisse_msg_file");
	
}

=head2 __MyComputation

 Title     : __MyComputation
 Usage     : $self->__MyComputation()
 Function  : THIS IS A DEMO  FUNCTION
             You may use this code as a template to write your own functions
 Args      : none
 Return    : The html code
 Errors	   : IOException
 Globals   : none

=cut
sub __MyComputation
{
	my $self = shift;
	
	# The objects defined by the Narcisse API
	my $o_io    = $self->_Get('_o_io');
	my $o_param = $self->_Get('_o_param');
	my $o_logger= $self->_Get('_o_logger');
	
	# The parameter MyComputation describes the region as follows: species:chrom:start:end
	my $region  = $o_param -> Get ( 'MyComputation' );
	my ( $species, $chr, $start, $end) = split( ':',$region );
	my $o_so    = $o_io -> GetSpeciesObserver( $species );

	# This is a rather stupid and useless treatement: the region is separated into 10 subregions,
	# put in a hash of hashes called %h_data.
	# In a REAL code, the treatment should be put in an external module.
    my %h_data;
	my $s = $start;
	my $l = int(($end - $start) / 10);
    for ( my $i=0; $i<10; $i++ )
    {
		my $elt = 'SEG' . $i;
		my %h_row;
		$h_row { 'chrom' }   = $chr;
		$h_row { 'element' } = $elt;
		$h_row { 'start' }   = $s;
		$h_row { 'end' }     = $s + $l - 1;
		$h_row { 'color' }   = ($i % 2 ) ? 'red' : 'blue';
		$h_row { 'label' }   = $elt;
		$h_data { $elt } = \%h_row;
		$s += $l;
	}
	
    # We are using the personal data functionality to display our treatment results to all the Narcisse screens
    # The first and only step is to create or append personal file.
    my @a_headers = qw( chrom element start end color label );
	my $file_by_chrom = $o_so->GetPersoFiles( -check => &FALSE, -chrom => $chr);
	my $o_ndw         = New NarcisseDataWriter::OnMemory( $file_by_chrom, \@a_headers, { append => &FALSE }, undef, $o_logger );
	$o_ndw -> Update( \%h_data );

	# Force the parameter ck_personal, to display personal data
	$o_param -> Set ( 'ck_personal','1');
	$self -> _SaveParamParser();
	
	# Do not forget to write something to the popup. Here, we only display the first and last segments.
	# It is very easy to use the _IncludeFromTmpl function found in NcScript.pm
	my $file   = File::Spec->catfile( $o_io->GetCfg( 'ROOT_INC' ), 'my_comput_reg.inc' );
	my $html   = $self->_IncludeFromTmpl($file, 
										s_seg0 => $h_data{'SEG0'}->{start}, 
										e_seg0 => $h_data{'SEG0'}->{end},
										s_seg9 => $h_data{'SEG9'}->{start}, 
										e_seg9 => $h_data{'SEG9'}->{end}
										);
										
	return $html;
}

=head2 __DisableChromosomeChange

 Title     : __DisableChromosomeChange
 Usage     : $self->__DisableChromosomeChange
 Function  : Block the change of chromosome in GlintFish form.
			 Change of chromosome should be done elsewhere using Narcisse interface
             
 Args      : $chrom entered from parameters

=cut
sub __DisableChromosomeChange
{
	my $self = shift;
	my ($param_chrom) = (@_);
	
	#Getting reference chromosome
	my $rh_organisms = $self->_Get('_rh_organisms');
	my $reference_chrom = $rh_organisms -> {CHROMS} ->[0];
	
	#Error message
	my $html_error_on_chr = "<br><b><font color=\"#FF0000\">Changing reference chromosome while using GlintFish form is not available. </font></b></br>";
	$html_error_on_chr .= "<br><b>Please use the Narcisse Interface to select a new reference chromosome or try again with chromosome: $reference_chrom</b></br>";
	
	if ($reference_chrom ne $param_chrom)
	{
		return $html_error_on_chr;
	}	
}


=head2 _GetImagesSuffix

 Title        : _GetImagesSuffix
 Usage        : my $suffix = $self->__GetImagesSuffix()
 Fonction     : Get the suffix used to save images (depends on paramters)
                see NcScript::_GetImagesSuffix
                This function is overloaded from NcScript::_GetImagesSuffix in order to simplify the file name				
 Args         : none
 Return       : $suffix => a string
 Globals      : none

=cut 
sub _GetImagesSuffix
{
    my $self = shift;

	my $o_param = $self ->_Get('_o_param');

	# Set the suffix :
    my $rh_organisms = $self->_Get('_rh_organisms');
    my $ra_species   = $rh_organisms->{ SPECIES };
    my $ra_chroms    = $rh_organisms->{ CHROMS };
    my $ra_programs  = $rh_organisms->{ PROGRAMS };

	my $suffix = '_';
    $suffix .= $ra_species->[0] .'_' . (defined ($ra_chroms->[0])?$ra_chroms->[0]:'') ;

    # Add targets info
    my $tgt_nb = $self->GetTargetsNumber();
    for ( my $i=1; $i<=$tgt_nb; $i++ )
    {
    	$suffix .= '_'.$ra_species->[$i] . '_' . $ra_programs->[$i];
    }

    # Add other info
	#$suffix .= '_'.$o_param->Get('level').'_'.$o_param->Get('view_level');
    #$suffix .= '_'.$o_param->Get('range')	if ($o_param->IsDefined('range'));
    #$suffix .= '_'.$o_param->Get('gene') 	if ($o_param->IsDefined('gene'));
	
	if ( $o_param->IsDefined('start') )
	{
		if ( $o_param->IsDefined('end') )
		{
			my $range = $o_param->Get('start').'_'.$o_param->Get('end');
			$suffix .= '_'.$range;
		}
	}
	
	# Remove spaces from url or file names, if any
	$suffix =~ s/ /+/g;

    return $suffix;
}


=head2 __GetDisplayUrl

 Title        : __GetDisplayUrl
 Usage        : my $path_to_display = $self->__GetDisplayUrl
 Fonction     : Get the path to the display of probes on chromosome            			
 Args         : none
 Return       : $path_to_display => a string
 Globals      : none

=cut 
sub __GetDisplayUrl
{
	my $self = shift;
	
	my $o_instanceObserver = $self->_Get( '_o_io' );		
	my $sessdir = $o_instanceObserver-> GetSessionDirectory();
	my $url = $o_instanceObserver-> GetCfg( 'HTTP_NARCISSE_CGI' ) . '?__wb_session='.$sessdir.'&script=DISPLAY&__wba_popup=0';
	
	return $url;
	
}


1;
