#
# 	emmanuel.courcelle@toulouse.inra.fr
#	Created: december 05, 2008
#	$Id$
#

package NcHome;

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod
=head1 NAME

 NcDotplot.pm

=head1 SYNOPSIS

 NcHome derives from NCScript

=head1 DESCRIPTION

 NcHome prints the Narcisse's home page
 
=cut

use strict;
use warnings;

use LipmError;
use LipmError::ObjectException;
use LipmError::IOException;

use Data::Dumper;
use File::Basename;
use Error qw( :try );
use Carp;

use General;
use WebBuilder;
use Runner;
use cgilib::NcScript;
use NARCISSE_utils;

use Configuration::NarcisseObserver;
use Configuration::InstanceObserver;

use Image;
use Planner;
use Dotplot;
use Planner::GenomeMap;

use constant IMG_EXT   => ".png";
use constant COORD_EXT => ".coord";
use constant JS_EXT    => ".js";
use constant SPECIES_NAME_BAND_SIZE => 80;
use constant DEBUG     => 1;

use base qw( NcScript );


=head2 function _Init

 Title        : _Init
 Fonction     : the constructor
 Args         : $o_io         : the configuration object, NarcisseObserver or InstanceObserver
                $o_web_builder: A valid WebBuilder object
                $o_logger     : a Logger

=cut
sub _Init
{
    my $self = shift;
    my ($o_io,$o_web_builder,$o_logger) = (@_);

    if ($o_io->isa('NarcisseObserver'))
    {
        $self->SUPER::_Init($o_io,$o_web_builder);
        $self->_Set('__multi_mode',&TRUE);
    }
    else
    {
        $self->SUPER::_Init($o_io,$o_web_builder,$o_logger,&TRUE);
        $self->_Set('__multi_mode',&FALSE);
    }
    
    #$o_web_builder->UnsetBehaviour('send_parameters');
    my $o_param = $self->_Get('_o_param');
    
    # The 'script' parameter must be removed
    $o_param->Delete('script');
    
    return $self;
}

=head2 Function GetHtml

 Title    : GetHtml
 Usage    : $html = $o_ncs->GetHtml()
 Function : Call __GetHtml or __GetHtmlMulti
 Returns  : The html generated
 Args     : none

=cut
sub GetHtml
{
    my $self = shift;
    if ( $self->_Get('__multi_mode') == &TRUE )
    {
        return $self->__GetHtmlMulti();
    }
    else
    {
		my $o_param = $self -> _Get ( '_o_param' );
		#print '<pre>'.Dumper (\%ENV).'</pre>';
		return ($o_param -> GetInteger('advanced') != 0) ? $self -> __GetHtmlAdvanced() : $self -> __GetHtml();
    }
}

=head2 Function __GetHtml

 Title    : __GetHtml
 Usage    : $html = $o_ncs->__GetHtml()
 Function : Generate the html of the home page, instance specific
 Returns  : The html generated
 Args     : none

=cut

sub __GetHtml
{
	my $self = shift;

	my $o_param = $self->_Get('_o_param');
	my $o_io    = $self->_Get('_o_io');
	my $o_wb    = $self->_Get('_o_wb');

	my ($site,$inc,$cgi) = $o_io->GetCfg('ROOT_SITE','ROOT_INC','HTTP_NARCISSE_CGI');
	
	# display the abstract for this instance, if any
	my $abstract = $self->_IncludeFromTmpl("$site/abstract.inc", warning => 0);

	# display the images of the instances, if any
	my ($ra_img_url,$ra_legend,$ra_alt) = $self -> __GetImgAll($o_io);
	
	my $img_html = "";
	if ( @$ra_img_url != 0 )
	{
		$img_html = '<table><tr>';
		for (my $i=0; $i <= $#$ra_img_url; $i++)
		{
			my $img_url = $ra_img_url -> [ $i ];
			my $alt     = $ra_alt     -> [ $i ];
			$img_html .= "<td class=\"photo\" ><img src=\"$img_url\" alt=\"$alt\" />";
			$img_html .= (defined $ra_legend->[ $i ]) ?  "<br />" . $ra_legend -> [ $i ] . "</td>" : "";
		}
		$img_html .= '</tr></table>';
	}
	
	# Display the instance name	      
	my $instance_name    = General::Capitalize($o_io->GetName());

	# compute the uri for the advanced form
	my $advanced_uri = $cgi . '?script=HOME&amp;__wb_session=' . $o_param->Get('__wb_session') . '&amp;advanced=1';
	
	# display the entry form, very few entry fields
	my $gensel_tiny_form = $self->_IncludeFromTmpl( "$inc/form/gensel_tiny_form.inc",advanced_uri => $advanced_uri );

	
	# compute the uri for the list of available species
	my $available_uri = $cgi . '?__wb_function=PrintRelnotes';
	my $nb_of_species = $o_io->GetNbSpecies();

    my %h_forms  = ( abstract     => $abstract,
					 available_uri=> $available_uri,
					 nb_of_species=> $nb_of_species,
                     img_html     => $img_html,
                     instance_name=> $instance_name,
                     gensel_tiny_form => $gensel_tiny_form );

	return $self->_IncludeFromTmpl("$inc/home.inc",%h_forms);
}

=head2 Function __GetHtmlAdvanced

 Title    : __GetHtmlAdvanced
 Usage    : $html = $o_ncs->__GetHtml()
 Function : Generate the html of the home page, instance specific, advanced query
 Returns  : The html generated
 Args     : none

=cut

sub __GetHtmlAdvanced
{
	my $self = shift;

	my $o_param = $self->_Get('_o_param');
	my $o_io    = $self->_Get('_o_io');
	my $o_wb    = $self->_Get('_o_wb');

	my ($site,$inc,$cgi) = $o_io->GetCfg('ROOT_SITE','ROOT_INC','HTTP_NARCISSE_CGI');
	# let's include a message if any

	# display the abstract for this instance, if any
	my $abstract = $self->_IncludeFromTmpl("$site/abstract.inc", warning => 0);

	# display the entry forms
	my $gensel_form   = $self->_IncludeFromTmpl( "$inc/form/gensel_form.inc" );
	my $access_form   = $self->_IncludeFromTmpl( "$inc/form/access_form.inc" );
	my $upload_form   = $self->_IncludeFromTmpl( "$inc/form/upload_form.inc" );
	my $blast_form	  = $self->_IncludeFromTmpl( "$inc/form/blast_form.inc" );

	# No js object to initialize __wb_session
	my $load_form     = $self->_IncludeFromTmpl("$inc/form/load_form.inc",'__wb_session',$o_wb->GetSession());

 	# compute the uri for the basic form
	my $uri = $cgi . '?script=HOME&amp;__wb_session=' . $o_param->Get('__wb_session') . '&amp;advanced=0';
	my %h_forms  = ( abstract     => $abstract,
                     gensel_form  => $gensel_form,
                     access_form  => $access_form,
                     upload_form  => $upload_form,
                     blast_form   => $blast_form,
                     load_form    => $load_form,
                     basic_uri    => $uri );

	return $self->_IncludeFromTmpl("$inc/home_advanced.inc",%h_forms);
}

=head2 Function __GetHtmlMulti

 Title    : __GetHtmlMulti
 Usage    : $html = $o_ncs->__GetHtmlMulti()
 Function : Generate the html of the home page, multi-instance
 Returns  : The html generated
 Args     : none

=cut

sub __GetHtmlMulti
{
    my $self = shift;
    
	my $o_param   = $self->_Get('_o_param');
    my $o_io    = $self->_Get('_o_io');

    my $home_inc_path  = File::Spec->catfile($o_io->GetCfg('ROOT_INC'),'home_multi.inc');
    
    # The InstanceObserver is in fact a NarcisseObserver (TODO - Call it _o_conf this should be better !)
    my $o_no = $self->_Get('_o_io'); 
    my $rh_instances = $o_no->GetInstanceObservers();

    my $html="";
    my $i = 0;
    my @a_classes = qw( InstTopLeft InstTopRight InstBottomLeft InstBottomRight );
    
    foreach my $key (sort keys %$rh_instances)
    {
        my $o_io = $rh_instances->{ $key };
        
        $html .= "<hr class=\"wb_clear\" />" if ($i != 0 && $i % 2 == 0);

        my $instance_name    = General::Capitalize($o_io->GetName());
        my $instance_version = $o_io->GetVersion();
        my $narcisse_version = $o_io->GetNarcisseVersion();
        my $instance_beta    = ( $narcisse_version =~ /beta/i or $instance_version =~ /beta/i ) ? " <span class=\"beta\">Beta</span>" : "";
        my $instance_url     = $o_io->GetCfg('HTTP_NARCISSE_CGI');
        $instance_url       .= '?__wb_debugging=1' if ($o_param->Get('__wb_debugging') !~ /^0*$/);

        #my $root_pictures = $o_io->GetCfg('ROOT_PICTURES');
        #my $http_pictures = $o_io->GetCfg('HTTP_PICTURES');
        
        #my $alt_path      = File::Spec->catfile($root_pictures,'img_1.alt');
        #my $inc_path      = File::Spec->catfile($root_pictures,'img_1.inc');
        my ($img_url,$legend,$alt) = $self -> __GetImg( $o_io );
        
        #if (-r File::Spec->catfile($root_pictures,'img_1.jpg'))
        #{
            #$img_path = File::Spec->catfile($root_pictures,'img_1.jpg');
            #$img_url  = $http_pictures.'/'.'img_1.jpg';
        #}
        #else
        #{
            #$img_path = File::Spec->catfile($root_pictures,'img_1.png');
            #$img_url  = $http_pictures.'/'.'img_1.png';
        #}

        my $nb_of_species = $o_io->GetNbSpecies();
        
        if ( $img_url ne '' )
        {
            #my ($fh,$alt,$inc);
            #$fh = new IO::File($alt_path) and $alt= <$fh>;
            #chomp($alt);
            #$fh = new IO::File($inc_path) and $inc= <$fh>;
            #chomp($inc);
            $html .= "<table class=\"".$a_classes[$i % 4]."\"><tr><td class=\"photo\"><a href=\"$instance_url\" class=\"instance_link\" ><img alt=\"$alt\" src=\"$img_url\" /></a><br />$legend</td>\n";
            $html .= "<td class=\"instances\"><h2><a href=\"$instance_url\"  class=\"instance_link\" >$instance_name</a>$instance_beta</h2><p>($nb_of_species species)</p></td></tr></table>\n";
        }
        else
        {
            $html .= "<table class=\"".$a_classes[$i % 4]."\"><tr><td class=\"instances\"><h2><a href=\"$instance_url\" class=\"instance_link\">$instance_name</a>$instance_beta</h2><p>($nb_of_species species)</p></td></tr></table>\n";
        }
        $i++;
    } 

    my $page_html = $self->_IncludeFromTmpl($home_inc_path, instances => $html);

    return $page_html;
}

=head2 Function __GetJsonMulti

 Title    : __GetJsonMulti
 Usage    : $html = $o_ncs->__GetHtmlMulti()
 Function : Same as __GethtmlMulti, but in json for rest mode
 Returns  : The json generated
 Args     : none

=cut
sub __GetJsonMulti
{
    my $self = shift;
    
    my $o_no = $self->_Get('_o_io'); 
    my $rh_instances = $o_no->GetInstanceObservers();
    
    my %h_instances;
    for my $key (keys %$rh_instances)
    {
        my %h_inst;
        my $o_io = $rh_instances->{ $key };
        my @a_species = $o_io->GetSpeciesLongName();
        map s/ /_/g,@a_species;
        $h_inst{species}          = \@a_species;
        $h_inst{instance_name}    = $o_io->GetName();
        $h_inst{instance_version} = $o_io->GetVersion();
        $h_inst{narcisse_version} = $o_io->GetNarcisseVersion();
        $h_inst{instance_url}     = $o_io->GetCfg('HTTP_NARCISSE_CGI');
        $h_instances{$key} = \%h_inst;
    }
    
    $self->_ToJson( \%h_instances );
}
        
=head2 Function __GetImg

 Title    : __GetImg
 Usage    : my ($img_url,$inc,$alt) = $self -> __GetImg()
 Function : Return url, legend and alt relative to the image instance
 Returns  : $img_url  ==> url to the image (may be "" if it does not exist) 
            $inc      ==> text of the legend  (may be "")
            $alt      ==> text of the alt tag (should NOT be "" if the url exists)
 Args     : $o_io     ==> An instance observer
 TODO - PERMETTRE D'AVOIR img_2, img_3, etc.

=cut
sub __GetImg
{
	my $self   = shift;
	my ($o_io) = @_;
	my ($root_pictures, $http_pictures) = $o_io->GetCfg('ROOT_PICTURES','HTTP_PICTURES');
	
	my ($img_path,$img_url,$inc,$alt);
	
	$img_path = File::Spec->catfile($root_pictures,'img_1.jpg');
	if (-r $img_path)
	{
		$img_url  = $http_pictures.'/'.'img_1.jpg';
	}
	else
	{
		$img_path = File::Spec->catfile($root_pictures,'img_1.png');
		if ( -r $img_path)
		{
			$img_url  = $http_pictures.'/'.'img_1.png';
		}
		else
		{
			$img_path = "";
		}
	}
	
	if ($img_path ne '')
	{
		my $alt_path      = File::Spec->catfile($root_pictures,'img_1.alt');
		my $inc_path      = File::Spec->catfile($root_pictures,'img_1.inc');
		my $fh;
		$fh = new IO::File($alt_path) and $alt= <$fh>;
		chomp($alt);
		$fh = new IO::File($inc_path) and $inc= <$fh>;
		chomp($inc);
	}
	
	return ($img_url,$inc,$alt);
}	
	
=head2 Function __GetImgAll

 Title    : __GetImgAll
 Usage    : my ($ra_img_url,$ra_inc,$ra_alt) = $self -> __GetImg()
 Function : Return array of url, array of legends and array of alt relative to the image instances
 Returns  : \@a_img_url  ==> url to the image (may be "" if it does not exist) 
            \@a_inc      ==> text of the legend  (may be "")
            \@a_alt      ==> text of the alt tag (should NOT be "" if the url exists)
 Args     : $o_io : An instance observer

=cut
sub __GetImgAll
{
	my $self  = shift;
	my ($o_io)= @_;
	my ($root_pictures, $http_pictures) = $o_io->GetCfg('ROOT_PICTURES','HTTP_PICTURES');
	
	my ($img_path,@a_img_url,@a_inc,@a_alt);
	my $cpt = 1;
	while (&TRUE)
	{
		my $img_path = File::Spec->catfile($root_pictures,"img_$cpt.jpg");
		my $img_url;
		if (-r $img_path)
		{
			$img_url  = $http_pictures.'/'."img_$cpt.jpg";
		}
		else
		{
			$img_path = File::Spec->catfile($root_pictures,"img_$cpt.png");
			if ( -r $img_path)
			{
				$img_url  = $http_pictures.'/'."img_$cpt.png";
			}
			else
			{
				$img_path = "";
				last;
			}
		}

		my $alt_path      = File::Spec->catfile($root_pictures,"img_$cpt.alt");
		my $inc_path      = File::Spec->catfile($root_pictures,"img_$cpt.inc");
		my ($fh,$alt,$inc);
		$fh = new IO::File($alt_path) and $alt= <$fh>;
		$fh = new IO::File($inc_path) and $inc= <$fh>;
		push @a_img_url, $img_url;
		push @a_inc, (defined $inc) ? $inc : '';
		push @a_alt, (defined $alt) ? $alt : '';
		$cpt++;
	}
	chomp(@a_alt);
	chomp(@a_inc);
	return (\@a_img_url,\@a_inc,\@a_alt);
}	
	
=head2 Function __GetJson

 Title    : __GetJson
 Usage    : $html = $o_ncs->__GetHtml()
 Function : Same as __Gethtml, but in json for rest mode
 Returns  : The json generated
 Args     : none

=cut
sub __GetJson
{
    my $self = shift;
	my $o_param = $self->_Get('_o_param');
    
    my $o_io = $self->_Get('_o_io'); 
    my @a_species = $o_io->GetSpeciesLongName();
    
    my %h_species;
    for my $species (@a_species)
    {
        my %h_sp;
        my $o_so = $o_io->GetSpeciesObserver($species);
        $h_sp{assembly} = $o_so->GetAssembly();
        $h_sp{tax_id}   = $o_so->GetTaxId();
        $h_sp{time_stamp} = $o_so->GetTimestamp();
        $h_sp{chroms_info}= $o_so->GetChromsInfo();
        my $sp = $o_so->GetLongNameWithoutSpace();
        $h_species{$sp} = \%h_sp; 
    }
    my %h_result;
    $h_result{species} = \%h_species;
    return $self->_ToJson( \%h_result );
}

=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license - www.cecill.info

=cut

1;
