#
# 	emmanuel.courcelle@toulouse.inra.fr
#	Created: december 05, 2008
#	$Id$
#

package NcJson;

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod
=head1 NAME

 NcJson.pm

=head1 SYNOPSIS

 NcJson derives from NCScript

=head1 DESCRIPTION

 NcJson is used when we have an ajax request asking for json response (__wa_ajax=j)
 
=cut

use strict;
use warnings;

use LipmError;
use LipmError::ObjectException;
use NarcisseServiceFactory;
use NarcisseServiceKeywords qw( &NS_FASTA );

use Data::Dumper;
use File::Basename;
use Error qw( :try );
use Carp;

use General;
use WebBuilder;
use Runner;
use cgilib::NcScript;
use NARCISSE_utils;
use Image;
use Planner;
use Dotplot;
use Planner::GenomeMap;
use DataReader::Whole;

use base qw( NcScript );

# les erreurs en attendant une gestion propre
# (pour l'instant utilise uniquement en mode ajax)
# <seb>j'ai un peu de mal avec la notion 'ERR_OK'</seb>
my ( $ERR_OK, $ERR_INV_ORG, $ERR_INV_CHR, $ERR_INV_PROG ) = ( 0..3 );

# nom des parametres de la requete - ne sert a rien de definir ces globales a mon avis - manu
my ( $REF, $TARGET, $PROG )
			= qw( reference target program );


=head2 Procedure AjaxCheckSynt

 Title        : AjaxCheckSynt
 Usage        : Through the WebBuilder Realize
 Prerequisite : read synteny.list file
 Procedure    : this function gets the reference, target, program, parameters
                and checks for the valid combinations, reading the file synteny.list.
                Every combination is marked with an error code or validation code
 Args         : none
 Globals      : none

=cut

sub AjaxCheckSynt
{
	my $self = shift;

	my @a_attrs = qw( _o_param _o_io );
	my( $o_param, $o_io ) = $self->_Get( @a_attrs );

	$self->_Trace("AjaxcheckSynt\n","*","X");

    my $rh_organisms = $self->_Get('_rh_organisms');
    my $ra_species   = $rh_organisms->{ SPECIES };
    my $ra_chroms    = $rh_organisms->{ CHROMS };
    my $ra_programs  = $rh_organisms->{ PROGRAMS };

	my @a_targets = @$ra_species; 
    my $reference = shift @a_targets;
    my @a_progs   = @$ra_programs;
    shift @a_progs;

	$self->_Trace("parameters to check: $reference against ".join(' ',@a_targets).' using '.join(' ',@a_progs)."\n","*","XV");

	# init returned array to $ERR_INV_ORG
	my @a_keys = ( $reference, @a_targets );
    $self->_Trace(join(' ',@a_keys)."\n","*","XV");

	# Create and initialize @a_values
	my @a_values;
	for( my $i=0; $i<scalar(@a_keys); ++$i )
	{
		push(@a_values,$ERR_INV_ORG);
	}

	# if no error on reference
	if( $reference ne &NONE )
	{
		$a_values[0] = $ERR_OK;

		my $unix_grep  = $o_io->GetUnixPath('UNIX_GREP');
        my $unix_cat   = $o_io->GetUnixPath('UNIX_CAT');
		my $synt_files = join(' ',$o_io->GetSyntenyListFilepaths());
# 		$self->_Trace("analysing $synt_list_filepath\n","*","XV");
		$self->_Trace("analysing $synt_files\n","*","XV");

		for( my $i=0; $i<@a_targets; ++$i )
		{
			my $target = $a_targets[$i];
			my $prog   = $a_progs[$i];

			if( $target eq &NONE )
			{
				$a_values[1+$i] = $ERR_OK; # pas d'erreur si tgt eq NONE ?
				next;
			}

			if( $prog eq &NONE )
			{	$a_values[1+$i] = $ERR_INV_PROG; next;	}


			my $regexp = "^[0-9]+\t$reference\t$target\t$prog";
			my $cmd = "$unix_cat $synt_files | $unix_grep -Ec '$regexp' ";
			my $nb = `$cmd`;
            chomp $nb;

			$self->_Trace("$synt_files Executing command $cmd\n","*","XV");
			$self->_Trace("$regexp found => $nb","*","XV");

 			if( 0 < $nb )
			{	$a_values[1+$i] = $ERR_OK;	}
			else
			{	$a_values[1+$i] = $ERR_INV_PROG;	}
		}
	}

	my $json = new JSON( );
	my $json_output = $json->encode( \@a_values );
	$self->_Trace("AjaxCheckSynt return $json_output\n","","XV");

	print $json_output;

	return;
}


=head2 procedure AjaxGetChromList

 Title      : AjaxGetChromList
 Usage      : Through Webbuilder Realize, should be called if __wba_ajax eq 'j'
 Prerequiste: DATA/org_name/ should exist, $o_param should have 'org' parameter set
 procedure  : Print the chromomsom list, json formatted
              Basically it's just a call to Instance Observer's GetSpeciesChromInfo.
 Returns    : The list of chrs
 Args       : none

=cut

sub AjaxGetChromList
{
	my $self = shift;

	my $o_param = $self->_Get('_o_param');
	my $o_io    = $self->_Get('_o_io');

	$self->_Trace("AjaxGetChromList\n","*","X");

	# Extract the short name, knowing the long/short name
    # A LipmError will be generated if the species does not exist
	my $org       = $o_param->Get('org');
	my $o_species = New SpeciesObserver( species => $org, -o_inst_obs => $o_io);

	my $org_short = $o_species->GetShortName();
	$self->_Trace("Chromosomes to load for : $org_short \n","*","XV");

	# Load the chromosomes
	my $rh_chrs;
	if ($o_species -> IsPseudo())
	{
		$rh_chrs = $o_species->GetChromsInfo();
	}
	else
	{
		$rh_chrs = $o_species->GetChromsInfo( $o_io -> GetCfg ( 'MAX_DISPLAYED_SCAFFOLDS' ) );
	}

	my $json = new JSON( );
	my $json_output = $json->encode( $rh_chrs );

	$self->_Trace("AjaxGetChromList return ".join(' ',keys %$rh_chrs)."\n","*","XV");
	print $json_output;

	return;
}


=head2 procedure GetGenselData

 Title      : GetGenselData
 Usage      : Through Webbuilder Realize, should be called if __wba_ajax eq 'j'
 procedure  : download the saved gensel configuration
 Returns    : The data, json formatted
 Args       : none

=cut

sub GetGenselData
{
	my $self = shift;

	my $o_param = $self->_Get('_o_param');
	my $o_io    = $self->_Get('_o_io');

	my $f_gensel  = $o_io->GetCfg('ROOT_LOCAL_GENSEL');
    my $js_gensel = "";
    if (-e $f_gensel)
    {
        my $fh_gensel = new IO::File($f_gensel) or throw LipmError::IOException($f_gensel,'f' );
        $js_gensel = <$fh_gensel>;
        chomp $js_gensel;
        $fh_gensel->close();
    }
	$self->_Trace("AjaxGetChromList return ".$js_gensel."\n","*","XV");
	print "$js_gensel\n";
	return;
}


=head2 Procedure ServiceFromLocus

 Title        : ServiceFromLocus
 Usage        : Through WebBuilder Realize, should be called only if __wba_ajax eq 'j'
 Procedure    : Launch a service for returning a sequence from a locus tag, and print the sequence, json formatted
                NOTE - ONLY GetProteinsFromLocusTags implemented now
 Args         : None

=cut

sub ServiceFromLocus
{
	my $self = shift;

	my $o_param       = $self->_Get('_o_param');
	my ( $service_key, $args ) = $o_param->Get('service','args');
	$self->_Trace("ServiceFromLocus service=$service_key#parameters=$args#\n","*","XV");

	my $seq;
	my $o_request = $self->__GetRequestFromKeyword ( $service_key, $args );
	if (defined $o_request)
	{
		my ($ra_headers,$rh_data) = $o_request->FetchResults();
		foreach my $header (@$ra_headers)
		{
			my $data = $rh_data->{$header};
			$seq    .= $header . "\n";
			$seq    .= &Sequence_utils::FormatSequenceLine(seq => $data) . "\n";
		}
		$self->_Trace("ServiceFromLocus return ".substr( $seq,0,40)."...\n","*",'X');
	}
	else
	{
		$self->_Trace("ServiceFromLocus problem: \$o_request undefined\n","*","X");
	}

	my $json = new JSON( );
	my $json_output = $json->encode( { seq => $seq } );

	print $json_output;

	return;
}


=head2 Function __GetRequestFromKeyword

 Title   : __GetRequestFromKeyword
 Function: Create and return a NarcisseService object
 Args    : $service_key (only "protein" supported yet)
           $args: string, the arguments passed to the constructor
 Return  : the created object, or undef if some problem

=cut
sub __GetRequestFromKeyword
{
    my $self = shift;
    my ( $service_key,$args ) = (@_);
    my $o_request = undef;
    if ($service_key eq 'protein')
    {
        # $args could be something like --locus_tag Ahi1 --species human
        # OR (better)                   --locus_tag Ahi1 --species Homo sapiens
        # Several --locus_tag switches are recognized (but this WILL NOT WORK with blast_example)
        # You can use --locus_tags instead of --locus_tag

        my @a_args = split(/ *--/,$args);

        my ( $species, @a_tags );
        foreach my $arg ( @a_args )
        {
			if ( $arg =~ /^species (.+)$/ )
			{
				$species = $1;
			};
			if ( $arg =~ /^locus_tags? (.+)$/ )
			{
				push @a_tags, $1;
			}
		}
        
        $o_request = &NarcisseServiceFactory::GetProteinsFromLocusTags(
            species => $species,
            ra_tags => \@a_tags,
            -o_logger => $self->_GetLogger(),
            out_type  => &NS_FASTA
            );
    }
    return $o_request;
}

=head2 Function GetHtml

 Title    : Gethtml
 Usage    : $html = $o_ncs->GetHtml()
 Function : Generate and return the html
 Returns  : The html generated
 Args     : none

=cut
sub GetHtml {return 'ERROR - NO __wb_function SPECIFIED FOR THIS AJAX REQUEST'}


=head2 GetSessionGenomes

 Usage    : my $json = $o_x->GetSessionGenomes()
 Function : seek for genomes in the session directory and return them
            as json data to update O_CFG.GENOME.
 Returns  : $json
 Args     : none
 Error    : none

=cut

sub GetSessionGenomes
{
	my $self = shift;

	my $o_io       = $self->_Get('_o_io');
	my $rh_species = {};
    
    my $filepath = $o_io->GetSpeciesListFilepath( 'SESSION' );
    if ( -e $filepath )
    {
        my $o_ndr  = New DataReader::Whole();
        $o_ndr->Prepare( -from => $filepath );
        $rh_species  = $o_ndr->FetchallHashref( 'long', -line_type => 'HASH' );
    }
    
	my $json = new JSON( );
	my $json_output = $json->encode( $rh_species );

	my $msg = "GetSessionGenomes return " . join( ' ', keys %$rh_species );
    
	$self->_Trace( $msg . "\n", "*", "XV");
	print $json_output;

# 	return $json;
} #GetSessionGenomes

=head2 GetAreas

 Usage    : my $json = $o_x->GetAreas()
 Function : Look for a .js file describing the areas and send this file
 Returns  : $json
 Args     : none
 Error    : If the file cannot be opened, print ""

=cut

sub GetAreas
{
	my $self = shift;
    
    my ($map_js) = $self->CalcJsNames(1,'map');
    my $json="";
    my $fh_map_js = new IO::File ($map_js);
    if (defined $fh_map_js)
    {
		$json = <$fh_map_js>;
	}
	else
	{
		warn "ERROR - FILE $map_js NOT FOUND";
	}

    print $json;

} #GetAreas

=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license - www.cecill.info

=cut


1;
