#
# 	sebastien.letort@toulouse.inra.fr
#	Created: avril 03, 2009
#	$Id: NcKnowledge.pm 3498 2010-09-16 13:35:16Z manu $
#

package NcKnowledge;

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod
=head1 NAME

 Knowledge.pm

=head1 SYNOPSIS


=head1 DESCRIPTION


=cut

use strict;
use warnings;

use LipmError;
use LipmError::ObjectException;

use Data::Dumper;
use File::Basename;
use Error qw( :try );

use General;
use WebBuilder;
use ParamParser;
use cgilib::NcScript;
use KnowledgeRemoteReader;
use SpeciesObserver;

use base qw( NcScript );


=head2 Function GetHtml

 Usage    : $html = $o_ncs->GetHtml()
 Function : Generate and return the html
 Returns  : The html generated
 Args     : none

=cut

sub GetHtml
{
	my $self = shift;

	my $o_wb = $self->_Get( '_o_wb' );
	my $o_param  = $o_wb->GetObjectParam();

	$self->_Trace( "Service GetKnowledgeInfos\n", "*","XV" );

	# 1- obtenir l'espece et les bornes ou le locus
	# *** Pour le moment si on a le locus, on ne s'occupe pas des bornes.
	# *** k_species existe car le nom legoo n'est pas celui de narcisse
	my ( $species, $k_species ) = $o_param->Get( qw( species k_species ) );
	my ( $tag, $chrom, $range ) = $o_param->Get( qw ( locus_tag chrom range ) );
	my $ra_tags = $self->__ExtractTagsFromParams();
	$self->_Trace("request parameters: $k_species, @$ra_tags\n", '*', 'XV' );

	# name associated to this species-tag
	my $filename = join( '_', $k_species, $range, 'knowledge' );
	$filename .= '.list';

	# retreive information
	my %h_args = ( species => $k_species, -ra_locus_tags => $ra_tags,
	               -cache => $filename, -o_logger => $self->_GetLogger(),
	               o_io => $self->_Get( '_o_io' ) );
	my $o_knowledge = New KnowledgeRemoteReader( %h_args );
	my $rh_data = $o_knowledge->GetData();

	my $page = $self->__WriteHtmlPage( $rh_data );

	return $page;
}


=head2 Function __WriteHtmlPage

 Usage        : my $page = $o_x->__WriteHtmlPage( $rh_data )
 Function     : .
 Returns      : $page
 Args         :
     $rh_data
 Error        : none

=cut

sub __WriteHtmlPage
{
	my $self = shift;
	my( $rh_data ) = @_;

	my $html = '';
	my @a_col = qw( effector pattern reference relationship
					confidence id factor source );

	if( 0 == scalar( keys %$rh_data ) )
	{	$html = "NO data found<br />";	}
	else
	{
		foreach my $key( sort { $a <=> $b } keys %$rh_data )
		{
			my $rh_row = $$rh_data{ $key };

			$html .= "\t\t\t<tr>\n";
# 			$html .= "\t\t\t\t<td><input type='checkbox' value='i' /></td>\n";

			# pour avoir les valeurs dans le bon ordre
			my @a_values = @$rh_row{ @a_col };
			foreach my $val ( @a_values )
			{
				$html .= "\t\t\t\t<td>$val</td>\n";
			}
			$html .= "\t\t\t</tr>\n";
		}
	}

	my $rh_cfg = $self->_Get( '_rh_cfg' );
	my $file   = File::Spec->catfile( $$rh_cfg{ ROOT_INC }, 'knowledge.inc' );

	my $page   = $self->_IncludeFromTmpl($file, table_body => $html);

	return $page;
} #WriteHtmlPage



=head2 Function __ExtractTagsFromParams

 Usage        : my $ra_tags = $o_x->__ExtractTagsFromParams( none )
 Function     : .
 Returns      : $ra_tags
 Args         :
     none
 Error        : none

=cut

sub __ExtractTagsFromParams
{
	my $self = shift;

	my $o_wb = $self->_Get( '_o_wb' );
	my $o_param  = $o_wb->GetObjectParam();

	my @a_params = qw( locus_tag chrom species range );
	my ( $tag, $chrom, $species, $range ) = $o_param->Get( @a_params );

	my @a_tags = ();
	if( $tag eq '' )
	{
		my @a_range = split( '_', $range );
		my $o_species = New SpeciesObserver( species => $species );
		my $o_dba   = $o_species->GetDBAdaptor( $species );
		my $o_sla   = $o_dba->get_SliceAdaptor();
		my $o_slice = $o_sla->fetch_by_region( 'chromosome', $chrom, @a_range );
		my $rao_genes = $o_slice->get_all_Genes();
		foreach my $o_gene ( @$rao_genes )
		{
			push( @a_tags, $o_gene->stable_id() );
		}
	}
	else
	{
		push( @a_tags, $tag );
	}

	return \@a_tags;
} #__ExtractTagsFromParams

1;
