#
# 	emmanuel.courcelle@toulouse.inra.fr
#	Created: december 05, 2008
#	$Id: WebBuilderFactory.pm 2317 2009-01-22 14:23:43Z sletort $
#

package NcSaveSession;

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod
=head1 NAME

 NcSaveSession.pm

=head1 SYNOPSIS


=head1 DESCRIPTION

 NcSaveSession derives from NCScript, it is used for downloading the session
 
=cut

use strict;
use warnings;

use LipmError;
use LipmError::ObjectException;

use Data::Dumper;
use File::Basename;
use Error qw( :try );

use General;
use WebBuilder;
use ParamParser;
use cgilib::NcScript;

use base qw( NcScript );


=head2 function _Init

 Title        : _Init
 Fonction     : the constructor
 Args         : $o_io         : the instance observer
                $o_web_builder: A valid WebBuilder object

=cut
sub _Init
{
    my $self = shift;
    
    my ($o_io,$o_wb,$o_logger) = (@_);
    $self->SUPER::_Init($o_io,$o_wb,$o_logger,&FALSE);
    
    # Set the mime type, do not send header, do not send parameters
    $self->_Trace("Now setting mime type to application/force-download\n","*",'SV');
    $o_wb->SetMimeType('application/force-download');
    $o_wb->UnsetBehaviour('send_parameters');
    $o_wb->SetBehaviour('do_not_send_header');
    
    # Retrieve the last local.cfg, because the script parameter is now lost !!!
    # We also dump local.cfg, because some parameters may have been changed since last request
    my $o_param = $self->_Get('_o_param');
    $o_param->Delete('script');
    my $o_param_alt = New ParamParser('local.cfg');
    $o_param->Update('PARAMPARSER','A',$o_param_alt);
    $o_param->Dump('local.cfg');
    return $self;
}

=head2 Function GetHtml

 Title    : Gethtml
 Usage    : $html = $o_ncs->GetHtml()
 Function : Generate and return the html
 Returns  : The html generated
 Args     : none

=cut

sub GetHtml
{
    my $self = shift;

	my $o_param = $self->_Get('_o_param');
    my $o_io    = $self->_Get('_o_io');
    
    my $tmp_dir   = 'tmp';
    my $tar_file  = $tmp_dir.'.tar';
    my $tgz_file  = $tar_file.'.gz';

    # Make the temporary directory
    mkdir $tmp_dir;

    # Save a Version marker
    my $version = $tmp_dir.'/VERSION.cfg';
    my $fh_version = new IO::File(">>$version") or throw LipmError::IOException( $version, 'f' );
    print $fh_version "NARCISSE_VERSION=" . $o_io->GetNarcisseVersion() . "\n";
    print $fh_version "INSTANCE_NAME="    . $o_io->GetName() . "\n";
    print $fh_version "INSTANCE_VERSION=" . $o_io->GetVersion() . "\n";
    print $fh_version "DATE=" . &NARCISSE_utils::GetDate() . "\n";
    $fh_version->close();

    # Copy the useful files
    my $unix_rm   = $o_io->GetUnixPath('UNIX_RM');
    my $unix_cp   = $o_io->GetUnixPath('UNIX_CP');
    my $unix_tar  = $o_io->GetUnixPath('UNIX_TAR');
    my $unix_gzip = $o_io->GetUnixPath('UNIX_GZIP');
    my $unix_find = $o_io->GetUnixPath('UNIX_FIND');


    my ($perso_data,$sessdir,$narcisse_bookmarks) = $o_io->GetCfg('ROOT_PERSONAL','ROOT_SESSDIR','ROOT_NARCISSE_BOOKMARKS');
    `$unix_cp ${perso_data} ${sessdir}/*.cfg $narcisse_bookmarks $tmp_dir >/dev/null 2>&1`;
    `$unix_find $tmp_dir -name .\\* -exec rm -rf {} \\;`;  # .htaccess or other system files, do not download them !!!
    
    #`$rh_cfg->{UNIX_CP} $rh_cfg->{ROOT_PERSO_DATA}* $rh_cfg->{ROOT_SESSDIR}/*.cfg $rh_cfg->{ROOT_NARCISSE_BOOKMARKS} $tmp_dir >/dev/null 2>&1`;

    # Tar and gzip the temporary directory
    `$unix_tar  -c -f $tar_file $tmp_dir`;
    `$unix_gzip $tar_file`;

    # Send this file
    my $fh_tgz = new IO::File($tgz_file) or throw LipmError::IOException( $tgz_file, 'f' );
    while(<$fh_tgz>) {
        print;
    };
    $fh_tgz = undef;

    # Remove temporary directory
    `$unix_rm -r $tmp_dir`;
}

1;
