#
# 	emmanuel.courcelle@toulouse.inra.fr
#	Created: december 05, 2008
#	$Id: WebBuilderFactory.pm 2317 2009-01-22 14:23:43Z sletort $
#

package NcSendSequences;

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod
=head1 NAME

 NcSendSequences.pm

=head1 SYNOPSIS


=head1 DESCRIPTION

 NcSendSequences derives from NCScript and is used to send a sequence
 (parameters GeneSeq, ProteinSeq, CdsSeq, GenomicSeq)
 If the SaveData parameter is != 0, we just propose to download data
 Else, we send a real html page

=cut

use strict;
use warnings;

use LipmError;
use LipmError::ObjectException;
use LipmError::IOException;


use Data::Dumper;
use File::Basename;
use Error qw( :try );

#use Region;
use GlintFish;
use Probe;

use General;
use WebBuilder;
use ParamParser;
use cgilib::NcScript;

use base qw( NcScript );


=head2 function _Init

 Title        : _Init
 Fonction     : the constructor
 Args         : $o_io         : the instance observer
                $o_web_builder: A valid WebBuilder object

=cut
sub _Init
{
	my $self = shift;

	my ($o_io,$o_wb,$o_logger) = (@_);
	$self->SUPER::_Init($o_io,$o_wb,$o_logger,&FALSE);

	# In SaveData mode, change the mime-type
	my $o_param = $self->_Get('_o_param');
	if ($o_param->GetInteger('SaveData') != 0)
	{
		# Set the mime type, do not send header, do not send parameters
		$self->_Trace("Now setting mime type to application/force-download and calling Realize(1)\n","*",'SV');
		$o_wb->SetMimeType('application/force-download');
		$o_wb->UnsetBehaviour('send_parameters');
		$o_wb->SetBehaviour('do_not_send_header');
	}
	$o_wb->UnsetBehaviour('send_parameters');

	$self->_Trace( "Hello\n" , '.', 'GS' );

	return;
}

=head2 Function GetHtml

 Title    : Gethtml
 Usage    : $html = $o_ncs->GetHtml()
 Function : Generate and return the html, calling a function 
            The function called depends on the parameter
 Returns  : The html generated
 Args     : none

=cut
sub GetHtml
{
    my $self = shift;
		
	my $o_param   = $self->_Get('_o_param');
	
	# Retrieving sequences aligned by OnlineCompare
	if ($o_param->Get('ViewAlign') ne '')
	{
		return $self->__ViewAlign();
	}
	
	# Just for a demo !
	# NOTE - You should add MyComputation (or whatever name your function gets) to the array
	#        NcScript::A_DO_NOT_SAVE (see the NcScript.pm file), to avoid it to be saved between requests
	elsif ($o_param->Get('MyComputation') ne '')
	{
		return $self->__MyComputation();
	}
	
	# Retrieving results of GlintFish computations
	elsif ($o_param->Get('GlintFish') ne '')
	{
		return $self->__GlintFish();
	}
	
	# Retrieving results of GlintFish computations
	elsif ($o_param->Get('LaunchGlintFish') ne '')
	{
		return $self->__LaunchGlintFish();
	}
	
	# Retrieving genomic of proteic sequences
	else
	{	
		return $self->__ViewSequences();
	}
}


=head2 __ViewSequences

 Title     : __ViewSequences
 Usage     : $self->__ViewSequences()
 Function  : 1/ Retrieve the wanted sequences; using 8 parameters (GeneSeq, GeneSeqF etc.)
             2/ Check the retrieved sequences to know how long they are
             3/ Build the html screen of download the wanted file
 Args      : none
 Return    : The html code
 Errors	   : IOException
 Globals   : none

=cut

sub __ViewSequences
{
	my $self = shift;

	my @a_attrs = qw( _o_param _o_io );
	my( $o_param, $o_io ) = $self->_Get( @a_attrs );
	my $def_species = $o_io -> GetSpeciesShortName( $o_param->Get('species') );
	my $sessdir = $o_io -> GetSessionDirectory();
	my @a_objects  = qw( Gene Protein Cds Genomic );
	my @a_keys  = qw( GeneSeq ProteinSeq CdsSeq GenomicSeq );
	
	# %h_loci: keys = The @a_objects array
	#          val  = A hash, the key is the species'short name
	#                 val: An array, containing the list of loci to retrieve
	my %h_loci;
	
	# %h_out:  keys = The @a_objects array
	#          val  = The output file name
	my %h_out;
	
	# Initialize %h_loci and %h_out
	foreach my $object (@a_objects)
	{
		my @a_loci;
		
		# Gene ==> GeneSeq
		my $key = $object . 'Seq';
		if ( $o_param -> Get ($key) ne '' )
		{
			my $in    = $o_param -> Get ($key);
			#warn "KOUKOU1 $key => $in";
			
			# The parameter is a file name inside the session directory: read the file
			# The file can be formatted as:
			#     - 1 locus/line, in which case the species is given by $o_param
			#     - for each line: species\tlocus, the species is embedded in the line
			if (-e $in)
			{
				my $fh_in = new IO::File( $in ) || throw LipmError::IOException ( $in, 'f' );
				while (my $line = <$fh_in>)
				{
					chomp $line;
					my ($species,$locus);
					my @a_f = split /\t/,$line,2;
					
					# species:locus
					if (@a_f == 2)
					{
						$species = $a_f[0];
						$locus   = $a_f[1];
					}
					
					# locus
					else
					{
						next if ($a_f[0] eq '');
						$species = $def_species;
						$locus   = $a_f[0];
					}
					if ( ! exists $h_loci { $object } -> { $species } )
					{
						$h_loci { $object } -> { $species } = [];
					}
					my $ra_loc = $h_loci { $object } -> { $species };
					push @$ra_loc, $locus;
				}
				# WARNING - The $in file name MUST end with some extension (toto.txt is OK, toto is not)
				my $out = $in;
				$out =~ s/\..+$/.fasta/;      # toto.txt   => toto.fasta
				$out = $object . '.' . $out;  # toto.fasta => Gene.toto.fasta
				$h_out{ $object } = $out;
			}
			
			# did not find a file - probably a locus_tag
			else
			{
				my $locus = $o_param -> Get ($key);
				push @a_loci, $locus;
				$h_loci { $object } -> { $def_species } = \@a_loci;
				$h_out  { $object } = $object . '.' .$locus . '.fasta';
			}
		}
		#$h_loci { $object } = \@a_loci;
		#warn "KOUKOU2 $object => " . $h_out{$object};
	}
	
	# Retrieve data and create the files, unless it exists already 
	foreach my $object ( keys %h_out)
	{
		if ( ! -e $h_out { $object } )
		{
			if ( exists $h_loci { $object } )
			{
				my $rh_loci_sp = $h_loci { $object };
				foreach my $long (keys %$rh_loci_sp)
				{
					my $short = $o_io -> GetSpeciesShortName( $long );
					
					# Is it a real locus tag, or a set of regions ?
					# We check only the FIRST one, let's hope it is the same for every line
					my $ra_loci = $h_loci{ $object } -> { $long };
					my @a_tags = split(':',$ra_loci->[0]);
					my $out = $h_out{ $object };
					my $out_tmp = $out . '.tmp';

		            try
		            {
		                # At least 4 fields: it is a region
		                if ( @a_tags >=4 )
		                {		                  
		                    $self->_GetSequenceRegion($object,\@a_tags,$out_tmp);
		                }
		                else
		                {
		                    $self->_GetSequenceLocusTag($object,$short,$ra_loci,$out_tmp);
		                };   

	                    # Append $fh_out_tmp to $out
	                    my $fh_out_tmp = new IO::File ( $out_tmp)  || throw LipmError::IOException( $out_tmp,'f');
	                    my $fh_out     = new IO::File ( $out, 'a') || throw LipmError::IOException( $out,'f');
	                    while (my $line = <$fh_out_tmp>)
	                    {
							print $fh_out $line;
						}

		            }
			        catch LipmError::ParameterException with {
	                    my $o_err = shift;
	                    throw $o_err;
	                }
	                otherwise
	                {};
#					}
				}
			}
		}
	}
	
	# If the retrieved files are not too big, we can display them
	my $nb_lines = 0;
	my $unix_wc = $o_io -> GetUnixPath('UNIX_WC');
	foreach my $object (keys %h_out)
	{
		my $out = $h_out { $object };
		if ( -e $out )
		{
			my $nb = `$unix_wc -l "$out"`;
			($nb) = split ' ',$nb,2;
            #warn "KOUKOU3 $unix_wc $out $nb $nb_lines";

			$nb_lines += $nb;
		}
	}

	# display the results
	if ( $o_param -> GetInteger( 'SaveData' ) == &TRUE )
	{
		return $self -> __ViewSequencesDownload(\@a_objects,\%h_loci,\%h_out);
	}
	else
	{
		my $html = '<div id="ID_ERROR"><div class="hd"></div><div class="bd"></div><div class="ft"></div></div>';
		if ($nb_lines > 40)
		{
			$html .=  $self -> __ViewSequencesDigestOnly(\@a_objects,\%h_loci,\%h_out);
		}
		else
		{
			$html .= $self -> __ViewSequencesComplete(\@a_objects,\%h_loci,\%h_out);
		}
		$html .= $self -> __ViewSequencesDownloadMenu(\@a_objects,\%h_loci,\%h_out);
		return $html;
	}
}

=head2 __ViewSequencesDigestOnly

 Title     : __ViewSequencesDigestOnly
 Function  : 1/ If a protein, display annotation
             2/ Check the error and display message if necessary
 Args      : $ra_objects,$rh_loci,$rh_out
 Return    : The html code
 Errors	   : IOException
 Globals   : none

=cut
sub __ViewSequencesDigestOnly
{
	my $self = shift;
	my ($ra_objects,$rh_loci,$rh_out) = @_;
	
	my $html = "<p class=\"Warning\"> WARNING - The retrieved sequences were too long to be displayed - Please download the file</p>";

	foreach my $object (@$ra_objects)
	{
		if ( $object eq 'Protein' )
		{
			my $out = $rh_out -> { $object };
			if (defined $out)
			{
				my $fh_out = new IO::File ($out) || throw LipmError::IOException($out,'f');
				my $header = <$fh_out>;
				$html .= $self -> __ViewProteinAnnotation($header);
			}
		}
	}
	
	# Check the error files
	foreach my $object (@$ra_objects)
	{
		next unless (exists $rh_out -> { $object });
		my $out = $rh_out -> { $object };
		my $err = $out;
		$err =~ s/fasta$/err/;
		if( -e $err and !-z $err )
		{
			my $err_msg = $self->_IncludeFromTmpl( $err );
			$html .= "<p class='Warning'>$err_msg</p>\n";
		}
	}
	
	return $html;
}

=head2 __ViewSequencesDigestOnly

 Title     : __ViewSequencesComplete
 Function  : 1/ If a protein, display annotation
             2/ Display the whole sequence
 Args      : $ra_objects,$rh_loci,$rh_out
 Return    : The html code
 Errors	   : IOException
 Globals   : none

=cut
sub __ViewSequencesComplete
{
	my $self = shift;
	my ($ra_objects,$rh_loci,$rh_out) = @_;
	my $html;
	
	# Check the error files, then display the file
	foreach my $object (@$ra_objects)
	{
		next unless exists ( $rh_out -> { $object } );
		$html .= "<h2>".$object." sequence</h2>";
		my $out = $rh_out -> { $object };
		my $err = $out;
		$err =~ s/fasta$/err/;
		if( -e $err and !-z $err )
		{
			my $err_msg = $self->_IncludeFromTmpl( $err );
			$html .= "<p class='Warning'>$err_msg</p>\n";
			$html .= '<pre>';
		}
		else
		{	
			my $fh_out = new IO::File ($out) || throw LipmError::IOException($out,'f');
			my $header = <$fh_out>;

			if ( $object eq 'Protein' )
			{
				$html .= $self -> __ViewProteinAnnotation($header);
			}
			
			$html .= "<pre>";
			$html .= $header;
			while(my $line = <$fh_out>)
			{
				$html .= $line;
			}
			$html .= "</pre>\n";
		}
	}
	
	return $html;
}

=head2 __ViewProteinAnnotation

 Title     : __ViewProteinAnnotation
 Function  : 1/ display the annotation from the fasta header (protein sequences only)
 Args      : $header
 Return    : The html code
 Errors	   : IOException
 Globals   : none

=cut
sub __ViewProteinAnnotation
{
	my $self = shift;
	my ($header) = @_;
	my $html = "<table class=\"TableAnnot\">";
	
	my @a_fields = split (' ', $header);
	shift @a_fields;		# skip >prot:id
	my ($std_annot,$ipr_annot)=('','');
	my $ipr_annot_flg = &FALSE;
	foreach my $f (@a_fields)
	{
		if ( $f =~ /(.+)=(.+)/ )
		{
			$html .= "<tr><th>$1</th><td>$2</td></tr>\n";
		}
		else
		{
			if ( $f eq 'AutoIPR:' )
			{
				$ipr_annot_flg = &TRUE;
			}
			elsif ( $ipr_annot_flg == &FALSE )
			{
				$std_annot .= $f . ' ';
			}
			else
			{
				$ipr_annot .= $f . ' ';
			}
		}
	}
	$html .= "<tr><th>Product</th><td>$std_annot</td></tr>";
	$html .= "<tr><th>interproscan</th><td>$ipr_annot</td></tr>";
	
	$html .= "</table>";
		
	return $html;
}

sub __ViewSequencesDownload
{
	my $self = shift;
	my ($ra_objects,$rh_loci,$rh_out) = @_;
	my $html;
	
	# Check the error files, ignore if exists
	foreach my $object (@$ra_objects)
	{
		next unless (exists $rh_out -> { $object } );
        #warn "KOUKOU2 $object $rh_out->{$object}";
		my $out = $rh_out -> { $object };
		my $err = $out;
		$err =~ s/fasta$/err/;
		next if( -e $err and !-z $err );

		my $fh_out = new IO::File ($out) || throw LipmError::IOException($out,'f');
		while(my $line = <$fh_out>)
		{
			$html .= $line;
		}
	}
	
	return $html;
}
	
sub __ViewSequencesDownloadMenu
{
	my $self = shift;
	my ($ra_objects,$rh_loci,$rh_out) = @_;
	my ($o_param,$o_io) = $self -> _Get( '_o_param','_o_io');
	my $species = $o_param->Get('species');
	
	my $html;
	
	$html .= "<h2>Download files</h2>";
	$html .= "<ul>\n";
	foreach my $object (@$ra_objects)
	{
		next unless (exists $rh_out -> { $object } );
        
		my $out = $rh_out -> { $object };
		my $err = $out;
		$err =~ s/fasta$/err/;
		next if( -e $err and !-z $err );

		# Access the .fasta files through an url
		my @a_params = qw( script=SENDSEQUENCES SaveData=1 );
		my $key = $object . 'Seq';
		push @a_params, "species=$species", "$key=" . $o_param -> Get($key);
		my $sessdir = $o_io->GetSessionDirectory();
		push( @a_params, "__wb_session=$sessdir" );
		my $url = $o_io->GetCfg( 'HTTP_NARCISSE_CGI' ) . '/' . $out . '?';
		$url   .= join( '&amp;', @a_params );
		$html  .= "<li><a href=\"$url\">".$object."</a> sequence</li>";
	}
	$html .= "</ul>\n";
	
	return $html;
}


=head2 __ViewAlign

 Title     : __ViewAlign
 Usage     : $self->__ViewAlign()
 Function  : This function shows the alignment result in a pop-up window.
 			 Matches and gaps are shown, and there are 2 links for retrieving the sequences
 Args      : none
 Return    : The html code
 Errors	   : IOException
 Globals   : none

=cut
sub __ViewAlign
{
    my $self = shift;
    
    my $o_io    = $self->_Get('_o_io');
    my $o_param = $self->_Get('_o_param');
    my $o_logger= $self->_Get('_o_logger');
    
	my @a_glint_results = ();
	my @a_mismatchs= ();
	my @a_ref_gaps = ();
	my @a_tgt_gaps = ();

	#--------------------------------
	# Get parameters
	my $region		= $o_param->Get('ViewAlign');
	my @a_region    = split(':',$region);
	my @a_reference = @a_region[0..3];
	my @a_target    = @a_region[4..7];
	
	my $reference = $a_region[0];
	my $ref_chrom = $a_region[1];
	my $ref_start = $a_region[2];
	my $ref_end   = $a_region[3];
	my $target	  = $a_region[4];
	my $tgt_chrom = $a_region[5];
	my $tgt_start = $a_region[6];
	my $tgt_end	  = $a_region[7];

	#--------------------------------
	# Files and url names.
	my $file_suffix = ".fasta";
	my $ref_sequence_file = $o_io -> GetCfg('ROOT_SESSDIR')."/".$reference.$ref_chrom."_".$ref_start."_".$ref_end.$file_suffix;
	my $tgt_sequence_file = $o_io -> GetCfg('ROOT_SESSDIR')."/".$target.$tgt_chrom."_".$tgt_start."_".$tgt_end.$file_suffix;
	my $ref_sequence_url  = $o_io -> GetCfg('HTTP_SESSDIR')."/".$reference.$ref_chrom."_".$ref_start."_".$ref_end.$file_suffix;
	my $tgt_sequence_url  = $o_io -> GetCfg('HTTP_SESSDIR')."/".$target.$tgt_chrom."_".$tgt_start."_".$tgt_end.$file_suffix;

    my $o_so        = $o_io -> GetSpeciesObserver ( $reference );
    my $level       = $o_param -> Get ( 'level' );
    my $target_short= $o_io -> GetSpeciesObserver ( $target ) -> GetShortName();
    my $result_file = $o_so -> GetHspFiles( prog => 'glint', level => $level, target => $target_short, -chrom => $ref_chrom );
	
	#--------------------------------
	# Get sequences.
	$self->_GetSequenceRegion('Genomic',\@a_reference,$ref_sequence_file) if (!-e $ref_sequence_file);
	$self->_GetSequenceRegion('Genomic',\@a_target,$tgt_sequence_file) if (!-e $tgt_sequence_file);

    # PROVISOIRE !!!
	throw LipmError::IOException($ref_sequence_file,'f') if (!-e $ref_sequence_file);
	throw LipmError::IOException($tgt_sequence_file,'f') if (!-e $tgt_sequence_file);

	#--------------------------------
	# Get glint result :
    my $fh_result_file = new IO::File("$result_file") or throw LipmError::IOException("$result_file",'f');
	my $result;
	while( my $line = <$fh_result_file> )
	{
		chomp $line;
		next if ( $line =~ /^#/ );
		my @a_splited_line = split('\t',$line);
		if( $a_splited_line[3] == $ref_start )
		{
			$result	= $a_splited_line[-1];
			last;
		}
	}
	$fh_result_file->close();	
		
	if ($result =~ /\((.*)\)\[(.*)\]\{(.*)\}\{(.*)\}\:$/)
	{
		@a_glint_results = split(',',$1);
		@a_mismatchs	 = split(',',$2) if (defined $2);
		@a_ref_gaps		 = split(',',$3) if (defined $3);
		@a_tgt_gaps		 = split(',',$4) if (defined $4);
	}

	my $len = $a_glint_results[0];
	my $ref_sequence = &__GapSequenceToString($ref_sequence_file,\@a_ref_gaps);
	my $tgt_sequence = &__GapSequenceToString($tgt_sequence_file,\@a_tgt_gaps,$a_glint_results[1]);
	my $mis_sequence = &__MismatchToString($len,\@a_mismatchs,\@a_ref_gaps,\@a_tgt_gaps);
	
	my $IsPlusMinus = ( lc($a_glint_results[1]) =~ /minus/ ) ? 1 : 0 ;
		
	#--------------------------------
	# Print results
    my $html = "<h2>Local homology between ".ucfirst($reference)." chrom.$ref_chrom  ($ref_start..$ref_end) and ".ucfirst($target)." chrom.$tgt_chrom ($tgt_start..$tgt_end)</h2>\n\n";
	
	# 1) Print the <pre> section.
	$html .= '<pre>';
    $html .= "Score\t\t= ".$a_glint_results[2]."\n";
    $html .= "Identities\t= ".($len-scalar(@a_mismatchs))."/$len (".$a_glint_results[4].")\n";
    $html .= "Gaps\t\t= ".(scalar(@a_ref_gaps)+scalar(@a_tgt_gaps))."\n";
    $html .= "Strand\t\t= ".lc($a_glint_results[1])."\n";
    $html .= "</pre><hr /><pre>";
		
    my $ref_pos_start = $ref_start;
    my $tgt_pos_start = ($IsPlusMinus ) ?  $tgt_end : $tgt_start;
    my ($ref_pos_end,$tgt_pos_end) = ($ref_pos_start,$tgt_pos_start);
    for (my $i=0; $i<=abs($len/100); $i++)
    {
    	my $reste = ($len > ($i+1)*100 ) ? 99 : $len%100-1;
    	my $start_id = $i*100;
    	my $end_id	 = $i*100+$reste;
    	my $ref_sub_seq = &__FormatedSliceSequence($ref_sequence,$start_id,$end_id);
    	my $tgt_sub_seq = &__FormatedSliceSequence($tgt_sequence,$start_id,$end_id);
    	my $mis_sub_seq = &__FormatedSliceSequence($mis_sequence,$start_id,$end_id);

		#--------------------------------
		# Calculate start of this line
		$ref_pos_start = $ref_pos_end;
		$tgt_pos_start = $tgt_pos_end;
		if ($i != 0)
    	{
    		$ref_pos_start += 1;
    		$tgt_pos_start += ($IsPlusMinus ) ? -1 : 1;
    	}

		#--------------------------------
		# Calculate end of this line
    	$ref_pos_end =  $ref_pos_start+$reste-&__CountGaps($ref_sub_seq);
    	$tgt_pos_end = ( $IsPlusMinus ) ? $tgt_pos_start-$reste+&__CountGaps($tgt_sub_seq) : $tgt_pos_start+$reste-&__CountGaps($tgt_sub_seq);
    	if ($reste < 99)
    	{
    		$ref_pos_end -=  1;
    		$tgt_pos_end -= ($IsPlusMinus ) ? -1 : 1;
    	}

    	# Color target sequence in grey if strand equal to plus/minus
    	$tgt_sub_seq = "<span style=\"font-color: 666666\">".$tgt_sub_seq."</span>" if ( $IsPlusMinus );
    	# Color reference sequence in grey if strand equal to plus/minus
    	#$ref_sub_seq = "<span style=\"font-color: 666666\">".$ref_sub_seq."</span>" if ((lc($a_glint_results[1]) =~ /minus/));
		
		#--------------------------------
		# Print the 3 lines
    	$html .= "\n";
        my $label_width = (length($reference) > length($target)) ? length($reference) : length($target);
        my $spe1 = ' ' x (10-length($ref_pos_start));
    	$html .= &__FormatLabel(ucfirst($reference),$label_width)." : $ref_pos_start" . (' ' x (10 - length($ref_pos_start))) . "$ref_sub_seq\t$ref_pos_end\n";
    	$html .= &__FormatLabel(' ',$label_width) . (' ' x 13)  . "$mis_sub_seq\n";
    	$html .= &__FormatLabel(ucfirst($target),$label_width)." : $tgt_pos_start" . (' ' x (10 - length($tgt_pos_start))) . "$tgt_sub_seq\t$tgt_pos_end\n";
    }
    $html .= "\n</pre><hr />";

	# 2) Print sequence links
	#system ("gzip $ref_sequence_file $tgt_sequence_file");
    #$ref_sequence_file =~ s/.*\/([\w\.]*)$/$rh_cfg->{HTTP_SESSDIR}\/$1.gz/;
    #$tgt_sequence_file =~ s/.*\/([\w\.]*)$/$rh_cfg->{HTTP_SESSDIR}\/$1.gz/;
    #$ref_sequence_url .= '.gz';
    #$tgt_sequence_url .= '.gz';

    $html .= "<h2>Download files :</h2>";
    $html .= "<ul>";
	$html .= "<li><a href='".$ref_sequence_url."' >".ucfirst($reference)."</a> sequence</li>";
	$html .= "<li><a href='".$tgt_sequence_url."' >".ucfirst($target)."</a> sequence</li>";
	$html .= "</ul>";

    return $html;

	#################################
	#		Private functions		#
	#################################
	# Format the sequence into slices separates by a space
    
    sub __FormatLabel
    {
        my ($label, $width) = @_;
        my $rvl = "";
        my $fmt = '%'.$width.'s';
        $rvl = sprintf($fmt,$label);
        return $rvl;
    }
    
	sub __FormatedSliceSequence
	{
		my ($sequence,$start,$end,$word_len) = (shift,shift,shift,shift);
		my $return = "";

		# Test parameters
		return $return if (!defined $sequence);
		$start		 = 0  if (!defined $start);
		$end		 = -1 if (!defined $end);
		$word_len    = 10 if (!defined $word_len);

		# Extract sequence slice
		my @a_sequence = (split('',$sequence))[$start..$end];

		# Format this slice
		for (my $i=0; $i<scalar(@a_sequence); $i++)
		{
			$return .= " " if ($i%$word_len == 0);
			$return .= $a_sequence[$i];
		}

		return $return;
	}

	# Count gaps numbers of the sequence
	sub __CountGaps
	{
		my $sequence = shift;
		my $return = 0;

		# Test parameters
		return $return if (!defined $sequence);

		# Extract sequence slice
		my @a_sequence = split('',$sequence);

		# Format this slice
		foreach my $letter (@a_sequence)
		{
			$return++ if ($letter eq "-");
		}

		return $return;
	}

	# Format sequence adding gaps
	sub __GapSequenceToString
	{
		my ($file,$ra_gaps,$strand) = (shift,shift,shift);
		my $sequence = "";

		# Test parameters
		return $sequence if (!defined $file);
		$strand = "" if (!defined $strand);

		# Open file and get sequence :
		my $handler = new IO::File($file);
	    while(my $line = <$handler>)
	    {
	    	next if ($line =~ /^\>/);
	  		chomp($line);
	        $sequence .= "$line";
	    };
	    $handler->close();

		# Get reverse complement if needed
		if (lc($strand) =~ /minus/)
		{
			$sequence = join("", reverse( split //, $sequence ) );
			$sequence =~ tr/atgcATGC/tacgTACG/;
		}

	    # Add gaps :
	    foreach my $gap (@$ra_gaps)
		{
		    my $head_seq  = substr($sequence,0,$gap);
			my $tail_seq  = substr($sequence,$gap);
			$sequence = $head_seq."-".$tail_seq;
		}


		return $sequence;
	}

	# Return mismatch array into a string, adding gaps
	sub __MismatchToString
	{
		my ($len,$ra_mismatch,$ra_ref_gap,$ra_tgt_gap) = (shift,shift,shift,shift);
		my $sequence = "|";

		push (@$ra_mismatch,@$ra_ref_gap,@$ra_tgt_gap);
		@$ra_mismatch = sort {$a<=>$b} @$ra_mismatch;

		# Test parameters
		return $sequence if (!defined $len or $len<=0);
		$sequence = "$sequence" x $len;

	    my @a_sequence = split ('',$sequence);
	    foreach my $mis_id (@$ra_mismatch)
	    {
	    	$a_sequence[$mis_id] = ' ';
	    }
	    $sequence = join('',@a_sequence);

		return $sequence;
	}
}

=head2 __GlintFish

 Title     : __GlintFish
 Usage     : $o_xxx->__GlintFish()
 Function  : According to the availability of Fish Probes calls the corresponding method
 Args      : none
 Return    : an html string
=cut

sub __GlintFish
{	
	my $self = shift;
	
	my $html;
	if ( $self->__IsFishProbeFishAvailable( ) )
	{
		$html = $self->__ViewGlintFish();	
	}
	else
	{
		$html = $self->__EmptyGlintFish() ;		
	}
	
	return $html;
}

=head2 __LaunchGlintFish

 Title     : __LaunchGlintFish
 Usage     : $o_xxx->__LaunchGlintFish()
 Function  : Launch the whole GlintFish process (selection of probes)
 Args      : none
 Return    : an html string
=cut

sub __LaunchGlintFish
{	
	my $self = shift;
	
	# The objects defined by the Narcisse API
	my $o_instanceObserver = $self->_Get( '_o_io' );
	my $o_param = $self->_Get('_o_param');
	my $o_logger= $self->_Get('_o_logger');
	
	# The parameter  __LaunchGlintFish describes the region as follows: species:chrom:start:end
	my $region  = $o_param -> Get ( 'LaunchGlintFish' );
	my ( $species, $chrom, $start, $end) = split( ':',$region );
	my $o_so    = $o_instanceObserver -> GetSpeciesObserver( $species );
	my $o_wb    = $self->_Get('_o_wb'); #Not in MyComputation example
   	
   	#Here Get default values for form
   	my $rh_cfg = $o_instanceObserver -> GetConfigHash();
   	
	#Looking at those for PRIMER3 and setting values (fetched from defaults.cfg file)
	my $probe_size = $rh_cfg -> { GLINTFISH_PRIMER3_PRIMER_PRODUCT_SIZE_RANGE };
   	my $primer_size = $rh_cfg -> { GLINTFISH_PRIMER3_PRIMER_OPT_SIZE };
   	my $max_primer_size = $rh_cfg -> { GLINTFISH_PRIMER3_PRIMER_MAX_SIZE};
   	my $min_primer_size = $rh_cfg -> { GLINTFISH_PRIMER3_PRIMER_MIN_SIZE};
   	my $tm_value = $rh_cfg -> { GLINTFISH_PRIMER3_PRIMER_OPT_TM };
   	my $max_tm_value = $rh_cfg -> { GLINTFISH_PRIMER3_PRIMER_MAX_TM };
   	my $min_tm_value = $rh_cfg -> { GLINTFISH_PRIMER3_PRIMER_MIN_TM };
   	my $probe_max_gc = $rh_cfg -> { GLINTFISH_PROBE_MAX_GC_PERCENT };
   	
   	# Force the parameter ck_personal, to display personal data
	$o_param -> Set ( 'ck_personal','1');
	$self -> _SaveParamParser();
	
   	my $file   = File::Spec->catfile( $o_instanceObserver->GetCfg( 'ROOT_INC' ), 'launchglintfish.inc' );
   	
	my $html   = $self->_IncludeFromTmpl( $file, 
										species => $species, 
										chrom => $chrom, 
										start => $start,
										end => $end,  
										probe_size => $probe_size, 
										primer_size => $primer_size,
										max_primer_size => $max_primer_size,
										min_primer_size => $min_primer_size,
										tm_value => $tm_value,
										probe_max_gc => $probe_max_gc, 
										max_tm_value => $max_tm_value,
										min_tm_value => $min_tm_value,
										'__wb_session' => $o_wb->GetSession() );
	
	#Here just as a reminder start and stop of extract sequence									
	my $start_chrom = $start;
	my $end_chrom = $end;	
	
	return $html;
}

=head2 __GlintFish

 Title     : __GlintFish
 Usage     : $self->__GlintFish()
 Function  : THIS IS A DEMO  FUNCTION
             You may use this code as a template to write your own functions
 Args      : none
 Return    : The html code
 Errors	   : IOException
 Globals   : none
=cut
sub __ViewGlintFish
{
	my $self = shift;
	print "Application started ";
	print time();
	
	#The objects defined by Narcisse API
	my $o_instanceObserver = $self->_Get( '_o_io' );
	my $o_param = $self->_Get('_o_param');
	my $o_logger= $self->_Get('_o_logger');
	
	# The parameter GlintFish describes the region as follows: species:chrom:start:end
	my $region  = $o_param -> Get ( 'GlintFish' );
	my ( $species, $chrom, $start, $end) = split( ':',$region );
	
	#warn "$species, $chrom, $start, $end \n";

	#Retrieving the region probes
	my $ra_data = $self->__GlintFishProbes( $species, $chrom, $start, $end );
								
	#Reformat the probes into an array
	my $ra_selprobes = __FormatArrayProbes( $ra_data );
	
	#Write probe results in the text file
	my $probe_file = $self->CalcCsvNames( 1, "probes" );
	__WriteProbeFile( $probe_file, $ra_data ); 
	
	my $file   = File::Spec->catfile( $o_instanceObserver->GetCfg( 'ROOT_INC' ), 'glintfish.inc' );
	my $html   = $self->_IncludeFromTmpl(	$file, 
											species => $species, 
											chrom => $chrom, 
											start => $start, 
											end => $end,
											probe_size => "",
											primer_size => "",
											tm_value => "",
											probe_max_gc => "0.6",
											 );
	
	#Write probe results in the html format
	$html .= "<h2> Selected probes </h2>";
	$html .= &NARCISSE_utils::TabArray2Html( $ra_selprobes );
		
	my $http_probe_file = $self->CalcCsvNames( 0, "probes" ); 	
	$html .= "<h2>Download files</h2>";
	$html .= "<ul> ";
	$html .= "<li> <a href=\"$http_probe_file\" >Probes</a></li>\n";
	
	# Finally write probe results in the session dir as personal data
	# Force the parameter ck_personal, to display personal data
	$o_param -> Set ( 'ck_personal','1');
	$self -> _SaveParamParser();
	$self->__CreateNarcissePrivateInput( $species, $chrom, $ra_data );
		
	return $html;
}

=head2 __EmptyGlintFish

 Title     : __EmptyGlintFish
 Usage     : $o_xxx->__EmptyGlintFish
 Function  : Returns a simple html message
 Args      : none
 Return    : the html string
=cut

sub __EmptyGlintFish
{
   	my $self = shift;
   
    my $o_param = $self->_Get('_o_param');
	my $region  = $o_param -> Get ( 'GlintFish' );
	my ( $species, $chrom, $start, $end) = split( ':',$region );
   
   	#The objects defined by Narcisse API
	my $o_instanceObserver = $self->_Get( '_o_io' );
   
   	my $file   = File::Spec->catfile( $o_instanceObserver->GetCfg( 'ROOT_INC' ), 'glintfish.inc' );
	my $html   = $self->_IncludeFromTmpl( 	$file, 
											species => $species, 
											start => $start, 
											chrom => $chrom, 
											end => $end,
											probe_size => "",
											primer_size => "",
											tm_value => "",
											probe_max_gc => ""
											  );	
	$html .= "<b>Fish probe (GlintFish) functionality not available for this species</br><br>";
		
	return $html;
}


=head2 __IsFishProbeFishAvailable

 Title     : __IsFishProbeFishAvailable
 Usage     : $o_xxx->__IsFishProbeFishAvailable
 Function  : Check the probe repository data_path/probe
				return true unless inexistent of empty directory
 Args      : none
 Return    : TRUE or FALSE
=cut

sub __IsFishProbeFishAvailable
{	
	my $self = shift;
	
	my $o_param = $self->_Get('_o_param');
	my $region  = $o_param -> Get ( 'GlintFish' );
	my ( $species, $chrom, $start, $end) = split( ':',$region );
		
	#The objects defined by Narcisse API
	my $o_instanceObserver = $self->_Get( '_o_io' );		
	my $o_speciesObserver  = $o_instanceObserver->GetSpeciesObserver( $species );	
	my $data_path = $o_speciesObserver->GetDataPath( 'INSTANCE' );		
	my $probe_path = $data_path."/"."probe" ;
	
	my $bool_IsAvailable;	
	if ( -s $probe_path )
	{
		$bool_IsAvailable = &TRUE;
	}
	else
	{
		$bool_IsAvailable = &FALSE;	
	}
	
	return $bool_IsAvailable;
}


=head2 __WriteProbeFile

 Title     : __WriteProbeFile
 Usage     : __WriteProbeFile( $file, $ra_data )
 Function  : Dump the data array into a file
 Args      : output file and a reference on an array
 Return    : none

=cut
sub __WriteProbeFile
{	
	my ( $output, $ra_probes )  = @_;
		
	my $fh = new IO::File( 	$output, "w" )
		or die "Impossible to open $output for writing";	
		
	my @a_fields = qw( element chrom start end size element leftPrimer rightPrimer leftTm rightTm leftEndStability rightEndStability leftSelfEnd rightSelfEnd leftSelfAny rightSelfAny PairPenality PairComplAny);
	print $fh "num"."\t".join("\t",@a_fields)."\n";
	my $num = 1;
	foreach my $r_probe ( @$ra_probes )
	{	
		print $fh $num."\t".join("\t", @{$r_probe}{ @a_fields })."\n";
		$num++;	
	}	
	$fh->close();
	
}

=head2 __GlintFishProbes

 Title     : __GlintFishProbes
 Usage     : $o_xxx->__GlintFishProbes( $species, $chrom, $start, $end  )
 Function  : Compute the file path and call _GetSelectedProbes
 Args      : reference on an array with the probes
 Return    : none

=cut
sub __GlintFishProbes
{
	my $self = shift;
	my ( $species, $chrom, $start, $end) = @_;
	
	#The objects defined by Narcisse API
	my $o_instanceObserver = $self->_Get( '_o_io' );		
	my $o_speciesObserver  = $o_instanceObserver->GetSpeciesObserver( $species );	
	my $data_path = $o_speciesObserver->GetDataPath( 'INSTANCE' );
		
	my $probe_file = $data_path."/"."probe"."/"."chromosome$chrom.probe";
				
	my $ra_data = _GetSelectedProbes(  $probe_file, $start, $end );	
		
	return $ra_data;
}

=head2 _GetSelectedProbes

 Title     : _GetSelectedProbes
 Usage     : _GetSelectedProbes(  file => $probe_file, start => $start, end => $end );
 Function  : Retrieve from a file in Narcisse format the probes corresponding to the specified region 
 Args      : file, start and end
 Return    : reference on an array of hash
=cut
sub _GetSelectedProbes
{
	my ( $file, $start, $end ) = @_;
	
	my @a_data = ();	
	my $o_data = &NARCISSE_utils::ReadChromFeatureData( file => $file, start => $start, end => $end );	
	my @a_fields = qw( chrom start end element leftPrimer rightPrimer size leftTm rightTm leftEndStability rightEndStability leftSelfEnd rightSelfEnd leftSelfAny rightSelfAny PairPenality	PairComplAny);
	foreach my $element_id ( $o_data->GetSortedElements() )
	{
		my %h_probe;
		@h_probe{ @a_fields } = $o_data->GetFields( $element_id, @a_fields );
		push @a_data, \%h_probe;			
	}
	
	return \@a_data;
}

=head2 __FormatArrayProbes

 Title     : __FormatArrayProbes
 Usage     : __FormatArrayProbes( $ra_probes )
 Function  : Reformat the array of probes for printing (html and txt)
 Args      : reference on an array of probes
 Return    : reference on an array of string
=cut
sub __FormatArrayProbes
{
	my $ra_probes = shift;	
	
	my @a_data = ();
	my @a_fields = qw( element leftPrimer rightPrimer chrom start end size);
	push @a_data, "#"."num"."\t".join("\t",@a_fields)."\n";
	my $num = 1;
	foreach my $r_probe ( @$ra_probes )
	{	
		push @a_data, $num."\t".join("\t", @{$r_probe}{ @a_fields })."\n";
		$num++;	
	}
	return \@a_data;
}

=head2 __CreateNarcissePrivateInput

 Title        : __CreateNarcissePrivateInput
 Usage        : $o_xxx->__CreateNarcissePrivateInput(  )
 Function     : Create a file, in the private hierarchy, corresponding to the csv data file format used as an input for Narcisse 
 Returns      : void
 Args         : species name, chrom and reference on an array of probes
=cut
sub __CreateNarcissePrivateInput
{
	my $self = shift;
	my ( $species, $chr, $ra_probes ) = @_;
	
	my $o_instanceObserver = $self->_Get( '_o_io' );		
	my $o_speciesObserver  = $o_instanceObserver->GetSpeciesObserver( $species );			
	my $o_logger= $self->_Get('_o_logger');
	
	my $rh_probes = __FormatProbeOutputForNarcisse( $ra_probes );
     
	my @a_headers = qw( chrom element start end color label );
	my $file_by_chrom = $o_speciesObserver->GetPersoFiles( -check => &FALSE, -chrom => $chr);
	my $o_ndw         = New NarcisseDataWriter::OnMemory( $file_by_chrom, \@a_headers, { append => &TRUE }, undef, $o_logger );
	$o_ndw -> Update($rh_probes);
}

=head2 __FormatProbeOutputForNarcisse

 Title        : __FormatProbeOutputForNarcisse
 Usage        : __FormatProbeOutputForNarcisse(  )
 Function     : Reformat the array of probes in a format suitable for NarcisseDataWriter
 Returns      : a reference on a hash with the probes
 Args         : reference on an array of probes
=cut
sub __FormatProbeOutputForNarcisse
{	
   my $ra_probes  = shift;		
   
   my %h_probes = ();
   my $id = 1;
   my @a_fields = qw( chrom element start end );
   foreach my $r_probe ( @$ra_probes )
   {
	   my ( $chr, $element, $start, $end ) = @{$r_probe}{@a_fields};
	   $h_probes{ $element } ={ chrom => $chr, element => $element, start => $start, end => $end, color => "green", label => "probe$id"};
	   $id++;
   } 
	return \%h_probes;	
}

=head2 __MyComputation

 Title     : __MyComputation
 Usage     : $self->__MyComputation()
 Function  : THIS IS A DEMO  FUNCTION
             You may use this code as a template to write your own functions
 Args      : none
 Return    : The html code
 Errors	   : IOException
 Globals   : none

=cut
sub __MyComputation
{
	my $self = shift;
	
	# The objects defined by the Narcisse API
	my $o_io    = $self->_Get('_o_io');
	my $o_param = $self->_Get('_o_param');
	my $o_logger= $self->_Get('_o_logger');
	
	# The parameter MyComputation describes the region as follows: species:chrom:start:end
	my $region  = $o_param -> Get ( 'MyComputation' );
	my ( $species, $chr, $start, $end) = split( ':',$region );
	my $o_so    = $o_io -> GetSpeciesObserver( $species );

	# This is a rather stupid and useless treatement: the region is separated into 10 subregions,
	# put in a hash of hashes called %h_data.
	# In a REAL code, the treatment should be put in an external module.
    my %h_data;
	my $s = $start;
	my $l = int(($end - $start) / 10);
    for ( my $i=0; $i<10; $i++ )
    {
		my $elt = 'SEG' . $i;
		my %h_row;
		$h_row { 'chrom' }   = $chr;
		$h_row { 'element' } = $elt;
		$h_row { 'start' }   = $s;
		$h_row { 'end' }     = $s + $l - 1;
		$h_row { 'color' }   = ($i % 2 ) ? 'red' : 'blue';
		$h_row { 'label' }   = $elt;
		$h_data { $elt } = \%h_row;
		$s += $l;
	}
	
    # We are using the personal data functionality to display our treatment results to all the Narcisse screens
    # The first and only step is to create or append personal file.
    my @a_headers = qw( chrom element start end color label );
	my $file_by_chrom = $o_so->GetPersoFiles( -check => &FALSE, -chrom => $chr);
	my $o_ndw         = New NarcisseDataWriter::OnMemory( $file_by_chrom, \@a_headers, { append => &FALSE }, undef, $o_logger );
	$o_ndw -> Update( \%h_data );

	# Force the parameter ck_personal, to display personal data
	$o_param -> Set ( 'ck_personal','1');
	$self -> _SaveParamParser();
	
	# Do not forget to write something to the popup. Here, we only display the first and last segments.
	# It is very easy to use the _IncludeFromTmpl function found in NcScript.pm
	my $file   = File::Spec->catfile( $o_io->GetCfg( 'ROOT_INC' ), 'my_comput_reg.inc' );
	my $html   = $self->_IncludeFromTmpl($file, 
										s_seg0 => $h_data{'SEG0'}->{start}, 
										e_seg0 => $h_data{'SEG0'}->{end},
										s_seg9 => $h_data{'SEG9'}->{start}, 
										e_seg9 => $h_data{'SEG9'}->{end}
										);
										
	return $html;
}

1;
