#
# 	sebastien.letort@toulouse.inra.fr
#	Created: 03-07, 2009
#	$Id: NcUploadUserSNPs.pm 3142 2009-12-22 12:38:52Z manu $
#

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

package NcUploadUserSNPs;

=pod

=head1 NAME

 NcUploadUserSNPs - le nom est peut-etre a revoir

=head1  SYNOPSIS

 	$o_script = New NcUploadUserSNPs( $o_io,$o_wb,$o_logger );
 	# prepare the data: the images are computed now if necessary
 	$o_nc_script->PrepareData();

 	#Call Realize, printing everything
 	# will call $o_nc_script->PrintHtml() ie GetHtml() method
 	$o_wb->Realize();

=head1 DESCRIPTION

 This class aims as a first task to load the user sequence
 then compares it/them to a given genome with glint
 and store results into session directory.

=cut

use strict;
use warnings;

use Data::Dumper;
use File::Spec;
use File::Basename;
use File::Path;

use cgilib::NcScript;
use General;

use base qw( NcScript );

use SnpManager;
#use General;

=head2 method _Init

 Usage        : my $o_x = New NcUploadUserSNPs( $o_io,$o_wb,$o_logger );
 Prerequisite : none
 Fonction     : initialise an instance of NcUploadUserSNPs class.
 Returns      : a valid NcUploadUserSNPs object
 Args         :
   $o_io, an InstanceObserver object.
   $o_wb, a WebBuilder object _the one which will draw the page.
   $o_logger, a Logger object
 Attributes   :

=cut


=head2 _PrepareData

 Usage        : $o_x->_PrepareData()
 Function     : 1- save user sequence into session directory.
 Args         :
     none
 Error        : none.

=cut

sub _PrepareData
{
	my $self   = shift;
# 	return;

	my $o_param = $self->_Get( '_o_wb' )->GetObjectParam();

	$self->_Set( __snp_file     => 'snp_input.fasta',
				 __glint_output => 'snp_glint.out' );

	$self->__ComputeAndStoreSnps( $o_param );

	return;
} #_PrepareData


=head2 __ComputeAndStoreSnps

 Usage        : $o_x->__ComputeAndStoreSnps( $o_param )
 Function     : clean input, run glint and store results.
 Args         : $o_param,
 Error        : none.

=cut

sub __ComputeAndStoreSnps
{
	my $self   = shift;
	my( $o_param ) = @_;

	my( $file, $tgt ) = $o_param->Get( qw( UploadFile target1 ) );
	my $o_species = $self->_Get( '_o_io' )->GetSpeciesObserver( $tgt );

	my $o_snp_man = New SnpManager( file      => $file,
	                                -o_logger => $self->GetLogger() );
	$o_snp_man->LaunchGlint( species => $o_species );

	my $rh_snps = $o_snp_man->GetSnpData();

	# store data
	my @a_header = qw( chr element start end color label );
	foreach my $chrom ( keys %$rh_snps )
	{
		my $filepath = $o_species->GetPersoFiles( -chrom => $chrom, -check => &FALSE );
		&NARCISSE_utils::AppendNarcisseData( $filepath, \@a_header, $$rh_snps{ $chrom } );
	}

	return;
} #__ComputeAndStoreSnps


=head2 Function GetHtml

 Title    : Gethtml
 Usage    : $html = $o_ncs->GetHtml()
 Function : Generate and return the html
 Returns  : The html generated
 Args     : none

=cut

sub GetHtml
{
	my $self = shift;

	return "J'y travaille patience !";
# 	# creation d'un autre objet NcScript : Dotplot
# 	my @a_attrs        = qw( _o_wb _o_io );
# 	my( $o_wb, $o_io ) = $self->_Get( @a_attrs );
# 
# # 	my $o_dotplot = New NcDotplot( $o_io, $o_wb, $self->GetLogger() );
# 	my $o_dotplot = New NcHome( $o_io, $o_wb, $self->GetLogger() );
# 
# 	$o_dotplot->PrepareData();
# 
# 	return $o_dotplot->GetHtml();

# 	# display the entry forms
# 	my ($site,$inc) = $o_io->GetCfg('ROOT_SITE','ROOT_INC');
# 	my $gensel_form = $self->_IncludeFromTmpl( "$inc/form/gensel_form.inc" );
# 
# 	# suppression du style par defaut
# 	$gensel_form =~ s/style="display:none"//;
# # warn Dumper $gensel_form;
# 	return $gensel_form;
}


1;

__END__
