#
# 	sebastien.letort@toulouse.inra.fr
#	Created: 03-07, 2009
#	$Id: NcUploadUserSequences.pm 3749 2012-01-12 14:33:28Z manu $
#

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

package NcUploadUserSequences;

=pod

=head1 NAME

 NcUploadUserSequences - le nom est peut-etre a revoir

=head1  SYNOPSIS

 	$o_script = New NcUploadUserSequences( $o_io,$o_wb,$o_logger );
 	# prepare the data: the images are computed now if necessary
 	$o_nc_script->PrepareData();

 	#Call Realize, printing everything
 	# will call $o_nc_script->PrintHtml() ie GetHtml() method
 	$o_wb->Realize();

=head1 DESCRIPTION

 This class aims as a first task to load the user sequence
 then compares it to a given genome
 and store results into session directory.

=cut

use strict;
use warnings;
use POSIX ":sys_wait_h";
use Error qw( :try );
use LipmError;

use DataReader::Whole;
use Data::Dumper;
use File::Spec;
use File::Basename;
use File::Path;
use LipmError::IOException;
use NarcisseError;
use NarcisseDataWriter::OnMemory;
use cgilib::NcScript;
use cgilib::NcScript::NcDotplot;
use cgilib::NcScript::NcHome;
use Sequence_utils;
use General;

use base qw( NcScript );

use Runner;
#use General;

# Si DEBUG == TRUE, on ne fait AUCUN CALCUL - ca marche si on est dans une session ou les calculs ont deja ete faits!
# Le truc est donc de faire un upload avec DEBUG FALSE, puis de recommencer dans la meme session avec DEBUG TRUE

my $DEBUG=&FALSE;
 
=head2 _PrepareData

 Usage        : $o_x->_PrepareData()
 Function     : Not used, everything is in Gethtml, to be able to print messages
 Args         : none
 Error        : none.

=cut

sub _PrepareData
{
	my $self   = shift;
 	return;
}

=head2 Function GetHtml

 Usage    : $html = $o_ncs->GetHtml()
 Function : Print the html, then redirect to the home page
 Returns  : ""
 Args     : none

=cut

sub GetHtml
{
	my $self = shift;
	my $o_io = $self->_Get('_o_io');
	my $o_wb = $self->_Get( '_o_wb' );
	my $o_param= $o_wb->GetObjectParam();
    my $unix_mv = $o_io->GetUnixPath('UNIX_MV');
    my $uploaded_fasta = 'uploaded_seq.fasta';
    
	# Start of page (printing messages)
	print "<div id=\"ID_HOME\"  >";

	my ($fasta_file, $sequence) = $o_param->Get( qw( UploadSeq sequence ) );
    
    # If a sequence was pasted
    if ( $sequence ne '' )
    {
        my $fasta_fh = new IO::File($uploaded_fasta,'w') or throw LipmError::IOException( $uploaded_fasta, 'f' );
        print $fasta_fh $sequence . "\n";
    }

	# If a file was uploaded: Use a standard file name and extract the fasta sequences
    # We must put the file name between quotes because when the user works with some exotic OS (like windows)
    # the file name looks like C:\some where in some disk\file.fasta and the \ must be escaped
    elsif ($fasta_file ne "" )
    {
        `$unix_mv \"$fasta_file\" $uploaded_fasta`;
    }
    
    # If none was done
    else
    {
        throw NarcisseError( -bd => "ERROR: No data to upload" );
    }
    
    my $rh_chrom_info = $self->__ExtractMultiFasta('uploaded_seq.fasta');
    #warn Dumper $rh_chrom_info;
    
	# find a suitable name for this new species and update species.list
    my $user_long = $o_param->Get('UploadSeqName');
	my $rh_species = $self->__UpdateSessionSpeciesList( $user_long );
	my $user_short= $rh_species->{ short };
	$user_long = $rh_species->{ long };

    # Keep species names in the private attributes
    $self->_Set('__user_short',$user_short);
    $self->_Set('__user_long',$user_long);

	# Updating InstanceObserver with this new species
	$o_io->UpdateGenomesListWithSessionData();

	# Print some info about the uploaded sequences
    $self->__PrintUploadedSummary($rh_chrom_info);
    
    # Converting the files to clean gbk files
    my $sts = $self->__ConvertInputToGenbank( $rh_chrom_info );
	if ( $sts != 0 )
	{
		$self->__PrintErrorAndRedirect("readseq returned the code $sts");
		return;
	}
    
	# launch nc_get_data
	my $o_species = $o_io->GetSpeciesObserver( $$rh_species{ short } );
	my $species_cfg_file = $self->__WriteSpeciesCfgFile( $o_species, $rh_chrom_info );
	if ($DEBUG == &FALSE)
	{
		my $sts = $self->__LaunchGetData($user_short);
		if ($sts != 0)
		{
			$self->__PrintErrorAndRedirect("nc_get_data returned the code $sts");
			return;
		}
	}

	# I need to build a fake .cfg file
	# only to specify the DST as the session datapath
    # TODO SUPPRIMER CELA 
	my $rh_organisms = $self->_Get('_rh_organisms');
	my $ra_species   = $rh_organisms->{ SPECIES };
	my $ra_chroms    = $rh_organisms->{ CHROMS };
	my $ra_programs  = $rh_organisms->{ PROGRAMS };

	my $reference   = $ra_species->[0];
	my $o_reference = $o_io->GetSpeciesObserver( $reference );
	my $ref_cfg     = $self->__WriteSpeciesCfgFile( $o_reference );
    
    # Keep reference names in the private attributes
    $self->_Set('__ref_short',$reference);
    $self->_Set('__ref_long',$o_io->GetSpeciesLongName($reference));
    
	# launch nc_get_genomic_comparison
    print "<p>Now comparing your data to " . $o_io->GetSpeciesLongName($reference) . "</p>";
    if ($DEBUG == &FALSE)
    {
        $sts = $self->__LaunchComparison( $user_short, $reference );
    }
    print "</div>";

	if( $sts == 0 )
	{
		$self->__UpdateLocalSyntenyListFile( $user_short, $reference, 'genomic' );
	}
	else
	{
		$self->__PrintErrorAndRedirect("The comparison program returned the code $sts");
	}
    
    # Print the result of the comparison
    my $rh_rh_data = $self->__ReadComparisonResults ( $user_short, $reference );
    $self->__PrintComparisonResults( $rh_rh_data );
    
	return "";
} # GetHtml

=head2 __PrintErrorAndRedirect
 
 title    : __PrintErrorAndRedirect
 Usage    : __PrintAndRedirect($msg)
 Procedure: Print the error message passed by parameters and redirect to the home page
 Args     : $msg: The message to print

=cut
sub __PrintErrorAndRedirect
{
    my $self = shift;
    my ($msg) = @_;
    
    my $o_io = $self->_Get('_o_io');
    my $o_wb = $self->_Get( '_o_wb' );
    print "<p class=\"Error\">ERROR - $msg<br />The upload is interrupted</p>\n";
    sleep 8;
    my $redirect_url = $o_io->GetCfg('HTTP_NARCISSE_CGI') . '?__wb_session=' . $o_wb->GetSession() . '&amp;script=HOME';
    $o_wb->RedirectJs($redirect_url);
}

=head2 __PrintWarning
 
 title    : __PrintWarning
 Usage    : __PrintWarning($msg)
 Procedure: Print the warning message passed by parameters
 Args     : $msg: The message to print

=cut
sub __PrintWarning
{
    my $self = shift;
    my ($msg) = @_;
    print "<p class=\"Warning\">WARNING - $msg</p>\n";
    sleep 5;
}

=head2 __PrintUploadedSummary
 
 title    : __PrintUploadedsummary
 Usage    : __PrintUploadedSummary($rh_chrom_info)
 Procedure: Using the data returned by __ExtractMultiFasta, print some information about the uploaded file.
 Args     : The multifasta file name

=cut
sub __PrintUploadedSummary
{
    my $self = shift;
    my ($rh_chrom_info) = @_;
    
    my $o_io       = $self->_Get('_o_io'); 
    my $threshold  = $o_io->GetCfg('MAX_UPLOAD_SEQ_SIZE') + 0;
    my $table_body = "";
    
    foreach my $chrom_name (keys %$rh_chrom_info)
    {
        my $rh_c_info  = $rh_chrom_info->{ $chrom_name };
        my $seqlen     = $rh_c_info->{ length } + 0;
        my $trunc      = ( $seqlen >= $threshold ) ? "<b>TRUNCATED</b>" : "&nbsp;";
        $table_body   .= "<tr class=\"even\"><td><b>$chrom_name</b></td><td>$seqlen</td><td>$trunc</td></tr>";
    }
    
    my $user_long  = $self->_Get('__user_long');
    
    my $tmpl_summary = '<table class="TableData">';
    $tmpl_summary   .= '<caption>Your uploaded file for the genomic data: #PERLEVAL|$LONG_NAME#</caption>';
    $tmpl_summary   .= '<tr><th>Uploaded sequence name</th><th>Length</th><th>Comment</th></tr>#PERLEVAL|$TABLE_BODY#</table>';
    my $o_tmpl       = New AnotherTemplate($tmpl_summary);
    $o_tmpl->AddVariable('LONG_NAME',$user_long);
    $o_tmpl->AddVariable('TABLE_BODY',$table_body);
    $o_tmpl->Parse();
    print $o_tmpl->Get('parsed_template');
}
    
=head2 __ExtractMultiFasta
 
 title    : __ExtractMultiFasta
 Usage    : my chrom_name = $self->__buildChromName($header);
 Function : From a fasta file, extract N fasta files and return the information about those sequences
 Return   : A ref to a hash of hash:
              key = the chromosome name
              value = A hash with keys:
                      header     => The original fasta header
                      length     => The sequence length
                      fasta_file => The fasta file name
                      gbk_file   => The genbank file name (see __ConvertInputToGenbank)
 Args     : The multifasta file name

=cut
sub __ExtractMultiFasta
{
    my $self = shift;
    my ($mfa_file) = @_;
    my $o_io = $self->_Get('_o_io');
    my %h_chrom_info = ();

    # Read the fasta file
	my $o_fr       = New FastaReader( $mfa_file );
	my $ra_hdr     = $o_fr->GetHeaders();
	if (@$ra_hdr == 0)
	{
		$self->__PrintErrorAndRedirect("There is no sequence in the uploaded file - not a fasta file ?");
		return;
	}

    foreach my $hdr (@$ra_hdr)
    {
        my $chrom_name = $self->__BuildChromName($hdr);
        my $fasta_file = $chrom_name . '.fasta';
        my $gbk_file   = $chrom_name . '.gbk';
        
        # truncating the sequence if necessary
        my $seq        = $o_fr->GetRawSequence($hdr);
        my $threshold  = $o_io->GetCfg('MAX_UPLOAD_SEQ_SIZE') + 0;
        my $seqlen     = length($seq);
        if ($seqlen > $threshold)
        {
            $self->__PrintWarning("the sequence $chrom_name will be truncated to $threshold bases");
            my $truncated_seq = substr($seq,0,$threshold);
            $seq = $truncated_seq;
        }
        
        # Writing the sequence (truncated or not) to a new fasta file
		my $formatted_seq = &Sequence_utils::FormatSequenceLine( seq => $seq);
		my $fh_fasta_file = new IO::File(">$fasta_file") or throw LipmError::IOException( $fasta_file, 'f' );
		print $fh_fasta_file "$hdr\n$formatted_seq\n";
        
        # Keeping this chromosome info in memory
        my %h_c_info = (header => "$hdr", length => $seqlen, fasta_file => $fasta_file, gbk_file => $gbk_file);
        if (exists $h_chrom_info{$chrom_name})
        {
            $self->__PrintWarning("The sequence $chrom_name is already loaded - The 1st version will be ignored");
        }
        $h_chrom_info{$chrom_name} = \%h_c_info;
	}
    
    return \%h_chrom_info;
}
        
=head2 __BuildChromName
 
 title    : __BuildChromName
 Usage    : my chrom_name = $self->__buildChromName($header);
 Function : From a fasta header, try to build a clever sequence name.
            Check for genbank, embl, ddbj or Narcisse standard headers
            or try to build a not_too_stupid name
 Return   : The built name
 Args     : $header: the fasta header

=cut
sub __BuildChromName
{
    my $self = shift;
    my ($hdr) = @_;
    
    my $rvl;
    if ($hdr =~ /gb\|([A-Z0-9._]+)\|/i)
    {
        $rvl = $1;
    }
    elsif ($hdr =~ /embl\|([A-Z0-9._]+)\|/i)
    {
        $rvl = $1;
    }
    elsif ($hdr =~ /dbj\|([A-Z0-9._]+)\|/i)
    {
        $rvl = $1;
    }
    elsif ($hdr =~ /^>nc\|.+\|([A-Z0-9._]+) /i)
    {
        $rvl = $1;
    }
    else
    {
        #General case: extract the first name, keep only some characters, keep 10 characters
        $rvl = $hdr;
        $rvl =~ s/^>([^ |]+).+/$1/;
        $rvl    =~ s/[^A-Za-z0-9.\-_]//g;
        if (length($rvl) >=10)
        {
            $rvl =~ s/^(...).+(......)$/$1-$2/;
        }
    };
    return $rvl;
}

=head2 Function __ReadComparisonResults

 Usage    : my $rh_data = $self->__ReadcomparisonResults($user_short,$ref)
 Function : Read the comparison results from the Narcisse Synteny files and store all the matches in a hash of hashes:
 Return   : $rh_rh_data
              keys: $level -> $chrom_name
              data: DATA  => $rh_data (result from DataReader)
 Args     : none

=cut

sub __ReadComparisonResults
{
	my $self = shift;

	my $user_short = $self->_Get('__user_short');
	my $ref        = $self->_Get('__ref_short');

	my $o_io = $self->_Get('_o_io');

	my $level = $o_io->GetCfg('SYNT_LEVEL_MAX');
	$level += 0;
	#warn "coucou $user_short $ref $level";

	my $o_sp  = $o_io->GetSpeciesObserver( $user_short );
	my $rh_chroms = $o_sp->GetChromsInfo();
	my @a_chroms  = keys %$rh_chroms;

	my $rh_rh_data;
	for (my $l = 0; $l<= $level; $l++)
	{
		foreach my $chr (@a_chroms)
		{
			my $data_file = $o_sp->GetSyntFiles(
				target => $ref,
				prog => 'genomic',
				level => $l,
				-chrom => $chr,
				-where => 'SESSION',
				-check => &FALSE);
			###warn "coucou $level $data_file " . ((-e $data_file) ? "OUI" : "NON");
			if (-e $data_file)
			{
				my $o_ndr = New DataReader::Whole();
				$o_ndr->Prepare( -from => $data_file );
				my $rh_data = $o_ndr->FetchallHashref( 'element', -line_type => 'HASH' );
				$rh_rh_data->{$l}->{$chr} = $rh_data;
			}
		}
	}
	#warn Dumper $rh_rh_data;
	return $rh_rh_data;
}

=head2 __PrintComparisonResults
 
 title    : __PrintComparisonResults
 Usage    : $self->__PrintComparisonResults($rh_rh_data)
 Procedure: Print the results returned by __ReadComparisonResults
 Args     : a hash, should be returned by __ReadComparisonResults

=cut
sub __PrintComparisonResults
{
    my $self = shift;
    my ($rh_rh_data) = @_;
    
    my $user_short = $self->_Get('__user_short');
    my $ref        = $self->_Get('__ref_short');

    my $o_io = $self->_Get('_o_io');
    my $long_name = $o_io->GetSpeciesLongName($user_short);
    my $ref_name  = $o_io->GetSpeciesLongName($ref);

    if ( keys %$rh_rh_data == 0)
    {
        print "ERROR";
        return;
    };
    
    # Select the highest level to display
    my @a_levels = sort keys %$rh_rh_data;
    my $level    = $a_levels[$#a_levels];

    print "<h1>Nucleic comparison between $long_name and $ref_name</h1>";

    my $parity = 0;
    print "<table class=\"TableData\">";
    print "<tr><th>species</th><th>chromosome</th><th>start</th><th>end</th><th>range</th><th>sign</th></tr>\n";
    my $rh_lvl_data = $rh_rh_data->{$level};
    foreach my $chr (keys %$rh_lvl_data)
    {
       # warn "coucou $level $chr";
        my $rh_data = $rh_lvl_data->{$chr};
        #print '<pre>'.Dumper $rh_d.'</pre>';
        foreach my $elt (keys %$rh_data)
        {
            $self->__PrintComparisonResultsOneMatch($parity,$rh_rh_data,$level,$chr,$elt);
            $parity = ! $parity;
        }
    }
    print "</table>\n";
}    

=head2 __PrintComparisonResultsOneMatch
 
 title    : __PrintComparisonResultsOneMatch
 Usage    : $self->__PrintComparisonResultsOneMatch($parity,$rh_rh_data,$level,$chr,$elt)
            Called by __PrintComparisonResults
 Procedure: Print the results of one match only as a couple of table raws
 Args     : $parity: 0/1, used to use an even or odd class
            $rh_rh_data The matches, returned by __ReadComparisonResults
            $level,$chr,,$level: Uniquely identifies the match to print

=cut
sub __PrintComparisonResultsOneMatch
{
    my $self = shift;
    my ($parity,$rh_rh_data,$level,$chr,$elt) = @_;

    my $class = ($parity) ? 'odd' : 'even';
    
    my $user_short = $self->_Get('__user_short');
    my $ref_short  = $self->_Get('__ref_short');
    my $user_long  = $self->_Get('__user_long');
    my $ref_long   = $self->_Get('__ref_long');
    
    my $rh_d    = $rh_rh_data->{$level}->{$chr}->{$elt};
    my $range   = $rh_d->{end} - $rh_d->{start};
    my $s_range = $rh_d->{s_end} - $rh_d->{s_start};
    my $sign    = $rh_d->{sign};
    my $url_user= $self->__GetSyntenyUrl($user_short,$ref_short,$level,$chr,$rh_d->{start},$rh_d->{end});
    my $url_ref = $self->__GetSyntenyUrl($ref_short,$user_short,$level,$rh_d->{s_chrom},$rh_d->{s_start},$rh_d->{s_end});
    
    print "<tr class=\"$class\" ><td>$user_long</td><td><strong><a  class=\"ExtLinks\" href=\"$url_user\">$chr</a></strong></td><td>$rh_d->{start}</td><td>$rh_d->{end}</td><td>$range</td><td>$sign</td></tr>\n";
    print "<tr class=\"$class\" ><td>$ref_long</td><td><strong><a  class=\"ExtLinks\" href=\"$url_ref\">$rh_d->{s_chrom}</a></strong></td><td>$rh_d->{s_start}</td><td>$rh_d->{s_end}</td><td>$s_range</td><td>&nbsp;</td></tr>\n";
}

=head2 __GetSyntenyUrl
 
 title    : __GetSyntenyUrl
 Usage    : $self->__GetSyntenyUrl($target,$ref,$level,$chr,$start,$end)
            Called by __PrintComparisonResultsOneMatch
 Function:  Return the url computed from the arguments
 Args     : $ref,$target,$level,$chr,$start,$end

=cut
sub __GetSyntenyUrl
{
    my $self = shift;
    my ($ref,$target,$level,$chr,$start,$end) = @_;
    
    $level -= 1 unless ($level==0);
    my $view_level = ($level==0) ? 0 : 1;
    
    my $o_param = $self->_Get('_o_param');
    my $session = $o_param->Get('__wb_session');
    my $range   = $end-$start;
    $start -= int $range/4;
    $end   += int $range/4;
    $range = "$start:$end";
    my $o_io = $self->_Get('_o_io');
    my $url  = $o_io->GetCfg('HTTP_NARCISSE_CGI');
    $url .= "?script=DISPLAY&amp;__wb_session=$session&amp;target1=$target&amp;program_target1=genomic&amp;reference=$ref&amp;chroms_reference=$chr&amp;level=$level&amp;view_level=$view_level&amp;range=$range&amp;ck_exon=1";
    return $url;
}


# PAREIL - OBSOLETE
sub __PrintComparisonResults_COMPLETE_TREE
{
    my $self = shift; 
    my ($rh_rh_data) = @_;
   
    my $user_short = $self->_Get('__user_short');
    my $ref        = $self->_Get('__ref_short');

    #print "<pre>";
    #print Dumper $rh_rh_data;
    #print "</pre>";
    #return;
    
    my $o_io = $self->_Get('_o_io');
    my $long_name = $o_io->GetSpeciesLongName($user_short);
    my $ref_name  = $o_io->GetSpeciesLongName($ref);
    
    if ( keys %$rh_rh_data == 0)
    {
        print "ERROR";
        return;
    };
    
    # Select the highest level to display
    my @a_levels = sort keys %$rh_rh_data;
    my $level    = $a_levels[$#a_levels];
    #warn Dumper \@a_levels;
    
    my $inc      = $o_io->GetCfg('ROOT_INC');
    print $self->_IncludeFromTmpl("$inc/uploadseq/results.inc", LONG_NAME => $long_name, REF_LONG_NAME => $ref_name);

    print "<div id=\"ID_MATCH_TREE\"><ul>\n";
    my $rh_lvl_data = $rh_rh_data->{$level};
    foreach my $chr (keys %$rh_lvl_data)
    {
       # warn "coucou $level $chr";
        my $rh_data = $rh_lvl_data->{$chr};
        my $parity = 0;
        #print '<pre>'.Dumper $rh_d.'</pre>';
        foreach my $elt (keys %$rh_data)
        {
            print "<li>";
            $self->__PrintComparisonResultsOneMatch($parity,$user_short,$ref,$rh_rh_data,$level,$chr,$elt);
            print "</li>";
        }
    }
    print "</ul></div>\n";
}

sub __PrintComparisonResultsOneMatch_COMPLETE_TREE
{
    my $self = shift;
    my ($parity,$rh_rh_data,$level,$chr,$elt) = @_;
    
    my $user_short = $self->_Get('__user_short');
    my $ref        = $self->_Get('__ref_short');

    my $rh_d    = $rh_rh_data->{$level}->{$chr}->{$elt};
    my $range   = $rh_d->{end} - $rh_d->{start};
    my $s_range = $rh_d->{s_end} - $rh_d->{s_start};
    my $sign    = $rh_d->{sign};
    my $url_user= $self->__GetSyntenyUrl($user_short,$ref,$level,$chr,$rh_d->{start},$rh_d->{end});
    my $url_ref = $self->__GetSyntenyUrl($ref,$user_short,$level,$rh_d->{s_chrom},$rh_d->{s_start},$rh_d->{s_end});
    
    my $rvl = "<table class=\"MatchResults\">";
    $rvl .= "<tr><th  style=\"text-align:center\" colspan=\"4\">Synteny level $level - sign $sign </th></tr>\n";
    $rvl .= "<tr><th>chrom</th><th>start</th><th>end</th><th>range</th></tr>\n";
    $rvl .= "<tr><th><a href=\"$url_user\">$chr</a></th><td>$rh_d->{start}</td><td>$rh_d->{end}</td><td>$range</td></tr>\n";
    $rvl .= "<tr><th><a href=\"$url_ref\">$rh_d->{s_chrom}</a></th><td>$rh_d->{s_start}</td><td>$rh_d->{s_end}</td><td>$s_range</td></tr>\n";
    $rvl .= "</table>";
    print $rvl;

    my $q_chain = $rh_d->{q_chain};
    return if ($q_chain eq ':');
    
    my @a_chains = split (':',$q_chain);

    # 
    my $chained_level = $level - 1;
    print "<ul>";
    foreach my $elt (@a_chains)
    {
        #my $rh_d = $rh_rh_data->{$chained_level}->{$chr}->{$elt};
        #print '<pre>'.Dumper($rh_d).'</pre>';
        print "<li>";
        $self->__PrintComparisonResultsOneMatch($parity,$user_short,$ref,$rh_rh_data,$chained_level,$chr,$elt);
        print "</li>";
    }
    print "</ul>";
    
    
};

=head2 __UpdateLocalSyntenyListFile

 Usage        : $self->__UpdateLocalSyntenyListFile( $ref, $target, $prog )
 Function     : .
 Args         :
     $ref, $target, $prog
 Error        : none.

=cut

sub __UpdateLocalSyntenyListFile
{
	my $self   = shift;
	my( $ref, $target, $prog ) = @_;

	my ($o_io, $o_wb) = $self->_Get( '_o_io','_o_wb' );
	my $synt_file = $o_io->GetSyntenyListFilepath( 'SESSION' );

	my ( %h_ref, %h_target );
	my @a_header           = qw( num_comp reference target program );
	@h_ref{ @a_header }    = ( 1, $ref, $target, $prog );
	@h_target{ @a_header } = ( 2, $target, $ref, $prog );
	my $rh_data            = { 1 => \%h_ref, 2 => \%h_target };

	my $o_ndw = New NarcisseDataWriter::OnMemory( $synt_file, \@a_header, { sort => &FALSE, append => &TRUE } );
	$o_ndw -> Update( $rh_data );

	$o_io -> SetSessionDirectory( $o_wb -> GetSession() );	

	return;
} #__UpdateLocalSyntenyListFile

=head2 __ExtractExeFromCmd

 Usage        : my $exe = $self->__ExtractExeFromCmd("cmd")
 Function     : From a complete cmd (/usr/bin/ls /usr/lib) axtract the executable part (ls)
 Args         : $cmd the command
 Return       : $exe the executable

=cut
sub __ExtractExeFromCmd
{
    my $self  = shift;
    my ($cmd)  = @_;
    
    my @a_cmd  = split(' ',$cmd,2);
    my @a_path = split('/',$a_cmd[0]);
    return $a_path[-1];
}

=head2 __LaunchGetData

 Usage        : if ( $self->__LaunchGetdata($cfg) == &TRUE ) ...
 Function     : Launch nc_get_data.pl.
 Args         : $cfg the species short name
 Return       : The status, 0 if OK

=cut
sub __LaunchGetData
{
    my $self    = shift;
    my ( $cfg ) = @_;
    my $o_io    = $self->_Get( '_o_io' );

    # launch nc_get_data
    my $prog = $o_io->GetCfg( qw( ROOT_NC_GET_DATA ) );
    my $sess_path = $o_io->GetRootSessDir();
    my $cmd = "$prog --cfg $cfg --no_check --no_suppdata --no_mail --sessdir " . $o_io->GetSessionDirectory();

    $self->_Trace( "running $cmd\n", '*', 'IU' );
    print "<p>Now converting data to the Narcisse format, and indexing files</p>";
    my ( $status, $killed ) = RunExt( -cmd => $cmd, -stdout => '>/dev/null' );
    $self->_Trace( "end of " . $self->__ExtractExeFromCmd($cmd) . " status $status\n", '*', 'IU' );
    
    return $status;
}
    
=head2 __LaunchComparison

 Usage        : $bool = $o_x->+( $cfg_q, $cfg_s )
 Function     : .
 Args         :
     $cfg_q
     $cfg_s
 Return       : The error code, if 0: OK
 Error        : none.

=cut

sub __LaunchComparison
{
	my $self   = shift;
	my( $cfg_q, $cfg_s ) = @_;

	my $o_io = $self->_Get( '_o_io' );

	# launch genomic_comparison
	my $prog = $o_io->GetCfg( qw( ROOT_NC_GENOMIC_COMPARISON ) );
	my $cmd  = "$prog --method=genomic  --cfgQ $cfg_q --cfgS $cfg_s";
	$cmd .= " --chaining_param 200000-500-0:700000-1000-0.8:1300000-1000-0.7";
	$cmd .= " --sessdir " . $o_io->GetSessionDirectory();

	$self->_Trace( "running $cmd\n", '*', 'IU' );
    
    # We run the nc_comparison script, redirecting the output to /dev/null (TODO => log the stuff ?)
    # We don't wait for the end of the process
    my $pid = RunExtNoWait( -cmd => $cmd, -stdout => '>/dev/null' );
    while (&__IsStillRunning($pid))
    {
        print '.';
        sleep 2;
    }
    print '<br />';
    
    # we should return immediately, terminating the zombie child
    waitpid $pid,0;
    
    # $? contains the job's status
    # does not work with the waitpid version of __IsStillRunning
    my $status = $? >> 8;
	$self->_Trace( "end of " . $self->__ExtractExeFromCmd($cmd) . " $status\n", '*', 'IU' );

    return $status;

    # __IsStillRunning executes a ps command to recognize the process state
    # The state is Z (zombie) when the process is finished.
    
    sub __IsStillRunning
    {
        my ($pid) = @_;
        # The following will work on at least on gnu/linux
        my $result = `ps -p $pid -o state=`;
        chomp $result;
        #print $result;
        return $result eq 'Z' ? &FALSE : &TRUE;
    }
    sub __IsStillRunning_DISABLED
    {
        my ($pid) = @_;
        my $rvl = waitpid($pid, WNOHANG);
        return $rvl == -1 ? &FALSE : &TRUE;
    }

     
    
} #__LaunchComparison


=head2 __ConvertInputToGenbank

 Usage    : if ( $self->__ConvertInputToGenbank( $src, $dst ) == &TRUE ) ...
 Function : call readseq to convert input fasta files to genbank format.
 Args     : A hash returned by See
 Return   : The status, 0 if OK, or the error code
 Error    : none

=cut

sub __ConvertInputToGenbank
{
	my $self = shift;
    my ($rh_chrom_info) = @_;
    
    my $o_io = $self->_Get('_o_io');
    my $path_readseq = $o_io->GetCfg('ROOT_READSEQ');
    my ($status, $killed) = (0,0);
    
    foreach my $chrom (keys %$rh_chrom_info)
    {
        my $rh_info = $rh_chrom_info->{$chrom};
        my $src = $rh_info->{'fasta_file'};
        my $dst = $rh_info->{'gbk_file'};
        my $cmd = "$path_readseq -a -format=GenBank -ouput=$dst $src";
        print "Now converting $src => $dst <br />";
        $self->_Trace( "running $cmd\n", '*', 'IU' );
        ( $status, $killed ) = RunExt( -cmd => $cmd );
        $self->_Trace( "end of " . $self->__ExtractExeFromCmd($cmd) . " status $status\n", '*', 'IU' );
    }
    
    return $status;
} 


=head2 __WriteSpeciesCfgFile

 Usage        : my $filename = $o_x->__WriteSpeciesCfgFile( $o_species )
 Function     : Write a minimal species.cfg file.
                DST is session datapath,
                org names are found through $o_species,
                FORMAT is 'GENBANK', and the associated file is the one
                created for the upload sequence.
                The created file will be in session datapath only.
 Returns      : $filename
 Args         : $o_species, a SpeciesObserver object.
                @rh_chrom_info, a hash returned by __ExtractMultiFasta
 Error        : none

=cut

sub __WriteSpeciesCfgFile
{
	my $self = shift;
	my( $o_species, $rh_chrom_info ) = @_;

	# ecriture du fichier cfg ==> besoin d'un objet utilise aussi dans nc_configure
	# je fais avec ParamParaser
	my $o_io = $self->_Get( '_o_io' );
	my( $data_path )  = $o_io->GetDataPaths( 'SESSION' );

	my %h_species_cfg = (	DST       => $data_path,
							ORG_SHORT => $o_species->GetShortName(),
							ORG_LONG  => $o_species->GetLongName(),
							FORMAT    => 'GENBANK' );

	my $sessdir  = $o_io->GetRootSessDir();
	my $gbk_file = File::Spec->catfile( $sessdir, $self->_Get( qw( __gbk_file ) ) );
    if (defined $rh_chrom_info)
    {
        foreach my $chr_name ( keys %$rh_chrom_info )
        {
            my $rh_c_info = $rh_chrom_info -> { $chr_name };
            $h_species_cfg{ 'FILE_GBK_'.$chr_name } = "file://" . File::Spec->catfile( $sessdir, $rh_c_info -> { gbk_file } );
        }
    }
    
	my $species_cfg_file = $o_species->GetCfgFilepath( 'SESSION' );
	my ( $filename, $dirpath, $suffix ) = fileparse( $species_cfg_file );
	my @a_created_dir = mkpath( $dirpath );
	if( 0 != @a_created_dir )
	{
		$self->_Trace( "created directories @a_created_dir\n", '.', 'IUV' );
	}

	my $o_pp_species_cfg = New ParamParser( 'HASH', \%h_species_cfg );
	$o_pp_species_cfg->Dump( $species_cfg_file );

	return $species_cfg_file;
} #__WriteSpeciesCfgFile


=head2 __UpdateSessionSpeciesList

 Usage        : my $rh_species = $o_x->__UpdateSessionSpeciesList()
 Function     : Read species.list file of the session and add a new species into.
 Returns      : $rh_species, a hash ref containing short and long species name.
 Args         : long_name (optional) If specified, use this as a long name
 Error        : none

=cut

sub __UpdateSessionSpeciesList
{
	my $self = shift;

    my ($preferred_name) = @_;
    $preferred_name = "" unless (defined $preferred_name);
    
	my( $o_io, $o_wb ) = $self->_Get( qw( _o_io _o_wb ) );
	my $short;
	my @a_keys = qw( short long where );
	my %h_row = ();

	my $species_file = $o_io->GetSpeciesListFilepath( 'SESSION' );
    
    # The new 'species' short names are genomeN
	my $name = 'genome';
	my $n    = 1;
    if ($DEBUG == &TRUE)
    {
        unlink $species_file;
    }
    
	if( -e $species_file )
	{
		# g pas envie de chercher autre chose.
		# => je lis tout, je compte le nb de ligne, c'est le n° de la nouvelle espece
		my $o_file  = new IO::File( $species_file, 'r' );
		my @a_lines = <$o_file>;
		$n          = @a_lines;
	}

    my $long_name = ($preferred_name eq '') ? "user $name $n" : $preferred_name;
	@h_row{ @a_keys } = ( $name.$n, $long_name, 'SESSION' );
	
	my $o_ndw = New NarcisseDataWriter::OnMemory( $species_file, \@a_keys, { sort => &FALSE, append => &TRUE } );
	$o_ndw -> Update( [ \%h_row ] );

	return \%h_row;
} #__UpdateSessionSpeciesList

1;

__END__
