#
# 	emmanuel.courcelle@toulouse.inra.fr
#	Created: december 05, 2008
#	$Id: WebBuilderFactory.pm 2317 2009-01-22 14:23:43Z sletort $
#

package NcView;

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod
=head1 NAME

 NcView.pm

=head1 SYNOPSIS


=head1 DESCRIPTION

 NcView derives from NCScript, and each object implementing a view should derive from NcView
 NcView implements generic behaviour common to every representation:
    - The main page structure
    - The control block
 Everything is computed by the constructor, who calls _ComputeImages, then the GetHtml method returns the html generated
 Three virtual functions are provided, the derived objects should implement those functions:
    - _GetMainTitle
    - _GetPrintableImage
    - _GetMainImages
    - _ComputeImages
 
=cut

use strict;
use warnings;

use LipmError;
use LipmError::ObjectException;

use Data::Dumper;
use File::Basename;
use Error qw( :try );
use Carp;

use General;
use WebBuilder;
use Runner;
use cgilib::NcScript;
use NARCISSE_utils;

use base qw( NcScript );


=head2 function _Init

 Title        : _Init
 Fonction     : The constructor
                Init WebBuilder and call _ComputeImages
 Args         : $o_io         : the configuration hash
                $o_web_builder: A valid WebBuilder object
                $o_logger     : a Logger

=cut
sub _Init
{
    my $self = shift;
    
    my ($o_io,$o_web_builder,$o_logger) = (@_);
    
    my $o_param = $o_web_builder->GetObjectParam();
    
    #use Data::Dumper;
    #warn Dumper($o_param->{__h_opt});
    
    $self->SUPER::_Init($o_io,$o_web_builder,$o_logger,&TRUE);

    # The reference parameter should be specified for all objects deriving from NcView
    my $ra_species = $self->_Get('_rh_organisms')->{SPECIES};
    if ($ra_species->[0] eq 'none')
    {
        throw NarcisseError( -hd => "NARCISSE ERROR", -bd => "Error in parameters: No reference specified");
    };
  
	###$self->_Set('__ra_features',$self->_GetFeatures());
    
    # Set / unset Ajax mode - We cannot relie on $o_param, because may be _ComputeImages will change '__wba_ajax' for the next request
    if ($o_param->Get('__wba_ajax') eq 'j' || $o_param->GetInteger('__wba_ajax') > 0)
    {
        $self->_Set('__ajax_mode',&TRUE);
    }
    else
    {
        $self->_Set('__ajax_mode',&FALSE);
    }
    
    # Set / unset __print_mode
    if ($o_param->GetInteger('PrintImage') > 0)
    {
        $self->_Set('__print_mode',&TRUE);
    }
    else
    {
        $self->_Set('__print_mode',&FALSE);
    }

    return;
}


=head2 _PrepareData

 Usage        : $self->_PrepareData()
 Function     : Call function _ComputeImages.
 Args         : none
 Error        : none.

=cut

sub _PrepareData
{
	my $self   = shift;

	# Prepare the images
	$self->_ComputeImages();

	return;
} #_PrepareData

sub __GetJson
{
    my $self = shift;
    return $self->_GetMainImagesJson();
}

=head2 Function GetHtml

 Title    : GetHtml
 Usage    : $html = $o_ncs->GetHtml()
 Function : Generate the html code, may be calling some protected virtual functions:
            _GetMainImages, _GetPrintableImage, _GetMainTitle
            __GetCtrlblock is a private function defined in this object
 Returns  : The html generated
 Access   : protected
 Args     : none

=cut

sub GetHtml
{
	my $self = shift;

	my $o_param = $self->_Get('_o_param');
	my $o_io    = $self->_Get('_o_io');

    return $self->__GetJson() if ($self->_Get('_rest') == &TRUE);

	my $html = "";

	# In Ajax mode, we DO NOT update the control block
	if ( $self->_Get('__ajax_mode') )
	{
		# Retrieving the code for displaying ONLY the main images
		$html = $self->_GetMainImages();
	}

	# In Print mode, we call _GetPrintableImage and do not display the controlblock either
	elsif ($o_param->GetInteger('PrintImage') == &TRUE )
	{
		$html = $self->_GetPrintableImage();
	}

	# In Textual mode, we call _GetMainText
	elsif ($o_param -> GetInteger('Textual') == &TRUE )
	{
		$html = $self -> _GetMainText();
	}
	
	# Else, print everything
	else
	{
		my $ctrl_block   = $self->__GetCtrlBlock();
		my $title        = $self->_GetMainTitle();
		my $content       = $self->_GetMainImages();
		my $inc           = $o_io->GetCfg('ROOT_INC');
		my $gensel_form   = "$inc/form/gensel_form.inc";
		my $gensel        = $self->_IncludeFromTmpl($gensel_form);

		$html        = $self->_IncludeFromTmpl("$inc/NcView.inc",
		controlblock => $ctrl_block,
		title        => $title,
		content      => $content,
		gensel       => $gensel
		);
	}

	$self->_Trace("html generated\n","*","M");
	return $html;
}

=head2 __GetCtrlBlock

 Title        : __GetCtrlBlock
 Usage        : my $code = $self->__GetControlBlock();
 Function     : Generate the html code for the control block
 Returns      : html code
 Access       : private
 Args         : none
 Globals      : none

=cut

sub __GetCtrlBlock
{
    my $self = shift;
    
	my $o_param = $self->_Get('_o_param');
    my $o_io    = $self->_Get('_o_io');
    my $inc     = $o_io->GetCfg('ROOT_INC');
    my $site_inc= File::Spec->catfile($o_io->GetCfg('ROOT_SITE'),'Web','inc');
    
    # 1/ Load The tabs managed by the YUI TabView
    my $content;

    $content .= $self->_IncludeFromTmpl("$inc/controlblock/position_form.inc");
    $content .= $self->_IncludeFromTmpl("$inc/controlblock/global_dotplot.inc");

    my $lcontent;
    $lcontent = $self->_IncludeFromTmpl("$inc/controlblock/data_form/level_form.inc");
    $lcontent .= $self->_IncludeFromTmpl("$inc/controlblock/data_form/vlevel_form.inc");

    $content .= $self->_IncludeFromTmpl("$inc/controlblock/data_form.inc", content => $lcontent);

    my $suppl = $self->_IncludeFromTmpl("$site_inc/controlblock/features_form.inc", warning => 0);
    $content .= $self->_IncludeFromTmpl("$inc/controlblock/features_form.inc", suppl => $suppl);

    my $ra_bkmk_list  = &_GetBookmarks($o_io);
    my $bcontent;
    $bcontent .= $self->_IncludeFromTmpl("$inc/controlblock/bookmarks_form/hist_form.inc");
    $bcontent .= $self->_IncludeFromTmpl("$inc/controlblock/bookmarks_form/bkm_form.inc",bookmarks_list=> join(',',@$ra_bkmk_list));
    $content .= $self->_IncludeFromTmpl("$inc/controlblock/bookmarks_form.inc",content => $bcontent);
    $content .= $self->_IncludeFromTmpl("$inc/controlblock/tab_popup.inc");

    # 2/ Load the menu of the control block
# TEMPLATE  => Add the new tab in control-block
    my $mcontent;
    $mcontent .= $self->_IncludeFromTmpl("$inc/controlblock/menu.inc", id => 'ID_MENU_POSITION', class => 'menu', link => 'ID_POSITION', text => 'Browse',   title => 'change the zoom level and the position of the window');
    $mcontent .= $self->_IncludeFromTmpl("$inc/controlblock/menu.inc", id => 'ID_MENU_GLOBAL', class => 'menu', link => 'ID_GLOBAL', text => 'Zoom',   title => 'change the zoom level and the position of the window');
    $mcontent .= $self->_IncludeFromTmpl("$inc/controlblock/menu.inc", id => 'ID_MENU_DATA', class => 'menu', link => 'ID_DATA',     text => 'Synteny level',   title => 'Change synteny level, add new data, etc.');
    $mcontent .= $self->_IncludeFromTmpl("$inc/controlblock/menu.inc", id => 'ID_MENU_FEATURES', class => 'menu', link => 'ID_FEATURES', text => 'Features',title => 'Display or hide selected features');
    $mcontent .= $self->_IncludeFromTmpl("$inc/controlblock/menu.inc", id => 'ID_MENU_BOOKMARKS', class => 'menu', link => 'ID_BOOKMARKS',text => 'Bookmarks',title => 'Manage session bookmarks, get the current url');
	$mcontent .= $self->_IncludeFromTmpl("$inc/controlblock/menu.inc", id => 'ID_MENU_TAB_POPUP', class => 'invisible_menu', link => 'ID_TAB_POPUP',text => 'Links',  title => 'Other options');

    # 3/ Load the howtolink hidden form
    my $howtolink = $self->_IncludeFromTmpl("$inc/controlblock/howtolink_form.inc");

    # 4/ Load the upload hidden form
    my $upload    = $self->_IncludeFromTmpl("$inc/controlblock/upload_form.inc");
    
    # 5/ Load the control block
    my $rvl = $self->_IncludeFromTmpl("$inc/controlblock.inc",menu => $mcontent ,content => $content, howtolink => $howtolink, upload => $upload);

    # 6/ Load the dynamic legend
    $rvl .= $self->_IncludeFromTmpl("$inc/dynamiclegend.inc");

    ### DEBUG ONLY $rvl .= "<script type=\"text/javascript\">DspProperties();</script>";
    return $rvl;

=head2 function _GetBookmarks

 Title        : GetBookmarks
 Usage        : GetBookmarks()
 Prerequisite : none
 Fonction     : Read the bookmarks file (if possible) and store them in an array
 Returns      : The array of bookmarks
 Args         : $o_io
 
=cut
   sub _GetBookmarks
   {
       my ($o_io) = (@_);
       
       my $bmk_file_name = $o_io->GetCfg('ROOT_NARCISSE_BOOKMARKS');
       my $bmk_file;
       my $rvl="";
       
       my $rh_bookmarks;
       if (-e $bmk_file_name)
       {
		   my $o_ndr = New DataReader::Whole();
           $o_ndr->Prepare( -from => $bmk_file_name );
           $rh_bookmarks = $o_ndr->FetchallHashref(-line_type => 'HASH');
       }
       else
       {
           my %h_tmp=();
           $rh_bookmarks = \%h_tmp;
       }
       
       my @a_rvl;
       foreach my $bk (sort keys %$rh_bookmarks)
       {
           push(@a_rvl,$rh_bookmarks->{$bk}->{name});
       }
       
       return \@a_rvl;
   }
}

=head2 functions _GetMainTitle,_GetPrintableImage,_GetMainImages

 Title        : _GetMainTitle,_GetPrintableImage,_GetMainImages
 Usage        : $html .= $self->_GetMainTitle()
 Fonction     : generate the title, the printable image, or the main images
 Returns      : html code
 Args         : none
 Access       : Protected, VIRTUAL
 Globals      : none

=cut
sub _GetMainTitle
{
    my $self = shift;
    throw LipmError::ObjectException( $self, 'abstract method', -text => '_GetImageTitle called from NcView' );
}
sub _GetPrintableImage
{
    my $self = shift;
    throw LipmError::ObjectException( $self, 'abstract method', -text => '_GetPrintableImage called from NcView' );
}
sub _GetMainImages
{
    my $self = shift;
    throw LipmError::ObjectException( $self, 'abstract method', -text => '_GetMainImages called from NcView' );
}
sub _GetMainText
{
    my $self = shift;
    throw LipmError::ObjectException( $self, 'abstract method', -text => '_GetMainText called from NcView' );
}
=head2 Procedure _ComputeImages

 Title        : _ComputeImages
 Usage        : $self->_ComputeImage()
 Fonction     : Compute the images
 Args         : none
 Access       : Protected, VIRTUAL
 Globals      : none

=cut
sub _ComputeImages
{
    my $self = shift;
    throw LipmError::ObjectException( $self, 'abstract method', -text => '_ComputeImages called from NcView' );
}

=head2 function _GetFeatures
 Title    : _GetFeatures
 Usage    : my $ra_features = $self->_GetFeatures()
 Function : Return the list of features, but excluding the "hidden" features
            (ie the features which are declared hidden by guidef.cfg)
 Args     : none
 Return   : the list of features
 
=cut
sub _GetFeatures
{
    my $self = shift;
    my $o_io    = $self->_Get('_o_io'); 
	my $o_param = $self->_Get('_o_param');    

	# The core features
    my @a_core_features = qw( synt prbh gene exon snp repeat rna personal hsp cyto contig clone );
    my @a_features;
    foreach my $feature (@a_core_features)
    {
        if ($o_param->GetInteger('hide_ck_'.$feature)==&FALSE)
        {
            push @a_features,$feature;
        }
    }
    
    # The supplementary features: thery are discovered from guidef.cfg
     my $rh_guidef = $o_io -> GetGuiDefHash();
     foreach my $key ( keys %$rh_guidef )
     {
		 if ($key =~ /^ck_(supp_.+)$/ && $o_param->GetInteger('hide_'.$key)==&FALSE)
		 {
			 push @a_features,$1;
		 }
	 }
   
    return \@a_features;
}

=head2 function _GetCheckedFeatures
 Title    : _GetCheckedFeatures
 Usage    : my $ra_features = $self->_GetCheckedFeatures()
 Function : Return the list of CHECKED features
 Args     : none
 Return   : the list of features
 
=cut
sub _GetCheckedFeatures
{
    my $self = shift;
    my $o_io    = $self->_Get('_o_io'); 
	my $o_param = $self->_Get('_o_param');    

	my $ra_features = $self -> _GetFeatures();
	my @a_checked_features;
	foreach my $feature (@$ra_features)
	{
		push @a_checked_features, $feature if ($o_param->GetInteger('ck_'.$feature) == &TRUE);
	}
	return \@a_checked_features;
}
=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license - www.cecill.info

=cut

1;

