#
# 	sebastien.letort@toulouse.inra.fr
#	Created: december 05, 2008
#	$Id: WebBuilderFactory.pm 2317 2009-01-22 14:23:43Z sletort $
#

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

package NcScriptFactory;

=pod

=head1 NAME

	WebBuilderFactory - a factory for Narcisse project

=head1  SYNOPSIS

	# code

=head1 DESCRIPTION

	This factory generate WebBuilder object for each case encounter in narcisse.cgi.

=cut

use strict;
use warnings;

use Data::Dumper;
use File::Basename;
use Error qw( :try );

use LipmError::ParameterException;
use General;
use WebBuilder;

use cgilib::NcScript::NcHome;

=head2 function GetNcScript

 Title    : GetNcScript
 Usage    : $o_script = NcScriptFactory::GetNcScript($o_io,$o_wb)
 Function : Create an object derived from NcScript
            The type of the object is computed from the script parameter, using the SCRIPT_ parameters
            An exception is thrown if the object cannot be created (ie bad script= parameter)
 Return   : a NcScript object
 Args     : $o_io     The configuration object
            $o_wb     A WebBuilder object
            $o_logger A logger

=cut

sub GetNcScript
{
	my ($o_io, $o_wb, $o_logger) = @_;

	my $o_param = $o_wb->GetObjectParam();
	my $script_param = $o_param->Get('script');
    my $script_class;
    try
    {
        $script_class = $o_io->GetCfg('SCRIPT_' . $script_param);
    }
    catch Error with
    {
        my $o_error = shift;
        throw NarcisseError ( -bd => "$script_param is unknown");
    };

	my $module = 'cgilib::NcScript::'.$script_class;
	eval("use $module");
	if ($@ ne "")
	{    # Something wrong in eval (may be the object does not work)
		throw LipmError(-text => "'use $module;' failed\n=> $@");
	}
   
	my $o_obj = $script_class->New($o_io,$o_wb,$o_logger);

	return $o_obj;
}

1;
