#
# 	sebastien.letort@toulouse.inra.fr
#	Created: december 05, 2008
#	$Id: WebBuilderFactory.pm 2317 2009-01-22 14:23:43Z sletort $
#

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

package WebBuilderFactory;

=pod

=head1 NAME

	WebBuilderFactory - a factory for Narcisse project

=head1  SYNOPSIS

	# code

=head1 DESCRIPTION

	This factory generate WebBuilder object for each case encounter in narcisse.cgi.

=cut

use strict;
use warnings;

use Data::Dumper;
use File::Basename;
use Error qw( :try );

use General;
use WebBuilder;
use NARCISSE_utils;

# ----	----	Static methods	----	----

=head2 function GetWebBuilder
 Title        : GetWebBuilder
 Usage        : my $o_wb = WebBuilderFactory::GetWebBuilder(o_io => $o_io);
 Function     : create a WebBuilderobject, calling one of the other functions defined in this module:
                   - If o_io is a NarcisseExplorer, call __BuildMultiInstancesWB
                   - If o_io is an InstanceExplorer, call __BuildInstanceWB 
 Returns      : $o_wb, a WebBuilder object
 Args         : $o_io, $o_param, InstanceObserver and ParamParser objects
                $bool, boolean, true for debug mode

=cut

sub GetWebBuilder
{
    my %h_args = (
        o_io => undef,
        @_ );
    my $o_io = $h_args{o_io};
    
    throw LipmError::ParameterException("Configuration", 'missing') 
    unless (&SafeIsa($o_io,"Configuration"));
    
    return __BuildMultiInstancesWB(@_) if ($o_io->isa("NarcisseObserver"));
    return __BuildInstanceWB(@_) if ($o_io->isa("InstanceObserver"));
    throw LipmError::ParameterException("configuration", 'type'); 
}

=head2 function __BuildMultiInstancesWB

 Title        : __BuildMultiInstancesWB
 Usage        : $o_wb = WebBuilderFactory::__BuildMultiInstancesWB(o_io => $o_io);
 Prerequisite : $ENV{ 'NARCISSE_CFG' } should exist for the instance 'multi'.
 Function     : create a WB object to display the multi instances home page.
                it look for existing instances with NarcisseObserver object.
 Returns      : $o_wb, a WebBuilder object
 Args         : $o_io

=cut

sub __BuildMultiInstancesWB
{
    my %h_args = (
        o_io => undef,
        o_param => undef,
        @_ );

    throw LipmError::ParameterException("NarcisseObserver", 'missing') 
    unless ( &SafeIsa($h_args{ o_io },"NarcisseObserver"));

    my $o_io = $h_args{ o_io };
    my $o_param = $h_args{o_param};
    
	# Create $o_param, no parameters handled
	#my $o_param = New ParamParser();

	# some hardcoded parameters, useful for Webbuilder
    my ($root_web,$root_site,$http_css,$http_logo,$http_documentation,$http_javascript,$http_narcisse_cgi,$root_core) = $o_io->GetCfg('ROOT_WEB','ROOT_SITE','HTTP_CSS','HTTP_LOGO','HTTP_DOCUMENTATION','HTTP_JAVASCRIPT','HTTP_NARCISSE_CGI','ROOT_CORE');
    #$o_param->Set('script','HOME');

    
	# create $o_web_builder
	my $o_web_builder = New WebBuilder( $o_param, '', "$root_web/narcisse_multi.xml",
	                                    '%c' => $http_css,
                                        '%d' => $http_documentation,
                                        '%e' => $http_narcisse_cgi,
	                                    '%l' => $http_logo,
	                                    '%j' => $http_javascript,
	                                    '%v' => $o_io->GetNarcisseVersion(),
	                                    '%W' => $root_web,
	                                    '%S' => $root_site,
	                                    '%R' => $root_core
                                     );

	# No session support, do not display the title
	$o_web_builder->UnsetBehaviour( 'use_session' );
	$o_web_builder->UnsetBehaviour( 'display_title' );

	return $o_web_builder;

} #MultiInstances

=head2 function __BuildInstanceWB

 Title        : __BuildInstanceWB
 Usage        : $o_wb = WebBuilderFactory::__BuildInstanceWB( o_io => $o_io, o_param => $o_param, -debug => $bool );
 Function     : Build a Webbuilder object, in an instance context
                    -if __wba_ajax exists and is equals to 'j', call __BuildJsonRequestWB
                    -else, call __BuildCompleteRequestWB
 Returns      : $o_wb
 Args         : $o_io, $o_param, InstanceObserver and ParamParser objects
                $bool, boolean, true for debug mode

=cut
sub __BuildInstanceWB
{
	my %h_args = ( o_io => undef,
	               o_param => undef,
                   -debug => 0,
	               @_ );
    throw LipmError::ParameterException("InstanceObserver", 'missing') 
    unless ( &SafeIsa($h_args{ o_io },"InstanceObserver"));
    throw LipmError::ParameterException("ParamParser", 'missing') 
    unless (&SafeIsa($h_args{ o_param },"ParamParser"));

    my $o_param = $h_args{o_param};
    if ($o_param->Get('__wba_ajax') eq 'j')
    {
        return WebBuilderFactory::__BuildJsonRequestWB(@_);
    }
    else
    {
        return WebBuilderFactory::__BuildCompleteRequestWB(@_);
    }
}

=head2 function __BuildJsonRequestWB

 Title        : __BuildJsonRequestWB
 Usage        : $o_web_builder = WebBuilderFactory::__BuildJsonRequestWB( o_io => $o_io, o_param => $o_param );
 Prerequisite : none
 Function     : none
 Returns      : $o_web_builder
 Args         : $o_io,$o_param,$debug

=cut

sub __BuildJsonRequestWB
{
	my %h_args = (
        -debug => 0,
        o_io => undef,
        o_param => undef,
        @_ );

    my $o_param = $h_args{o_param};
    my $o_io    = $h_args{o_io};
    
	# object webBuilder - Some xml file must be defined
	# Most substitution strings are not needed, because css, js, etc. files are not needed as well
    my ($root_ajax_xml,$root_tmp,$http_tmp) = $o_io->GetCfg('ROOT_AJAX_XML','ROOT_TMP','HTTP_TMP');
	my $o_web_builder = New WebBuilder( $o_param, '', $root_ajax_xml,
	                                    '%T' => $root_tmp,
	                                    '%t' => $http_tmp,
	                                    '%v' => $o_io->GetNarcisseVersion() );
	$o_web_builder->UnsetBehaviour('encoded_url');
	$o_web_builder->UnsetBehaviour('error_is_security_issue');

	# Init the session directory, refuse the request if __wb_session not present
	if( $o_param->IsDefined('__wb_session') )
	{
		### TODO - AVOIR UN PARAMETRE POUR LA DUREE MAX DE SESSION $o_web_builder->SetOldestSession(100);
		$o_web_builder->InitSession();
	}
	else
	{
		die ("ERROR - the session id SHOULD BE DEFINED for the ajax requests !")
	}

	return $o_web_builder;
}


=head2 function __BuildCompleteRequestWB

 Title        : GetICompleteRequestWB
 Usage        : $o_wb = WebBuilderFactory::GetCompleteRequestWB( o_io => $o_io, o_param => $o_param, -session_flg => &FALSE );
 Returns      : $o_wb
 Args         : $o_io,$o_param,$debug,$flg (&TRUE)
                If $flg is &FALSE, the behaviour use_session is unset and the session is NOT initialized (used by nc_configure)
 

=cut

sub __BuildCompleteRequestWB
{
	my %h_args = ( 
        -debug  => 0,
        o_io    => undef,
        o_param => undef,
        -session_flg => &TRUE,
        @_ );

    my $o_io    = $h_args{ o_io };
    my $o_param = $h_args{ o_param };
    my ($root_narcisse_xml,$narcisse_central,$root_tmp,$http_tmp,$http_css,$root_css,$http_logo,$http_documentation,$http_javascript,$root_javascript,$http_narcisse_cgi,$http_web,$root_web,$root_site,$root_core) = 
    $o_io->GetCfg('ROOT_NARCISSE_XML','NARCISSE_CENTRAL','ROOT_TMP','HTTP_TMP','HTTP_CSS','ROOT_CSS','HTTP_LOGO','HTTP_DOCUMENTATION','HTTP_JAVASCRIPT','ROOT_JAVASCRIPT','HTTP_NARCISSE_CGI','HTTP_WEB','ROOT_WEB','ROOT_SITE','ROOT_CORE');

    my $multi_cgi = $http_narcisse_cgi;
    $multi_cgi =~ s|/[^/]+//?cgi-bin/|/multi/cgi-bin/|;
	my $o_web_builder = New WebBuilder( $o_param,'',$root_narcisse_xml,
	                                    '%v' => $o_io->GetNarcisseVersion(),
	                                    '%n' => $o_io->GetName(),
	                                    '%i' => $o_io->GetVersion(),
	                                    '%e' => $multi_cgi,
	                                    '%t' => $http_tmp,
	                                    '%T' => $root_tmp,
	                                    '%c' => $http_css,
	                                    '%C' => $root_css,
	                                    '%l' => $http_logo,
                                        '%d' => $http_documentation,
	                                    '%j' => $http_javascript,
	                                    '%J' => $root_javascript,
	                                    '%g' => $http_narcisse_cgi,
	                                    '%w' => $http_web,
	                                    '%W' => $root_web,
	                                    '%S' => $root_site,
	                                    '%R' => $root_core );

	# Don't repeat the title
	$o_web_builder->UnsetBehaviour('display_title');
	$o_web_builder->SetBehaviour('using_yui');
    $o_web_builder->SetBehaviour('send_parameters');
    $o_web_builder->UnsetBehaviour('encoded_url');

	if( 1 == $h_args{ -debug } )
	{
		$o_web_builder->UnsetBehaviour('error_is_security_issue');
	}
	else
	{
		$o_web_builder->SetBehaviour('error_is_security_issue');
	}

	# Init the session directory
	if ( &TRUE == $h_args{ -session_flg } )
	{    
		$o_web_builder->SetBehaviour('session_autoclean');
		### TODO - AVOIR UN PARAMETRE POUR LA DUREE MAX DE SESSION $o_web_builder->SetOldestSession(100);
		$o_web_builder->InitSession() if ( -d $root_tmp);
	}
	else
	{
		 $o_web_builder->UnsetBehaviour('use_session');
	 }
	 
	&__AddOtherLogo($o_io,$o_web_builder);

	return $o_web_builder;

=head2 function __AddOtherLogo

	Title       : __AddOtherLogo
	Usage       :	&AddOtherLogo($o_web_builder)
	procedure   :	add to Webbuilder the logos declared in the file logo.cfg
	Args        :	$o_web_builder, a valid web_builder object
    Access      :   private to __BuildCompleteRequestWB
=cut
	sub __AddOtherLogo
	{
		my ( $o_io, $o_web_builder ) = @_;

		my ($cfg,$http_logo) = $o_io->GetCfg('ROOT_OTHER_CFG','HTTP_LOGO');
		return if (! -e $cfg);

		my $o_param = New ParamParser( $cfg );
		$o_param->SetSubstitution( '%o', "$http_logo/other" );
		$o_param->SetBehaviour( 'use_substitution_table' );

		my @a_logos = $o_param->GetKeys( '^logo_[0-9]$' );
		foreach my $k_logo ( @a_logos )
		{
			my $i = $k_logo;
			$i =~ s/logo_//;
			my $id  = "OTHER_$i";
			my ($link,$img_src,$alt)= $o_param->Get("url_$i","logo_$i","alt_$i");
			$o_web_builder->AddLogo({ id => $id, img_src => $img_src, link => $link, alt => $alt});
		}

		return;
	}

}

1;
