#!/usr/bin/perl
#
# $Id: nc_check_updates.pl 3851 2014-02-07 10:04:36Z  $
#
# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


=head1 NAME

 nc_check_updates.pl

=head1 SYNOPSIS

 chins some_instance (but NOT multi !)
 nc_check_updates.pl

=head1 DESCRIPTION

 For each species known to this instance, call nc_get_data.pl --only_check, to know which
 species need to be updated

 You could put the following in your cronfile to check every month:
 
 01 01 01 * * (cd .../narcisse; . .setenv.sh; chins the_instance_to_check; nc_check_updates.pl)

=cut

BEGIN
{		# pathway to our modules
		use File::Basename;
		my $dirprg=dirname($0);
		unshift @INC, "$dirprg/../lib";
		unshift @INC, "$dirprg/../API";
}

use strict;
use warnings;
use Configuration::InstanceObserver;
use Configuration::NarcisseObserver;
use Error qw( :try );
use LipmError;
use LipmError::ParameterException;

my $VERSION = do {my @r = (q$Rev: 3851 $ =~ /\d+/g); $r[0]};

# Main program
MAIN:
{
    if  ( exists $ENV { 'NARCISSE_CFG' } )
    {
        try
        {
            my $o_no          = New NarcisseObserver();
            my $rh_o_instances = $o_no->GetInstanceObservers();
            foreach my $root (keys %$rh_o_instances)
            {
                my $o_io = $rh_o_instances->{$root};
                &CheckInstance($o_io);
            }
        }
        catch LipmError::ParameterException with
        {
            my ( $o_error ) = @_;
            my $o_io = New InstanceObserver();
            &CheckInstance($o_io);
        };
    }        
    else
    {
        die "ERROR - Please execute chins <some_instance> before running this program\n";
    }
}


sub CheckInstance
{
    my ($o_io) = @_;

    my $name        = $o_io->GetName();
        
    my @a_species   = $o_io->GetSpeciesShortName();
    my $cmd_pattern = $o_io->GetUnixPath('UNIX_BASH') . ' -c "';
    $cmd_pattern   .= '. ' . $o_io -> GetCfg('ROOT_CHINS_SH') . " $name ; ";
    $cmd_pattern   .= $o_io -> GetCfg('ROOT_NC_GET_DATA') . ' --only_check --cfg ';
    $cmd_pattern   .= 'SPECIES "';
    
    my $nb_tobeup = 0;
    print "=== INSTANCE " . $name . " ===\n";
    foreach my $sp (@a_species)
    {
        my $cmd = $cmd_pattern;
        $cmd    =~ s/SPECIES/$sp/;
        #warn $cmd;
        my $out = `$cmd`;
        chomp $out;
        if ($out =~ / -1- /)
        {
	        print "$out\n";
	        $nb_tobeup++;
		}
    }
    if ($nb_tobeup==0)
    {
		print "NOTHING TO BE UPDATED\n";
	}
	else
	{
		print "$nb_tobeup SPECIES TO BE UPDATED\n";
	}
}
