#!/usr/bin/perl
#
# $Id: nc_data_for_lipm.pl 3567 2010-12-01 13:30:57Z manu $
#
# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=head1 NAME

 nc_data_for_lipm.pl

=head1 SYNOPSIS

 nc_data_for_lipm.pl --species <short_name> 

=head1 DESCRIPTION

 Cree un fichier .tgz avec les donnees Narcisse qui seront recuperees sur les serveurs du lipm
 AVANT d'executer ce script executez nc_create_gff3.pl afin de generer les fichiers gff3
 
=cut

use FindBin;
use lib "$FindBin::Bin/../lib";
use lib "$FindBin::Bin/../API";
use strict;
use warnings;
use ParamParser;
use File::Basename;
use Configuration::InstanceObserver;
use NARCISSE_utils;
use FastaReader;
use File::Path;
use File::Spec;
use Data::Dumper;
use URI::Escape;

my $VERSION = do {my @r = (q$Rev: 3567 $ =~ /\d+/g); $r[0]};

# Main program
MAIN:
{
    if  ( ! exists $ENV { 'NARCISSE_CFG' } )
    {
        die "ERROR - Please execute chins <some_instance> before running this program\n";
	}

	ParamParser::SetDefaultUsage( \&Usage );
	ParamParser::SetDefaultBehaviour( 'exit_on_getopt_error' );
	my $o_param = New ParamParser('GETOPTLONG','species=s','help');
	$o_param -> AssertDefined ( 'species' );
	my $species = $o_param -> Get ( 'species' );
    my $o_io = New InstanceObserver();
	my $o_sp = $o_io -> GetSpeciesObserver ( $species );
	
	&RemoveTmp($o_sp);
	&CreateKaryotype($o_sp);
	&CreateInfo($o_sp);
	&CopyGff3($o_sp);
	&CreateFasta($o_sp);
	&CreateTar($o_sp);
	&RemoveTmp($o_sp);	
}

sub CreateKaryotype
{
	my ($o_sp) = @_;
	warn "Creation du fichier karyotype.bed\n";
	my $rh_chroms_info = $o_sp->GetChromsInfo();
	my $tmp_dir = $o_sp -> GetTmpPath();
	my $karyo_file = File::Spec->catfile($tmp_dir,'karyotype.bed');
	my $fh_karyo_file = new IO::File $karyo_file, 'w';
	my @a_chrs = keys %$rh_chroms_info;
	my @a_sorted_chrs = NARCISSE_utils::SortByChromID( @a_chrs );
	foreach my $chr ( @a_sorted_chrs )
	{
		print $fh_karyo_file join( "\t", $chr, $rh_chroms_info -> { $chr } -> { length } ) . "\n";
	}
}

sub CreateInfo
{
	my ($o_sp) = @_;
	warn "Creation du fichier info.cfg\n";
	my $o_io = $o_sp -> GetInstanceObserver();
	my $species_dir = $o_io -> GetSpeciesCfgDirpath('');
	my $short = $o_sp -> GetShortName();
	my $cfg_src = File::Spec->catfile($species_dir,$short . '.cfg');
	my $tmp_dir = $o_sp -> GetTmpPath();
	my $cfg_dst = File::Spec->catfile($tmp_dir,'info.cfg');
	`cp -a $cfg_src $cfg_dst`;
}

sub CopyGff3
{
	my ($o_sp) = @_;
	warn "Copie du repertoire gff3\n";
	my $data_dir = $o_sp -> GetDataPath();
	my $gff3_src = File::Spec->catfile($data_dir,'gff3');
	my $tmp_dir = $o_sp -> GetTmpPath();
	`cp -a $gff3_src $tmp_dir`;
}

sub CreateFasta
{
	my ($o_sp) = @_;
	warn "Creation des fichiers fasta\n";
	my $tmp_dir = $o_sp -> GetTmpPath();
	my $short = $o_sp -> GetShortName();
	my $data_dir  = $o_sp -> GetDataPath();
	my $fasta_dir = File::Spec->catfile($tmp_dir,'fasta');
	mkdir $fasta_dir;
	my %h_fasta_list = ("db/cds/${short}_cds.fasta" => 'cds', "db/gene/${short}_gene.fasta" => 'gene', "db/prot/${short}_prot.fasta" => 'prot', "db/transcript/${short}_mrna.fasta" => 'mrna');
	foreach my $f (keys %h_fasta_list)
	{
		my $src = File::Spec->catfile($data_dir,$f);
		my $dst = File::Spec->catfile($fasta_dir,basename($f));
		my $label = $h_fasta_list { $f };
		`perl -pne 's/^>$label:/>lcl|/' $src >"$dst"`;
	}
}

sub CreateTar
{
	my ($o_sp) = @_;
	warn "Creation du fichier tar\n";
	my $short = $o_sp -> GetShortName();
	my $tmp_dir = $o_sp -> GetTmpPath();
	my $data_dir = $o_sp -> GetDataPath();	
	my $tar_file = File::Spec->catfile($data_dir,"$short.tgz");
	`tar czf $tar_file -C $tmp_dir .`;
	print "Le fichier $tar_file est pret\n";
}

sub RemoveTmp
{
	my ($o_sp) = @_;
	warn "Suppression du repertoire tmp\n";
	my $tmp_dir = $o_sp -> GetTmpPath();
	`rm -rf $tmp_dir`;
}

sub Usage
{
	my $self = shift;
	my $exe = basename $0;
	print <<END
Usage : $exe [--species] <species short or long name>

	Generate files for lipm-mdata
END
;
	exit 1;
}
	
