#! /usr/bin/perl
#
# $Id: nc_install.pl 3142 2009-12-22 12:38:52Z manu $
#

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

BEGIN
{
		use File::Basename;
		my $dirprg=dirname($0);
		unshift @INC, "$dirprg/../lib";
		unshift @INC, "$dirprg/../API";
		unshift @INC, "$dirprg/../API/Bio/Narcisse/Utils";
}

use strict;
use warnings;
use IO::File;
use IO::Dir;

use Error ":try";
use LipmError::IOException;

use Getopt::Long;
use ParamParser;
use OnlyOrNo;
use Sufine;
use Cwd;
use General;
use File::Basename;
use Configuration::InstanceObserver;
use NARCISSE_utils;
use Logger;
my $VERSION = do {my @r = (q$Rev: 3142 $ =~ /\d+/g); $r[0]};

#-------------------------------------------------------------------------------
#                            Constantes
#-------------------------------------------------------------------------------

# TODO ET $o_io ALORS ?
my $PATH_DIR_CFG = "/Site/cfg";
my $PATH_PRG_LIST = "/Site/cfg/species.list";

#----------------------------------------------------------------------------
#                      Main Program
#----------------------------------------------------------------------------
                 
MAIN:
{
	try
	{
		&Main();
	}
	catch LipmError with
	{
		my ($err, $r ) = @_;
		print $err->Explain( 1 );
        print $err->stacktrace();
        exit 1;
	}
	otherwise
	{
		my ($err, $r ) = @_;

		print STDERR $err->text(), "\n";
        print $err->stacktrace();
        exit 1;
	};
}

sub Main
{

    #----------------------------------------------------------------------------
    #                      Lecture des parametres
    #----------------------------------------------------------------------------
                 
    &ParamParser::SetDefaultBehaviour('use_substitution_table');
    &ParamParser::SetDefaultUsage(\&Usage);

    my $o_param = New ParamParser;

    # Update from ENV, for $NARCISSE_CFG
    $o_param->Update('ENV','O');
    $o_param->AssertDefined('NARCISSE_CFG');
    $o_param->AssertFileExists('NARCISSE_CFG');

    # Init Instance Observer
    my $o_io = New InstanceObserver();
    
    my $root = $o_io->GetCfg('ROOT');
    my $http = $o_io->GetCfg('HTTP');
    my $narcisse_cgi = $o_io->GetCfg('HTTP_NARCISSE_CGI');
    
    #
    # Print a welcome message
    #
    print '-' x 30 . "\n" . basename($0) . " rel $VERSION: \n" . '.' x 5 . "\n";
    print "NARCISSE_CFG = ".$o_param->Get('NARCISSE_CFG')."\n";

    $o_param->Update('GETOPTLONG','I',('only_configure_syntenies','no_configure_syntenies','only_merge_db','no_merge_db','only_index_data','no_index_data','finish_upgrade','help'));
    $o_param->Update('ENV',"O");

    # Check, compute and print $target
    my $target = "install";
    print "target $target\n" . '-' x 30 . "\n";

    #
    # create the target directory and change to it
    # Then, open the log files
    my $sav_dir     = getcwd();
    my $target_root = $o_io->GetCfg('ROOT_LOGS').'/'.$target;
    my $unix_mkdir  = $o_io->GetUnixPath('UNIX_MKDIR');
    `$unix_mkdir $target_root`;
    my $target_path = Sufine::MkDir(-dir => $target_root,-name => $target);
    die unless(defined $target_path);
    die unless(defined Sufine::ChDir(-dir => $target_root,-name => $target));

    &NARCISSE_utils::RedirectOE($o_io,"narcisse.out","narcisse.err");

    $o_param->Dump("narcisse.param");
    my $o_logger = New Logger (">>$target.log",$o_io->GetCfg('LOG_LEVEL'));
    $o_logger->Trace("Starting nc_install.pl\n","*","A");

    # Check the species installed, disable some of them if necessary, and may be recreate the instance observer
    $o_io = &CheckSpecies($o_io,$o_param);
    
    my $o_only = New OnlyOrNo($o_param);
    
    # Finish the upgrade if specified, may be REMOVING SOME FILES
    if ($o_param->Get('finish_upgrade'))
    {
        $o_io = &FinishUpgrade($o_io,$o_param,$o_logger);
    };
	if ($o_only->Get('configure_syntenies'))
    {
		&ConfigureSyntenies($o_io);
    };
    if ($o_only->Get('merge_db'))
    {
        &_MergeDataBank($o_io);
    }
    if ($o_only->Get('index_data'))
    {
        &_IndexData() unless ($o_param->Get('no_index_data'));
    }

    $o_logger->Trace("OK\n","*","A");

	print STDOUT "Installation finished\n";
	print STDOUT "Now,  you can go to $narcisse_cgi and play with NARCISSE\n\n";
	print STDOUT "If the program fails to start, check the configuration of your web daemon or/and add something like\n\n";
	my($apache)="\tScriptAlias /narcisse/cgi-bin/ \"$root/cgi-bin/\"\n\tAlias /narcisse/ \"$root\"\n\n";
	$apache =~ s/\/\//\//g;
	print STDOUT "$apache";

    # Back to the original directory
    chdir $sav_dir;

    print "Log files created in directory: $target_path\n";
}


#----------------------------------------------------------------------------
#                      Fonctions
#----------------------------------------------------------------------------
=head2 function CheckSpecies

 Title    : CheckSpecies
 Usage    : CheckSpecies()
 Function : For each recorded species, try to open the file karyotype
            If the file cannot be opened, comment out the species in species.list
            Then, create a new Instance observer
 Args     : $o_io: An instance observer
 Return   : $o_io: The same - or another, instance observer

=cut
sub CheckSpecies
{
    my ($o_io, $o_param) = @_;
    
    my %h_bad_species = ();
    foreach my $sp ($o_io->GetSpeciesShortName())
    {
        # Keep the species as bad if we cannot open the file, or if the file is empty
        try
        {
            my $o_so = $o_io->GetSpeciesObserver($sp);
            my $rh_chr_info = $o_so->GetChromsInfo();
            my @a_chrs = keys %$rh_chr_info;
            $h_bad_species{$sp} = 1 if ( @a_chrs == 0 );
        }
        catch Error with
        {
            $h_bad_species{$sp} = 1;
        };
	} 
    
    # If some bad species are found, we write the file back, and we create a new instance observer
    my @a_bad_species = keys %h_bad_species;    
    if (@a_bad_species != 0)
    {
        if ( $o_param->IsDefined('finish_upgrade') )
        {
            print "ERROR - Some bad species are found (no karyotype found)\n";
            print "ERROR - Bad species are: " . join (' ',@a_bad_species) . "\n";
            print "ERROR - I do not want to go further with finished_upgrade if there are bad species\n";
            print "Please correct the problem, OR try again without --finish_ugrade\n";
            exit 1;
        }
        
        my $src_file = $o_io->GetSpeciesListFilepath('INSTANCE');
        my $dst_file = $src_file . '.new';
        my $rh_src_file = new IO::File($src_file) or throw LipmError::IOException($src_file,'f');
        my $rh_dst_file = new IO::File(">$dst_file") or throw LipmError::IOException($dst_file,'f');
        
        # Copy the header
        my $line = <$rh_src_file>;
        print $rh_dst_file $line;
        
        # Copy the other lines, may comment them out
        while ($line = <$rh_src_file>)
        {
            # Warning - we'll have to change this if species.list changes its format
            my ($sp) = ($line =~ /^(\w+)\t/);
            if (exists $h_bad_species{$sp})
            {
                print $rh_dst_file "; The following line commented out by nc_install\n";
                print $rh_dst_file "; " . $line;
                print "Disabled the species $sp\n";
            }
            else
            {
                print $rh_dst_file $line;
            }
        }
        
        $rh_dst_file->close();
        $rh_src_file->close();
        link($src_file,$src_file.'.bck');
        unlink($src_file);
        link($dst_file,$src_file);
        unlink($dst_file);

        # Create a new $o_io 
        $o_io = '';
        $o_io = New InstanceObserver();
    }
    
    return $o_io;
}


=head2 procedure ConfigureSyntenies

 Title        : ConfigureSyntenies
 Usage        : ConfigureSyntenies()
 Prerequisite : none
 Procedure    : generate synteny.list
 Args         : $o_io

=cut

sub ConfigureSyntenies
{
	my ($o_io) = @_;

	#
	# Build the synteny file
	#
	my $file_synteny_list_old = $o_io->GetSyntenyListFilepath();
    my $file_synteny_list     = $file_synteny_list_old . '.new';
    my $fh_synteny_list       = new IO::File(">$file_synteny_list")
    				or throw LipmError::IOException($file_synteny_list,'f');

	# Write the header
	print $fh_synteny_list "#num_comp\treference\ttarget\tprogram\n";

	my $cpt = 1;

	# for every species found in this instance
	foreach my $query ( $o_io->GetSpeciesShortName() )
	{
		my $rh_synt_data = &_SearchSyntData($o_io,$query);

		# for every species used as a "subjt" versus this query
		foreach my $subjt ( $o_io->GetSpeciesShortName() )
		{
			# For each program used for this ($reference, $target) pair (proteic, nucleic, other comparisons)
			my $ra_programs = $rh_synt_data->{$subjt};
			foreach my $prog ( @$ra_programs )
			{
                print $fh_synteny_list $cpt++."\t$query\t$subjt\t$prog\n";
			}
		}
	}
	$fh_synteny_list->close();

	# Install the new synteny.list file
	link $file_synteny_list_old,"$file_synteny_list_old.bkp";
	unlink $file_synteny_list_old;
	link $file_synteny_list, $file_synteny_list_old;
	unlink $file_synteny_list;

	return;
}


=head2 function _SearchSyntData

 Title        : _SearchSyntData
 Usage        : my $rh_data = _SearchSyntData( $o_io, $query );
 Function     : Search the synteny data of a given species
 Args         : $o_io = An instance observer
                $query = A species short name, we look for directories holding data for every subject used versus this query
 Returns      : ref to hash with key   = species (1 key / subject)
 			                     value = ref to array, each element is a program used for comparing $query and this subjt

=cut

sub _SearchSyntData
{
	my ($o_io,$query) = @_;

	my %h_rvl;

	# Where are the root folders ?
	#my $species_folder     = &NARCISSE_utils::GetSpeciesRoot($query);
	my $comparative_folder = File::Spec->catfile($o_io->GetCfg('ROOT_DATA'),$o_io->GetCfg('COMPARATIVE'),$query);

	# The possible subjects are directories under $comparative_folder
    if ( -d $comparative_folder )
    {
        my $dh_comparative_folder =  new IO::Dir($comparative_folder) or throw LipmError::IOException("Cannot open $comparative_folder");
        my @subjt_species         =  File::Spec->no_upwards($dh_comparative_folder->read());
        $dh_comparative_folder->close();
        
        # For each subject, find the directory synt and form the list of directories found here (1 directory/program used)
        foreach my $subjt (	@subjt_species )
        {
            my $program_dir = File::Spec->catfile($comparative_folder,$subjt,$o_io->GetCfg('SYNT'));
            if ( -d $program_dir )
            {
                my $dh_program_dir = new IO::Dir( $program_dir) or throw LipmError::IOException("Cannot open $program_dir");
                my @programs       = File::Spec->no_upwards($dh_program_dir->read());
                $h_rvl{$subjt}     = \@programs;
            }
            else
            {
                warn "Warning: the directory $program_dir does not exists!\n";
            }
        }
    }
    else
    {
        warn "Warning: the directory $comparative_folder does not exists!\n";
    }
	return \%h_rvl;
};


=head2 procedure _IndexData

 Title        : _IndexData
 Usage        : _IndexData()
 Prerequisite : none
 Fonction     : Lance le programme d'indexation des donnees Narcisse pour toutes les especes
 Returns      : none
 Args         : none
 Globals      : none

=cut
sub _IndexData
{
    print "Indexation routine not yet written\n";
    return;
}

=head2 procedure _MergeDataBank

 Title        : _MergeDataBank
 Usage        : _MergeDataBank($o_io)
 Procedure    : Prepare le merge des banques prot et cds de toutes les especes de l'instance
 Args         : none

=cut
sub _MergeDataBank
{
    my ($o_io) = @_;

    my $unix_rm     = $o_io->GetUnixPath('UNIX_RM');
    my $unix_mv = $o_io->GetUnixPath('UNIX_MV');

    my $db_dir      = File::Spec->catfile($o_io->GetCfg('ROOT_DATA'),$o_io->GetCfg('DB'));
    my $db_prot     = File::Spec->catfile($db_dir,$o_io->GetCfg('DB_PROT_MERGE'));
    my $db_cds      = File::Spec->catfile($db_dir,$o_io->GetCfg('DB_CDS_MERGE'));
    
    my $db_dir_new  = File::Spec->catfile($o_io->GetCfg('ROOT_DATA'),$o_io->GetCfg('DB'),'new');
    my $db_prot_new = File::Spec->catfile($db_dir_new,$o_io->GetCfg('DB_PROT_MERGE'));
    my $db_cds_new  = File::Spec->catfile($db_dir_new,$o_io->GetCfg('DB_CDS_MERGE'));

    mkdir $db_dir_new;
    
    # Merge db to the "new" subdirectory
	&MergeDb($o_io,$db_prot_new,$o_io->GetCfg('PROT'));
	&MergeDb($o_io,$db_cds_new,$o_io->GetCfg('CDS'));
    
    # Install the new files
    $db_prot     .= '*';
    $db_cds      .= '*';
    $db_prot_new .= '*';
    $db_cds_new  .= '*';
    
    `$unix_rm $db_prot $db_cds`;
    `$unix_mv $db_prot_new $db_cds_new $db_dir`;
    `$unix_rm -r $db_dir_new`;
    
	return;
}

=head2 procedure _MergeDb

 Title        : _MergeDb
 Usage        : _MergeDb($db_merge,$type,$rh_species)
 Prerequisite : none
 Fonction     : Realise le merge de banque multi fasta
 Returns      : none
 Args         : $o_io    : Instance Observer
                $db_merge: Banque finale
                $type    : type de donnees (prot, cds)

=cut
sub MergeDb
{
    my ($o_io,$db_merge,$type) = @_;

	warn "Merge $type bank\n";

    my $unix_rm = $o_io->GetUnixPath('UNIX_RM');
	`$unix_rm $db_merge`;

	foreach my $species ( $o_io->GetSpeciesShortName() )
	{
		&CatDbPart($db_merge,$o_io->GetDB($type,$species));
	}

	my $is_prot = ( $type eq $o_io->GetCfg('PROT') ) ? "T" : "F";
	my $cde = $o_io->GetCfg('ROOT_FORMATDB') . " -i $db_merge -p $is_prot -t " . $o_io->GetName();
	`$cde`;
	return;
}

=head2 procedure CatDbPart

 Title        : CatDbPart
 Usage        : CatDbPart($db_merge,$db_part)
 Prerequisite : none
 Fonction     : Concatenation de $db_part dans $db_merge
 Returns      : none
 Args         : $db_merge : Banque finale
                $db_part : Banque partiel
 Globals      : none

=cut
sub CatDbPart
{
	my ($db_merge,$db_part) = (shift,shift);

    my $fh_fasta=new IO::File "$db_part" or warn "Warning: $db_part could not be opened";
	my $fh_merge=new IO::File ">>$db_merge" or die "Warning: $db_merge could not be opened";

    return unless ($fh_fasta);

	while ( my $line = <$fh_fasta> )
	{
		if ( $line =~ /sp=(\w+)/ )
		{
			my $species = $1;
			$line =~ s/^>([^:]+)+:/>$species:/;
		}
		print $fh_merge $line;
	}

	$fh_fasta->close();
	$fh_merge->close();

	return;
}

=head2 Function FinishUpgrade

 Title   : FinishUpgrade
 Usage   : FinishUpgrade($o_io,$o_param,$o_logger)
 Function: Finish the upgrade, removing the 'old' species
 Args    : $o_io,$o_param,$o_logger
 Return  : The new instance observer

=cut
sub FinishUpgrade
{
    my ($o_io,$o_param,$o_logger) = @_;
    
    print "WARNING - THIS FUNCTION MAY DESTROY SOME DATA - DO YOU HAVE A BACKUP ? (Y/[N]) ";
    if (&NARCISSE_utils::GetAdmAnswer("","Y|N","N") eq 'Y')
    {
        print "          ARE YOU SURE YOU WANT TO CONTINUE ? (Y/[N]) ";
        if (&NARCISSE_utils::GetAdmAnswer("","Y|N","N") eq 'N')
        {
            warn "Cancelled by the user\n";
            exit 1;
        }
    }
    else
    {
        warn "Please backup your data and try again\n";
        exit 1;
    }
    
    print "OK - Let's go\n";
    
    print "Now modifying the file species.list\n";
    my ( $rh_new_species, $rh_old_species, $rh_unch_species) = &FinishUpgradeSpeciesList( $o_io, $o_logger );
    my @a_new = keys %$rh_new_species;
    if ( @a_new != 0 )
    {
        my @a_upgraded = keys %$rh_new_species;
        my @a_removed  = keys %$rh_old_species;
        my @a_unchanged= keys %$rh_unch_species;
        
        print "Now managing the species directory\n";
        &FinishUpgradeSpeciesCfg ( $o_io, $o_logger, $rh_new_species, $rh_old_species, $rh_unch_species);
        
        print "Now managing the data directory\n";
        &FinishUpgradeData ( $o_io, $o_logger, $rh_new_species, $rh_old_species, $rh_unch_species);
        
        print "Now Re-initializing the instance observer\n";
        $o_io = '';
        $o_io = New InstanceObserver();
    }
    return $o_io;
}

    
=head2 function FinishUpgradeSpeciesList

 Title    : FinishUpgrade
 Usage    : my ( $rh_new_species, $rh_old_species, $rh_unch_species) = &FinishUpgradeSpeciesList( $o_io, $o_logger );
 function : dispatch the species between new, old, unchanged
            Modify species.list, commenting out the old species, 
            removing 'NEW UPGRADE' at the end of the new species's long name
 Args     : $o_io,$o_param,$o_logger
 Return   : 3 hashes (keys = short names, values = info)

=cut
sub FinishUpgradeSpeciesList
{
    my ($o_io, $o_logger) = @_;
    my @a_rh_species = $o_io->GetSpeciesInfo();

    $o_logger->Trace("Now modifying the file species.list\n","*","A");
    
    # Dispatch the species:
    #    - %h_new_species  The new species (upgrade = 1)
    #    - %h_old_species  The old species (upgrade = -1) 
    #    - %h_unch_species The unchanged species (upgrade = 0)
    #
    my (%h_new_species, %h_old_species,%h_unch_species);
    if ( exists $a_rh_species[0]->{ upgrade } )
    {
        foreach my $rh_species (@a_rh_species)
        {
            if (! exists $rh_species->{ upgrade } )
            {
                print "WARNING - There is no upgrade to finish !\n";
                return (\%h_new_species, \%h_old_species, \%h_unch_species);
            }
            my $short = $rh_species->{short};
            $h_new_species{$short} = $rh_species if ($rh_species->{ upgrade } eq '1');
            $h_old_species{$short} = $rh_species if ($rh_species->{ upgrade } eq '-1');
            $h_unch_species{$short}= $rh_species if ($rh_species->{ upgrade } eq '0');
        }
        
        # Remove the NEW UPGRADE from the long name in the new species
        foreach my $species (keys %h_new_species)
        {
            $h_new_species{$species}->{long} =~ s/NEW UPGRADE$//;
        }
        
        # Write back the species.list file, but do not write %h_old_species
        # (in fact we write them, but commented out)
        # The column 'upgrade' is no more useful
        # TODO quand les fonctions WriteNarcisse seront au point ce sera mieux
        my $dst_file = $o_io->GetSpeciesListFilepath('INSTANCE');
        my $rh_dst_file = new IO::File(">$dst_file") or throw LipmError::IOException($dst_file,'f');
        print $rh_dst_file "#short\tlong\twhere\ttaxid\tassembly\ttype\n";
        foreach my $species (keys %h_new_species)
        {
            my $rh_species = $h_new_species{$species};
            print $rh_dst_file $rh_species->{short},"\t",$rh_species->{long},"\t",$rh_species->{where},"\t",$rh_species->{taxid},"\t",$rh_species->{assembly},"\t",$rh_species->{type},"\n";
            $o_logger->Trace("Added a new line to species.list $rh_species->{short}\n","*","ACV");
            
        }
        foreach my $species (keys %h_unch_species)
        {
            my $rh_species = $h_unch_species{$species};
            print $rh_dst_file $rh_species->{short},"\t",$rh_species->{long},"\t",$rh_species->{where},"\t",$rh_species->{taxid},"\t",$rh_species->{assembly},"\t",$rh_species->{type},"\n";
        }
        my @a_now = localtime();
        my $now   = sprintf "%4i-%02i-%02i",$a_now[5]+1900,$a_now[4]+1,$a_now[3];
        print $rh_dst_file "; the following commented out by nc_install.pl --finish_upgrade on $now\n";
        foreach my $species (keys %h_old_species)
        {
            my $rh_species = $h_old_species{$species};
            print $rh_dst_file '; ',$rh_species->{short},"\t",$rh_species->{long},"\t",$rh_species->{where},"\t",$rh_species->{taxid},"\t",$rh_species->{assembly},"\t",$rh_species->{type},"\n";
            $o_logger->Trace("Commented out a new line in species.list $species\n","*","ACV");
        }
    }
    else
    {
        print "WARNING - There is no upgrade to finish !\n";
    }        
    return (\%h_new_species, \%h_old_species, \%h_unch_species);
}

=head2 procedure FinishUpgradeSpeciesCfg

 Title        : FinishUpgradeSpeciesCfg
 Usage        : FinishUpgradeSpeciesCfg($o_io,$o_logger,$rh_new_species, $rh_old_species, $rh_unch_species)
 Procedure    : Rename .cfg files corresponding to 'old' species
                Replace ORG_NEW=1 by ORg_NEW=0 in the .cfg files corresponding to 'new' species
 Args         : $o_io,$o_logger, hashes with new, old, unchanged species

=cut
sub FinishUpgradeSpeciesCfg
{
    my ( $o_io, $o_logger, $rh_new_species, $rh_old_species, $rh_unch_species) = @_;
    
    $o_logger->Trace("Now managing the species directory\n","*","A");
    
    my $species_dir = $o_io->GetSpeciesCfgDirpath( 'INSTANCE' );
    my $unix_ls     = $o_io->GetUnixPath('UNIX_LS');
    my $unix_mv     = $o_io->GetUnixPath('UNIX_MV');
    my $unix_grep   = $o_io->GetUnixPath('UNIX_GREP');
	my @a_list_cfg  = split /\n+/, `$unix_ls -c1 $species_dir`;
    my $sav_dir = cwd;
    chdir $species_dir or die "ERROR - Cannot change to $species_dir directory";
    
    # 1st loop = Detect the *.cfg files corresponding to the "old_species", they will be renamed
    foreach my $species (keys %$rh_old_species)
    {
        my $file_cfg = $species . '.cfg';
        `$unix_mv $file_cfg ${file_cfg}_old` if (-e $file_cfg);
    }
    
    # 2nd loop = Remove the ORG_NEW parameter from the *.cfg files corresponding to the new species
    foreach my $species (keys %$rh_new_species)
    {
        my $file_cfg = $species.'.cfg';
        my $fh_file_cfg = new IO::File($file_cfg) or throw LipmError::IOException($file_cfg,'f');
        my @a_parameters = <$fh_file_cfg>;
        $fh_file_cfg = undef;
        foreach my $p (@a_parameters)
        {
            $p="ORG_NEW=0\n" if ($p =~ /^ORG_NEW=/);
        }
        $fh_file_cfg = new IO::File(">$file_cfg") or throw LipmError::IOException($file_cfg,'f');
        print $fh_file_cfg @a_parameters;
        $fh_file_cfg = undef;
    }
    
    # foreach my $cfg_file (@a_list_cfg)
    # {
        # next unless ($cfg_file =~ /\.cfg$/);
        # next if ($cfg_file eq 'template.cfg');
        # my ($species) = ($cfg_file =~ /(.+)\.cfg$/);
        # 
        # if ( exists $rh_old_species->{$species} )
        # {
            # `$unix_mv $cfg_file ${cfg_file}_old`;
            # $o_logger->Trace("renamed $cfg_file => ${cfg_file}_old\n","*","ACV");
        # }
    # }
    
    chdir $sav_dir or die "ERROR - Cannot change to $sav_dir directory";

}

=head2 procedure FinishUpgradeData

 Title        : FinishUpgradeData
 Usage        : FinishUpgradeData($o_io,$o_logger,$rh_new_species, $rh_old_species, $rh_unch_species)
 Procedure    : Finish the upgrade, removing the 'old' data species
 Args         : $o_io,$o_logger, hashes with new, old, unchanged species

=cut
sub FinishUpgradeData
{
    my ( $o_io, $o_logger, $rh_new_species, $rh_old_species, $rh_unch_species) = @_;
    
    my $unix_mv   = $o_io->GetUnixPath('UNIX_MV');
    my $unix_rm   = $o_io->GetUnixPath('UNIX_RM');
    my $unix_find = $o_io->GetUnixPath('UNIX_FIND');

    $o_logger->Trace("Now managing the data\n","*","A");
    
    my %h_all_species_obs;
    foreach my $species ( keys(%$rh_old_species), keys(%$rh_new_species), keys(%$rh_unch_species) )
    {
        $h_all_species_obs{$species} = $o_io->GetSpeciesObserver( $species );
    }
    
    # Removing the data directories of the old species
    foreach my $species (keys %$rh_old_species)
    {
        my $ra_data_dir = &_GetDataDirectories($species,\%h_all_species_obs);
        foreach my $dir (@$ra_data_dir)
        {
            #warn "KOUKOU $dir";
            &_RemoveIfExists($dir,$unix_rm,$o_logger);
        }
    }
    
    print "\n\n\n";
    
    # Return the list of directories where you could find data for the species passed by parameter
    sub _GetDataDirectories
    {
        my ($species,$rh_all_species_obs) = @_;
        
        my $o_species = $rh_all_species_obs->{$species};
        my @a_dir;
        my @a_all_methods = ( 'genomic', 'genetic' );
        
        # removing bana, comparison/bana, comparative/bana 
        push @a_dir, $o_species->GetDataPath();
        push @a_dir, $o_species->GetComparativeDirpath ( target => '', -where => 'INSTANCE', -check => &FALSE);
        foreach my $method (@a_all_methods)
        {
            push @a_dir, $o_species->GetComparisonPath ( method => $method, target => '', -where => 'INSTANCE', -check => &FALSE);
        }
        
        # removing comparison/banb/bana, comparative/banb/bana
        foreach my $target (keys %$rh_all_species_obs)
        {
            # We do not treat here the self comparisons 
            #next if ($target eq $species);

            my $o_target = $rh_all_species_obs->{$target};
            push @a_dir, $o_target->GetComparativeDirpath ( target => $species, -where => 'INSTANCE', -check => &FALSE);
            foreach my $method (@a_all_methods)
            {
                push @a_dir, $o_target->GetComparisonPath ( method => $method, target => $species, -where => 'INSTANCE', -check => &FALSE);
            }
        }
        
        return \@a_dir;
    }

    # If the directory passed by parameter exists, it is removed
    sub _RemoveIfExists
    {
        my ($path,$unix_rm,$o_logger) = @_;
        if ( -e $path )
        {
            `$unix_rm -r $path`;
            $o_logger->Trace("removed $path\n","*","ACV");
        }
    }
}
   
    

=head2 procedure Usage

 Title        : Usage
 Usage        : Usage($code)
 Prerequisite : none
 Fonction     : fonction usage
 Returns      : none
 Args         : none
 Globals      : none

=cut
sub Usage()
{
	my $o_param = shift;


print STDERR<<END

Usage : $0
    --finish_upgrade       Before anything, Terminate the upgrade process, maybe removing data

    --only_configure_syntenies  Generation of the file synteny.list
    --no_configure_syntenies
    
    --only_merge_db             merge db
    --no_merge_db
    
    --only_index_data           Index data
    --no_index_data
    
    --help
END
;
exit 1;
}

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license - www.cecill.info

=cut


