#!/usr/bin/perl
#
# $Id: nc_species.pl 3464 2010-08-24 13:05:37Z manu $
#
# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


=head1 NAME

 nc_species.pl

=head1 SYNOPSIS

 chins some_instance (but NOT multi !)
 nc_species.pl

=head1 DESCRIPTION

 Prints the list of species found in this instance - This may be convenient for use wih a script:
 
 in csh/tcsh, you could write:
 foreach $s (`nc_species.pl`)
    do_something
 end
 
 in sh/ksh/bash, you could write:
 for $in in `nc_species.pl`
 do
    do_something
 done
 
=cut

use FindBin;
use lib "$FindBin::Bin/../lib";
use lib "$FindBin::Bin/../API";
use strict;
use warnings;
use ParamParser;
use File::Basename;
use Configuration::InstanceObserver;

my $VERSION = do {my @r = (q$Rev: 3464 $ =~ /\d+/g); $r[0]};

# Main program
MAIN:
{
	ParamParser::SetDefaultUsage( \&Usage );
	ParamParser::SetDefaultBehaviour( 'exit_on_getopt_error' );

	my $o_param = New ParamParser('GETOPTLONG','long','short','help');
    if  ( exists $ENV { 'NARCISSE_CFG' } )
    {
        my $o_io = New InstanceObserver();
        if ($o_param->IsDefined('long') && $o_param->IsDefined('short'))
        {
			my @a_short_names = $o_io -> GetSpeciesShortName();
			my @a_long_names  = $o_io -> GetSpeciesLongName();
			for (my $i=0; $i<=$#a_short_names; $i++)
			{
				print $a_short_names[$i] . "\t" . $a_long_names[$i] . "\n";
			}
		}
		elsif ($o_param->IsDefined('long'))
        {
			print join( "\n",$o_io->GetSpeciesLongName() );
			print "\n";
		}
		elsif ($o_param->IsDefined('short'))
		{
	        print join( "\n",$o_io->GetSpeciesShortName() );
			print "\n";
		}
		else
		{
	        print join( ' ',$o_io->GetSpeciesShortName() );
		}
    }
    else
    {
        die "ERROR - Please execute chins <some_instance> before running this program\n";
    }
}

sub Usage
{
	my $self = shift;
	my $exe = basename $0;
	print <<END
Usage : $exe [--long][--short]

	--long : print long names , 1 name/line
	--short: print short names, 1 name/line
	
	
	Print the list of species known by this instance. The default is to print shortnames, 
	each name being separated by a space: this is convenient to write scripts:
	     for s in `nc_species.pl`; do something with \$s; done
	If --long is specified,  we print long names, 1 name / line. this is more convenient for humans
	If --short is specified, we print short names, 1 name / line. this is more convenient for quick-and-dirty perl scripts
	If both are specified, we print short and long names, 1 species / line, this is convenient for everybody
END
;
	exit 1;
}

	
	

	


