#!/usr/bin/perl

# $Id: ncws_configure.pl 3851 2014-02-07 10:04:36Z  $
#
# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

BEGIN
{
		use File::Basename;
		my $dirprg=dirname($0);
		unshift @INC, "$dirprg/../lib";
		unshift @INC, "$dirprg/../API";
		unshift @INC, "$dirprg/../API/Bio/Narcisse/Utils";
}

use strict;
use warnings;

use Cwd;
use File::Basename;
use File::Spec::Functions qw(rel2abs);
use IO::File;
use JSON;
use File::Temp;

use ParamParser;
use Runner;
use OnlyOrNo;
#use NARCISSE_vars;
use NARCISSE_utils;
use Configuration::NarcisseObserver;
use ConfigurationFactory;
use Logger;
use Carp;
use Sufine;
use General;

my $VERSION = do {my @r = (q$Rev: 3851 $ =~ /\d+/g); $r[0]};

# TODO - REECRIRE CE PROGRAMME EN UTILISANT UN OBJET NARCISSEOBSERVER

# Main program
MAIN:
{

    # Parsing the command file
    #&ParamParser::SetDefaultBehaviour('use_substitution_table');
    my $o_param = New ParamParser();
    $o_param->SetUsage(\&Usage);
    $o_param->SetBehaviour('exit_on_getopt_error');

    # Create an observer (should be a narcisse observer)
    my $o_no = ConfigurationFactory::GetConfiguration();
    if ($o_no->isa('NarcisseObserver') == &FALSE )
    {
        throw Error::Simple ("NARCISSE ERROR - please type \"chins multi\" and restart");
    }
    
    my $rh_instances = $o_no->GetInstanceObservers();
    if (keys(%$rh_instances) == 0)
    {
        throw Error::Simple ("NARCISSE ERROR - I could not find any configured instance");
    }

    $o_param->Update('GETOPTLONG','O',("install_playmoby","only_create","no_create","only_deploy","no_deploy","service=s","deregister","help"));

    # Install playmoby ?
	if ($o_param->Get('install_playmoby'))
    {
        # We do not return from this sub
		&InstallPlaymoby($o_no,$o_param);
    };

    # Do not forget to source $NARCISSE_ROOT/web_services/playmoby/.setenv !
    $o_param->Update('ENV','A');
    $o_param->AssertDefined('PMBBIN');
    $o_param->AssertDirExists('PMBBIN');

    # reading the list of web_services
    my $o_ndr = New Bio::Narcisse::Utils::NarcisseDataReader();
    $o_ndr->Prepare( -from => $o_no->GetCfg('ROOT_WEBSERVICES_LIST') );
    my $rh_wsl = $o_ndr->FetchallHashref( 'service' );
    
    # Managing the --only_, --no_ switches
    my $o_only = New OnlyOrNo($o_param);

    if ($o_param->IsDefined('deregister'))
    {
        &DeployWs($o_no,$rh_wsl,$o_param) if ($o_only->Get('deploy'));
    }
    else
    {
        my $tmpdir = &BuildXml($o_no,$rh_wsl,$o_param);
        &CreateWs($o_no,$rh_wsl,$o_param, $tmpdir ) if ($o_only->Get('create'));
        &DeployWs($o_no,$rh_wsl,$o_param) if ($o_only->Get('deploy'));
    }
    print "try to look at http://lipm-bioinfo.toulouse.inra.fr/registry/cgi/registry.cgi to be sure your web_service is correctly registered\n";
    print "bye\n";
    exit;
}

=head2 function InstallPlaymoby

 Title    : InstallPlaymoby
 Usage    : InstallPlaymoby($o_no);
 Procedure: Install playmoby AND EXIT: NEVER RETURN !

 Returns  : Never return
 Arg      : The NarcisseObserver object
 Globals  : none

=cut

sub InstallPlaymoby
{
    my ($o_no) = (@_);
    my $unix_cp =  $o_no->GetUnixPath('UNIX_CP');
    my $unix_wget= $o_no->GetUnixPath('UNIX_WGET');
    my $unix_tar = $o_no->GetUnixPath('UNIX_TAR');
    
    # Saving old installation
    my $root_playmoby = $o_no->GetCfg('ROOT_PLAYMOBY');
    if (-e $root_playmoby)
    {
        my $stamp=&Logger::CalcStamp();
        $stamp =~ s/#//g;
        my $dst = $root_playmoby . '.' . $stamp;
        my $cmd = "$unix_cp  $root_playmoby $dst";
        print "Keeping old playmoby hierarchy in $dst\n";
        system($cmd);
    };

    # working in web_services directory
    my $web_services_dir = $o_no->GetCfg('ROOT_WEBSERVICES');
    mkdir $web_services_dir or die "Cannot make $web_services_dir" unless (-d $web_services_dir);
    chdir $web_services_dir or die "Cannot change to directory $web_services_dir";

    my $url_playmoby = $o_no->GetCfg('URL_PLAYMOBY');
    my $playmoby = basename $url_playmoby;

    unlink $playmoby;
    print "Now getting playmoby from its standard distribution location\n";
    my $cmd = "$unix_wget $url_playmoby";
    system($cmd);

    print "Now installing or upgrading playmoby\n";
    $cmd = "$unix_tar xjf $playmoby";
    system($cmd);
    
    chdir $o_no->GetCfg('ROOT_PLAYMOBY');
    my $unix_path = $o_no->GetCfg('ROOT_PLAYMOBY');
    my $http_path = $o_no->GetCfg('HTTP_PLAYMOBY');
    my $auth_uri  = ($o_no->GetCfg('WS_TEST')==0) ? $o_no->GetCfg('WS_PROD_AUTH_URI') : $o_no->GetCfg('WS_TEST_AUTH_URI');
    my $adm_email = $o_no->GetCfg('PLAYMOBY_ADM_EMAIL');
    my $timeout   = $o_no->GetCfg('PLAYMOBY_TIMEOUT');
    $cmd = "./pmb_configure.pl --title 'NARCISSE WEBSERVICES' --unix_path=$unix_path --http_path=$http_path --auth_uri=$auth_uri --email=$adm_email --timeout=$timeout";
    system ($cmd);

    mkdir "services/narcisse" or die "Cannot create narcisse directory";
    
    print "OK - DO NOT FORGET TO TEST YOUR BROWSER CONFIGURATION\n";
    
    exit;
}

=head2 function BuildXml

 Usage    : &BuildXml($o_no,$rh_wsl,$o_param);
 function : For each web_service to be installed, build the xml file

 Returns  : the temporary directory
 Arg      : The Narcisse observer, the service list, the parameters
 Globals  :

=cut

sub BuildXml
{
	my ($o_no, $rh_wsl, $o_param) = (@_);

	# creer un rep temporaire pour les fichiers XML
	my $tmpdir = File::Temp::tempdir('NCS_XML_XXXX', CLEANUP => 1, TMPDIR => 1);

	print "Now building the xml files for the requests wrapped in the wanted web_services\n";
	my $ra_list = &GetListOfWs($o_no, $rh_wsl, $o_param);
	foreach my $rh_ws_cfg (@$ra_list)
	{
		my $service = $rh_ws_cfg->{service};
		my $request = $rh_ws_cfg->{request};
		$request = 'ROOT_'.$request if ($request !~ /^ROOT_/);
		my $cmd = $o_no->GetCfg($request)." --mobyle_dir=$tmpdir";
		my ($status, $killed) = &RunExt(-cmd     => $cmd);
		print "execute $cmd - status = $status\n";
	}

	return $tmpdir;
}

=head2 function CreateWs

 Title    : CreateWs
 Usage    : &CreateWs($o_no, $rh_wsl, $o_param, $tmpdir );
 Procedure: For each web_service to be installed, call pmb_MobyleParser.pl to create the service

 Returns  : nothing
 Arg      : The Narcisse observer, the service list, the parameters
 Globals  :

=cut

sub CreateWs
{
	my( $o_no, $rh_wsl, $o_param, $tmpdir ) = @_;

	my $contact    = $o_no->GetCfg('NARCISSE_ADM_EMAIL');
	my $service_dir= $o_no->GetCfg('ROOT_PLAYMOBY_NARCISSE');
	mkdir $service_dir unless (-d $service_dir);

	my $pmbbin     = $o_param->Get('PMBBIN');

	print "Now creating the web_services\n";
	my $dir_sav = cwd;
	chdir$o_no->GetCfg('ROOT_SERVICE_TEST') or die "ERROR - Cannot change to " . $o_no->GetCfg('ROOT_SERVICE_TEST');

	my $ra_list = &GetListOfWs($o_no, $rh_wsl, $o_param);
# use Data::Dumper;
# warn Dumper $ra_list;

	foreach my $rh_ws_cfg (@$ra_list)
	{
		my $fh_tmp_file = new File::Temp( SUFFIX => '.sh', UNLINK => 1 )
							or die "Cannot create temporary file";

		my $service = $rh_ws_cfg->{service};
		my $key_req = $rh_ws_cfg->{request};
		$key_req    = 'ROOT_'.$key_req if ($key_req !~ /^ROOT_/);
		my $request = $o_no->GetCfg($key_req);
		my $xml_file= File::Spec->catfile( $tmpdir, $service.'.xml' );
		my $bz_file = $service.'.bz2';

		my $cmd     = "$pmbbin/pmb_MobyleParser.pl --async --xmlfile=$xml_file --tmp_dir=/tmp --bin_fullpath=$request --service_contact=$contact";
		print $fh_tmp_file "$cmd <<EOF\n";
        
        # Primary articles, up to 3
		print $fh_tmp_file $rh_ws_cfg->{iar1}."\n" if (defined $rh_ws_cfg->{iar1} and $rh_ws_cfg->{iar1} ne "");
		print $fh_tmp_file $rh_ws_cfg->{iar2}."\n" if (defined $rh_ws_cfg->{iar2} and $rh_ws_cfg->{iar2} ne "");
		print $fh_tmp_file $rh_ws_cfg->{iar3}."\n" if (defined $rh_ws_cfg->{iar3} and $rh_ws_cfg->{iar3} ne "");
        
        # No more question
		print $fh_tmp_file "EOF\n";

		print "executing $cmd\n";
        my $unix_bash = $o_no->GetUnixPath('UNIX_BASH');
		system( $unix_bash.' '. $fh_tmp_file->filename() );
		$fh_tmp_file->close();

		if( -r $bz_file )
		{
            my $unix_mv = $o_no->GetUnixPath('UNIX_MV');
			system( "$unix_mv $bz_file $service_dir" );
		}
		else
		{
			die "ERROR - $bz_file not found !";
		}
    }

    chdir $dir_sav;
}

=head2 function DeployWs

 Title    : DeployWs
 Usage    : &DeployWs($o_no,$o_param);
 Procedure: For each web_service to be installed, call pmb_DeployWS.pl to deploy the service
            Then, if WS_TEST_SPECIES and WS_TEST_INSTANCE are specified, create the file parameters to restrict the tests
 Returns  : nothing
 Arg      : The Narcisse observer, the service list, the parameters
 Globals  :

=cut

sub DeployWs
{
	my ($o_no, $rh_wsl, $o_param) = (@_);

	my $pmbbin    = $o_param->Get('PMBBIN');
	my $work_dir  = $o_no->GetCfg('ROOT_PLAYMOBY_NARCISSE');
    my $unix_rm   = $o_no->GetUnixPath('UNIX_RM');
    my $unix_tar  = $o_no->GetUnixPath('UNIX_TAR');

	die "ERROR - The directory $work_dir should exist !" unless (-d $work_dir);
	my $dir_sav = cwd;
	chdir $work_dir or die "ERROR - Cannot change to $work_dir";

	my $registry  = ($o_no->GetCfg('WS_TEST')==0) ? $o_no->GetCfg('WS_PROD_REGISTRY') : $o_no->GetCfg('WS_TEST_REGISTRY');

	# Last chance to cancel things
	if ($o_param->IsDefined('deregister'))
	{
		print "Now Deregistering and removing the web_services - registry = $registry - OK to go (Y/[N]) ? ";
	}
	else
	{
		print "Now Deploying the web_services - registry = $registry- OK to go (Y/[N]) ? ";
	}

	if (&NARCISSE_utils::GetAdmAnswer("","Y|N","N") eq 'N')
	{
		print "Cancelled\n";
		return;
	}
    else
    {
        unless ($o_param->IsDefined('service'))
        {
            print "WARNING !!! WE ARE WORKING WITH ALL SERVICES ! - STILL OK (Y/[N]) ? ";
            if (&NARCISSE_utils::GetAdmAnswer("","Y|N","N") eq 'N')
            {
                print "Cancelled\n";
                return;
            }
        }
    }

	my $ra_list = &GetListOfWs($o_no, $rh_wsl, $o_param);
	foreach my $rh_ws_cfg (@$ra_list)
	{
		my $service   = $rh_ws_cfg->{service};
		my $service_test_prm = $service.'/test/parameters';

		my $bz_file   = $service . '.bz2';
		if ($o_param->IsDefined('deregister'))
		{
			my $cmd = "$pmbbin/pmb_DeployWS.pl --central=$registry --services=$service --deregister";
			print "Now deregistering service $service...\n";
			system($cmd);
			print "now removing files related to service $service...\n";
			system("$unix_rm -r $service ${service}.pm ${service}.bz2");
		}
        else
        {
            if (-d $service)
            {
                print "Now removing old service files $service...\n";
                system("$unix_rm -r $service");
            }

            print "Now extracting the bz2 file\n";
            system("$unix_tar xjf $bz_file");

            if (&General::IsNotNull( $$rh_ws_cfg{ instance } ) || &General::IsNotNull( $$rh_ws_cfg{ species } ))
            {
                print "Now restricting tests to some instance, species\n";
                my $fd_parameters = new IO::File(">$service_test_prm") or die "ERROR - Cannot create the file $service_test_prm";
                print $fd_parameters "instance=".$rh_ws_cfg->{instance}."\n" if ( &General::IsNotNull( $$rh_ws_cfg{ instance } ) );
                print $fd_parameters "species=".$rh_ws_cfg->{species}."\n" if ( &General::IsNotNull( $$rh_ws_cfg{ species } ) );
            }

            print "Now deploying service $service...\n";
            my $cmd = "$pmbbin/pmb_DeployWS.pl --central=$registry --services=$service --all";
            print "executing $cmd\n";
            system ($cmd );
        }
    }

    chdir $dir_sav or die "ERROR - Cannot change to $dir_sav";
}

=head2 function GetListOfWs

 Title    : GetListOfWs
 Usage    : my $rh_web_services = &GetListOfWs($o_no);
 Function : Return the list of web_services to install: the services declared through --service, OR all services
            ONLY the services declared in web_services.list are returned
 Returns  : a ref to an array, each value is a hash representing the web_service configuration
 Arg      : The Narcisse observer, the web_services list, the parameters
 Globals  : none

=cut

sub GetListOfWs
{
    my ($o_no,$rh_wsl,$o_param) = (@_);

    my @a_list_of_services;
    if ($o_param->IsDefined('service'))
    {
        my @a_tmp = split(/[ ,]/,$o_param->Get('service'));
        foreach my $s (@a_tmp)
        {
            if (exists $rh_wsl->{$s})
            {
                push (@a_list_of_services,$rh_wsl->{$s});
            }
            else
            {
                warn " WARNING - Could not find service $s - Skipping this service\n";
            }
        }
    }
    else
    {
        @a_list_of_services = values %$rh_wsl;
    }
    return \@a_list_of_services;
}

=head2 procedure Usage

 Title        : USAGE
 Usage        : USAGE($code)
 Prerequisite : none
 Fonction     : fonction usage
 Returns      : none
 Args         : none
 Globals      : none

=cut
sub Usage()
{
    my $prgm = basename $0;
    if (exists $ENV{PMBBIN} and -d $ENV{PMBBIN})
    {
        my $playmoby = dirname $ENV{PMBBIN};
        my $o_playmoby_cfg_params = New ParamParser($playmoby.'/.configure');
        my %h_config;
        $o_playmoby_cfg_params -> Dump('HASH',\%h_config);
        print <<END
        
        NARCISSE_CFG   = $ENV{NARCISSE_CFG}
        PMBBIN         = $ENV{PMBBIN}
        
        auth_uri       = $h_config{auth_uri}
        http_path      = $h_config{http_path}
        unix_path      = $h_config{unix_path}
        timeout        = $h_config{timeout}
        
        
        Usage : $prgm
        --install_playmoby        upgrade playmoby and leave
        
        --only_create             only or no create the web service
        --no_create
        
        --only_deploy             only or no deploy the web service
        --no_deploy
        
        --service                 Work only on the service specified (may be a list - separator is ,), if not specified work on all services
        --deregister              Useful only if Deploying service: instead of deploying service(s), deregister them
        
        --help                    This message
        
        NOTE - HAVE A LOOK TO THE PARAMETER WS_TEST in the file Site/cfg/default.cfg
               IF YOU CHANGE THIS PARAMETER, PLEASE RE-INSTALL PLAYMOBY
               
END
;
    }
    else
    {
        print <<END
        playmoby is NOT CORRECTLY INSTALLED !!!
        Please run "$prgm --install_playmoby", then type again "chins multi" and restart ncws_configure.pl.
END
;        
    };
    exit;
}

=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license - www.cecill.info

=cut

