#!/usr/bin/perl

=pod

=head1 NAME

 API

=head1 SYNOPSIS

 perl API @args

 @args:
  --

=head1 DESCRIPTION

 # description

=head1 Author

 Sebastien.letort@toulouse.inra.fr

=head1 Version

 Created: 28-04, 2009
 $Id: API.pl 3702 2011-07-27 12:37:29Z manu $

=cut


use strict;
use warnings;

use Error qw( :try );
use Data::Dumper;
use Pod::Usage;

BEGIN
{	# pathway to our modules
	use File::Basename;
	my $dirprg=dirname($0);
	unshift @INC, "$dirprg/../lib";
	unshift @INC, "$dirprg/../API";
}

# nos modules, nos classes
use LipmError;
use General;
use ParamParser;
use Logger;
use Bio::Narcisse::DBAdaptor;

# les constantes
use constant VERSION => do {my @r = (q$Rev: 3702 $ =~ /\d+/g); $r[0]};

use constant DEBUG => &TRUE;
# use constant DEBUG => &FALSE;
$Error::Debug = &DEBUG;

&TryMain();


=head2 Procedure Main

 Title        : Main
 Usage        : &Main();
 Prerequisite : none
 Procedure    : the main function
 Args         : none

=cut

sub Main
{
	my $o_log = New Logger( 'API.log', 'V' );

	my $rh_params = &GetParameters();

# 	my( $ref, $locus_tag, $elt_id, $prot_id ) =
# 		qw( mtr AC133572_12.5 mtr_s3g2588 AC133572_12.5 );

#	my( $ref, $locus_tag, $elt_id, $prot_id ) =
#		qw( ath CLE19 NIL CLE19 );

	my( $ref, $locus_tag, $elt_id, $prot_id ) =
		qw( caeel Y74C9A.2 NIL CLE19 );

	# DBAdaptor
	my $o_species = New SpeciesObserver( species => $ref );
	my $o_dba = $o_species->GetDBAdaptor();

	my $species = $o_dba->species();
# 	my $path = $o_dba->species_path();
	print "Data obtain from DBA ( $ref ) :\n\tspecies => $species\n\t";#path => $path\n\n";

	print "test de GeneAdaptor :\n";
	my $o_adpt = $o_dba->get_GeneAdaptor();
	my $o_feat = $o_adpt->fetch_by_stable_id( $locus_tag );
	&PrintFeatureInfo( 'fetch_by_stable_id', $o_feat );
	$o_feat    = $o_adpt->fetch_by_dbID( $elt_id );
	&PrintFeatureInfo( 'fetch_by_dbID', $o_feat );
	$o_feat    = $o_adpt->fetch_by_symbol( $prot_id );
	&PrintFeatureInfo( 'fetch_by_symbol', $o_feat );

	print "test de ExonAdaptor :\n";
	$o_adpt    = $o_dba->get_ExonAdaptor();
	$o_feat    = $o_adpt->fetch_by_stable_id( $locus_tag );
	&PrintFeatureInfo( 'fetch_by_stable_id', $o_feat );
	$o_feat    = $o_adpt->fetch_by_dbID( $elt_id );
	&PrintFeatureInfo( 'fetch_by_dbID', $o_feat );

	print "test de RepeatAdaptor :\n";
	$o_adpt    = $o_dba->get_RepeatFeatureAdaptor();
	$o_feat    = $o_adpt->fetch_by_dbID( $elt_id );
	if( !$o_feat )
	{	print "No repeat feature found\n\n";	}
	else
	{	&PrintFeatureInfo( 'fetch_by_dbID', $o_feat );	}

	print "test de TranscriptAdaptor :\n";
	$o_adpt    = $o_dba->get_TranscriptAdaptor();
	$o_feat    = $o_adpt->fetch_by_dbID( $elt_id );
	&PrintFeatureInfo( 'fetch_by_dbID', $o_feat );
	$o_feat    = $o_adpt->fetch_by_stable_id( $locus_tag );
	&PrintFeatureInfo( 'fetch_by_stable_id', $o_feat );
	
	print "start coding end coding " . $o_feat->cdna_coding_start() . " " . $o_feat->cdna_coding_end() . "\n";
	$o_feat    = $o_adpt->fetch_by_translation_id( $locus_tag );
	&PrintFeatureInfo( 'fetch_by_translation_id', $o_feat );

	print "test de TranslationAdaptor :\n";
	$o_adpt    = $o_dba->get_TranslationAdaptor();
	$o_feat    = $o_adpt->fetch_by_stable_id( $locus_tag );
	&PrintFeatureInfo( 'fetch_by_stable_id', $o_feat );
# 	$o_feat    = $o_adpt->fetch_by_Transcript( $locus_tag );
# 	&PrintFeatureInfo( 'fetch_by_stable_id', $o_feat );


$o_adpt = $o_dba->get_SliceAdaptor();
my $o_slice = $o_adpt->fetch_by_region( 'chromosome', '3');

$o_adpt = $o_dba->get_SequenceAdaptor();
my $o_s = $o_adpt->fetch_by_Slice_start_end_strand( $o_slice, 10,70,1 );

	return;
}

sub PrintFeatureInfo
{
	my( $str, $o_feature ) = @_;
	print "$str\n";
	if ( defined $o_feature )
	{
		my $start      = $o_feature->start();
		my $end        = $o_feature->end();
		my $length     = $o_feature->length();
		my $display_id = $o_feature->display_id();
		my $db_id      = $o_feature->{ dbID };
	
		print "feature $display_id ( $db_id ) :";
		print " $start-$end <=> length = $length\n\n";
	}
	else
	{
		print "FEATURE NOT DEFINED\n\n";
	}

	return;
}

=head2 GetParameters

 Usage        : my $rh_params = $o_x->GetParameters()
 Function     : Read parameters from command line and return them formatted as a hash ref.
 Returns      : $rh_params, hash ref with keys ...
 Args         : none

=cut

sub GetParameters
{
	ParamParser::SetDefaultUsage( sub { pod2usage(1) } );

	my @a_keys   = qw(  help );
#	push( @a_keys, );
	my $o_param  = New ParamParser( 'GETOPTLONG', @a_keys );

	my $rh_params = {};
	$o_param->Dump( 'HASH', $rh_params );

	return $rh_params;
}



__END__

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
