#!/usr/bin/perl

=pod

=head1 NAME

 InstanceObserver

=head1 SYNOPSIS

 perl InstanceObserver @args

 @args:
  --

=head1 DESCRIPTION

 # description

=head1 Author

 Sebastien.letort@toulouse.inra.fr

=head1 Version

 Created: 24-04, 2009
 $Id: InstanceObserver.pl 2679 2009-06-10 15:17:58Z sletort $

=cut


use strict;
use warnings;

use Error qw( :try );
use Data::Dumper;
use Pod::Usage;

BEGIN
{	# pathway to our modules
	use File::Basename;
	my $dirprg=dirname($0);
	unshift @INC, "$dirprg/../lib";
	unshift @INC, "$dirprg/../API";
}

# nos modules, nos classes
use LipmError;
use General;
use ParamParser;
use Logger;

use Configuration::InstanceObserver;

# les constantes
use constant VERSION => do {my @r = (q$Rev: 2679 $ =~ /\d+/g); $r[0]};

use constant DEBUG => &TRUE;
# use constant DEBUG => &FALSE;

MAIN:
{
	try
	{
		&Main();
	}
	catch LipmError with
	{
		my ( $o_err, $r ) = @_;

		if( &TRUE == &DEBUG )
		{	warn( $o_err->Explain(), "\n" );	}
		else
		{	print '*' . $o_err->GetUserMessage() . "*\n";	}
	}
	otherwise
	{
		my ( $o_err, $r ) = @_;

		my $str = $o_err->file;
		$str   .= ' line ' . $o_err->line;
		$str   .= "\n" . $o_err->text . "\n";

		print STDERR $str;
	};

	exit( 0 );
}


=head2 Procedure Main

 Title        : Main
 Usage        : &Main();
 Prerequisite : none
 Procedure    : the main function
 Args         : none

=cut

sub Main
{
	my $o_log = New Logger( 'InstanceObserver.log', 'IV' );

	my $rh_params = &GetParameters();

	print "Will Build an InstanceObserver for animals\n";
	my $o_io = New InstanceObserver( -cfg => "../../animals/Site/cfg/narcisse.cfg" );

	my $nb   = $o_io->GetNbSpecies();
	my $name = $o_io->GetName();
	print "Instance named $name has $nb species, check it\n";

	my( $species, $chrom, $tgt ) = qw( human Y mouse );

	print "***---***\n";
	&GetPaths( $o_io );

	print "***---***\n";
	&GetInfos( $o_io, $species );


	#refaire le code dans des fonctions au fur et a mesure de l'utilite.

# 	my @a_paths = $o_io->GetComparativePaths( 'SESSION,INSTANCE' );
# 	warn Dumper @a_paths;

# 	my $gene_file = $o_io->GetGeneFile( species => $species, chrom => $chrom );
# 	print "Good format is data_dir/ref/gene/chromosomex.gene\n";
# 	print "=> $gene_file\n\n";

# 	$o_io->SetSessionDirectory( '.' );
# 	my $hsp_file = $o_io->GetHspFile( species => $species, chrom => $chrom );
# 	print "Good format is sessdir/ref/hsp/chromosomex.hsp\n";
# 	print "=> $hsp_file\n\n";

# 	$o_io->SetSessionDirectory( 'WB00001' );
# 	my $file = $o_io->GetHspFile( species => $species,
# 									 chrom   => $chrom );
# 	print "GetHspFile : $file\n\n";
# 
# 
# 	print "Dumper \$o_io->GetSpeciesRoot( $species, 'SESSION' )\n";
# 	warn Dumper( $o_io->GetSpeciesRoot( $species, 'SESSION' ) ) . "\n";
# 
# 	print "Dumper \$o_io->GetSpeciesRoot( $species, 'INSTANCE' )\n";
# 	warn Dumper( $o_io->GetSpeciesRoot( $species, 'INSTANCE' ) ) . "\n";
# 

	# doit lever une exception
# 	( $species, $chrom, $tgt ) = qw( bob 7 spe );
# 	$gene_file    = $o_io->GetGeneFile( species => $species, chrom => $chrom );
# 	warn $gene_file;

# 	$prbh_file = $o_io->GetPrbhFile( reference => $species, chrom => $chrom,
# 									 target => $tgt );
# 	warn $prbh_file;

# 	$synt_file    = $o_io->GetSyntFile( reference => $species, chrom => $chrom,
# 										target => $tgt,
# 										prog => 'genomic', level => '2' );
# 	warn $synt_file;

#--------------------

# 	my $fasta = $o_io->GetChromFastaFile( species => $species,
# 										  chrom   => $chrom );
# 	print "GetFastaFile : we want chrom file\n\t=>$fasta\n\n";
# 
# 	$fasta    = $o_io->GetCDSFastaFile( species => $species );
# 	print "GetFastaFile : we want cds file\n\t=>$fasta\n\n";
# 
# 	$fasta    = $o_io->GetGeneFastaFile( species => $species );
# 	print "GetFastaFile : we want gene file\n\t=>$fasta\n\n";
# 
# 	$fasta    = $o_io->GetProtFastaFile( species => $species );
# 	print "GetFastaFile : we want protein file\n\t=>$fasta\n\n";
# 
# 	$fasta    = $o_io->GetTranscriptFastaFile( species => $species );
# 	print "GetFastaFile : we want transcript file\n\t=>$fasta\n\n";
# 
# 	my $data  = $o_io->GetPersoFile( species => $species, chrom => $chrom );
# 	print "GetPersoFile : we want personnal file sor ($species,$chrom)\n\t=>$data\n\n";

#--------------------

# 	my $o_dba = $o_io->GetDBAdaptor( $species );
# 	my $str = ( &SafeIsa( $o_dba, 'Bio::Narcisse::DBAdaptor' ) ) ? 'ok' : 'ko';
# 	print "GetDBAdaptor => $str\n\n";

	print "***---***\n";
	&TestSingleton();

	return;
}


sub GetInfos
{
	my ( $o_io, $species ) = @_;

	my $rh_infos = $o_io->GetSpeciesInfo( $species );
	print "what is returned by GetSpeciesInfo($species) \n";
	print Dumper $rh_infos;

	my $short = $o_io->GetSpeciesShortName( $species );
	print "GetSpeciesShortName( '$species' ) => $short\n\n";

	my $ra_spe = $o_io->ExtractMySpecies( [ $species ] );
	print "ExtractMySpecies( [ '$species' ] ) => " . Dumper $ra_spe;
	print "\n";

	$ra_spe = $o_io->ExtractMySpecies( [] );
	print "ExtractMySpecies( [] ) => " . scalar( @$ra_spe ) . "\n";

	return;
}


sub GetPaths
{
	my $o_io = shift;

	print "=>GetPaths tests :\n\n";

	my $path = $o_io->GetDB();
	print "Access DB default : $path\n";

	$path = $o_io->GetDB( 'prot' );
	print "Access DB prot    : $path\n";

	$path = $o_io->GetDB( 'nuc' );
	print "Access DB nuc     : $path\n";
	
}


=head2 TestSingleton

 Usage        : $o_x->TestSingleton( none )
 Function     : .
 Args         :
     none
 Error        : none.

=cut

sub TestSingleton
{
	my $self   = shift;

	print "=>Singleton pattern :\n\n";

	my $o_inst_obs = InstanceObserver::Singleton();
	my $o_io       = InstanceObserver::Singleton();

	if( $o_inst_obs == $o_io )
	{
		print "Singleton method works well\n" ;
	}
	else
	{
		print "Singleton methods did not work\n";
	}

	return;
} #TestSingleton



=head2 GetParameters

 Usage        : my $rh_params = $o_x->GetParameters()
 Function     : Read parameters from command line and return them formatted as a hash ref.
 Returns      : $rh_params, hash ref with keys ...
 Args         : none

=cut

sub GetParameters
{
	ParamParser::SetDefaultUsage( sub { pod2usage(1) } );

	my @a_keys   = qw(  help );
#	push( @a_keys, );
	my $o_param  = New ParamParser( 'GETOPTLONG', @a_keys );

	my $rh_params = {};
	$o_param->Dump( 'HASH', $rh_params );

	return $rh_params;
}



__END__

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
