#!/usr/bin/perl

=pod

=head1 NAME

 LaunchDisplayMap.pl

=head1 SYNOPSIS

 perl LaunchDisplayMap.pl @args

 @args:
  --reference  : string, short species name to use as the reference.
  --chromosome : string, chromosome to display.
  --targets    : strings, comma separated list of short species name to use as targets. optionnal.

=head1 DESCRIPTION

 # description

=head1 Author

 Sebastien.letort@toulouse.inra.fr

=head1 Version

 Created: 30-03, 2009
 $Id: LaunchDisplayMap.pl 2609 2009-05-26 15:15:20Z tfaraut $

=cut

BEGIN
{	# pathway to our modules
	use File::Basename;
	my $dirprg=dirname($0);
	unshift @INC, "$dirprg/../lib";
	unshift @INC, "$dirprg/../API";
}


package DisplayMap;

use strict;
use warnings;

use Data::Dumper;
use Error qw( :try );

use base qw( LipmObject );

use ParamParser;
use Planner::ChromosomeMap;
use ChromosomeSlice::Reference;
use ChromosomeSliceBigMac;
use Configuration::InstanceObserver;

sub _Init
{
	my $self = shift;
	my %h_args = ( o_io      => undef,
	               reference => undef,
	               chroms_reference => undef,
	               target1   => undef,
	               target2   => undef,
				   range	 => undef,
	               level     => undef,
	               @_ );
	$self->SUPER::_Init( %h_args );

	my $o_param = New ParamParser( 'HASH', \%h_args );

	my $o_inst_obs = $h_args{ o_io };

	# Initialization of Planner::ChromosomeMap and Image :
	my $o_planner = Planner::ChromosomeMap -> New( o_io => $o_inst_obs,
													o_param => $o_param );
	$self->_Set( __o_planner  => $o_planner,
				 __o_inst_obs => $o_inst_obs );

	return;
}


=head2 ComputeImage

 Usage        : $o_x->ComputeImage( none )
 Function     : .
 Args         :
     none
 Error        : none.

=cut

sub ComputeImage
{
	my $self   = shift;

	my $o_planner = $self->_Get( qw( __o_planner ) );

	#---------------------------------------
	# Initialization of Image :
	#	- The frame transformation acts only in the vertical dimension (the chromosomes)
	#   - chrom and map are two obligate layers
	my $rh_ImageParameters = $o_planner->GetImageParameters();
	my @a_ = qw( width height frame_transf_type );
	my( $width, $height, $type ) = @$rh_ImageParameters{ @a_ };

	my @mandatory_layers = qw( chrom map );

	my $o_image = Image->New( frame_transf_type => $type,
							  width				=> $width,
							  height 			=> $height,
							  layers			=> \@mandatory_layers
							);
	$self->_Set( __o_image => $o_image );

	$self->__DisplayReference();

	# Foreach target, draw the corresponding synteny slices : Le BigMac
warn "targets";
	foreach my $target ( $o_planner->GetTargetsList() )
	{
		$self->_Trace("Loading $target data...\n","*","MV");

		$self->__DisplayTarget( $target );
	}

	# Save map information for all elements
	$o_planner->DumpMapInformation( 'display_map' );

	# Print each feature in separate images and keep track of the features printed
	my $sessdir = $self->_Get( '__o_inst_obs' )->GetRootSessDir();
	$o_image->PrintImage( $sessdir.'/display_map.png' );

	return;
} #ComputeImage


=head2 __DisplayTarget

 Usage        : $o_x->__DisplayTarget( $target )
 Function     : .
 Args         :
     $target
 Error        : none.

=cut

sub __DisplayTarget
{
	my $self   = shift;
	my( $target ) = @_;

	my @a_attr = qw( __o_planner __o_image __o_ref __o_ref_box __o_inst_obs );
	my( $o_planner, $o_image, $o_ref, $o_ref_box, $o_inst_obs ) = $self->_Get( @a_attr );


	# on besoin d'un accesseur ou d'autres choses.
	@a_attr = qw( chromosome start end );
	my( $chrom, $start, $end ) = $o_planner->_Get( @a_attr );

	@a_attr = qw( level view_level );
	my( $level, $view_lvl ) = $o_planner->_Get( @a_attr );

	my %h_params = ( organism     => $o_planner->GetRefOrg(),
					 chromosome   => $chrom,
					 start        => $start,
					 end          => $end,
					 level        => $level,
					 view_level   => $view_lvl,
					 target       => $target,
					 program      => $o_planner->GetTargetProgram( $target ),
					 o_planner    => $o_planner,
					 o_image      => $o_image,
					 -o_logger    => $self->_GetLogger(),
					graduation_scale => $o_ref->GetGraduationScale(),

					o_io          => $o_inst_obs,
					o_planner     => $o_planner
				);

	try
	{
		my $o_bigmac = New ChromosomeSliceBigMac( %h_params );
		$o_bigmac->Display();

		# The box (rectangular area) of the target genome
		my $o_target_box = $o_planner->GetBox( $target );

		# links
		if( 1 == $o_target_box->IsReferenceNeighbor() )
		{
			my %h_boxes = ( reference_box => $o_ref_box,
							target_box    => $o_target_box );
			$o_bigmac->DisplayPrbh( %h_boxes );
			$o_bigmac->DisplayHsp ( %h_boxes );
		}
	}
	catch LipmError with
	{
		my $o_err = shift;

		my $msg = $o_err->Explain();
		$self->_Trace( $msg, '.', 'MV' );
	};

	return;
} #__DisplayTarget



=head2 __DisplayReference

 Usage        : $o_x->__DisplayReference( none )
 Function     : .
 Args         :
     none
 Error        : none.

=cut

sub __DisplayReference
{
	my $self   = shift;
# 	my( none ) = @_;

	my @a_attr = qw( __o_planner __o_image __o_inst_obs );
	my( $o_planner, $o_image, $o_inst_obs ) = $self->_Get( @a_attr );

	# The box (rectangular area) of the reference genome
	my $o_ref_box = $o_planner->GetBox();


	# on besoin d'un accesseur ou d'autres choses.
	@a_attr = qw( chromosome start end );
	my( $chrom, $start, $end ) = $o_planner->_Get( @a_attr );

	@a_attr = qw( gene_structure gene );
	my ( $gene_struc, $gene ) = $o_planner->_Get( @a_attr );

	@a_attr = qw( graduation_scale selected_features o_maparea o_param_graph );
	my ( $graduation_scale, $features, $map, $graph_cfg ) = $o_planner->_Get( @a_attr );

	my $o_species = New SpeciesObserver( species => $o_planner->GetRefOrg(),
										 -o_inst_obs => $o_inst_obs );

	my %h_params = ( o_species         => $o_species,
					 chromosome        => $chrom,
					 start             => $start,
					 end               => $end,
					 selected_features => $features,
					 gene_structure    => $gene_struc,
					 gene              => $gene,
					 o_maparea         => $map,
					 o_param_graph     => $graph_cfg,
					 o_box             => $o_ref_box,
					 graduation_scale  => $graduation_scale,
					 -o_logger         => $self->_GetLogger(),

					 o_planner         => $o_planner
				);
	my $o_reference = ChromosomeSlice::Reference -> New( %h_params );

	$o_reference->Display( image => $o_image );

	$self->_Set( __o_ref => $o_reference,
	             __o_ref_box => $o_ref_box );

	return;
} #__DisplayReference



1;

# --------------------------------------------------------------

use strict;
use warnings;

use Error qw( :try );
use Data::Dumper;
use Pod::Usage;
use Benchmark;
use File::Path;


# nos modules, nos classes
use LipmError;
use General;
use ParamParser;
use Logger;

# les constantes
use constant VERSION => do {my @r = (q$Rev: 2609 $ =~ /\d+/g); $r[0]};

use constant DEBUG => &TRUE;
# use constant DEBUG => &FALSE;
$Error::Debug = &DEBUG;

use constant REF    => 'reference';
use constant CHROMS => 'chromosome';
use constant TGT    => 'targets';
use constant RANGE  => 'range';
use constant NB  => 'nb';


MAIN:
{
	try
	{
		&Main();
	}
	catch LipmError with
	{
		my ( $o_err, $r ) = @_;

		if( &TRUE == &DEBUG )
		{	warn( $o_err->Explain(), "\n" );	}
		else
		{	print '*' . $o_err->GetUserMessage() . "*\n";	}
	}
	otherwise
	{
		my ( $o_err, $r ) = @_;

		my $str = $o_err->file;
		$str   .= ' line ' . $o_err->line;
		$str   .= "\n" . $o_err->text . "\n";

		print STDERR $str;
	};

	exit( 0 );
}


=head2 Procedure Main

 Title        : Main
 Usage        : &Main();
 Prerequisite : none
 Procedure    : the main function
 Args         : none

=cut

sub Main
{
	my $o_log = New Logger( 'LaunchDisplayMap.log', 'MV' );

	my $rh_params = &GetParameters();
# 	warn Dumper $rh_params;

	my $o_inst_obs = New InstanceObserver();

	my $sessdir = 'WB00001';
	$o_inst_obs->SetSessionDirectory( $sessdir );
	my $tmp = $o_inst_obs->GetRootSessDir();
	mkpath( $tmp );
	print "Results are in $tmp\n";

	$$rh_params{ o_io } = $o_inst_obs;

	my $n = $$rh_params{ &NB };
	print "Building of DisplayMap image $n time(s).\n";

	timethis( $n, sub{ ReBuildDisplayMap( $rh_params, $o_log ) } );

	return;
}


=head2 ReBuildDisplayMap

 Usage        : ReBuildDisplayMap( $rh_params )
 Function     : .
 Args         :
     $rh_params
 Error        : none.

=cut

sub ReBuildDisplayMap
{
	my( $rh_params, $o_log ) = @_;

	my $ra_tgts = $$rh_params{ &TGT };

	my %h_params = ( reference        => $$rh_params{ &REF },
	                 chroms_reference => $$rh_params{ &CHROMS },
	                 target1          => $$ra_tgts[0],
	                 target2          => $$ra_tgts[1],
					 range			  => $$rh_params{ &RANGE },
# 	                 program   => 'genomic',
	                 level     => 3,
	                 o_io      => $$rh_params{ o_io },
	                 -o_logger => $o_log );

	my $o_display = New DisplayMap( %h_params );
	$o_display->ComputeImage();

	return;
} #ReBuildDotplot


=head2 GetParameters

 Usage        : my $rh_params = $o_x->GetParameters()
 Function     : Read parameters from command line and return them formatted as a hash ref.
 Returns      : $rh_params, hash ref with keys ...
 Args         : none

=cut

sub GetParameters
{
	ParamParser::SetDefaultUsage( sub { pod2usage(1) } );

	my @a_keys   = qw(  help );
	push( @a_keys, &REF . '=s', &CHROMS . '=s', &TGT . '=s',  &RANGE . '=s', &NB . '=i' );
	my $o_param  = New ParamParser( 'GETOPTLONG', @a_keys );

	$o_param->SetUnlessDefined( &NB, 1 );

	my $rh_params = {};
	$o_param->Dump( 'HASH', $rh_params );
	
	my $str_tgt = delete $$rh_params{ &TGT };
	if( defined $str_tgt )
	{
		my @a_tgts  = split( /,/, $str_tgt );
		$$rh_params{ &TGT } = \@a_tgts;
	}		

	return $rh_params;
}



__END__

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
