#!/usr/bin/perl

=pod

=head1 NAME

 LaunchDotplot.pl

=head1 SYNOPSIS

 perl LaunchDotplot.pl @args

 @args:
  --reference : string, short species name to use as the reference.
  --target    : string, short species name to use as target.
  --nb        : integer, the number of time to the dotplot will be built.
                default=1

=head1 DESCRIPTION

 # description

=head1 Author

 Sebastien.letort@toulouse.inra.fr

=head1 Version

 Created: 30-03, 2009
 $Id: LaunchDotplot.pl 2577 2009-05-20 12:55:07Z manu $

=cut


use strict;
use warnings;

use Error qw( :try );
use Data::Dumper;
use Pod::Usage;
use Benchmark;

BEGIN
{	# pathway to our modules
	use File::Basename;
	my $dirprg=dirname($0);
	unshift @INC, "$dirprg/../lib";
	unshift @INC, "$dirprg/../API";
}

# nos modules, nos classes
use LipmError;
use General;
use ParamParser;
use Logger;
use Dotplot;

# les constantes
use constant VERSION => do {my @r = (q$Rev: 2577 $ =~ /\d+/g); $r[0]};

use constant DEBUG => &TRUE;
# use constant DEBUG => &FALSE;
$Error::Debug = &DEBUG;

use constant REF => 'reference';
use constant TGT => 'target';
use constant NB  => 'nb';

MAIN:
{
	try
	{
		&Main();
	}
	catch LipmError with
	{
		my ( $o_err, $r ) = @_;

		if( &TRUE == &DEBUG )
		{	warn( $o_err->Explain(), "\n" );	}
		else
		{	print '*' . $o_err->GetUserMessage() . "*\n";	}
	}
	otherwise
	{
		my ( $o_err, $r ) = @_;

		my $str = $o_err->file;
		$str   .= ' line ' . $o_err->line;
		$str   .= "\n" . $o_err->text . "\n";

		print STDERR $str;
	};

	exit( 0 );
}


=head2 Procedure Main

 Title        : Main
 Usage        : &Main();
 Prerequisite : none
 Procedure    : the main function
 Args         : none

=cut

sub Main
{
# 	my $o_log = New Logger( 'LaunchDotplot.pl.log', 'V' );

	my $rh_params = &GetParameters();
# 	warn Dumper $rh_params;

	my $n = $$rh_params{ &NB };
	print "Building of Dotplot image $n time(s).\n";

	timethis( $n, sub{ ReBuildDotplot( $rh_params ) } );

	return;
}


=head2 ReBuildDotplot

 Usage        : ReBuildDotplot( $rh_params )
 Function     : .
 Args         :
     $rh_params
 Error        : none.

=cut

sub ReBuildDotplot
{
	my( $rh_params ) = @_;

	`nc_configure.pl --clear_cache`;

	my %h_params = ( reference => $$rh_params{ &REF },
	                 target    => $$rh_params{ &TGT },
	                 o_io      => New InstanceObserver(),
	                 program   => 'genomic',
# # 	                 program   => 'glint',
	                 level     => 1 );
	my $o_dp = New Dotplot( %h_params );
# 	warn Dumper $o_dp;
	$o_dp->ComputeDotPlots();

	return;
} #ReBuildDotplot


=head2 GetParameters

 Usage        : my $rh_params = $o_x->GetParameters()
 Function     : Read parameters from command line and return them formatted as a hash ref.
 Returns      : $rh_params, hash ref with keys ...
 Args         : none

=cut

sub GetParameters
{
	ParamParser::SetDefaultUsage( sub { pod2usage(1) } );

	my @a_keys   = qw(  help );
	push( @a_keys, &REF . '=s', &TGT . '=s', &NB . '=i' );
	my $o_param  = New ParamParser( 'GETOPTLONG', @a_keys );

	$o_param->SetUnlessDefined( &NB, 1 );

	my $rh_params = {};
	$o_param->Dump( 'HASH', $rh_params );

	return $rh_params;
}



__END__

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
