#!/usr/bin/perl

=pod

=head1 NAME

 NDetector.pl

=head1 SYNOPSIS

 perl NDetector.pl @args

 @args:
  --

=head1 DESCRIPTION

 This script allows to rapidly test if classes SlidingWindowCalculator and
 ThroughSpeciesComputer produce results.
 This script is an enhanced version of SlidingWindowCalculator.pl.

 It will search sequence of at least 1000 N into mtr-1 and write them into N.dat.

=head1 Author

 Sebastien.letort@toulouse.inra.fr

=head1 Version

 Created: 27-03, 2009
 $Id: NDetector.pl 3664 2011-03-10 12:53:47Z manu $

=cut

BEGIN
{	# pathway to our modules
	use File::Basename;
	my $dirprg=dirname($0);
	unshift @INC, "$dirprg/../lib";
	unshift @INC, "$dirprg/../API";
}

package Fixture;

use base qw( NDetector );

=head2 function __ComputeFilepath

	Title        : _ComputeFilepath
	Usage        : $filepath = $o_xx->_ComputeFilepath();
	Prerequisite : none
	Function     : Compute the filepath and make the path if necessary.
	Returns      : $filepath, the formatted absolute filepath
	Args         : none

=cut

sub _ComputeFilepath
{
	my $self = shift;

	return "./N.dat";
}


1;
#-------------------

use strict;
use warnings;

use Error qw( :try );
use Data::Dumper;
use Pod::Usage;

# nos modules, nos classes
use LipmError;
use General;
use ParamParser;
use Logger;

# les constantes
use constant VERSION => do {my @r = (q$Rev: 3664 $ =~ /\d+/g); $r[0]};

use constant DEBUG => &TRUE;
# use constant DEBUG => &FALSE;

MAIN:
{
	try
	{
		&Main();
	}
	catch LipmError with
	{
		my ( $o_err, $r ) = @_;

		if( &TRUE == &DEBUG )
		{	warn( $o_err->Explain(), "\n" );	}
		else
		{	print '*' . $o_err->GetUserMessage() . "*\n";	}
	}
	otherwise
	{
		my ( $o_err, $r ) = @_;

		my $str = $o_err->file;
		$str   .= ' line ' . $o_err->line;
		$str   .= "\n" . $o_err->text . "\n";

		print STDERR $str;
	};

	exit( 0 );
}


=head2 Procedure Main

 Title        : Main
 Usage        : &Main();
 Prerequisite : none
 Procedure    : the main function
 Args         : none

=cut

sub Main
{
	my $o_log = New Logger( 'NDetector.pl.log', 'ICVIO' );

	my $rh_params = &GetParameters();

	my @a_species = qw( mtr );
	my @a_chroms  = qw( 1 );

	my %h_params = ( -species => \@a_species,
	                 -chroms  => \@a_chroms,
	                 -size    => 1000,
	                 -force   => 1 );
	my $o_detector = New Fixture( %h_params );
	$o_detector->RollOverSpecies();

	return;
}


=head2 GetParameters

 Usage        : my $rh_params = $o_x->GetParameters()
 Function     : Read parameters from command line and return them formatted as a hash ref.
 Returns      : $rh_params, hash ref with keys ...
 Args         : none

=cut

sub GetParameters
{
# 	ParamParser::SetDefaultUsage( pod2usage(1) );

	my @a_keys   = qw( help );
	my $o_param  = New ParamParser( 'GETOPTLONG', @a_keys );

	my %h_params = ();

	return \%h_params;
}



__END__

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
	