#!/usr/bin/perl

=pod

=head1 NAME

 NarcisseObserver

=head1 SYNOPSIS

 perl NarcisseObserver @args

 @args:
  --

=head1 DESCRIPTION

 # description

=head1 Author

 Sebastien.letort@toulouse.inra.fr

=head1 Version

 Created: 28-04, 2009
 $Id: NarcisseObserver.pl 2678 2009-06-10 14:34:00Z sletort $

=cut


use strict;
use warnings;

use Error qw( :try );
use Data::Dumper;
use Pod::Usage;

BEGIN
{	# pathway to our modules
	use File::Basename;
	my $dirprg=dirname($0);
	unshift @INC, "$dirprg/../lib";
	unshift @INC, "$dirprg/../API";
}

# nos modules, nos classes
use LipmError;
use General;
use ParamParser;
use Logger;

use Configuration::NarcisseObserver;

# les constantes
use constant VERSION => do {my @r = (q$Rev: 2678 $ =~ /\d+/g); $r[0]};

use constant DEBUG => &TRUE;
# use constant DEBUG => &FALSE;

MAIN:
{
	try
	{
		&Main();
	}
	catch LipmError with
	{
		my ( $o_err, $r ) = @_;

		if( &TRUE == &DEBUG )
		{	warn( $o_err->Explain(), "\n" );	}
		else
		{	print '*' . $o_err->GetUserMessage() . "*\n";	}
	}
	otherwise
	{
		my ( $o_err, $r ) = @_;

		my $str = $o_err->file;
		$str   .= ' line ' . $o_err->line;
		$str   .= "\n" . $o_err->text . "\n";

		print STDERR $str;
	};

	exit( 0 );
}


=head2 Procedure Main

 Title        : Main
 Usage        : &Main();
 Prerequisite : none
 Procedure    : the main function
 Args         : none

=cut

sub Main
{
	my $o_log = New Logger( 'NarcisseObserver.log', 'IO' );

	my $rh_params = &GetParameters();

	print "You should test InstanceObserver before performing this test\n";
	print "WARNING : This test only work in 'multi' environnement\n\n";

	my $o_nar_obs = New NarcisseObserver( -o_logger => $o_log );

	# various instance infos
	print "***---***\n";
	&VariousInformations( $o_nar_obs );

	# methode disparue, on ne sait jamais elle peut revenir
# 	my $o_inst_obs = $o_nar_obs->GetInstanceObserver( $a_list[0] );
# 	&SafeIsa( $o_inst_obs, 'InstanceObserver' );
# 	my $name = $o_inst_obs->GetName();
# 	print "GetInstanceObserver ok if ( $name == $a_list[0] )\n";


	# filter sur instance
	print "***---***\n";
	&GetSomeInstanceObserver( $o_nar_obs );

	# method to access things
	print "***---***\n";
	&AccessThings( $o_nar_obs );

	return;
}


sub AccessThings
{
	my $o_nar_obs = shift;

	print "=>Accesthings tests :\n\n";

	my $path = $o_nar_obs->GetDB();
	print "Access DB default : $path\n";

	$path = $o_nar_obs->GetDB( 'prot' );
	print "Access DB prot    : $path\n";

	$path = $o_nar_obs->GetDB( 'nuc' );
	print "Access DB nuc     : $path\n";

	return;
}

sub GetSomeInstanceObserver
{
	my $o_nar_obs = shift;

	print "=>FindInstanceObservers tests :\n\n";

	my $ra_io = $o_nar_obs->FindInstanceObservers();
	foreach my $o_io ( @$ra_io )
	{
		print $o_io->GetName() . "\n";
	}
	print "FindInstanceObservers() ok if all instance have been listed.\n";

	my $i_name = 'plants';
	$ra_io     = $o_nar_obs->FindInstanceObservers( ra_instances => [ $i_name ] );
	my $name   = $$ra_io[0]->GetName();
	print "FindInstanceObservers( ra_instances => [...] ) ok if ( $name == '$i_name' )\n";

	$ra_io = $o_nar_obs->FindInstanceObservers( ra_species => [ 'mtr' ] );
	$name  = $$ra_io[0]->GetName();
	print "FindInstanceObservers( ra_species => [...] )   ok if ( $name == 'plants' )\n";

	$ra_io = $o_nar_obs->FindInstanceObservers( ra_instances => [ qw( animals plants ) ],
												ra_species => [ 'mtr' ] );
	$name  = $$ra_io[0]->GetName();
	print "FindInstanceObservers( \%h_logique )           ok if ( $name == 'plants' )\n";

	$ra_io = $o_nar_obs->FindInstanceObservers( ra_instances => [ qw( animals ) ],
												ra_species => [ 'mtr' ] );
	my $nb = @$ra_io;
	print "FindInstanceObservers( \%h_incoherent )        ok if ( $nb == 0 )\n";

	return;
}


sub VariousInformations
{
	my $o_nar_obs = shift;

	my %h_ = $o_nar_obs->GetInstancesDescr();
	print "Should see a hash with instance's name as key and a description as value \n";
	print Dumper( \%h_ ) . "\n";

	print "Should see the keys of previous Hash\n";
	my @a_list = $o_nar_obs->GetInstancesNames();
	print Dumper( \@a_list ) . "\n";

	return;
}

=head2 GetParameters

 Usage        : my $rh_params = $o_x->GetParameters()
 Function     : Read parameters from command line and return them formatted as a hash ref.
 Returns      : $rh_params, hash ref with keys ...
 Args         : none

=cut

sub GetParameters
{
	ParamParser::SetDefaultUsage( sub { pod2usage(1) } );

	my @a_keys   = qw(  help );
#	push( @a_keys, );
	my $o_param  = New ParamParser( 'GETOPTLONG', @a_keys );

	my $rh_params = {};
	$o_param->Dump( 'HASH', $rh_params );

	return $rh_params;
}



__END__

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
