#!/usr/bin/perl

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


=pod

=head1 NAME

 NarcisseUtils

=head1 SYNOPSIS

 perl NarcisseUtils @args

 @args:
  --

=head1 DESCRIPTION

 # description

=head1 Author

 Sebastien.letort@toulouse.inra.fr

=head1 Version

 Created: 31-07, 2009
 $Id: NarcisseUtils.pl 3595 2011-01-15 11:33:13Z sletort $

=cut


use strict;
use warnings;

use Error qw( :try );
use Data::Dumper;
use Pod::Usage;
use FindBin;

use lib "$FindBin::Bin/../lib";
use lib "$FindBin::Bin/../API";

# nos modules, nos classes
use LipmError;
use General;
use ParamParser;
use Logger;
use NARCISSE_utils;


# les constantes
use constant VERSION => do {my @r = (q$Rev: 3595 $ =~ /\d+/g); $r[0]};

use constant DEBUG => &TRUE;
# use constant DEBUG => &FALSE;
$Error::Debug = &DEBUG;

my $SPECIES = "human_ncbi36_3";
my $CHROM   = "5";

MAIN:
{
	try
	{
		&Main();
	}
	catch LipmError with
	{
		my ( $o_err, $r ) = @_;

		if( &TRUE == &DEBUG )
		{
			warn( $o_err->Explain(), "\n" );
			warn $o_err->stacktrace(); 
		}
		else
		{	print '*' . $o_err->GetUserMessage() . "*\n";	}
	}
	otherwise
	{
		my ( $o_err, $r ) = @_;

		my $str = $o_err->file;
		$str   .= ' line ' . $o_err->line;
		$str   .= "\n" . $o_err->text . "\n";

		print STDERR $str;
	};

	exit( 0 );
}


=head2 Procedure Main

 Title        : Main
 Usage        : &Main();
 Prerequisite : none
 Procedure    : the main function
 Args         : none

=cut

sub Main
{
	my $o_log = New Logger( 'NarcisseUtils.log', 'V' );

	my $rh_params = &GetParameters();

	# seb - 15/01/2011
	# les fonctions testes n'existe plus !
	# &TestPrintNarcisseData();

	&TestParseNarcisseRegions();

	&TestReadNarcisseTabFile();

	return;
}


sub TestReadNarcisseTabFile
{
	# my $o_io = InstanceObserver::Singleton();
	my $o_sp = New SpeciesObserver( species => $SPECIES );

	my $transcript_file = $o_sp->GetTranscriptFiles( -chrom  => $CHROM );
	warn "reading $transcript_file\n";
	my( $rh_data ) = &NARCISSE_utils::ReadNarcisseTabFile( file => $transcript_file );

	warn Dumper $rh_data;

	return;
}


sub TestParseNarcisseRegions
{
	print "Test de ParseNarcisseRegion :\n";

	my @a_regions = qw( Medicago_truncatula:6:19550672:19550737:+
						Medicago_truncatula:4:9751750:9751832:+
						Medicago_truncatula:4:5:1000:- );
	my $o_io = InstanceObserver::Singleton();

	my $rh_regions = &NARCISSE_utils::ParseNarcisseRegion( $o_io, \@a_regions );
	&General::ShowDebug( $rh_regions );

	return;
}

sub TestPrintNarcisseData
{
	# trie d'un tableau de hash standard
	my $ra_vals = [	{ start => 1,  end => 5 },
					{ start => 70, end => 80 },
					{ start => 55, end => 75 },
					{ start => 100,end => 101 },
					{ start => 55, end => 70 } ];
	print Dumper $ra_vals;
	my $ra_sorted = NARCISSE_utils::SortNarcisseDataOnStartEnd( $ra_vals );
	print Dumper $ra_sorted;

	print "\n\n";
	NARCISSE_utils::WriteNarcisseData( 'toto.dat', [qw( element end start )], $ra_vals, 13, 'test' );

	NARCISSE_utils::InsertNarcisseData( 'toto.dat', [qw( element end start )], [ { start => 30, end => 22 }] );

	return;
}


=head2 GetParameters

 Usage        : my $rh_params = $o_x->GetParameters()
 Function     : Read parameters from command line and return them formatted as a hash ref.
 Returns      : $rh_params, hash ref with keys ...
 Args         : none

=cut

sub GetParameters
{
	ParamParser::SetDefaultUsage( sub { pod2usage(1) } );

	my @a_keys   = qw(  help );
#	push( @a_keys, );
	my $o_param  = New ParamParser( 'GETOPTLONG', @a_keys );

	my $rh_params = {};
	$o_param->Dump( 'HASH', $rh_params );

	return $rh_params;
}



__END__
