#! /usr/bin/perl

use strict;

use Data::Dumper;
use File::Basename;
use IO::File;
use Cwd qw( abs_path );


use lib qw( ../../API ../../modules );
BEGIN
{
	my $dirprg = dirname( $0 );
	my $modules_dir = $dirprg . '/../../modules';
	my $lib_dir = $dirprg . '/../lib';
	unshift( @INC, $modules_dir )	if( -d $modules_dir );
	unshift( @INC, $lib_dir )	if( -d $lib_dir );
}

use Bio::Narcisse::DBAdaptor;
use ParamParser;
use Narcisse_tools;	#PrintSimpleOutput, FindNarcisseInstances, FormatSequenceLine
use General qw( :DEFAULT ShowDebug );
use Services::SpeciesNames;

use constant DEF_DB => 'proteic';


MAIN:
{
	my @a_options = qw( instance=s db=s );
	my $o_pp = New ParamParser( 'GETOPTLONG', @a_options );

	# 1- find on which instances of Narcisse job will be perform
	my $instances_key = $o_pp->Get('instance');
	$instances_key = ''	if( $instances_key eq 'all' );
	my $rh_instances = &FindNarcisseInstances( $instances_key );

	my @a_list;
	my $db = &GetDB( $o_pp );

	foreach my $rh_data ( values %$rh_instances )
	{
		my @a_inst_list;

		my $cfg = $$rh_data{ cfg_file };
# 		print STDERR Dumper $cfg;
		my $o_pp = New ParamParser( $cfg );

		# une liste d'espece
		my $o_service = New Services::SpeciesNames(
							cfg  => $$rh_data{ cfg_file },
							mode => 'Short'
						);
		my @a_species = $o_service->Get();

		foreach my $species ( @a_species )
		{
			my @a_list_species;
			# le chemin vers la bd
			my $fasta_file = $o_pp->Get( 'ROOT_DATA' );
			my $db = $species . '/db/gene/' . $species . '_gene.fasta';
			$fasta_file .= '/' . $db;

			my $fe = new IO::File( $fasta_file, 'r' )
					or throw LipmError::IOException( $fasta_file, 'f' );
			while( my $line = <$fe> )
			{
				# plusieurs grep sont plus rapide qu'un seul !
				if( $line =~ /^>/ )
				{
print STDERR '.';
					$line =~ /locus_tag=(\S+)/;
					push @a_list_species, $1;
				}
			}
			print "\n$species\n";
			&CountDoublons(@a_list_species);

			push @a_inst_list, @a_list_species;
		} # specie
print STDERR "\n";
		sort @a_inst_list;
# print STDERR Dumper @a_inst_list;
		push @a_list, @a_inst_list;
	}# instances

}


sub CountDoublons
{
	my ( @a_list ) = @_;

	my %h_ids;
	foreach my $id ( sort @a_list )
	{
# 		print $id, "\n";
		$h_ids{ $id } = 0	unless defined( $h_ids{ $id } );
		$h_ids{ $id }++;
	}
	while( my ($key, $val ) = each( %h_ids ) )
	{
		if( $val > 1 )
		{	print "$key est un doublon qui apparait $val fois.\n";	}
		if( $val == 0 )
		{	print STDERR "!!!!$key!!!!!!.\n";	}
	}
}


sub GetDB
{
	my ( $o_options ) = @_;
	my $db;

	$o_options->SetUnlessDefined( 'db', &DEF_DB );

	# 1- l'espece
	my $species = $o_options->Get( 'species' );

	if( IsNotNull( $species ) )
	{
		$db = $species . '/db';
		my $tmp = ( $o_options->Get( 'db' ) =~ /prot/ ) ? 'prot': 'cds';
		$db .= '/' . $tmp . "/$species\_$tmp.fasta";
	}
	else
	{
		if( $o_options->Get( 'db' ) =~ /prot/ )
		{	$db = 'DB_PROT_MERGE';	}
		else
		{	$db = 'DB_CDS_MERGE';	}
	}

	return $db;
}

__END__
NE MARCHE PAS AVEC L'API

MAIN:
{

		# 0- obtenir une liste d'organisme
		my @a_species;
		my $species = $o_pp->Get( 'species' );
		if( IsNotNull( $species ) )
		{	@a_species = ( $species );	}
		else
		{
			my $o_service = New Services::SpeciesNames(
								cfg  => $$rh_data{ cfg_file },
# cfg  => '/mnt/data2/www-narcisse/Dev/seb_ws/Site/cfg/narcisse.cfg',
								mode => 'Short'
							);
			@a_species = $o_service->Get();
		}

			# 1- pour chaque organisme
			foreach $species ( @a_species )
			{
# print STDERR Dumper [ $species ];
				my $o_dba = new Bio::Narcisse::DBAdaptor(
												-cfg => $$rh_data{ cfg_file },
												-dbname => $species
											);
				next	if( !$o_dba );

				my $ra_list = $o_dba->get_GeneAdaptor()->list_stable_ids( );
print Dumper $ra_list;
				push @a_list, @$ra_list	if( $ra_list );
			}
	}

foreach my $id (@a_list)
{	print $id, "\n";	}
}
