#!/usr/bin/perl
use strict;
use warnings;

use Error qw( :try );
use Data::Dumper;

BEGIN
{	# pathway to our modules
	use File::Basename;
	my $dirprg=dirname($0);
	unshift @INC, "$dirprg/../API";
}


# nos modules, nos classes
use LipmError;
use ParamParser;
use General;

use NarcisseServiceFactory;
use InstanceObserver;

MAIN:
{
	try
	{
		&Main();
	}
	catch LipmError with
	{
		my ($o_err, $r ) = @_;

		warn( $o_err->Explain(), "\n" );
		print '*' . $o_err->GetUserMessage() . "*\n";
		print STDERR $o_err->stacktrace();
	}
	otherwise
	{
		my ($o_err, $r ) = @_;

		my $str = $o_err->file;
		$str   .= ' line ' . $o_err->line;
		$str   .= "\n" . $o_err->text . "\n";

		print STDERR $str;
		print STDERR $o_err->stacktrace();
	};

	exit( 0 );
}

sub Main
{
	my @a_params = ();
	my $o_param = New ParamParser('GETOPTLONG', @a_params );

	my $species    = 'mtr';

	# liste des chroms
	my $o_inst_obs = New InstanceObserver();
	my $o_dba = $o_inst_obs->GetDBAdaptor( $species );
		#get the appropriate structure
	my $slice_adaptor = $o_dba->get_SliceAdaptor();
	my $rh_chroms = $slice_adaptor->karyotype();

	my $out = new IO::File( 'codant.mtr', 'w' );

	# pour chaque chrom lancer le service
	foreach my $chrom ( keys %$rh_chroms )
# 	foreach my $chrom ( 6 )
	{
		print "working with chrom $chrom\n";
		my %h_params = ( species => $species,
						 chrom   => $chrom,
						 ra_regions => [ { start => 1, end => $$rh_chroms{ $chrom }{ length } } ] );
		my $o_ns = NarcisseServiceFactory::GetCDSsFromRegions( %h_params );

		my $rh_results = $o_ns->FetchResults();
		print "found " .  scalar( keys %$rh_results ) . " results\n";

		my $ra_not_found = $o_ns->FetchNotFound();
		print "not found " . scalar( @$ra_not_found ) . " values\n";

		foreach my $header ( keys %$rh_results )
		{
			print $out $header . "\n";
			print $out $$rh_results{ $header } . "\n";
		}
	}

}
