#!/usr/bin/perl

=pod

=head1 NAME

 qd_multi_test

=head1 SYNOPSIS

 perl qd_multi_test @args

 @args:
  --

=head1 DESCRIPTION

 # description

=head1 Author

 Sebastien.letort@toulouse.inra.fr

=head1 Version

 Created: 16-12-2008
 $Id: qd_multi_test.pl 2660 2009-06-05 09:01:54Z manu $

=cut


use strict;
use warnings;

use Error qw( :try );
use Data::Dumper;

BEGIN
{	# pathway to our modules
	use File::Basename;
	my $dirprg=dirname($0);
	unshift @INC, "$dirprg/../API";
}


# nos modules, nos classes
use LipmError;
use ParamParser;
use General;
use Logger;

use NarcisseObserver;

use NarcisseService::Accessor;
use NarcisseService::Accessor::WhatDecorator::GenesDecorator;
use NarcisseService::Accessor::FromDecorator::RegionsDecorator;
use NarcisseServiceFactory;


# les constantes
use constant VERSION => do {my @r = (q$Rev: 2660 $ =~ /\d+/g); $r[0]};

MAIN:
{
	try
	{
		&Main();
	}
	catch LipmError with
	{
		my ( $o_err, $r ) = @_;

		warn( $o_err->Explain(), "\n" );
		print '*' . $o_err->GetUserMessage() . "*\n";
	}
	otherwise
	{
		my ( $o_err, $r ) = @_;

		my $str = $o_err->file;
		$str   .= ' line ' . $o_err->line;
		$str   .= "\n" . $o_err->text . "\n";

		print STDERR $str;
	};

	exit( 0 );
}


=head2 Procedure Main

 Title        : Main
 Usage        : &Main();
 Prerequisite : none
 Procedure    : the main function
 Args         : none

=cut

sub Main
{
	my $instance = 'plants';
	my $species    = 'mtr';
	my $chrom      = 5;
	my $ra_regions = [ { start => 10, end => 100 } ];
	my @a_mtr_locus_tags = qw( AC126011_43.4 AC202362_19.3 AC175311_20.4 CT963073_42.4 );
	my @a_ath_locus_tags = qw( SCPL37 WRKY58 );

	my $o_log = New Logger( '>>qd.log', 'IOVISV' );

	my ( $o_nar_obs, $rao_inst_obs );

    # ATTENTION (manu)
    # GetInstanceObserver n'existe PLUS dans le nouvel objet NarcisseObserver
    
# 	my $o_nar_obs = New NarcisseObserver( -o_logger => $o_log,
# 	                                      -str_instance => $instance );
# 	my $o_inst_obs = $o_nar_obs->GetInstanceObserver( $instance );
#
# # 	warn Dumper { nar_obs => $o_nar_obs->{ __rh_instances } };
# # warn Dumper { main => $o_inst_obs->{ __cfg } };
#
# 	my $o_ns = New NarcisseService::Accessor( species => $species,
# 	                                          -o_instance_obs => $o_inst_obs,
# 	                                          -o_logger => $o_log );
#
# 	# homogeneiser les noms => tout au singulier.
# 	$o_ns    = New FromDecorator::RegionsDecorator( o_accessor => $o_ns,
# 	                                 chrom => $chrom,
# 	                                 ra_regions => $ra_regions );
#
# 	$o_ns    = New WhatDecorator::GenesDecorator( o_accessor => $o_ns );
#
# 	warn Dumper [ keys %$o_ns ];
#
#
# 	my $ra_results = $o_ns->FetchResults();
#
# 	warn Dumper $ra_results;
#
# 	print "------------------------------\n";

# 	$o_nar_obs = New NarcisseObserver( -o_logger => $o_log );
# # 	warn Dumper { nar_obs => $o_nar_obs->{ __rh_instances } };
#
# 	$rao_inst_obs = $o_nar_obs->FindInstanceObserver( short => $species );
# # warn Dumper { main => scalar( @$rao_inst_obs ) };
#
# 	foreach my $o_inst_obs ( @$rao_inst_obs )
# 	{
# 		my $o_ns = New NarcisseService::Accessor( species => $species,
# 		                                          -o_instance_obs => $o_inst_obs,
# 		                                          -o_logger => $o_log );
#
# 		$o_ns    = New FromDecorator::RegionsDecorator( o_accessor => $o_ns,
# 		                                 chrom => $chrom,
# 		                                 ra_regions => $ra_regions );
#
# 		$o_ns    = New WhatDecorator::GenesDecorator( o_accessor => $o_ns );
#
# # warn Dumper { ref => ref $o_ns, keys => [ keys %$o_ns ] };
#
# 		my $ra_results = $o_ns->FetchResults();
#
# 		warn Dumper $ra_results;
# 	}
	print "-------NSF::GetGenesFromRegions----------\n";

	$o_nar_obs = New NarcisseObserver( -o_logger => $o_log );
# 	warn Dumper { nar_obs => $o_nar_obs->{ __rh_instances } };

	$rao_inst_obs = $o_nar_obs->FindInstanceObservers( short => $species );
# warn Dumper { main => scalar( @$rao_inst_obs ) };

	foreach my $o_inst_obs ( @$rao_inst_obs )
	{
		my %h_params = ( species => $species,
		                 chrom   => $chrom,
		                 ra_regions => $ra_regions,
		                 -o_instance_obs => $o_inst_obs,
		                 -o_logger => $o_log
		                  );
# warn Dumper { main => \%h_params };
		my $o_ns = NarcisseServiceFactory::GetGenesFromRegions( %h_params );
		my $ra_results = $o_ns->FetchResults();

		warn Dumper $ra_results;
	}

#--------------------------
	$species = undef;
#--------------------------

	print "--------NSF::GetGenesFromLocusTags--------\n";


	$o_nar_obs = New NarcisseObserver( -o_logger => $o_log );
# 	warn Dumper { nar_obs => $o_nar_obs->{ __rh_instances } };

	$rao_inst_obs = $o_nar_obs->FindInstanceObservers();
# warn Dumper { main => scalar( @$rao_inst_obs ) };

	foreach my $o_inst_obs ( @$rao_inst_obs )
	{
		my $rh_results = ();

		if( defined $species )
		{
			&_RunService( $o_inst_obs, $species, $rh_results );
		}
		else
		{
			my $ra_species = $o_inst_obs->ExtractMySpecies( [] );
			foreach my $species ( @$ra_species )
			{
				&_RunService( $o_inst_obs, $species, $rh_results );
			}
		}

		warn Dumper $rh_results;
	}
	print "------------------------------\n";


	sub _RunService
	{
		my $o_inst_obs = shift;
		my $species    = shift;
		my $rh_results = shift;
warn $species;
		my %h_params = ( species  => $species,
						ra_tags  =>[@a_mtr_locus_tags, @a_ath_locus_tags ],
						-o_instance_obs => $o_inst_obs,
						-o_logger=> $o_log
						);
	# warn Dumper { main => \%h_params };
		my $o_ns = NarcisseServiceFactory::GetGenesFromLocusTags( %h_params );
		my $rh_ = $o_ns->FetchResults();
		my @a_keys = keys %$rh_;
		@$rh_results{ @a_keys } = @$rh_{ @a_keys };
	}

	return;
}

__END__
