/*
  Copyright INRA/CNRS

 Emmanuel.Courcelle@toulouse.inra.fr
 Jerome.Gouzy@toulouse.inra.fr
 Thomas.Faraut@toulouse.inra.fr

 This software is a computer program whose purpose is to provide a
 web-based interface for analyzing the different levels of genome
 conservations.

 This software is governed by the CeCILL license under French law and
 abiding by the rules of distribution of free software.  You can  use,
 modify and/ or redistribute the software under the terms of the CeCILL
 license as circulated by CEA, CNRS and INRIA at the following URL
 "http://www.cecill.info".

 As a counterpart to the access to the source code and  rights to copy,
 modify and redistribute granted by the license, users are provided only
 with a limited warranty  and the software's author,  the holder of the
 economic rights,  and the successive licensors  have only  limited
 liability.

 In this respect, the user's attention is drawn to the risks associated
 with loading,  using,  modifying and/or developing or reproducing the
 software by the user in light of its specific status of free software,
 that may mean  that it is complicated to manipulate,  and  that  also
 therefore means  that it is reserved for developers  and  experienced
 professionals having in-depth computer knowledge. Users are therefore
 encouraged to load and test the software's suitability as regards their
 requirements in conditions enabling the security of their systems and/or
 data to be ensured and,  more generally, to use and operate it in the
 same conditions as regards security.

 The fact that you are presently reading this means that you have had
 knowledge of the CeCILL license and that you accept its terms

*/
/*

=pod

=head1 NAME
 NcView.js - Global objects and classes for NcView.pl

=cut
*/

/*
=pod

=head1 SYNOPSIS

 NarcisseAreaClass               see narcisse_area.js
    PersonalClass

 NcViewClass

=cut
*/
/* EXPERIMENTAL !	
DraggingClass = Class.create({
			initialize: function(inner_container)
			{
				this.__inner_cont = $(inner_container);
				this.__start_top  = 0;
				this.__top        = 0;
				this.__dragging   = false;
				
				Event.observe(this.__inner_cont,'mousedown',this.__StartMove.bindAsEventListener(this));
				Event.observe(this.__inner_cont,'mousemove',this.__Moving.bindAsEventListener(this));
				Event.observe(this.__inner_cont,'mouseup',this.__StopMove.bindAsEventListener(this));
			},
			
			__StartMove: function(evt)
			{
				this.__start_top = evt.clientY;
				this.__inner_cont.style.cursor = "-moz-grab";
				this.__top = this.__stripPx(this.__inner_cont.style.top);
				this.__dragging = true;
			},
			
			__Moving: function(evt)
			{
				if (this.__dragging)
				{
					this.__inner_cont.setStyle({top: this.__top + (evt.clientY - this.__start_top)});
				}
			},
			
			__StopMove: function(evt)
			{
				this.__inner_cont.style.cursor = "";
				this.__dragging   = false;
			},
			
			__StripPx: function(value)
			{
				if (value == "") return 0;
				return parseFloat(value.substring(0, value.length - 2));
			}
	});
*/



/*
=pod

=head2 Personal Class

 Constructor Args:
    top_id     : The parent element of the created area
    x1,y1,x2,y2: The coordinates of the area
    orga       : The organism
    chrom      : The chromosome
    start      : The start position (in bases) of the user-defined area
    end        : The end position
    label      : The label

=cut
*/

PersonalClass = Class.create(NarcisseAreaClass,{
	initialize: function($super,a_graphics_objects){
		$super(a_graphics_objects);
		this._feature = 'personal';
	},
	MouseOver: function($super,x1,y1,x2,y2,organism,chrom,start,end,label)
	{


	if ( $super(x1,y1,x2,y2) == false ) { return; };

	var html='<h2>Personal data<br>'+label+'</h2>';
	var length = end-start;

	html += '<table>';
	html += '<tr><th>organism</th><td>' + organism + ' (' + chrom + ')</td></tr>';
	html += '<tr><th>start</th><td>' + PrettyDisplayPosition(start) + ' (' + start + ')</td></tr>';
	html += '<tr><th>end</th><td>' + PrettyDisplayPosition(end) + ' (' + end + ')</td></tr>';
	html += '<tr><th>length</th><td>' + PrettyDisplayPosition(length) + ' (' + length + ')</td></tr>';
	html += '</table>';
	$(this._legend).innerHTML = html;
	}
});


/*
=pod

=head2 NcView Class

 This class is an abstract class. It is a base class for the NcDotplot, NcDisplayMap etc. objects used with the different representations
 Some methods are defined by Ncview, they'll be used the the control block (cf. controlblock.js)
      GetScript returns the script= parameter value for this representation
	  CBToHide() returns the list of IDs to hide from the control block for this representation
	  CBToShow returns the list of IDs to show from the control block for this representation
	  UpdateTitle is an abstract procedure, the overloaded procedures update the title box: this is specific to each view 
	  _GetSpeciesNames may be called by the derived classes, from their UpdateTitle methods
 
 Constructor Args:
    a_graphics_objects : A lot of graphics objects
	TODO - Remplacer ca par un objet !

=cut
*/

NcViewClass = Class.create({
		initialize:function(a_graphics_objects){
			// TODO - PAS BON DU TOUT !!! A corriger avec a_graphics_objects
			this._title_div = $('ID_TITLE');
		},
		GetScript: function() 
		{
			if (this._script != undefined)
			{
				return this._script;
			}
			else
			{
				throw('NcViewClass: this._script not initialized');
			}
		},
		GetCBToHide: function()
		{
			if (this._to_hide != undefined)
			{
				return this._to_hide;
			}
			else
			{
				throw('NcViewClass: this._to_hide not initialized');
			}
		},
		GetCBToShow: function() 
		{
			if (this._to_hide != undefined)
			{
				return this._to_show;
			}
			else
			{
				throw('NcViewClass: this._to_show not initialized');
			}
		},
		UpdateTitle: function()
		{
			throw('NcViewClass: UpdateTitle is an abstract function');
		},
		
		// display the list of species, reference and targets (may belimited to N)
		_GetSpeciesNames: function(N)
		{
			var html= '<table class="TableRelease">';
			var parity = 'odd';
			html += '<tr class="' + parity + '"><th>Reference</th><td>'+PrettyDisplaySpecies(GetReference())+'</td></tr>';
			GetTargets(N).each(function(s){
				parity = (parity=='odd') ? 'even' : 'odd';
				html += '<tr class="' + parity + '"><th>Target</th><td>'+PrettyDisplaySpecies(s)+'</td></tr>';
			});
			html += '</table>';
			if (IsOnlyPseudo(N) == false )
			{
				var max_dsp_scaf = O_CFG.MAX_DISPLAYED_SCAFFOLDS;
				html += '<p class="Warning"><strong>WARNING</strong> - For species labelled as CONTIGS, SUPERCONTIGS, SCAFFOLDS, display is limited to the ' + max_dsp_scaf + ' LONGEST objects</p>';
			}
			return html;
		}
});


MAIN:
{

	/* Global variables, they will be initialized later */
	O_PERSONAL=0;
	O_POPUP=0;
	O_SCREEN=0;
	O_CONTROL_BLOCK=0;
	O_AJAX_SIMPLE=0;
	O_ERROR="";
	E_IMG_CONTAINER="";

	function Load()
	{

		/* This could happen with very simple pages (just an error message for instance) */
		if ( $('ID_CONTROLBLOCK')==null ) {return;};

		/* We create now a few objects, 1 object for each area type */
		var o_marker1       = new DynamicMarkerClass('ID_DYNAMIC_MARKER');
		var o_marker2       = ($('ID_DYNAMIC_MARKER2') == null) ? undefined : new DynamicMarkerClass('ID_DYNAMIC_MARKER2'); 

		/* O_ERROR, o_led and o_history are useful for the control block */
		O_ERROR         = new LipmPopupYUIClass('ID_ERROR');
		//O_ERROR         = new LipmPopupSimpleClass('ID_ERROR');

		var o_led1 = new LipmLedClass('ID_LED','off');
		var o_led2 = new LipmLedPatienceClass('wb_patience','off');
		var o_led  = new LipmLedCompositeClass('off');
		o_led.RegisterLed(o_led1);
		o_led.RegisterLed(o_led2);

		var o_history       = new LipmHistoryUlClass('ID_HISTORY','start','read_bookmark='+O_PARAM.history_stamp);

		/* Create and register the control block and its tabs */
		E_IMG_CONTAINER=$('ID_IMG_CONTAINER');

		var o_ajax_container= new LipmPopupSimpleClass('ID_IMG_CONTAINER');
		O_CONTROL_BLOCK     = new ControlBlockClass('ID_CONTROLBLOCK',o_ajax_container,O_ERROR,o_led,o_history);
		O_CONTROL_BLOCK.SetSubmitUrl(O_CFG["HTTP_NARCISSE_CGI"]);

		// Update the control block after each partial request
		O_CONTROL_BLOCK.SetPartialHandler(O_CONTROL_BLOCK.UpdatePage.bind(O_CONTROL_BLOCK));
		
		var o_position_form = new PositionFormClass('ID_BROWSE',O_CONTROL_BLOCK);
		var o_data_form     = new DataFormClass('ID_DATA',O_CONTROL_BLOCK);
		var o_features_form = new FeaturesFormClass('ID_DISPLAY',O_CONTROL_BLOCK);
		var o_bookmarks_form= new BookmarksFormClass('ID_SESSION',O_CONTROL_BLOCK);
		
		// The dotplot and circos pseudo-features
		O_CONTROL_BLOCK.RegisterTab(new DotplotFeatureClass);
		O_CONTROL_BLOCK.RegisterTab(new CircosFeatureClass);

		// Render the control block (useful for the Tabview object)
		O_CONTROL_BLOCK.Render();

		// Create the graphics objects
		var	o_dessus_dessous= new DessusDessousClass(O_CONTROL_BLOCK,'ID_DYNAMIC_LEGEND_CONTAINER');
		//O_POPUP             = new LipmPopupYUIClass('ID_DYNAMIC_POPUP',false);
		O_POPUP             = new LipmPopupYUITabClass('ID_TAB_POPUP',O_CONTROL_BLOCK.GetTabview(),5);
		
		// See also the same line in narcisse.js. Here it is still useful because of the
		// O_POPUP.RegisterLinkObserver which follows
		var o_new_window_seq= new LipmPopupNewWindowClass('WindowSequence',{
				directories: false,
				location: false,
				menubar: false,
				toolbar: false,
				scrollbars: true,
				status: true,
				resizable: true,
				height: 500,
				width: 600
		});
		O_POPUP.RegisterLinkObserver(o_new_window_seq);

		var o_new_window_blank = new LipmPopupNewWindowClass('NewWindowBlank','',true);
		O_POPUP.RegisterLinkObserver(o_new_window_blank);

		var o_new_big_window = new LipmPopupNewWindowClass('BigWindow',{
				directories: false,
				location: false,
				menubar: false,
				toolbar: false,
				scrollbars: true,
				status: true,
				resizable: true,
/*				height: 600,
				width: 800,*/
				fullscreen:true
		});
		O_POPUP.RegisterLinkObserver( o_new_big_window );

		O_OPENWIN_ALIGNMENT = new LipmPopupNewWindowClass('WindowAlignment',{
				directories: false,
				location: false,
				menubar: false,
				toolbar: false,
				scrollbars: true,
				status: true,
				resizable: true,
				height: 500,
				width: 1100
		});
		var o_new_window_help = new LipmPopupNewWindowClass('WindowHelp',{
				directories: false,
				location: false,
				menubar: false,
				toolbar: false,
				scrollbars: true,
				status: true,
				resizable: true,
				height: 500,
				width: 1100
		});
		o_new_window_help.Observe();

		/* Create a LipmAjaxSimpleClass, for retrieving the sequences */
		O_AJAX_SIMPLE = new LipmAjaxSimpleClass( O_POPUP, O_ERROR, o_led1 );
		O_AJAX_SIMPLE.SetPopupHdFt(undefined,'<a href="javascript:O_ERROR.Close();">close</a>');

		var a_graphics_objects = $A(['ID_DYNAMIC_LEGEND',O_CONTROL_BLOCK,O_POPUP,o_marker1,o_dessus_dessous,o_marker2]);
		O_PERSONAL          = new PersonalClass(a_graphics_objects);
		
		// For each vizualization, we create a new object, and we register this object in the control block
		['NcDotplotClass','NcDisplayMapClass','NcKaryotypeClass','NcCircosClass'].each( function (name) {
				var cmd = 'new ' + name + '(a_graphics_objects)';
				var o_view = eval(cmd);
				O_CONTROL_BLOCK.RegisterView(o_view);
		});

		// Manage the full screen status
		if (O_PARAM.full_screen != undefined && O_PARAM.full_screen==1)
		{
			O_SCREEN = new LipmFullScreenClass('ID_FULL_SCREEN','HideOnFullScreen','ID_SUPERCONTAINER',true);
		}
		else
		{
			O_SCREEN = new LipmFullScreenClass('ID_FULL_SCREEN','HideOnFullScreen','ID_SUPERCONTAINER');
		};

		// Manage the ID_IMG_CONTAINER size - See also the function RegisterHook in LipmFullScreen

		// Change size when we resize the object - We prefer to use the click event on 'window', because a lot of resize events are lost
		// (specially when the resive is too fast)
		// So: 1/ resize, 2/ click, it will resize ID_IMG_CONTAINER
		Event.observe(E_IMG_CONTAINER,'click',ResizeMapsImgContainer.bindAsEventListener());

		// change size when using the FullScreen object
		O_SCREEN.RegisterHook(ResizeMapsImgContainer);

		// Change size now
		ResizeMapsImgContainer();
		
		// Everything should be OK now: display the fields
		// This is important to display/hide the buttons at top of control block
		O_CONTROL_BLOCK.UpdatePage();
	};

	//new DraggingClass(E_IMG_CONTAINER);
	
	// Manage the ID_IMG_CONTAINER size - See also the function __GetWantedHeight in LipmfullScreen
	// TODO ==> CE TRUC EST A REVOIR - PEUT-ETRE METTRE TOUT CE BORDEL DANS UN POPUP ?
	//          AU MOINS FAIRE UN OBJET POUR EVITER LA VARIABLE GLOBALE, HOULALA

	function ResizeMapsImgContainer() {
		var container_height = document.viewport.getHeight() - E_IMG_CONTAINER.viewportOffset().top;
				//alert(container_height);
				E_IMG_CONTAINER.setStyle({
						height: container_height + 'px'
				});
	};


}
	Event.observe(window,'load',Load);



/*
=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license under French law - www.cecill.info

=cut
*/

