/*
  Copyright INRA/CNRS

 Emmanuel.Courcelle@toulouse.inra.fr
 Jerome.Gouzy@toulouse.inra.fr
 Thomas.Faraut@toulouse.inra.fr

 This software is a computer program whose purpose is to provide a
 web-based interface for analyzing the different levels of genome
 conservations.

 This software is governed by the CeCILL license under French law and
 abiding by the rules of distribution of free software.  You can  use,
 modify and/ or redistribute the software under the terms of the CeCILL
 license as circulated by CEA, CNRS and INRIA at the following URL
 "http://www.cecill.info".

 As a counterpart to the access to the source code and  rights to copy,
 modify and redistribute granted by the license, users are provided only
 with a limited warranty  and the software's author,  the holder of the
 economic rights,  and the successive licensors  have only  limited
 liability.

 In this respect, the user's attention is drawn to the risks associated
 with loading,  using,  modifying and/or developing or reproducing the
 software by the user in light of its specific status of free software,
 that may mean  that it is complicated to manipulate,  and  that  also
 therefore means  that it is reserved for developers  and  experienced
 professionals having in-depth computer knowledge. Users are therefore
 encouraged to load and test the software's suitability as regards their
 requirements in conditions enabling the security of their systems and/or
 data to be ensured and,  more generally, to use and operate it in the
 same conditions as regards security.

 The fact that you are presently reading this means that you have had
 knowledge of the CeCILL license and that you accept its terms

*/
/*

=pod

=head1 NAME
 NcDisplayMap.js - Global objects used with the DisplayMap representation

=cut
*/

/*
=pod

=head1 SYNOPSIS

 NarcisseAreaClass               see narcisse_area.js
    ChromosomeCursorClass
    RepeatsClass
	RnaClass
	ExonsClass
    GenesClass
    SyntClass
    SnpClass
    HspClass
    PersonalClass

=cut
*/
/* EXPERIMENTAL !
DraggingClass = Class.create({
			initialize: function(inner_container)
			{
				this.__inner_cont = $(inner_container);
				this.__start_top  = 0;
				this.__top        = 0;
				this.__dragging   = false;

				Event.observe(this.__inner_cont,'mousedown',this.__StartMove.bindAsEventListener(this));
				Event.observe(this.__inner_cont,'mousemove',this.__Moving.bindAsEventListener(this));
				Event.observe(this.__inner_cont,'mouseup',this.__StopMove.bindAsEventListener(this));
			},

			__StartMove: function(evt)
			{
				this.__start_top = evt.clientY;
				this.__inner_cont.style.cursor = "-moz-grab";
				this.__top = this.__stripPx(this.__inner_cont.style.top);
				this.__dragging = true;
			},

			__Moving: function(evt)
			{
				if (this.__dragging)
				{
					this.__inner_cont.setStyle({top: this.__top + (evt.clientY - this.__start_top)});
				}
			},

			__StopMove: function(evt)
			{
				this.__inner_cont.style.cursor = "";
				this.__dragging   = false;
			},

			__StripPx: function(value)
			{
				if (value == "") return 0;
				return parseFloat(value.substring(0, value.length - 2));
			}
	});
*/
/*
=pod

=head2 ChromosomeCursorClass

 Title:     Chromosome Cursor class
 Usage:     var o_cursor=new ChromosomeCursorClass(legend,o_popup,o_marker,o_dessus_dessous);
 Class:     Extends NarcisseAreaClass
 Constructor Args: see NarcisseAreaClass
 Globals:   none

=cut
*/

ChromosomeCursorClass = Class.create(NarcisseAreaClass,{
	initialize: function($super,a_graphics_objects)
	{
		$super(a_graphics_objects);
		this._feature = undefined;
	},

	_GetTitle: function() {
		return 'Click here to zoom to this position';
	},

	/*
=pod

=head3 MouseOver

   Title:     MouseOver
   Usage:     through a mouseover event handler
   Procedure: Display some information in the dynamic legend
   Args: a_args:
         x1,y1,x2,y2: see NarcisseAreaClass
         pos   : position (in base pairs) of the cursor
         orga  : the organism
         chrom : the chromosome
         length: the chromosome length (in base pairs)
         start : the 1st displayed base pair
         end   : the last displayed base pair
         Globals:   none

=cut
	*/
	MouseOver: function($super,x1,y1,x2,y2,pos,orga,chrom,length,start,end)
	//MouseOver: function($super,a_args)
	{
		/*var x1 = a_args[0];
		var y1 = a_args[1];
		var x2 = a_args[2];
		var y2 = a_args[3];
		var pos    = a_args[5];
		var orga   = a_args[6];
		var chrom  = a_args[7];
		var length = a_args[8];
		var start  = a_args[9];
		var end    = a_args[10];*/

		/* We do not test the value returned by the superclass, because it is impossible to remove the chromosomes layer */
		$super(x1,y1,x2,y2);

		var dsp_len = Math.round(100 * (end-start)/length);
		var html='<h2>Reference chromosome</h2>';
		html += '<table>';
		html += '<tr><th>Organism</th><td>' + orga + '</td></tr>';
		html += '<tr><th>chrom</th><td>' + chrom + '</td></tr>';
		html += '<tr><th>length</th><td>' + PrettyDisplayPosition(length) + ' (' + dsp_len + '% displayed)</td></tr>';
		html += '<tr><th>start</th><td>' + PrettyDisplayPosition(start) + ' (' + start + ')</td></tr>';
		html += '<tr><th>end</th><td>' + PrettyDisplayPosition(end) + ' (' + end + ')</td></tr>';
		html += '<tr><th>&nbsp;</th><td>&nbsp;</td></tr>';
		html += '<tr><th>cursor </th><td>' + PrettyDisplayPosition(pos) + ' (' + pos + ')</td></tr>';
		html += '</table>';
		$(this._legend).innerHTML = html;
	},

	/*
=pod

=head3 MouseDown

   Title    : MouseDown
   Usage    : through a mousedown event handler
   Procedure: Open a popup
   Args     : x1,y1,x2,y2: Coordinates of the area
   pos      : position (in base pairs) of the cursor
   Globals  : none

=cut
	*/
	MouseDown: function($super,x1,y1,x2,y2,pos,orga,chr,length,strt,end)
	//MouseDown: function($super,a_args)
	{
		/*var x1 = a_args[0];
		var y1 = a_args[1];
		var x2 = a_args[2];
		var y2 = a_args[3];
		var pos  = a_args[5];
		var orga = a_args[6];
		var chr  = a_args[7];
		var length = a_args[8];
		var strt = a_args[9];
		var end  = a_args[10];*/
		var whole_chrom = orga + ':' + chr + ':' + 1 + ':' + length;
		var only_dsp    = orga + ':' + chr + ':' + strt + ':' + end;
		
		var whole_chrom_url = this._base_url + "&__wba_popup=1&script=SENDSEQUENCES&GenomicSeq=" + whole_chrom;
		var only_dsp_url    = this._base_url + "&__wba_popup=1&script=SENDSEQUENCES&GenomicSeq=" + only_dsp;

		var head = '<h1>Position</h1>';
		var body = '<ul>';
		body += '<li><a href="javascript:O_CONTROL_BLOCK.SubmitAjax(\'' + this.__ToCursor(pos) + '\',\'search ' + pos + '\')">Go&nbsp;to&nbsp;this&nbsp;position</a></li>';
		body += '<li><a href="javascript:O_CONTROL_BLOCK.SubmitAjax(\'' + this.__ToCursor(pos) + '&click=ZoomPos\',\'search ' + pos + ' and zoom in\')">Go&nbsp;to&nbsp;this&nbsp;position&nbsp;and&nbsp;zoom</a></li>';
		body += '</ul><p>&nbsp;</p><ul>';
		body += '<li><a class="WindowSequence" href="' + whole_chrom_url + '">Get ' + orga + ' chr ' + chr +'</a></li>';
		body += '<li><a class="WindowSequence" href="' + only_dsp_url + '">Get ' + orga + ' Displayed chr ' + chr + '</a></li>';
		body += '</ul>';
		//this._o_popup.Display(body,head,undefined,0,this._o_control_block.GetBottom()+5);
		this._o_popup.Display(body,head);
	},

	/*
 Title   : __ToCursor
 Function: Generate a link to center the graphics to this cursor
 Args    : pos: the position of the cursor
 Access  : Private
 return  : The string to be used in a command or in an url
*/
	__ToCursor: function(pos)
	{
		return 'search=1&amp;pos=' + pos;
	}
});

/*
=pod

=head2 Repeat Class

 Constructor Args:
    top_id     : The parent element of the created area
    x1,y1,x2,y2: The coordinates of the area
	orga       : The organism
	chrom      : The chromosome
    start      : The start position (in AA bases) of the repeat
    end        : The end position of the repeat
    strand     : The strand (+1/-1)
    locus      : The locus tag
	name       : The name (?)

=cut
*/

RepeatClass = Class.create(NarcisseAreaClass,{
	initialize: function($super,a_graphics_objects)
	{
		$super(a_graphics_objects);
		this._feature = 'repeat';
	},
	MouseOver: function($super,x1,y1,x2,y2,orga,chrom,start,end,strand,locus,name)
	//MouseOver: function($super,a_args)
	{
		/*var x1 = a_args[0];
		var y1 = a_args[1];
		var x2 = a_args[2];
		var y2 = a_args[3];
		var orga   = a_args[5];
		var chrom  = a_args[6];
		var start  = a_args[7];
		var end    = a_args[8];
		var strand = a_args[9];
		var locus  = a_args[10];
		var name   = a_args[11];*/

		// If the 'repeat' image is not visible, there's nothing to display
		//if ($('repeat').style.visibility != 'visible')
		//{
			//return;
		//}

		if ( $super(x1,y1,x2,y2) == false ) { return; };

		var html='<h2>repeat segment</h2>';
		var length = end-start;

		html += '<table>';
		html += '<tr><th>organism</th><td>' + orga + ' (' + chrom + ')</td></tr>';
		html += '<tr><th>locus</th><td>' + locus + '</td></tr>';
		html += '<tr><th>name</th><td>' +name + '</td></tr>';
		html += '<tr><th>start</th><td>' + PrettyDisplayPosition(start) + ' (' + start + ')</td></tr>';
		html += '<tr><th>end</th><td>' + PrettyDisplayPosition(end) + ' (' + end + ')</td></tr>';
		html += '<tr><th>length</th><td>' + PrettyDisplayPosition(length) + ' (' + length + ')</td></tr>';
		html += '<tr><th>strand</th><td>' + (strand==1?'+':'-') + '</td></tr>';
		html += '</table>';
		$(this._legend).innerHTML = html;
	}
});

/*
=pod

=head2 Rna Class

 Constructor Args:
    top_id     : The parent element of the created area
    x1,y1,x2,y2: The coordinates of the area
	orga       : The organism
	chrom      : The chromosome
    start      : The start position (in AA bases) of the repeat
    end        : The end position of the repeat
    strand     : The strand (+1/-1)
    symbol     : The symbol (?)
    name       : The name (?)

=cut
*/

RnaClass = Class.create(NarcisseAreaClass,{
		initialize: function($super,a_graphics_objects){
			$super(a_graphics_objects);
			this._feature = 'rna';
		},
		MouseOver: function($super,x1,y1,x2,y2,organism,chrom,start,end,strand,symbol,name)
		//MouseOver: function($super,a_args)
		{
			/*var x1 = a_args[0];
			var y1 = a_args[1];
			var x2 = a_args[2];
			var y2 = a_args[3];
			var organism = a_args[5];
			var chrom  = a_args[6];
			var start  = a_args[7];
			var end    = a_args[8];
			var strand = a_args[9];
			var symbol = a_args[10];
			var name   = a_args[11];*/

			if ( $super(x1,y1,x2,y2) == false ) { return; };

			var html='<h2>rna</h2>';
			var length = end-start;

			html += '<table>';
			html += '<tr><th>organism</th><td>' + organism + ' (' + chrom + ')</td></tr>';
			html += '<tr><th>symbol</th><td>' + symbol + '</td></tr>';
			html += '<tr><th>name</th><td>' + name + '</td></tr>';
			html += '<tr><th>start</th><td>' + PrettyDisplayPosition(start) + ' (' + start + ')</td></tr>';
			html += '<tr><th>end</th><td>' + PrettyDisplayPosition(end) + ' (' + end + ')</td></tr>';
			html += '<tr><th>length</th><td>' + PrettyDisplayPosition(length) + ' (' + length + ')</td></tr>';
			html += '<tr><th>strand</th><td>' + (strand==1?'+':'-') + '</td></tr>';
			html += '</table>';
			$(this._legend).innerHTML = html;
		}
});

/*
=pod

=head2 Cyto Class

 Constructor Args:
    top_id     : The parent element of the created area
    x1,y1,x2,y2: The coordinates of the area
    orga       : The organism
    chrom      : The chromosome
    start      : The start position (in AA bases)
    end        : The end position of the repeat
    element    : The element

=cut
*/

CytoClass = Class.create(NarcisseAreaClass,{
	initialize: function($super,a_graphics_objects){
		$super(a_graphics_objects);
		this._feature = 'cyto';
	},
	MouseOver: function($super,x1,y1,x2,y2,organism,chrom,start,end,element)
	//MouseOver: function($super,a_args)
	{
		/*var x1 = a_args[0];
		var y1 = a_args[1];
		var x2 = a_args[2];
		var y2 = a_args[3];
		var organism = a_args[5];
		var chrom  = a_args[6];
		var start  = a_args[7];
		var end    = a_args[8];
		var element= a_args[9];*/

		if ( $super(x1,y1,x2,y2) == false ) { return; };

		var html='<h2>cytologic data</h2>';
		var length = end-start;

		html += '<table>';
		html += '<tr><th>organism</th><td>' + organism + ' (' + chrom + ')</td></tr>';
		html += '<tr><th>element</th><td>' + element + '</td></tr>';
		html += '<tr><th>start</th><td>' + PrettyDisplayPosition(start) + ' (' + start + ')</td></tr>';
		html += '<tr><th>end</th><td>' + PrettyDisplayPosition(end) + ' (' + end + ')</td></tr>';
		html += '<tr><th>length</th><td>' + PrettyDisplayPosition(length) + ' (' + length + ')</td></tr>';
		html += '</table>';
		$(this._legend).innerHTML = html;
	}
});


/*
=pod

=head2 Exon Class

Constructor Args:
   top_id     : The parent element of the created area
   x1,y1,x2,y2: The coordinates of the area
   orga       : The organism
   chrom      : The chromosome
   start      : The start position (in AA bases) of the exon
   end        : The end position of the exon

=cut
*/

ExonClass = Class.create(NarcisseAreaClass,{
	initialize: function($super,a_graphics_objects)
	{
		$super(a_graphics_objects);
		this._feature = 'exon';
	},
	MouseOver: function($super,x1,y1,x2,y2,orga,chrom,start,end)
	//MouseOver: function($super,a_args)
	{
		/*var x1 = a_args[0];
		var y1 = a_args[1];
		var x2 = a_args[2];
		var y2 = a_args[3];
		var orga = a_args[5];
		var chrom= a_args[6];
		var start= a_args[7];
		var end  = a_args[8];*/

		if ( $super(x1,y1,x2,y2) == false ) { return; };
		var html='<h2>Exon</h2>';
		var length = end-start;

		html += '<table>';
		html += '<tr><th>organism</th><td>' + orga + ' (' + chrom + ')</td></tr>';
		html += '<tr><th>start</th><td>' + PrettyDisplayPosition(start) + ' (' + start + ')</td></tr>';
		html += '<tr><th>end</th><td>' + PrettyDisplayPosition(end) + ' (' + end + ')</td></tr>';
		html += '<tr><th>length</th><td>' + PrettyDisplayPosition(length) + ' (' + length + ')</td></tr>';
		html += '</table>';
		$(this._legend).innerHTML = html;
	}
});


/*
=pod

=head2 Snp Class

Constructor Args:
   top_id     : The parent element of the created area
   x1,y1,x2,y2: The coordinates of the area
   orga       : The organism
   chrom      : The chromosome
   start      : The start position (in AA bases) of the Snp
   end        : The end position of the Snp

=cut
*/

SnpClass = Class.create(NarcisseAreaClass,{
	initialize: function($super,a_graphics_objects)
	{
		$super(a_graphics_objects);
		this._feature = 'snp';
	},
	MouseOver: function($super,x1,y1,x2,y2, orga,chrom,pos,strand, values, src_id, mutation )
	//MouseOver: function($super, a_args)
	{
		/*var x1 = a_args[0];
		var y1 = a_args[1];
		var x2 = a_args[2];
		var y2 = a_args[3];
		var orga = a_args[5];
		var chrom= a_args[6];
		var pos  = a_args[7];
		var strand = a_args[8];
		var values = a_args[9];
		var src_id = a_args[10];
		var mutation = a_args[11];*/

		if ( $super(x1,y1,x2,y2) == false ) { return; };
		var html='<h2>Snp</h2>';
		html += '<table>';
		html += '<tr><th>organism</th><td>' + orga + ' (' + chrom + ')</td></tr>';
		html += '<tr><th>position</th><td>' + PrettyDisplayPosition( pos ) + ' (' + pos + ')</td></tr>';
		html += '<tr><th>strand</th><td>' + strand + '</td></tr>';
		html += '<tr><th>possible values</th><td>' + values;
		if( mutation != undefined )
		{
			html += ' (' + mutation + ')';
		}
		html += '</td></tr>';
		html += '<tr><th>source</th><td>' + src_id + '</td></tr>';
		html += '</table>';

		$(this._legend).innerHTML = html;
	}
});


/*
=pod

=head2 Gene Class

 Constructor Args:
    top_id     : The parent element of the created area
    x1,y1,x2,y2: The coordinates of the area
    orga       : The organism
    chrom      : The chromosome
    start      : The start position (in AA bases) of the exon
    end        : The end position of the exon
    strand     : The strand (+1/-1)
    element_id : ???
    name       : The name (?)
    symbol     : The symbol (?)
    locus_tag  : The locus tag
    protein_id : The protein id
    prbh       : ???
    accession  : ???
    xd1...yd2  : the coordinates of the linked gene, if any (prbh)
    Global     : O_CFG, O_PARAM

=cut
*/

GeneClass = Class.create(NarcisseAreaClass,{
	initialize: function($super,a_graphics_objects)
	{
		$super(a_graphics_objects);
		//alert("succeeded with gene");
		this._feature         = 'gene';
		this.__buildfam_base_url= this._base_url + '&script=DISPLAYFAM';
	},

	_GetTitle: function() {
		return 'Click here to zoom to this gene';
	},

	MouseOver: function($super,x1,y1,x2,y2,orga,chrom,start,end,strand,element_id,name,locus_tag,protein_id,prbh,accession,xd1,yd1,xd2,yd2)
	//MouseOver: function($super,a_args)
	{
		/*var x1 = a_args[0];
		var y1 = a_args[1];
		var x2 = a_args[2];
		var y2 = a_args[3];
		var orga   = a_args[5];
		var chrom  = a_args[6];
		var start  = a_args[7];
		var end    = a_args[8];
		var strand = a_args[9];
		var element_id= a_args[10];
		var name   = a_args[11];
		var locus_tag  = a_args[12];
		var protein_id = a_args[13];
		var prbh       = a_args[14];
		var accession  = a_args[15];
		var xd1 = a_args[16];
		var yd1 = a_args[17];
		var xd2 = a_args[18];
		var yd2 = a_args[19];*/

		if (name == "") {name = locus_tag};
		if ($super(x1,y1,x2,y2,xd1,yd1,xd2,yd2) == false ) { return; };

		var html='<h2>Gene</h2>';
		var length = end-start;

		html += '<table>';
		html += '<tr><th>name</th><td>' + name + '</td></tr>';
		html += '<tr><th>organism</th><td>' + orga + ' (' + chrom + ')</td></tr>';
		html += '<tr><th>start</th><td>' + PrettyDisplayPosition(start) + ' (' + start + ')</td></tr>';
		html += '<tr><th>end</th><td>' + PrettyDisplayPosition(end) + ' (' + end + ')</td></tr>';
		html += '<tr><th>length</th><td>' + PrettyDisplayPosition(length) + ' (' + length + ')</td></tr>';
		html += '<tr><th>strand</th><td>' + (strand==1?'+':'-') + '</td></tr>';
		html += '<tr><th>locus tag</th><td>' + locus_tag + '</td></tr>';
		html += '<tr><th>protein_id</th><td>' + protein_id + '</td></tr>';
		html += '<tr><th>accession</th><td>' + accession + '</td></tr>';
		html += '</table>';
		$(this._legend).innerHTML = html;
	},
	MouseDown: function($super,x1,y1,x2,y2,orga,chrom,start,end,strand,element_id,name,locus_tag,protein_id,prbh,accession)
	//MouseDown: function($super,a_args)
	{
		/*var x1 = a_args[0];
		var y1 = a_args[1];
		var x2 = a_args[2];
		var y2 = a_args[3];
		var orga   = a_args[5];
		var chrom  = a_args[6];
		var start  = a_args[7];
		var end    = a_args[8];
		var strand = a_args[9];
		var element_id= a_args[10];
		var name   = a_args[11];
		var locus_tag  = a_args[12];
		var protein_id = a_args[13];
		var prbh       = a_args[14];
		var accession  = a_args[15];*/

		if ( $super() == false ) {return;};
		if (name == "") {name = locus_tag};

		var head;
		if (name == 'none')
		{
			head='<h1>G E N E</h1>';
		}
		else
		{
			head = '<h1>G E N E<br />' + name + ' (' + orga + ') </h1>';
		}

		var body =  '<h2>VIEW</h2>';
		body += '<ul>';
		body += '<li>Zoom&nbsp;to&nbsp;<a href="'+this.__ToGeneLevel(orga,chrom,name)+'">gene&nbsp;level</a></li>';
		body += '</ul>';
		body += '<h2>EXTRACT</h2>';
		body += '<ul>';
		body += this.__ExternalLinks(orga,chrom,start,end,strand,element_id,name,locus_tag,protein_id,prbh,accession);
		body += this.__ExtractSeq(orga,chrom,start,end,strand,element_id,name,locus_tag,protein_id,prbh,accession);
		body += this.__BuildFamilies(orga,chrom,start,end,strand,element_id,name,locus_tag,protein_id,prbh,accession);
// 		body += this.__testPrimerDesign( orga, chrom, start, end );
		body += '</ul>';

		//this._o_popup.Open(x2+5,y2+5,head,body);
		//this._o_popup.Open(0,this._o_control_block.GetBottom()+5,head,body);
		//this._o_popup.Display(body,head,undefined,0,this._o_control_block.GetBottom()+5);
		this._o_popup.Display(body,head);
	},

	/*
	Title:    __ToGeneLevel
	Function: Generate a link to go to gene level, centered to this gene
	Args:     None
	Access:   Private
	return:   The string to be used in a command or in an url
	*/
	__ToGeneLevel: function(orga,chrom,name)
	{
		// exchange (reference, targetN) organisms if necessary
		var N;
		if (orga != O_PARAM.reference)
		{
			N = FindValue(orga).gsub('target','');
		}
		else
		{
			N = 0;
		};
		var url='search=1&gene='+name;
		var history = 'search to ' + orga + ' ' + chrom + ' ' + name;
		var rvl = 'javascript:O_CONTROL_BLOCK.SubmitExchange(' + N + ',\'' + chrom + '\',\'' + url + '\',\'' + history + '\')';
		return rvl;
	},

	/*
	Title:    __ExternalLinks
	Prerequiste : use url.txt
	Function: Using the globals:
	            -O_CFG.GENOME[orga].URL if present, or O_CFG.URL, generate a list of external links
	          URL is a hash of strings formatted as follows:
	              text target_url flag
	          the links's url is formed target_url, using the flag (cf. code)
	Args:     none
	Access:   Private
	return:   The html code
	*/

	__ExternalLinks: function(orga,chrom,start,end,strand,element_id,name,locus_tag,protein_id,prbh,accession)
	{
		var html         = '';
		var nc_cgi       = new RegExp("(~NARCISSE_CGI~)");
		var regex        = new RegExp("(~NARCISSE_ID~)");
		var iccare_regex = new RegExp("(~ICCARE_ORG~)");

		var new_win_class = 'NewWindowBlank';

		var a_url;
		if (O_CFG.GENOME[orga].URL != undefined)
		{
			a_url = $A(O_CFG.GENOME[orga].URL);
		}
		else
		{
			if (O_CFG.URL != undefined)
			{
				a_url = $A(O_CFG.URL);
			}
			else
			{
				return '';
			}
		}

		a_url.each( function(i)
		{
			var text       = i.text;
			var target_url = i.url;
			var flag       = i.key;

			switch( flag )
			{
				case "cds" :
					target_url = target_url.replace(regex,name);
					break;
				case "prot":
				case "protein":
					target_url = target_url.replace(regex,protein_id);
					break;
				case "gene" :
					target_url = target_url.replace(regex,accession);
					break;
				case "locus_tag" :
					target_url = target_url.replace(regex,locus_tag);
					break;
				case "element_id" :
					target_url = target_url.replace(regex,element_id);
					break;
				case "apollo" :
					var appendice = chrom;
					var margin = parseInt( O_CFG.APOLLO_MARGIN );
					var apollo_start = parseInt( start ) - margin
					var apollo_end   = parseInt( end ) + margin
					appendice  += '&chromosome_start_apollo=' + apollo_start;
					appendice  += '&chromosome_end_apollo=' + apollo_end;
					target_url += appendice;
					break;
				case "gbrowse" :
					appendice  = start + '..' + end;
					target_url += appendice;
					new_win_class = 'BigWindow';	//pour un pop-up
					break;
				case "knowledge" :
					target_url    = target_url.replace( nc_cgi, O_CFG.HTTP_NARCISSE_CGI);

					var appendice = '&range=' + start + '_' + end
					appendice    += '&chrom=' + chrom;
					appendice    += '&locus_tag=' + locus_tag;
					appendice    += '&__wb_session=' + O_PARAM.__wb_session;
					appendice    += '&__wba_popup=1';

					// en attendant l'uniformisation des noms
					appendice  += '&species=' + orga;
					target_url += appendice;
					new_win_class = 'BigWindow';	//pour un pop-up
					break;
				default:
					break;
			}

			if (iccare_regex.test(target_url))
			{   
				// supprime a cause du compilateur
				// TODO - Eclaircir ce point
				//target_url = target_url.replace(iccare_regex,O_CFG.GENOME[orga].long);
				target_url = target_url.replace(iccare_regex,orga);
			}

			if( -1 == target_url.indexOf( 'javascript:' ) )
				html += "<li><a class='" + new_win_class + "' ";
			else
				html += "<li><a ";

			html += 'href="' + target_url + '">' + text + '</a></li>';
		});
		return html;
	},

	/*
	Title:    __ExtractSeq
	Function: Generate the links for the sequence extraction
	Args:     none
	Access:   Private
	return:   The html code
	*/
	__ExtractSeq: function(orga,chrom,start,end,strand,element_id,name,locus_tag,protein_id,prbh,accession)
	{
		var html='';
		var url = this._base_url + "&__wba_popup=1&script=SENDSEQUENCES&species="+orga;
		var url_gene = url + "&GeneSeq="   + locus_tag;
		var url_prot = url + "&ProteinSeq="+ locus_tag;
		var url_cds  = url + "&CdsSeq="    + locus_tag;
		html  = '<li>Get&nbsp;';
		//html += '<a href="javascript:O_AJAX_SIMPLE.SetPopupHdFt(\'Gene sequence\');O_AJAX_SIMPLE.SubmitAjax(\''  + url_gene + '\')">gene</a>';
		html += '<a class="WindowSequence" href="'+url_gene+'">gene</a>';
		html += '/<a class="WindowSequence" href="'+url_cds+'">cds</a>';
		html += '/<a class="WindowSequence" href="'+url_prot+'">prot</a>';

		//html += '/<a href="javascript:O_AJAX_SIMPLE.SetPopupHdFt(\'CDS sequence\');O_AJAX_SIMPLE.SubmitAjax(\'' + url_cds  + '\')">cds</a>';
		//html += '/<a href="javascript:O_AJAX_SIMPLE.SetPopupHdFt(\'Proteic sequence\');O_AJAX_SIMPLE.SubmitAjax(\'' + url_prot + '\')">prot</a></li>';
		return html;
	},

	/*
	Title    : __testPrimerDesign
	Function :
	Args     : orga, string, the organisme name
	           chrom,
	           start,
	           end
	Access:   Private
	return:   The html code <---- ??? TODO
	*/
	__testPrimerDesign: function( orga, chrom, start, end )
	{
		var args = "&__wb_function=TestPrimerDesign&species="+orga;
		args += "&chrom=" + chrom;
		args += "&start=" + start + "&end=" + end;
		var url = this._base_url  + args;
 		var html = '<a class=\'NewWindowBlank\' href="'+url+'">PrimerDesign</a>&nbsp;';

		return html;
	},

	/*
	Title:    __BuildFamilies
	Function: Generate the links for the build family
	Args:     none
	Access:   Private
	return:   The html code
	*/
	__BuildFamilies: function(orga,chrom,start,end,strand,element_id,name,locus_tag,protein_id,prbh,accession)
	{
		var html='';
		var url = this.__buildfam_base_url + '&species=' + orga;
		var url_50 = url + '&ProteinFam=' + locus_tag + '&Cutoff=50';
		var url_80 = url + '&ProteinFam=' + locus_tag + '&Cutoff=80';
		html += '<li>Display&nbsp;protein&nbsp;family&nbsp;<a  class=\'NewWindowBlank\' href="'+url_50+'">50</a>&nbsp;';
		html += '<a class=\'NewWindowBlank\' href="'+url_80+'">80</a>&nbsp;';
		html += '%span</li>';
		return html;
	}
});

/*
=pod

=head2 Synteny Class

 Constructor Args:
    top_id     : The parent element of the created area
    x1,y1,x2,y2: The coordinates of the area
            ...: A lot of parameters

=cut
*/

SyntClass = Class.create(NarcisseAreaClass,{
	initialize: function($super,a_graphics_objects)
	{
		$super(a_graphics_objects);
		this._feature          = 'synt';
	},

	_GetTitle: function() {
		return 'Click here to zoom to this synteny';
	},

	MouseOver: function($super,x1,y1,x2,y2,reference,chrom,start,end,order,target,s_chrom,s_start,s_end,s_order,prog,xd1,yd1,xd2,yd2)
	//MouseOver: function($super,a_args)
	{
		/*var x1 = a_args[0];
		var y1 = a_args[1];
		var x2 = a_args[2];
		var y2 = a_args[3];
		var reference = a_args[5];
		var chrom     = a_args[6];
		var start     = a_args[7];
		var end       = a_args[8];
		var order     = a_args[9];
		var target    = a_args[10];
		var s_chrom   = a_args[11];
		var s_start   = a_args[12];
		var s_end     = a_args[13];
		var s_order   = a_args[14];
		var prog      = a_args[15];
		var xd1 = a_args[16];
		var yd1 = a_args[17];
		var xd2 = a_args[18];
		var yd2 = a_args[19];*/

		if ( $super(x1,y1,x2,y2,xd1,yd1,xd2,yd2) == false ) { return; };
		//var html='<h2>Synteny element ' + order + '</h2>';
		var html='<h2>Synteny element</h2>';
		var length = end-start;
		var s_length = s_end-s_start;
		html += '<table>';
		html += '<tr><th>organism</th><td>' + reference + ' (' + chrom + ')</td></tr>';
		html += '<tr><th>from</th><td>' + PrettyDisplayPosition(start) + ' (' + start + ')</td></tr>';
		html += '<tr><th>to</th><td>' + PrettyDisplayPosition(end) + ' (' + end + ')</td></tr>';
		html += '<tr><th>length</th><td>' + PrettyDisplayPosition(length) + ' (' + length + ')</td></tr>';
		html += '</table><h2>vs</h2><table>';
		html += '<tr><th>organism</th><td>' + target + ' (' + s_chrom + ')</td></tr>';
		html += '<tr><th>from</th><td>' + PrettyDisplayPosition(s_start) + ' (' + s_start + ')</td></tr>';
		html += '<tr><th>to</th><td>' + PrettyDisplayPosition(s_end) + ' (' + s_end + ')</td></tr>';
		html += '<tr><th>length</th><td>' + PrettyDisplayPosition(s_length) + ' (' + s_length + ')</td></tr>';
		html += '</table>';
		$(this._legend).innerHTML = html;
	},
	MouseDown: function($super,x1,y1,x2,y2,reference,chrom,start,end,order,target,s_chrom,s_start,s_end,s_order,prog)
	//MouseDown: function($super,a_args)
	{
		/*var x1 = a_args[0];
		var y1 = a_args[1];
		var x2 = a_args[2];
		var y2 = a_args[3];
		var reference = a_args[5];
		var chrom     = a_args[6];
		var start     = a_args[7];
		var end       = a_args[8];
		var order     = a_args[9];
		var target    = a_args[10];
		var s_chrom   = a_args[11];
		var s_start   = a_args[12];
		var s_end     = a_args[13];
		var s_order   = a_args[14];
		var prog      = a_args[15];*/

		if ( $super() == false ) {return;};

		var head='<h1>S Y N T E N Y ';
		var length = end-start;
		head += '(' + PrettyDisplayPosition(length) + ')<br />';
		head += reference + '(' + chrom + ') ' + PrettyDisplayPosition(start) + '<br />';
		head += target + '(' + s_chrom + ') ' + PrettyDisplayPosition(s_start);
		head += '</h1>';

		var body = '<h2>VIEW</h2>';
		body += '<ul>';
		body += '<li><a href="'+this.__ToSyntenyLevel(reference,target,chrom,order)+'">Zoom&nbsp;to&nbsp;this&nbsp;synteny</a></li>';
		body += '<li><a href="'+this.__CenterSynteny(reference,target,chrom,order)+'">Center&nbsp;this&nbsp;synteny</a></li>';
		body += '</ul>';
		
		body += '<h2>ALIGN</h2>';
		body += '<ul>' + this.__OnlineCompare(reference,start,end,target,s_start,s_end) + '</ul>';
		
		body += '<h2>EXTRACT</h2>';
		body += '<ul>';
		body += this.__ExtractGenomicSeq(reference,chrom,start,end,target,s_chrom,s_start,s_end);
		body += this.__ExtractGenes(reference,chrom,start,end,target,s_chrom,s_start,s_end);
		body += this.__ExternalLinks( reference, chrom, start, end );
		//orga,chrom,start,end,strand,element_id,name,locus_tag,protein_id,prbh,accession);
		body += '</ul>';
		this._o_popup.Display(body,head);
	},

	/*
	Title:    __Onlinecompare
	Function: Generate the links for the online genomic comparison
	Args:     a lot
	Access:   Private
	return:   The html code
	*/
	__OnlineCompare: function(reference,start,end,target,s_start,s_end)
	{
		// We have to test if 'target' organism is the correct reference (in ParamParser), and then use it's start/end
		if (O_PARAM['reference'] == target)
		{
			var tmp_start = start;
			var tmp_end	  = end;

			start = s_start;
			end   = s_end;
			s_start = tmp_start;
			s_end   = tmp_end;
		}

		//var url = 'OnlineCompare=' + start + ':' + end + ':' + s_start + ':' + s_end;
		var url = 'OnlineCompare=' + start + ':' + end ;		
		url += '&ck_hsp=1';


		// Test the max range allowed by CFG['GLINT_MAX_LENGTH']
		var max_length = this.__GetValidePosition(O_CFG['GLINT_MAX_LENGTH']);
		if ( parseInt(end)-parseInt(start) <= max_length )
		{
			//var html = '<li><a href="javascript:O_CONTROL_BLOCK.SubmitAjax(\'' + url  + '\',\'online compare\')">Online compare</a></li>';
			var html = '<li><a href="javascript:O_CONTROL_BLOCK.SubmitAjax(\'' + url  + '\')">Local align</a></li>';
			return html;
		}
		else
		{
			var msg = 'You can t comput a local alignment for a synteny longer than ' + O_CFG['GLINT_MAX_LENGTH'];
			var html = '<li><a href="javascript:alert(\'' + msg  + '\');">Local align</a></li>';
			return html;
		}
	},

	/*
	Title:    __ExtractGenomicSeq
	Function: Generate the links for the genomic sequence extraction
	Args:     a lot
	Access:   Private
	return:   The html code
	*/
	__ExtractGenomicSeq: function(reference,chrom,start,end,target,s_chrom,s_start,s_end)
	{
		var html='';
		var region_chr    = target + ':' + s_chrom + ':1:0:';
		var region = reference + ':' + chrom + ':' + start + ':' + end;
		var region2= region + ':' + target + ':' + s_chrom + ':' + s_start + ':' + s_end;

		var url = this._base_url + "&__wba_popup=1&script=SENDSEQUENCES";
		var url0= url + "&GenomicSeq=" + region_chr;
		var url1= url + "&GenomicSeq=" + region;
		var url2= url + "&GenomicSeq=" + region2;
		html  = '<li><a class="WindowSequence" href="' + url0 + '">Get chromosome ' + s_chrom + ' sequence ' + target + '</a></li>';
		html += '<li><a class="WindowSequence" href="' + url1 + '">Get synteny sequence ' + reference + ' side</a></li>';
		html += '<li><a class="WindowSequence" href="' + url2 + '">Get synteny sequence both sides</a></li>';
		return html;
	},

	/*
	Title:    __ExtractGenes
	Function: Generate the links for the gene or prot extraction
	Args:     a lot
	Access:   Private
	return:   The html code
	*/
	__ExtractGenes: function(reference,chrom,start,end,target,s_chrom,s_start,s_end)
	{
		var html='';
		var region = reference + ':' + chrom + ':' + start + ':' + end;
		region += ':' + target + ':' + s_chrom + ':' + s_start + ':' + s_end;

		var url = this._base_url + "&__wba_popup=1&script=SENDSEQUENCES";
		var url_gene = url + "&GeneSeq="   + region;
		var url_prot = url + "&ProteinSeq="+ region;
		html  = '<li>Get all ';
		html += '<a class="WindowSequence" href="' + url_gene + '">genes</a>';
		html += '/<a class="WindowSequence" href="' + url_prot + '">proteins</a></li>';
		return html;
	},

	/*
	Title:    __ToSyntenyLevel
	Function: Generate a link to go to synteny level, centered to this synteny
	Args:     orga,target,chrom,order
	Access:   Private
	return:   The string to be used in a command or in an url
	*/
	__ToSyntenyLevel: function(orga,target,chrom,order)
	{
		return this.__CenterZoomSynteny(orga,target,chrom,order,true);
	},

	/*
	Title:    __CenterSynteny
	Function: Generate a link to center the graphics to this synteny
	Args:     orga,target,chrom,order
	Access:   Private
	return:   The string to be used in a command or in an url
	*/
	__CenterSynteny: function(orga,target,chrom,order)
	{
		return this.__CenterZoomSynteny(orga,target,chrom,order,false);
	},
	/*
	Title:    __CenterZoomSynteny
	Function: This code is common to __CenterSynteny and __ToSyntenyLevel
	          avoiding copy-paste !
	Args:     orga,target,chrom,order,IsSyntenyLevel
	Access:   Private
	          Used by __ToSyntenyLevel and __ToCenterSynteny
	return:   a javascript: URL
	*/
	__CenterZoomSynteny: function(orga,target,chrom,order,IsSyntenyLevel)
	{
		// exchange (reference, targetN) organisms if necessary
		var N;
		var cs = 'center_synt';
		if (orga != O_PARAM.reference)
		{
			// n = X if orga is targetX (X=1,2,...)
			var n = FindValue(orga).gsub('target','');
			N = n;
			cs += n;
		}
		else
		{
			// n=X if orga is targetX (X=1,2,...)
			var n = FindValue(target).gsub('target','');
			N = 0;
			cs += n;
		};
		var url = cs+'='+order;
		var history;
		if (IsSyntenyLevel==true)
		{
			url     += '&click=zoom_level_synteny';
			history  = 'zoom to synteny';
		}
		else
		{
			history = 'center synteny';
		}
		var rvl = 'javascript:O_CONTROL_BLOCK.SubmitExchange(' + N + ',\'' + chrom + '\',\'' + url + '\',\'' + history + '\')';
		return rvl;
	},

	/*
	Title:    __ExternalLinks
	Prerequiste : use synt_url.txt
	Function: Using the global O_CFG.GENOME[orga].SYNT_URL, generate a list of external links
	          SYNT_URL is a hash of strings formatted as follows:
	              text target_url flag
	          the links's url is formed target_url, using the flag (cf. code)
	Args:     orga, string, species name
	          chrom, string, chrom label
	          start, end, int, start-end position of the synteny
	Access:   Private
	return:   The html code
	*/
	__ExternalLinks: function( orga, chrom, start, end )
// 	 strand,element_id,name,locus_tag,protein_id,prbh,accession)
	{
		var html         = '';
		var nc_cgi       = new RegExp("(~NARCISSE_CGI~)");

		var new_win_class = 'NewWindowBlank';

		var a_url = $A(O_CFG.GENOME[orga].SYNT_URL);
		a_url.each(function(i)
		{
			var text       = i.text;
			var target_url = i.url;
			var flag       = i.key;
			var new_win_class = "";
			switch( flag )
			{
				case "knowledge" :
					target_url    = target_url.replace( nc_cgi, O_CFG.HTTP_NARCISSE_CGI);

					var appendice = '&range=' + start + '_' + end
					appendice    += '&chrom=' + chrom;
					appendice    += '&__wb_session=' + O_PARAM.__wb_session;
					appendice    += '&__wba_popup=1';

					// en attendant l'uniformisation des noms
					appendice  += '&species=' + orga;
					target_url += appendice;
					new_win_class = 'BigWindow';	//pour un pop-up
					break;

				case "apollo" :
					appendice = chrom;
					var margin = parseInt( O_CFG.APOLLO_MARGIN );
					var apollo_start = parseInt( start ) - margin
					var apollo_end   = parseInt( end ) + margin
					appendice  += '&chromosome_start_apollo=' + apollo_start;
					appendice  += '&chromosome_end_apollo=' + apollo_end;
					target_url += appendice;
					break;

				case "gbrowse" :
					appendice  = start + '..' + end;
					target_url += appendice;
					new_win_class = 'BigWindow';	//pour un pop-up
					break;

				default:
					break;
			}

			if( -1 == target_url.indexOf( 'javascript:' ) )
				html += "<li><a class='" + new_win_class + "' ";
			else
				html += "<li><a ";

			html += 'href="' + target_url + '">' + text + '</a></li>';
		}); //each

		return html;
	},


	/*
	Title:    __GetTargetProgram
	Function: Return the target program which is use for the target.Packages
			  If reference not equal to O_PArAM[reference], this function return 'reference' value program
	Args:     None
	Access:   Private
	return:   target_programN value (glint or prbh)
	*/
	/*
	__GetTargetProgram: function(reference,target)
	{
		// If no target_program found, return the default program value
		var result = O_PARAM['program'];

		// Test if reference and target are revert
		var test_orga = target;
		if (reference != O_PARAM['reference'] && target == O_PARAM['reference'])
			test_orga = reference;

		// Try to return target_programN value
		var count = 1;
		while (count <= O_CFG['DISPLAY_MAP_MAX_ORG'])
		{
			if (test_orga == O_PARAM['target'+count])
			{
				if (O_PARAM['program_target'+count] != undefined)
					result = O_PARAM['program_target'+count];
				break;
			}
			count++;
		}


		return result;
	},
	*/
	__GetValidePosition: function(value)
	{
		// Test if value is defined
		if (value == undefined)
			return value;

		// Test the value with a regex :
		//   1) get the integer value
		//   2) get the units
		var match = value.match(/^(\d+)(Mb|mb|Kb|kb)?$/);

		// If this value is correctly set, return reformated value
		if ( match != null )
		{
			var integer = match[1];
			var units   = match[2];

			if (units == 'Mb' || units == 'mb')
				units = Math.pow(10,6);
			else if (units == 'Kb' || units == 'kb')
				units = Math.pow(10,3);
			else
				units = 1;

			return integer*units;
		}

		// Else return undefined.
		else
			return undefined;
	}
});


/*
=pod

=head2 Hsp Class

Constructor Args:
    top_id     : The parent element of the created area
    x1,y1,x2,y2: The coordinates of the area
	reference  : The reference organism
	chrom      : The reference chromosome
    ref_start  : The start of hsp, reference side
    ref_end    : The end of hsp, reference side
    target     : The target organism
    tgt_chrom  : The target chromosome
    tgt_start  : The start of hsp, target side
    tgt_end    : The end of hsp, target side
    score      : The score of the match
    identity   : The % identity of the match
    strand     : The strand (+/-)
    start      : The start position (in AA bases) of the repeat
    end        : The end position of the repeat
    ...         A lot of parameters

=cut
*/

HspClass = Class.create(NarcisseAreaClass,{
	initialize: function($super,a_graphics_objects)
	{
		$super(a_graphics_objects);
		this._feature          = 'hsp';
	},
	_GetTitle: function() {
		return 'Click here to zoom to this match';
	},

	MouseOver: function($super,x1,y1,x2,y2,reference,ref_chrom,ref_start,ref_end,target,tgt_chrom,tgt_start,tgt_end,score,identity,strand,file)
	//MouseOver: function($super,a_args)
	{
		/*var x1 = a_args[0];
		var y1 = a_args[1];
		var x2 = a_args[2];
		var y2 = a_args[3];
		var reference = a_args[5];
		var ref_chrom = a_args[6];
		var ref_start = a_args[7];
		var ref_end   = a_args[8];
		var target    = a_args[9];
		var tgt_chrom = a_args[10];
		var tgt_start = a_args[11];
		var tgt_end   = a_args[12];
		var score     = a_args[13];
		var identity  = a_args[14];
		var strand    = a_args[15];
		var file      = a_args[16];*/

		if ( $super(x1,y1,x2,y2) == false ) { return; };

		var html='<h2>Match</h2>';
		var ref_length = ref_end-ref_start;
		var tgt_length = tgt_end-tgt_start;
		html += '<table>';
		html += '<tr><th>score</th><td>' + score + '  ( ' + identity + '% identity )</td></tr>';
		html += '<tr><th>strand</th><td>' + strand + '</td></tr>';
		html += '</table><br/><table>';
		html += '<tr><th>organism</th><td>' + reference + '</td></tr>';
		html += '<tr><th>from</th><td>' + PrettyDisplayPosition(ref_start) + ' (' + ref_start + ')</td></tr>';
		html += '<tr><th>to</th><td>' + PrettyDisplayPosition(ref_end) + ' (' + ref_end + ')</td></tr>';
		html += '<tr><th>length</th><td>' + PrettyDisplayPosition(ref_length) + ' (' + ref_length + ')</td></tr>';
		html += '</table><h2>vs</h2><table>';
		html += '<tr><th>organism</th><td>' + target + '</td></tr>';
		html += '<tr><th>from</th><td>' + PrettyDisplayPosition(tgt_start) + ' (' + tgt_start + ')</td></tr>';
		html += '<tr><th>to</th><td>' + PrettyDisplayPosition(tgt_end) + ' (' + tgt_end + ')</td></tr>';
		html += '<tr><th>length</th><td>' + PrettyDisplayPosition(tgt_length) + ' (' + tgt_length + ')</td></tr>';
		html += '</table>';
		$(this._legend).innerHTML = html;
	},
	MouseDown: function($super,x1,y1,x2,y2,reference,ref_chrom,ref_start,ref_end,target,tgt_chrom,tgt_start,tgt_end,score,identity,strand,file)
	//MouseDown: function($super,a_args)
	{
		/*var x1 = a_args[0];
		var y1 = a_args[1];
		var x2 = a_args[2];
		var y2 = a_args[3];
		var reference = a_args[5];
		var ref_chrom = a_args[6];
		var ref_start = a_args[7];
		var ref_end   = a_args[8];
		var target    = a_args[9];
		var tgt_chrom = a_args[10];
		var tgt_start = a_args[11];
		var tgt_end   = a_args[12];
		var score     = a_args[13];
		var identity  = a_args[14];
		var strand    = a_args[15];
		var file      = a_args[16];*/
		if ( $super() == false ) {return;};

		var region =  reference + ':' + ref_chrom + ':' + ref_start + ':' + ref_end;
		region    += ':' + target + ':' + tgt_chrom + ':' + tgt_start + ':' + tgt_end;
		region	  += ':' + file;

		var url = this._base_url + '&__wba_popup=1&script=SENDSEQUENCES&ViewAlign='+region;

		O_OPENWIN_ALIGNMENT.Open(url);
		// Open the Ajax pop-up and print the alignment
		// O_AJAX_SIMPLE.SetPopupHdFt('<h1>View the alignment</h1>');
		// O_AJAX_SIMPLE.SubmitAjax(url);
	}
});

/*
	Title:    FindValue
	Usage:    var key = FindValue(value)
	Function: search value in the global O_PARAM and return the corresponding key
	Args:     value
	Return:   the key
	Error  :  throw an exception if the value is not found
	Global :  O_PARAM
	TODO:     Soit la supprimer soit la generaliser

*/
function FindValue(value)
{
	var key;
	for (key in O_PARAM)
	{
		if (O_PARAM[key]==value){return key;}
	}
	throw("ERROR - Could not find value " + value + " in O_PARAM");
};

/*
	Title:     IsScrollable
	Usage:     bool = IsScrollable()
	Prerequiste : none
	Function:  Test if the Narcisse representation is 'scrollable', in fact if the representation range is included on maximum range.
	Args:      none
	Return:    boolean
	Global :   O_PARAM

*/

/*
=pod

=head2 NcDisplayMapClass

 NcdisplayMapClass derives from NcViewClass
 Private fields are initialized, virtual functions are redefined

 Constructor Args:
    a_graphics_objects : A lot of graphics objects
	TODO - Remplacer ca par un objet !

=cut
*/
NcDisplayMapClass = Class.create(NcViewClass,{
		initialize: function($super,a_graphics_objects){
			$super(a_graphics_objects);
			this._script  = 'DISPLAY';
			this._to_hide = ['ID_MENU_GLOBAL'];
			this._to_show = ['ID_MENU_POSITION','ID_MENU_FEATURES','ID_MENU_DATA','ID_MENU_TAB_POPUP','ID_VLVL','ID_OVERVIEW'];

			// faire une boucle ?
			O_CHROMOSOME_CURSOR = new ChromosomeCursorClass(a_graphics_objects);
			O_REPEAT            = new RepeatClass(a_graphics_objects);
			O_RNA               = new RnaClass(a_graphics_objects);
			O_GENE              = new GeneClass(a_graphics_objects);
			O_EXON              = new ExonClass(a_graphics_objects);
			O_PERSONAL          = new PersonalClass(a_graphics_objects);
			O_SYNT         		= new SyntClass(a_graphics_objects);
			O_SNP				= new SnpClass( a_graphics_objects );
			O_CYTO              = new CytoClass(a_graphics_objects);
			O_HSP				= new HspClass(a_graphics_objects);
		},

		/*
=pod

=head2 UpdateTitle

Title: UpdateTitle
Usage: obj.UpdateTitle()
Procedure: Updates the title table, with the names of reference, target1, target2, ...
Args:     none
Access:   Public
Global: O_CFG, O_PARAM

=cut
*/
		UpdateTitle: function()
		{
			var a_columns = $A();
			var col_tgt_strt = "<td class=\"DisplayMapTitleSubjt\" style=\"width:" + O_CFG['SYNT_MAP_WIDTH'] + "px\">";
			var col_ref_strt = "<td class=\"DisplayMapTitleQuery\" style=\"width:" + O_CFG['SYNT_MAP_WIDTH'] + "px\">";
			var col_end      = "</td>";

			//-------------------------------
			// Store target organisms
			var i = 1;
			while ( ValidSpeciesParameter('target'+i))
			{
				var tgt_name = O_PARAM['target'+i];
				var tgt_prog = O_PARAM['program_target'+i];
				if (tgt_prog == undefined) { tgt_prog = O_PARAM['program']; }

				tgt_title = '<em>'+tgt_name+'</em><br /><small>method:'+tgt_prog+'</small>';
				tgt_title = col_tgt_strt + tgt_title + col_end;
				a_columns.push(tgt_title);

				i++;
			}

			// If there is no target, we add a column to manage center reference position
			if (a_columns.length == 0)
			{
				a_columns.push(col_tgt_strt+'&nbsp</td>');
			}

			//-------------------------------
			// Get the reference title
			if ( O_PARAM.reference != undefined )
			{
				ref_title = '<em>'+O_PARAM.reference+'</em><br /><small>chromosome:'+O_PARAM.chroms_reference+'</small>';
				ref_title = col_ref_strt + ref_title + col_end;
			}
			else
			{
				ref_title = col_ref_strt + '&nbsp;' + col_end;
			};

			//-------------------------------
			// Test reference position value and construct correct ordered list.
			var ref_position = O_PARAM.ref_position;
			if ( ref_position == 'left' )
			{
				a_columns.unshift(ref_title);
			}
			else if ( ref_position == 'right' )
			{
				a_columns.push(ref_title);
			}
			else
			{
				var a_left  = new Array();
				var a_right = new Array();
				var a_center= new Array(ref_title);

				// 0) We sort organism, setting odd targets at left and pair at right (don't forget that array index start to 0 => odd index equal to pair target)
				for (var i=0; i<a_columns.length; i++)
				{
					if (i%2 == 0) { a_left.push(a_columns[i]); }
					if (i%2 != 0) { a_right.push(a_columns[i]); }
				}

				// 1) Reverse odd list to set first target near the reference
				a_left.reverse();

				// 2) Add reference title
				a_columns = a_left.concat(a_center);

				// 3) Add right array if it's fill
				if (a_right.length != 0) { a_columns = a_columns.concat( a_right ); }
			};


			//-------------------------------
			// Build the html code for the table
			var tab_width = a_columns.length * O_CFG['SYNT_MAP_WIDTH'];
			var html = "<table  style=\"width:" + tab_width + "px\" class=\"DisplayMapTitle\"><tr>";
			for (var i=0; i<a_columns.length; i++)
			{
				html += a_columns[i];
			}
			html += "</table>";

			this._title_div.innerHTML = html;
			//-------------------------------

		}
});

MAIN:
{
	/* Global variables, they will be initialized later */
	O_CHROMOSOME_CURSOR=0;
	O_REPEAT=0;
	O_RNA=0;
	O_MARKERS=0;
	O_GENE=0;
	O_EXON=0;
	O_SYNT=0;
	O_SNP=0;
	O_HSP=0;
	O_CYTO=0;
	O_OPENWIN_ALIGNMENT=0;
}

/*
=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license under French law - www.cecill.info

=cut
*/

