/* 
  Copyright INRA/CNRS

 Emmanuel.Courcelle@toulouse.inra.fr
 Jerome.Gouzy@toulouse.inra.fr
 Thomas.Faraut@toulouse.inra.fr

 This software is a computer program whose purpose is to provide a
 web-based interface for analyzing the different levels of genome
 conservations.

 This software is governed by the CeCILL license under French law and
 abiding by the rules of distribution of free software.  You can  use,
 modify and/ or redistribute the software under the terms of the CeCILL
 license as circulated by CEA, CNRS and INRIA at the following URL
 "http://www.cecill.info".

 As a counterpart to the access to the source code and  rights to copy,
 modify and redistribute granted by the license, users are provided only
 with a limited warranty  and the software's author,  the holder of the
 economic rights,  and the successive licensors  have only  limited
 liability.

 In this respect, the user's attention is drawn to the risks associated
 with loading,  using,  modifying and/or developing or reproducing the
 software by the user in light of its specific status of free software,
 that may mean  that it is complicated to manipulate,  and  that  also
 therefore means  that it is reserved for developers  and  experienced
 professionals having in-depth computer knowledge. Users are therefore
 encouraged to load and test the software's suitability as regards their
 requirements in conditions enabling the security of their systems and/or
 data to be ensured and,  more generally, to use and operate it in the
 same conditions as regards security.

 The fact that you are presently reading this means that you have had
 knowledge of the CeCILL license and that you accept its terms
 
*/
/* 

=pod

=head1 NAME
 NcDotplot.js - Global objects for NcDotplot.pm
 
=cut
*/


/*
=pod

=head1 SYNOPSIS

 NarcisseAreaClass               see narcisse_area.js
    DotplotChromosomeArmClass
 
=cut
*/

/*           
=pod

=head2 DotplotChromosomeArm Class 

 Constructor Args: 
    top_id     : The parent element of the created area
    x1,y1,x2,y2: The coordinates of the area
    ... a lot of arguments to be displayed
    Global     : O_CFG, O_PARAM
	
=cut
*/

DotplotChromosomeArmClass = Class.create(NarcisseAreaClass,{
	initialize: function($super,a_graphics_objects)
	{
		$super(a_graphics_objects);
		this._feature         = 'dotplot';
	},
	MouseOver: function($super,x1,y1,x2,y2,target,reference,ref_chrom,start,end)
	//MouseOver: function($super,a_args)
	{
		/*var x1 = a_args[0];
		var y1 = a_args[1];
		var x2 = a_args[2];
		var y2 = a_args[3];
		var target    = a_args[5];
		var reference = a_args[6];
		var ref_chrom = a_args[7];
		var start  = a_args[8];
		var end    = a_args[9];*/
		
		if ($super(x1,y1,x2,y2) == false ) { return; };
		
		var length = end - start + 1;
		var html='<h2>Chromosome arm</h2>';
		
		html += '<table>';
		html += '<tr><th>Reference</th><td>' + reference + '</td></tr>';
		var arm = (start==1)?'p arm ':'q arm ';
		html += '<tr><th>Chromosome</th><td>' + ref_chrom + ' (' + arm + PrettyDisplayPosition(length) + ')</td></tr>';
		html += '<tr><th>Target</th><td>' + target + '</td></tr>';
		html += '</table>';
		$(this._legend).innerHTML=html;
	},
	MouseDown: function($super,x1,y1,x2,y2,target,reference,ref_chrom,start,end)
	//MouseDown: function($super,a_args)
	{
		/*var x1 = a_args[0];
		var y1 = a_args[1];
		var x2 = a_args[2];
		var y2 = a_args[3];
		var target    = a_args[5];
		var reference = a_args[6];
		var ref_chrom = a_args[7];
		var start  = a_args[8];
		var end    = a_args[9];*/
		
		if ( $super() == false ) {return;};
	
		var p = 'script=DISPLAY';
		p += '&reference=' + reference;
		p += '&target1=' + target;
		p += '&chroms_reference=' + ref_chrom;
		//p += '&pos='   + (end+start)/2;
		p += '&range=' + start + ':' + end;
		//p += '&click=zoom_level_arm';		
		//O_CONTROL_BLOCK.Submit(p);
		var hist = 'DISPLAY ' + reference + ':' + ref_chrom + ' ' + target;
		O_CONTROL_BLOCK.SubmitAjax(p,hist);
	}
});
	


/*
=pod

=head2 DotplotChrom2ChromClass

Constructor Args: 
   top_id     : The parent element of the created area
   x1,y1,x2,y2: The coordinates of the area
   ... a lot of arguments to be displayed
   Global:    : O_CFG, O_PARAM
   
=cut
*/

DotplotChrom2ChromClass = Class.create(NarcisseAreaClass,{
	initialize: function($super,a_graphics_objects)
	{
		$super(a_graphics_objects);
		this._feature         = 'dotplot';
	},
	MouseOver: function($super,x1,y1,x2,y2,reference,ref_chrom,target,target_chrom)
	//MouseOver: function($super,a_args)
	{
		/*var x1 = a_args[0];
		var y1 = a_args[1];
		var x2 = a_args[2];
		var y2 = a_args[3];
		var reference = a_args[5];
		var ref_chrom = a_args[6];
		var target    = a_args[7];
		var target_chrom = a_args[8];*/

		if ($super(x1,y1,x2,y2) == false ) { return; };
		
		var html='<h2>Dotplot</h2>';
		
		html += '<table>';
		html += '<tr><th>Reference</th><td>' + reference + '</td></tr>';
		html += '<tr><th>Chromosome</th><td>' + ref_chrom + '</td></tr>';
		html += '<tr><th>Target</th><td>' + target + '</td></tr>';
		html += '<tr><th>Chromosome</th><td>' + target_chrom + '</td></tr>';

		html += '</table>';
		$(this._legend).innerHTML=html;
	},
	MouseDown: function($super,x1,y1,x2,y2,reference,ref_chrom,target,target_chrom)
	//MouseDown: function($super,a_args)
	{
		/*var x1 = a_args[0];
		var y1 = a_args[1];
		var x2 = a_args[2];
		var y2 = a_args[3];
		var reference = a_args[5];
		var ref_chrom = a_args[6];
		var target    = a_args[7];
		var target_chrom = a_args[8];*/

		if ( $super() == false ) {return;};
		
		var p = 'chroms_reference=' + ref_chrom + '&chroms_target1=' + target_chrom;
		var h = 'DOTPLOT ' + reference + ':' + ref_chrom + ' ' + target + ':' + target_chrom;
		O_CONTROL_BLOCK.SubmitAjax(p,h);
	}
	
});

/*      DotplotSyntClass Class      

Constructor Args: 
           top_id       The parent element of the created area
		   x1,y1,x2,y2  The coordinates of the area
		   ... a lot of arguments to be displayed
		   Global:      O_CFG, O_PARAM
*/

DotplotSyntClass = Class.create(NarcisseAreaClass,{
	initialize: function($super,a_graphics_objects)
	{
		$super(a_graphics_objects);
		this._feature         = 'dotplot';
		//this.__display_base_url          = O_CFG['HTTP_NARCISSE_CGI']+'?__wb_session='+O_PARAM['__wb_session']+'&amp;'+'script=DISPLAY';
		//this.extract_base_url = base_url + 'script=EXTRACT';
		//this.buildfam_base_url= base_url + 'script=BUILD_FAM';
	},
	MouseOver: function($super,x1,y1,x2,y2,synt_id,reference,ref_chrom,target,target_chrom)
	//MouseOver: function($super,a_args)
	{
		/*var x1 = a_args[0];
		var y1 = a_args[1];
		var x2 = a_args[2];
		var y2 = a_args[3];
		var synt_id   = a_args[5];
		var reference = a_args[6];
		var ref_chrom = a_args[7];
		var target    = a_args[8];
		var target_chrom = a_args[9];*/

		if ($super(x1,y1,x2,y2) == false ) { return; };
		
		var html='<h2>Dotplot</h2>';
		
		html += '<table>';
		//html += '<tr><th>Synteny</th></td>' + synt_id + '</td></tr>';
		html += '<tr><th>Reference</th><td>' + reference + '</td></tr>';
		html += '<tr><th>Chromosome</th><td>' + ref_chrom + '</td></tr>';
		html += '<tr><th>Target</th><td>' + target + '</td></tr>';
		html += '<tr><th>Chromosome</th><td>' + target_chrom + '</td></tr>';

		html += '</table>';
		$(this._legend).innerHTML=html;
	},
	MouseDown: function($super,x1,y1,x2,y2,synt_id,reference,ref_chrom,target,target_chrom)
	//MouseDown: function($super,a_args)
	{
		/*var x1 = a_args[0];
		var y1 = a_args[1];
		var x2 = a_args[2];
		var y2 = a_args[3];
		var synt_id   = a_args[5];
		var reference = a_args[6];
		var ref_chrom = a_args[7];
		var target    = a_args[8];
		var target_chrom = a_args[9];*/

		if ( $super() == false ) {return;};
		
		var prms ='script=DISPLAY&click=zoom_level_synteny&view_level=0&reference='+reference+'&target1='+target+'&chroms_reference='+ref_chrom+'&center_synt='+synt_id;
		var hist = 'DISPLAY ' + reference + ':' + ref_chrom + ' ' + target;
		O_CONTROL_BLOCK.SubmitAjax(prms,hist);
	}
});

// This is a trivial object, just here for the NarcisseArea objects
DotplotFeatureClass = Class.create({
	IsFeatureOn: function(feature){return (feature=='dotplot');},
	UpdatePage:  function(){},
	GetKey:      function(){return 'dotplot';}
});

/*
=pod

=head2 NcDotplotClass

 NcDotplotClass derives from NcViewClass
 Private fields are initialized, virtual functions are redefined
 
 Constructor Args:
    a_graphics_objects : A lot of graphics objects
	TODO - Remplacer ca par un objet !

=cut
*/
NcDotplotClass = Class.create(NcViewClass,{
		initialize: function($super,a_graphics_objects){
			$super(a_graphics_objects);
			this._script  = 'DOTPLOT';
			this._to_hide = ['ID_MENU_POSITION','ID_MENU_FEATURES','ID_MENU_TAB_POPUP','ID_VLVL','ID_OVERVIEW'];
			this._to_show = ['ID_MENU_GLOBAL','ID_LEVEL'];
			O_DOTPLOT_CHROMOSOME_ARM = new DotplotChromosomeArmClass(a_graphics_objects);
			O_DOTPLOT_CHROM2CHROM    = new DotplotChrom2ChromClass(a_graphics_objects);
			O_DOTPLOT_SYNT           = new DotplotSyntClass(a_graphics_objects);
		},
		UpdateTitle: function()
		{
			var html = '<h2>Dotplot representation</h2>';
			html += this._GetSpeciesNames(1);
			this._title_div.innerHTML = html;
		}
});

/* GLOBAL VARIABLES */
MAIN:
{
	/* Global variables, they will be initialized later */
	O_DOTPLOT_CHROMOSOME_ARM = 0;
	O_DOTPLOT_CHROM2CHROM = 0;
	O_DOTPLOT_SYNT = 0;
};


/*
=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license under French law - www.cecill.info

=cut
*/

