/*
  Copyright INRA/CNRS

 Emmanuel.Courcelle@toulouse.inra.fr
 Jerome.Gouzy@toulouse.inra.fr
 Thomas.Faraut@toulouse.inra.fr

 This software is a computer program whose purpose is to provide a
 web-based interface for analyzing the different levels of genome
 conservations.

 This software is governed by the CeCILL license under French law and
 abiding by the rules of distribution of free software.  You can  use,
 modify and/ or redistribute the software under the terms of the CeCILL
 license as circulated by CEA, CNRS and INRIA at the following URL
 "http://www.cecill.info".

 As a counterpart to the access to the source code and  rights to copy,
 modify and redistribute granted by the license, users are provided only
 with a limited warranty  and the software's author,  the holder of the
 economic rights,  and the successive licensors  have only  limited
 liability.

 In this respect, the user's attention is drawn to the risks associated
 with loading,  using,  modifying and/or developing or reproducing the
 software by the user in light of its specific status of free software,
 that may mean  that it is complicated to manipulate,  and  that  also
 therefore means  that it is reserved for developers  and  experienced
 professionals having in-depth computer knowledge. Users are therefore
 encouraged to load and test the software's suitability as regards their
 requirements in conditions enabling the security of their systems and/or
 data to be ensured and,  more generally, to use and operate it in the
 same conditions as regards security.

 The fact that you are presently reading this means that you have had
 knowledge of the CeCILL license and that you accept its terms

*/
/*

=pod

=head1 NAME
 NcKaryotype.js - Global objects used with the Karyotype representation

=cut
*/

/*
=pod

=head1 SYNOPSIS

 NarcisseAreaClass               see narcisse_area.js
    ChromosomeCursorClass
    RepeatsClass
	RnaClass
	ExonsClass
    GenesClass
    SyntClass
    HspClass
    PersonalClass

=cut
*/
/* EXPERIMENTAL !
DraggingClass = Class.create({
			initialize: function(inner_container)
			{
				this.__inner_cont = $(inner_container);
				this.__start_top  = 0;
				this.__top        = 0;
				this.__dragging   = false;

				Event.observe(this.__inner_cont,'mousedown',this.__StartMove.bindAsEventListener(this));
				Event.observe(this.__inner_cont,'mousemove',this.__Moving.bindAsEventListener(this));
				Event.observe(this.__inner_cont,'mouseup',this.__StopMove.bindAsEventListener(this));
			},

			__StartMove: function(evt)
			{
				this.__start_top = evt.clientY;
				this.__inner_cont.style.cursor = "-moz-grab";
				this.__top = this.__stripPx(this.__inner_cont.style.top);
				this.__dragging = true;
			},

			__Moving: function(evt)
			{
				if (this.__dragging)
				{
					this.__inner_cont.setStyle({top: this.__top + (evt.clientY - this.__start_top)});
				}
			},

			__StopMove: function(evt)
			{
				this.__inner_cont.style.cursor = "";
				this.__dragging   = false;
			},

			__StripPx: function(value)
			{
				if (value == "") return 0;
				return parseFloat(value.substring(0, value.length - 2));
			}
	});
*/

/*      Karyotype Class

Constructor Args:
           top_id       The parent element of the created area
		   x1,y1,x2,y2  The coordinates of the area
		   ... a lot of arguments to be displayed
		   Global:      O_CFG, O_PARAM
*/

KaryotypeClass = Class.create(NarcisseAreaClass,{
	initialize: function($super,a_graphics_objects)
	{
		$super(a_graphics_objects);
		this._feature         = 'cyto';
	},
	MouseOver: function($super,x1,y1,x2,y2,species,chrom,elt,start,end,color)
	//MouseOver: function($super,a_args)
	{
		/*var x1 = a_args[0];
		var y1 = a_args[1];
		var x2 = a_args[2];
		var y2 = a_args[3];
		var species = a_args[5];
		var chrom   = a_args[6];
		var elt     = a_args[7];
		var start   = a_args[8];
		var end     = a_args[9];
		var color   = a_args[10];*/

		if ($super(x1,y1,x2,y2) == false ) { return; };

		var html='<h2>Karyotype</h2>';

		html += '<table>';
		html += '<tr><th>Species</th><td>' + species + '</td></tr>';
		html += '<tr><th>Chromosome</th><td>' + chrom + '</td></tr>';
		html += '<tr><th>Element</th><td>' + elt + '</td></tr>';
		html += '<tr><th>Start</th><td>' + PrettyDisplayPosition(start) + ' (' + start + ') </td></tr>';
		html += '<tr><th>End</th><td>' + PrettyDisplayPosition(end) + ' (' + end + ') </td></tr>';
		html += '<tr><th>Color</th><td>' + color + '</td></tr>';
		html += '</table>';
		$(this._legend).innerHTML=html;
	},
	MouseDown: function($super,x1,y1,x2,y2,species,chrom)
	//MouseDown: function($super,a_args)
	{
		/*var x1 = a_args[0];
		var y1 = a_args[1];
		var x2 = a_args[2];
		var y2 = a_args[3];
		var species = a_args[5];
		var chrom   = a_args[6];*/

		if ( $super() == false ) {return;};

		var prms = 'script=DISPLAY&click=zoom_level_chromosome&reference='+species+'&chroms_reference='+chrom+'&target1=none';
		var hist = 'DISPLAY ' + species + ':' + chrom;
		O_CONTROL_BLOCK.SubmitAjax(prms,hist);
	}
});


/*
=pod

=head2 NcKaryotypeClass

 NcdisplayMapClass derives from NcViewClass
 Private fields are initialized, virtual functions are redefined

 Constructor Args:
    a_graphics_objects : A lot of graphics objects
	TODO - Remplacer ca par un objet !

=cut
*/
NcKaryotypeClass = Class.create(NcViewClass,{
		initialize: function($super,a_graphics_objects){
			$super(a_graphics_objects);
			this._script  = 'KARYOTYPE';
			this._to_hide = ['ID_MENU_GLOBAL','ID_MENU_POSITION','ID_MENU_FEATURES','ID_MENU_TAB_POPUP','ID_MENU_DATA','ID_OVERVIEW'];
			this._to_show = [];
			O_KARYOTYPE         = new KaryotypeClass(a_graphics_objects);
		},

		/*
=pod

=head2 UpdateTitle

Title: UpdateTitle
Usage: obj.UpdateTitle()
Procedure: Updates the title table, with the names of reference, target1, target2, ...
Args:     none
Access:   Public
Global: O_CFG, O_PARAM

=cut
*/
		UpdateTitle: function()
		{
			var html = '<h2>The karyotype</h2>';
			html += this._GetSpeciesNames(0);
			this._title_div.innerHTML = html;
		}
});

MAIN:
{
	/* Global variables, they will be initialized later */
	O_CHROMOSOME_CURSOR=0;
	O_REPEAT=0;
	O_RNA=0;
	O_MARKERS=0;
	O_GENE=0;
	O_EXON=0;
	O_SYNT=0;
	O_HSP=0;
	O_CYTO=0;
	O_OPENWIN_ALIGNMENT=0;
}

/*
=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license under French law - www.cecill.info

=cut
*/

