/*
  Copyright INRA/CNRS

 Emmanuel.Courcelle@toulouse.inra.fr
 Jerome.Gouzy@toulouse.inra.fr
 Thomas.Faraut@toulouse.inra.fr

 This software is a computer program whose purpose is to provide a
 web-based interface for analyzing the different levels of genome
 conservations.

 This software is governed by the CeCILL license under French law and
 abiding by the rules of distribution of free software.  You can  use,
 modify and/ or redistribute the software under the terms of the CeCILL
 license as circulated by CEA, CNRS and INRIA at the following URL
 "http://www.cecill.info".

 As a counterpart to the access to the source code and  rights to copy,
 modify and redistribute granted by the license, users are provided only
 with a limited warranty  and the software's author,  the holder of the
 economic rights,  and the successive licensors  have only  limited
 liability.

 In this respect, the user's attention is drawn to the risks associated
 with loading,  using,  modifying and/or developing or reproducing the
 software by the user in light of its specific status of free software,
 that may mean  that it is complicated to manipulate,  and  that  also
 therefore means  that it is reserved for developers  and  experienced
 professionals having in-depth computer knowledge. Users are therefore
 encouraged to load and test the software's suitability as regards their
 requirements in conditions enabling the security of their systems and/or
 data to be ensured and,  more generally, to use and operate it in the
 same conditions as regards security.

 The fact that you are presently reading this means that you have had
 knowledge of the CeCILL license and that you accept its terms

*/


/*

=pod

=head1 NAME
 controlblock.js - The DessusDessous class as well as the main control block code

=cut
*/

/*
=pod

=head2 DessusDessousClass

 Title:     DessusDessousClass
 Usage:     var o_dessus_dessous = new DessusDessousclass(o_control_block,dynamic_legend)
 Class:     This class defines a tool to manage the relative position of the control block and of the dynamic legend
 Constructor Args:
               o_controlblock  : The ControlblockClass object
			   dynamic_legend  : A div
 Prerequisite: o_control_block MUST contain a TabView object (see YUI)

=cut
*/

DessusDessousClass = Class.create({
	initialize: function(o_controlblock,dynamic_legend)
	{
		//this.__ctrl     = $(controlblock);
		this.__ctrl_top = $(o_controlblock.GetId());
		this.__lgnd     = $(dynamic_legend);

		// The tabs managed by the tabView
		this.__ctrl_content = this.__ctrl_top.select('.yui-content')[0];

		// Control Block up when the mouse goes to the navigation bar
		Event.observe(this.__ctrl_top,'mouseover',this.BlockIsUp.bindAsEventListener(this));
		Event.observe(this.__ctrl_top,'click',this.BlockIsUp.bindAsEventListener(this));

		// Using YUI's addListener to listen to non DOM events
		// When an active tab is changed, we force the display
		o_controlblock.GetTabview().addListener('activeTabChange',this.BlockIsUp.bindAsEventListener(this));
	},

	/*
	Title: __IsOverlap
	Usage:     if (this.__IsOverlap())...
	Function:  returns true if the controlbox and the dynamic legend overlap
	Args:      none
    Return:    boolean
	Access:    private
	Globals:
	*/
	__IsOverlap: function()
	{
		var btm_content = this.__ctrl_content.viewportOffset()[1]+this.__ctrl_content.getHeight();
		var top_legend  = this.__lgnd.viewportOffset()[1];
		if ( btm_content >= top_legend )
		{
			//console.log("overlap %s >= %s",btm_content,top_legend);
			return true;
		}
		else
		{
			//console.log("pas d'overlap %s < %s",btm_content,top_legend);
			return false;
		}
	},

	/*
=pod

=head3 BlockIsUp

	Title: BlockIsUp
	Usage:     o_dessus_dessous.BlockIsUp()
	Procedure:  Force the active tab of the control block to be displayed, may be hiding the dynamic legend
	Args:      none
	Globals:

=cut
	*/
	BlockIsUp: function()
	{
		if (this.__IsOverlap())
		{
			this.__lgnd.hide();
		}
		this.__ctrl_content.show();
	},

	/*
=pod

=head3 LegendIsUp

	Title: LegendIsUp
	Usage:     o_dessus_dessous.LegendIsUp()
	Procedure:  Force the dynamic legend to be active, may be hiding the active tab of the control block
	Args:      none
	Globals:

=cut
	*/
	LegendIsUp: function()
	{
		if (this.__IsOverlap())
		{
			this.__ctrl_content.hide();
		}
		this.__lgnd.show();
	}
});

/*
=pod

=head2 ControlBlockClass ==> Connected to controlblock.inc

 XHTML LAYOUT:
 =============
 <div id="top_id" class="yui-navset">
    <form>                    ==> this.form
       <p>
       <input>...                 several hidden fields
       </p>
         <ul class="History...">   ==> The history navigation bar and the led
         <ul class="yui-nav...">   ==> The navigation bar
         <div class="yui-content">
                                   ==> The tabs
         </div>
    </form>
 <div>

=head2 class ControlBlockClass

 Title        : ControlBlockClass
 Derivation   : Derives from LipmAjaxFormClass
 Usage        : ControlBlockClass is related to the controlblock form
 Constructor arg: top_id             the id of the top element (should be a div, it is also the top_id for the encapsulted tabView object)
                  ajax_container     the id of the element (a div) which will be updated by the SubmitAjax method
				  o_error (optional) An object of class LipmPopupBaseClass, will be passed to the super class
				  o_led              An object of class LipmLedBaseClass, will be passed to the super class
				  o_history_manager  An object to pass to the super class, to manage the history
 Member functions:
                UpdatePage, MAY BE overloaded - Update the displayed page from the parameters managed by the tab
 Globals      : O_CFG, O_PARAM

=cut
*/

ControlBlockClass = Class.create(LipmAjaxFormClass,{
	initialize: function($super,top_id,ajax_container,o_error,o_led,o_history_manager)
	{
		this.__top_id    = $(top_id);
		this.__o_tabview = '';
		this.__nav       = this.__top_id.down('ul.yui-nav',0);
		this.__overview  = $('ID_OVERVIEW');
		this.__overview_height = this.__overview.getHeight();
		this.__initial_request = true;

		// Calling the super class
		$super($(top_id).down('form'),ajax_container,o_error,o_led,o_history_manager);

		// Registering myself to the history manager
		o_history_manager.RegisterAjax(this);

		// There are several "tabs" in controlblock - Each tab is kept in the hash this.__h_tabs
		// Each tab derives from ControlBlockMemberClass
		// You can add a new tab with the method RegisterTab
		this.__h_tabs = $H();

		// Each 'view' object is registered by in the control block (cf. RegisterView)
		// The registered object is kept in the hash, the key is the script= parameter
		this.__h_views = $H();

		// Observing the Back and Forward buttons
		this.__ObserveBackFwd();

		// Init the form
		this._form['__wb_session'].value = O_PARAM['__wb_session'];
		this.__script = '';	// will be managed by UpdatePage

		InitForm(this._form,O_PARAM);
		this.__backup = this._form.serialize(true);

		// Is it still useful ????
		if( ( this._form['target1'] == undefined ) && ( O_PARAM.target1 != undefined ) )
		{
			this.SetValue('target1='+O_PARAM.target1);
		}

		// Observing the tools from the toolbar
		this.__ObserveTools();

	},
	__ObserveBackFwd: function()
	{
			// Observing the Back and Forward buttons
		var back_btn = this.__top_id.down('ul').down('li');
		var forw_btn = back_btn.next('li').next('li');
		var refr_btn = forw_btn.next('li');
		var thisControlBlock = this;
		Event.observe(back_btn,'click',thisControlBlock.Back.bindAsEventListener(thisControlBlock));
		Event.observe(forw_btn,'click',thisControlBlock.Forward.bindAsEventListener(thisControlBlock));
		Event.observe(refr_btn,'click',thisControlBlock.Reload.bindAsEventListener(thisControlBlock));
	},
	__ObserveTools: function()
	{
		this.__ObserveOpenGensel();
		this.__ObserveUpload();
		this.__ObserveSave();
		//this.__ObserveCopy(); (this is just an href link)
		this.__ObserveFullScreen();
		this.__ObserveHowtoLink();
		this.__ObservePrinter();
	},
	__ObserveOpenGensel: function()
	{
		// Creating the gensel tool, observing the gensel button through an ExpandableClass object
		var thisControlBlock = this;
		var o_exp  = new ExpandableClass('ID_GENSEL','ID_GENSEL_CONTAINER');
		var o_data_source  = new YAHOO.widget.DS_JSArray( GetSpeciesList() );
		o_data_source.queryMatchCase     = false;
		o_data_source.queryMatchContains = true;
		var o_gensel = new GenselFormClass($('ID_GENSEL_CONTAINER'),o_data_source,this._o_error,o_exp,this);
		o_exp.AddExpandHook(o_gensel.__GenselExpanded.bind(o_gensel));
		o_exp.AddCollapseHook(o_gensel.__GenselCollapsed.bind(o_gensel));
		this.__o_gensel = o_gensel;
	},
	__ObserveUpload: function()
	{
		// Creating an expandable class for the "upload" window
		var thisControlBlock = this;
		var upl_file   = $('ID_UPLOAD_CONTAINER').down('input',0);
		var upl_button = $('ID_UPLOAD_CONTAINER').down('input',1);
		var o_exp  = new ExpandableClass('ID_UPLOAD','ID_UPLOAD_CONTAINER');
		Event.observe(upl_button,'click',function(e){
				//Event.stop(evt);
				var prm = upl_file.serialize();
				thisControlBlock.Submit(prm);
		});
	},
	__ObserveSave: function()
	{
		// Observe the "Save" tool
		var thisControlBlock = this;
		Event.observe($('ID_SAVE'),'click',function(e){	thisControlBlock.Submit('script=SAVESESSION','NARCISSE_tgz'); });
	},
	__ObserveFullScreen: function()
	{
		// Observe the "FullScreen tool" of the toolbar
		Event.observe($('ID_FULL_SCREEN_1'),'click',function(e){O_SCREEN.ToFullScreen();});
	},
	__ObservePrinter: function()
	{
		// Observing the Printer button through a LipmPopupNewWindowClass
		var w = new LipmPopupNewWindowClass('Printer',{
				directories: false,
				location: false,
				menubar: true,
				toolbar: false,
				scrollbars: true,
				status: true,
				resize: true,
				copyhistory: false,
				height: 1000,
				width: 1000
		});
		w.Observe(this.__top_id);
		w.RegisterUrlComp(this.__ComputeUrlForPrinting.bindAsEventListener(this));
	},
	__ObserveHowtoLink: function()
	{
		// Creating an expandable class for the "howtolink" window
		var thisControlBlock = this;
		var o_exp  = new ExpandableClass('ID_HOWTOLINK','ID_HOWTOLINK_CONTAINER');
		var current_url = $('ID_HOWTOLINK_CONTAINER').down('input');
		o_exp.AddExpandHook(function(){current_url.setValue(thisControlBlock.ComputeUrl());});

		// Observe the 'current_url' field: the url is computed as soon as the field gets the focus, and the field is selected
		Event.observe(current_url,'focus',function(evt){current_url.setValue(thisControlBlock.ComputeUrl());current_url.select();});

	},
	__ComputeUrlForPrinting: function()
	{
		var url = this.ComputeUrl();
		url += '&PrintImage=1&__wba_popup=1';
		url += '&__wb_session='+ O_PARAM.__wb_session;
		return url;
	},
	// Three wrappers for some methods of the LipmHistory object encapsulated by this Controlblock
	Reload: function() { this._o_ajax_history.Reload(); },
	Back: function() { this._o_ajax_history.Back(); },
	Forward: function() { this._o_ajax_history.Forward(); },

/*

=pod

=head3 ComputeUrl

	Title   : ComputeUrl
	Usage   : var url = obj.ComputeUrl()
	Function: Compute the current url
	          We declare the list of parameters to be used when computing the URL
	Args    : none

=cut
	*/
    ComputeUrl: function()
	{
		var h_reduced_parameters = $H();

		/* We keep the base parameters from O_PARAM */
		var script = O_PARAM['script'];
		['script','reference','chroms_reference','level'].each(function(p){
				if (O_PARAM[p] != undefined){h_reduced_parameters.set(p,O_PARAM[p]);}
		});

		/* We keep also the parameters relative to the targets */
		var max = O_CFG[script+'_MAX_ORG'];
		$R(1,max).each(function(n){
				var t = 'target' + n;
				var p = 'program_' + t;
				var c = 'chroms_' + t;
				[t,p,c].each(function(prm){
						if (O_PARAM[t] != undefined){h_reduced_parameters.set(t,O_PARAM[t]);}
				});
		});

		/* If DISPLAY: We keep more parameters */
		if (script == 'DISPLAY')
		{
			['range','view_level'].each(function(p){
					if (O_PARAM[p] != undefined){h_reduced_parameters.set(p,O_PARAM[p]);}
			});

			Object.keys(O_PARAM).each(function(p){
					if (p.startsWith('ck_')) {h_reduced_parameters.set(p,O_PARAM[p]);}
			});

			/* Merge the parameters that were changed on the client */
			h_reduced_parameters = h_reduced_parameters.merge(this.GetChangedFeatures());
		};

		/* We compute the URL from the parameters */
		var url = O_CFG.HTTP_NARCISSE_CGI + '?';
		url += h_reduced_parameters.toQueryString();
		return url;
	},

	/*
=pod

=head3 SubmitAjax

	Title   : SubmitAjax
	Usage   : var form = obj.GetForm();
	Function: The SubmitAjax from LipmFormClass is overloaded:
	          1/ clear the ajax queue, cancelling the planned partial requests, if any
			  2/ Compute the fields passed to the server with the function this.GetChangedFeatures, taking into account the features changed locally.
              The superclass function is called anyway
	Args    : none

=cut
	*/
	SubmitAjax: function($super,field_values,history_text,on_success)
	{
		var h_features = this.GetChangedFeatures();
		this.ClearChangedFeatures();
		this.ClearAjaxQueue();

		//var map = $('ID_MAP_MAP');
		//if (map != null)
		//{
		//	map.remove();
		//}

		// This is used with partial requests - Only the FIRST request calls this function
		this.__initial_request = true;
		$super(field_values+'&'+h_features.toQueryString(),history_text,on_success);
	},

	/*
=pod

=head3 SubmitExchange

	Title   : SubmitExchange
	Usage   : O_CONTROLBLOCK.SubmitExchange(N,chrom,url,history)
	Function: Call the RefExchange method from the encapsulted gensel tool, append the url, and call SubmitAjax
	Args    : N       : If 0, nothing to change, if >0, exchanges targetN with the reference
	          chrom   : The new chrom reference
			  url     : url to add to the reference, targetN, etc. computed
			  history :

=cut
	*/
	SubmitExchange: function(N,chrom,url,history)
	{
		if (this.__o_gensel != undefined)
		{
			//var submit_url = this.__o_gensel.RefExchange(N,chrom);
			var submit_url = this.__RefExchange(N,chrom);
			if (submit_url != '') { submit_url += '&'; };
			submit_url += url;
			//alert (submit_url);
			this.SubmitAjax(submit_url,history);
		}
		else
		{
			alert ('THIS FUNCTION IS DISABLED');
		}
	},

    /*
    =pod

    =head2 __RefExchange

    Title:    __RefExchange
    Usage:    url = this.__RefExchange(N,chrom)
    Procedure: If N > 0, exchange reference and targetN, and call this.__o_gensel.RefExchange, trying to synchronize data
	           If N == 0, makes nothing
	           Return the corresponding url (ONLY reference, chrom_reference, targetN)
    Args: N        : If 0, nothing changed, if >0, exchanges targetN with the reference
	      chrom    : The new chrom reference

	*/
	__RefExchange: function(N,chrom)
	{
		var tgt = 'target'+N;
		var prog= 'program_' + tgt;
		if (N > 0)
		{
			if (O_PARAM[tgt] == undefined)
			{
				throw('ERROR - cannot find target' + N + ' among the parameters');
			}
			var tg = O_PARAM[tgt];
			var rf = O_PARAM['reference'];
			var pg = O_PARAM['program_' + tgt];
			this.__o_gensel.RefExchange(rf,N,chrom);
			return 'reference=' + tg + '&' + tgt + '=' + rf + '&' + prog + '=' + pg + '&chroms_reference=' + chrom;
		}
		else
		{
			return '';
		}
	},

	/*
=pod

=head3 _AddToHistory

	Title   : _AddToHistory
	Usage   : from the superclass's _AjaxSuccess
	Function: The standard function _AddtoHistory is overloaded here, so that the behaviour of LipmAjaxformClass is changed
              from the History point of vue

=cut
	*/
	_AddToHistory: function(history_text, history_stamp)
	{
		if (this._o_ajax_history != undefined)
		{
			var stamp = 'read_bookmark='+O_PARAM.history_stamp;
			this._o_ajax_history.Add(history_text,stamp);
		};
	},

	/*
=pod

=head3 GetId, GetForm, GetErrorObject, GetTabview, GetGensel

	Title   : GetForm, GetErrorObject, GetTabview,GetGensel
	Usage   : var form = obj.GetForm();
	Function: Getter for the top id, form, the error popup, of the tabview
              NOTE - For the Tabview, this call is invalid until the Render method has been applied
	Args    : none

=cut
	*/
	GetId: function() {return this.__top_id.identify();},
	GetForm: function() { return this._form; },
	GetErrorObject: function() { return this._o_error; },
	GetTabview: function() { return this.__o_tabview;},
	GetGensel: function() {return this.__o_gensel;},

	/*
=pod

=head3 GetBottom

	Title   : GetBottom
	Usage   : var y_pos = obj.GetBottom();
	Function: Get the y coordinate of the control block bottom - Useful to put another element under the block
	Args    : none

=cut
	*/
	GetBottom: function()
	{
		var rvl = this.__top_id.positionedOffset()[1];
		rvl += this.__top_id.getHeight();
		return rvl;
	},

	/*
=pod

=head3 RegisterTab

	Title    : RegisterTab
	Usage    : obj.RegisterTab(o_tab);
	Procedure: Register a new object, generally a tab, for UpdatePage
	Args     : o_tab     The object to register

=cut
	*/
	RegisterTab: function(o_tab)
	{
		this.__h_tabs.set(o_tab.GetKey(),o_tab);
		return;
	},
	/*
=pod

=head3 RegisterView

	Title    : RegisterView
	Usage    : obj.RegisterView(o_view);
	Procedure: Register a new view_object
	           The registered object must be dervied from NcView (see NcView.js)
	Args     : o_view     The object to register

=cut
	*/
	RegisterView: function(o_view)
	{
		this.__h_views.set(o_view.GetScript(),o_view);
		return;
	},

	/*
=pod

=head3 Render

	Title    : Render
	Usage    : obj.Render();
	Procedure: Render the control block
	           Call it after tabs registration
               The Yahoo TabView object is initialized
	Args     : none

=cut
	*/
	Render: function()
	{
		// Managing the tabs with YUI
		this.__o_tabview = new YAHOO.widget.TabView(this.__top_id.identify());
	},

	/*
=pod

=head3 UpdatePage

	Title: UpdatePage
	Usage: obj.UpdatePage();
	Procedure: Call UpdatePage on each object found in this.__h_tabs
	           Each object updates the page, displaying or hiding some stuff
	Args:      none

=cut
	*/
	UpdatePage: function()
	{
		// check the script field, compare it with the private field __script
		// If different, the was a change: in this case there are things to hide/show
		var script = $F(this.GetForm()['script']);
		var o_view = this.__h_views.get(script);

		// Update the title box
		if (o_view != undefined)
		{
			o_view.UpdateTitle();
		}

		// Update the controlbox itself, but only if necessary
		if (script != this.__script)
		{
			// Open the active tab
			var tab_list = ['zoom','zoom2','data','display','session','align','links'];

			var active_key = O_PARAM['active_'+script+'_tab'];
			var active_index = tab_list.indexOf(active_key);
			if (active_index != -1) {this.__o_tabview.set('activeIndex', active_index);};

			this.__script = script;
			this.SetDefaultParams('script=' + script + '&__wb_session='+O_PARAM.__wb_session);
			if (o_view != undefined)
			{
				o_view.GetCBToHide().each(function(id) {
						$(id).hide();
				});
				o_view.GetCBToShow().each(function(id) {
						$(id).show();
				});
			}

			// Synchronize the gensel tool with the data - in fact only single chromosome
			if (this.__o_gensel != undefined)
			{
				this.__o_gensel.RefExchange(O_PARAM.reference,0,O_PARAM.chroms_reference);
			}
		}

		// Call UpdatePage for every registered tab
		this.__h_tabs.each(function(p) {
				/* Don't know why, but an exception may escape from here using MSIE */
				try
				{
					p.value.UpdatePage();
				}
				catch(e){};
		});

		// Build the areas only if initial request (only for DisplayMap yet)
		if (this.__initial_request)
		{
			this.__initial_request = false;


			//this.__BuildAreas.defer();

			// NOTE - Useless modifying the overview if hidden

			if (this.__overview.visible() && O_PARAM['overview'] != undefined)
			{
				var a_overview  = O_PARAM['overview'].split(':');
				var id_overview = this.__overview.down('div');
				id_overview.setStyle({
					top:    this.__overview_height * a_overview[0] / 100 + 'px',
					height: this.__overview_height * a_overview[1] / 100 + 'px'
				});
			}
		}
	},

	/*
=pod

=head3 __BuildAreas

	Title: __BuildAreas
	Usage: obj.BuildAreas();
	Procedure: Called by UpdatePage through a Defer() because it can be time consuming - 
	           Initiates a NEW ajax request to retrieve the areas and create the areas
	Args:      none

=cut
	*/
	__BuildAreas: function()
	{
		var params = "__wb_function=GetAreas";
		var o_ajax = new LipmAjaxDataClass();
		var thisObject = this;
		o_ajax.SubmitAjax(params,function (){
			var o_areas = o_ajax.GetData();
			if (o_areas != undefined)
			{
				thisObject.__a_areas = $A(o_areas);
				var map = $('ID_MAP_MAP');
				
				/* TODO - Pas fameux, c'est pour eviter d'avoir des pbs de perfs si on a trop de
				          map areas a faire... */
				var step = 1;
				if (thisObject.__a_areas.length > 1000)
				{
					step = Math.round(thisObject.__a_areas.length / 1000);
				}
				for (var key=0; key < thisObject.__a_areas.length; key += step)
				{
					var a_val = thisObject.__a_areas[key];
					var o_tmp = a_val[4];
					var o_obj = 'o_obj = O_' + a_val[4];
		
					// Ignore errors, they could be due to a buggy perl script
					try
					{
						eval(o_obj);
					} catch(e)
					{
						//alert (o_obj);
						return;
					};
		
					a_val[4] = o_obj;
		
					// Create an area
					var coords= a_val[0] + ',' + a_val[1] + ',' + a_val[2] + ',' + a_val[3];
					var title = o_obj._GetTitle();
					var area;
					if (title == '')
					{
						area  = new Element('area', {
							'id'    : 'ID_AREA_'+key,
							'coords': coords,
							'href'  : '#',
							'shape' : 'rect',
							'alt'   : 'area'
						});
					}
					else
					{
						area  = new Element('area', {
							'id'    : 'ID_AREA_'+key,
							'coords': coords,
							'href'  : '#',
							'shape' : 'rect',
							'alt'   : title,
							'title' : title
						});
					};
		
					//alert (o_tmp+' '+a_val[10]);
		
					// Observe several mouse events
					Event.observe(area,'mouseover',function(evt) {
						var key = evt.element().identify();
						key     = key.gsub('ID_AREA_','');
						var a_prms = thisObject.__a_areas[key];
						if (a_prms != undefined)
						{
							var o_obj  = a_prms[4];
							o_obj.MouseOver(a_prms);
						}
					});
					Event.observe(area,'click',function(evt) {
						var key = evt.element().identify();
						key     = key.gsub('ID_AREA_','');
						var a_prms = thisObject.__a_areas[key];
						if (a_prms != undefined)
						{
							var o_obj  = a_prms[4];
							o_obj.MouseDown(a_prms);
						}
					});
		
					//	Insert the area to the map
					map.insert({
						'bottom': area
					});
		
					// Increment the key
					//key++;
				};
			}
		});
	},
	
	/*
=pod

=head3 Reload

	Title: Reload
	Usage: obj.Reload();
	Procedure: reload the current page
	Args:      none

=cut
	*/
	Reload: function()
	{
		if (this._o_ajax_history != undefined){this._o_ajax_history.Reload();};
	},

	/*
=pod

=head3 IsFeatureOn, GetChangedFeatures, GetLevel

	Title: IsFeatureOn
	Usage: if (obj.IsFeatureOn('some_feature')) ...
	Function: Call the IsFeatureOn method on all tabs, until a "true" answer is found ((then return true) or until each tab is asked (then return false)
	          Call the GetLevel method on all tabs, until a GetLevel function is found
	Args:     A string (the feature to check)
	Return:   A boolean
    TODO ==> CHANGER CA
=cut
	*/
	IsFeatureOn: function(feature)
	{
		// Feature selection is valid ONLY for DisplayMap
		if (this.__script != "DISPLAY") { return true;};
		var rvl = false;
		this.__h_tabs.each(function(pair) {
				if(pair.value.IsFeatureOn(feature)){
					rvl = true;
					throw($break);
		}});
		return rvl;
	},
	GetChangedFeatures: function()
	{
		var rvl = 0;

		// May happen if a partial ajax request is sent before complete initialization
		// TODO - GetChangedFeatures should NOT be called during partial requests
		if (this.__h_tabs==undefined) {return $H();};
		this.__h_tabs.each(function(pair) {
				if(pair.value.GetChangedFeatures != undefined){
					rvl = pair.value.GetChangedFeatures();
					throw($break);
		}});
		return (rvl==0) ? $H() : rvl;
	},
	ClearChangedFeatures: function()
	{
		if (this.__h_tabs!=undefined){this.__h_tabs.each(function(pair) {if(pair.value.ClearChangedFeatures != undefined){pair.value.ClearChangedFeatures();}})};
	},
	GetLevel: function()
	{
		var level=0; // valeur par defaut totalement arbitraire
		this.__h_tabs.each(function(pair) {
				if(pair.value.GetLevel != undefined){
					level=pair.value.GetLevel();
					throw($break);
		}});
		return level;
	}
});

/*
=pod

=head2 class ControlBlockMemberClass

 Title          : ControlBlockMemberClass
 Usage          : ControlBlockMemberClass is an BASE CLASS: EVERY "tab" managed by a ControlBlockclass MUST derive from this class
                  However, this is NOT an abstract class, so that it is legal to instance a ControlBlockMemberClass
 Constructor arg: top_div     : a container for the html elements
                  controlblock: the controlblock you are a member of
 Member functions:
                UpdatePage, MAY BE overloaded - Update the displayed page from the parameters managed by the tab
 Globals      : none

=cut
*/

ControlBlockMemberClass = Class.create({
	initialize: function(top_div,controlblock)
	{
		this._top_div = $(top_div);
		this._controlblock = $(controlblock);
		this._controlblock.RegisterTab(this);
		//Event.observe(this.top_id,'click',function(evt){alert('coucou');};
	},

	/*
=pod

=head3 UpdatePage, IsFeatureOn

    Title: UpdatePage, IsFeatureOn
	Functions: Dummy functions (but NOT virtual), they may be overloaded by the derived classes

=cut
	*/
	UpdatePage: function(){},
	IsFeatureOn: function() {return false;},

	/*
=pod

=head3 GetKey

    Title: GetKey
	Function: return a unique key, it will be used by the controlblock to register this member

=cut
	*/
	GetKey: function() {
		var key = this._top_div.identify();
		key = key.gsub('ID_','');
		key = key.toLowerCase();
		return key;
	}
});

/*
=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license under French law - www.cecill.info

=cut
*/

