/*
 Copyright INRA/CNRS

 Emmanuel.Courcelle@toulouse.inra.fr
 Jerome.Gouzy@toulouse.inra.fr

 This software is a perl module whose purpose is to help you writing
 your own scripts

 This software is governed by the CeCILL license under French law and
 abiding by the rules of distribution of free software.  You can  use,
 modify and/ or redistribute the software under the terms of the CeCILL
 license as circulated by CEA, CNRS and INRIA at the following URL
 "http://www.cecill.info".

 As a counterpart to the access to the source code and  rights to copy,
 modify and redistribute granted by the license, users are provided only
 with a limited warranty  and the software's author,  the holder of the
 economic rights,  and the successive licensors  have only  limited
 liability.

 In this respect, the user's attention is drawn to the risks associated
 with loading,  using,  modifying and/or developing or reproducing the
 software by the user in light of its specific status of free software,
 that may mean  that it is complicated to manipulate,  and  that  also
 therefore means  that it is reserved for developers  and  experienced
 professionals having in-depth computer knowledge. Users are therefore
 encouraged to load and test the software's suitability as regards their
 requirements in conditions enabling the security of their systems and/or
 data to be ensured and,  more generally, to use and operate it in the
 same conditions as regards security.

 The fact that you are presently reading this means that you have had
 knowledge of the CeCILL license and that you accept its terms.

*/
/* 

=pod

=head1 NAME
 bookmarks_form.js - code related to the "Session" tab of controlblock

=head1 SYNOPSIS

 ControlBlockMemberClass        see controlblock.js
    BookmarksFormClass

=cut
*/


/*
=pod

=head2 BookmarksFormClass

   BookmarksFormClass ==> Connected to bookmarks_form.inc

   XHTML LAYOUT:
   =============
   <form> ==> this._controlblock.form
   <div>  ===> top_div
   	  <div ID_HIST>
	     <ul>     ==> history_ul
		   <li>   
		 </ul>
	  </div>

	  <div ID_BKMK>
         <h1><img (help system)><div (help)>
	     <input />
	     <input />               ==> btn_new
	     <input type="hidden" /> ==> bookmarks_list
         <ul>
	        <li>                 ==> first_li
	     </ul>
      </div>
	  <div ID_URL>
	  <div>
	     <input>                 ==> current_url
      </div>
      <div ID_SAVE>
	     <input>                 ==> btn_save
	  </div>
   </div>

=cut
*/

/*
=pod

=head2 BookmarksFormClass

 Title        : BookmarksFormClass
 Usage        : The behaviour associated to the controlblock tab "Session" (= bookmarks management)
 Constructor arg : top_div        The container
                   o_controlblock The controlblock we live in
				   script         The script parameter (DISPLAY, DOTPLOT, CIRCOS)

=cut
*/

BookmarksFormClass = Class.create(ControlBlockMemberClass,{
/*
   The constructor:
*/
	initialize: function($super,top_div,o_controlblock,script) 
	{
		$super(top_div,o_controlblock);
		if (script=='undefined') {script='DISPLAY'};

		// Initialize other private fields
		this.__hist        = $('ID_HIST');
		this.__bkm         = $('ID_BKM');
		//this.__url         = $('ID_URL');
		this.__save        = $('ID_SAVE');
		
		// The bookmarks area
		if (this.__bkm != null)
		{
			this.__bkm_new     = this.__bkm.down('input');
			this.__btn_new     = this.__bkm.down('input',1);
			this.__first_li    = this.__bkm.down('ul').down('li');

			// Observe the 'New bookmark' button
			// field_keypressed = will be called through an event (kbd)
			function bkm_keypressed(evt)
			{
				var key = evt.which || evt.keyCode;
				if (key == Event.KEY_RETURN) 
				{
					evt.stop();
					this.__AddBookmark(evt);
				};
			}
		

			Event.observe(this.__btn_new,'click',this.__AddBookmark.bindAsEventListener(this));
			Event.observe(this.__bkm_new,'keypress',bkm_keypressed.bindAsEventListener(this));
			
			// If necessary, create and observe the other bookmarks
			var bookmarks_list_field = this._top_div.down('input',2);
			var bookmarks_list       = $F(bookmarks_list_field);
			var a_bkmks              = bookmarks_list.split(',');
			
			// for each bookmark, create a new li at the top of the list
			var thisBookmarksFormClass = this;
			a_bkmks.each(function (bk)
			{
				thisBookmarksFormClass.__AddBookmarkByName(bk);
			});

			// then remove the hidden field
			bookmarks_list_field.remove();
		}
	},
	
	/*
	procedure __AddBookmark
	
	Title        : AddBookmark
	Usage        : Called when the user clicks the New Bookmark button
	Procedure    : If the field is empty, display an alert
	               Submit a request to add a new bookmark
				   AddBookmarkLocal is automatically called when we successfully return from the ajax request
	Returns      : nothing
	Args         : field: the field id where the bookmark name should be
	Access       : private
	*/
	__AddBookmark: function(evt)
	{
		var name_bookmark = $F(this.__bkm_new);
		
		if ( name_bookmark.blank() )
		{
			alert("Please enter a name");
			return; 
		}
		
		// Remove special characters	
		var name_no_space = name_bookmark.replace(/ /g,"_");
		var name_no_space = name_no_space.replace(/#/g,"_");
		
		// Submit the cgi, processing the add_bookmark command
		var fields = 'add_bookmark=' + name_no_space + '&script=ADDBOOKMARK';
		this._controlblock.SubmitAjax( fields, undefined, this.__AddBookmarkPostAjax.bind(this) );
		
		// Remove the parameter
		//document.getElementById("HiddenFields").removeChild(document.getElementById("add_bookmark"));
	},

	/*
	procedure __AddBookmarkPostAjax
	
	Title        : AddBookmarkPostAjax
	Usage        : Called when we return from the Ajax request, after an AddBookmark
	Procedure    : Retrieve the name from the input field and call AddBookmarkByName
	Args         : none
	Access       : private
	*/
	__AddBookmarkPostAjax: function()
	{
		var name   = $F(this.__bkm_new);
		this.__AddBookmarkByName(name);
	},

	/*
	procedure __AddBookmarkByName
	
	Title        : AddBookmarkByName
	Usage        : AddBookmarkByName(name);
	Procedure    : Update the form, adding a new <li> entry to the bookmarks list
	Args         : name The new bookmark name
	Access       : private
	*/
	__AddBookmarkByName: function(name)
	{
		var html   = '<li><a class="menu" href="#">'+name+'</a></li>';
		new Insertion.Before(this.__first_li,html);
		
		// Mark the new node as the first li
		this.__first_li = this.__first_li.previous();
		
		// clear the "new marker" field
		this.__bkm_new.setValue('');
		
		// Observe the clicks
		var ctrl = this._controlblock;
		Event.observe(this.__first_li,'click',function(evt){ctrl.SubmitAjax('read_bookmark='+name)});
		
	}
});

/*
=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license under French law - www.cecill.info

=cut
*/


			
