/* 
  Copyright INRA/CNRS

 Emmanuel.Courcelle@toulouse.inra.fr
 Jerome.Gouzy@toulouse.inra.fr
 Thomas.Faraut@toulouse.inra.fr

 This software is a computer program whose purpose is to provide a
 web-based interface for analyzing the different levels of genome
 conservations.

 This software is governed by the CeCILL license under French law and
 abiding by the rules of distribution of free software.  You can  use,
 modify and/ or redistribute the software under the terms of the CeCILL
 license as circulated by CEA, CNRS and INRIA at the following URL
 "http://www.cecill.info".

 As a counterpart to the access to the source code and  rights to copy,
 modify and redistribute granted by the license, users are provided only
 with a limited warranty  and the software's author,  the holder of the
 economic rights,  and the successive licensors  have only  limited
 liability.

 In this respect, the user's attention is drawn to the risks associated
 with loading,  using,  modifying and/or developing or reproducing the
 software by the user in light of its specific status of free software,
 that may mean  that it is complicated to manipulate,  and  that  also
 therefore means  that it is reserved for developers  and  experienced
 professionals having in-depth computer knowledge. Users are therefore
 encouraged to load and test the software's suitability as regards their
 requirements in conditions enabling the security of their systems and/or
 data to be ensured and,  more generally, to use and operate it in the
 same conditions as regards security.

 The fact that you are presently reading this means that you have had
 knowledge of the CeCILL license and that you accept its terms
 
*/


/* 

=pod

=head1 NAME

 data_form.js - Code related to the "Data" tab of controlblock

=head1 SYNOPSIS

 ControlBlockMemberClass        see controlblock.js
    DataFormClass

=cut
*/



/*
=pod

=head2 DataFormclass

   DataFormClass ==> Connected to data_form.inc

   XHTML LAYOUT:
   =============
   <form> ==> this._controlblock.form
   <div ID_DATA>  ===> top_div
      <div ID_LEVEL>
      <img (help system)>
	  <img>
	  <label><input />0</label>
	  <label><input />1</label>
	  ...
	  <label><input />5</label>
	  </div>
	  <div ID_VLVL>
	  <label><input /></label>
	  <label><input /></label>
	  <label><input /></label>
      </div>
	  <div ID_UPL_DATA>
	  </div>
	  <div ID_ADD_DATA>
	     <table></table>
	     <div ID_TARGET_INPUT_CONTAINER>
	        <input id=subjt name=subjt />
		    <div ID_COMPLETION_CONTAINER>
	     </div>
	  </div>
   </div>

=cut
*/

var ID_COMPLETION_CONTAINER = 'ID_COMPLETION_CONTAINER';

/*

=pod

=head2 DataFormClass

 Title        : DataFormClass
 Usage        : The behaviour associated to the controlblock tab "Data"
 Constructor arg : top_div        The container
                   o_controlblock The controlblock we live in

=cut

*/
DataFormClass = Class.create(ControlBlockMemberClass,{
/*
   The constructor: 
*/
	initialize: function($super,top_div,o_controlblock) 
	{
		$super(top_div,o_controlblock);

		var ctl          = this._controlblock;
		this.__level_div = $('ID_LEVEL');
		this.__vlvl_div  = $('ID_VLVL');
		this.__radio     = ctl.GetForm()['level'];
		this.__radio1    = ctl.GetForm()['view_level'];
		this.__img_container = $('ID_IMG_CONTAINER');
		this.__wb_header1 = $('wb_header1');
		this.__wb_header2 = $('wb_header2');
		
		// The 6 Level radio buttons should be there
		// Declare the onchange and click handlers for the [0..SYNT_LEVEL_MAX] level radio buttons
		// Hide the radio buttons [SYNT_LEVEL_MAX..5] 
		// (We hide in fact the nearest 'Removable' ancester, hiding the label as well - See also InitForm)
		var ThisDataFormClass = this;
		
		// May be, the level area is NOT there
		if (this.__level_div != null)
		{
			var l                 = this.__level_div.down('input');
			var lvl_max = O_CFG['SYNT_LEVEL_MAX'];
			$R(0,5).each(function( v )
			{
				var c = 'level='+v;
				var t = 'level '+v;
				
				if (v <= lvl_max)
				{
					Event.observe(l,'click',
						function(evt)
						{
							// Check the value of view_level, it may be necessary to change this value too
							var view_level = GetRadioValue(ThisDataFormClass.__radio1);
							var c1;
							if (view_level >= v)
							{
								c1 = c + '&view_level='+v;
							}
							else
							{
								c1 = c;
							}
							ctl.SubmitAjax(c1,t);
						});
				}
				else
				{
					l.up('.Removable').hide();
				};
				
				var u = l.up();
				if (u.next()==undefined) {throw $break;};
				l = l.up().next().down();
			});
		}
		
		// May be, the view_level area is NOT there
		if (this.__vlvl_div != null)
		{
			l = this.__vlvl_div.down('input');
			this.__r0  = l;
			this.__rl0 = l.next('span');
			Event.observe(l,'click',function(evt){ctl.SubmitAjax('view_level=2','view level n-2');});
			l = l.up().next().down();
			
			this.__r1  = l;
			this.__rl1 = l.next('span');
			Event.observe(l,'click',function(evt){ctl.SubmitAjax('view_level=1','view level n-1');});
			l = l.up().next().down();

			this.__r2  = l;
			this.__rl2 = l.next('span');
			Event.observe(l,'click',function(evt){ctl.SubmitAjax('view_level=0','view level n');});
		}
	},	//initialize

	/*
	Title:     __ImgSizeSync
	Usage:     Called by UpdatePage
	Procedure: computes and set the ID_IMG_CONTAINER width, it depends on the number of targets
	           (useful ONLY with DISPLAY representation)
	Args:      none
	Globals: O_CFG
	*/
	__ImgSizeSync: function()
	{
		var ctl      = this._controlblock;
		var o_gensel = ctl.GetGensel();
		// TODO - pas bien fameux...
		var script = $F(ctl.GetForm()['script']);
		
		// We cannot adapt the size if o_gensel is undefined (devt time only)
		if (o_gensel==undefined){return;};

		if (script == 'DISPLAY')
		{
			var nb_targets = o_gensel.GetNbTargets();
			
			// Synchronize the width of ID_IMG_CONTAINER with the number of targets
			var width;
			if (nb_targets <=1)
			{
				width = 50 + 2 * O_CFG.SYNT_MAP_WIDTH;
			}
			else
			{
				width = 50 + (nb_targets+1) * O_CFG.SYNT_MAP_WIDTH;
			}
		}
		else
		{
			width = 950;
		}
		
		this.__img_container.setStyle({
				width: width+'px'
		});
		
		width += 200;
		this.__wb_header1.setStyle({
				width: width+'px'
		});
		this.__wb_header2.setStyle({
				width: width+'px'
		});	
	},
	
	/*
	Title:     __RadioSync
	Usage:     Called when one of the Synteny levels radio buttons is pressed, changes the label in front of the 
	           3 'element' radio buttons, disable some if necessary
	Args:      The radio button we clicked on
	
	*/
	__RadioSync: function()
	{
		// Nothing to sync if view level area not there
		if (this.__vlvl_div==null){return;};
		
		var level      = GetRadioValue(this.__radio);
		
		if (level>=2)
		{
			this.__rl0.update(level-2);
			this.__rl0.up().show();
		}
		else
		{
			this.__rl0.up().hide();;
		};
		
		if (level>=1)
		{
			this.__rl1.update(level-1);
			this.__rl1.up().show();
		}
		else
		{
			this.__rl1.up().hide();
		};
		
		this.__rl2.update(level);
		
	},
	
	GetLevel: function()
	{
		return GetRadioValue(this.__radio);
	},
	
	UpdatePage: function()
	{
		this.__RadioSync();
		this.__ImgSizeSync();
	}

});

/*
=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license under French law - www.cecill.info

=cut
*/

