/* 
  Copyright INRA/CNRS

 Emmanuel.Courcelle@toulouse.inra.fr
 Jerome.Gouzy@toulouse.inra.fr
 Thomas.Faraut@toulouse.inra.fr

 This software is a computer program whose purpose is to provide a
 web-based interface for analyzing the different levels of genome
 conservations.

 This software is governed by the CeCILL license under French law and
 abiding by the rules of distribution of free software.  You can  use,
 modify and/ or redistribute the software under the terms of the CeCILL
 license as circulated by CEA, CNRS and INRIA at the following URL
 "http://www.cecill.info".

 As a counterpart to the access to the source code and  rights to copy,
 modify and redistribute granted by the license, users are provided only
 with a limited warranty  and the software's author,  the holder of the
 economic rights,  and the successive licensors  have only  limited
 liability.

 In this respect, the user's attention is drawn to the risks associated
 with loading,  using,  modifying and/or developing or reproducing the
 software by the user in light of its specific status of free software,
 that may mean  that it is complicated to manipulate,  and  that  also
 therefore means  that it is reserved for developers  and  experienced
 professionals having in-depth computer knowledge. Users are therefore
 encouraged to load and test the software's suitability as regards their
 requirements in conditions enabling the security of their systems and/or
 data to be ensured and,  more generally, to use and operate it in the
 same conditions as regards security.

 The fact that you are presently reading this means that you have had
 knowledge of the CeCILL license and that you accept its terms
 
*/


/* 

=pod

=head1 NAME

 features_form.js - Code related to the "display" tab of controlblock (features-related)

=head1 SYNOPSIS

 ControlBlockMemberClass        see controlblock.js
    FeaturesFormClass

=cut
*/

/*
=pod

=head2 FeaturesFormClass

   FeaturesFormClass ==> Connected to features_form.inc

   XHTML LAYOUT:
   =============
   <form> ==> this._controlblock.form
   <div>  ===> top_div
      <h1><img (help system)><div (help)>
	  <label class="Removable">Cyto:<input type="checkbox" name="cyto" /><input type="hidden" name="ck_cyto" /><br /></label>
	  ...
   </div>
   
   NOTE - The ck_box must be JUST AFTER the monitored box

=cut
*/

/*

=pod

=head2 FeaturesFormClass

 Title        : FeaturesFormClass
 Usage        : The behaviour associated to the controlblock tab "Display"
 Constructor arg : top_div        The container
                   o_controlblock The controlblock we live in

=cut

*/
FeaturesFormClass = Class.create(ControlBlockMemberClass,{
/*
   The constructor: 
*/
	initialize: function($super,top_div,o_controlblock) 
	{
		$super(top_div,o_controlblock);

		var ctl = this._controlblock;
		
		// Collect all checkboxes of this div and keep them inside a_checkboxes
		this.__a_checkboxes = $A();
		this.__a_features   = $A();
		this.__h_changed_buttons = $H();
		var FeaturesFormClassThis = this;
		this._top_div.childElements().each(function(e)
		{
			var i=e.down('input');
			while(i!=undefined && i.type!="checkbox"){i=i.next('input');};
			if (i!=undefined){
				
				// fill the __a_feature array
				var f = i.name;
				FeaturesFormClassThis.__a_features.push(f);
				
				// observe the hidden field
				i=i.next();
				FeaturesFormClassThis.__a_checkboxes.push(i);
				Event.observe(i,'initform:change',FeaturesFormClassThis.__ToggleImage.bindAsEventListener(FeaturesFormClassThis));
			};
		});
	},
	/*
	Title    : __ToggleImage
	Usage    : Connected to the checkboxes's callbacks
	Procedure: show/hide the image corresponding to the feature handled by the checkbox 
	Args     : An event
	Access   : private
	
	*/
	__ToggleImage: function(evt)
	{
		var hid = evt.element();
		var index = this.__a_checkboxes.indexOf(hid);
		var f     = this.__a_features[index];
		this.__h_changed_buttons.set(hid.name,$F(hid));
		var id='ID_' + f.toUpperCase();
		if ($F(hid)=="1")
		{
			$(id).show();
		}
		else
		{
			$(id).hide();
		}
	},

	/*
=pod

=head2 UpdatePage

    Title: UpdatePage
    Usage: obj.UpdatePage()
    Procedure: For each feature in this.__a_features, display or hide, if possible, the corresponding html element, following the IsFeatureOn() return value
	           If the corresponding image is not found, disable the checkbox
    Args:     none
    Access:   Public

=cut
*/
	UpdatePage: function()
	{
		for (var i=0; i<this.__a_features.length; ++i)
		{
			var feature = this.__a_features[i];
			var cb      = this.__GetCheckBox(feature);
			//if (cb==undefined) {continue;}; // should NOT happen !

			var img = $('ID_' + feature.toUpperCase());
			if (img == undefined) {
				//cb.disable();
				//Form.Element.disable(cb.previous());
				cb.previous().disable();
				continue
			}
			else
			{
				//Form.Element.enable(cb.previous());
				cb.previous().enable();
			}
			// we do not use IsFeatureOn here, for optimization (do not call __GetCheckBox again)
			//if (this.IsFeatureOn(feature))
			if($F(cb)!="0")
			{
				img.show();
			}
			else
			{
				img.hide();
			}
		}
	},
	
	/*
=pod

=head2 IsFeatureOn

    Title: IsFeatureOn
    Usage: if (obj.IsFeatureOn('some_feature')) ...
    Function: return true or false, following the status of the requested feature. 
              A feature is the name of the checkboxes defined in the feature tab (see features_form.inc for the details)
    Args:     A string (the feature to check)
    Return:   A boolean
    Access:   Public
	
=cut	
	*/
	IsFeatureOn: function(feature)
	{
		// Is the feature known by this object ?
		var cb = this.__GetCheckBox(feature);
		if (cb!=undefined)
		{
			return $F(cb)!="0";
		}
		else
		{
			return false;
		}
	},
	
		/*
=pod

=head2 GetChangedFeatures

    Title: GetChangedFeatures
    Usage: var status =  obj.RetrieveChangedFeatures();
    Function: return the ck_XXX values, as a hash
              This function returns ONLY the features which have changed since last request
    Args:     Nothing
    Return:   A string
    Access:   Public
	
=cut	
	*/
	GetChangedFeatures: function()
	{
		return this.__h_changed_buttons;
	},
	ClearChangedFeatures: function() {this.__h_changed_buttons=$H();},
	
	/*
=pod

=head2 __GetCheckBox

    Title: __GetCheckBox
    Usage:    var cb = this.__GetCheckBox('some_feature');
    Function: return the hidden field just after the checkbox corresponding to the feature, or return undefined 
    Args:     A string (the feature to check)
    Return:   a checkbox or undefined
    Access:   Private
	
=cut	
	*/
	__GetCheckBox: function(feature)
	{
		var index = this.__a_features.indexOf(feature);
		if (index!=-1)
		{
			return this.__a_checkboxes[index];
		}
		else
		{
			return undefined;
		}
	}
});

/*
=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license under French law - www.cecill.info

=cut
*/

