/* 
  Copyright INRA/CNRS

 Emmanuel.Courcelle@toulouse.inra.fr
 Jerome.Gouzy@toulouse.inra.fr
 Thomas.Faraut@toulouse.inra.fr

 This software is a computer program whose purpose is to provide a
 web-based interface for analyzing the different levels of genome
 conservations.

 This software is governed by the CeCILL license under French law and
 abiding by the rules of distribution of free software.  You can  use,
 modify and/ or redistribute the software under the terms of the CeCILL
 license as circulated by CEA, CNRS and INRIA at the following URL
 "http://www.cecill.info".

 As a counterpart to the access to the source code and  rights to copy,
 modify and redistribute granted by the license, users are provided only
 with a limited warranty  and the software's author,  the holder of the
 economic rights,  and the successive licensors  have only  limited
 liability.

 In this respect, the user's attention is drawn to the risks associated
 with loading,  using,  modifying and/or developing or reproducing the
 software by the user in light of its specific status of free software,
 that may mean  that it is complicated to manipulate,  and  that  also
 therefore means  that it is reserved for developers  and  experienced
 professionals having in-depth computer knowledge. Users are therefore
 encouraged to load and test the software's suitability as regards their
 requirements in conditions enabling the security of their systems and/or
 data to be ensured and,  more generally, to use and operate it in the
 same conditions as regards security.

 The fact that you are presently reading this means that you have had
 knowledge of the CeCILL license and that you accept its terms
 
*/


/* 

=pod

=head1 NAME

 positions_form.js - Code related to the "Zoom" tab of controlblock

=head1 SYNOPSIS

 ControlBlockMemberClass        see controlblock.js
    PositionFormClass

=cut
*/

/*
=pod

=head2 PositionFormClass

   PositionFormClass ==> Connected to position_form.inc

   XHTML LAYOUT:
   =============
   <form> ==> this._controlblock.form
   <div>  ===> top_div
      <img (help system)>
	  <div (help system)>
	  <img>
	  <ul>
	     <li>
		 ...
      </ul>
	  <img>
      <div> ==> this.scroll_div
      <div> ==> this.goto_div
         <input type="text" />
	     <input type="button" />

=cut
*/

/*

=pod

=head2 PositionFormClass

 Title        : PositionFormClass
 Usage        : The behaviour associated to the controlblock tab "Zoom"
 Constructor arg : top_div        The container
                   o_controlblock The controlblock we live in

=cut

*/
PositionFormClass = Class.create(ControlBlockMemberClass,{
/*
   The constructor: 
*/
	initialize: function($super,top_div,o_controlblock)
	{
		$super(top_div,o_controlblock);
		
		this.__zoom_div     = $('ID_BROWSE_ZOOM');
		this.__scroll_div   = $('ID_BROWSE_SCROLL');
		this.__region_div    = $('ID_BROWSE_REGION');
		this.__goto_div     = $('ID_BROWSE_GOTO');
		this.__rng_start    = '';
		this.__rng_end      = '';
		this.__pos          = '';
		var ctl             = this._controlblock;

		// THE REGION div
		if ( this.__region_div != null)
		{
			this.__rng_start = TableGotoCell(this.__region_div.down('table'),0,1).down('input');
			this.__rng_end   = TableGotoCell(this.__region_div.down('table'),1,1).down('input');
			var b_view       = $('ID_B_VIEW');
			var b_extract    = $('ID_B_EXTRACT');
			var b_align      = $('ID_B_ALIGN');
			
			var ThisRegionDiv= this;
			// region_view_clicked = will be called through an event
			function region_view_clicked(evt) 
			{
				var rng_start_val = $F(ThisRegionDiv.__rng_start);
				var rng_end_val   = $F(ThisRegionDiv.__rng_end);
				var range = rng_start_val + ':' + rng_end_val;
				ctl.SubmitAjax('search=1&pos='+range,'manual range');
			};
			
			/* The VIEW BUTTON */
			// region_keypressed = will be called through an event (kbd)
			function region_view_keypressed(evt)
			{
				var key = evt.which || evt.keyCode;
				if (key == Event.KEY_RETURN) 
				{
					evt.stop();
					region_view_clicked(evt);
				};
			}
			// mouse interaction
			Event.observe(b_view,'click',region_view_clicked.bindAsEventListener(this));
			
			// keyboard shortcut
			Event.observe(this.__rng_start,'keypress',region_view_keypressed.bindAsEventListener(this));
			Event.observe(this.__rng_end,'keypress',region_view_keypressed.bindAsEventListener(this));
			
			/* The ALIGN BUTTON */
			// region_align_clicked = will be called through an event
			function region_align_clicked(evt) 
			{
				var rng_start_val = $F(ThisRegionDiv.__rng_start);
				var rng_end_val   = $F(ThisRegionDiv.__rng_end);
				var start = this.__GetValidePosition(rng_start_val);
				var end	  = this.__GetValidePosition(rng_end_val);
				if ( this.__IsRangeValide(start,end) == 1 )
				{
					var url = '&OnlineCompare=' + start + ':' + end;
					url += '&ck_hsp=1';
					
					//ctl.SubmitAjax(url,'online compare');
					ctl.SubmitAjax(url);
				}
			};
			// mouse interaction
			Event.observe(b_align,'click',region_align_clicked.bindAsEventListener(this));
			
			/* The EXTRACT BUTTON */
			// region_extract_clicked = will be called through an event
			function region_extract_clicked(evt) 
			{
				var header = '<h1>R E G I O N' + '<br/>' + $F(this.__rng_start) + ':' + $F(this.__rng_end) + '</h1>';
				var body   = '<h2>EXTRACT</h2>' + ThisRegionDiv.__GetExtractionMenu();
				O_POPUP.Display(body,header);
			};
			// mouse interaction
			Event.observe(b_extract,'click',region_extract_clicked.bindAsEventListener(this));
			

		};
		
		// THE GOTO div
		if ( this.__goto_div != null )
		{
			var position = this.__goto_div.down('input');
			this.__pos   = position;
			var button   = position.next('input');
			
			// button_clicked = will be called through an event
			function button_clicked(evt) 
			{
				var pos = $F(position);
				if (pos.blank()) {pos='none';};
				ctl.SubmitAjax('search=1&pos=' + pos,'search '+pos);
			};
			
			// position_keypressed = will be called through an event (kbd)
			function position_keypressed(evt)
			{
				var key = evt.which || evt.keyCode;
				if (key == Event.KEY_RETURN) 
				{
					evt.stop();
					button_clicked(evt);
				};
			}
			
			// The GO! button (mouse interaction)
			Event.observe(button,'click',button_clicked.bindAsEventListener(this));
			
			// The input field (keyboard shortcut)
			Event.observe(position,'keypress',position_keypressed.bindAsEventListener(this));

		};

		// THE ZOOM BUTTONS
		if ( this.__zoom_div != null )
		{
			// + -
			var neg = this.__zoom_div.down('img',4);
			var pos = neg.next('img');
			Event.observe(neg,'mouseover',function(evt){Event.element(evt).src=O_CFG['HTTP_LOGO']+"/zoom/zoomNeg1.gif";});
			Event.observe(neg,'mouseout',function(evt){Event.element(evt).src=O_CFG['HTTP_LOGO']+"/zoom/zoomNeg0.gif";});
			Event.observe(neg,'click',function(evt){ctl.SubmitAjax('click=ZoomNeg','zoom out');});
			Event.observe(pos,'mouseover',function(evt){Event.element(evt).src=O_CFG['HTTP_LOGO']+"/zoom/zoomPos1.gif";});
			Event.observe(pos,'mouseout',function(evt){Event.element(evt).src=O_CFG['HTTP_LOGO']+"/zoom/zoomPos0.gif";});
			Event.observe(pos,'click',function(){ctl.SubmitAjax('click=ZoomPos','zoom in');});

			// Zoom levels: they are read from CFG, we order them and keep in private in a variable
			var a_zoom_level = $H(O_CFG['ZOOM_LEVEL']);
			var a_zoom_level_sorted=["","","","",""];
			a_zoom_level.each(function(z)
			{
				a_zoom_level_sorted[z.value.id] = z.key;
			});
			this.__a_zoom_levels = a_zoom_level_sorted;
			
			// Choose the images to draw
			var a_li_tags = this._top_div.down('ul').immediateDescendants();
			var i=0;
			var thisPositionForm = this;
			
			// we MUST use an each loop here, j MUST be a local variable
			a_li_tags.each(function(elt){
					var j = i;
					//Event.observe(elt,'click',function(evt){alert(j + ' click=zoom_level_' + thisPositionForm.__a_zoom_levels[j]);});
					Event.observe(elt,'click',function(evt){ctl.SubmitAjax('click=zoom_level_' + thisPositionForm.__a_zoom_levels[j],'to ' + thisPositionForm.__a_zoom_levels[j]);});
					// TODO through a css this would be probably better... but I don't know how to do this
					Event.observe(elt,'mouseover',function(evt){
							var img = elt.down('img');
							var src = img.src;
							thisPositionForm.__save_img_src = src;
							src = src.gsub('_Cursor_','_Text_');
							src = src.gsub('_Level_','_Text_');
							img.src = src;
					});
					Event.observe(elt,'mouseout',function(evt){
							var img = elt.down('img');
							if (thisPositionForm.__save_img_src!=undefined)
							{
								img.src = thisPositionForm.__save_img_src;
							}
					});
					i += 1;
			});
		};

		// THE SCROLL BUTTONS
		if ( this.__scroll_div != null )
		{
			var btn_up   = this.__scroll_div.down('img',2);
			var btn_down = this.__scroll_div.down('img',1);
			Event.observe(btn_up,'click',function(evt){ctl.SubmitAjax('click=ScrollUp','scroll up');});
			Event.observe(btn_down,'click',function(){ctl.SubmitAjax('click=ScrollDown','scroll down');});
		};
	},
	UpdatePage: function(){
		if (this.__zoom_div != null)
		{
			var zoom_level = '';
			// Is there a zoom_level specified through the parameter click ?
			var zoom_level = undefined;
			if (O_PARAM.click != undefined && O_PARAM['click'].startsWith('zoom_level_'))
			{
				zoom_level = O_PARAM['click'].gsub('zoom_level_','');
			}
			var a_li_tags = this._top_div.down('ul').immediateDescendants();
			for (var i=0; i<a_li_tags.length;++i)
			{
				var elt = a_li_tags[i];
				var img = elt.down('img');
				var src = img.src;
				if (this.__a_zoom_levels[i]==zoom_level)
				{
					img.src = src.gsub('_Level_','_Cursor_');
				}
				else
				{
					img.src = src.gsub('_Cursor_','_Level_');;
				}
			}
		};
		if ( O_PARAM.range != undefined )
		{
			var range = O_PARAM.range;
			var a_range = range.split(':');
			this.__rng_start.setValue(a_range[0]);
			this.__rng_end.setValue(a_range[1]);
			this.__pos.setValue('');
		}
	},
	__IsRangeValide: function(start,end)
	{
		var max_range = this.__GetValidePosition(O_CFG['GLINT_MAX_LENGTH']);
		var a_errors = new Array("Start and end should be correctly formated",
							"Start should be correctly formated",
							"End should be correctly formated",
							"Start must be lower than end",
							"The range of this comparison must be inferior to " + O_CFG['GLINT_MAX_LENGTH'] + ".");
		
		var error = -1;
		if (start == undefined && end == undefined)
			error = 0;
		else if (start == undefined || parseInt(start) < 0)
			error = 1;
		else if (end   == undefined || parseInt(end)   < 0) 
			error = 2;
		else if (parseInt(start) > parseInt(end))
			error = 3;
		else if ( (parseInt(end)-parseInt(start)) > max_range)
			error = 4;
					
		
		// If their is a problem, alert the user and return 0
		if (error >= 0)
		{
			var msg = a_errors[error];
			alert (msg);
			return 0;
		}
		// Else return 1
		else
			return 1;

	},
	__GetValidePosition: function(value)
	{
		// Test if value is defined
		if (value == undefined)
			return value;
		
		// Test the value with a regex :
		//   1) get the integer value
		//   2) get the units
		var match = value.match(/^(\d+)(Mb|mb|Kb|kb)?$/);

		// If this value is correctly set, return reformated value
		if ( match != null )
		{
			var integer = match[1];		
			var units   = match[2];			
			
			if (units == 'Mb' || units == 'mb')
				units = Math.pow(10,6);
			else if (units == 'Kb' || units == 'kb')
				units = Math.pow(10,3);
			else
				units = 1;
			
			return integer*units;
		}
		
		// Else return undefined.
		else 
			return undefined;
	},
	__GetExtractionMenu: function()
	{
		var base_url = O_CFG['HTTP_NARCISSE_CGI'] + "?__wb_session="+O_PARAM["__wb_session"];
		var start    = this.__GetValidePosition($F(this.__rng_start));
		var end      = this.__GetValidePosition($F(this.__rng_end));
		var seq_prms = GetReference() + ':' + GetChromReference() + ':' + start + ':' + end;
		var html = "<ul>";
		html += "<li><a class=\"WindowSequence\" href=\"" + base_url + "&amp;Textual=1&amp;__wba_popup=1\" > Get Biological objects </a></li>";
		html += "<li><a class=\"WindowSequence\" href=\"" + base_url + "&amp;__wba_popup=1&script=SENDSEQUENCES&GenomicSeq=" + seq_prms + "\">Get genomic sequence</a></li>";
		//html += "<li><a class=\"WindowSequence\" href=\"" + base_url + "&amp;__wba_popup=1&script=SENDSEQUENCES&GlintFish=" + seq_prms + "&ck_personal=1\" title=\"Get long-range PCR primers for Fish\" >Fish Probes</a></li>";
		html += "<li><a class=\"WindowSequence\" href=\"" + base_url + "&amp;__wba_popup=1&script=SENDSEQUENCES&LaunchGlintFish=" + seq_prms + "&ck_personal=1\" title=\"Get long-range PCR primers for Fish\" >Compute Fish Probes</a></li>";
		html += "</ul>";
		return html;
	}
});
/*
=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license under French law - www.cecill.info

=cut
*/

