/* 
  Copyright INRA/CNRS

 Emmanuel.Courcelle@toulouse.inra.fr
 Jerome.Gouzy@toulouse.inra.fr
 Thomas.Faraut@toulouse.inra.fr

 This software is a computer program whose purpose is to provide a
 web-based interface for analyzing the different levels of genome
 conservations.

 This software is governed by the CeCILL license under French law and
 abiding by the rules of distribution of free software.  You can  use,
 modify and/ or redistribute the software under the terms of the CeCILL
 license as circulated by CEA, CNRS and INRIA at the following URL
 "http://www.cecill.info".

 As a counterpart to the access to the source code and  rights to copy,
 modify and redistribute granted by the license, users are provided only
 with a limited warranty  and the software's author,  the holder of the
 economic rights,  and the successive licensors  have only  limited
 liability.

 In this respect, the user's attention is drawn to the risks associated
 with loading,  using,  modifying and/or developing or reproducing the
 software by the user in light of its specific status of free software,
 that may mean  that it is complicated to manipulate,  and  that  also
 therefore means  that it is reserved for developers  and  experienced
 professionals having in-depth computer knowledge. Users are therefore
 encouraged to load and test the software's suitability as regards their
 requirements in conditions enabling the security of their systems and/or
 data to be ensured and,  more generally, to use and operate it in the
 same conditions as regards security.

 The fact that you are presently reading this means that you have had
 knowledge of the CeCILL license and that you accept its terms
 
*/


/* 

=pod

=head1 NAME

 gensel_form.js - Code related to the "Genome selection" form on the home page

=head1 SYNOPSIS

 NarcisseFormWithChromsClass        see narcisse.js
    GenselFormClass


=head2 GenselFormClass

   DotplotFormClass ==> Extends NarcisseFormClass
                        Connected to dotplot_form.inc
   XHTML LAYOUT:
   =============
   <div>                The top id
      <div></div>       The table container (the table is generated by the __Refresh method)
      <form>
	    <div>...</div>  Tools for adding data
		<div>...</div>  Tools for chromosome selection
		<div>...</div>  Submit buttons
		<p class="hidden"></p> Hidden fields, mainly used for default values
	  </form>     A form with some fields and tools
   </div>

=head2 GenselFormclass

 Title: GenselFormClass
 Usage: The behaviour of the genome selection tool
 Constructor Args:
    top_id       : The main container      
    o_data_source: the data source used for autocompletion (see YUI)
    o_exp [optional]         : an Expandable tool, used to show the genome selection tool 
	o_controlblock[optional] : A control block present in the same page, if present we use SubmitAjax to submit the commands
    Globals:   none

=cut
*/
	
GenselFormClass = Class.create({
	initialize: function(top_id,o_data_source,o_error,o_exp,o_controlblock)
	{
		this.__gensel_container  = $(top_id);

		this.__o_chrom_lists = {},

		// Do we live in a page with an 'ID_MAIN_CONTAINER' element ?
		this.__main_container    = $('ID_MAIN_CONTAINER');
		this.__o_exp             = o_exp
		this.__o_controlblock    = o_controlblock;
		this.__o_error           = o_error;

		// The table container and the two forms
		// MSIE - Better specifying a div around the table, then using replace to write the table (cf. __Refresh)

		this.__table_container  = this.__gensel_container.down('div');
		this.__table            = undefined;   // See the __Refresh method
		this.__form             = this.__gensel_container.down('form');
		
		// Create a LipmAjax object, only if no control block
		this.__ajax_form = (o_controlblock==undefined)?new LipmAjaxFormClass(this.__form,null):undefined;

		// Clear this flag
		this.__changed_ref = false;

		// The fields we need to address
		var add_data_div        = this.__form.down('div');
		var chrom_sel_div       = add_data_div.next('div');
		var submit_buttons      = chrom_sel_div.next('div');
		var a_buttons           = $A();

		this.__add_input        = add_data_div.down('input');
		
		this.__add_prog         = this.__form['program'];
		this.__add_button       = this.__form['add_species'];

		// The buttons
		this.__reset_button     = submit_buttons.down('input.display_type',0);
		var rst = this.__reset_button;
		$R(0,3).each(function(n){
			a_buttons.push(rst.next('input.display_type',n));
		});
		this.__a_repr_buttons = a_buttons;

		// the chromosome selection
		this.__chroms_selection_container = chrom_sel_div;
		this.__chroms_selection           = chrom_sel_div.down('select');
		this.__chroms_selection_btn       = chrom_sel_div.down('button');
		this.__chroms_selection_ttl       = chrom_sel_div.down('h1');
		
		// Create 1 object for autocompletion 
		var autocomplete = new YAHOO.widget.AutoComplete(this.__add_input.identify(), this.__add_input.next('div'), o_data_source);
		autocomplete.forceSelection = true;
		autocomplete.minQueryLength = 0; // pas besoin d'avoir une lettre pour lancer l'autocompletion
		Event.observe(this.__add_input,'focus',function(){
				if (autocomplete.isContainerOpen() == false){
					autocomplete.sendQuery('');
		}});
		
		// Some static handlers
		var thisGensel = this;
		Event.observe(this.__chroms_selection_btn,'click',this.__SelectChromMultipleHandler.bindAsEventListener(this));
		Event.observe(this.__add_button,'click',this.__AddDataHandler.bindAsEventListener(this));
		Event.observe(this.__reset_button,'click',function(evt){evt.stop();thisGensel.__selection=-1;thisGensel.__Refresh();
		});
		var i;
		for(i=0; i<this.__a_repr_buttons.length;i++)
		{
			var b = this.__a_repr_buttons[i];
			Event.observe(b,'click',thisGensel.__SubmitHandler.bindAsEventListener(thisGensel));
		}

		// Create several arrays for the data
		// The first elt (index 0) is related to the REFERENCE, the others to targetN
		// __InitfFromServer launches some asynchronous requests, then calls __Refresh itself 
		this.__InitFromServer();
	},
	GetNbTargets: function() { return (this.__selection<0)?0:this.__selection;},

	__GenselExpanded: function()
	{
		if(this.__main_container!=null) {this.__main_container.hide();}
		this.__Refresh();
	},
	__GenselCollapsed: function()
	{
		if(this.__main_container!=null) {this.__main_container.show();}
	},

	/*
=head2 __InitFromServer

 Title    : __InitFromServer
 Procedure: From the constructor, trigger an Ajax request to init the object from json data stored on the server 
            If there is nothing on the server, call __InitFromFields
 Args:      none
 Access:    private

=cut
   */
	__InitFromServer: function()
	{
		
		var params = "__wb_function=GetGenselData";
		var o_ajax = new LipmAjaxDataClass();
		var thisObject = this;
		o_ajax.SubmitAjax(params,__OnSuccess.bind(thisObject));
		
		function __OnSuccess()
		{
			var a_data = o_ajax.GetData();
		
			// No result: First start of Narcisse
			if (a_data==null)
			{
				this.__a_species        = $A();
				this.__a_programs       = $A();
				this.__a_chrom_single   = $A();
				this.__a_chrom_multiple = $A();
				this.__selection        = -1;
				this.__InitFromFields();
			}
			else
			{
				this.__a_species        = a_data[0];
				this.__a_programs       = a_data[1];
				this.__a_chrom_single   = a_data[2];
				this.__a_chrom_multiple = a_data[3];
				this.__selection        = a_data[4];
				//InitForm(this.__form, O_CFG.GUI_DEFAULTS);
			}
			this.__Refresh();
		}
		
	},

	/*
=head2 __InitFromFields

 Title    : __InitFromFields
 Procedure: From the constructor, init the private variables this.__a_species,this.__a_programs,this.__a_chrom_multiple,this.__selection from the fields 
            reference, target1, program1 or program,chroms_reference, chroms_target1 as read by O_PARAM
 Args:      none
 Access:    private

=cut
   */
	__InitFromFields: function()
	{
		// Init the radio buttons "program"	
		// InitForm(this.__ajax_form.GetForm(), O_CFG['GUI_DEFAULTS'] ); //<-- valeur par defaut
		var form      = this.__form;
		var script    = O_PARAM.script;

		// Init the hidden fields reference, target1, chroms_reference, chroms_target1: from parameters
		InitForm(form, O_PARAM);

		this.__selection=-1;
		var program  = GetValue(form.program);
		if (program == undefined )
		{
			program = 'auto';
		}
		if (this.__ValidSpeciesField('reference'))
		{
			//this.__a_species.push(GetLongName($F(form['reference'])));
			this.__a_species.push($F(form['reference']));
			this.__a_programs.push(program);
			if (script=='DISPLAY')
			{
				this.__a_chrom_single.push($F(form['chroms_reference']));
				this.__a_chrom_multiple.push('all');
			}
			else
			{
				this.__a_chrom_single.push('none');
				this.__a_chrom_multiple.push(($F(form['chroms_reference'])=='')? 'all' : $F(form['chroms_reference']));
			}
			this.__selection += 1;
			
			if (this.__ValidSpeciesField('target1'))
			{
				// this.__a_species.push(GetLongName($F(form['target1'])));
				this.__a_species.push($F(form['target1']));
				this.__a_programs.push(($F(form['program_target1'])=='')? program : $F(form['program_target1']));
				this.__a_chrom_single.push('none');
				this.__a_chrom_multiple.push(($F(form['chroms_target1'])=='')? 'all' : $F(form['chroms_target1']));
				this.__selection += 1;
			}
			
			// Init the other fields directly from O_PARAM (no default value, so not initialized by InitForm)
			var i = 2;
			while(ValidSpeciesParameter('target' + i))
			{		
				//this.__a_species.push(GetLongName(O_PARAM['target'+i]));
				this.__a_species.push(O_PARAM['target'+i]);
				this.__a_programs.push((O_PARAM['program_target'+i]==undefined)?program:O_PARAM['program_target'+i]);
				this.__a_chrom_single.push('none');
				this.__a_chrom_multiple.push((O_PARAM['chroms_target'+i]==undefined)?'all':O_PARAM['chroms_target'+i]);
				this.__selection=i;
				++i;
			}
		}
	},

	__ValidSpeciesField: function(field)
	{
		var form = this.__form;
		return ((form[field] != undefined) && ($F(form[field]) != "") && ($F(form[field]) != "none"));
	},

    /*
    =pod
	
    =head2 RefExchange
	
    Title:    RefExchange
    Usage:    url = o_gensel.RefExchange(N,chrom)
    Procedure: change the internal data, exchanging targetN with the reference - Set the single chrom selection the chrom
	           Return the corresponding url (ONLY reference, chrom_reference, targetN)
    Args: reference: The current reference - We check and execute the exchange ONLY if still synchronized
	      N        : If 0, nothing changed, if >0, exchanges targetN with the reference
	      chrom    : The new chrom reference
	
	*/
	RefExchange: function(reference,N,chrom) 
	{
		//if (N > this.__selection || GetShortName(reference) != GetShortName(this.__a_species[0]) )
		if (N > this.__selection || this.__a_species == undefined || reference != this.__a_species[0])
		{
			return;
		}
		if (N > 0)
		{
			var sp = this.__a_species[N];
			var cm = this.__a_chrom_multiple[N];
			var pg = this.__a_programs[N];
			
			this.__a_species[N]        = this.__a_species[0];
			this.__a_chrom_single[N]   = this.__a_chrom_single[0];
			this.__a_chrom_multiple[N] = this.__a_chrom_multiple[0];
			this.__a_programs[N]       = this.__a_programs[0];
			this.__a_species[0]        = sp;
			this.__a_chrom_multiple[0] = cm;
			this.__a_programs[0]       = pg;
		}
		this.__a_chrom_single[0]   = chrom;
	},

    /*
    =pod
	
    =head2 __UpdateChromsList
	
    Title:    __UpdateChromsList
    Usage:    bool = this.__UpdateChromsList( form,elt )
    Function:  Given an input text field element called 'something' and the form containing the list to update,
               this function updates the field called 'chroms_something', which must be a selection list, with the chromosomes list
    Args: name    The long name of the genome
		  sel_elt The select element we are updating
		  max_sze The max size of the element, after initialization
		  none_flg If set, the first option is 'none' - If cleared (default), the first option is 'all'
    Return:   true on success, false if no chroms was found for the org.
    Global :   none
	
	*/

	// Hack IE => Pour remplir correctement les champs d'une balise <select> il ne faut pas utiliser innerHTML, mais l'object javascript Option
	
	// o_chrom_lists keeps in memory the Ajax requests already done, minimizing the number of requests
	__UpdateChromsList: function( organism, sel_elt, max_sze, none_flg )
	{
		if (none_flg == undefined) {none_flg = 0;};
		var o_chrom_lists = this.__o_chrom_lists;
		
		// If necessary, we start an Ajax request to get the chrom list
		if (o_chrom_lists[organism] == undefined || o_chrom_lists[organism] == '')
		{
			var params = "__wb_function=AjaxGetChromList&org=" + organism;
			var o_ajax = new LipmAjaxDataClass();
			o_ajax.UnsetAsynchronous();
			o_ajax.SubmitAjax(params,function (){
					o_chrom_lists[organism]=o_ajax.GetData();
					return __UpdateElt();
			});
		}
		
		// no request, just call __UpdateElt
		else
		{
			return __UpdateElt();
		}
		
		// __UpdateElt: Update the element from the o_chrom_lists, which should be initialized			
		function __UpdateElt()
		{
			// Get json data : all chrs infos for this org.
			var json_data = o_chrom_lists[organism];
			if (json_data =="") { return false; };
			
			var chrs_list = $H( json_data );
			
			// Set the first option to 'all' or 'none' and select it.
			var form = this.__form;
			var opt1 = (none_flg==0) ? 'all' : 'none';
			sel_elt.options[0] = new Option(opt1,opt1,true,true);
			
			// Sort and format chrs_list items to option elements
			var count = 1;
			SortList(chrs_list.keys()).each( function( key )
				{
					sel_elt.options[count] = new Option(key,key);
					count++;
				});
			
			// Delete last options values which are in memory.
			while (sel_elt.options[count] != null)
			{
				sel_elt.options[count] = null;
			}
			
			// Set select's field attribute 'size'
			sel_elt.size = max_sze;
			//if( max_sze < chrs_list.size() )
			//	sel_elt.size = max_sze;
			//else
			//	sel_elt.size = chrs_list.size();
			
			return true;
		}
	},

	/*
=head2 __Refresh

 Title    : __Refresh
 Procedure: Clear and re-write the main table, using the internal data structures
 Args:      none
 Access:    private

=cut
   */

   __Refresh: function()
   {
	   var html='<table class="GenselTable">';
	   html += '<tr><th>&nbsp;</th><th>Role</th><th>species</th><th>chrom selection (single)</th><th>chrom selection (multiple)</th><th>&nbsp;</th></tr>';
	   
	   var sel_tool_ttl   = ' title="Select this species"';
	   var desel_tool_ttl = ' title="Deselect this species" checked="checked"';
	   for (var i=0; i < this.__a_species.length; ++i)
	   {
		   var selected = (i<=this.__selection) ? ' class="selected" ' : ' ';
		   html += '<tr' + selected + '>';
		   html += '<td><input type="checkbox"' + (i<=this.__selection ? desel_tool_ttl : sel_tool_ttl) + '" /></td>';
		   if (i<=this.__selection)
		   {
			   html += (i==0) ? '<td>reference</td>' : '<td>target' + i + '</td>';
		   }
		   else
		   {
			   html += '<td>&nbsp;</td>';
		   }
		   html += '<td>' + this.__a_species[i] + '</td>';
		   html += (i==0 && i <= this.__selection) ? '<td><select name="chroms_s_sel" size="1"><option></option></select></td>' : '<td>&nbsp;</td>';
		   html += '<td><img src="'+O_CFG.HTTP_LOGO+'/move/blank16.png" alt="plus" />';
		   html += (this.__a_chrom_multiple[i]=='') ? 'all' : this.__a_chrom_multiple[i];
		   html += '</td>';
		   html += (i<=this.__selection) ?'<td>&nbsp;</td>' : '<td><img src="'+O_CFG.HTTP_LOGO+'/move/trash.png" alt="recycler" title="remove this species" /></td>';
		   html += '</tr>';
	   }
	   html += '</table>';
	   this.__table_container.innerHTML = html;
	   this.__table = this.__table_container.down('table');
	   
	   // Create the ExpandableClass objects
	   for (var i=1; i <= this.__a_species.length; ++i)
	   {
		   var img_plus = this.__table.down('tr',i).down('td').next('td',3).down('img');
		   Event.observe(img_plus,'click',this.__TableExpHandler.bindAsEventListener(this));
		   new ExpandableClass(img_plus,this.__chroms_selection_container.identify());
		   //o_exp.AddExpandHook(this.__TableExpHandler.bindAsEventListener(this));
	   }
	   
	   // Update the select element and observe the change
	   var sel_elt = this.__table.down('select');
	   if (sel_elt != undefined)
	   {
		   var a_chrom_single = this.__a_chrom_single;
		   this.__UpdateChromsList(this.__a_species[0],sel_elt,1,1); // 1 line only, none_flg = 1
		   SetValue(sel_elt,a_chrom_single[0]);
		   Event.observe(sel_elt,'change',function(evt){
				   a_chrom_single[0] = $F(sel_elt);
		   });
	   }
	   
	   var thisGenesel = this;
	   // Observe the selection tools (col 0) and the recyclers (col 6, only the not-yet selected lines)
	   for (var row=1; row <= this.__a_species.length; ++row)
	   {
		   var cb;
		   cb = TableGotoCell(this.__table,row,0).down('input');
		   Event.observe(cb,'click',this.__SelecterHandler.bindAsEventListener(this));
		   if (row-1 > this.__selection)
		   {
			   var img = TableGotoCell(this.__table,row,5).down('img');
			   Event.observe(img,'click',this.__RecyclerHandler.bindAsEventListener(this));
		   }
	   }
	   
	   // Clear the 'Add' field
	   this.__add_input.clear();
	   
   },

   /*
=head2 __SubmitHandler

 Title:  __submitHandler
 Handler: called when you click a button to start some representation
 		  Detect if the data are correctly selected, and if OK send the request
   */ 
   __SubmitHandler: function(evt)
   {
	   evt.stop();
	   var repr = evt.element().value;
	   
	   this.Submit(repr.toUpperCase());
   },
   Submit: function(repr)
	{
	   //var session = GetValue(this.__ajax_form.GetForm()['__wb_session']);
	   
	   var url='';
	   var history_text='';
	   if (repr == 'DOTPLOT')
	   {
		   /* DOTPLOT: we should have 2 genomes selected */
		   if (this.__selection < 1)
		   {
			   alert('Please select TWO genomes before calling the Dotplot representation');
			   return;
		   }
		   else
		   {
			   if (this.__selection > 1)
			   {
				   alert('More than 2 genomes are selected, but only ' + this.__a_species[0] + ' and ' + this.__a_species[1] + ' will be used for this DOTPLOT');
			   }
			   url +=  'reference=' + this.__a_species[0] + '&target1=' + this.__a_species[1];
			   if (this.__a_programs[1] != '')       {url += '&program_target1=' + this.__a_programs[1];};
			   if (this.__a_chrom_multiple[0] != '') {url += '&chroms_reference=' + this.__a_chrom_multiple[0];};
			   if (this.__a_chrom_multiple[1] != '') {url += '&chroms_target1=' + this.__a_chrom_multiple[1];};
			   url += '&script=' + repr;
			   history_text = 'DOTPLOT ' + this.__a_species[0] + ' vs ' + this.__a_species[1];
		   }
	   }
	   
	   if (repr == 'DISPLAY')
	   {
		   /* DISPLAY: we should have at least 1 genome selected, and only 1 chromosome selected in the reference genome */
		   if (this.__selection == -1)
		   {
			   alert('Please select AT LEAST ONE genome before calling the DisplayMap representation');
			   return;
		   };
		   
		   /* But not more than DISPLAY_MAX_ORG */
		   if (this.__selection > O_CFG[repr+'_MAX_ORG'])
		   {
			   alert('Please select less than ' + O_CFG[repr+'_MAX_ORG'] + ' targets before calling the DisplayMap representation');
			   return;
		   }
		   if (this.__a_chrom_single[0] == "" || this.__a_chrom_single[0] == 'none')
		   {
			   alert('Please select one specific chromosome in the reference before calling the DisplayMap representation');
			   return;
		   }
		   
			   
		   url += 'reference=' + this.__a_species[0];
		   url += '&chroms_reference=' + this.__a_chrom_single[0];
		   for (var i=1; i<=this.__selection; ++i)
		   {
			   url += '&target' + i + '=' + this.__a_species[i];
			   url += '&program_target' + i + '=' + this.__a_programs[i];
		   }
		   for (var i=this.__selection+1; i<=O_CFG[repr+'_MAX_ORG']; ++i)
		   {
			   url += '&target' + i + '=none';
		   }
		   url += '&script=' + repr;
		   
		   // Remove the pos= parameter, if any
		   url += '&pos=none';
		   
		   // If the reference was changed, display the whole chromosome
		   if (this.__changed_ref == true)
		   {
			   url += '&click=zoom_level_chromosome';
			   this.__changed_ref = false;
		   }
		   history_text = 'DISPLAYMAP ' + this.__a_species[0] + ' ' + this.__a_chrom_single[0];
		   //alert(url);
	   }

	   if (repr == 'CIRCOS')
	   {
		   /* CIRCOS: we should have at least 2 genomes */
		   if (this.__selection <= 0)
		   {
			   alert('Please select at least TWO genomes before calling the Circos representation');
			   return;
		   }
		   url += 'reference=' + this.__a_species[0];
		   url += '&chroms_reference=' + this.__a_chrom_multiple[0];
		   for (var i=1; i<=this.__selection; ++i)
		   {
			   url += '&target' + i + '=' + this.__a_species[i];
			   url += '&program_target' + i + '=' + this.__a_programs[i];
			   if (this.__a_chrom_multiple[i] != '') {url += '&chroms_target'+i+'=' + this.__a_chrom_multiple[i];};
		   }
		   for (var i=this.__selection+1; i<=O_CFG[repr+'_MAX_ORG']; ++i)
		   {
			   url += '&target' + i + '=none';
		   }
		   url += '&script=' + repr;
		   history_text = 'CIRCOS ' + this.__a_species[0];
	   }
	   
	   if (repr == 'KARYOTYPE')
	   {
		   /* KARYOTYPE: we should have at least 1 genome selected */
		   if (this.__selection < 0)
		   {
			   alert('Please select ONE genome before calling the Karyotype representation');
			   return;
		   }
		   else
		   {
			   if (this.__selection > 0)
			   {
				   alert('More than 1 genome is selected, but only ' + this.__a_species[0] + ' will be used for this KARYOTYPE');
			   }
			   url +=  'reference=' + this.__a_species[0];
			   if (this.__a_chrom_multiple[0] != '') {url += '&chroms_reference=' + this.__a_chrom_multiple[0];};
			   url += '&script=' + repr;
			   history_text = 'KARYOTYPE ' + this.__a_species[0];
		   }
	   }

	   var a_data = [this.__a_species,this.__a_programs,this.__a_chrom_single,this.__a_chrom_multiple,this.__selection];
	   url += '&__json_A_GENSEL_DATA=';
	   url += Object.toJSON(a_data);
	   
	   // Submit only if the validation is OK
	   if (this.__Validate())
	   {
		   // If no control block known, just do a submit on the form	   
		   if (this.__o_controlblock == undefined)
		   {
			   // submit the url
			   /* Disable some fields, they will be ignored by the submit */
			   var form = this.__form;
			   ['target','chroms_multiple_selection'].each(function(f){form[f].disable();});
			   this.__ajax_form.Submit(url);
		   }
		   
		   // If there is a control block, call SubmitAjax
		   else
		   {
			   this.__o_controlblock.SubmitAjax(url,history_text);
		   }
		   
		   // Call the method Hideelt on this.__o_exp, this will close the tool
		   if (this.__o_exp != undefined)
		   {
			   this.__o_exp.HideElt();
		   }
	   }	   
   },

   	/*
	Title    : __Validate
    Usage    : Event handler
    Procedure: Starts an ajax request for validation
	           Analyze the result and display the errors if any
	Globals: O_CFG, O_PARAM
	*/
	
	__Validate: function()
	{

		// compute the parameters for the ajax request
		var params = '__wb_function=AjaxCheckSynt';
		if (this.__selection < 1) { return true };
		for (var i=0; i <= this.__selection; ++i)
		{
			if (i==0)
			{
				params += '&reference=' + this.__a_species[i];
			}
			else
			{
				params += '&target' + i + '=' + this.__a_species[i];
				params += '&program_target' + i + '=' + this.__a_programs[i];
			}
		}
		
		// Create a LipmAjaxDataclass object and submit the request
		var o_ajax = new LipmAjaxDataClass(this.__o_error);
		o_ajax.UnsetAsynchronous();
		var thisObject = this;
		o_ajax.SubmitAjax(params);
		var a_data = o_ajax.GetData();

		// display an error of submit the form
		return this.__AnalyzeData(a_data);
	},

	/*
	Title    : __AnalyzeData
	Usage    : __AnalyzeData(a_data);
	Procedure: analyze the a_data array
               If there is an error, display the message through the popup this.__o_error
               else, submit the form
    Access: private			   
    */
	__AnalyzeData: function(a_err)
	{
		var msg = '';
		var err_flg=0;
		//var ref=GetLongName(this.__a_species[0]);
		var ref=this.__a_species[0];
		for (var i=0; i < a_err.length; ++i)
		{
			if (a_err[i] !=0)
			{ 
				// The error comes from the reference
				if (i==0)
				{
					msg +='<p>reference data for <em>'+ ref + '</em> not available</p>';
					err_flg=1;
				}
				
				// The error comes from the parameter 'targetN'
				else
				{
					//var tgt = GetLongName(this.__a_species[i]);
					var tgt = this.__a_species[i];
					var prgm= this.__a_programs[i];
					msg += '<p>comparison <br /><em>'+ref+'</em><br />vs<br /><em>'+tgt+'</em><br />using method <em>' + prgm + '</em><br />not available</p>';
					err_flg = 1;
				}
			}
		};
		
		if (err_flg != 0)
		{
			this.__o_error.Display(msg,'DATA ERROR');
			return false;
		}
		else
		{
			this.__o_error.Close();
			return true;
		}
	},

   /*
=head2 __SelecterHandler

 Title:  __SelecterHandler
 Handler: called when you click a select tool inside some row
          Put the genome just after the selection list to expand the selection
   */ 
   __SelecterHandler: function(evt)
   {
	   // Compute the correct index
	   var thisGensel = this;
	   var o_coord = TableWhereAmI(this.__table,evt.element());
	   var index = o_coord.row-1;

	   // We are INSIDE the selection: shrink the selection
	   if (index <= this.__selection)
	   {
		   this.__selection -= 1;
		   // If still in the selection, reorder the corresponding cells of the tables
	   	   if (index <= this.__selection)
		   {
			   [this.__a_species,this.__a_programs,this.__a_chrom_single,this.__a_chrom_multiple].each(function(a_tbl){
					   var tmp = a_tbl[index];
					   a_tbl.splice(index,1);   // Delete at index
					   a_tbl.push(tmp);         // Insert at end of table
			   })
		   }
	   }
	   
	   // We are OUTSIDE the selection: expand the selection
	   else
	   {
		   this.__selection += 1;
		   // If still out of the selection, reorder the corresponding cells of the tables
	   	   if (index > this.__selection)
		   {
			   [this.__a_species,this.__a_programs,this.__a_chrom_single,this.__a_chrom_multiple].each(function(a_tbl){
					   var tmp = a_tbl[index];
					   a_tbl.splice(index,1);		                // Delete at index
					   a_tbl.splice(thisGensel.__selection,0,tmp);	// Insert at end of selection
			   })
		   }
	   }
		
	   // If index==0, we changed the reference
	   if (index == 0)
	   {
		   this.__changed_ref = true;
	   }
	   
	   // Refresh display
	   this.__Refresh();
   },

   /*
=head2 __RecyclerHandler

 Title:  __RecyclerHandler
 Handler: called when you click recycler inside some row
          Remove the genome and call Refresh again
   */ 
   __RecyclerHandler: function(evt)
   {
	   // Compute the correct index
	   var o_coord = TableWhereAmI(this.__table,evt.element());
	   var index = o_coord.row-1;

// no need to confirm as no data will be lost
/*	   // Ask the user for confirmation
	   if(confirm('Do you really want to remove ' + this.__a_species[index] + ' (' + this.__a_programs[index] + ') ?'))
	   {*/

		   // Remove the corresponding cell from the tables
		   [this.__a_species,this.__a_programs,this.__a_chrom_single,this.__a_chrom_multiple].each(function(a_tbl){
				   a_tbl.splice(index,1)
		   });
		   
		   // May be change the selection marker
		   if (index <= this.__selection) {this.__selection -= 1;};
		   // Refresh display
		   this.__Refresh();
// 	   }
   },
   
   /*
=head2 __AddDataHandler

 Title:  __AddDataHandler
 Handler: called when you click the add button
          Read the species and the program and append them to the data areas
   */ 
   __AddDataHandler: function(evt)
   {
	   // get the data from the fields
	   var species = GetValue(this.__add_input);
	   var program = '';
	   if (this.__add_prog != undefined)
	   {
		   program = GetValue(this.__add_prog);
	   }

	   if (species == '' ) {
		   alert('species not choosen. Please try again');
		   return;
	   };
	   
	   // If program not selected, we switch to 'auto': let the server to choose.
	   if ( program == undefined || program == '' ) {
		   program = 'auto';
	   };
	   
	   this.__a_species.push(species);
	   this.__a_programs.push(program);
	   this.__a_chrom_single.push('');
	   this.__a_chrom_multiple.push('all');
	   
	   // Refresh display
	   this.__Refresh();
   },

   /*
=head2 __TableExpHandler

 Title:  __TableExpHandler
 Handler: called when you click the + image inside the table
          The multiple chromosome selection tool is initialized
		  The private variable this.__current_row is set
   */
   __TableExpHandler: function(evt)
   {

	   // Compute the correct index and keep it inside this.__current_row
	   var o_coord = TableWhereAmI(this.__table,evt.element());
	   var index = o_coord.row-1;
	   this.__current_row = index;

	   // Update the chrom list
	   var spe = this.__a_species[index];
	   this.__UpdateChromsList(spe,this.__chroms_selection,5);

	   // Mark the already selected chromosomes
	   if (this.__a_chrom_multiple[index]!='')
	   {
		   this.__chroms_selection.setValue(this.__a_chrom_multiple[index].split(' '));
	   }
	   
	   // display the species
	   this.__chroms_selection_ttl.innerHTML = spe;
   },

   /*
=head2 __SelectChromMultipleHandler

 Title:  __SelectChromMultipleHandler
 Handler: called when you click the OK button for multiple chromosomes selection
   */
   __SelectChromMultipleHandler: function(evt)
   {
	   var i   = this.__current_row;
	   var sel = String(GetValue(this.__chroms_selection));

	   this.__a_chrom_multiple[i] = sel.gsub(',',' ');
	   this.__Refresh();
   }

});

/*
=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license under French law - www.cecill.info

=cut
*/

