/* 
  Copyright INRA/CNRS

 Emmanuel.Courcelle@toulouse.inra.fr
 Jerome.Gouzy@toulouse.inra.fr
 Thomas.Faraut@toulouse.inra.fr

 This software is a computer program whose purpose is to provide a
 web-based interface for analyzing the different levels of genome
 conservations.

 This software is governed by the CeCILL license under French law and
 abiding by the rules of distribution of free software.  You can  use,
 modify and/ or redistribute the software under the terms of the CeCILL
 license as circulated by CEA, CNRS and INRIA at the following URL
 "http://www.cecill.info".

 As a counterpart to the access to the source code and  rights to copy,
 modify and redistribute granted by the license, users are provided only
 with a limited warranty  and the software's author,  the holder of the
 economic rights,  and the successive licensors  have only  limited
 liability.

 In this respect, the user's attention is drawn to the risks associated
 with loading,  using,  modifying and/or developing or reproducing the
 software by the user in light of its specific status of free software,
 that may mean  that it is complicated to manipulate,  and  that  also
 therefore means  that it is reserved for developers  and  experienced
 professionals having in-depth computer knowledge. Users are therefore
 encouraged to load and test the software's suitability as regards their
 requirements in conditions enabling the security of their systems and/or
 data to be ensured and,  more generally, to use and operate it in the
 same conditions as regards security.

 The fact that you are presently reading this means that you have had
 knowledge of the CeCILL license and that you accept its terms
 
*/


/* 

=pod

=head1 NAME

 gensel_tiny_form.js - Code related to the light home page"

=head1 SYNOPSIS

 NarcisseFormWithChromsClass        see narcisse.js
    GenselTinyFormClass


=head2 GenselFormClass

   DotplotFormClass ==> Extends NarcisseFormClass
                        Connected to dotplot_form.inc
   XHTML LAYOUT:
   =============
   <div>                The top id
      <div></div>       The table container (the table is generated by the __Refresh method)
      <form>
	    <div>...</div>  Tools for adding data
		<div>...</div>  Tools for chromosome selection
		<div>...</div>  Submit buttons
		<p class="hidden"></p> Hidden fields, mainly used for default values
	  </form>     A form with some fields and tools
   </div>

=head2 GenselFormclass

 Title: GenselFormClass
 Usage: The behaviour of the genome selection tool
 Constructor Args:
    top_id       : The main container      
    o_data_source: the data source used for autocompletion (see YUI)
    o_exp [optional]         : an Expandable tool, used to show the genome selection tool 
	o_controlblock[optional] : A control block present in the same page, if present we use SubmitAjax to submit the commands
    Globals:   none

=cut
*/

GenselTinyFormClass = Class.create({
	initialize: function(top_id,o_data_source,o_error)
	{
		this.__gensel_container  = $(top_id);
		this.__gensel_form       = this.__gensel_container.down('form');
		this.__o_error           = o_error;


		// Create 2 objects for autocompletion 
		var a_input_fields = this.__gensel_form.getInputs('text');
		a_input_fields.each(function(f) {
			var auto = new YAHOO.widget.AutoComplete(f.identify(), f.next('div'), o_data_source);
			auto.forceSelection = true;
			auto.minQueryLength = 0; // pas besoin d'avoir une lettre pour lancer l'autocompletion
			Event.observe(f,'focus',function(){
				if (auto.isContainerOpen() == false){
					auto.sendQuery('');
			}});
		});
		
		// Create a LipmAjax object
		this.__o_ajax_form = new LipmAjaxFormClass(this.__gensel_form,null);

		// Connect the buttons to the submit handler
		var a_buttons = $$('input.display_type');
		for(i=0; i<a_buttons.length;i++)
		{
			var b = a_buttons[i];
			Event.observe(b,'click',this.__SubmitHandler.bindAsEventListener(this));
		}

		InitForm(this.__gensel_form,O_PARAM);

	},
	  	/*
	Title    : __Validate
    Usage    : Event handler
    Procedure: Starts an ajax request for validation
	           Analyze the result and display the errors if any
	Globals: O_CFG, O_PARAM
	*/
	
	__Validate: function()
	{

		// compute the parameters for the ajax request
		var params = '__wb_function=AjaxCheckSynt';
		
		// Retrieve the fields, and check their values
		var a_input_fields = this.__gensel_form.getInputs('text');
		
		// If some field is empty, it's easy to check
		var a_data;
		if ( a_input_fields[0].getValue() == '' || a_input_fields[1].getValue() == '')
		{
			a_data = [ 1, 1 ];
		}
		
		// Else, we ask the server about this comparison
		else
		{
			params += '&reference=' + a_input_fields[0].getValue();
			params += '&target1='   + a_input_fields[1].getValue();
			
			// Create a LipmAjaxDataclass object and submit the request
			var o_ajax = new LipmAjaxDataClass(this.__o_error);
			o_ajax.UnsetAsynchronous();
			var thisObject = this;
			o_ajax.SubmitAjax(params);
			a_data = o_ajax.GetData();
		}

		// display an error or submit the form
		return this.__AnalyzeData(a_data);
	},

	/*
	Title    : __AnalyzeData
	Usage    : __AnalyzeData(a_data);
	Procedure: analyze the a_data array
               If there is an error, display the message through the popup this.__o_error
               else, submit the form
    Access: private			   
    */
	__AnalyzeData: function(a_err)
	{
		var msg = '';
		var err_flg = a_err[0] || a_err[1];
		if (err_flg != 0)
		{
			msg = '<p>ERROR - Comparison data NOT available for those species</p>';
			this.__o_error.Display(msg,'DATA ERROR');
			return false;
		}
		else
		{
			this.__o_error.Close();
			return true;
		}
	},
	
   __SubmitHandler: function(evt)
   {
	   evt.stop();
	   var repr = evt.element().getValue();
	   
	   this.Submit(repr.toUpperCase());
   },
    Submit: function(repr)
	{
	   //var session = GetValue(this.__ajax_form.GetForm()['__wb_session']);
	   
		var a_input_fields = this.__gensel_form.getInputs('text');
		var url = 'script=' + repr;
		url += '&reference=' + a_input_fields[0].getValue();
		url += '&target1=' + a_input_fields[1].getValue();
	   
		if (this.__Validate())
		{
			this.__o_ajax_form.Submit(url);
		}
	}
});

/*
=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license under French law - www.cecill.info

=cut
*/

