/* 
  Copyright INRA/CNRS

 Emmanuel.Courcelle@toulouse.inra.fr
 Jerome.Gouzy@toulouse.inra.fr
 Thomas.Faraut@toulouse.inra.fr

 This software is a computer program whose purpose is to provide a
 web-based interface for analyzing the different levels of genome
 conservations.

 This software is governed by the CeCILL license under French law and
 abiding by the rules of distribution of free software.  You can  use,
 modify and/ or redistribute the software under the terms of the CeCILL
 license as circulated by CEA, CNRS and INRIA at the following URL
 "http://www.cecill.info".

 As a counterpart to the access to the source code and  rights to copy,
 modify and redistribute granted by the license, users are provided only
 with a limited warranty  and the software's author,  the holder of the
 economic rights,  and the successive licensors  have only  limited
 liability.

 In this respect, the user's attention is drawn to the risks associated
 with loading,  using,  modifying and/or developing or reproducing the
 software by the user in light of its specific status of free software,
 that may mean  that it is complicated to manipulate,  and  that  also
 therefore means  that it is reserved for developers  and  experienced
 professionals having in-depth computer knowledge. Users are therefore
 encouraged to load and test the software's suitability as regards their
 requirements in conditions enabling the security of their systems and/or
 data to be ensured and,  more generally, to use and operate it in the
 same conditions as regards security.

 The fact that you are presently reading this means that you have had
 knowledge of the CeCILL license and that you accept its terms
 
*/


/* 

=pod

=head1 NAME
 narcisse.js - Some objects or functions useful for the whole Narcisse application
 
 TODO : NarcisseFormClass et NarcisseFormWithChromsClass devraient plutot se trouver dans home.js


=cut
*/

/*
=head2 TableGotoCell

 Title    : TableGotoCell
 Usage    : var cell = TableGotoCell('ID_TABLE',3,4);
 Function : goto the cell whose coord are passed, in the table whose Id OR element is passed
            The upper-left cell is (0,0)
            WARNING ! THE TH ARE IGNORED BY THIS FUNCTION !
 Return   : The td element (extended by prototype), or undefined if not found
 Args:      table:   The table id or the table element
            row  :   The row
			col: The column

=cut
*/
function TableGotoCell(table,row,col)
{
	if (row<0 ||col<0) {return undefined;}
	var e_table = $(table);
	var e_row   = e_table.down('tr');
	var e_cell  = undefined;
	
	if (e_row != undefined && row>0)
	{
		e_row = e_row.next('tr',row-1);
	}
	if (e_row != undefined)
	{
		e_cell = e_row.down('td');
		if (e_cell != undefined && col > 0)
		{
			e_cell = e_cell.next('td',col-1);
		}
	}
	return e_cell;
}

/*
=head2 TableWhereAmI

 Title    : TableWhereAmI
 Usage    : var o_coord = TableWhereAmI('ID_TABLE',elt);
 Function : return the coordinates of the cell inside which elt lives
            WARNING ! THE TH ARE IGNORED BY THIS FUNCTION !
 Return   : the coordinate of the cell (an object with 2 properties: row, col)
 Args:      table: The table id or the table element
            elt  : The element (or its id) who wants tho know where he lives

=cut
*/
function TableWhereAmI(table,elt)
{
	var e_table = $(table);
	var e_cell  = $(elt);
	var e_row   = undefined;
	var row     = 0;
	var col     = 0;
	
	// find the 'td' and the 'tr' we live in
	if(e_cell.nodeType!=1 || e_cell.tagName.toLowerCase()!="td")
	{
		e_cell = e_cell.up('td');
		if (e_cell==undefined)
		{
			throw('ERROR - we do not live in a td');
		}
		e_row = e_cell.up('tr');
	}
	
	// count the previous columns
	while(true)
	{
		e_cell = e_cell.previous('td');
		if (e_cell==undefined){break;};
		col++;
	}
	
	// count the previous rows
	while(true)
	{
		e_row = e_row.previous('tr');
		if (e_row==undefined){break;};
		row++;
	}
	
	var o_coord = { 
		row: row, 
		col: col 
	};
	return o_coord;
}

/*
=pod

=head2 class NarcisseFormClass

 Title        : NarcisseformClass
 Prerequisite : A form element whose id is top_id+Form must exist in the page 
 Base Class   : You may define some objects which derive from NarcisseFormclass, the form elements will be automagically intialized
                from O_PARAM             (the parameters)

 Globals      : O_PARAM

=cut
*/

NarcisseFormClass = Class.create({
	initialize: function(top_id)
	{
		this.form = $(top_id+'_FORM');
		//this.gui_defaults = O_CFG['GUI_DEFAULTS'];
		//InitForm(this.form,this.gui_defaults);
		InitForm(this.form,O_PARAM);
	}
});

/*
=pod

=head2 PrettyDisplayPosition

 Title: PrettyDisplayPosition
 Usage:     pretty_pos = Prettydisplay(pos);
 Function:  display a genomic position in b, kb or mb
 Args:      pos The position in b
 return:    The position, pretty displayed (string)

=cut
*/

function PrettyDisplayPosition(pos)
{
	if (pos < 1000)
	{
		return pos+"";
	};
	if (pos < 1000000)
	{
		pos = Math.round(pos / 1000);
		return pos+'kb';
	}
	pos = Math.round(pos / 1000000);
	return pos+'mb';
}

/*
=pod

=head2 PrettyDisplaySpecies

 Title: PrettyDisplaySpecies
 Usage:     pretty_species = PrettydisplaySpecies( species );
 Function:  display a species name, follwoed by its assembly phase if different from PSEUDOMOLECULES
 Args:      species The species
 return:    The species, ready to display

=cut
*/

function PrettyDisplaySpecies ( sp )			
{
	if ( O_CFG.GENOME[sp].assembly_phase == 'PSEUDOMOLECULES' )
	{
		return sp;
	}
	else
	{
		return sp + ' <em>(' + O_CFG.GENOME[sp].assembly_phase + ')</em>';
	}
}

/*
=pod

=head2 GetReference,GetChromReference

 Title:    GetReference,GetChromReference
 Usage:    var ref = GetReference();
 Function: Returns the reference species
 Args:     none
 return:   The long name of the reference species
 globals:  O_PARAM

=cut
*/
function GetReference()
{
	return O_PARAM.reference;
}
function GetChromReference()
{
	return O_PARAM.chroms_reference;
}
/*
=pod

=head2 GetTargets

 Title:    GetTargets
 Usage:    var ref = GetTargets();
 Function: Returns an array with the target species
 Args:     N (optional): If specified (may be 0), limit the number of targets returned
 return:   The target species in an array
 globals:  O_PARAM

=cut
*/
function GetTargets(N)
{
	var imax=9;
	if (N!=undefined){ imax=N; };

	var a_tgt=$A();
	for (var i=1; i<=imax; i++)
	{
		var key = 'target' + i;
		if (ValidSpeciesParameter(key) == false) { return a_tgt; };
		a_tgt.push(O_PARAM[key]);
	}
	return a_tgt;
}		

/*
=pod

=head2 IsPopup

 Title:    IsPopup
 Usage:    if (IsPopup())...
 Function: Checks if we are executing inside a popup
 Args:     none
 return:   true/false

=cut
*/
function IsPopup()
{
	return ($('wb_popup') != null) ? true : false;
}

/*
=pod

=head2 IsPseudo

 Title:    IsPseudo
 Usage:    if (IsPseudo('homoe sapiens'))...
 Function: Checks the species whose long name is passed by parameters and return true if pseudomolecules
 Args:     a species name
 return:   true/false
 globals:  O_CFG

=cut
*/
function IsPseudo( species )
{
	if (O_CFG.GENOME[species]==undefined)
	{
		throw("ERROR - I do not know the species " + species);
	}
	return (O_CFG.GENOME[species].assembly_phase==undefined || O_CFG.GENOME[species].assembly_phase == 'PSEUDOMOLECULES');
}

/*
=pod

=head2 IsOnlyPseudo()

 Title:    IsOnlyPseudo(N)
 Usage:    if (IsOnlyPseudo())...
 Function: Checks the reference species and the targets
           Returns fals if at least one of them is NOT pseudomolecules
 Args:     N (optional): The number of targets to check
 return:   true/false
 globals:  none directly, O_CFG/O_PARAM through preceding functions

=cut
*/
function IsOnlyPseudo(N)
{
	if (!IsPseudo(GetReference())) { return false; };
	var rvl = true;
	GetTargets(N).each(function(s){
		if (!IsPseudo(s)) 
		{ 
			rvl = false;
			throw $break;
		}
	});
	return rvl;
}

		
/*
=pod

=head2 IsMsie6

 Title:     IsMsie6
 Usage:     if (IsMsie6()) ...

 Function:  Return true if the client is Msie6 or lower, false if the client is a modern browser (including MSIE7)
 Args:      none
 Return:    true if Msie6 or lower, false if correct browser

=cut
*/

function IsMsie6()
{
	//alert(navigator.appName+'#'+navigator.appVersion+'#'+navigator.appCodeName+'#'+navigator.appMinorVersion+"\n"+navigator.userAgent);
	var user_agent = navigator.userAgent
	var regex=new RegExp('MSIE *6');
	if (regex.test(user_agent))
	{
		return true;
	}
	else
	{
		return false;
	}
}
 
// 
// /*
// =pod
// 
// =head2 GetShortName
// 
 // Title:  GetShortName
 // Usage:  var name = GetShortName(name);
 // Args:   name: a species long or short name
 // Return: The short name
// 
 // Global :   O_CFG
 // 
// =cut
// 
// */
// /*
// function GetShortName(name)
// {
	// if (name == '') { return ''};
	// 
	// // If name is a key of O_CFG.GENOME, it is already a short name
	// if (O_CFG.GENOME[name] != undefined) {return name;};
// 
	// // Else, we must look for the long name and retrieve the short name
	// var sname= '';
	// $H(O_CFG.GENOME).keys().each( function( key )
	// {
		// var h_orga = O_CFG.GENOME[key];
		// if (h_orga.long == name)
		// {
			// sname = key;
			// throw $break;
		// }
	// });
// 
	// if (sname != '')
	// {
		// return sname;
	// }
	// else
	// {
		// throw ("ERROR - GetShortName cannot find species " + name);
	// }
// }
// */
// /*
// =pod
// 
// =head2 GetLongName
// 
 // Title:  GetLongName
 // Usage:  var name = GetLongName(name);
 // Args:   name: a species long or short name
 // Return: The long name
// 
 // Global :   O_CFG
 // 
// =cut
// 
// */
// 
// function GetLongName(name)
// {
	// if (name == '') {return '';};
	// 
	// // If name is a key of O_CFG.GENOME, it is a LONG name
	// if (O_CFG.GENOME[name] != undefined) {return O_CFG.GENOME[name].long;};
// 
	// // Else, we must look for the short name and retrieve it as soon as found
	// var long = '';
	// $H(O_CFG.GENOME).keys().each( function( key )
	// {
		// var h_orga = O_CFG.GENOME[key];
		// if (h_orga.short == name)
		// {
			// long = h_orga.long;
		// }
	// });
	// 
	// if (long != '') {return long;};
	// 
	// // not found
	// throw ("ERROR - GetLongName cannot find species " + name);
// }
// // function GetLongName(name)
// // {
	// // if (name == '') {return '';};
	// // 
	// // // If name is a key of O_CFG.GENOME, it is a short name
	// // if (O_CFG.GENOME[name] != undefined) {return O_CFG.GENOME[name].long;};
// // 
	// // // Else, we must look for the long name and retrieve it as soon as found
	// // var long = '';
	// // $H(O_CFG.GENOME).keys().each( function( key )
	// // {
		// // var h_orga = O_CFG.GENOME[key];
		// // if (h_orga.long == name)
		// // {
			// // long = name;
		// // }
	// // });
	// // 
	// // if (long != '') {return long;};
	// // 
	// // // not found
	// // throw ("ERROR - GetLongName cannot find species " + name);
// // }
// 

/*
=pod

=head2 ValidSpeciesParameter

 Title:  ValidSpeciesParameter
 Usage:  if (ValidSpeciesParameter(key)) ...
 Function: Check O_PARAM[key], returns false if undefined, or blank, or 'none'
 Args:   key: A key (must be reference or targetN to valid
 Return: A boolean
 Global : O_PARAM
 
=cut

*/
function ValidSpeciesParameter(key)
{
	return (O_PARAM[key] != undefined && O_PARAM[key] != "" && O_PARAM[key] != "none");
}

/*
=pod

=head2 GetSpeciesList

 Title:  GetSpeciesList
 Usage:  var a_species = GetSpeciesList();
 Function: return an array containing the list of species
           The species whose name ends with 'NEW UPGRADE' are excluded, unless the parameter UPGRADING was specified
 Args:   none
 Return: a_species, an array of string

 Global : O_CFG,O_PARAM
 
=cut

*/

function GetSpeciesList()
{
	var a_species = $H(O_CFG.GENOME).keys().sort();
	if ( typeof (O_PARAM) != 'undefined' && (O_PARAM.UPGRADING!=undefined) )
	{
		return a_species;
	}
	else
	{
		var a_prod_species = $A();
		for (var i=0; i<a_species.length;++i)
		{
			if (a_species[i].endsWith('NEW UPGRADE')==false)
			{
				a_prod_species.push(a_species[i]);
			}
		}
		return a_prod_species;
	}
}


/*
=pod

=head2 SortList

 Title:     SortList
 Usage:     array = SortList( list )
 Prerequiste : none
 Function:  Sort an heterogeneous list (integers + strings), inserting integers before strings
 Args:      list => list to sort
 Return:    sorted list (integers,chars)
 Global :   none

=cut

*/
function SortList (list)
{
	var list_int = new Array();
	var list_char = new Array();
	
	// Separates integers and chars
	var model = /^\d*$/;
	list.each( function(elt) 
	{
		var match = elt.match(model);
		if (match != null)
			list_int.push(match[0]);
		else 
			list_char.push(elt);
	});

	// Sort integers => use only sort() function doesn't well sort our integer list
	list_int.sort( function(a,b)
	{
		return a-b;
	});
	
	// Sort chars and return concatened list.
	return list_int.concat(list_char.sort());
}

/*
=pod

=head2 GlobalKeyPressed

 Title:     GlobalKeyPressed
 Usage:     GlobalKeyPressed(evt)
 Procedure: Load handler for global keyboard shortcuts
            
 Args:      evt: the event
 Global :   none

=cut

*/
function GlobalKeyPressed(evt)
{
	var key = evt.which || evt.keyCode;
	if (key == Event.KEY_ESC) 
	{
		evt.stop();
		//var a_popups = $$('.yui-panel-container');
		var a_popups = YAHOO.util.Dom.getElementsByClassName('yui-module');
		for (var i=0; i < a_popups.length; i++)
		{
			//YAHOO.util.Dom.setStyle(a_popups, 'Display', 'none');
			a_popups[i].hide();
		}
	};
}	

/*
=pod

=head2 Reload

 Title:     Reload
 Usage:     Reload();
 procedure: Reload the current page, using O_CONTROL_BLOCK reload method, or the window.location.reload() method if O_CONTROL_BLOCK is not available
 Global :   none

=cut

*/
function Reload()
{
	if (O_CONTROL_BLOCK != undefined && O_CONTROL_BLOCK != 0)
	{
		O_CONTROL_BLOCK.Reload();
	}
	else
	{
		window.location.reload();
	}
}


/*
=head2 UpdateGenomeWithSessionData

 Usage    : UpdateGenomeWithSessionData()
 Procedure: Read Session directory to get user genomes
            then update O_CFG.GENOME to add those new genomes.
 Args:      none

=cut
*/

function UpdateGenomeWithSessionData()
{
	var params = "__wb_function=GetSessionGenomes";
	var o_ajax = new LipmAjaxDataClass();
	o_ajax.UnsetAsynchronous();
	o_ajax.SubmitAjax(params,function ()
	{
			var h_json = $H( o_ajax.GetData() );
			h_json.each( function( genome )
			{
				O_CFG.GENOME[ genome.key ] = genome.value;
			} );
	} );
}


/*
=pod

=head2 Load

 Title:     Load
 Usage:     Load hander
            We observe the click event of any element belonging to wb_logo, wb_main_a_inactive (WebBuilder) or ExtLinks (other links), to open the url in a new window
			We observe the window's keyboard event, to close any container belonging to yui-panel-container
			For each element belonging to the class img_help, we create a Help object
 Global :   O_CFG

=cut

*/
function Load()
{
	var o_new_win_logos = new LipmPopupNewWindowClass('wb_logo','',true);
	o_new_win_logos.Observe();
	var o_new_menus_items = new LipmPopupNewWindowClass('wb_main_a_inactive','',true);
	o_new_menus_items.Observe();
	var o_new_extlinks = new LipmPopupNewWindowClass('ExtLinks','',true);
	o_new_extlinks.Observe();
	
	// This is useful here to be able to use WindowSequence windows from outsid NcView.pm (NcAccess.pm for example)
	var o_new_window_seq= new LipmPopupNewWindowClass('WindowSequence',{
		directories: false,
		location: false,
		menubar: false,
		toolbar: false,
		scrollbars: true,
		status: true,
		resizable: true,
		height: 500,
		width: 600
	});
	o_new_window_seq.Observe();

	// This is useful for the popup open when you press the More Info link from the PrintRelNotes page
	var o_new_moreinfo = new LipmPopupNewWindowClass('MoreInfo',{
				directories: false,
				location: false,
				menubar: false,
				toolbar: false,
				scrollbars: true,
				status: true,
				resizable: true,
				height: 500,
				width: 600
	});
	o_new_moreinfo.Observe();
	
	// Build the help objects when we have time
	__BuildHelp.defer();
	
	function __BuildHelp()
	{
		$$('.img_help').each(function(i) {
				var o_h = new HelpClass(i);
		});
	}
	//Event.observe(window,'keypress',GlobalKeyPressed);

	// MaJ de O_CFG.GENOME
	UpdateGenomeWithSessionData();
}

Event.observe(window,'load',Load);

/*
=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license under French law - www.cecill.info

=cut
*/

