/*
  Copyright INRA/CNRS

 Emmanuel.Courcelle@toulouse.inra.fr
 Jerome.Gouzy@toulouse.inra.fr
 Thomas.Faraut@toulouse.inra.fr

 This software is a computer program whose purpose is to provide a
 web-based interface for analyzing the different levels of genome
 conservations.

 This software is governed by the CeCILL license under French law and
 abiding by the rules of distribution of free software.  You can  use,
 modify and/ or redistribute the software under the terms of the CeCILL
 license as circulated by CEA, CNRS and INRIA at the following URL
 "http://www.cecill.info".

 As a counterpart to the access to the source code and  rights to copy,
 modify and redistribute granted by the license, users are provided only
 with a limited warranty  and the software's author,  the holder of the
 economic rights,  and the successive licensors  have only  limited
 liability.

 In this respect, the user's attention is drawn to the risks associated
 with loading,  using,  modifying and/or developing or reproducing the
 software by the user in light of its specific status of free software,
 that may mean  that it is complicated to manipulate,  and  that  also
 therefore means  that it is reserved for developers  and  experienced
 professionals having in-depth computer knowledge. Users are therefore
 encouraged to load and test the software's suitability as regards their
 requirements in conditions enabling the security of their systems and/or
 data to be ensured and,  more generally, to use and operate it in the
 same conditions as regards security.

 The fact that you are presently reading this means that you have had
 knowledge of the CeCILL license and that you accept its terms

*/
/*

=pod

=head1 NAME
 nc_knowledge.js - js for presenting knowledge data

=cut
*/

/*
=pod

=head1 SYNOPSIS

 KnowledgeClass

=cut
*/

/*
=pod

=head2 KnowledgeClass

 Title:     KnowledgeClass
 Usage:     var o_know=new KnowledgeClass();
 Constructor Args: none
 Globals:   none

=cut
*/

KnowledgeClass = Class.create( {
	initialize: function()
	{
		// 1- source de donnee
		data_source               =new YAHOO.util.DataSource($('ID_KNOWLEDGE'));
		data_source.responseType  = YAHOO.util.DataSource.TYPE_HTMLTABLE;
		data_source.responseSchema= {
		       fields     : [ //"selection",
		                      "effector", "pattern", "reference",
		                      "relationship", "confidence", "id",
		                      "factor", "source" ]
			};

		// 2- la description des colonnes
		var cols = [ //{ key: "selection",  },
		             { key: "effector", sortable: true },
		             { key: "pattern", label: "condition", sortable: true },
		             { key: "reference", sortable: true },
		             { key: "relationship", sortable: true },
		             { key: "confidence", parser:"number", sortable: true },
		             { key: "id", sortable: true },
		             { key: "factor", parser:"number", sortable: true },
		             { key: "source", sortable: true }
		           ];

		// 2- le tableau
		this._knowledge_dt = new YAHOO.widget.DataTable( 'ID_DT_KNOWLEDGE',
		                                                   cols, data_source );

		// 3- la selection des lignes
		this._knowledge_dt.subscribe( 'rowClickEvent', this._knowledge_dt.onEventSelectRow );
	},

	GetSelectedRows: function()
	{
		return this._knowledge_dt.getSelectedRows();
	}

} );


MAIN:
{
	var o_knowledge;

	function Load()
	{
		o_knowledge = new KnowledgeClass();

		// l'elt correspondant au lien
		var elt = $('ID_DT_KNOWLEDGE');
		if (elt != null)
		{
			elt = $('ID_DT_KNOWLEDGE').next( 'p' ).down( 'a' );
			// je ne sais pas si this a sa place ici
			Event.observe( elt, 'click', ReloadMotherAndCloseThisOne.bind( this, o_knowledge ) );
		}
	};

/*
	Title       : ReloadMotherAndCloseThisOne
	Usage       : ReloadMotherAndCloseThisOne.bind( this, o_knowledge )
	Prerequiste : none
	procedure   : Gets selected rows of o_knowledge, makes line number a string,
	              and submits an ajax requests with mother control_block object.
	Args        : o_knowledge, a KnowledgeClass object.
	Global      : none ( o_mum.O_CONTROL_BLOCK ? )

*/
	function ReloadMotherAndCloseThisOne( o_know )
	{
		var a_list = o_know.GetSelectedRows();

		// je ne sais pas comment obtenir seulement les infos voulus
		// getTrEl ?
		// list contient une liste de string de la forme 'yui-rec$i'
		// ou $i est le numero de la ligne ( ordre ecrit dans le html )
		for( var i=0; i < a_list.length; ++i )
		{
			a_list[i] = a_list[i].substring( 'yui-rec'.length );
		}

		var o_mum = window.opener;
		var o_mum_controlblock = o_mum.O_CONTROL_BLOCK;

		var list_val = ( 0 == a_list.length ) ? 'none' : a_list.join('_');
		var args = 'knowledge_list=' + list_val ;
		args += '&species=' + O_PARAM.species;
		args += '&chrom='   + O_PARAM.chrom;
		o_mum_controlblock.SubmitAjax( args );

		window.close();
	};

	Event.observe(window,'load',Load);
}
