#!/usr/bin/perl

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod
=head1 NAME

 narcisse2.cgi -

=head1 SYNOPSIS

	# to access multi-instance home
	narcisse2.cgi

	# to access ...

=head1 DESCRIPTION

 - should be called by the web server,
 	but it can also be called (for debugging) from the cgi-bin directory:
 	`pwd`/narcisse.cgi [some_param=some_value]

 This script is the ONLY entry point of the Narcisse's gui 2.0.
 this script build a WebBuilder throught a factory regarding each case.
 Then it display the web page of this Web-Builder object.

=cut


use strict;
use warnings;
use diagnostics;

use IO::File;
use File::Copy;
use File::Spec;
use Data::Dumper;
use Carp;
use CGI qw(:standard);

$main::FAST_CGI = 0;

# NOTE If you DO NOT USE fast cgi, it may be more clever to comment out the two following lines
eval("use CGI::Fast qw(:standard)");
$main::FAST_CGI = 1 if ($@ eq "");

use Cwd;

BEGIN
{
		use File::Basename;
		my $dirprg=dirname($0);
		unshift @INC, "$dirprg/../Core/lib";
		unshift @INC, "$dirprg/../Core/API";
}

use constant DEBUG => 1;

#use Unix;

use Error qw(:try);
use JSON;
use ParamParser;
use WebBuilder;
use cgilib::WebBuilderFactory;
use cgilib::NcScriptFactory;
use File::Path qw( mkpath rmtree );
use LipmError;
use LipmError::ParameterException;
use LipmError::ObjectException;
use NarcisseError;
use NarcisseServiceFactory;
use NARCISSE_utils;
use Logger;
use Sequence_utils;
use General qw(:DEFAULT ShowDebug);

use NarcisseServiceFactory;
use NarcisseServiceKeywords qw ( &NS_LOCUS &NS_INFOS &NS_FASTA );
use NarcisseDataWriter::OnMemory;
use NarcisseService::GrepBased::GetLocusTagsFromKeywords;

use ConfigurationFactory;
use Configuration::InstanceObserver;

my $VERSION = do {my @r = (q$Rev: 3450 $ =~ /\d+/g); $r[0]};
my $O_LOGGER;

# POUR DEV ONLY (detection des cycles qui produisent des fuites de memoire !
#use Devel::Cycle;


# This only changes the error messages which SHOULD NOT happen
# In production mode, you have just a fine message telling you to restart Narcisse
# Outside production mode, you have an awfull red screen with a lot of details very useful for the developers
use constant PRODUCTION => &TRUE;
#use constant PRODUCTION => &FALSE;

#-------------------------------------------------------------------------------
#           Arguments du programme
#-------------------------------------------------------------------------------

##################################################
##########    Programme principal   ##############
##################################################

#
# MAIN ERROR ROUTINE
#
# If there is an error so that we cannot start at all, we print the message in a tiny html page.
#my $warn_handler = sub { print STDERR @_ };
#my $die_handler = sub { print STDERR @_ unless $^S };


=head2 THE MAIN BLOCK

    If we are in fast cgi mode, we continuously read the socket and call TryMainNarcisse
    Else, we just call TryMainNarcisse one time, then we exit

=cut


MAIN:
{
    # If the Fastcgi modules are not available, we cannot use fast_cgi
    if ($main::FAST_CGI == 0)
    {
        &TryMainNarcisse();
    }

    # If they are available, we may try to use them (not sure fast cgi is installed in apache)
    else
    {
		my $cpt = 0;
        while (my $cgi = new CGI::Fast)
        {
            &TryMainNarcisse($cgi);

            # If the file has changed (very often at devt time), the process will be restarted by apache
            exit if (! exists $ENV{SCRIPT_FILENAME});
            exit if (-M $ENV{SCRIPT_FILENAME}) < 0;
            
            # After 10 executions, we exit: this is a ***VERY DIRTY TRICK*** useful because there are
            # HUGE MEMORY LEAKS in our code. This is probably due to circular references...
            exit if ($cpt++ >= 10);
        }
    }
} # MAIN


=head2 Procedure TryMainNarcisse

	Title      : TryMainNarcisse
	Usage      : &TryMainNarcisse()
	Procedure  : Call the Main() function from a try bloc
                 If some exception is catched, display the exception to the browser
    Args       : $cgi (optional) If available, it is passed to ParamParser

=cut
sub TryMainNarcisse
{
    my ($cgi) = (@_);

    # to get the stacktrace in case of error
    $Error::Debug = 1;
    my $debugging = 0;
    $debugging += param('DEBUGGING') if (defined param('DEBUGGING'));

    # Are we in REST mode ?
    my $rest = 0;
    $rest += param('__wb_rest') if (defined param('__wb_rest'));

#Main($cgi,$debugging);return;

    try
    {
        Main($cgi,$debugging);
    }
    catch LipmError with
    {
        my ( $o_error ) = @_;

        my $msg        = $o_error->Explain();
        &_PrintError($msg,$o_error,$debugging,$rest);
    }
    otherwise
    {
        my ( $o_error, $r ) = @_;
        &_PrintError("Error ",$o_error,$debugging,$rest);
    }
    catch NarcisseError with
    {
        my ( $o_error ) = @_;

        my $hd = $o_error->hd;
        my $bd = $o_error->bd;
        my $ft = $o_error->ft;
        if ($ft eq "")
        {
            $ft = "You may try the following:<ol>";
            $ft .= "<li>click the <a href=\"javascript:O_CONTROL_BLOCK.Reload()\">Refresh icon</a> to refresh the display</li>";
            $ft .= "<li>If this does not work, <a href=\"" . url() . "\" >click here</a> to restart Narcisse</li>";
            $ft .= "</ol>";
        }
        _PrintNarcisseError($hd,$bd,$ft,$rest);
    };

    sub _PrintNarcisseError
    {
        my ($hd,$bd,$ft,$rest) = (@_);
        if ( $rest != &FALSE )
        {
            my %h_error;
            $h_error{type} = 'NarcisseError';
            $h_error{hd}   = $hd;
            $h_error{bd}   = $bd;
            $h_error{ft}   = $ft;
            my %h_rvl;
            $h_rvl{error}        = \%h_error;
            $h_rvl{__wb_session} = param('__wb_session');
            my $json = new JSON;
            print "Content-Type: text/html; charset=utf-8\n\n".$json->encode(\%h_rvl);
            return;
        }
        my $error_html = "Content-Type: text/html; charset=utf-8\n\n<html><head><title>ERROR</title></head>\n";
        $error_html .= "<style type=\"text/css\">";
        $error_html .= "#ID_ERROR {margin: 3em auto; padding:1em; width: 50%;border:blue solid 3px;background-color:skyblue;color:red}";
        $error_html .= ".hd,.ft {font-weight:bold;text-align:center;margin:1em}";
        $error_html .= ".ft {font-size: .8em}";
        $error_html .= ".bd {font-weight: normal; text-align: justify}";
        $error_html .= "#ID_ERROR table,#ID_ERROR td,#ID_ERROR th {border:solid black 1px;border-collapse:collapse;text-align:left;margin: 1em auto}</style>";
        $error_html .= "<div id=\"ID_ERROR\"><div class=\"hd\">$hd</div><div class=\"bd\">$bd</div><div class=\"ft\">$ft</div></div>";
        $error_html .= "</body></html>\n";
        print $error_html;
    }

    sub _PrintError
    {
        my ($msg,$o_error,$debugging,$rest) = (@_);
        if ( $rest == &TRUE )
        {
            my %h_error;
            $h_error{type}       = 'UnexpectedError';
            $h_error{file}       = $o_error->{-file};
            $h_error{line}       = $o_error->{-line};
           # $h_error{stacktrace} = $o_error->stacktrace();
            $h_error{message}    = $msg;
            my %h_rvl;
            $h_rvl{error} = \%h_error;
            $h_rvl{__wb_session} = param('__wb_session');
            my $json = new JSON;
            print "Content-Type: text/html; charset=utf-8\n\n".$json->encode(\%h_rvl);
            return;
        }
        if ( &PRODUCTION == &FALSE || $debugging != &FALSE )
        {
            my $error_html = "Content-Type: text/html; charset=utf-8\n\n<html><head><title>ERROR</title></head>\n";
            $error_html .= "<style type=\"text/css\">#ID_MAIN{background: red;font-family:sans-serif;font-weight:bold}pre{font-size:.9em;font-weight: bold;}</style></head>\n";
            $error_html .= "<body><div id=\"ID_MAIN\"><h1>ERROR</h1><p>file: $o_error->{-file}<br />line: $o_error->{-line}</p><p>$msg</p><pre>\n";
            $error_html .= $o_error->{-text}."\n".$o_error->stacktrace()."\n</pre></div></body></html>\n";
            print $error_html;
        }
        else
        {
            my $hd = "SYSTEM ERROR";
            my $bd = "Sorry, an unexpected error happened";
            my $ft = "You may try the following:<ol>";
            $ft .= "<li>click the <a href=\"javascript:O_CONTROL_BLOCK.Reload()\">Refresh icon</a> to refresh the display</li>";
            $ft .= "<li><a href=\"javascript:window.location.reload()\">Reload</a> to reload the page</li>";
            $ft .= "<li>If this does not work, <a href=\"" . url() . "\" >click here</a> to restart Narcisse</li>";
            $ft .= "</ol>";
            _PrintNarcisseError($hd,$bd,$ft,$rest);
        }
    }
}


=head2 Procedure Main

	Title      : Main
	Usage      : &Main(), &Main($cgi)
	Procedure  : This SUB makes the whole job
    Args       : $cgi (optional) If available, it is passed to ParamParser

=cut

sub Main
{
	my( $cgi,$debugging ) = @_;

    # NOTES - 1/ $o_io may be an instance observer OR a Narcisse observer (multi)
    #         2/ The factory returns a singleton, so that this object is created FOR THE WHOLE PROCESS LIFETIME
	#my $o_io  = &ConfigurationFactory::GetConfigurationNoSingleton();
    my $o_io  = &ConfigurationFactory::GetConfiguration();
    $ENV{'NARCISSE_CFG'} = $o_io->GetCfgPath();

	# Create a ParamParser object - It will be fully initialized later, because we need the session directory
    my $o_param = New ParamParser();             # first init the object
	#my $o_param = New ParamParser('CGIPM',$cgi);
	$o_param->UnsetBehaviour('assert_strict');
    # We do not filter out \ because of Windows computers which give the whola path as parameter
    # $o_param->SetAuthorizedCharacters('[^\\\`\'|"*?~^(){}$\[\]]');
    $o_param->SetAuthorizedCharacters('[^`|*?~^$]');    # The only dangerous characters
    $o_param->Update('CGIPM','A',$cgi);          # then read the parameters
    #$o_param->SetUnlessDefined('script','HOME');

	# Set a stupid usage, to give not too much information to the hackers
	$o_param->SetUsage(my $usage= sub { print 'oops' });

	# Everybody may read/write the files created by the cgi-bin.
	# ONLY useful for debugging purpose, when you are not root
	# In a production context, you should comment this
	# TODO - UN PARAMETRE POUR CA !

	umask 0 if ( &PRODUCTION==&FALSE || $debugging == &TRUE );

    # We check for the work in progress file only for the new sessions, users may continue working in their sessions
    # this is also good for the performance, as the file is checked only from time to time
    #&IsWorkInProgress($o_io) if ($o_param->Get('__wb_session') eq '');

	# Create a WebBuilder object
	my $o_wb = WebBuilderFactory::GetWebBuilder( o_io => $o_io, o_param => $o_param, -debug => &DEBUG );
    $o_wb->SetBehaviour('use_exceptions');
    $o_wb->SetBehaviour('dsp_stacktrace') if ( &PRODUCTION==&FALSE || $debugging == &TRUE );
    $o_wb->SetBehaviour('no_robots');

    # If we got a NarcisseObserver, we just display the multi-instances page
	if ($o_io->isa( "NarcisseObserver" ))
	{
        # Force the script parameter to HOME, it is the only value acceptable for this parameter
        $o_param->Set('script','HOME');
		my $o_nc_script = NcScriptFactory::GetNcScript( $o_io, $o_wb);
		$o_wb->SetFunction('PrintHtml') unless $o_param->IsDefined('__wb_function');
		$o_wb->Realize();

		# Call the $o_nc_script destructor NOW
		$o_nc_script = undef;
	}

	# We are working from a specific instance, there are more things to do
	else
	{
		# Initialize the environment, creating a session and a Logger object
		my $o_logger = &InitEnvironment( $o_io, $o_wb );
		$O_LOGGER = $o_logger;          # O_LOGGER used by the old code

		# Fully initialization of the parameters
		# WARNING $o_param IS CHANGED

		&InitializeParam($o_io,$o_param);

		# Synchronize $o_wb with $o_param
		$o_wb->SetFunction();

        # If the parameter RemoveData is set, remove the personal data
		if( $o_param->Get('RemoveData') ne '' )
		{
			&RemoveData($o_io,$o_param,$o_logger);
		}

		# If the parameter UploadData is set, finish uploading the file
		if( $o_param->Get('UploadData') ne '' )
		{
			&UploadData($o_io,$o_param,$o_logger);
		}

		if( $o_param->Get( 'UploadSeq' ) ne '' )
		{
			# ecriture du fichier dans la session
            &UploadFile( $o_param, 'UploadSeq' );
		}

		# Create a $o_nc_script object, it is connected to $o_wb through $o_wb->SetObject
		my $o_nc_script = NcScriptFactory::GetNcScript( $o_io, $o_wb, $o_logger);

		# prepare the data: the images are computed now if necessary
		$o_nc_script->PrepareData();

		# Call Realize, printing everything, or may be sending an http_status
        $o_wb->Realize();

		# Call the $o_nc_script destructor NOW
		$o_nc_script = undef;

		#log the end of script
		$o_logger->Trace("narcisse.cgi: narcisse20.cgi elapsed\n","T0","NV");

		#chdir to main tmp directory (important for fast_cgi)
        &TiniEnvironment( $o_io );

        $O_LOGGER='';
	}

	return;
} # Main

#=head2 Procedure IsWorkInProgress

 #Title       : SetSigWarn
 #Usage       : &SetSigWarn( $warn_file )
 #Function    : Open the warning file in append mode and redirect all warnings to this file
 #Args        : $warn_file: The new warning file
 #Globals     : WARN_FILE, the associated file descriptor

#=cut
#sub IsWorkInProgress
#{
    #my ($o_conf) = @_;
    #my $file_work_in_progress = $o_conf->GetCfg('ROOT_WORK_IN_PROGRESS');
    #if (-e $file_work_in_progress)
    #{
        #my $fh_work_in_progress = new IO::File($file_work_in_progress) or throw LipmError::IOException( $file_work_in_progress, 'f' );
        #my @a_msg = <$fh_work_in_progress>;
        #my $msg = join('<br />',@a_msg);
        #throw NarcisseError(-hd => "NARCISSE DISABLED FOR THE MOMENT", -bd=>$msg);
    #}
#}


=head2 Procedure SetSigWarn

 Title       : SetSigWarn
 Usage       : &SetSigWarn( $warn_file )
 Function    : Open the warning file in append mode and redirect all warnings to this file
 Args        : $warn_file: The new warning file
 Globals     : WARN_FILE, the associated file descriptor

=cut
sub SetSigWarn
{
    my ($warn_file) = (@_);

    close(WARN_FILE);
    open(WARN_FILE, ">>$warn_file") or throw LipmError::IOException( $warn_file, 'f' );
    $SIG{'__WARN__'} = sub { print WARN_FILE $_[0]; }
}


=head2 Function InitEnvironment

 Title       : InitEnvironment
 Usage       : $o_logger = &InitEnvironment( $o_io, $o_wb )
 Function    : Set the session directory, open *our* STDERR and create a logger object
 Returns     : $o_logger, a logger object
 Args        : $o_io (config)
               $o_wb, a WebBuilder object

=cut

sub InitEnvironment
{
	my ( $o_io, $o_wb ) = (@_);

    $o_io->SetSessionDirectory($o_wb->GetSession());

	# Redirect stderr to a file inside the session directory,
	# this will redirect also every child's stderr (bdbh, nc_XXX scripts, etc)
	my $warn_file = $o_io->GetCfg( 'STDERR_FILE' );
    &SetSigWarn($warn_file);

	return CreateLogger( $o_io,$o_wb );
}

=head2 Procedure TiniEnvironment

 Title       : TiniEnvironment
 Usage       : &TiniEnvironment( $o_io )
 Function    : Clear the session from $o_io, chdir to tmp directory
               Does the reverse of InitEnvironment
 Args        : $o_io (config)

=cut

sub TiniEnvironment
{
	my ( $o_io ) = (@_);

    # Reset SigWarn to the tmp directory, and release the global variables
    my ($root_tmp,$stderr) = $o_io->GetCfg('ROOT_TMP','STDERR_FILE');
    &SetSigWarn( File::Spec->catfile( $root_tmp,$stderr ));

    # Change to the tmp directory
    chdir ( $root_tmp );

	return;
}

=head2 Procedure InitializeParam

 Title       : InitializeParam
 Usage       : $o_param = &InitializeParam( $o_io, $o_param )
 Function    : Init and check the parameters
                    - If UploadSession is set, call UploadSession, this will initialize $o_param
               Else:
                    - Update $o_param from one of those two sources:
                         - bookmark.cfg, if some bookmark was specified (Init mode)
                         - local.cfg (Append mode)
               check the parameters
 Args        : $o_io (config)
             : $o_param (In/Out)
 Error       : May throw an exception if parameters not validated for security
 TODO  - Revoir tout ca, y compris avec ValidParams et CleanParams

=cut

sub InitializeParam
{
	my ($o_io,$o_param) = (@_);

	my $local_cfg = 'local.cfg';
	my $bookmark_cfg = "";

	# If UploadSession was set, call UploadSession now
	if ($o_param->IsDefined('UploadSession'))
	{
		&UploadSession($o_io,$o_param);
	}
	else
	{
		# If no wb_function defined, the default is PrintHtml
		# they are defined here, so that we have the priority over local.cfg !
		if ($o_param->IsDefined('__wba_ajax') and $o_param->Get('__wba_ajax') eq 'j')
		{
			$o_param->SetUnlessDefined('script','JSON');
		}
		else
		{
			$o_param->SetUnlessDefined('__wb_function','PrintHtml');
		}

		if ($o_param->IsDefined('read_bookmark'))
		{
			$bookmark_cfg = $o_param->Get('read_bookmark').'.cfg';
			$bookmark_cfg = "" unless(-r $bookmark_cfg);  # If the cfg file does not exist, we forget the bookmark
			$o_param->Delete('read_bookmark');            # This parameter is useless from this point
		}

		# If reading from bookmark, update $o_param, initializing again.
		# But first save the __wba_ajax/__wba_index parameters
		if ($bookmark_cfg ne "")
		{
			# __wba_ajax is never kept in the cfg files, take care of it !
			my ($ajax,$index)=('','');
			$ajax  = $o_param->Get('__wba_ajax') if ($o_param->IsDefined('__wba_ajax'));
			$index = $o_param->Get('__wba_index') if ($o_param->IsDefined('__wba_index'));
			$o_param->Update($bookmark_cfg,'I');
			$o_param->Set('__wba_ajax',$ajax) if ($ajax ne '');
			$o_param->Set('__wba_index',$index) if ($index ne '');
		}

        # If __wba_index defined, we drop every parameter and use those from local.cfg instead
        # This is useful for parameters like Zoom +/- or position shift, to avoid a new zoom/shift between the layers
        elsif ($o_param->IsDefined('__wba_index'))
        {
            my $index = $o_param->Get('__wba_index');
            $o_param->Update($local_cfg,'I');
            $o_param->Set('__wba_index',$index);
            $o_param->Set('__wba_ajax',&TRUE);
        }
        # The parameters read from local.cfg are used only if not specified on command line
        elsif (-e $local_cfg)
        {
            $o_param->Update($local_cfg,'A');
        }

        # The Gui Defaults parameters are used as a last resort
        $o_param->Update('HASH','A',$o_io->GetGuiDefHash());


        # If no script= parameter specified, the default is HOME
        $o_param->SetUnlessDefined('script','HOME');

    }

	# The default is NOT to have full screen
    $o_param->SetUnlessDefined('full_screen','0');

    # Reorganizing the parameters: pos may be specified for a position OR for a gene, if it is a string we prefer the parameter 'gene'
    # if ($o_param->IsDefined('pos'))
    # {
        # if ($o_param->Get('pos') ne "0" and $o_param->Get('pos') !~ /^[\d\.]+[mk]?b?$/i)
        # {
            # $o_param->Set('gene',$o_param->Get('pos'));
            # $o_param->Delete('pos');
        # }
    # }
    # # Remove any separator character from 'pos'
    # # MANU - EST-CE VRAIMENT UTILE ?
    # if ($o_param->IsDefined('pos'))
    # {
        # my $pos = $o_param->Get('pos');
        # $pos =~ s/\s//g;
        # $o_param->Set('pos',$pos);
    # }

    return $o_param;
}


# ##################################################
# ##########          Routines		##############
# ##################################################
#
=head2 function CreateLogger

	Title       :	CreateLogger
	Usage       :	&CreateLogger($o_io,$o_wb)
	function    :	create the logger object and do something if LOG_LEVEL is high
	Returns     :	the Logger object
	Args        :	$o_io, the config
                    $o_web_builder, a valid web_builder object
	globals     :

=cut

sub CreateLogger
{
	my ($o_io,$o_wb) = (@_);

	#my $session_path = $o_wb->GetSessionPath();
	my $opt = "NOCOMPRESSION ";

    my ($log_file,$log_level) = $o_io->GetCfg('LOG_FILE','LOG_LEVEL');
	my $o_logger = New Logger (">>".$log_file,$log_level,$opt);
	$o_logger->StartTimer('T0');      # The main timer

	#
	# If LOG_LEVEL is high enough (NV/NVV), we make some other things:
	#
	#    1- copy the url to narcisse.history
	#    2- make a directory whose name is the stamp
	#    3- make a soft link to this directory, the link is called LastClick
	#       This link will be used by the function NARCISSE_utils::TraceData
	#

	if ($log_level =~ /NV/)
	{
		my $hist= new IO::File(">>narcisse.history") or die "Cannot open narcisse.history for writing";
		my $stamp = $o_logger->CalcStamp();
		my $stamp_dir = "$stamp";
		my $last_dir  = $o_io->GetCfg('ROOT_LASTCLICKDIR');

        # $ENV{HTTP_HOST} should exist, unless we are debugging with perl [-d] narcisse16.cgi
        if (exists $ENV{HTTP_HOST})
        {
            print $hist $stamp. ' http://'.$ENV{HTTP_HOST}.'/'.$ENV{SCRIPT_FILENAME}.'?'.$ENV{QUERY_STRING}."\n";
        }
        else
        {
            print $hist $stamp. 'command line execution, are you debugging ?'."\n";
        }
		if ($log_level =~ /NVV/)
		{
			mkdir $stamp_dir;
			unlink $last_dir;
			symlink($stamp_dir,$last_dir);
            my $o_param = $o_wb->GetObjectParam();
            $o_param->Dump($last_dir.'/params.cfg');
            my $env_file = $last_dir.'/env.txt';
            my $fh_env_file = new IO::File(">$env_file");
            print $fh_env_file Dumper(\%ENV);
        }
    }

	return $o_logger;
}

# ##	##	##	##	##	##	##	##	##	##	##

=head2 Procedure UploadSession

 Title        : UploadSession
 Usage        : &UploadSession($o_io,$o_param);
 Procedure    : Check that the program and data versions are more or less correct
                REMOVE EVERY FILE found in the session directory
                Upload the session file and put the retrieved files to the session directory
                WARNING After this function, $o_param is COMPLETELY MODIFIED !
 Args         : $o_io,$o_param

=cut
sub UploadSession
{
    my ($o_io,$o_param) = (@_);

    my $file = &UploadFile($o_param, 'UploadSession');

    # untar the uploaded file
    my $tar = $o_io->GetUnixPath( 'UNIX_TAR' );
    `$tar -x -f $file`;

    # check the version number
    my $v_param = New ParamParser('tmp/VERSION.cfg');

    # TODO We do not check NARCISSE_VERSION yet
    my $msg="";
    my $instance_name    = $o_io->GetName();
    my $instance_version = $o_io->GetVersion();
    my $narcisse_version = $o_io->GetNarcisseVersion();

    if ($instance_name ne $v_param->Get('INSTANCE_NAME'))
    {
        $msg .= "There is a mismatch between the instance names<br />";
    }

    if ($instance_version lt $v_param->Get('INSTANCE_VERSION'))
    {
        $msg .= "You saved the data on an instance newer than the current instance<br />";
    }

    if ($msg ne "")
    {
        my $narcisse_cgi = $o_io->GetCfg( 'HTTP_NARCISSE_CGI' );
        $msg .= "<p>Sorry, we cannot load your data<br />$msg</p>\n";
        $msg .= "<table class=\"TableRelease\"><tr><th>&nbsp;</th><th>Current release</th><th>Saved release</th></tr>\n";
        $msg .="<tr><th>NARCISSE_VERSION</th><td>$narcisse_version</td><td>".$v_param->Get('NARCISSE_VERSION')."</td></tr>\n";
        $msg .= "<tr><th>INSTANCE_NAME</th><td>$instance_name</td><td>".$v_param->Get('INSTANCE_NAME')."</td></tr>\n";
        $msg .= "<tr><th>INSTANCE_VERSION</th><td>$instance_version</td><td>".$v_param->Get('INSTANCE_VERSION')."</td></tr></table>\n";
        $msg .= "<p>Press the BACK button on your browser or go to <a href=\"$narcisse_cgi\"> home</a></p>\n";
        throw NarcisseError(-hd => "UPLOAD ERROR - MISMATCH IN VERSIONS",-bd=>$msg);
    }

    # Save some files (more convenient for debug)
    my ($rm,$mv,$cp)            = $o_io->GetUnixPath( 'UNIX_RM','UNIX_MV','UNIX_CP' );
    my ($log_file,$stderr_file) = $o_io->GetCfg( 'LOG_FILE','STDERR_FILE' );
    `$mv $log_file $stderr_file tmp`;

    # Remove every file (NOT DIRECTORIES) from the session directory
    `$rm *`;

    # Remove every file and directories (NOT HIDDEN FILES LIKE .htaccess) from the private directory
    `$rm -r private/*`;

    # Move files from tmp directory to session directory
    `$mv tmp/* .`;

    # Move everything from tmp/private directory to session directory
    my $personal = $o_io->GetCfg('PERSONAL');
    `$cp -r tmp/$personal/* $personal`;

    # Remove tmp directory
    `$rm -r tmp`;

    # Init ParamParser from local.cfg
    my $local_cfg = 'local.cfg';
    my $sav_sess = $o_param->Get('__wb_session');
    $o_param->Update($local_cfg,'I');
    $o_param->Set('__wb_session',$sav_sess);

    # Update the Instance Observer object with session data
    $o_io->UpdateGenomesListWithSessionData();

    return;
}

=head2 procedure RemoveData

 Title        : RemoveData
 Usage        : &RemoveData($o_param);
 Procedure    : Remove the data that were previously uploaded for the reference species
 Args         : $o_io,$o_param,$o_logger

=cut
sub RemoveData
{
	my ($o_io, $o_param, $o_logger) = (@_);
    my $unix_rm = $o_io->GetUnixPath('UNIX_RM');
 	my $species       = $o_param->Get('reference');
	my $o_so          = $o_io->GetSpeciesObserver($species);
    my @a_perso_files = $o_so->GetPersoFiles( -check => &FALSE);

    # Remove 1 time only
    $o_param->Delete('RemoveData');

    # remove the files
    foreach my $f (@a_perso_files)
    {
        `$unix_rm $f`;
    }
 	$o_logger->Trace( "Personal data removed for $species\n", '.', 'NV' );

    # The users are so lazy we must click the button for them...
    $o_param->Set('ck_personal',0);

    # A lot of stuff must be computed again, so we remove every image or coord file
    my $rm = $o_io->GetUnixPath('UNIX_RM');
    `$rm *.png *.coord *.txt`;

	# circos file
	my $nb = unlink( <circos_*.png> );
	my $msg = "$nb circos files deleted.\n";
	$o_logger->Trace( $msg, '.', 'NV' );

}

=head2 procedure UploadData

 Title        : UploadData
 Usage        : &UploadData($o_io,$o_param,$o_logger);
 Procedure    : Upload the user data, creating or appending one or several files
 Args         : $file, $o_io, $o_param, $o_logger

=cut
sub UploadData
{
	my( $o_io, $o_param, $o_logger ) = @_;

	# $file est un objet 'Fh'
	my $file = &UploadFile( $o_param, 'UploadData' );

	# Avoiding an error if there is a second call with the same parameters
	$o_param->Delete('UploadData');

	my $reference= $o_param->Get( 'reference' );

	&__RemoveQuotes( $file );

	my $ra_data = &__CheckFile( $file, $o_logger );

	#my $rh_incomplete_data = &__FindIncompleteData($rh_data);
	my $species = $o_param->Get('reference');
	my $o_so    = $o_io->GetSpeciesObserver($species);

	# fill empty necessary columns
	$ra_data = &__PopulateData( $o_io, $ra_data, $species, $o_logger );
	$o_logger -> Trace("Populated data " . Dumper($ra_data),'.','NVV');
	
    # Convert the data (kb => *1000, mb => x1000000)
    &__ConvertData( $ra_data );
	$o_logger -> Trace("Converted data " . Dumper($ra_data),'.','NVV');

    # Split the data by chrom
    my $rh_splitted_data = &__SplitDataByChrom( $ra_data );
	#$o_logger -> Trace("SPLITTED DATA " . Dumper($rh_splitted_data),'.','NVV');
	
    # Create or append personal file (1 file / chromosome)
    my @a_headers = qw( chrom element start end color label );
    for my $chr (keys %$rh_splitted_data)
    {
		my $file_by_chrom = $o_so->GetPersoFiles( -check => &FALSE, -chrom => $chr);
		my $o_ndw         = New NarcisseDataWriter::OnMemory( $file_by_chrom, \@a_headers, { append => &TRUE }, undef, $o_logger );
		$o_ndw -> Update($rh_splitted_data->{$chr});
	}
	
    # The users are so lazy we must click the button for them...
    $o_param->Set('ck_personal',1);

    # A lot of stuff must be computed again, so we remove every image or coord file
    my $rm = $o_io->GetUnixPath('UNIX_RM');
    `$rm *.png *.coord *.txt`;

	# circos file
	my $nb = unlink( <circos_*.png> );
	my $msg = "$nb circos files deleted.\n";
	$o_logger->Trace( $msg, '.', 'NV' );

	return;

	# -------------------

    # Excel or ooo put quotes around the fields, and DataReader cannot analyze the file
    # If the quotes are present, we remove them FROM ALL LINES
    sub __RemoveQuotes
    {
        my( $file ) = @_;

        my $fh_file = new IO::File( $file, 'r' )
						or throw LipmError::IOException( $file, 'f' );
        my $line1   = <$fh_file>;
        chomp $line1;

        if( $line1 =~ /^['"]#/ )
        {
            $line1 =~ s/['"]//g;
            my $file_new = $file . '.new';
            my $fh_file_new = new IO::File( $file_new, 'w' )
								or throw LipmError::IOException( $file, 'f' );
            print $fh_file_new "$line1\n";

            while( my $line = <$fh_file> )
            {
                $line =~ s/['"]//g;
                print $fh_file_new $line;
            }

            # Close files (input and output)
			map{ $_->close(); } ( $fh_file, $fh_file_new );

            move( $file, $file_new ); # File::Copy
        }

		return;
    }

	sub __CheckFile
	{
		my( $file, $o_logger ) = @_;

		my $o_ndr = New DataReader::Whole();
		$o_ndr->Prepare( -from   => $file,
						-force_reading => &TRUE );
		my $ra_data = $o_ndr->FetchallArrayref(-line_type => 'HASH');

		my $rh_row  = $$ra_data[0];
		my @a_keys  = keys %$rh_row;
		if( 1 == @a_keys && 'element' ne $a_keys[0] )
		{
			# besoin de réécrire le fichier avec un en-tête
			$o_ndr->Finish();

			my $msg = "Upload personnal file : no header and 1 column, will be rewritten.\n";
			$o_logger->Trace( $msg, '.', 'NV' );

			# add the 'element' header
			my $new_file    = $file . '.new';
			my $fh_new_file = new IO::File( $new_file, 'w' )
							or throw LipmError::IOException( $new_file, 'f' );
			print $fh_new_file "element\n";
			my $fh_file     = new IO::File( $file, 'r' )
							or throw LipmError::IOException( $file, 'f' );
			while( my $line = <$fh_file> )
			{
				next if( $line =~ /^;/ );
				print $fh_new_file $line;
			}
			map{ $_->close(); } ( $fh_file, $fh_new_file );
			move( $new_file, $file );

			$o_ndr = New DataReader::Whole();
			$o_ndr->Prepare( -from   => $file,
							-force_reading => &TRUE );
			$ra_data = $o_ndr->FetchallArrayref(-line_type => 'HASH');
		}
		elsif( 1 < @a_keys )
		{
			my( $chrom, $start, $end ) = @$rh_row{ qw( chrom start end ) };
			if( !defined $chrom
				|| !defined $start
				|| !defined $end )
			{
				my $msg = "You must have those columns 'chrom', 'start', 'end'.\n";
				throw LipmError( -text => $msg );
			}
		}

		return $ra_data;
	}


    # Split the data and return a hash structured as follows:
    # %h_rvl: {key = Chromosome label, val = $rh_chr_data}
    #         %$rh_chr_data: {key = element, val = $rh_line}
    #                 $rh_line: {key = column_name, val = column_value}
    #
	sub __SplitDataByChrom
	{
		my( $ra_data ) = @_;
		
		my %h_rvl;
		foreach my $rh_row ( @$ra_data )
		{
			my( $chr, $elt ) = @$rh_row{ qw( chrom element ) };
			next unless defined $chr;
		
			# create a hash for this chrom if it does not exist yet
			if( !exists $h_rvl{ $chr } )
			{
				my %h_chr;
				$h_rvl{ $chr } = \%h_chr;
			}
			$h_rvl{ $chr }{ $elt } = $rh_row;
		}
	
		return \%h_rvl;
	}

	# Look for "lines" data where the 'start' or 'end' field is empty
	# Populate those lines using our services, or remove the line
	# Return the updated data structure
	# If no "element" column, add it
	sub __PopulateData
	{
		my( $o_io, $ra_data, $species, $o_logger ) = @_;

		my @a_incomplete_tags;
		my %h_data = (); # necessaire pour controle
		my $last_elt = 0; # pour ajouter element : A NOTER si qqs valeur existe dans les donnees, elles peuvent etre perdu dans la suite du traitement car on ne garantit plus l'unicite
		
		$o_logger->Trace( "DATA " . Dumper($ra_data),'.','NVV');
		foreach my $rh_row ( @$ra_data )
		{
			my( $start, $end, $elt, $color ) = @$rh_row{ qw( start end element color ) };
			if( !defined $elt )
			{
				$elt = ++$last_elt;
				$$rh_row{ element } = $elt;
			}
			$$rh_row{ color } = 'red' unless defined( $color );

			if( !defined $start || '' eq $start
				|| !defined $end || '' eq $end )
			{
				push( @a_incomplete_tags, $elt );
			}
			$h_data{ uc( $elt ) } = $rh_row;
		}

		# If there are data to populate, populate them now
		try
		{
			if( 0 != @a_incomplete_tags )
			{
				$o_logger->Trace( "INCOMPLETE TAGS " . Dumper(\@a_incomplete_tags),'.','NVV');
				
				#  Calling GetGenesFromLocusTags to find info from the locus tags
				my $o_service = &NarcisseServiceFactory::GetGenesFromLocusTags(
					species         => $species,
					ra_ids     => \@a_incomplete_tags,
					-out_type       => &NS_INFOS,
					-o_logger       => $o_logger,
					-o_instance_obs => $o_io);
				my ($ra_results,$rh_results) = $o_service->FetchResults();
				$o_logger->Trace( "GENES FOUND " . Dumper($rh_results),'.','NVV');

				my @a_keys = qw( chrom start end color label );

				my $ra_infos = $$rh_results{ data };
				foreach my $infos ( @$ra_infos )
				{
					$infos =~ /^>nc\|gene\|(\S+).+chrom=(\S+) range=(\d+)-(\d+)/;
					my( $key, $chrom, $start, $end ) = ( $1, $2, $3, $4 );
					my $rh_row = $h_data{ $key };
					if( defined $rh_row )
					{
						# TODO - Verifier que le chr donne par le fichier s'il existe est le meme que le chr donne par Narcisse
						@$rh_row{ @a_keys } = ( $chrom, $start, $end, 'red', $key );
#						push( @a_new_data, $rh_row );
					}
					else
					{
						# VIRE POUR L'INSTANT CAR IL Y A UN PB AVEC LE SERVICE
						# ou avec l'utilisation de celui-ci <--Seb.
						#throw LipmError( -text => "INTERNAL ERROR $key should be defined in \$rh_data." );
					}
				}
				#$ra_data = [ values %h_data ];
				#$o_logger->Trace( "data " . Dumper($ra_data),'.','NVV');
				#return;

				# Remove the NOT FOUND names from %h_data
				my $ra_not_found = $o_service->FetchNotFound();
				$o_logger->Trace( "GENES NOT FOUND " . Dumper($ra_not_found),'.','NVV');
				my @v = map { delete $h_data{ uc( $_ ) } } @$ra_not_found;
				
				# If NOT FOUND is not empty, we try to convert "not found" names to locus tags
				# However we may have more locus tags than names !
				if ( 0 != @$ra_not_found )
				{
					#  1/ Calling GetLocusTagsFromKeywords  to try to get locus tags
					my $o_service = New GetLocusTagsFromKeywords(
						-species         => $species,
						ra_keywords     => $ra_not_found,
						-out_type       => &NS_LOCUS,
						-o_logger       => $o_logger,
						-o_instance_obs => $o_io);
					my ($ra_dummy,$rh_locus_tags) = $o_service->FetchResults();
					$o_logger->Trace( "LOCUS TAGS FROM KEYWORDS FOUND " . Dumper($rh_locus_tags),'.','NVV');
					my $ra_locus_tags = $rh_locus_tags->{'data'}->[0];
					
					# 2/ Calling again GetGenesFromLocusTags to find info from the locus tags
					if( 0 != @$ra_locus_tags )
					{
						my $o_service = &NarcisseServiceFactory::GetGenesFromLocusTags(
							species         => $species,
							ra_tags         => $ra_locus_tags,
							-out_type       => &NS_INFOS,
							-o_logger       => $o_logger,
							-o_instance_obs => $o_io);
						my ($ra_results,$rh_results) = $o_service->FetchResults();
						$o_logger->Trace( "GENES FOUND => " . Dumper($rh_results),'.','NVV');
						my @a_keys = qw( chrom start end color label element);

						my $ra_infos = $$rh_results{ data };
						foreach my $infos ( @$ra_infos )
						{
							$infos =~ /^>nc\|gene\|(\S+).+chrom=(\S+) range=(\d+)-(\d+)/;
							my( $key, $chrom, $start, $end ) = ( $1, $2, $3, $4 );
							my %h_row;
							@h_row{ @a_keys } = ( $chrom, $start, $end, 'red', $key, $key );
							$h_data{ $key } = \%h_row;
						}
					}
				}
				$ra_data = [ values %h_data ];
				$o_logger->Trace( "data " . Dumper($ra_data),'.','NVV');
			}
		}
		catch LipmError with
		{
			my ( $o_error ) = @_;
			my $msg        = $o_error->Explain();
			warn $msg;
			#Ignore any error (an error may be generated if we upload data to a species without any gene data)
		};

		return $ra_data;
	}

    # Replace 1kb and 1Mb in the start or end fields
    sub __ConvertData
    {
        my( $ra_data ) = @_;

        foreach my $rh_line ( @$ra_data )
        {
            foreach my $key ( qw( start end ) )
            {
				my $value = $$rh_line{ $key };
				$$rh_line{ $key } = Sequence_utils::Unit2Base( $value );
            }
        }
		
		return;
    } # __ConvertData

}

=head2 function UploadFile

 Title      :	UploadFile
 Usage      :	&UploadFile($file, $session_path, $o_param)
 Prerequiste:	form method=post and don't forget enctype="multipart/form-data"
 function   :	Upload the file and create a copy with the same file name to the current directory (should be the session directory)
 Returns    :	the path to the uploaded file in the session directory
 Args       :	$o_param   the ParamParser object
                $field     The field where the uploaded filename is kept

=cut

sub UploadFile
{
	my( $o_param, $field ) = @_;

    my $local_file    = $o_param->Get( $field );

	my $fh_local_file = new IO::File( $local_file, "w" )
						or throw LipmError::IOException( $local_file, 'f');
    my $fh_uploaded   = upload( $field )
						or throw LipmError::IOException( $local_file, 'f');
	while( my $line = <$fh_uploaded> )
	{
		# Converting \r\n (win) to \n (linux)
		# or         \r (mac) to \n (linux)
		$line =~ s/\r\n?$/\n/;
		print $fh_local_file $line;
	}
    close( $fh_uploaded );
	$fh_local_file->close();

	# "" convert the Fh object $local_file to its filename
	return "$local_file";
}


__END__
=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license - www.cecill.info

=cut

