#! /usr/bin/perl

BEGIN
{
    if (exists $ENV{'PARALOOP'})
    {
        unshift(@INC, $ENV{'PARALOOP'}.'/lib');
    }
    elsif (exists $ENV{'LIPMLIB'})
    {
        unshift(@INC, $ENV{'LIPMLIB'});
    }
    else
    {
        my ($dirprg, $nameprg) = $0 =~ /(.+)\/(.+)/;
        unshift(@INC, "$dirprg/lib", "$dirprg/../lib");
    }
}


#
# Usage: ./input_generation.pl  100 300 20
#
# 20 = nb_cpus
# 300 = nb d'iterations
# 100 = temps d'attente de la premiere iteration et de la seconde moitie de threads
#
# Pour N cpus, on a au debut N/2 threads qui avancent tres vite et N/2 threads qui attendent tres longtemps mais seulement la premiere iteration
# De la sorte les threds rapides piquent le job aux threads lentes et on peut tester le mode load_balancing
#
# Pour que ca marche bien il faut que niter soit un multiple de ncpus - on le teste
#

use strict;
use warnings;
use ParamParser;

my $o_param = New ParamParser();
$o_param->SetBehaviour('exit_on_getopt_error');
$o_param->SetUsage(\&Usage);
$o_param->Update('GETOPTLONG','I',"ncpus=i","niter=i","short_sleep=i","long_sleep=i");
$o_param->AssertDefined('ncpus','niter','short_sleep','long_sleep');

my $ncpus = $o_param->Get('ncpus');
my $niter = $o_param->Get('niter');
my ($short_sleep,$long_sleep) = $o_param->Get('short_sleep','long_sleep');

#print "$niter $ncpus $short_sleep $long_sleep\n";
#exit;

if ( $niter % $ncpus != 0 )
{
	print STDERR "--niter doit etre un multiple de --ncpus !\n";
	exit;
}

my $inner = $niter / $ncpus;

for ( my $j=0; $j < $ncpus; $j++ )
{
	for ( my $i=0; $i<$inner;$i++)
	{
		my $s;
		if ($i==0 && $j>=$ncpus/2)
		{
			$s = $long_sleep	;
		}
		else
		{
			$s = $short_sleep;
		}
		print "echo \'$i-$j\'; sleep $s\n";
	}
}



sub Usage
{
	print STDERR "$0 --ncpus=20 --niter=200 --short_sleep=1 --long_sleep=100\n";
	exit 1;
}

